MODULE plot_eA_hadron
  USE eA_hadron_global
  IMPLICIT NONE
  INTEGER,PARAMETER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=11
  INTEGER,PARAMETER::n_width=1,n_height=1
  CHARACTER(len=24),PARAMETER::output_dir="./output/"
CONTAINS
  SUBROUTINE initplot_eA_hadron
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(max_weight)::weights_info
    INTEGER::i
    nwgt=1
    weights_info(nwgt)=" eAhadron"
    ! output plot files
    CALL plot_begin_eA_hadron(nwgt,weights_info)
    RETURN
  END SUBROUTINE initplot_eA_hadron

  SUBROUTINE plotout_eA_hadron
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_eA_hadron(xnorm)
    RETURN
  END SUBROUTINE plotout_eA_hadron

  SUBROUTINE outfun_eA_hadron(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(max_weight)::wgts,wgtden
    nwgt=1
    wgts(1)=www
    ! output plot file 
    CALL plot_fill_eA_hadron(wgts)
    RETURN
  END SUBROUTINE outfun_eA_hadron

  SUBROUTINE plot_begin_eA_hadron(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CALL INIHIST
    !CALL INIHIST_3D
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_psiX_cb: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    wgt_Ne=0d0
    wgt_Ntot=0d0
    DO kk=1,nwgt_analysis
       l=(kk-1)*num_plots
       CALL bookup(l+ 1,'total rate    '//weights_info(kk),&
            1.0d0,0.5d0,5.5d0)
       CALL bookup(l+ 2,'gamma v '//weights_info(kk),&
            1d0,0d0,100d0)
       CALL bookup(l+ 3,'gamma Q2 '//weights_info(kk),&
            1d0,0d0,100d0)
       CALL bookup(l+ 4,'hadron z '//weights_info(kk),&
            1d-2,0d0,1d0)
       CALL bookup(l+ 5,'hadron pt '//weights_info(kk),&
            1d0,0d0,100d0)
       CALL bookup(l+ 6,'hadron p '//weights_info(kk),&
            1d0,0d0,100d0)
       CALL bookup(l+ 7,'hadron xF '//weights_info(kk),&
            2d-2,-1d0,1d0)
       CALL bookup(l+ 8,'quark x '//weights_info(kk),&
            1d-2,0d0,1d0)
       CALL bookup(l+ 9,'electron pt '//weights_info(kk),&
            1d0,0d0,100d0)
       CALL bookup(l+10,'electron p '//weights_info(kk),&
            1d0,0d0,100d0)
       CALL bookup(l+11,'hadron multi. vs v '//weights_info(kk),&
            1d0,0d0,100d0)
    ENDDO
    RETURN
  END SUBROUTINE plot_begin_eA_hadron

  SUBROUTINE plot_end_eA_hadron(xnorm)
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    CALL mclear
    !CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'+',i,i,xnorm,0.d0)
       CALL mfinal(i)
       !CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
       !CALL mfinal_3d(i)
    ENDDO

    OPEN(unit=1354,FILE=TRIM(output_dir)//'eA_hadron_wgt.out',&
         status='unknown')
    WRITE(1354,*)wgt_Ntot,wgt_Ne
    CLOSE(1354)
    
    ytit='sigma per bin '
    CALL open_topdrawer_file_eA_hadron
    DO kk=1,nwgt_analysis
       l=(kk-1)*num_plots
       CALL multitop(l+ 1,n_width,n_height,'total rate   ',ytit,'LIN')
       CALL multitop(l+ 2,n_width,n_height,'gamma v  ',ytit,'LOG')
       CALL multitop(l+ 3,n_width,n_height,'gamma Q2 ',ytit,'LOG')
       CALL multitop(l+ 4,n_width,n_height,'hadron z ',ytit,'LOG')
       CALL multitop(l+ 5,n_width,n_height,'hadron pt ',ytit,'LOG')
       CALL multitop(l+ 6,n_width,n_height,'hadron p ',ytit,'LOG')
       CALL multitop(l+ 7,n_width,n_height,'hadron xF ',ytit,'LOG')
       CALL multitop(l+ 8,n_width,n_height,'quark x ',ytit,'LOG')
       CALL multitop(l+ 9,n_width,n_height,'electron pt ',ytit,'LOG')
       CALL multitop(l+10,n_width,n_height,'electron p ',ytit,'LOG')
       CALL multitop(l+11,n_width,n_height,'hadron multi vs v ',ytit,'LOG')
    END DO
    CALL close_topdrawer_file_eA_hadron

    CALL open_gnuplot_file_eA_hadron
    psfile="eA_hadron.ps"
    DO kk=1,nwgt_analysis
       l=(kk-1)*num_plots
       CALL MGNUPLOT(l+ 1,'total rate   ',ytit,'LIN',psfile)
       CALL MGNUPLOT(l+ 2,'gamma v ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 3,'gamma Q2 ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 4,'hadron z ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 5,'hadron pt ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 6,'hadron p ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 7,'hadron xF ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 8,'quark x ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+ 9,'electron pt ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+10,'electron p ',ytit,'LOG',psfile)
       CALL MGNUPLOT(l+11,'hadron multi vs v ',ytit,'LOG',psfile)
    ENDDO
    CALL close_gnuplot_file_eA_hadron

    CALL open_root_file_eA_hadron
    rootfile="eA_hadron.root"
    DO kk=1,nwgt_analysis
       l=(kk-1)*num_plots
       CALL MROOTPLOT(l+ 1,'total rate  ',ytit,'LIN',rootfile)
       CALL MROOTPLOT(l+ 2,'gamma v ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 3,'gamma Q2 ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 4,'hadron z ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 5,'hadron pt ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 6,'hadron p ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 7,'hadron xF ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 8,'quark x ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+ 9,'electron pt ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+10,'electron p ',ytit,'LOG',rootfile)
       CALL MROOTPLOT(l+11,'hadron mutli vs v ',ytit,'LOG',rootfile)
    ENDDO
    ! wirte the end of the .C file for root
    WRITE (96, 100)
100 FORMAT(/1X,&
         ' hohisto -> cd();',/1X,&
         ' if (histos -> GetEntries() > 0 ) then {',/1X,&
         '  histos->Write();',/1X,&
         '  hohisto -> Close();',/1X,&
         ' }',/1X,'}')
    CALL close_root_file_eA_hadron
    RETURN
  END SUBROUTINE plot_end_eA_hadron

  SUBROUTINE plot_fill_eA_hadron(wgts)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2,Q2,pt_hadron,p_hadron,xF_hadron
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,pt_electron,p_electron
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    Q2=gamma_pmom(4)**2-gamma_pmom(1)**2-gamma_pmom(2)**2-gamma_pmom(3)**2
    pt_hadron=DSQRT(hadron_pmom(1)**2+hadron_pmom(2)**2)
    p_hadron=DSQRT(pt_hadron**2+hadron_pmom(3)**2)
    pt_electron=DSQRT(parton_pmom(3,1)**2+parton_pmom(3,2)**2)
    p_electron=DSQRT(pt_electron**2+parton_pmom(3,3)**2)
    ! boost from the lab frame to the collision frame
    pboo(1:2)=0d0
    pboo(4)=Mn+ele_energy
    pboo(3)=-ele_energy
    e=DSQRT(Mn**2+2d0*ele_energy*Mn)
    CALL boostl(e,pboo,hadron_pmom(1:4))
    xF_hadron=xFeynman(hadron_pmom(1:4),e)
    ! boost from the collision frame to the lab frame
    pboo(3)=-pboo(3)
    CALL boostl(e,pboo,hadron_pmom(1:4))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    wgt_Ntot=wgt_Ntot+wgts(1)
    wgt_Ne=wgt_Ne+wgts(1)/wgt_FF
    DO kk=1,nwgt_analysis
       www=wgts(kk)
       l=(kk-1)*num_plots
       CALL mfill(l+1,var,www)
       CALL mfill(l+2,v_gamma,www)
       CALL mfill(l+3,Q2,www)
       CALL mfill(l+4,z_FF,www)
       CALL mfill(l+5,pt_hadron,www)
       CALL mfill(l+6,p_hadron,www)
       CALL mfill(l+7,xF_hadron,www)
       CALL mfill(l+8,x_pdf,www)
       CALL mfill(l+9,pt_electron,www)
       CALL mfill(l+10,p_electron,www)
       CALL mfill(l+11,v_gamma,www/z_FF)
    ENDDO
    RETURN
  END SUBROUTINE plot_fill_eA_hadron

  SUBROUTINE open_topdrawer_file_eA_hadron
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'eA_hadron.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_eA_hadron

  SUBROUTINE close_topdrawer_file_eA_hadron
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_eA_hadron

  SUBROUTINE open_gnuplot_file_eA_hadron
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'eA_hadron.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_eA_hadron

  SUBROUTINE close_gnuplot_file_eA_hadron
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_eA_hadron

  SUBROUTINE open_root_file_eA_hadron
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'eA_hadron.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_eA_hadron

  SUBROUTINE close_root_file_eA_hadron
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_eA_hadron

  FUNCTION xFeynman(p,Ecms)
    ! xF is the Feynman parameter (for example see hep-ph/9702263) 
    ! Ecms is the total energy in cms 
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::xFeynman,Ecms
    IF(Ecms.LT.0d0)THEN
       xFeynman=0d0
    ELSE
       xFeynman=2d0*p(3)/Ecms ! yi = Log[xFi*E/mti/2 + Sqrt[1 + (xFi*E/mti/2)^2]]
    ENDIF
  END FUNCTION xFeynman

  SUBROUTINE BOOSTL(Q,PBOO,P)
    ! momentums are in normal representation with fourth comp is the zero comp
    ! Boost P via PBOO(PBOO^2=Q^2) to PLB,and set to P
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PBOO
    REAL(KIND(1d0)),DIMENSION(4),INTENT(INOUT)::P
    REAL(KIND(1d0)),DIMENSION(4)::PCM,PLB
    REAL(KIND(1d0))::FACT
    INTEGER::J
    PCM(1:4)=P(1:4)
    PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)+PBOO(1)*PCM(1))/Q
    FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
    DO J=1,3
       PLB(J)=PCM(J)+FACT*PBOO(J)
    ENDDO
    P(1:4)=PLB(1:4)
  END SUBROUTINE BOOSTL
END MODULE plot_eA_hadron
