MODULE ProcessInfo
  IMPLICIT NONE
CONTAINS
  SUBROUTINE Read_ProcessInfo()
    USE gg_Higgs_NNLO_Global
    IMPLICIT NONE
    INTEGER,PARAMETER::inunit=30654
    CHARACTER(len=500)::comment
    INTEGER::proc_ID,nstatus
    CHARACTER(len=500)::processinfo
    INTEGER::idigit
    CHARACTER(len=2)::fmt,fmt0
    LOGICAL::lexist
    IF(max_process.LE.0)THEN
       WRITE(*,*)"Error: max_process must be > 0"
       STOP
    ENDIF
    idigit=FLOOR(LOG10(REAL(IABS(max_process)))+1)
    IF(idigit.GE.10)THEN
       WRITE(*,*)"Error: too large max_process = ",max_process
       STOP
    ENDIF
    WRITE(fmt0,'(I1)')idigit
    fmt='I'//TRIM(fmt0)
    n_process=0
    process_status(1:max_process)=" N"
    INQUIRE(FILE=TRIM(input_dir)//"process.dat",EXIST=lexist)
    IF(.NOT.lexist)THEN
       WRITE(*,*)"Error: cannot find the file "//TRIM(input_dir)//"process.dat"
       STOP
    ENDIF
    OPEN(UNIT=inunit,FILE=TRIM(input_dir)//"process.dat")
    READ(inunit,*,ERR=1099,END=1099)comment
    DO WHILE(.TRUE.)
       READ(inunit,'('//fmt//',A)',ERR=1099,END=1099)proc_ID,processinfo
       n_process=n_process+1
       IF(n_process.GT.max_process)THEN
          WRITE(*,*)"Error: find too many processes. Please enlarge max_process first !"
          WRITE(*,*)"       max_process = ",max_process
          STOP
       ENDIF
       processinfo=adjustl(processinfo)
       nstatus=LEN_TRIM(processinfo)
       process_ID(n_process)=proc_ID
       process_info(n_process)=processinfo(1:nstatus-1)
       process_status(n_process)=processinfo(nstatus-1:nstatus)
    ENDDO
1099 CONTINUE
    CLOSE(UNIT=inunit)
    RETURN
  END SUBROUTINE Read_ProcessInfo  

  SUBROUTINE PRINT_ProcessInfo(valid)
    USE gg_Higgs_NNLO_Global
    USE readfiles
    IMPLICIT NONE
    LOGICAL,INTENT(IN)::valid
    INTEGER::i,maxlen=-1,lenv
    CHARACTER(len=100)::fmt,fmt0,fmt1
    DO i=1, n_process
       IF(valid.AND.TRIM(process_status(i)).EQ." N")CYCLE
       lenv=LEN_TRIM(process_info(i))
       IF(lenv.GT.maxlen)maxlen=lenv
    ENDDO
    maxlen=MAX(maxlen,3)+3
    fmt0=Int2Char(maxlen-6)
    fmt=TRIM(fmt0)//"X"
    WRITE(*,*)repeat("=",MAX(maxlen+13,30))
    IF(.NOT.valid)THEN
       WRITE(*,*)"All available processes are :"
    ELSE
       WRITE(*,*)"All validated processes are :"
    ENDIF
    WRITE(*,*)repeat("-",MAX(maxlen+13,30))
    WRITE(*,"(A13,"//fmt//",A6)")" ID   process","status"
    WRITE(*,*)repeat("-",MAX(maxlen+13,30))
    DO i=1, n_process
       IF(valid.AND.TRIM(process_status(i)).EQ." N")CYCLE
       fmt1=Int2Char(maxlen-LEN_TRIM(process_info(i)))
       fmt=TRIM(fmt1)//"X"
       WRITE(*,"(I3,3X,A,"//fmt//",A)")process_ID(i),TRIM(process_info(i)), &
            TRIM(process_status(i)) 
    ENDDO
    WRITE(*,*)repeat("=",MAX(maxlen+13,30))
    RETURN
  END SUBROUTINE PRINT_ProcessInfo

  FUNCTION Int2Char(i)
    IMPLICIT NONE
    CHARACTER(len=100)::Int2Char
    INTEGER,INTENT(IN)::i
    INTEGER::idigit
    CHARACTER(len=10)::fmt,fmt0
    IF(i.EQ.0)THEN
       Int2Char=""
       RETURN
    ENDIF
    idigit=FLOOR(LOG10(REAL(IABS(i)))+1)
    IF(idigit.LT.10)THEN
       WRITE(fmt0,'(I1)')idigit
    ELSEIF(idigit.LT.100)THEN
       WRITE(fmt0,'(I2)')idigit
    ELSE
       ! i >= 10^100 (we should also increase len of Int2Char
       WRITE(*,*)"Error: i is too large : ",i
       STOP
    ENDIF
    fmt='I'//TRIM(fmt0)
    WRITE(Int2Char,"("//TRIM(fmt)//")")i
    RETURN
  END FUNCTION Int2Char

END MODULE ProcessInfo
