MODULE bbHH_NNLO_hard_function
  ! This is the hard function and born partonic cross section of
  ! b b~ > h h process
  ! L=Log(-s12/mu**2)=Log(-MHH**2/mu**2)
  ! RHL=Log(MH**2/mu**2)
  USE qcd_constants
  USE global_constants
  IMPLICIT NONE
  PRIVATE
  PUBLIC::bbHH_sigma0A,bbHH_sigma0B
  PUBLIC::bbHH_H1A,bbHH_H2A
  PUBLIC::bbHH_H1B,bbHH_H2B
  !PUBLIC::MBfin2Loop_L0_nf0,MBfin1Loop,MBfin2Loop
CONTAINS
  ! it has a global yq**2 factor and two-body phase space (class A)
  FUNCTION bbHH_sigma0A(vev,lam3,mh2,s12)
    IMPLICIT NONE
    REAL(KIND(1d0))::bbHH_sigma0A
    REAL(KIND(1d0)),INTENT(IN)::vev
    REAL(KIND(1d0)),INTENT(IN)::lam3 ! trilinear coupling of Higgs
    REAL(KIND(1d0)),INTENT(IN)::mh2 ! mass square of Higgs
    REAL(KIND(1d0)),INTENT(IN)::s12 ! s12 the inv. mass square of two initila partons
    bbHH_sigma0A=lam3**2*3d0/4d0*vev**2/(s12*(s12-mh2))
    RETURN
  END FUNCTION bbHH_sigma0A

  ! it has a global yq**4 factor and two-body phase space (class B)
  FUNCTION bbHH_sigma0B(vev,mh2,s12,s13,s14)
    IMPLICIT NONE
    REAL(KIND(1d0))::bbHH_sigma0B
    REAL(KIND(1d0)),INTENT(IN)::vev
    REAL(KIND(1d0)),INTENT(IN)::mh2 ! mass square of Higgs
    REAL(KIND(1d0)),INTENT(IN)::s12 ! s12 the inv. mass square of two initila partons (k1+k2)**2
    REAL(KIND(1d0)),INTENT(IN)::s13 ! s13=(k1-k3)**2
    REAL(KIND(1d0)),INTENT(IN)::s14 ! s14=(k1-k4)**2
    bbHH_sigma0B=1d0/96d0*(1d0/s13-1d0/s14)**2*(s13*s14-mh2**2)/s12**2
    RETURN
  END FUNCTION bbHH_sigma0B

  ! (as/2/pi) term of hard function (class A)
  FUNCTION bbHH_H1A(L) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0))::res
    res=CF*(-DREAL(L**2)+zeta2-2d0)
    RETURN
  END FUNCTION bbHH_H1A

  ! (as/2/pi)**2 term of hard function (class A)
  FUNCTION bbHH_H2A(L) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0))::rL,rL2,rL3,rL4,aL4
    REAL(KIND(1d0))::res
    ! WARNING: we should take the module of the amplitude
    ! instead of Re(A^2) at NNLO
    ! the difference is from one-loop square
    rL=DREAL(L)
    rL2=DREAL(L**2)
    rL3=DREAL(L**3)
    rL4=DREAL(L**4)
    aL4=ABS(L**2)**2
    res=CF**2*((rL4+aL4)/four-rL2*(zeta2-two)+6d0*rL*(two*zeta3-zeta2)&
         +(-39d0/10d0*zeta2**2-15d0*zeta3+6d0*zeta2+four))&
         +CA*CF*(11d0/18d0*rL3+rL2*(zeta2-67d0/18d0)+rL*(-13d0*zeta3+11d0/three*zeta2+121d0/27d0)&
         +22d0/5d0*zeta2**2+151d0/18d0*zeta3-103d0/36d0*zeta2-467d0/162d0)&
         +CF*nfTF*(-two/9d0*rL3+10d0/9d0*rL2-rL*(four/three*zeta2+56d0/27d0)&
         +two/9d0*zeta3+5d0/9d0*zeta2+200d0/81d0)
    RETURN
  END FUNCTION bbHH_H2A

  ! (as/2/pi) term of hard function (class B)
  FUNCTION bbHH_H1B(L,x,y) RESULT(res)
    ! Here RHL=LOG(mh**2/mu**2)
    ! s12=mh**2*(1+x)**2/x
    ! t=s13=-mh**2*y
    ! We have no nf dependence at 1-loop
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0)),INTENT(IN)::x,y
    REAL(KIND(1d0))::RHL,rL,rL2
    REAL(KIND(1d0))::res,ramp
    CALL MBfin1Loop_L0R(x,y,ramp)
    rL=DREAL(L)
    rL2=rL**2
    RHL=rL+DLOG(x/(one+x)**2)
    ! convert from Catani's I subtraction to SCET subtraction
    res=CF*(three*rL-rL**2+7d0*zeta2)-8d0*RHL+ramp
    RETURN
  END FUNCTION bbHH_H1B

  ! (as/2/pi)**2 term of hard function (class B)
  FUNCTION bbHH_H2B(L,x,y,rnf,interp) RESULT(res)
    ! Here RHL=LOG(mh**2/mu**2)
    ! s12=mh**2*(1+x)**2/x
    ! t=s13=-mh**2*y
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0)),INTENT(IN)::x,y
    REAL(KIND(1d0)),INTENT(IN)::rnf ! number of light quark flavour
    INTEGER,INTENT(IN),OPTIONAL::interp
    INTEGER::interp2
    REAL(KIND(1d0))::RHL,rL,rL2,rL3,rL4,costh,z
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::Mfin1Loop0R,Mfin1Loop0I
    REAL(KIND(1d0))::Mfin2Loop2R,Mfin2Loop1R
    REAL(KIND(1d0))::Mfin2Loop0NfR,Mfin2Loop0noNfR
    ! one-loop amplitude with Catnani's I subtracted
    CALL MBfin1Loop_L0R(x,y,Mfin1Loop0R)
    CALL MBfin1Loop_L0I(x,y,Mfin1Loop0I)
    rL=DREAL(L)
    rL2=rL**2
    rL3=rL2*rL
    rL4=rL3*rL
    RHL=rL+DLOG(x/(one+x)**2)
    costh=(one+x**2-two*x*y)/(-one+x**2)
    z=(one+x)**2/x-y-two
    ! L^2 term
    Mfin2Loop2R=RHL**2*4d0*(2d0*rnf-57d0)*z*(-1d0-x**2+2d0*x*y)*(1d0+x**2-x*z)&
         /(3d0*y*(-1d0-x**2+x*y)*(1d0+x**2-2d0*x*z))
    ! L^1 term
    CALL MBfin2Loop_L1R(x,y,rnf,Mfin2Loop1R)
    Mfin2Loop1R=RHL*Mfin2Loop1R
    IF(PRESENT(interp))THEN
       interp2=interp
       IF(interp2.GT.3.OR.interp2.LT.1)interp2=1
    ELSE
       interp2=1
    ENDIF
    ! L^0 but nf dependent term
    CALL MBfin2Loop_L0_nf1_R(x,y,rnf,Mfin2Loop0NfR)
    ! L^0 and no nf dependent term
    CALL RMBfin2Loop_L0_nf0(x,costh,Mfin2Loop0noNfR,interp2)
    IF(Mfin2Loop0noNfR.EQ.0d0)THEN
       ! If it is outside the interpolation range
       res=0d0
       RETURN
    ENDIF
    ! convert from Catani's I subtraction to SCET subtraction
    res=(Mfin1Loop0R**2+Mfin1Loop0I**2)/four+half*(Mfin2Loop2R+Mfin2Loop1R+Mfin2Loop0NfR+Mfin2Loop0noNfR)
    res=res+Mfin1Loop0R*(-four*RHL+CF*(three*rL-rL2+7d0*zeta2))
    res=res+16D0*RHL**2+8D0*CF*RHL*(-three*rL+rL2-7D0*zeta2)+(CF*(CA*(-699D0*rL2+66D0*rL3+4146D0*zeta2+108D0*rL2*zeta2&
         -675D0*zeta2**2+rL*(961D0-792D0*zeta2-1404D0*zeta3)-99D0*zeta3)+four*nfTF*&
         (57D0*rL2-6D0*rL3-336D0*zeta2+rL*(-65D0+72D0*zeta2)+9D0*zeta3)+27D0*CF*&
         (two*(9D0*rL2-6D0*rL3+rL4-14D0*rL2*zeta2+49D0*zeta2**2)+rL*(three+60D0*zeta2+48D0*zeta3))))/108D0
    RETURN
  END FUNCTION bbHH_H2B

  ! 1-loop class-B amplitude (arXiv:1811.01853) normalised by the Born amplitude
  SUBROUTINE MBfin1Loop(RHL,x,y,amp)
    ! Here RHL=LOG(mh**2/mu**2)
    ! s12=mh**2*(1+x)**2/x
    ! t=s13=-mh**2*y
    ! y=(1+costh+x**2-costh*x**2)/(2*x) where costh is cosine of the scattering angle in the initial partonic rest frame
    ! the amplitude is symmetric for costh and -costh
    ! We have no nf dependence at 1-loop
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::RHL,x,y
    COMPLEX(KIND(1d0)),INTENT(OUT)::amp
    REAL(KIND(1d0))::ramp,iamp
    CALL MBfin1Loop_L0R(x,y,ramp)
    CALL MBfin1Loop_L0I(x,y,iamp)
    ! The RHL dependence is predicted by RGE
    amp=-8d0*RHL+DCMPLX(ramp,iamp)
    RETURN
  END SUBROUTINE MBfin1Loop

  ! 2-loop class-B amplitude (arXiv:1811.01853) normalised by the Born amplitude
  SUBROUTINE MBfin2Loop(RHL,x,y,rnf,amp,interp)
    ! Here RHL=LOG(mh**2/mu**2)
    ! s12=mh**2*(1+x)**2/x
    ! t=s13=-mh**2*y
    ! y=(1+costh+x**2-costh*x**2)/(2*x) where costh is cosine of the scattering angle in the initial partonic rest frame
    ! the amplitude is symmetric for costh and -costh
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::RHL,x,y,rnf
    COMPLEX(KIND(1d0)),INTENT(OUT)::amp
    INTEGER,INTENT(IN),OPTIONAL::interp ! the method of interpolation : 1, HTL_INTERPOLATE_2D; 2, lagrange_interp_2d; 3, pwl_interp_2d
    INTEGER::interp2
    REAL(KIND(1d0))::ramp,iamp,z
    COMPLEX(KIND(1d0))::ampl2,ampl1,ampl0nf
    REAL(KIND(1d0))::costh
    COMPLEX(KIND(1d0))::ampl0nf0
    costh=(1d0+x**2-2d0*x*y)/(-1d0+x**2)
    z=(1d0+x)**2/x-y-2d0
    ! L^2 term
    ramp=4d0*(2d0*rnf-57d0)*z*(-1d0-x**2+2d0*x*y)*(1d0+x**2-x*z)&
         /(3d0*y*(-1d0-x**2+x*y)*(1d0+x**2-2d0*x*z))
    ampl2=DCMPLX(RHL**2*ramp,0d0)
    ! L^1 term
    CALL MBfin2Loop_L1R(x,y,rnf,ramp)
    CALL MBfin2Loop_L1I(x,y,rnf,iamp)
    ampl1=DCMPLX(RHL*ramp,RHL*iamp)
    IF(PRESENT(interp))THEN
       interp2=interp
       IF(interp2.GT.3.OR.interp2.LT.1)interp2=1
    ELSE
       interp2=1
    ENDIF
    ! L^0 but nf dependent term
    CALL MBfin2Loop_L0_nf1_R(x,y,rnf,ramp)
    CALL MBfin2Loop_L0_nf1_I(x,y,rnf,iamp)
    ampl0nf=DCMPLX(ramp,iamp)
    ! L^0 and no nf dependent term
    CALL MBfin2Loop_L0_nf0(x,costh,ampl0nf0,interp2)
    ! The RHL dependence is predicted by RGE
    amp=ampl2+ampl1+ampl0nf+ampl0nf0
    RETURN
  END SUBROUTINE MBfin2Loop

  ! 2-loop class-B amplitude (arXiv:1811.01853) normalised by the Born amplitude
  ! The grid are generated in Mathematica with the help of PolyLogTools
  ! This is just non log(mu) dependent and non nf (number of light quark flavours) dependent term
  SUBROUTINE MBfin2Loop_L0_nf0(x,costh,amp,interp)
    ! s12=mh**2*(1+x)**2/x
    ! t=s13=-mh**2*y
    ! y=(1+costh+x**2-costh*x**2)/(2*x) where costh is cosine of the scattering angle in the initial partonic rest frame
    ! the amplitude is symmetric for costh and -costh
    USE interpolation
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::x,costh
    COMPLEX(KIND(1d0)),INTENT(OUT)::amp
    INTEGER,INTENT(IN),OPTIONAL::interp ! the method of interpolation : 1, HTL_INTERPOLATE_2D; 2, lagrange_interp_2d; 3, pwl_interp_2d
    INTEGER::interp2
    INTEGER::nx,ncth,ix,icth,i,j
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::tabx,tabcth
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::ramp_grid,iamp_grid
    REAL(KIND(1d0))::ramp,iamp
    INTEGER::init=0
    SAVE init,nx,ncth,tabx,tabcth,ramp_grid,iamp_grid
    LOGICAL::lexist
    INTEGER,PARAMETER::funit=31244
    REAL(KIND(1d0))::tmp1,tmp2
    CHARACTER(len=10)::comment
    REAL(KIND(1d0)),DIMENSION(1)::XI3,YI3,ZI3
    INTEGER,PARAMETER::n_interp=6
    REAL(KIND(1d0)),DIMENSION(n_interp)::XD2,YD2
    REAL(KIND(1d0)),DIMENSION(n_interp,n_interp)::RZD2,IZD2
    INTEGER::ninterp_m, ninterp_p
    SAVE ninterp_m,ninterp_p
    IF(init.EQ.0)THEN
       ninterp_p=(n_interp-1)/2
       ninterp_m=n_interp-1-ninterp_p
       ! getenv is for GUN compiler
       !call getenv("HOPATH", gdir)
       INQUIRE(FILE=TRIM(input_dir)//"bbHH_MBfin2Loop_L0_nf0.grid",EXIST=lexist)
       IF(.NOT.lexist)THEN
          WRITE(*,*)"ERROR: Cannot find the grid bbHH_MBfin2Loop_L0_nf0.grid"
          STOP
       ENDIF
       WRITE(*,*)"INFO: We are loading the grid file"
       OPEN(UNIT=funit,FILE=TRIM(input_dir)//"bbHH_MBfin2Loop_L0_nf0.grid")
       READ(funit,*)nx,ncth
       ALLOCATE(tabx(nx))
       ALLOCATE(tabcth(ncth))
       ALLOCATE(ramp_grid(nx,ncth))
       ALLOCATE(iamp_grid(nx,ncth))
       DO ix=1,nx
          READ(funit,'(a)')comment
          DO icth=1,ncth
             ramp_grid(ix,icth)=0d0
             iamp_grid(ix,icth)=0d0
             READ(funit,*)tmp1,tmp2,ramp_grid(ix,icth),iamp_grid(ix,icth)
             IF(icth.EQ.1)THEN
                tabx(ix)=tmp1
             ENDIF
             IF(ix.EQ.1)THEN
                tabcth(icth)=tmp2
             ENDIF
          ENDDO
       ENDDO
       CLOSE(UNIT=funit)
       IF(ramp_grid(nx-1,ncth-1).NE.0d0)THEN
          WRITE(*,*)"INFO: The grid file is loaded successfully !"
       ELSE
          WRITE(*,*)"ERROR: The grid file is not loaded or not correct !"
          STOP
       ENDIF
       init=1
    ENDIF
    IF(PRESENT(interp))THEN
       interp2=interp
       IF(interp2.GT.3.OR.interp2.LT.1)interp2=1
    ELSE
       interp2=1
    ENDIF
    IF(interp2.EQ.1)THEN
       ! a compromise between 2 and 3
       ramp=HTL_INTERPOLATE_2d(x,DABS(costh),nx,ncth,tabx,tabcth,ramp_grid)
       iamp=HTL_INTERPOLATE_2d(x,DABS(costh),nx,ncth,tabx,tabcth,iamp_grid)
    ELSEIF(interp2.EQ.2)THEN
       ! Lagrange interpolate
       ! most accurate but slowest
       XI3(1)=x
       YI3(1)=DABS(costh)
       IF(XI3(1).GT.tabx(nx).OR.XI3(1).LT.tabx(1).OR.YI3(1).GT.tabcth(ncth).OR.YI3(1).LT.tabcth(1))THEN
          amp=DCMPLX(0d0,0d0)
          RETURN
       ENDIF
       ix=0
       DO i=1,nx
          IF(XI3(1).GE.tabx(i))ix=ix+1
       ENDDO
       IF(ix.LE.ninterp_m)ix=ninterp_m+1
       IF(ix.GT.nx-ninterp_p)ix=nx-ninterp_p
       XD2(1:n_interp)=tabx(ix-ninterp_m:ix+ninterp_p)
       icth=0
       DO i=1,ncth
          IF(YI3(1).GE.tabcth(i))icth=icth+1
       ENDDO
       IF(icth.LE.ninterp_m)icth=ninterp_m+1
       IF(icth.GT.ncth-ninterp_p)icth=ncth-ninterp_p
       YD2(1:n_interp)=tabcth(icth-ninterp_m:icth+ninterp_p)
       DO i=1,n_interp
          DO j=1,n_interp
             RZD2(i,j)=ramp_grid(ix-ninterp_m+i-1,icth-ninterp_m+j-1)
             IZD2(i,j)=iamp_grid(ix-ninterp_m+i-1,icth-ninterp_m+j-1)
          ENDDO
       ENDDO
       CALL lagrange_interp_2d(n_interp-1,n_interp-1,XD2,YD2,RZD2,1,XI3,YI3,ZI3)
       ramp=ZI3(1)
       CALL lagrange_interp_2d(n_interp-1,n_interp-1,XD2,YD2,IZD2,1,XI3,YI3,ZI3)
       iamp=ZI3(1)
    ELSEIF(interp2.EQ.3)THEN
       ! piecewise linear interpolate
       ! fastest but less accurate
       XI3(1)=x
       YI3(1)=DABS(costh)
       IF(XI3(1).GT.tabx(nx).OR.XI3(1).LT.tabx(1).OR.YI3(1).GT.tabcth(ncth).OR.YI3(1).LT.tabcth(1))THEN
          amp=DCMPLX(0d0,0d0)
          RETURN
       ENDIF
       CALL pwl_interp_2d(nx,ncth,tabx,tabcth,ramp_grid,1,XI3,YI3,ZI3)
       ramp=ZI3(1)
       CALL pwl_interp_2d(nx,ncth,tabx,tabcth,iamp_grid,1,XI3,YI3,ZI3)
       iamp=ZI3(1)
    ENDIF
    amp=DCMPLX(ramp,iamp)
    RETURN
  END SUBROUTINE MBfin2Loop_L0_nf0

  ! 2-loop class-B amplitude (arXiv:1811.01853) normalised by the Born amplitude (the real part only !!!)
  ! The grid are generated in Mathematica with the help of PolyLogTools
  ! This is just non log(mu) dependent and non nf (number of light quark flavours) dependent term
  SUBROUTINE RMBfin2Loop_L0_nf0(x,costh,ramp,interp)
    ! s12=mh**2*(1+x)**2/x
    ! t=s13=-mh**2*y
    ! y=(1+costh+x**2-costh*x**2)/(2*x) where costh is cosine of the scattering angle in the initial partonic rest frame
    ! the amplitude is symmetric for costh and -costh
    USE interpolation
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::x,costh
    REAL(KIND(1d0)),INTENT(OUT)::ramp
    INTEGER,INTENT(IN),OPTIONAL::interp ! the method of interpolation : 1, HTL_INTERPOLATE_2D; 2, lagrange_interp_2d; 3, pwl_interp_2d
    INTEGER::interp2
    INTEGER::nx,ncth,ix,icth,i,j
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::tabx,tabcth
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::ramp_grid
    INTEGER::init=0
    SAVE init,nx,ncth,tabx,tabcth,ramp_grid
    LOGICAL::lexist
    INTEGER,PARAMETER::funit=31245
    REAL(KIND(1d0))::tmp1,tmp2,tmp3
    CHARACTER(len=10)::comment
    REAL(KIND(1d0)),DIMENSION(1)::XI3,YI3,ZI3
    INTEGER,PARAMETER::n_interp=6
    REAL(KIND(1d0)),DIMENSION(n_interp)::XD2,YD2
    REAL(KIND(1d0)),DIMENSION(n_interp,n_interp)::RZD2
    INTEGER::ninterp_m, ninterp_p
    SAVE ninterp_m,ninterp_p
    IF(init.EQ.0)THEN
       ninterp_p=(n_interp-1)/2
       ninterp_m=n_interp-1-ninterp_p
       ! getenv is for GUN compiler
       !call getenv("HOPATH", gdir)
       INQUIRE(FILE=TRIM(input_dir)//"bbHH_MBfin2Loop_L0_nf0.grid",EXIST=lexist)
       IF(.NOT.lexist)THEN
          WRITE(*,*)"ERROR: Cannot find the grid bbHH_MBfin2Loop_L0_nf0.grid"
          STOP
       ENDIF
       WRITE(*,*)"INFO: We are loading the grid file"
       OPEN(UNIT=funit,FILE=TRIM(input_dir)//"bbHH_MBfin2Loop_L0_nf0.grid")
       READ(funit,*)nx,ncth
       ALLOCATE(tabx(nx))
       ALLOCATE(tabcth(ncth))
       ALLOCATE(ramp_grid(nx,ncth))
       DO ix=1,nx
          READ(funit,'(a)')comment
          DO icth=1,ncth
             ramp_grid(ix,icth)=0d0
             READ(funit,*)tmp1,tmp2,ramp_grid(ix,icth),tmp3
             IF(icth.EQ.1)THEN
                tabx(ix)=tmp1
             ENDIF
             IF(ix.EQ.1)THEN
                tabcth(icth)=tmp2
             ENDIF
          ENDDO
       ENDDO
       CLOSE(UNIT=funit)
       IF(ramp_grid(nx-1,ncth-1).NE.0d0)THEN
          WRITE(*,*)"INFO: The grid file is loaded successfully !"
       ELSE
          WRITE(*,*)"ERROR: The grid file is not loaded or not correct !"
          STOP
       ENDIF
       init=1
    ENDIF
    IF(PRESENT(interp))THEN
       interp2=interp
       IF(interp2.GT.3.OR.interp2.LT.1)interp2=1
    ELSE
       interp2=1
    ENDIF
    IF(interp2.EQ.1)THEN
       ! a compromise between 2 and 3
       ramp=HTL_INTERPOLATE_2d(x,DABS(costh),nx,ncth,tabx,tabcth,ramp_grid)
    ELSEIF(interp2.EQ.2)THEN
       ! Lagrange interpolate
       ! most accurate but slowest
       XI3(1)=x
       YI3(1)=DABS(costh)
       IF(XI3(1).GT.tabx(nx).OR.XI3(1).LT.tabx(1).OR.YI3(1).GT.tabcth(ncth).OR.YI3(1).LT.tabcth(1))THEN
          ramp=0d0
          RETURN
       ENDIF
       ix=0
       DO i=1,nx
          IF(XI3(1).GE.tabx(i))ix=ix+1
       ENDDO
       IF(ix.LE.ninterp_m)ix=ninterp_m+1
       IF(ix.GT.nx-ninterp_p)ix=nx-ninterp_p
       XD2(1:n_interp)=tabx(ix-ninterp_m:ix+ninterp_p)
       icth=0
       DO i=1,ncth
          IF(YI3(1).GE.tabcth(i))icth=icth+1
       ENDDO
       IF(icth.LE.ninterp_m)icth=ninterp_m+1
       IF(icth.GT.ncth-ninterp_p)icth=ncth-ninterp_p
       YD2(1:n_interp)=tabcth(icth-ninterp_m:icth+ninterp_p)
       DO i=1,n_interp
          DO j=1,n_interp
             RZD2(i,j)=ramp_grid(ix-ninterp_m+i-1,icth-ninterp_m+j-1)
          ENDDO
       ENDDO
       CALL lagrange_interp_2d(n_interp-1,n_interp-1,XD2,YD2,RZD2,1,XI3,YI3,ZI3)
       ramp=ZI3(1)
    ELSEIF(interp2.EQ.3)THEN
       ! piecewise linear interpolate
       ! fastest but less accurate
       XI3(1)=x
       YI3(1)=DABS(costh)
       IF(XI3(1).GT.tabx(nx).OR.XI3(1).LT.tabx(1).OR.YI3(1).GT.tabcth(ncth).OR.YI3(1).LT.tabcth(1))THEN
          ramp=0d0
          RETURN
       ENDIF
       CALL pwl_interp_2d(nx,ncth,tabx,tabcth,ramp_grid,1,XI3,YI3,ZI3)
       ramp=ZI3(1)
    ENDIF
    RETURN
  END SUBROUTINE RMBfin2Loop_L0_nf0
END MODULE bbHH_NNLO_hard_function
