MODULE bbH_NNLO_hard_function
  ! This is the hard function and born partonic cross section of
  ! b b~ > h process
  ! L=Log(-s12/mu**2)=Log(-MH**2/mu**2)
  USE qcd_constants
  USE global_constants
  IMPLICIT NONE
  PRIVATE
  PUBLIC::bbH_sigma0
  PUBLIC::bbH_H1,bbH_H2
CONTAINS
  ! it has a global yq**2 factor and two-body phase space
  FUNCTION bbH_sigma0(mh2)
    IMPLICIT NONE
    REAL(KIND(1d0))::bbH_sigma0
    REAL(KIND(1d0)),INTENT(IN)::mh2 ! mass square of Higgs
    bbH_sigma0=PI/(12d0*mh2)
    RETURN
  END FUNCTION bbH_sigma0


  ! (as/2/pi)**1 term of hard function
  FUNCTION bbH_H1(L) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0))::res
    res=CF*(-DREAL(L**2)+zeta2-2d0)
    RETURN
  END FUNCTION bbH_H1

  ! (as/2/pi)**2 term of hard function
  FUNCTION bbH_H2(L) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0))::rL,rL2,rL3,rL4,aL4
    REAL(KIND(1d0))::res
    ! WARNING: we should take the module of the amplitude
    ! instead of Re(A^2) at NNLO
    ! the difference is from one-loop square
    rL=DREAL(L)
    rL2=DREAL(L**2)
    rL3=DREAL(L**3)
    rL4=DREAL(L**4)
    aL4=ABS(L**2)**2
    res=CF**2*((rL4+aL4)/four-rL2*(zeta2-two)+6d0*rL*(two*zeta3-zeta2)&
         +(-39d0/10d0*zeta2**2-15d0*zeta3+6d0*zeta2+four))&
         +CA*CF*(11d0/18d0*rL3+rL2*(zeta2-67d0/18d0)+rL*(-13d0*zeta3+11d0/three*zeta2+121d0/27d0)&
         +22d0/5d0*zeta2**2+151d0/18d0*zeta3-103d0/36d0*zeta2-467d0/162d0)&
         +CF*nfTF*(-two/9d0*rL3+10d0/9d0*rL2-rL*(four/three*zeta2+56d0/27d0)&
         +two/9d0*zeta3+5d0/9d0*zeta2+200d0/81d0)
    RETURN
  END FUNCTION bbH_H2

END MODULE bbH_NNLO_hard_function
