MODULE ggH_NNLO_hard_function
  ! This is the hard function and born partonic cross section of
  ! g g > h process
  ! L=Log(-MH**2/mu**2), Lt=Log(mu**2/MT**2), MT is the pole mass of top
  USE qcd_constants
  USE global_constants
  IMPLICIT NONE
  PRIVATE
  PUBLIC::ggH_sigma0
  PUBLIC::ggH_H1,ggH_H2
CONTAINS
  ! it has a global (as/2/pi)**2 factor
  FUNCTION ggH_sigma0(vev)
    IMPLICIT NONE
    REAL(KIND(1d0))::ggH_sigma0
    REAL(KIND(1d0)),INTENT(IN)::vev
    ggH_sigma0=PI/(144d0*vev**2)
    RETURN
  END FUNCTION ggH_sigma0

  ! (as/2/pi) term of hard function
  FUNCTION ggH_H1(L,Lt) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L,Lt
    REAL(KIND(1d0))::res
    res=CA*(-DREAL(L**2)+zeta2+5d0)-three*CF
    RETURN
  END FUNCTION ggH_H1

  ! (as/2/pi)**2 term of hard function
  FUNCTION ggH_H2(L,Lt) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L,Lt
    REAL(KIND(1d0))::rL,rLt,rL2,rL3,rL4,aL4
    REAL(KIND(1d0))::res
    ! WARNING: we should take the module of the amplitude
    ! i.e., |Cg|^2*|Ch|^2 instead of Re(Cg^2*Ch^2)
    ! at NNLO, the difference is from one-loop square
    rL=DREAL(L)
    rLt=DREAL(Lt)
    rL2=DREAL(L**2)
    rL3=DREAL(L**3)
    rL4=DREAL(L**4)
    aL4=ABS(L**2)**2
    res=CA**2*((rL4+aL4)/four+11d0/18d0*rL3-157d0/18d0*rL2&
         +rL*(-zeta3-11d0/3d0*zeta2+40d0/27d0)+7d0/2d0*rLt&
         +zeta2**2/two+127d0/12d0*zeta2-143d0/18d0*zeta3&
         +23827d0/648d0)+CA*CF*(three*rL2-11d0/two*rLt-three*zeta2&
         -145d0/6d0)+9d0*CF**2-5d0/12d0*CA*Tf-two/three*CF*Tf&
         +CA*nfTF*(-two/9d0*rL3+10d0/9d0*rL2+rL*(four/three*zeta2+52d0/27d0)&
         -5d0/three*zeta2-46d0/9d0*zeta3-2255d0/162d0)&
         +CF*nfTF*(two*rL+four*rLt+8d0*zeta3-41d0/three)
    RETURN
  END FUNCTION ggH_H2
END MODULE ggH_NNLO_hard_function
