MODULE gg_Higgs_NNLO
  USE Helac_Global
  USE gg_Higgs_NNLO_Global
  USE Helac_Func_1
  USE plot_gg_Higgs_NNLO
  USE Constants
  USE Kinetic_Func
  USE gg_Higgs_NNLO_setscale
  IMPLICIT NONE
  INTEGER::nprint
  INTEGER::varnum,nunwei
  LOGICAL::lunwei2
  REAL(KIND(1d0))::EBMUP1,EBMUP2
  INTEGER::NPRUP=0,lwmax
  INTEGER,PARAMETER::maxprint=8
  LOGICAL::force_initial_reweight_scale=.FALSE.
  LOGICAL::force_initial_reweight_pdf=.FALSE.
  LOGICAL::force_initial_reweight_npdf=.FALSE.
  LOGICAL::force_initial_reweight_ref=.FALSE.
  LOGICAL::force_initial_reweight_muNRQCD=.FALSE.
  REAL(KIND(1d0))::weight_Romberg
  SAVE
CONTAINS
  SUBROUTINE calc_gg_Higgs_NNLO
    USE qcd_setup
    USE ProcessInfo
    IMPLICIT NONE
    CHARACTER(len=2)::quark_tag
    INTEGER::i,i1,j1,k1,s1x
    INTEGER::nh,innum,outnum
    INTEGER::ioerror,icase=0
    LOGICAL::lunwei=.FALSE.,lhewgt=.FALSE.
    REAL(KIND(1d0)),DIMENSION(3)::rslt
    INTEGER::itmxn,ncalln
    REAL(KIND(1d0)),DIMENSION(4)::pmomtemp
    LOGICAL::lexist
    INTEGER::lfound
    SAVE lunwei,lhewgt,icase
    REAL(KIND(1d0))::temp
    INTEGER::beam1_pdf,beam2_pdf
    WRITE(*,*)'                     THE BEGINNING OF HELAC-Onia'
    WRITE(*,*)'    AddOn Process: '
    WRITE(*,*)'    g g > Higgs/Eta(Q) + X up to O(alpha_s**4)'
    WRITE(*,*)'======================================================='
    WRITE(*,*)'======================================================='
    WRITE(*,*)' '
    CALL ReadElem_integer('colpar',Coll_Type)
    lunwei=.FALSE.
    lhewgt=.FALSE.
    CALL ReadElem_logic('topdrawer_output',topdrawer_output)
    CALL ReadElem_logic('gnuplot_output',gnuplot_output)
    CALL ReadElem_logic('root_output',root_output)
    CALL ReadElem_logic('hwu_output',hwufile_output)
    plot_output=topdrawer_output.OR.gnuplot_output.OR.root_output.OR.hwufile_output
    CALL ReadElem_integer('literature',literature_cutoffs)
    CALL ReadElem_logic('fixtarget',fixtarget)
    IF(fixtarget.AND.Coll_Type.EQ.3)THEN
       WRITE(*,*)"ERROR: Cannot treat fixed-target experiment in e+e- collisions"
       STOP
    ENDIF
    CALL ReadElem_logic("useMCFMrun",useMCFMrun)
    CALL ReadElem_logic('lhapdf',uselhapdf)
    INQUIRE(FILE=TRIM(input_dir)//"paths/lhapdf_data",EXIST=lexist)
    IF(lexist)THEN
       OPEN(UNIT=30333,FILE=TRIM(input_dir)//"paths/lhapdf_data")
       READ(30333,'(A)')LHAPath
       CLOSE(UNIT=30333)
    ELSE
       INQUIRE(FILE=TRIM(input_dir)//"paths/lhapdfpath",EXIST=lexist)
       IF(lexist)THEN
          OPEN(UNIT=30333,FILE=TRIM(input_dir)//"paths/lhapdfpath")
          READ(30333,'(A)')LHAPath
          CLOSE(UNIT=30333)
          i=LEN_TRIM(LHAPath)
          LHAPath=LHAPath(1:i-13)
       ENDIF
    ENDIF
    uselhapdf=uselhapdf.AND.lexist
    WRITE(*,*)'Use LHAPDF ?',uselhapdf
    CALL ReadElem_logic("absoluterap",absrap)
    ! some necesarry initialization
    OPEN(UNIT=12321,FILE="./input/states.inp")
    READ(12321,*)Higgs_state
    CLOSE(UNIT=12321)
    CALL Read_ProcessInfo()
    lfound=0
    DO i=1,n_process
       IF(process_ID(i).EQ.Higgs_state)THEN
          lfound=i
          EXIT
       ENDIF
    ENDDO
    IF(lfound.EQ.0)THEN
       CALL Print_ProcessInfo(.FALSE.)
       WRITE(*,*)"Error: Do not find right process !"
       STOP
    ELSEIF(process_status(lfound).EQ."N ")THEN
       CALL Print_ProcessInfo(.TRUE.)
       WRITE(*,*)"Error: Do not find right process !"
       STOP
    ENDIF
    WRITE(*,*)"INFO: Running "//TRIM(process_info(lfound))

    OPEN(UNIT=12321,FILE="./input/order_ptcut.inp")
    READ(12321,*)MEorder,H_pTveto
    CLOSE(UNIT=12321)
    IF(MEorder.LT.0.OR.MEorder.GT.2)THEN
       WRITE(*,*)"ERROR:the order exceeds the range in 0-2: ",MEorder
       STOP
    ENDIF
    IF(H_pTveto.LE.0d0)THEN
       WRITE(*,*)"ERROR:the Higgs pT veto should be larger than 0"
       STOP
    ENDIF

    OPEN(UNIT=12321,FILE="./input/integrator.inp")
    READ(12321,*)integrator
    CLOSE(UNIT=12321)
    IF(integrator.LT.1.OR.integrator.GT.2)THEN
       WRITE(*,*)"ERROR:unknown the integrator: ",integrator
       STOP
    ENDIF

    CALL setup_process(Higgs_state)

    ! set QCD colour factors
    CALL SET_QCD_GROUP(ggH_QCD_CA,ggH_QCD_CF,ggH_QCD_TF)
    CALL SET_QCD_NF(nlq)

    !CALL ReadDecayInfo
    IF(nunit1.NE.6)THEN
       OPEN(UNIT=nunit1,FILE=TRIM(output_dir)//"RESULT_gg_Higgs_NNLO.out")
    ENDIF
    nunit2=32
    CLOSE(nunit2)
    OPEN(nunit2,FILE=TRIM(output_dir)//'kine_gg_Higgs_NNLO.out')
    nunit3=30
    CLOSE(nunit3)
    OPEN(nunit3,FILE=TRIM(tmp_dir)//'even_gg_Higgs_NNLO.out',FORM='unformatted')
    ! for LHA
    CLOSE(200)
    OPEN(200,FILE=TRIM(tmp_dir)//'sample_gg_Higgs_NNLO.init')
    ! Multiple PDFs and special nPDFs
    CALL ReadElem_integer('pdf',beam1_pdf)
    CALL ReadElem_integer('beam2_pdf',beam2_pdf)
    CALL ReadElem_integer('nPDF_id',nPDF_id)
    CALL ReadElem_integer('nuclearA_beam1',nuclearA_beam1)
    CALL ReadElem_integer('nuclearA_beam2',nuclearA_beam2)
    CALL ReadElem_integer('nuclearZ_beam1',nuclearZ_beam1)
    CALL ReadElem_integer('nuclearZ_beam2',nuclearZ_beam2)
    lnuclearPDF=.FALSE.
    multipdfloading=uselhapdf.AND.beam2_pdf.NE.beam1_pdf.AND.beam2_pdf.GT.0
    IF(.NOT.multipdfloading)THEN
       IF(nPDF_id.NE.0.AND.(nuclearA_beam1.NE.0.or.nuclearA_beam2.NE.0)&
            .AND.(.NOT.uselhapdf))THEN
          WRITE(*,*)"nuclear PDF can only be used with LHAPDF"
          WRITE(*,*)"nPDF_id=",nPDF_id
          WRITE(*,*)"nuclearA_beam1=",nuclearA_beam1
          WRITE(*,*)"nuclearA_beam2=",nuclearA_beam2
          WRITE(*,*)"Please set lhapdf=T and set the correct path of lhapdf in ho_configuration.txt before ./config"
          STOP
       ENDIF
       rnuclearA_beam1=DBLE(nuclearA_beam1)
       rnuclearA_beam2=DBLE(nuclearA_beam2)
       rnuclearZ_beam1=DBLE(nuclearZ_beam1)
       rnuclearZ_beam2=DBLE(nuclearZ_beam2)
       IF(nPDF_id.EQ.0.AND.(nuclearA_beam1.NE.0.or.nuclearA_beam2.NE.0))THEN
          WRITE(*,*)"WARNING:you are using normal proton PDF for nuclear"
       ENDIF
       IF(nPDF_id.NE.0.AND.(nuclearA_beam1.NE.0.or.nuclearA_beam2.NE.0))THEN
          lnuclearPDF=.TRUE.
       ENDIF
    ELSE
       ! multiple PDF loading already allow one to use nuclear PDFs
       ! no need the tricks for nuclear PDFs
       rnuclearA_beam1=0d0
       rnuclearA_beam2=0d0
       nuclearA_beam1=0
       nuclearA_beam2=0
       rnuclearZ_beam1=0d0
       rnuclearZ_beam2=0d0
       nuclearZ_beam1=0
       nuclearZ_beam2=0
       lnuclearPDF=.FALSE.
    ENDIF
    ! Scale scheme
    CALL ReadElem_integer('Scale',nscheme)
    CALL ReadElem_real('FScaleValue',fschemevalue)
    CALL ReadELem_real('ScaleFactor',scalefactor)
    CALL ReadElem_real('muR_over_ref',muR_over_ref)
    CALL ReadElem_real('muF_over_ref',muF_over_ref)
    IF(muF_over_ref.NE.muR_over_ref)THEN
       WRITE(*,*)"WARNING: muR_over_ref =!= muF_over_ref is not allowed"
       muF_over_ref=muR_over_ref
    ENDIF
    ! reweighting stuff
    reweight_pdf=.FALSE.
    reweight_npdf=.FALSE.
    reweight_ref=.FALSE.
    IF(uselhapdf)THEN
       CALL ReadElem_logic('reweight_pdf',reweight_pdf)
       IF(reweight_pdf.AND.lnuclearPDF.AND.nPDF_id.EQ.1000)THEN
          WRITE(*,*)"WARNING:reweight proton PDF are not allowed when nPDF_id=1000 in PA"
          reweight_pdf=.FALSE.
       ENDIF
       IF(reweight_pdf)THEN
          CALL ReadElem_logic('beams_pdf_correlation',beams_pdf_correlation)
       ENDIF
       IF(reweight_pdf)THEN
          CALL ReadElem_integer('pdf_min',ipdf_set_min)
          CALL ReadElem_integer('pdf_max',ipdf_set_max)
          IF(multipdfloading)THEN
             CALL ReadElem_integer('beam2_pdf_min',ipdf2_set_min)
             CALL ReadElem_integer('beam2_pdf_max',ipdf2_set_max)
             IF(ipdf2_set_min.GT.0.AND.ipdf2_set_max.GT.0)THEN
                IF(beams_pdf_correlation.AND.&
                     (ipdf2_set_max-ipdf2_set_min).NE.&
                     (ipdf_set_max-ipdf_set_min))THEN
                   WRITE(*,*)"ERROR: The numbers of error PDFs for two beams are different"
                   WRITE(*,*)"ERROR: The two beams cannot be fully correlated "
                   STOP
                ENDIF
             ENDIF
          ELSE
             ipdf2_set_min=-1
             ipdf2_set_max=-1
          ENDIF
          IF(ipdf_set_max.LT.0.AND.ipdf_set_min.LT.0)THEN
             ho_npdf=0
          ELSE
             ho_npdf=ipdf_set_max-ipdf_set_min+1
          ENDIF
          IF(multipdfloading.AND..NOT.beams_pdf_correlation.AND.&
               (ipdf2_set_min.GT.0.AND.ipdf2_set_max.GT.0))THEN
             ho_npdf2=ipdf2_set_max-ipdf2_set_min+1
          ELSE
             ho_npdf2=0
          ENDIF
       ENDIF
       ho_npdf=MAX(ho_npdf,0)
       ho_npdf2=MAX(ho_npdf2,0)
       IF(ho_npdf.LE.0.AND.ho_npdf2.LE.0)THEN
          ho_npdf=0
          ho_npdf2=0
          reweight_pdf=.FALSE.
       ENDIF
       IF(ho_npdf.GT.0)THEN
          tot_nrwgt=tot_nrwgt+ho_npdf+1
       ENDIF
       IF(ho_npdf2.GT.0)THEN
          tot_nrwgt=tot_nrwgt+ho_npdf2+1
       ENDIF
       IF(lnuclearPDF)THEN
          CALL ReadElem_logic('reweight_npdf',reweight_npdf)
          IF(reweight_pdf.AND.ho_npdf.GE.2.AND.nPDF_id.LE.500)THEN
             WRITE(*,*)"ERROR: Cannot perform proton PDF reweighting for EPS09"
             STOP
          ENDIF
          IF(reweight_npdf)THEN
             CALL ReadElem_integer('npdf_min',inpdf_set_min)
             CALL ReadElem_integer('npdf_max',inpdf_set_max)
             ho_nnpdf=inpdf_set_max-inpdf_set_min+1
             tot_nrwgt=tot_nrwgt+ho_nnpdf+1
          ENDIF
          INCLUDE '../../../input/lhapdf/check_npdf_set_min'
          CALL ReadElem_logic('include_ref',reweight_ref)
          IF(reweight_ref)THEN
             IF(nuclearA_beam1*nuclearA_beam2.NE.0.AND.COLL_TYPE.LE.2)THEN
                ! heavy ion collision, AB or AA
                ho_nref=3
                iref_set_min=1
                iref_set_max=3
             ELSE
                ! pA or Ap or eA collisions
                ho_nref=1
                iref_set_min=1
                iref_set_max=1
             ENDIF
          ENDIF
          tot_nrwgt=tot_nrwgt+ho_nref+1
       ENDIF
    ENDIF
    CALL ReadElem_integer('alphasrun',irun)
    CALL ReadElem_logic('reweight_Scale',reweight_scale)
    CALL ReadElem_logic('reweight_muNRQCD',reweight_muNRQCD)
    CALL ReadElem_logic('LDME_RG_RUN',LDME_RG_RUN)
    CALL ReadElem_logic('muNRQCD_eq_QMass',muNRQCD_eq_QMass)
    reweight_scale=reweight_scale.AND.(irun.EQ.1)
    LDME_RG_RUN=LDME_RG_RUN.AND.(irun.EQ.1).AND.(Higgs_state.NE.1)
    reweight_muNRQCD=reweight_muNRQCD.AND.LDME_RG_RUN
    IF(reweight_scale)THEN
       CALL ReadElem_real('rw_RScale_down',rw_Rscale_down)
       CALL ReadElem_real('rw_RScale_up',rw_Rscale_up)
       CALL ReadElem_real('rw_FScale_down',rw_Fscale_down)
       CALL ReadElem_real('rw_FScale_up',rw_Fscale_up)
       ho_nscale=8
       IF(rw_Rscale_down.LE.0d0.OR.rw_Rscale_up.LE.0d0.OR.&
            rw_Fscale_down.LE.0d0.OR.rw_Fscale_up.LE.0d0)THEN
          reweight_scale=.FALSE.
          ho_nscale=0
          WRITE(*,*)"WARNING:Scale reweighting is off."
          WRITE(*,*)"WARNING:Please make sure rw_Rscale_up, rw_Rscale_down, rw_Fscale_up, rw_Fscale_down are larger than 0."
       ENDIF
       IF(reweight_scale.AND.rw_Rscale_down.GT.rw_Rscale_up)THEN
          temp=rw_Rscale_down
          rw_Rscale_down=rw_Rscale_up
          rw_Rscale_up=temp
       ENDIF
       IF(reweight_scale.AND.rw_Fscale_down.GT.rw_Fscale_up)THEN
          temp=rw_Fscale_down
          rw_Fscale_down=rw_Fscale_up
          rw_Fscale_up=temp
       ENDIF
       rw_Rscales(1)=1d0
       rw_Rscales(2)=rw_Rscale_up
       rw_Rscales(3)=rw_Rscale_down
       rw_Fscales(1)=1d0
       rw_Fscales(2)=rw_Fscale_up
       rw_Fscales(3)=rw_Fscale_down
    ENDIF
    IF(reweight_scale)THEN
       tot_nrwgt=tot_nrwgt+ho_nscale+1
    ENDIF
    IF(reweight_muNRQCD)THEN
       CALL ReadElem_real('rw_muNRQCD_down',rw_muNRQCD_down)
       CALL ReadElem_real('rw_muNRQCD_up',rw_muNRQCD_up)
       ho_nmuNRQCD=2
       IF(rw_muNRQCD_down.LE.0d0.OR.rw_muNRQCD_up.LE.0d0)THEN
          reweight_muNRQCD=.FALSE.
          ho_nmuNRQCD=0
          WRITE(*,*)"WARNING:NRQCD scale reweighting is off."
          WRITE(*,*)"WARNING:Please make sure rw_muNRQCD_up, rw_muNRQCD_down are larger than 0."
       ENDIF
       IF(reweight_muNRQCD.AND.rw_muNRQCD_down.GT.rw_muNRQCD_up)THEN
          temp=rw_muNRQCD_down
          rw_muNRQCD_down=rw_muNRQCD_up
          rw_muNRQCD_up=temp
       ENDIF
       rw_Mscales(1)=1d0
       rw_Mscales(2)=rw_muNRQCD_up
       rw_Mscales(3)=rw_muNRQCD_down
    ENDIF
    IF(reweight_muNRQCD)THEN
       tot_nrwgt=tot_nrwgt+ho_nmuNRQCD+1
    ENDIF
    ! rwgt_values are used for LHE event file
    !IF(reweight_scale.OR.reweight_pdf.OR.reweight_npdf.OR.reweight_ref)THEN
    !   IF(ALLOCATED(rwgt_values))THEN
    !      DEALLOCATE(rwgt_values)
    !   ENDIF
    !   ALLOCATE(rwgt_values(tot_nrwgt))
    !ENDIF
    CALL ReadElem_integer("itmax",itmxn)
    CALL ReadElem_integer("nmc",ncalln)
    WRITE(*,*)' '
    CALL Helac_mtime()
    WRITE(*,*)' '
    IF(integrator.EQ.1)THEN
       CALL gg_Higgs_NNLO_VEGAS(rslt,itmxn,ncalln)
    ELSEIF(integrator.EQ.2)THEN
       CALL gg_Higgs_NNLO_Romberg(rslt,itmxn,ncalln)
    ELSE
       WRITE(*,*)"ERROR: unknown the integrator:",integrator
       STOP
    ENDIF
    WRITE(nunit1,*)"sigma(nb)                   sd"
    WRITE(nunit1,*)rslt(1),rslt(2)
    WRITE(*,*)' '
    CALL Helac_mtime()
    CLOSE(nunit2)
    CLOSE(nunit3)
    CLOSE(21)
    CLOSE(200)
  END SUBROUTINE calc_gg_Higgs_NNLO

  SUBROUTINE gg_Higgs_NNLO_Romberg(rslt,itmxn,ncalln)
    USE NINTLIB
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(3),INTENT(OUT)::rslt
    INTEGER,INTENT(IN),OPTIONAL::itmxn,ncalln
    INTEGER::ITMX
    REAL(KIND(1d0))::vfes,sd,chi2a
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XL,XU
    INTEGER,DIMENSION(:),ALLOCATABLE::sub_num
    INTEGER::ncalm,nc,ii
    CHARACTER(len=4),DIMENSION(20)::chchar
    INTEGER::iday0,ihr0,imin0,isec0,i100th0,iday1,ihr1,imin1,isec1,i100th1,iyr0,iyr1,imon0,imon1
    INTEGER::IDBMUP1,IDBMUP2,IDWTUP
    REAL(KIND(1d0))::TOL
    INTEGER::ind_err,eval_num
    CALL ReadElem_logic('unwgt',lunwei2)
    CALL ReadElem_integer('preunw',nunwei)
    ! maximal 3 fold of finite integrals
    varnum=3
    IF(ALLOCATED(XL))THEN
       DEALLOCATE(XL)
    ENDIF
    ALLOCATE(XL(varnum))
    IF(ALLOCATED(XU))THEN
       DEALLOCATE(XU)
    ENDIF
    ALLOCATE(XU(varnum))
    IF(ALLOCATED(sub_num))THEN
       DEALLOCATE(sub_num)
    ENDIF
    ALLOCATE(sub_num(varnum))
    DO ii=1,varnum
       XL(ii)=0.0d0
       XU(ii)=1.0d0
    ENDDO
    IF(PRESENT(itmxn))THEN
       ITMX=itmxn
    ELSE
       ITMX=5
    ENDIF
    ITMX=1
    IF(PRESENT(ncalln))THEN
       ncalm=ncalln
    ELSE
       ncalm=5000
    ENDIF
    sub_num(1)=MAX(INT(DBLE(ncalm)**(1d0/3d0))+1,12)
    sub_num(2)=sub_num(1)-2
    sub_num(3)=sub_num(2)
    nprint=ncalm/10
    TOL=MAX(1d0/DSQRT(DBLE(ncalm)),1d-5)
    weight_Romberg=PRODUCT((XU(1:varnum)-XL(1:varnum))&
         /DBLE(sub_num(1:varnum)))
    IF(plot_output)CALL initplot_gg_Higgs_NNLO
    CALL Romberg_nd(gg_Higgs_NNLO_fxn4Romberg,XL,XU,varnum,sub_num,ITMX,tol,&
         rslt(1),ind_err,eval_num)
    rslt(2)=DBLE(ind_err)
    rslt(3)=0d0
    WRITE(*,*)rslt(1),"+\-",rslt(2)
    WRITE(*,*)"precision:",rslt(2)/MAX(ABS(rslt(1)),1D-99)
    IF(plot_output)CALL plotout_gg_Higgs_NNLO
    RETURN
  END SUBROUTINE gg_Higgs_NNLO_Romberg

  FUNCTION gg_Higgs_NNLO_fxn4Romberg(dim_num,x)
    USE global_constants
    IMPLICIT NONE
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::x
    REAL(KIND(1d0))::gg_Higgs_NNLO_fxn4Romberg
    REAL(KIND(1d0))::wgt
    REAL(KIND(1d0)),DIMENSION(4)::pmom
    INCLUDE "fxn_args.inc"
    INCLUDE "fxn_init.inc"
    nnntot=nnntot+1
    wgt=weight_Romberg
    wtotal=0d0
    INCLUDE "fxn_phasespace_cuts.inc"
    IF(icuts)THEN
       gg_Higgs_NNLO_fxn4Romberg=0d0
       RETURN
    ENDIF
    Higgs_pmom(1:5,1:4)=Higgs_pmom_real_real(1:5,1:4)
    INCLUDE "fxn_evaluate.inc"
    gg_Higgs_NNLO_fxn4Romberg=wtotal
    nnn=nnn+1
    IF(recmax.LT.gg_Higgs_NNLO_fxn4Romberg)recmax=gg_Higgs_NNLO_fxn4Romberg
    IF(recmin.GT.gg_Higgs_NNLO_fxn4Romberg)recmin=gg_Higgs_NNLO_fxn4Romberg
    IF(MOD(nnn,nprint).EQ.0)THEN
       PRINT *,"max=",recmax
       PRINT *,"min=",recmin
       !IF(lunwei2.AND.lwmax.GT.1)THEN
       !   PRINT *,"      n_event,    n_pass,    n_total"
       !   PRINT *,nwri,nnn,nnntot
       !ELSE
       PRINT *, "      n_pass,     n_total"
       PRINT *,nnn,nnntot
       !ENDIF
    ENDIF
    RETURN
  END FUNCTION gg_Higgs_NNLO_fxn4Romberg

  SUBROUTINE gg_Higgs_NNLO_VEGAS(rslt,itmxn,ncalln)
    USE MC_VEGAS
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(3),INTENT(OUT)::rslt
    INTEGER,INTENT(IN),OPTIONAL::itmxn,ncalln
    REAL(KIND(1d0))::vfes,sd,chi2a
    INTEGER::ncalm,nc,ii
    CHARACTER(len=4),DIMENSION(20)::chchar
    INTEGER::iday0,ihr0,imin0,isec0,i100th0,iday1,ihr1,imin1,isec1,i100th1,iyr0,iyr1,imon0,imon1
    INTEGER::IDBMUP1,IDBMUP2,IDWTUP
    CALL ReadElem_logic('unwgt',lunwei2)
    CALL ReadElem_integer('preunw',nunwei)
    lwmax=0
    NPRN=-1
    ! maximal 3 fold of finite integrals
    varnum=3
    DO ii=1,varnum
       XL(ii)=0.0d0
       XU(ii)=1.0d0
    ENDDO
    IF(PRESENT(itmxn))THEN
       ITMX=itmxn
    ELSE
       ITMX=5
    ENDIF
    ITMX=1
    IF(PRESENT(ncalln))THEN
       ncalm=ncalln
    ELSE
       ncalm=5000
    ENDIF
    chchar(1)="40K"
    chchar(2)="80K"
    chchar(3)="160K"
    chchar(4)="320K"
    chchar(5)="640K"
    chchar(6)="1M"
    chchar(7)="2M"
    chchar(8)="4M"
    chchar(9)="8M"
    chchar(10)="16M"
    chchar(11)='32M'
    chchar(12)='64M'
    chchar(13)='120M'
    chchar(14)='240M'
    chchar(15)='480M'
    chchar(16)='960M'
    chchar(17)='2G'
    chchar(18)='4G'
    chchar(19)='8G'
    chchar(20)='16G'
    nprint=10000
    NCALL=20000
    CALL DAYTIME(iyr0,imon0,iday0,ihr0,imin0,isec0,i100th0)
    IF(plot_output)CALL initplot_gg_Higgs_NNLO
    CALL VEGAS(varnum,gg_Higgs_NNLO_fxn,vfes,sd,chi2a)
    IF(plot_output)CALL plotout_gg_Higgs_NNLO
    WRITE(*,*)' '
    WRITE(*,*)"====================NCALL=20K==========================="
    WRITE(*,*)" "
    CALL Helac_mtime()
    WRITE(*,*)" "
    ii=1
    WRITE(*,*)"ITERATION ",ii,":"
    WRITE(*,*)vfes,"+\-",sd
    WRITE(*,*)"precision:",sd/vfes
    DO ii=2,10
       IF(plot_output)CALL initplot_gg_Higgs_NNLO
       CALL VEGAS(varnum,gg_Higgs_NNLO_fxn,vfes,sd,chi2a,1)
       IF(plot_output)CALL plotout_gg_Higgs_NNLO
       WRITE(*,*)"ITERATION ",ii,":"
       WRITE(*,*)vfes,"+\-",sd
       WRITE(*,*)"precision:",sd/vfes
    ENDDO
    CALL DAYTIME(iyr1,imon1,iday1,ihr1,imin1,isec1,i100th1)
    iday1=iday1-iday0
    ihr1=ihr1-ihr0
    imin1=imin1-imin0
    isec1=isec1-isec0
    i100th1=i100th1-i100th0
    CALL Vegas_speed(10*NCALL,iday1,ihr1,imin1,isec1,i100th1)
    WRITE(*,*)' '
    WRITE(*,*)' '
    ii=1
    DO
       nc=2*NCALL
       IF(nc.GT.ncalm)EXIT
       IF(2*nc.GT.ncalm.AND.lunwei2.AND.lwmax.EQ.0)THEN
          lwmax=1
       ENDIF
       NCALL=nc
       IF(NCALL/maxprint.GT.nprint)nprint=NCALL/maxprint
         CALL DAYTIME(iyr0,imon0,iday0,ihr0,imin0,isec0,i100th0)
         WRITE(*,*)"====================NCALL="//chchar(ii)//"==========================="
         WRITE(*,*)" "
         CALL Helac_mtime()
         WRITE(*,*)" "
         IF(plot_output)CALL initplot_gg_Higgs_NNLO
         CALL VEGAS(varnum,gg_Higgs_NNLO_fxn,vfes,sd,chi2a,1)
         IF(plot_output)CALL plotout_gg_Higgs_NNLO
         WRITE(*,*)vfes,"+\-",sd
         WRITE(*,*)"precision:",sd/vfes
         CALL DAYTIME(iyr1,imon1,iday1,ihr1,imin1,isec1,i100th1)
         iday1=iday1-iday0
         ihr1=ihr1-ihr0
         imin1=imin1-imin0
         isec1=isec1-isec0
         i100th1=i100th1-i100th0
         CALL Vegas_speed(NCALL,iday1,ihr1,imin1,isec1,i100th1)
         WRITE(*,*)" "
         ii=ii+1
      ENDDO
      SELECT CASE(COLL_TYPE)
      CASE(1)
         IDBMUP1=2212
         IDBMUP2=2212
      CASE(2)
         IDBMUP1=2212
         IDBMUP2=-2212
      CASE DEFAULT
         IDBMUP1=11
         IDBMUP2=-11
      END SELECT
      IDWTUP=1
      NPRUP=NPRUP+1
      WRITE(200,5100) IDBMUP1,IDBMUP2,EBMUP1,EBMUP2,&
           iPDFGUP1,iPDFGUP2,iPDFSUP1,iPDFSUP2,IDWTUP,NPRUP
      WRITE(200,5200) vfes*10d0**3,sd*10d0**3,1d0, 82
      rslt(1)=vfes
      rslt(2)=sd
      rslt(3)=chi2a
      RETURN
5100  FORMAT(1P,2I8,2E14.6,6I8)
5200  FORMAT(1P,3E20.10,I6)
    END SUBROUTINE gg_Higgs_NNLO_VEGAS

  FUNCTION gg_Higgs_NNLO_fxn(x,wgt)
    USE global_constants
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(varnum),INTENT(IN)::x
    REAL(KIND(1d0)),INTENT(IN)::wgt
    REAL(KIND(1d0))::gg_Higgs_NNLO_fxn
    REAL(KIND(1d0))::phi,costh,sinth
    REAL(KIND(1d0)),DIMENSION(4)::pmom
    INCLUDE "fxn_args.inc"
    INCLUDE "fxn_init.inc"
    nnntot=nnntot+1
    wtotal=0d0
    INCLUDE "fxn_phasespace_cuts.inc"
    IF(icuts)THEN
       gg_Higgs_NNLO_fxn=0d0
       RETURN
    ENDIF
    Higgs_pmom(1:5,1:4)=Higgs_pmom_real_real(1:5,1:4)
    INCLUDE "fxn_evaluate.inc"
    gg_Higgs_NNLO_fxn=wtotal
    nnn=nnn+1
    IF(recmax.LT.gg_Higgs_NNLO_fxn)recmax=gg_Higgs_NNLO_fxn
    IF(recmin.GT.gg_Higgs_NNLO_fxn)recmin=gg_Higgs_NNLO_fxn
    IF(MOD(nnn,nprint).EQ.0)THEN
       PRINT *,"max=",recmax
       PRINT *,"min=",recmin
       !IF(lunwei2.AND.lwmax.GT.1)THEN
       !   PRINT *,"      n_event,    n_pass,    n_total"
       !   PRINT *,nwri,nnn,nnntot
       !ELSE
       PRINT *, "      n_pass,     n_total"
       PRINT *,nnn,nnntot
       !ENDIF
    ENDIF
    RETURN
  END FUNCTION gg_Higgs_NNLO_fxn

  SUBROUTINE gg_Higgs_NNLO_reweight_xsec_scale(inum,itype,ireal,wsf0,wme0,me,me_order)
    USE global_constants
    IMPLICIT NONE
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    ! ireal:
    ! 1: 6<=ss_piece<=9 (double real)
    ! 2: ss_piece = 3,5 (real born)
    ! 3: ss_piece = 2,4 (born real)
    ! 4: ss_piece = 1   (born born)
    INTEGER,INTENT(IN)::ireal
    REAL(KIND(1d0)),INTENT(IN)::wme0,wsf0
    ! the following two are for muR/muR0=1 and muF/muF0=1
    ! that have been evaluated
    REAL(KIND(1d0)),DIMENSION(4,9),INTENT(IN)::me
    REAL(KIND(1d0)),DIMENSION(0:2,4,9),INTENT(IN)::me_order
    REAL(KIND(1d0))::alphasc=1d0,alphasu=1d0,alphasd=1d0
    REAL(KIND(1d0)),DIMENSION(3)::R_pdf
    REAL(KIND(1d0)),DIMENSION(3)::muR_scales,muF_scales,Lcut_rwgt,LS_rwgt
    COMPLEX(KIND(1d0)),DIMENSION(3)::LMH_rwgt,LMT_rwgt,LMetaQ_rwgt
    COMPLEX(KIND(1d0))::LmuNRQCD
    REAL(KIND(1d0)),DIMENSION(3)::aso2pimuR,aso2pimuF
    SAVE aso2pimuR,aso2pimuF
    REAL(KIND(1d0))::wsfw,wmew,LRF
    INTEGER::i,j
    INTEGER::init=0
    SAVE init,muR_scales,muF_scales
    SAVE Lcut_rwgt,LS_rwgt,LMH_rwgt,LMT_rwgt,LMetaQ_rwgt
    REAL(KIND(1d0)),DIMENSION(2:3)::me_born_rwgt
    REAL(KIND(1d0)),DIMENSION(2:3,0:2,4,9)::me_bsb_rwgt
    REAL(KIND(1d0)),DIMENSION(2:3,0:2)::me_hard_rwgt
    SAVE me_hard_rwgt,me_born_rwgt
    REAL(KIND(1d0)),DIMENSION(3,0:2,4,9)::me_order_rwgt
    REAL(KIND(1d0)),DIMENSION(3,4,9)::me_rwgt
    SAVE me_rwgt,me_order_rwgt
    REAL(KIND(1d0))::za,zb
    IF(.NOT.reweight_scale)THEN
       WRITE(*,*)"ERROR:Please set reweight_scale to be T first !"
       STOP
    ENDIF
    IF(force_initial_reweight_scale)THEN
       IF(nscheme.NE.0.OR.(init.EQ.0.AND.nscheme.EQ.0))THEN
          ! force to update phase space point by phase space point when using dynamical scale
          ! if use fixed scale, only the first time evaluation is necessary
          CALL Higgs_scale(scale)
          scale=scale*muR_over_ref
          muR_scales(1)=scale
          muR_scales(2)=scale*rw_Rscale_up
          muR_scales(3)=scale*rw_Rscale_down
          scale=scale/muR_over_ref*muF_over_ref
          muF_scales(1)=scale
          muF_scales(2)=scale*rw_Fscale_up
          muF_scales(3)=scale*rw_Fscale_down
          alphasc=ALPHAS(muR_scales(1))
          aso2pimuR(1)=alphasc/TWOPI
          alphasu=ALPHAS(muR_scales(2))
          aso2pimuR(2)=alphasu/TWOPI
          alphasd=ALPHAS(muR_scales(3))
          aso2pimuR(3)=alphasd/TWOPI
          !R_alphas(1)=1d0
          !R_alphas(2)=alphasu/alphasc
          !R_alphas(3)=alphasd/alphasc
          IF(muR_over_ref.EQ.muF_over_ref)THEN
             aso2pimuF(1:3)=aso2pimuR(1:3)
          ELSE
             ! this is not allowed in fact
             alphasc=ALPHAS(muF_scales(1))
             aso2pimuF(1)=alphasc/TWOPI
             alphasu=ALPHAS(muF_scales(2))
             aso2pimuF(2)=alphasu/TWOPI
             alphasd=ALPHAS(muF_scales(3))
             aso2pimuF(3)=alphasd/TWOPI
          ENDIF
          Lcut_rwgt(1)=DLOG(H_pTveto**2/muF_scales(1)**2)
          Lcut_rwgt(2)=DLOG(H_pTveto**2/muF_scales(2)**2)
          Lcut_rwgt(3)=DLOG(H_pTveto**2/muF_scales(3)**2)
          IF(Higgs_state.EQ.1)THEN
             ! Higgs
             DO i=1,3
                LS_rwgt(i)=two*DLOG(MH**2/muF_scales(i)**2)
                LMH_rwgt(i)=CDLOG(DCMPLX(-MH**2/muF_scales(i)**2))
                LMT_rwgt(i)=CDLOG(DCMPLX(muF_scales(i)**2/MT**2))
                IF(i.GT.1)THEN
                   ! get Born partonic cross section
                   CALL call_gg_Higgs_Born(vev,aso2pimuF(i),me_born_rwgt(i))
                   ! get Hard function
                   CALL call_gg_Higgs_NNLO_HardFunction(MEorder,aso2pimuF(i),LMH_rwgt(i),&
                        LMT_rwgt(i),me_hard_rwgt(i,0:2))
                ENDIF
             ENDDO
          ELSE
             ! quarkonium
             LmuNRQCD=CDLOG(DCMPLX(MetaQ**2/onium_muNRQCD**2))
             DO i=1,3
                LS_rwgt(i)=two*DLOG(MetaQ**2/muF_scales(i)**2)
                LMetaQ_rwgt(i)=CDLOG(DCMPLX(-MetaQ**2/muF_scales(i)**2))
                IF(i.GT.1)THEN
                   ! get Born partonic cross section
                   CALL call_gg_etaQ_Born(LDMEetaQ,aso2pimuF(i),me_born_rwgt(i))
                   ! get Hard function
                   CALL call_gg_etaQ_NNLO_HardFunction(MEorder,aso2pimuF(i),LMetaQ_rwgt(i),&
                        LmuNRQCD,me_hard_rwgt(i,0:2))
                ENDIF
             ENDDO
          ENDIF
       ENDIF
       ! the following must be updated phase space point by phase space point
       za=xp1_born/xp1_real
       zb=xp2_born/xp2_real
       DO i=2,3
          ! get all beam*soft*beam functions according to the cuts
          CALL call_gg_Higgs_NNLO_BSB(za,zb,xp1_born,xp2_born,Lcut_rwgt(i),LS_rwgt(i),aso2pimuF(i),&
               MEorder,phase_space_cut,me_bsb_rwgt(i,0:2,1:4,1:9))
          ! combine me's
          ! me_order is used for reweighting renormalisation/factorisation scales
          CALL combine_me_by_order(MEorder,me_born_rwgt(i),me_hard_rwgt(i,0:2),&
               me_bsb_rwgt(i,0:2,1:4,1:9),me_rwgt(i,1:4,1:9),me_order_rwgt(i,0:2,1:4,1:9))
       ENDDO
       ! i=1
       ! we have evaluated them
       me_rwgt(1,1:4,1:9)=me(1:4,1:9)
       me_order_rwgt(1,0:2,1:4,1:9)=me_order(0:2,1:4,1:9)

       force_initial_reweight_scale=.FALSE.
    ENDIF
    
    ! assign xp1 and xp2 values
    SELECT CASE(itype)
    CASE(1)
       xp1=xp1_real
       xp2=xp2_real
    CASE(2)
       xp1=xp1_real
       xp2=xp2_born
    CASE(3)
       xp1=xp1_born
       xp2=xp2_real
    CASE(4)
       xp1=xp1_born
       xp2=xp2_born
    CASE DEFAULT
       WRITE(*,*)"ERROR: gg_Higgs_NNLO_reweight_xsec_scale does not know the itype = ",itype
       STOP
    END SELECT

    R_pdf(1)=1d0
    IF(istruc.EQ.1)THEN
       reweight_Fscale_phase=1 ! 1: multiply central scale * rw_Fscale_up
       CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
       R_pdf(2)=wsfw/wsf0
       reweight_Fscale_phase=-1 ! -1: mutliply central scale * rw_Fscale_down
       CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
       R_pdf(3)=wsfw/wsf0
       ! recover it
       reweight_Fscale_phase=0
    ELSE
       R_pdf(2:3)=1d0
    ENDIF

    wgtxsecmu(1:3,1:3)=1d0
    DO i=1,3
       ! cycle for muR
       DO j=1,3
          ! cycle for muF
          ! use Eqs.(H.2,H.3) in my DGLAPSolver.pdf to obtain the muR dependence
          ! according to muR=muF
          LRF=DLOG(muR_scales(i)**2/muF_scales(j)**2)
          wmew=get_wme_muR_neq_muF_gg_Higgs_NNLO(inum,itype,ireal,aso2pimuR(i),aso2pimuF(j),&
               LRF,me_order_rwgt(j,0:2,1:4,1:9))
          wgtxsecmu(i,j)=R_pdf(j)*wmew/wme0
       ENDDO
    END DO
    IF(init.EQ.0)init=1
    RETURN
  END SUBROUTINE gg_Higgs_NNLO_reweight_xsec_scale

  FUNCTION get_wme_muR_neq_muF_gg_Higgs_NNLO(inum,itype,ireal,aso2pimuR,aso2pi0,LRQ,me_order) RESULT(res)
    ! use Eqs.(H.2,H.3) in my DGLAPSolver.pdf
    USE qcd_constants
    IMPLICIT NONE
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    ! ireal:
    ! 1: 6<=ss_piece<=9 (double real)
    ! 2: ss_piece = 3,5 (real born)
    ! 3: ss_piece = 2,4 (born real)
    ! 4: ss_piece = 1   (born born)
    INTEGER,INTENT(IN)::ireal
    REAL(KIND(1d0)),DIMENSION(0:2,4,9),INTENT(IN)::me_order
    REAL(KIND(1d0))::res
    ! as(muR)/2/pi
    REAL(KIND(1d0)),INTENT(IN)::aso2pimuR
    ! LRQ=LOG(muR**2/muF**2), muF=Q
    REAL(KIND(1d0)),INTENT(IN)::LRQ
    ! as(muF)/2/pi, muF=Q
    REAL(KIND(1d0)),INTENT(IN)::aso2pi0
    REAL(KIND(1d0)),DIMENSION(0:2)::meval,mevalmuR
    INTEGER::i
    
    res=0d0
    IF(MEorder.LT.0)RETURN
    DO i=0,MEorder
       meval(i)=get_wme_gg_Higgs_NNLO(inum,itype,ireal,me_order(i,1:4,1:9))
       meval(i)=meval(i)/aso2pi0**(bpower+i) ! this is the \hat{\sigma}^{(k)}(muF)
    ENDDO
    ! \hat{\sigma}^{(k)}(muQ) uses Eq.(H.3)
    mevalmuR(0)=meval(0)
    res=res+aso2pimuR**(bpower)*mevalmuR(0)
    IF(MEorder.LT.1)RETURN
    mevalmuR(1)=meval(1)+bpower*beta0*meval(0)*LRQ
    res=res+aso2pimuR**(bpower+1)*mevalmuR(1)
    IF(MEorder.LT.2)RETURN
    mevalmuR(2)=meval(2)+((bpower+1d0)*beta0*meval(1)+bpower*beta1*meval(0))*LRQ&
         +bpower*(bpower+1d0)/2d0*beta0**2*meval(0)*LRQ**2
    res=res+aso2pimuR**(bpower+2)*mevalmuR(2)
    RETURN
  END FUNCTION get_wme_muR_neq_muF_gg_Higgs_NNLO

  FUNCTION get_wme_gg_Higgs_NNLO(inum,itype,ireal,me) RESULT(res)
    IMPLICIT NONE
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    ! ireal:
    ! 1: 6<=ss_piece<=9 (double real)
    ! 2: ss_piece = 3,5 (real born)
    ! 3: ss_piece = 2,4 (born real)
    ! 4: ss_piece = 1   (born born)
    INTEGER,INTENT(IN)::ireal
    REAL(KIND(1d0)),DIMENSION(4,9),INTENT(IN)::me
    REAL(KIND(1d0))::res
    IF(itype.GT.4.OR.itype.LT.1)THEN
       WRITE(*,*)"ERROR: Do not know the value of itype=",itype
       STOP
    ENDIF
    SELECT CASE(ireal)
    CASE(1)
       SELECT CASE(itype)
       CASE(1)
          res=me(inum,6)+me(inum,7)+me(inum,8)+me(inum,9)
       CASE(2)
          res=-(me(inum,6)+me(inum,8))
       CASE(3)
          res=-(me(inum,6)+me(inum,7))
       CASE(4)
          res=me(inum,6)
       END SELECT
    CASE(2)
       SELECT CASE(itype)
       CASE(1)
          res=0d0
       CASE(2)
          res=(me(inum,3)+me(inum,5))
       CASE(3)
          res=0d0
       CASE(4)
          res=-me(inum,3)
       END SELECT
    CASE(3)
       SELECT CASE(itype)
       CASE(1)
          res=0d0
       CASE(2)
          res=0d0
       CASE(3)
          res=(me(inum,2)+me(inum,4))
       CASE(4)
          res=-me(inum,2)
       END SELECT
    CASE(4)
       SELECT CASE(itype)
       CASE(1)
          res=0d0
       CASE(2)
          res=0d0
       CASE(3)
          res=0d0
       CASE(4)
          res=me(inum,1)
       END SELECT
    CASE DEFAULT
       WRITE(*,*)"ERROR: Do not know the value of ireal=",ireal
       STOP
    END SELECT
    RETURN
  END FUNCTION get_wme_gg_Higgs_NNLO

  SUBROUTINE gg_Higgs_NNLO_reweight_xsec_muNRQCD(inum,itype,ireal,wme0,me_born0,me_bsb0)
    USE global_constants
    IMPLICIT NONE
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    ! ireal:
    ! 1: 6<=ss_piece<=9 (double real)
    ! 2: ss_piece = 3,5 (real born)
    ! 3: ss_piece = 2,4 (born real)
    ! 4: ss_piece = 1   (born born)
    INTEGER,INTENT(IN)::ireal
    REAL(KIND(1d0)),INTENT(IN)::wme0
    ! the following two are for onium_muNRQCD (central value)
    ! that have been evaluated
    REAL(KIND(1d0)),INTENT(IN)::me_born0
    REAL(KIND(1d0)),DIMENSION(0:2,4,9),INTENT(IN)::me_bsb0
    REAL(KIND(1d0))::alphasc=1d0
    REAL(KIND(1d0)),DIMENSION(3)::muNRQCD_scales,LDMEetaQ_rwgt
    REAL(KIND(1d0))::Lcut_rwgt,LS_rwgt,muR_scale,muF_scale
    COMPLEX(KIND(1d0)),DIMENSION(3)::LmuNRQCD_rwgt
    COMPLEX(KIND(1d0))::LMetaQ_rwgt
    REAL(KIND(1d0))::aso2pimuR,aso2pimuF
    SAVE aso2pimuR,aso2pimuF
    REAL(KIND(1d0))::wmew
    INTEGER::i,j
    INTEGER::init=0
    SAVE init,muNRQCD_scales,muR_scale,muF_scale,LDMEetaQ_rwgt
    SAVE Lcut_rwgt,LS_rwgt,LMetaQ_rwgt,LmuNRQCD_rwgt
    REAL(KIND(1d0)),DIMENSION(2:3)::me_born_rwgt
    REAL(KIND(1d0)),DIMENSION(2:3,0:2)::me_hard_rwgt
    SAVE me_hard_rwgt,me_born_rwgt
    REAL(KIND(1d0)),DIMENSION(0:2,4,9)::me_order_rwgt
    REAL(KIND(1d0)),DIMENSION(3,4,9)::me_rwgt
    SAVE me_rwgt ! ,me_order_rwgt
    !REAL(KIND(1d0))::za,zb
    IF(.NOT.reweight_muNRQCD)THEN
       WRITE(*,*)"ERROR:Please set reweight_muNRQCD to be T first !"
       STOP
    ENDIF
    IF(force_initial_reweight_muNRQCD)THEN
       IF(init.EQ.0)THEN
          ! Only the first time evaluation is necessary
          muNRQCD_scales(1)=onium_muNRQCD
          muNRQCD_scales(2)=onium_muNRQCD*rw_muNRQCD_up
          muNRQCD_scales(3)=onium_muNRQCD*rw_muNRQCD_down
          LmuNRQCD_rwgt(1)=CDLOG(DCMPLX(MetaQ**2/muNRQCD_scales(1)**2))
          LmuNRQCD_rwgt(2)=CDLOG(DCMPLX(MetaQ**2/muNRQCD_scales(2)**2))
          LmuNRQCD_rwgt(3)=CDLOG(DCMPLX(MetaQ**2/muNRQCD_scales(3)**2))
          LDMEetaQ_rwgt(1)=1d0
          LDMEetaQ_rwgt(2)=LDMEetaQ_up/LDMEetaQ
          LDMEetaQ_rwgt(3)=LDMEetaQ_down/LDMEetaQ
       ENDIF
       IF(nscheme.NE.0.OR.(init.EQ.0.AND.nscheme.EQ.0))THEN
          ! force to update phase space point by phase space point when using dynamical scale
          ! if use fixed scale, only the first time evaluation is necessary
          CALL Higgs_scale(scale)
          scale=scale*muR_over_ref
          muR_scale=scale
          scale=scale/muR_over_ref*muF_over_ref
          muF_scale=scale
          alphasc=ALPHAS(muR_scale)
          aso2pimuR=alphasc/TWOPI
          IF(muR_over_ref.EQ.muF_over_ref)THEN
             aso2pimuF=aso2pimuR
          ELSE
             ! this is not allowed in fact
             alphasc=ALPHAS(muF_scale)
             aso2pimuF=alphasc/TWOPI
          ENDIF
          Lcut_rwgt=DLOG(H_pTveto**2/muF_scale**2)
          IF(Higgs_state.EQ.1)THEN
             ! Higgs
             WRITE(*,*)"ERROR: cannot reweight muNRQCD for the Higgs process"
             STOP
          ELSE
             ! quarkonium
             LS_rwgt=two*DLOG(MetaQ**2/muF_scale**2)
             LMetaQ_rwgt=CDLOG(DCMPLX(-MetaQ**2/muF_scale**2))
             DO i=2,3
                ! get Born partonic cross section
                !CALL call_gg_etaQ_Born(LDMEetaQ,aso2pimuF(i),me_born_rwgt(i))
                me_born_rwgt(i)=me_born0*LDMEetaQ_rwgt(i) ! me_born0 needs an input
                ! get Hard function
                CALL call_gg_etaQ_NNLO_HardFunction(MEorder,aso2pimuF,LMetaQ_rwgt,&
                     LmuNRQCD_rwgt(i),me_hard_rwgt(i,0:2))
             ENDDO
          ENDIF
       ENDIF
       ! the following must be updated phase space point by phase space point
       DO i=2,3
          ! combine me's
          ! me_order we do not need
          CALL combine_me_by_order(MEorder,me_born_rwgt(i),me_hard_rwgt(i,0:2),&
               me_bsb0(0:2,1:4,1:9),me_rwgt(i,1:4,1:9),me_order_rwgt(0:2,1:4,1:9))
       ENDDO

       force_initial_reweight_muNRQCD=.FALSE.
    ENDIF
    
    wgtxsecmuNRQCD(1:3)=1d0
    DO i=2,3
       ! cycle for muNRQCD
       wmew=get_wme_gg_Higgs_NNLO(inum,itype,ireal,me_rwgt(i,1:4,1:9))
       wgtxsecmuNRQCD(i)=wmew/wme0
    END DO
    IF(init.EQ.0)init=1
    RETURN
  END SUBROUTINE gg_Higgs_NNLO_reweight_xsec_muNRQCD

  SUBROUTINE gg_Higgs_NNLO_reweight_xsec_pdf(wsf0,inum,itype)
    IMPLICIT NONE
    INTEGER::init=0,i,k,kk
    REAL(KIND(1d0)),INTENT(IN)::wsf0
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    REAL(KIND(1d0))::wsfw
    SAVE init
    LOGICAL::lexist
    REAL(KIND(1d0)),DIMENSION(:,:,:),ALLOCATABLE::wsf_save
    LOGICAL,DIMENSION(4,4)::wsf_saved
    SAVE wsf_save,wsf_saved
    
    IF(.NOT.reweight_pdf)THEN
       WRITE(*,*)"ERROR:Please set reweight_pdf to be T first !"
       STOP
    ENDIF
    IF(init.EQ.0)THEN
       IF(MOD(ho_npdf,2).NE.0)THEN
          WRITE(*,*)'The number of error sets must be even',ho_npdf
          STOP
       ENDIF
       IF(MOD(ho_npdf2,2).NE.0)THEN
          WRITE(*,*)'The number of error sets for beam 2 must be even',ho_npdf2
          STOP
       ENDIF
       IF(ALLOCATED(wgtxsecpdf))THEN
          DEALLOCATE(wgtxsecpdf)
       ENDIF
       ALLOCATE(wgtxsecpdf(ho_npdf+ho_npdf2))
       IF(ALLOCATED(wsf_save))THEN
          DEALLOCATE(wsf_save)
       ENDIF
       ALLOCATE(wsf_save(ho_npdf+ho_npdf2,4,4))
       IF(ho_npdf.GT.0)THEN
          IF(ALLOCATED(idpdf))THEN
             DEALLOCATE(idpdf)
          ENDIF
          ALLOCATE(idpdf(0:ho_npdf))
          idpdf(0)=iPDFSUP1
          idpdf(1)=ipdf_set_min
          IF(ipdf_set_max-ipdf_set_min+1.NE.ho_npdf)THEN
             WRITE(*,*)'Incorrect the number of error pdf',ho_npdf,ipdf_set_min,ipdf_set_max
             STOP
          ENDIF
          DO i=2,ho_npdf
             idpdf(i)=idpdf(1)+i-1
          ENDDO
          WRITE(*,*) 'Doing PDF reweight:'
          WRITE(*,*) 'Central set id: ',idpdf(0)
          WRITE(*,*) 'Min error set id: ',idpdf(1)
          WRITE(*,*) 'Max error set id: ',idpdf(ho_npdf)
       ENDIF
       IF(ho_npdf2.GT.0)THEN
          IF(ALLOCATED(idpdf2))THEN
             DEALLOCATE(idpdf2)
          ENDIF
          ALLOCATE(idpdf2(0:ho_npdf2))
          idpdf2(0)=iPDFSUP2
          idpdf2(1)=ipdf2_set_min
          IF(ipdf2_set_max-ipdf2_set_min+1.NE.ho_npdf2)THEN
             WRITE(*,*)'Incorrect the number of error pdf2',ho_npdf2,ipdf2_set_min,ipdf2_set_max
             STOP
          ENDIF
          DO i=2,ho_npdf2
             idpdf2(i)=idpdf2(1)+i-1
          ENDDO
          WRITE(*,*) 'Doing PDF Beam 2 reweight:'
          WRITE(*,*) 'Central set id: ',idpdf2(0)
          WRITE(*,*) 'Min error set id: ',idpdf2(1)
          WRITE(*,*) 'Max error set id: ',idpdf2(ho_npdf2)
       ENDIF
       init=1
    ENDIF
    IF(force_initial_reweight_pdf)THEN
       ! force to update the wsf saved values
       wsf_saved(1:4,1:4)=.FALSE.
       force_initial_reweight_pdf=.FALSE.
    ENDIF
    ! assign xp1 and xp2 values first
    SELECT CASE(itype)
    CASE(1)
       xp1=xp1_real
       xp2=xp2_real
    CASE(2)
       xp1=xp1_real
       xp2=xp2_born
    CASE(3)
       xp1=xp1_born
       xp2=xp2_real
    CASE(4)
       xp1=xp1_born
       xp2=xp2_born
    CASE DEFAULT
       WRITE(*,*)"ERROR: gg_Higgs_NNLO_reweight_xsec_pdf does not know the itype = ",itype
       STOP
    END SELECT
    IF(.NOT.multipdfloading)THEN
       IF(.NOT.wsf_saved(inum,itype))THEN
          DO i=1,ho_npdf
             INCLUDE "../../../input/lhapdf/call_initpdf"
             IF(istruc.EQ.1)THEN
                CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
             ELSE
                WRITE(*,*)"ERROR: do not know the meaning of istruc with reweighting PDF"
                STOP
             ENDIF
             wsf_save(i,inum,itype)=wsfw
             wgtxsecpdf(i)=wsfw/wsf0
          ENDDO
          wsf_saved(inum,itype)=.TRUE.
       ELSE
          ! use the saved ones
          DO i=1,ho_npdf
             wsfw=wsf_save(i,inum,itype)
             wgtxsecpdf(i)=wsfw/wsf0
          ENDDO
       ENDIF
    ELSE
       IF(.NOT.beams_pdf_correlation)THEN
          IF(.NOT.wsf_saved(inum,itype))THEN
             DO i=1,ho_npdf
                INCLUDE "../../../input/lhapdf/call_initmpdf1"
                CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
                wgtxsecpdf(i)=wsfw/wsf0
                wsf_save(i,inum,itype)=wsfw
             ENDDO
             INCLUDE "../../../input/lhapdf/call_initmpdf10"
             DO i=1,ho_npdf2
                INCLUDE "../../../input/lhapdf/call_initmpdf2"
                CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
                wgtxsecpdf(i+ho_npdf)=wsfw/wsf0
                wsf_save(i+ho_npdf,inum,itype)=wsfw
             ENDDO
             INCLUDE "../../../input/lhapdf/call_initmpdf20"
             wsf_saved(inum,itype)=.TRUE.
          ELSE
             ! use the saved ones
             DO i=1,ho_npdf
                wsfw=wsf_save(i,inum,itype)
                wgtxsecpdf(i)=wsfw/wsf0
             ENDDO
             DO i=1,ho_npdf2
                wsfw=wsf_save(i+ho_npdf,inum,itype)
                wgtxsecpdf(i)=wsfw/wsf0
             ENDDO
          ENDIF
       ELSE
          ! it is correlated for two beams
          IF(.NOT.wsf_saved(inum,itype))THEN
             DO i=1,ho_npdf
                INCLUDE "../../../input/lhapdf/call_initmpdf1"
                INCLUDE "../../../input/lhapdf/call_initmpdf2"
                CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
                wgtxsecpdf(i)=wsfw/wsf0
                wsf_save(i,inum,itype)=wsfw
             ENDDO
             INCLUDE "../../../input/lhapdf/call_initmpdf10"
             INCLUDE "../../../input/lhapdf/call_initmpdf20"
             wsf_saved(inum,itype)=.TRUE.
          ELSE
             ! use the saved ones
             DO i=1,ho_npdf
                wsfw=wsf_save(i,inum,itype)
                wgtxsecpdf(i)=wsfw/wsf0
             ENDDO
          ENDIF
       ENDIF
       RETURN
    ENDIF
    ! recover it
    INCLUDE "../../../input/lhapdf/call_initpdf0"
    RETURN
  END SUBROUTINE gg_Higgs_NNLO_reweight_xsec_pdf

  SUBROUTINE gg_Higgs_NNLO_reweight_xsec_npdf(wsf0,inum,itype)
    IMPLICIT NONE
    INTEGER::init=0,i,k,kk
    INTEGER,DIMENSION(2)::icase
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    REAL(KIND(1d0)),INTENT(IN)::wsf0
    REAL(KIND(1d0))::wsfw
    SAVE init
    INTEGER::icheck=0
    SAVE icheck
    LOGICAL::lexist
    REAL(KIND(1d0)),DIMENSION(:,:,:),ALLOCATABLE::wsfn_save
    LOGICAL,DIMENSION(4,4)::wsfn_saved
    SAVE wsfn_save,wsfn_saved

    IF(.NOT.reweight_npdf)THEN
       WRITE(*,*)"ERROR:Please set reweight_npdf to be T first !"
       STOP
    ENDIF
    IF(init.EQ.0)THEN
       IF(MOD(ho_nnpdf,2).NE.0)THEN
          WRITE(*,*)'The number of error sets must be even',ho_nnpdf
          STOP
       ENDIF
       IF(ALLOCATED(wgtxsecnpdf))THEN
          DEALLOCATE(wgtxsecnpdf)
       ENDIF
       ALLOCATE(wgtxsecnpdf(ho_nnpdf))
       IF(ALLOCATED(idnpdf))THEN
          DEALLOCATE(idnpdf)
       ENDIF
       ALLOCATE(idnpdf(0:ho_nnpdf))
       IF(ALLOCATED(wsfn_save))THEN
          DEALLOCATE(wsfn_save)
       ENDIF
       ALLOCATE(wsfn_save(ho_nnpdf,4,4))
       IF(nPDF_id.NE.1000)THEN
          idnpdf(0)=nPDF_id
          idnpdf(1)=inpdf_set_min
       ELSE
          idnpdf(0)=iPDFSUP1
          idnpdf(1)=inpdf_set_min-1000+iPDFSUP1
       ENDIF
       IF(inpdf_set_max-inpdf_set_min+1.NE.ho_nnpdf)THEN
          WRITE(*,*)'Incorrect the number of error npdf',ho_nnpdf,inpdf_set_min,inpdf_set_max
          STOP
       ENDIF
       DO i=2,ho_nnpdf
          idnpdf(i)=idnpdf(1)+i-1
       ENDDO
       WRITE(*,*) 'Doing nPDF reweight:'
       WRITE(*,*) 'Central set id: ',idnpdf(0)
       WRITE(*,*) 'Min error set id: ',idnpdf(1)
       WRITE(*,*) 'Max error set id: ',idnpdf(ho_nnpdf)
       init=1
    ENDIF
    IF(force_initial_reweight_npdf)THEN
       ! force to update the wsf saved values
       wsfn_saved(1:4,1:4)=.FALSE.
       force_initial_reweight_npdf=.FALSE.
    ENDIF
    ! assign xp1 and xp2 values first
    SELECT CASE(itype)
    CASE(1)
       xp1=xp1_real
       xp2=xp2_real
    CASE(2)
       xp1=xp1_real
       xp2=xp2_born
    CASE(3)
       xp1=xp1_born
       xp2=xp2_real
    CASE(4)
       xp1=xp1_born
       xp2=xp2_born
    CASE DEFAULT
       WRITE(*,*)"ERROR: gg_Higgs_NNLO_reweight_xsec_npdf does not know the itype = ",itype
       STOP
    END SELECT
    IF(.NOT.wsfn_saved(inum,itype))THEN
       DO i=1,ho_nnpdf
          use_neutron_rand=.TRUE.
          INCLUDE "../../../input/lhapdf/call_initnpdfi"
          INCLUDE "../../../input/lhapdf/call_initnpdfi_lhapdf6"
          IF(istruc.EQ.1)THEN
             CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
          ELSE
             WRITE(*,*)"ERROR: cannot understand the value of istruc within gg_Higgs_NNLO_reweigt_xsec_npdf"
             STOP
          ENDIF
          wsfn_save(i,inum,itype)=wsfw
          wgtxsecnpdf(i)=wsfw/wsf0
       ENDDO
       ! recover it
       INCLUDE "../../../input/lhapdf/call_initnpdf"
       INCLUDE "../../../input/lhapdf/call_strf_lhanpdf_0"
       wsfn_saved(inum,itype)=.TRUE.
    ELSE
       ! use the saved ones
       DO i=1,ho_nnpdf
          wsfw=wsfn_save(i,inum,itype)
          wgtxsecnpdf(i)=wsfw/wsf0
       ENDDO
    ENDIF
    RETURN
  END SUBROUTINE gg_Higgs_NNLO_reweight_xsec_npdf

  SUBROUTINE gg_Higgs_NNLO_reweight_xsec_ref(wsf0,inum,itype)
    IMPLICIT NONE
    INTEGER::init=0,i
    ! inum: 1,gg; 2, gq+gqbar; 3, qg+qbarg; 4, qq'
    INTEGER,INTENT(IN)::inum
    ! itype:
    ! 1: xp1=xp1_real,xp2=xp2_real, wsf11
    ! 2: xp1=xp1_real,xp2=xp2_born, wsf10
    ! 3: xp1=xp1_born,xp2=xp2_real, wsf01
    ! 4: xp1=xp1_born,xp2=xp2_born, wsf00
    INTEGER,INTENT(IN)::itype
    REAL(KIND(1d0)),INTENT(IN)::wsf0
    REAL(KIND(1d0))::wsfw
    SAVE init
    REAL(KIND(1d0)),DIMENSION(:,:,:),ALLOCATABLE::wsfr_save
    LOGICAL,DIMENSION(4,4)::wsfr_saved
    SAVE wsfr_save,wsfr_saved

    IF(.NOT.reweight_ref)THEN
       WRITE(*,*)"ERROR:Please set reweight_ref to be T first !"
       STOP
    ENDIF
    IF(init.EQ.0)THEN
       IF(ho_nref.NE.1.AND.ho_nref.NE.3)THEN
          WRITE(*,*)'Incorrect the number of references #1:',ho_nref
          STOP
       ENDIF
       IF(ALLOCATED(wgtxsecref))THEN
          DEALLOCATE(wgtxsecref)
       ENDIF
       ALLOCATE(wgtxsecref(ho_nref))
       IF(ALLOCATED(idref))THEN
          DEALLOCATE(idref)
       ENDIF
       ALLOCATE(idref(0:ho_nref))
       IF(ALLOCATED(wsfr_save))THEN
          DEALLOCATE(wsfr_save)
       ENDIF
       ALLOCATE(wsfr_save(ho_nref,4,4))
       idref(0)=0
       idref(1)=iref_set_max
       IF(iref_set_max-iref_set_min+1.NE.ho_nref)THEN
          WRITE(*,*)'Incorrect the number of references #2:',ho_nref,iref_set_min,iref_set_max
          STOP
       ENDIF
       DO i=2,ho_nref
          idref(i)=idref(1)+i-1
       ENDDO
       WRITE(*,*) 'Doing reference reweight'
       init=1
    END IF
    IF(force_initial_reweight_ref)THEN
       ! force to update the wsf saved values
       wsfr_saved(1:4,1:4)=.FALSE.
       force_initial_reweight_ref=.FALSE.
    ENDIF
    ! assign xp1 and xp2 values first
    SELECT CASE(itype)
    CASE(1)
       xp1=xp1_real
       xp2=xp2_real
    CASE(2)
       xp1=xp1_real
       xp2=xp2_born
    CASE(3)
       xp1=xp1_born
       xp2=xp2_real
    CASE(4)
       xp1=xp1_born
       xp2=xp2_born
    CASE DEFAULT
       WRITE(*,*)"ERROR: gg_Higgs_NNLO_reweight_xsec_ref does not know the itype = ",itype
       STOP
    END SELECT

    IF(.NOT.wsfr_saved(inum,itype))THEN
       ! reference: pp
       INCLUDE "../../../input/lhapdf/call_initnpdf"
       PureProton1=.TRUE.
       PureProton2=.TRUE.
       IF(istruc.EQ.1)THEN
          CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
       ELSE
          WRITE(*,*)"Erraor: unknow the value of istruc in pp_oniumX_reweight_xsec_ref # 1"
          STOP
       ENDIF
       wgtxsecref(1)=wsfw/wsf0
       wsfr_save(1,inum,itype)=wsfw
       IF(ho_nref.LE.1)THEN
          PureProton1=.FALSE.
          PureProton2=.FALSE.
          wsfr_saved(inum,itype)=.TRUE.
          RETURN
       ENDIF
       ! reference: pB collision for AB
       PureProton2=.FALSE.
       CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
       wgtxsecref(2)=wsfw/wsf0
       wsfr_save(2,inum,itype)=wsfw
       ! reference: Ap collision for AB
       PureProton2=.TRUE.
       PureProton1=.FALSE.
       CALL strf_pdf_gg_Higgs_NNLO(wsfw,inum)
       wgtxsecref(3)=wsfw/wsf0
       wsfr_save(3,inum,itype)=wsfw

       PureProton1=.FALSE.
       PureProton2=.FALSE.
       wsfr_saved(inum,itype)=.TRUE.
    ELSE
       ! use the saved ones
       wsfw=wsfr_save(1,inum,itype)
       wgtxsecref(1)=wsfw/wsf0
       IF(ho_nref.LE.1)RETURN
       wsfw=wsfr_save(2,inum,itype)
       wgtxsecref(2)=wsfw/wsf0
       wsfw=wsfr_save(3,inum,itype)
       wgtxsecref(3)=wsfw/wsf0
    ENDIF
    RETURN
  END SUBROUTINE gg_Higgs_NNLO_reweight_xsec_ref

  SUBROUTINE strf_pdf_gg_Higgs_NNLO(wsf,inum)
    USE CTEQ6PDF
    USE Structf_PDFs
    IMPLICIT NONE
    INTEGER::ipp=1
    INTEGER::ih=1 ! ih=1 no photon PDF, ih=2, photon from proton/anti-proton, ih=3 phton from electron/positron
    REAL(KIND(1d0)),INTENT(OUT)::wsf
    INTEGER,INTENT(IN)::inum ! inum = 1: gg; inum=2: gq+gqbar; inum=3: qg+qbarg; inum=4: qqbar+qbarq+qq'
    INTEGER::init=0
    LOGICAL::use_cteq6_f90=.TRUE.
    REAL(KIND=DBL),DIMENSION(-7:7)::pdflist
    SAVE init,ipp,use_cteq6_f90
    REAL(KIND(1d0))::glu1_ct,glu2_ct,u1_ct,u2_ct,d1_ct,d2_ct,s1_ct,s2_ct,c1_ct,c2_ct,b1_ct,b2_ct,&
         ub1_ct,ub2_ct,db1_ct,db2_ct,sb1_ct,sb2_ct,cb1_ct,cb2_ct,bb1_ct,bb2_ct,sf_ct
    REAL(KIND(1d0))::quarks1,quarks2
    IF(iPDFSUP1.EQ.iPDFSUP2)THEN
       INCLUDE "../lhapdf/call_strf_lhapdf"
    ELSE
       INCLUDE "../lhapdf/call_strf_mlhapdf"
    ENDIF
    IF(init.EQ.0)THEN
       SELECT CASE(iPDFSUP1)
       CASE(10000)
          CALL SetCtq6f90(1)
          pdlabel='cteq6_m'
          nloop=2
          alphaQCD2=0.118d0
          use_cteq6_f90=.TRUE.
       CASE(10041)
          CALL SetCtq6f90(3)
          pdlabel='cteq6_l'
          nloop=2
          alphaQCD2=0.118d0
          use_cteq6_f90=.TRUE.
       CASE(10042)
          CALL SetCtq6f90(4)
          pdlabel='cteq6l1'
          nloop=1
          alphaQCD2=0.130d0
          use_cteq6_f90=.TRUE.
       CASE DEFAULT
          CALL pdfset_internal
          use_cteq6_f90=.FALSE.
       END SELECT
       IF(COLL_TYPE.EQ.1)THEN
          ipp=2
       ELSE
          ipp=1
       ENDIF
       init=1
    ENDIF
    
    CALL Higgs_scale(scale)
    scale=muF_over_ref*scale
    IF(reweight_scale)THEN
       IF(reweight_Fscale_phase.EQ.1)THEN
          scale=scale*rw_Fscale_up
       ELSEIF(reweight_Fscale_phase.EQ.-1)THEN
          scale=scale*rw_Fscale_down
       ENDIF
    ENDIF
    
    IF(use_cteq6_f90)THEN
       glu1_ct = xp1*Ctq6Pdf_f90(0,xp1,scale)
       glu2_ct = xp2*Ctq6Pdf_f90(0,xp2,scale)
       u1_ct   = xp1*Ctq6Pdf_f90(1,xp1,scale)
       u2_ct   = xp2*Ctq6Pdf_f90(1,xp2,scale)
       d1_ct   = xp1*Ctq6Pdf_f90(2,xp1,scale)
       d2_ct   = xp2*Ctq6Pdf_f90(2,xp2,scale)
       s1_ct   = xp1*Ctq6Pdf_f90(3,xp1,scale)
       s2_ct   = xp2*Ctq6Pdf_f90(3,xp2,scale)
       c1_ct   = xp1*Ctq6Pdf_f90(4,xp1,scale)
       c2_ct   = xp2*Ctq6Pdf_f90(4,xp2,scale)
       b1_ct   = xp1*Ctq6Pdf_f90(5,xp1,scale)
       b2_ct   = xp2*Ctq6Pdf_f90(5,xp2,scale)
       ub1_ct  = xp1*Ctq6Pdf_f90(-1,xp1,scale)
       ub2_ct  = xp2*Ctq6Pdf_f90(-1,xp2,scale)
       db1_ct  = xp1*Ctq6Pdf_f90(-2,xp1,scale)
       db2_ct  = xp2*Ctq6Pdf_f90(-2,xp2,scale)
       sb1_ct  = xp1*Ctq6Pdf_f90(-3,xp1,scale)
       sb2_ct  = xp2*Ctq6Pdf_f90(-3,xp2,scale)
       cb1_ct  = xp1*Ctq6Pdf_f90(-4,xp1,scale)
       cb2_ct  = xp2*Ctq6Pdf_f90(-4,xp2,scale)
       bb1_ct  = xp1*Ctq6Pdf_f90(-5,xp1,scale)
       bb2_ct  = xp2*Ctq6Pdf_f90(-5,xp2,scale)
    ELSE
       CALL fdist(ih,xp1,scale,pdflist(-7:7))
       glu1_ct = xp1*pdflist(0)
       u1_ct   = xp1*pdflist(2)
       d1_ct   = xp1*pdflist(1)
       s1_ct   = xp1*pdflist(3)
       c1_ct   = xp1*pdflist(4)
       b1_ct   = xp1*pdflist(5)
       ub1_ct  = xp1*pdflist(-2)
       db1_ct  = xp1*pdflist(-1)
       sb1_ct  = xp1*pdflist(-3)
       cb1_ct  = xp1*pdflist(-4)
       bb1_ct  = xp1*pdflist(-5)
       CALL fdist(ih,xp2,scale,pdflist(-7:7))
       glu2_ct = xp2*pdflist(0)
       u2_ct   = xp2*pdflist(2)
       d2_ct   = xp2*pdflist(1)
       s2_ct   = xp2*pdflist(3)
       c2_ct   = xp2*pdflist(4)
       b2_ct   = xp2*pdflist(5)
       ub2_ct  = xp2*pdflist(-2)
       db2_ct  = xp2*pdflist(-1)
       sb2_ct  = xp2*pdflist(-3)
       cb2_ct  = xp2*pdflist(-4)
       bb2_ct  = xp2*pdflist(-5)
    ENDIF
    ! recover it
    IF(reweight_scale)THEN
       IF(reweight_Fscale_phase.EQ.1)THEN
          scale=scale/rw_Fscale_up
       ELSEIF(reweight_Fscale_phase.EQ.-1)THEN
          scale=scale/rw_Fscale_down
       ENDIF
    ENDIF
    IF(inum.EQ.1)THEN
       ! gg
       wsf=glu1_ct*glu2_ct*wjac/xp1/xp2
    ELSEIF(inum.EQ.2)THEN
       ! gq+gqbar
       wsf=glu1_ct*(u2_ct+ub2_ct+d2_ct+db2_ct+s2_ct+sb2_ct)
       IF(nlq.GT.3)THEN
          wsf=wsf+glu1_ct*(c2_ct+cb2_ct)
       ENDIF
       IF(nlq.GT.4)THEN
          wsf=wsf+glu1_ct*(b2_ct+bb2_ct)
       ENDIF
       wsf=wsf*wjac/xp1/xp2
    ELSEIF(inum.EQ.3)THEN
       ! qg+qbarg
       wsf=glu2_ct*(u1_ct+ub1_ct+d1_ct+db1_ct+s1_ct+sb1_ct)
       IF(nlq.GT.3)THEN
          wsf=wsf+glu2_ct*(c1_ct+cb1_ct)
       ENDIF
       IF(nlq.GT.4)THEN
          wsf=wsf+glu2_ct*(b1_ct+bb1_ct)
       ENDIF
       wsf=wsf*wjac/xp1/xp2
    ELSEIF(inum.EQ.4)THEN
       ! qq' (all two (ani-)quark combinations)
       quarks1=u1_ct+ub1_ct+d1_ct+db1_ct+s1_ct+sb1_ct
       quarks2=u2_ct+ub2_ct+d2_ct+db2_ct+s2_ct+sb2_ct
       IF(nlq.GT.3)THEN
          quarks1=quarks1+c1_ct+cb1_ct
          quarks2=quarks2+c2_ct+cb2_ct
       ENDIF
       IF(nlq.GT.4)THEN
          quarks1=quarks1+b1_ct+bb1_ct
          quarks2=quarks2+b2_ct+bb2_ct
       ENDIF
       wsf=quarks1*quarks2
       wsf=wsf*wjac/xp1/xp2
    ENDIF
    IF(init.EQ.0)init=1
  END SUBROUTINE strf_pdf_gg_Higgs_NNLO

  SUBROUTINE readcuts_gg_Higgs_NNLO
    IMPLICIT NONE
    CHARACTER(len=24)::file
    LOGICAL::lexist
    INTEGER::iounit,flag=0,i,i1,j,j1
    REAL(KIND(1d0))::ptq,ptcharm,ptconia,etaq,ycq,ycqlow,etaconia,ycconia,ycconialow
    REAL(KIND(1d0))::cutoff,ptbonia,etabonia,ycbonia,ycbonialow
    REAL(KIND(1d0))::xFcq,xFcqlow,xFcconia,xFcconialow,xFcbonia,xFcbonialow
    REAL(KIND(1d0))::gbeamq,drqq,gqq,maxptq,maxptcharm,maxptconia,maxptbonia
    ! open default input file
    INQUIRE(FILE=TRIM(input_dir)//"default.inp",EXIST=lexist)
    IF(.NOT.lexist)THEN
       PRINT *,"Warning: the file default.inp does not exist ! STOP !"
       STOP
    ENDIF
    INQUIRE(FILE=TRIM(input_dir)//"default.inp",OPENED=lexist)
    IF(lexist)THEN
       INQUIRE(FILE=TRIM(input_dir)//"default.inp",NUMBER=iounit)
       IF(iounit.NE.udefault)THEN
          PRINT *,"WARNING: the default.inp has been linked with another unit ! Close and reopen !"
          CLOSE(UNIT=iounit)
          OPEN(UNIT=udefault,FILE=TRIM(input_dir)//"default.inp")
       ENDIF
    ELSE
       OPEN(UNIT=udefault,FILE=TRIM(input_dir)//"default.inp")
    ENDIF
    ! open user's input file
    IF(TRIM(Input_File)/="default.inp")THEN
       INQUIRE(FILE=TRIM(input_dir)//TRIM(Input_File),EXIST=lexist)
       IF(.NOT.lexist)THEN
          PRINT *,"Warning: the file "//TRIM(Input_File)//" does not exist ! STOP !"
          STOP
       ENDIF
       INQUIRE(FILE=TRIM(input_dir)//TRIM(Input_File),OPENED=lexist)
       IF(lexist)THEN
          INQUIRE(FILE=TRIM(input_dir)//TRIM(Input_File),NUMBER=iounit)
          IF(iounit.NE.uinput)THEN
             PRINT *,"WARNING: the "//TRIM(Input_File)//" has been linked with another unit ! Close and reopen !"
             CLOSE(UNIT=iounit)
             OPEN(UNIT=uinput,FILE=TRIM(input_dir)//TRIM(Input_File))
          ENDIF
       ELSE
          OPEN(UNIT=uinput,FILE=TRIM(input_dir)//TRIM(Input_File))
       ENDIF
    ELSE
       flag=1
    ENDIF
    cutoff=readvalue_r("cutoffp",flag)
    ptc(3:20)=cutoff
    maxptc(3:20)=-1d0
    drc(3:20,3:20)=0
    etac(3:20)=20
    yycut(3:20)=1d9
    IF(absrap)THEN
       yycutlow(3:20)=0d0
    ELSE
       yycutlow(3:20)=-1d9
    ENDIF
    xFcut(3:20)=1d0
    xFcutlow(3:20)=-1d0
    xFcutflag=.FALSE.
    ec(3:20)=cutoff
    c1(3:20)=1d0
    c2(3:20)=1d0
    cc(3:20,3:20)=1d0
    gmas(3:20,3:20)=cutoff
    gbeammass(1:2,3:20)=0d0
    y1cup=30d0
    y1clow=0d0
    y1cup=readvalue_r("maxy1c",flag)
    y1clow=readvalue_r("miny1c",flag)
    ! minimum quark pt
    ptq=readvalue_r("minptq",flag)
    ! maximum quark pt
    maxptq=readvalue_r("maxptq",flag)
    ! minimum charm pt
    ptcharm=readvalue_r("minptc",flag)
    ! maximum charm pt
    maxptcharm=readvalue_r("maxptc",flag)
    ! minimum charmonia pt
    ptconia=readvalue_r("minptconia",flag)
    ! maximum charmonia pt
    maxptconia=readvalue_r("maxptconia",flag)
    ! minimum bottomonia pt
    ptbonia=readvalue_r("minptbonia",flag)
    ! maximum bottomonia pt
    maxptbonia=readvalue_r("maxptbonia",flag)
    ! maximum rapidity quark
    etaq=readvalue_r("maxrapq",flag)
    ! maximum y rapidity quark
    ycq=readvalue_r("maxyrapq",flag)
    ! minimum y rapidity quark
    ycqlow=readvalue_r("minyrapq",flag)
    ! maximum Feynman parameter xF
    xFcq=readvalue_r("maxxFq",flag)
    ! minimum Feynman parameter xF
    xFcqlow=readvalue_r("minxFq",flag)
    ! maximum rapidity charmonium
    etaconia=readvalue_r("maxrapconia",flag)
    ! maximum y rapidity charmonium
    ycconia=readvalue_r("maxyrapconia",flag)
    ! minimum y rapidity charmonium
    ycconialow=readvalue_r("minyrapconia",flag)
    ! maximum Feynman parameter xF
    xFcconia=readvalue_r("maxxFconia",flag)
    ! minimum Feynman parameter xF
    xFcconialow=readvalue_r("minxFconia",flag)
    ! maxmimum rapidity bottomonia
    etabonia=readvalue_r("maxrapbonia",flag)
    ! maximum y rapidity bottomonia
    ycbonia=readvalue_r("maxyrapbonia",flag)
    ! minimum y rapidity bottomonia
    ycbonialow=readvalue_r("minyrapbonia",flag)
    ! maximum Feynman parameter xF
    xFcbonia=readvalue_r("maxxFbonia",flag)
    ! minimum Feynman parameter xF
    xFcbonialow=readvalue_r("minxFbonia",flag)
    ! minimum DR quark with quark
    drqq=readvalue_r("mindrqq",flag)
    ! minimum mass quark with quark
    gqq=readvalue_r("minmqqp",flag)
    ! minimum mass u,d,s quarks and gluon  with partonic beam
    gbeamq=readvalue_r("minmqbeam",flag)
    CLOSE(UNIT=udefault)
    CLOSE(UNIT=uinput)
    ! gluon as 2nd final particle
    ptc(4)=ptq
    IF(maxptq.GE.0d0)maxptc(4)=maxptq
    etac(4)=etaq
    yycut(4)=ycq
    yycutlow(4)=ycqlow
    xFcut(4)=xFcq
    xFcutlow(4)=xFcqlow
    ! Higgs or quarkonium as 1st final particle
    IF(iflh(3).GE.440000.AND.iflh(3).LE.449999)THEN
       ptc(3)=ptconia
       IF(maxptconia.GE.0d0)maxptc(3)=maxptconia
       etac(3)=etaconia
       yycut(3)=ycconia
       yycutlow(3)=ycconialow
       xFcut(3)=xFcconia
       xFcutlow(3)=xFcconialow
    ELSEIF(iflh(3).GE.550000.AND.iflh(3).LE.559999)THEN
       ptc(3)=ptbonia
       IF(maxptbonia.GE.0d0)maxptc(3)=maxptbonia
       etac(3)=etabonia
       yycut(3)=ycbonia
       yycutlow(3)=ycbonialow
       xFcut(3)=xFcbonia
       xFcutlow(3)=xFcbonialow
    ELSE
       ! Higgs otherwise
       ptc(3)=0d0
       maxptc(3)=0d0
       etac(3)=30d0
       yycutlow(3)=-30d0
       yycut(3)=30d0
       xFcut(3)=1d0
       xFcutlow(3)=-1d0
    ENDIF
    WRITE(*,*)'---------------------------------------------------'
    WRITE(*,*)'        the cuts        '
    DO i=3,4
!       WRITE(*,*)'pt     of  ',i,'   particle   ',ptc(i)
!       IF(maxptc(i).GE.0d0)THEN
!          WRITE(*,*)'max pt of  ',i,'   particle   ',maxptc(i)
!          IF(maxptc(i).LE.ptc(i))THEN
!             WRITE(*,*)"ERROR: One final state ",i," was cut off by pt cut"
!             STOP
!          ENDIF
!       ELSE
!          WRITE(*,*)'no max pt cut of  ',i,'   particle   '
!       ENDIF
       WRITE(*,*)'rapidity of  ',i,'   particle   ',etac(i)
       WRITE(*,*)'max y rapidity of ',i,'   particle   ',yycut(i)
       WRITE(*,*)'min y rapidity of ',i,'   particle   ',yycutlow(i)
       IF(yycut(i).LE.yycutlow(i))THEN
          WRITE(*,*)"ERROR:One final state ",i," was cut off by y rapidity"
          STOP
       ENDIF
    ENDDO
    RETURN
  END SUBROUTINE readcuts_gg_Higgs_NNLO

  SUBROUTINE Cuts_gg_Higgs_NNLO(icut,inum)
    IMPLICIT NONE
    INTEGER,INTENT(OUT)::icut
    INTEGER,INTENT(IN)::inum ! 1: (n+2) body; 2-3: (n+1) body; 4: n body
    INTEGER::l,l1,l2,flag
    REAL(KIND(1d0))::s,d1,d2,dr,pt,eta,aaa,bbb,ptcut
    REAL(KIND(1d0)),DIMENSION(4)::ponia,pboo2
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    REAL(KIND(1d0))::e,q
    REAL(KIND(1d0)),DIMENSION(3:5,4)::pmom_temp
    INTEGER::lmax
    icut=0
    flag=0
    IF(inum.EQ.1)THEN
       pmom_temp(3,1:4)=Higgs_pmom_real_real(3,1:4)
       pmom_temp(4,1:4)=Higgs_pmom_real_real(4,1:4)
       pmom_temp(5,1:4)=Higgs_pmom_real_real(5,1:4)
       lmax=5
    ELSEIF(inum.EQ.2)THEN
       pmom_temp(3,1:4)=Higgs_pmom_real_born(3,1:4)
       pmom_temp(4,1:4)=Higgs_pmom_real_born(4,1:4)
       pmom_temp(5,1:4)=Higgs_pmom_real_born(5,1:4)
       lmax=4
    ELSEIF(inum.EQ.3)THEN
       pmom_temp(3,1:4)=Higgs_pmom_born_real(3,1:4)
       pmom_temp(4,1:4)=Higgs_pmom_born_real(4,1:4)
       pmom_temp(5,1:4)=Higgs_pmom_born_real(5,1:4)
       lmax=4
    ELSE
       pmom_temp(3,1:4)=Higgs_pmom_born_born(3,1:4)
       pmom_temp(4,1:4)=Higgs_pmom_born_born(4,1:4)
       pmom_temp(5,1:4)=Higgs_pmom_born_born(5,1:4)
       lmax=3
    ENDIF
    ! let us assume only impose cuts on Higgs boson
    ! redefine lmax
    lmax=3
    DO l=3,lmax
       pt=SQRT(pmom_temp(l,1)**2+pmom_temp(l,2)**2)
       !IF(pt.LT.ptc(l))THEN
       !   flag=1
       !   EXIT
       !ENDIF
       !IF(maxptc(l).GE.0d0)THEN
       !   IF(pt.GT.maxptc(l))THEN
       !      flag=1
       !      EXIT
       !   ENDIF
       !ENDIF
       eta=prapidity(pmom_temp(l,1:4))
       IF(ABS(eta).GT.etac(l))THEN
          flag=1
          EXIT
       ENDIF
       eta=rapidity(pmom_temp(l,1:4))
       IF(absrap)eta=ABS(eta)
       IF(eta.GT.yycut(l).OR.eta.LT.yycutlow(l))THEN
          flag=1
          EXIT
       ENDIF
       ! special for the first particle , which can be used to calculate the y distribution
       IF(l.EQ.3)THEN
          IF(eta.GT.y1cup.OR.eta.LT.y1clow)THEN
             flag=1
             EXIT
          ENDIF
       ENDIF
    ENDDO
    IF(flag.EQ.0)icut=1
    ! treat xF cut
    ! it shoule be done in the collision frame
    IF(xFcutflag.AND.icut.EQ.1.AND.flag.EQ.0)THEN
       q=ehat
       e=q/SQRT(xp1*xp2)
       IF(.NOT.labeqcoll)THEN
          IF(.NOT.fixtarget)THEN
             pboo2(4)=(ebeam(1)+ebeam(2))
             pboo2(3)=-(ebeam(1)-ebeam(2))
          ELSE
             IF(.NOT.fixtargetrev)THEN
                pboo2(4)=(ebeam(1)+ebeam(2))
                pboo2(3)=-ebeam(1)
             ELSE
                pboo2(4)=(ebeam(1)+ebeam(2))
                pboo2(3)=ebeam(2)
             ENDIF
          ENDIF
          pboo2(1:2)=0
          ! boost from the lab frame to the collision frame
          DO l=3,lmax
             CALL boostl(e,pboo2,pmom_temp(l,1:4))
          ENDDO
       ENDIF
       DO l=3,lmax
          eta=xFeynman(pmom_temp(l,1:4),e)
          IF(eta.GT.xFcut(l).OR.eta.LT.xFcutlow(l))THEN
             icut=0
             EXIT
          ENDIF
       ENDDO
       IF(.NOT.labeqcoll)THEN
          pboo2(3)=-pboo2(3)
          ! boost back to the lab frame
          DO l=3,lmax
             CALL boostl(e,pboo2,pmom_temp(l,1:4))
          ENDDO
       ENDIF
    ENDIF
    RETURN
  END SUBROUTINE Cuts_gg_Higgs_NNLO

  SUBROUTINE call_gg_Higgs_NNLO_BSB(za,zb,xa,xb,Lcut,LS,aso2pi,iorder,icut,me)
    USE convolution_communicator
    USE qcd_tmd_beamsoftbeam_functions_expinas
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb,Lcut,LS
    INTEGER,INTENT(IN)::iorder ! 0: LO; 1: NLO; 2: NNLO
    ! 0:2 - 0:iorder the order we need
    ! 1:4 - gg, gq+gqbar, qg+qbarg, qq'
    ! 1:9 - ss_piece
    REAL(KIND(1d0)),DIMENSION(0:2,4,9),INTENT(OUT)::me
    REAL(KIND(1d0)),INTENT(IN)::aso2pi
    ! icut - 0 (do not pass cuts); 1 (pass cuts)
    ! 1: real real
    ! 2: real born
    ! 3: born real
    ! 4: born born
    INTEGER,DIMENSION(4),INTENT(IN)::icut
    INTEGER::i,j,k
    me(0:2,1:4,1:9)=0d0
    IF(iorder.LT.0)RETURN
    DO k=1,9
       IF(k.EQ.1.AND.icut(4).EQ.0)CYCLE
       IF((k.EQ.2.OR.k.EQ.4).AND.icut(3).EQ.0)CYCLE
       IF((k.EQ.3.OR.k.EQ.5).AND.icut(2).EQ.0)CYCLE
       IF((k.GE.6.AND.k.LE.9).AND.icut(1).EQ.0)CYCLE
       ss_piece=k
       me(0,1,k)=TMD_BeamSoftBeam_IggSggIgg0(za,zb,xa,xb,Lcut,LS)
       me(0,2,k)=TMD_BeamSoftBeam_IggSggIgq0(za,zb,xa,xb,Lcut,LS)
       me(0,3,k)=TMD_BeamSoftBeam_IgqSggIgg0(za,zb,xa,xb,Lcut,LS)
       me(0,4,k)=TMD_BeamSoftBeam_IgqSggIgq0(za,zb,xa,xb,Lcut,LS)
       !DO j=1,4
       !   me(-1,j,k)=me(-1,j,k)+me(0,j,k)
       !ENDDO
    ENDDO

    IF(iorder.LT.1)RETURN

    DO k=1,9
       IF(k.EQ.1.AND.icut(4).EQ.0)CYCLE
       IF((k.EQ.2.OR.k.EQ.4).AND.icut(3).EQ.0)CYCLE
       IF((k.EQ.3.OR.k.EQ.5).AND.icut(2).EQ.0)CYCLE
       IF((k.GE.6.AND.k.LE.9).AND.icut(1).EQ.0)CYCLE
       ss_piece=k
       me(1,1,k)=TMD_BeamSoftBeam_IggSggIgg1(za,zb,xa,xb,Lcut,LS)
       me(1,2,k)=TMD_BeamSoftBeam_IggSggIgq1(za,zb,xa,xb,Lcut,LS)
       me(1,3,k)=TMD_BeamSoftBeam_IgqSggIgg1(za,zb,xa,xb,Lcut,LS)
       me(1,4,k)=TMD_BeamSoftBeam_IgqSggIgq1(za,zb,xa,xb,Lcut,LS)
       DO j=1,4
          me(1,j,k)=aso2pi*me(1,j,k)
       !   me(-1,j,k)=me(-1,j,k)+me(1,j,k)
       ENDDO
    ENDDO

    IF(iorder.LT.2)RETURN
    
    DO k=1,9
       IF(k.EQ.1.AND.icut(4).EQ.0)CYCLE
       IF((k.EQ.2.OR.k.EQ.4).AND.icut(3).EQ.0)CYCLE
       IF((k.EQ.3.OR.k.EQ.5).AND.icut(2).EQ.0)CYCLE
       IF((k.GE.6.AND.k.LE.9).AND.icut(1).EQ.0)CYCLE
       ss_piece=k
       me(2,1,k)=TMD_BeamSoftBeam_IggSggIgg2(za,zb,xa,xb,Lcut,LS)
       me(2,2,k)=TMD_BeamSoftBeam_IggSggIgq2(za,zb,xa,xb,Lcut,LS)
       me(2,3,k)=TMD_BeamSoftBeam_IgqSggIgg2(za,zb,xa,xb,Lcut,LS)
       me(2,4,k)=TMD_BeamSoftBeam_IgqSggIgq2(za,zb,xa,xb,Lcut,LS)
       DO j=1,4
          me(2,j,k)=aso2pi**2*me(2,j,k)
       !   me(-1,j,k)=me(-1,j,k)+me(2,j,k)
       ENDDO
    ENDDO
    RETURN
  END SUBROUTINE call_gg_Higgs_NNLO_BSB

  SUBROUTINE call_gg_Higgs_NNLO_HardFunction(iorder,aso2pi,L,Lt,me)
    USE ggH_NNLO_hard_function
    IMPLICIT NONE
    INTEGER,INTENT(IN)::iorder
    COMPLEX(KIND(1d0)),INTENT(IN)::L,Lt
    REAL(KIND(1d0)),INTENT(IN)::aso2pi
    REAL(KIND(1d0)),DIMENSION(0:2),INTENT(OUT)::me
    me(0:2)=0d0
    IF(iorder.LT.0)RETURN
    me(0)=1d0
    IF(iorder.LT.1)RETURN
    me(1)=ggH_H1(L,Lt)
    me(1)=aso2pi*me(1)
    IF(iorder.LT.2)RETURN
    me(2)=ggH_H2(L,Lt)
    me(2)=aso2pi**2*me(2)
    RETURN
  END SUBROUTINE call_gg_Higgs_NNLO_HardFunction

  SUBROUTINE call_gg_Higgs_Born(vev,aso2pi,me)
    USE ggH_NNLO_hard_function
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vev,aso2pi
    REAL(KIND(1d0)),INTENT(OUT)::me
    me=ggH_sigma0(vev)
    bpower=2
    me=me*aso2pi**bpower
    RETURN
  END SUBROUTINE call_gg_Higgs_Born

  SUBROUTINE call_gg_etaQ_NNLO_HardFunction(iorder,aso2pi,L,LmuNRQCD,me)
    USE ggetaQ_NNLO_hard_function
    IMPLICIT NONE
    INTEGER,INTENT(IN)::iorder
    COMPLEX(KIND(1d0)),INTENT(IN)::L,LmuNRQCD
    REAL(KIND(1d0)),INTENT(IN)::aso2pi
    REAL(KIND(1d0)),DIMENSION(0:2),INTENT(OUT)::me
    me(0:2)=0d0
    IF(iorder.LT.0)RETURN
    me(0)=1d0
    IF(iorder.LT.1)RETURN
    me(1)=ggetaQ_H1(L)
    me(1)=aso2pi*me(1)
    IF(iorder.LT.2)RETURN
    me(2)=ggetaQ_H2(L,LmuNRQCD)
    me(2)=aso2pi**2*me(2)
    RETURN
  END SUBROUTINE call_gg_etaQ_NNLO_HardFunction

  SUBROUTINE call_gg_etaQ_Born(LDME,aso2pi,me)
    USE ggetaQ_NNLO_hard_function
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::LDME,aso2pi
    REAL(KIND(1d0)),INTENT(OUT)::me
    me=ggetaQ_sigma0(LDME)
    bpower=2
    me=me*aso2pi**bpower
    RETURN
  END SUBROUTINE call_gg_etaQ_Born

  SUBROUTINE combine_me_by_order(iorder,me_born,me_hard,me_bsb,me,me_order)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::iorder
    REAL(KIND(1d0)),INTENT(IN)::me_born
    REAL(KIND(1d0)),DIMENSION(0:2),INTENT(IN)::me_hard
    REAL(KIND(1d0)),DIMENSION(0:2,4,9),INTENT(IN)::me_bsb
    REAL(KIND(1d0)),DIMENSION(4,9),INTENT(OUT)::me
    REAL(KIND(1d0)),DIMENSION(0:2,4,9),INTENT(OUT)::me_order
    INTEGER::i,j,k
    me(1:4,1:9)=0d0
    me_order(0:2,1:4,1:9)=0d0
    IF(iorder.LT.0)RETURN
    DO j=1,4
       DO k=1,9
          me_order(0,j,k)=me_born*me_hard(0)*me_bsb(0,j,k)
          me(j,k)=me_order(0,j,k)
       ENDDO
    ENDDO
    IF(iorder.LT.1)RETURN
    DO j=1,4
       DO k=1,9
          me_order(1,j,k)=me_born*(me_hard(0)*me_bsb(1,j,k)+me_hard(1)*me_bsb(0,j,k))
          me(j,k)=me(j,k)+me_order(1,j,k)
       ENDDO
    ENDDO
    IF(iorder.LT.2)RETURN
    DO j=1,4
       DO k=1,9
          me_order(2,j,k)=me_born*(me_hard(0)*me_bsb(2,j,k)+me_hard(1)*me_bsb(1,j,k)&
               +me_hard(2)*me_bsb(0,j,k))
          me(j,k)=me(j,k)+me_order(2,j,k)
       ENDDO
    ENDDO
    RETURN
  END SUBROUTINE combine_me_by_order

  SUBROUTINE setup_process(istate)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::istate
    SELECT CASE(istate)
    CASE(1)
       ! g g > H
       nhad=5
       iflh(1)=35
       iflh(2)=35
       iflh(3)=25
       iflh(4)=35
       iflh(5)=35
       nlq=5
       ! the input parameters
       CALL ReadElem_real('higmass',MH)
       CALL ReadElem_real('tmass',MT)
       CALL ReadElem_real('gfermi',GF)
       vev=1d0/DSQRT(DSQRT(2d0)*GF)
    CASE(2)
       ! b b~ > H
       nhad=5
       iflh(1)=12
       iflh(2)=-12
       iflh(3)=25
       iflh(4)=35
       iflh(5)=35
       nlq=5
       ! the input parameters
       CALL ReadElem_real('higmass',MH)
       CALL ReadElem_real('bmass',MB) ! this is interpreted as MSbar MB at scale MSbar MB
    CASE(11)
       ! b b~ > H H (Higgstralung+Yukawa)
       nhad=6
       iflh(1)=12
       iflh(2)=-12
       iflh(3)=25
       iflh(4)=25
       iflh(5)=35
       iflh(6)=35
       nlq=5
       ! the input parameters
       CALL ReadElem_real('higmass',MH)
    CASE(15)
       ! g g > eta_c
       nhad=5
       iflh(1)=35
       iflh(2)=35
       iflh(3)=441001
       iflh(4)=35
       iflh(5)=35
       nlq=3
       ! the input parameters
       CALL ReadElem_real('cmass',MH)
       onium_muNRQCD0=MH ! the initial NRQCD scale is MQ
       MH=2d0*MH
       MetaQ=MH
       ! this is LDME at mu0=mQ
       CALL ReadElem_real('LDMEcc1S01',LDMEetaQ_MQ)
       LDMEetaQ_MQ=6d0*LDMEetaQ_MQ ! convert to the normal convention
       LDMEetaQ=LDMEetaQ_MQ
       LDMEetaQ_up=LDMEetaQ
       LDMEetaQ_down=LDMEetaQ
    CASE(16)
       ! g g > eta_b
       nhad=5
       iflh(1)=35
       iflh(2)=35
       iflh(3)=551001
       iflh(4)=35
       iflh(5)=35
       nlq=4
       CALL ReadElem_real('bmass',MH)
       onium_muNRQCD0=MH ! the initial NRQCD scale is MQ
       MH=2d0*MH
       MetaQ=MH
       CALL ReadElem_real('LDMEbb1S01',LDMEetaQ_MQ)
       LDMEetaQ_MQ=6d0*LDMEetaQ_MQ ! convert to the normal convention
       LDMEetaQ=LDMEetaQ_MQ
       LDMEetaQ_up=LDMEetaQ
       LDMEetaQ_down=LDMEetaQ
    CASE DEFAULT
       WRITE(*,*)"Error: Do not know the process id = ",istate
       STOP
    END SELECT
  END SUBROUTINE setup_process

  FUNCTION state2hoid(istate) RESULT(ho_id)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::istate
    INTEGER::ho_id
    SELECT CASE(istate)
    CASE(1)
       ! Higgs
       ho_id=25
    CASE(2)
       ! eta_c
       ho_id=441001
    CASE(3)
       ! eta_b
       ho_id=551001
    CASE DEFAULT
       WRITE(*,*)"ERROR: do not know the state=",istate
       STOP
    END SELECT
    RETURN
  END FUNCTION state2hoid

  ! according to Eq.(2.5) in 2007.01737
  ! note that is at the amplitude level
  ! here we should take at the amplitude square level
  FUNCTION LDME_RG_RUNNING(nloop,as0,asmuNRQCD,LDME0) RESULT(res)
    USE qcd_constants
    USE global_constants
    IMPLICIT NONE
    ! nloop: 0 (only beta0), 1 (+beta1), 2 (+beta2)
    INTEGER,INTENT(IN)::nloop
    REAL(KIND(1d0)),INTENT(IN)::as0,asmuNRQCD,LDME0
    REAL(KIND(1d0))::res,logterm,atanterm1,atanterm2,DELTA
    INTEGER::init=0
    SAVE init
    SELECT CASE(nloop)
    CASE(0)
       IF(init.EQ.0)THEN
          IF(beta0.EQ.0d0)THEN
             WRITE(*,*)"Error: Please update beta functions first"
             STOP
          ENDIF
       ENDIF
       res=TWOPI*CF*(CF+CA*0.5d0)/beta0*(as0-asmuNRQCD)
       res=DEXP(res)*LDME0
    CASE(1)
       IF(init.EQ.0)THEN
          IF(beta0.EQ.0d0.OR.beta1.EQ.0d0)THEN
             WRITE(*,*)"Error: Please update beta functions first"
             STOP
          ENDIF
       ENDIF
       logterm=(beta1*as0/TWOPI+beta0)/(beta1*asmuNRQCD/TWOPI+beta0)
       IF(logterm.LE.0d0)THEN
          WRITE(*,*)"Error: the argument of log is negative"
          STOP
       ENDIF
       logterm=DLOG(logterm)
       res=TWOPI2*CF*(CF+CA*0.5d0)/beta1&
            *logterm
       res=DEXP(res)*LDME0
    CASE DEFAULT
       IF(init.EQ.0)THEN
          IF(beta0.EQ.0d0.OR.beta1.EQ.0d0.OR.beta2.EQ.0d0)THEN
             WRITE(*,*)"Error: Please update beta functions first"
             STOP
          ENDIF
       ENDIF
       DELTA=four*beta0*beta2-beta1**2
       IF(DELTA.LE.0d0)THEN
          WRITE(*,*)"Error: the delta term is zero"
          STOP
       ENDIF
       DELTA=DSQRT(DELTA)
       atanterm1=(beta2/PI*as0+beta1)/DELTA
       atanterm2=(beta2/PI*asmuNRQCD+beta1)/DELTA
       atanterm1=DATAN(atanterm1)
       atanterm2=DATAN(atanterm2)
       res=TWO*TWOPI2*CF*(CF+CA*0.5d0)/DELTA*(atanterm1-atanterm2)
       res=DEXP(res)*LDME0
    END SELECT
    IF(init.EQ.0)init=1
    RETURN
  END FUNCTION LDME_RG_RUNNING
  
END MODULE gg_Higgs_NNLO
