MODULE ggetaQ_NNLO_hard_function
  ! This is the hard function and born partonic cross section of
  ! g g > etaQ process
  ! L=Log(-MetaQ**2/mu**2)
  ! LmuNRQCD=LOG(MetaQ**2/muNRQCD**2)
  USE qcd_constants
  USE global_constants
  USE gg_Higgs_NNLO_Global
  IMPLICIT NONE
  PRIVATE
  PUBLIC::ggetaQ_sigma0
  PUBLIC::ggetaQ_H1,ggetaQ_H2
CONTAINS
  ! it has a global (as/2/pi)**2 factor
  FUNCTION ggetaQ_sigma0(LDME)
    IMPLICIT NONE
    REAL(KIND(1d0))::ggetaQ_sigma0
    REAL(KIND(1d0)),INTENT(IN)::LDME
    ggetaQ_sigma0=8d0*PIQUINTIC/(9d0*MetaQ**5)*LDME
    RETURN
  END FUNCTION ggetaQ_sigma0

  ! (as/2/pi) term of hard function
  FUNCTION ggetaQ_H1(L) RESULT(res)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L
    REAL(KIND(1d0))::res
    res=CA*(-DREAL(L**2)-2d0/3d0*zeta2-22d0/9d0)
    RETURN
  END FUNCTION ggetaQ_H1

  ! (as/2/pi)**2 term of hard function
  FUNCTION ggetaQ_H2(L,LmuNRQCD) RESULT(res)
    ! this is from Claude, Matteo, Melih & Samuel
    ! see those in /Users/erdissshaw/Works/FLibatM/TMDBeamFunctions/etaQ_hardfunction
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),INTENT(IN)::L,LmuNRQCD
    REAL(KIND(1d0))::rL,rL2,rL3,rL4,rLmuNRQCD
    REAL(KIND(1d0))::res
    REAL(KIND(1d0)),PARAMETER::a0=-433.314918767315914061385906314788538525958103494d0
    REAL(KIND(1d0)),PARAMETER::a1=-208.135077008444719804981814657679549016167027378d0
    REAL(KIND(1d0)),PARAMETER::a2=-0.88964614543688194760845864779533868614782294096d0
    REAL(KIND(1d0)),PARAMETER::a3=30.45329850015803113902035637249435645071800483696d0
    REAL(KIND(1d0)),PARAMETER::a4=4.5d0
    REAL(KIND(1d0)),PARAMETER::a1NRQCD=149.140688727572530240165641775906283822518124376d0
    REAL(KIND(1d0)),PARAMETER::a0nl=0.0236550315920693782801613082384580791098916331155871d0
    REAL(KIND(1d0)),PARAMETER::a1nl=8.5615293492100415728828778709504545430789162414443538d0
    REAL(KIND(1d0)),PARAMETER::a2nl=0.2803723055467760478322024237503135305156663979461561d0
    REAL(KIND(1d0)),PARAMETER::a3nl=-0.333333333333333333333333333333333333333333333333333333d0
    rL=DREAL(L)-TWO*LOGTWO
    rL2=rL**2
    rL3=rL**3
    rL4=rL**4
    rLmuNRQCD=DREAL(LmuNRQCD)-TWO*LOGTWO
    res=a0+a1*rL+a2*rL2+a3*rL3+a4*rL4+a1NRQCD*rLmuNRQCD&
         +DBLE(nlq)*(a0nl+a1nl*rL+a2nl*rL2+a3nl*rL3)
    RETURN
  END FUNCTION ggetaQ_H2

END MODULE ggetaQ_NNLO_hard_function
