MODULE lhapdf_rwgt
  USE Helac_Global
  USE Helac_Func_1
  USE lhapdf_rwgt_global
  USE Constants
  USE plot_lhapdf_rwgt
  IMPLICIT NONE
CONTAINS
  SUBROUTINE generate_lhapdf_rwgt
    USE Structf_PDFs
    IMPLICIT NONE
    CHARACTER(len=10),DIMENSION(20)::char
    INTEGER::i,i1,j1,k1,s1x
    INTEGER::nh,innum,outnum
    REAL(KIND(1d0))::minx,dxdx,xnow
    INTEGER::ioerror
    LOGICAL::lexist
    CHARACTER(len=200000)::wk_header
    INTEGER::pdfnumpdf
    REAL(KIND(1d0)),DIMENSION(-7:7)::pdflist
    INTEGER::ih=1,d3min_pdf_wgts,d3max_pdf_wgts
    INTEGER::d3min_replica_wgts,d3max_replica_wgts
    INTEGER::d3min_rst_wgts,d3max_rst_wgts
    !CHARACTER(len=2),DIMENSION(-7:7)::pdflabel
    LOGICAL::isisishesspdf
    WRITE(*,*)'                     THE BEGINNING OF HELAC-Onia'
    WRITE(*,*)'    AddOn Process: '
    WRITE(*,*)'    Generate (n)PDF information via the reweighting method '
    WRITE(*,*)'======================================================================='
    WRITE(*,*)'======================================================================='
    WRITE(*,*)' '
    ! nuclear effect related
    CALL ReadElem_integer('nuclearA_beam1',nuclearA_beam1)
    CALL ReadElem_integer('nuclearA_beam2',nuclearA_beam2)
    CALL ReadElem_integer('nPDF_id',nPDF_id)
    ! Converting Hessian PDFs to MC PDFs
    CALL ReadElem_logic('PDF_Hessian2MC',PDF_Hessian2MC)
    CALL ReadElem_integer('N_MCPDFs',N_MCPDFs)
    CALL ReadElem_logic('lhapdf',uselhapdf)
    CALL ReadElem_integer('pdf',pdfnumpdf)
    iPDFSUP1=pdfnumpdf
    INQUIRE(FILE=TRIM(input_dir)//"paths/lhapdf_data",EXIST=lexist)
    IF(lexist)THEN
       OPEN(UNIT=30333,FILE=TRIM(input_dir)//"paths/lhapdf_data")
       READ(30333,'(A)')LHAPath
       CLOSE(UNIT=30333)
    ELSE
       INQUIRE(FILE=TRIM(input_dir)//"paths/lhapdfpath",EXIST=lexist)
       IF(lexist)THEN
          OPEN(UNIT=30333,FILE=TRIM(input_dir)//"paths/lhapdfpath")
          READ(30333,'(A)')LHAPath
          CLOSE(UNIT=30333)
          i=LEN_TRIM(LHAPath)
          LHAPath=LHAPath(1:i-13)
       ENDIF
    ENDIF
    uselhapdf=uselhapdf.AND.lexist
    WRITE(*,*)'Use LHAPDF ?',uselhapdf
    IF(.NOT.uselhapdf)THEN
       WRITE(*,*)"ERROR: PDF reweighting can only be used when using LHAPDF"
       STOP
       !IF(PDF_Hessian2MC)THEN
       !   PDF_Hessian2MC=.FALSE.
       !   WRITE(*,*)"WARNING: convert Hessian PDF to MC PDF is only possible with LHAPDF"
       !ENDIF
    ENDIF
    lnuclearPDF=.FALSE.
    IF(nPDF_id.NE.0.AND.(nuclearA_beam1.NE.0.OR.nuclearA_beam2.NE.0)&
         .AND.(.NOT.uselhapdf))THEN
       WRITE(*,*)"nuclear PDF can only be used with LHAPDF"
       WRITE(*,*)"nPDF_id=",nPDF_id
       WRITE(*,*)"nuclearA_beam1=",nuclearA_beam1
       WRITE(*,*)"nuclearA_beam2=",nuclearA_beam2
       WRITE(*,*)"Please set lhapdf=T and set the correct path of lhapdf in ho_configuration.txt before ./config"
       STOP
    ENDIF
    rnuclearA_beam1=DBLE(nuclearA_beam1)
    rnuclearA_beam2=DBLE(nuclearA_beam2)
    IF(nPDF_id.EQ.0.AND.(nuclearA_beam1.NE.0.or.nuclearA_beam2.NE.0))THEN
       WRITE(*,*)"WARNING:you are using normal proton PDF for nuclear"
    ENDIF
    IF(nPDF_id.NE.0.AND.(nuclearA_beam1.NE.0.or.nuclearA_beam2.NE.0))THEN
       lnuclearPDF=.TRUE.
    ENDIF
    IF(uselhapdf)THEN
       CALL ReadElem_logic('reweight_pdf',reweight_pdf)
       IF(reweight_pdf.AND.lnuclearPDF.AND.nPDF_id.EQ.1000)THEN
          WRITE(*,*)"WARNING:reweight proton PDF are not allowed when nPDF_id=1000 in PA"
          reweight_pdf=.FALSE.
       ENDIF
       IF(reweight_pdf)THEN
          CALL ReadElem_integer('pdf_min',ipdf_set_min)
          CALL ReadElem_integer('pdf_max',ipdf_set_max)
          IF(PDF_Hessian2MC.AND..NOT.check_HessianPDF(ipdf_set_min))THEN
             PDF_Hessian2MC=.FALSE.
             WRITE(*,*)"WARNING: the current PDF is not an Hessian PDF"
          ENDIF
          ho_npdf=ipdf_set_max-ipdf_set_min+1
          IF(ho_npdf.LE.0.AND.PDF_Hessian2MC)THEN
             PDF_Hessian2MC=.FALSE.
             WRITE(*,*)"WARNING: convert Hessian PDF to MC PDF is only possible by using more than 1 error PDFs"
          ENDIF
          IF(PDF_Hessian2MC.AND.lnuclearPDF)THEN
             WRITE(*,*)"WARNING: one cannot have both proton PDF and nuclear"//&
                  " PDF rewigting when setting Hessian to MC PDF convertion"
             PDF_Hessian2MC=.FALSE.
          ENDIF
          tot_nrwgt=tot_nrwgt+ho_npdf+1
       ELSEIF(PDF_Hessian2MC.AND..NOT.lnuclearPDF)THEN
          PDF_Hessian2MC=.FALSE.
          WRITE(*,*)"WARNING: convert Hessian PDF to MC PDF is only possible by setting reweight_pdf=True"
       ENDIF
       IF(lnuclearPDF)THEN
          INCLUDE '../../../input/lhapdf/check_nPDF_id'
          dPN=.FALSE.
          CALL ReadElem_logic('reweight_npdf',reweight_npdf)
          IF(reweight_pdf.AND.ho_npdf.GE.2.AND.nPDF_id.LE.500)THEN
             WRITE(*,*)"ERROR: Cannot perform proton PDF reweighting for EPS09"
             STOP
          ENDIF
          IF(reweight_npdf)THEN
             CALL ReadElem_integer('npdf_min',inpdf_set_min)
             CALL ReadElem_integer('npdf_max',inpdf_set_max)
             ho_nnpdf=inpdf_set_max-inpdf_set_min+1
             IF(PDF_Hessian2MC.AND..NOT.check_HessianPDF(nPDF_id))THEN
                PDF_Hessian2MC=.FALSE.
                WRITE(*,*)"WARNING: the current nPDF is not an Hessian PDF"
             ENDIF
             IF(ho_nnpdf.LE.0.AND.PDF_Hessian2MC)THEN
                PDF_Hessian2MC=.FALSE.
                WRITE(*,*)"WARNING: convert Hessian PDF to MC PDF is only possible by using more than 1 error nPDFs"
             ENDIF
             tot_nrwgt=tot_nrwgt+ho_nnpdf+1
          ELSEIF(PDF_Hessian2MC)THEN
             PDF_Hessian2MC=.FALSE.
             WRITE(*,*)"WARNING: convert Hessian nPDF to MC nPDF is only possible by setting reweight_npdf=True"
          ENDIF
          INCLUDE '../../../input/lhapdf/check_npdf_set_min'
       ! ELSE
          ! do nothing
       ENDIF
    ! ELSE
       ! do nothing
    ENDIF
    IF(reweight_pdf.AND.reweight_npdf.AND.PDF_Hessian2MC)THEN
       WRITE(*,*)"ERROR: Cannot reweight proton PDF and nuclear PDF simultanously with Hessian to MC PDF convertion"
       STOP
    ENDIF
    IF(reweight_pdf.OR.reweight_npdf.OR.reweight_ref)THEN
       IF(ALLOCATED(rwgt_values))THEN
          DEALLOCATE(rwgt_values)
       ENDIF
       ALLOCATE(rwgt_values(tot_nrwgt))
    ENDIF
    ! some necesarry initialization
    OPEN(UNIT=12315,FILE="./input/muF.inp")
    READ(12315,*)muF_input
    CLOSE(UNIT=12315)
    OPEN(UNIT=12315,FILE="./input/output.inp")
    READ(12315,*)OUTPUT_FILES
    CLOSE(UNIT=12315)
    OPEN(UNIT=12315,FILE="./input/PDF_uncer.inp")
    READ(12315,*)Hessian_Error,Replica_Error
    CLOSE(UNIT=12315)
    IF(OUTPUT_FILES.EQ.0.OR.OUTPUT_FILES.EQ.2)THEN
       OPEN(UNIT=12315,FILE="./input/wk_path")
       READ(12315,'(A)')WkFilePath
       CLOSE(UNIT=12315)
       ! read wk
       INQUIRE(FILE=TRIM(WkFilePath),EXIST=lexist)
       IF(.NOT.lexist)THEN
          WRITE(*,*)"ERROR:cannot find the wk file !"
          STOP
       ENDIF
       OPEN(UNIT=12315,FILE=TRIM(WkFilePath))
       READ(12315,'(A)')RkiFilePath
       IF(RkiFilePath(1:2).EQ."#?")THEN
          WRITE(*,*)"ERROR:cannot find the PDF_Rki.dat file from wk.dat !"
          STOP
       ENDIF
       RkiFilePath=RkiFilePath(12:500)
       READ(12315,'(A)')
       READ(12315,'(A)')wk_header
       IF(OUTPUT_FILES.EQ.0)THEN
          N_WK=CHAR_COUNT(wk_header,200000,'&')+1
          IF(N_WK.NE.N_MCPDFs)THEN
             WRITE(*,*)"ERROR: the number of weight wk does not match with N_MCPDFs"
             STOP
          ENDIF
          IF(ALLOCATED(LIST_WK))THEN
             DEALLOCATE(LIST_WK)
          ENDIF
          ALLOCATE(LIST_WK(N_WK))
          DO WHILE(.TRUE.)
             READ(12315,'(A)')wk_header
             IF(wk_header(1:4).EQ.'<wk>')THEN
                READ(12315,*)(LIST_WK(i),i=1,N_WK)
                EXIT
             ENDIF
          ENDDO
       ELSE
          N_WK=N_MCPDFs
       ENDIF
       CLOSE(UNIT=12315)
       ! read Rki
       INQUIRE(FILE=TRIM(RkiFilePath),EXIST=lexist)
       IF(.NOT.lexist)THEN
          WRITE(*,*)"ERROR:cannot find the PDF_Rki.dat file !"
          STOP
       ENDIF
       OPEN(UNIT=12315,FILE=TRIM(RkiFilePath))
       READ(12315,*)i,i1
       IF(i.NE.N_MCPDFs)THEN
          WRITE(*,*)"ERROR: the PDF_Rki.dat does not match to this setup #1"
          STOP
       ENDIF
       IF(reweight_pdf)THEN
          IF(i1.NE.ho_npdf/2)THEN
             WRITE(*,*)"ERROR: the PDF_Rki.dat does not match to this setup #2"
             STOP
          ELSE
             N_Rki_Dim2=i1
          ENDIF
       ELSEIF(reweight_npdf)THEN
          IF(i1.NE.ho_nnpdf/2)THEN
             WRITE(*,*)"ERROR: the PDF_Rki.dat does not match to this setup #3"
             STOP
          ELSE
             N_Rki_Dim2=i1
          ENDIF
       ELSE
          WRITE(*,*)"ERROR: lhapdf_rwgt cannot run without reweighting PDF or nPDF"
          STOP
       ENDIF
    ENDIF
    IF(OUTPUT_FILES.EQ.0.OR.OUTPUT_FILES.EQ.2)THEN
       IF(ALLOCATED(PDF_Rki))THEN
          DEALLOCATE(PDF_Rki)
       ENDIF
       ALLOCATE(PDF_Rki(N_MCPDFs,N_Rki_Dim2))
       DO i=1,N_MCPDFs
          READ(12315,*)(PDF_Rki(i,i1),i1=1,N_Rki_Dim2)
       ENDDO
       CLOSE(UNIT=12315)
    ENDIF
    ! max_ixp, min_ixp, nip
    ! the x range would be (10**(min_ixp),10**(max_ixp))
    ! in each 10**(-i) to 10**(-i+1), we divide the inteval into nip equal intervals
    OPEN(UNIT=12315,FILE="./input/x_interval.inp")
    READ(12315,*)max_ixp,min_ixp,nip
    CLOSE(UNIT=12315)
    IF(max_ixp.LE.min_ixp.OR.nip.LE.0)THEN
       WRITE(*,*)"ERROR: max_ixp <= min_ixp or nip <= 0"
       STOP
    ENDIF
    IF(max_ixp.GT.0)THEN
       WRITE(*,*)"ERROR: max_ixp > 0"
       STOP
    ENDIF
    ! generate the x points
    N_xpoint=(max_ixp-min_ixp)*nip+1
    IF(ALLOCATED(LIST_xpoint))THEN
       DEALLOCATE(LIST_xpoint)
    ENDIF
    ALLOCATE(LIST_xpoint(N_xpoint))
    j1=0
    DO i=min_ixp,max_ixp-1
       minx=(10d0)**i
       dxdx=(10d0*minx-minx)/DBLE(nip)
       DO i1=0,nip-1
          xnow=minx+dxdx*i1
          j1=j1+1
          LIST_xpoint(j1)=xnow
       ENDDO
    ENDDO
    LIST_xpoint(N_xpoint)=(10d0)**max_ixp

    pdflabel(0)='gl'
    pdflabel(1)='dq'
    pdflabel(-1)='db'
    pdflabel(2)='uq'
    pdflabel(-2)='ub'
    pdflabel(3)='sq'
    pdflabel(-3)='sb'
    pdflabel(4)='cq'
    pdflabel(-4)='cb'
    pdflabel(5)='bq'
    pdflabel(-5)='bb'

    ! calling PDFs
    CALL lhapdfwrap
    INCLUDE "../../../input/lhapdf/call_initnpdf"

    scale=muF_input ! scale

    IF(OUTPUT_FILES.EQ.0.OR.OUTPUT_FILES.EQ.2)THEN
       IF(ALLOCATED(MC_PDF_wgts))THEN
          DEALLOCATE(MC_PDF_wgts)
       ENDIF
       ALLOCATE(MC_PDF_wgts(-5:5,1:N_xpoint,0:N_MCPDFs))
       d3min_replica_wgts=0
       d3max_replica_wgts=N_MCPDFs
    ENDIF

    IF(lnuclearPDF)THEN

       IF(ALLOCATED(LIST_PDF_wgts))THEN
          DEALLOCATE(LIST_PDF_wgts)
       ENDIF
       ALLOCATE(LIST_PDF_wgts(-5:5,1:N_xpoint,-1:ho_nnpdf))
       d3min_pdf_wgts=-1
       d3max_pdf_wgts=ho_nnpdf

       IF(ALLOCATED(RST_PDF_wgts))THEN
          DEALLOCATE(RST_PDF_wgts)
       ENDIF
       ALLOCATE(RST_PDF_wgts(-5:5,1:N_xpoint,1:10))
       d3min_rst_wgts=1
       d3max_rst_wgts=10

       IF(ALLOCATED(idnpdf))THEN
          DEALLOCATE(idnpdf)
       ENDIF
       ALLOCATE(idnpdf(0:ho_nnpdf))
       IF(nPDF_id.NE.1000)THEN
          idnpdf(0)=nPDF_id
          idnpdf(1)=inpdf_set_min
       ELSE
          idnpdf(0)=iPDFSUP1
          idnpdf(1)=inpdf_set_min-1000+iPDFSUP1
       ENDIF
       PDF_LHAID=idnpdf(0)
       IF(inpdf_set_max-inpdf_set_min+1.NE.ho_nnpdf)THEN
          WRITE(*,*)'Incorrect the number of error npdf',ho_nnpdf,inpdf_set_min,inpdf_set_max
          STOP
       ENDIF
       DO i=2,ho_nnpdf
          idnpdf(i)=idnpdf(1)+i-1
       ENDDO

       ! output nuclear PDF
       ! first to get the proton PDFs as the reference
       INCLUDE "../../../input/lhapdf/call_strf_lhanpdf_proton"
       DO i=1,N_xpoint
          xp1=LIST_xpoint(i)
          CALL pftopdglha(1,xp1,scale,pdflist(-7:7))
          LIST_PDF_wgts(-5:5,i,-1)=pdflist(-5:5)
       ENDDO
       ! then the nuclear PDFs
       DO i=0,ho_nnpdf
          INCLUDE "../../../input/lhapdf/call_initnpdfi"
          INCLUDE "../../../input/lhapdf/call_initnpdfi_lhapdf6"
          DO i1=1,N_xpoint
             xp1=LIST_xpoint(i1)
             INCLUDE "../../../input/lhapdf/call_strf_lhanpdf1"
             LIST_PDF_wgts(-5:5,i1,i)=pdflist(-5:5)
          ENDDO
       ENDDO
    ELSE
       ! output proton PDF
       IF(ALLOCATED(LIST_PDF_wgts))THEN
          DEALLOCATE(LIST_PDF_wgts)
       ENDIF
       ALLOCATE(LIST_PDF_wgts(-5:5,1:N_xpoint,0:ho_npdf))
       d3min_pdf_wgts=0
       d3max_pdf_wgts=ho_npdf

       IF(ALLOCATED(RST_PDF_wgts))THEN
          DEALLOCATE(RST_PDF_wgts)
       ENDIF
       ALLOCATE(RST_PDF_wgts(-5:5,1:N_xpoint,1:9))
       d3min_rst_wgts=1
       d3max_rst_wgts=9

       IF(ALLOCATED(idpdf))THEN
          DEALLOCATE(idpdf)
       ENDIF
       ALLOCATE(idpdf(0:ho_npdf))
       idpdf(0)=iPDFSUP1
       idpdf(1)=ipdf_set_min
       PDF_LHAID=idpdf(0)
       IF(ipdf_set_max-ipdf_set_min+1.NE.ho_npdf)THEN
          WRITE(*,*)'Incorrect the number of error pdf',ho_npdf,ipdf_set_min,ipdf_set_max
          STOP
       ENDIF
       DO i=2,ho_npdf
          idpdf(i)=idpdf(1)+i-1
       ENDDO
       
       DO i=0,ho_npdf
          INCLUDE "../../../input/lhapdf/call_initpdf"
          DO i1=1,N_xpoint
             xp1=LIST_xpoint(i1)
             CALL pftopdglha(ih,xp1,scale,pdflist(-7:7))
             LIST_PDF_wgts(-5:5,i1,i)=pdflist(-5:5)
          ENDDO
       ENDDO
    ENDIF

    IS_HessianPDF=check_HessianPDF(PDF_LHAID)
    
    ! output the original pdf
    DO i=-5,5
       OPEN(UNIT=12315,FILE="./output/pdf_"//pdflabel(i)//".dat")
       DO i1=1,N_xpoint
          WRITE(12315,*)LIST_xpoint(i1),(LIST_PDF_wgts(i,i1,j1),&
               j1=d3min_pdf_wgts,d3max_pdf_wgts)
       ENDDO
       CLOSE(UNIT=12315)
    ENDDO
    
    IF(OUTPUT_FILES.EQ.0.OR.OUTPUT_FILES.EQ.2)THEN
       ! output the replicas
       DO i=-5,5
          OPEN(UNIT=12315,FILE="./output/replica_"//pdflabel(i)//".dat")
          DO i1=1,N_xpoint
             MC_PDF_wgts(i,i1,0)=LIST_PDF_wgts(i,i1,0)
             DO j1=1,N_MCPDFs
                MC_PDF_wgts(i,i1,j1)=LIST_PDF_wgts(i,i1,0)
                DO k1=1,N_Rki_Dim2
                   MC_PDF_wgts(i,i1,j1)=MC_PDF_wgts(i,i1,j1)+(LIST_PDF_wgts(i,i1,2*k1-1)&
                        -LIST_PDF_wgts(i,i1,2*k1))/2d0*PDF_Rki(j1,k1)
                ENDDO
             ENDDO
             WRITE(12315,*)LIST_xpoint(i1),(MC_PDF_wgts(i,i1,j1),&
                  j1=d3min_replica_wgts,d3max_replica_wgts)
          ENDDO
          CLOSE(UNIT=12315)
       ENDDO
    ENDIF

    ! output the central min max results
    ! x    pdf_cen    pdf_min     pdf_max   replica_cen    replica_min    replica_max    wreplica_cen      wreplica_min     wreplica_max    (proton PDF)
    isisishesspdf=IS_HessianPDF
    DO i=-5,5
       OPEN(UNIT=12315,FILE="./output/result_"//pdflabel(i)//".dat")
       WRITE(12315,*)"# scale=",scale
       IF(lnuclearPDF)THEN
          WRITE(12315,*)"# x nPDF_cen  nPDF_min nPDF_max replica_cen "&
               //"replica_min replica_max wreplica_cen wreplica_min wreplica_max proton_PDF"
       ELSE
          WRITE(12315,*)"# x PDF_cen  PDF_min PDF_max replica_cen replica_min "&
               //"replica_max wreplica_cen wreplica_min wreplica_max"
       ENDIF
       DO i1=1,N_xpoint
          RST_PDF_wgts(i,i1,d3min_rst_wgts:d3max_rst_wgts)=0d0
          ! calculate the PDF_cen, PDF_min, PDF_max
          CALL PDF_uncertainty(d3max_pdf_wgts,LIST_PDF_wgts(i,i1,0:d3max_pdf_wgts),&
               RST_PDF_wgts(i,i1,1),RST_PDF_wgts(i,i1,2),&
               RST_PDF_wgts(i,i1,3))
          IF(OUTPUT_FILES.EQ.0.OR.OUTPUT_FILES.EQ.2)THEN
             ! calculate the replica_cen, replica_min, replica_max
             IS_HessianPDF=.FALSE.
             CALL PDF_uncertainty(d3max_replica_wgts,MC_PDF_wgts(i,i1,0:d3max_replica_wgts),&
                  RST_PDF_wgts(i,i1,4),RST_PDF_wgts(i,i1,5),&
                  RST_PDF_wgts(i,i1,6))
             IS_HessianPDF=isisishesspdf
          ENDIF
          IF(OUTPUT_FILES.EQ.0)THEN
             ! calculate the weighted ones, wreplica_cen, wreplica_min, wreplica_max
             CALL wPDF_uncertainty(N_MCPDFs,MC_PDF_wgts(i,i1,1:d3max_replica_wgts),&
                  LIST_WK(1:N_MCPDFs),RST_PDF_wgts(i,i1,7),RST_PDF_wgts(i,i1,8),&
                  RST_PDF_wgts(i,i1,9))
          ENDIF
          ! proton PDF as a reference for nuclear PDF
          IF(lnuclearPDF)THEN
             RST_PDF_wgts(i,i1,10)=LIST_PDF_wgts(i,i1,-1)
          ENDIF
          WRITE(12315,*)LIST_xpoint(i1),(RST_PDF_wgts(i,i1,j1),&
               j1=d3min_rst_wgts,d3max_rst_wgts)
       ENDDO
       CLOSE(UNIT=12315)
    ENDDO

    ! generate gnuplot files
    CALL GNUPLOT_PDF_ERROR(-100D0,100D0)

    CALL GNUPLOT_xPDF(0d0,1D99)

    IF(lnuclearPDF)THEN
       CALL GNUPLOT_RPDF(0d0,1.6d0)
    ENDIF

    IF(IS_HessianPDF.AND.REPLICA_Error.EQ.2.AND.&
         (OUTPUT_FILES.EQ.0.OR.OUTPUT_FILES.EQ.2))THEN
       CALL GNUPLOT_REPLICAoPDF(0.9D0,1.10D0)
    ENDIF

    RETURN
    
  END SUBROUTINE generate_lhapdf_rwgt

  SUBROUTINE wPDF_uncertainty(N_MCPDF,MCPDF_wgts,wwk,PDF_cen_wgt,&
       PDF_min_wgt,PDF_max_wgt)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N_MCPDF
    REAL(KIND(1d0)),DIMENSION(N_MCPDF),INTENT(IN)::MCPDF_wgts,wwk
    REAL(KIND(1d0)),INTENT(OUT)::PDF_cen_wgt,PDF_min_wgt,PDF_max_wgt
    INTEGER::i
    REAL(KIND(1d0))::pdf_stdev
    ! Eq.(3.7) in arXiv:1610.02925
    PDF_cen_wgt=0d0
    DO i=1,N_MCPDF
       PDF_cen_wgt=PDF_cen_wgt+wwk(i)*MCPDF_wgts(i)
    ENDDO
    PDF_cen_wgt=PDF_cen_wgt/DBLE(N_MCPDF)
    pdf_stdev=0d0
    DO i=1,N_MCPDF
       pdf_stdev=pdf_stdev+wwk(i)*(MCPDF_wgts(i)-PDF_cen_wgt)**2
    ENDDO
    pdf_stdev=DSQRT(pdf_stdev/DBLE(N_MCPDF))
    PDF_max_wgt=PDF_cen_wgt+pdf_stdev
    PDF_min_wgt=PDF_cen_wgt-pdf_stdev
    RETURN
  END SUBROUTINE wPDF_uncertainty

  SUBROUTINE PDF_uncertainty(N_PDF,PDF_wgts,PDF_cen_wgt,PDF_min_wgt,PDF_max_wgt)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N_PDF
    REAL(KIND(1d0)),DIMENSION(0:N_PDF),INTENT(IN)::PDF_wgts
    REAL(KIND(1d0)),INTENT(OUT)::PDF_cen_wgt,PDF_min_wgt,PDF_max_wgt
    REAL(KIND(1d0))::pdf_cen,pdf_up,pdf_down,pdf_stdev
    INTEGER::i
    pdf_cen=PDF_wgts(0)
    IF(N_PDF.LE.1)THEN
       PDF_cen_wgt=pdf_cen
       PDF_max_wgt=pdf_cen
       PDF_min_wgt=pdf_cen
       RETURN
    ENDIF
    IF(MOD(N_PDF,2).NE.0.AND.IS_HessianPDF)THEN
       WRITE(*,*)"Error: Number of error eigenstate PDFs is not an even number !"
       STOP
    ENDIF
    IF(IS_HessianPDF)THEN
       ! Hessian PDFs (MSTW, CTEQ)
       IF(Hessian_Error.EQ.2)THEN
          pdf_stdev=0d0
          DO i=1,N_PDF/2
             pdf_stdev=pdf_stdev+(PDF_wgts(2*i-1)-PDF_wgts(2*i))**2
          ENDDO
          pdf_stdev=DSQRT(pdf_stdev)*0.5D0
          PDF_cen_wgt=pdf_cen
          PDF_max_wgt=pdf_cen+pdf_stdev
          PDF_min_wgt=pdf_cen-pdf_stdev
       ELSE
          pdf_up=0d0
          pdf_down=0d0
          DO i=1,N_PDF/2
             pdf_up=pdf_up+MAX(0d0,MAX(PDF_wgts(2*i-1)-pdf_cen,&
                  PDF_wgts(2*i)-pdf_cen))**2
             pdf_down=pdf_down+MAX(0d0,MAX(pdf_cen-PDF_wgts(2*i-1),&
                  pdf_cen-PDF_wgts(2*i)))**2
          ENDDO
          PDF_cen_wgt=pdf_cen
          PDF_max_wgt=pdf_cen+DSQRT(pdf_up)
          PDF_min_wgt=pdf_cen-DSQRT(pdf_down)
       ENDIF
    ELSE
       ! MC PDFs (NNPDF)
       IF(REPLICA_Error.EQ.2)THEN
          ! Eq.(3.2) in arXiv:1610.02925
          PDF_cen_wgt=0d0
          DO i=1,N_PDF
             PDF_cen_wgt=PDF_cen_wgt+PDF_wgts(i)
          ENDDO
          PDF_cen_wgt=PDF_cen_wgt/N_PDF
          pdf_stdev=0d0
          DO i=1,N_PDF
             pdf_stdev=pdf_stdev+(PDF_wgts(i)-PDF_cen_wgt)**2
          ENDDO
          pdf_stdev=DSQRT(pdf_stdev/DBLE(N_PDF))
          PDF_max_wgt=PDF_cen_wgt+pdf_stdev
          PDF_min_wgt=PDF_cen_wgt-pdf_stdev
       ELSE
          pdf_stdev=0d0
          DO i=1,N_PDF
             pdf_stdev=pdf_stdev+(PDF_wgts(i)-pdf_cen)**2
          ENDDO
          pdf_stdev=DSQRT(pdf_stdev/DBLE(N_PDF-1))
          PDF_cen_wgt=pdf_cen
          PDF_max_wgt=pdf_cen+pdf_stdev
          PDF_min_wgt=pdf_cen-pdf_stdev
       ENDIF
    ENDIF

    RETURN
  END SUBROUTINE PDF_uncertainty

  FUNCTION CHAR_COUNT(STR,STR_LEN,CHAR)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::STR_LEN
    CHARACTER(LEN=STR_LEN),INTENT(IN)::STR
    CHARACTER(LEN=1),INTENT(IN)::CHAR
    INTEGER::CHAR_COUNT
    CHARACTER(LEN=STR_LEN)::STR2
    INTEGER::i
    CHAR_COUNT=0
    STR2=TRIM(STR)
    DO WHILE(.TRUE.)
       i=INDEX(STR2,CHAR)
       IF(i.GT.0)THEN
          STR2=STR2(i+1:STR_LEN)
          CHAR_COUNT=CHAR_COUNT+1
       ELSE
          EXIT
       ENDIF
    ENDDO
    RETURN
  END FUNCTION CHAR_COUNT
END MODULE lhapdf_rwgt
