MODULE plot_pp_NOnia_mps
  USE HELAC_Global
  USE pp_NOnia_mps_global
  USE Kinetic_Func 
  IMPLICIT NONE
  INTEGER,PARAMETER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=19*3
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_NOnia_mps
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(max_weight)::weights_info
    INTEGER::i
    nwgt=1
    weights_info(nwgt)="central value"
    ! output plot files
    CALL plot_begin_pp_NOnia_mps(nwgt,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_NOnia_mps

  SUBROUTINE plotout_pp_NOnia_mps
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_NOnia_mps(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_NOnia_mps

  SUBROUTINE outfun_pp_NOnia_mps(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(max_weight)::wgts,wgtden
    nwgt=1
    wgts(1)=www
    ! output plot file 
    CALL plot_fill_pp_NOnia_mps(wgts)
    RETURN
  END SUBROUTINE outfun_pp_NOnia_mps

  SUBROUTINE plot_begin_pp_NOnia_mps(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    CHARACTER(len=4),DIMENSION(3)::cuts=(/'    ','LHCb','CMS '/)
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_NOnia_mps: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    DO i=1,3
       l=(i-1)*19
       CALL histogram_book(l+ 1,'total rate    '//cuts(i),&
            5,0.5d0,5.5d0,.FALSE.)
       CALL histogram_book(l+ 2,'|Delta y| min '//cuts(i),&
            100,0d0,5d0,.TRUE.)
       CALL histogram_book(l+ 3,'|Delta y| max '//cuts(i),&
            100,0d0,5d0,.TRUE.)
       CALL histogram_book(l+ 4,'|Delta phi| min '//cuts(i),&
            100,-0.0001d0,3.1416d0,.TRUE.)
       CALL histogram_book(l+ 5,'|Delta phi| max '//cuts(i),&
            100,-0.0001d0,3.1416d0,.TRUE.)
       CALL histogram_book(l+ 6,'psi psi inv. mass min '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 7,'psi psi inv. mass max '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 8,'psi psi psi inv. mass '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 9,'psi psi psi trans mom '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 10,'|Delta eta| min '//cuts(i),&
            100,0d0,4d0,.TRUE.)
       CALL histogram_book(l+ 11,'|Delta eta| max '//cuts(i),&
            100,0d0,4d0,.TRUE.)
       CALL histogram_book(l+ 12,'random Pt 1 '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 13,'random Pt 2 '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 14,'random Pt 3 '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 15,'leading Pt '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 16,'subleading Pt '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 17,'subsubleading Pt '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 18,'Pt inclusive '//cuts(i),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book3d(l+ 19,'|Delta y| min vs. |Delta phi| min '&
            //cuts(i),100,0d0,5d0,100,-0.0001d0,3.1416d0,.FALSE.)
    ENDDO
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_NOnia_mps

  SUBROUTINE plot_end_pp_NOnia_mps(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_NOnia_mps
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_NOnia_mps
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_NOnia_mps
       psfile="pp_nonia_mps.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_NOnia_mps
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_NOnia_mps
       rootfile="pp_nonia_mps.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_NOnia_mps
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_NOnia_mps
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_NOnia_mps

  SUBROUTINE plot_fill_pp_NOnia_mps(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4)::pboo
    REAL(KIND(1d0)),DIMENSION(4)::ppsi1,ppsi2,ppsi3
    REAL(KIND(1d0)),DIMENSION(4)::ppsi12,ppsi13,ppsi23,ppsi123
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,ypsi1,ypsi2,ypsi3
    REAL(KIND(1d0))::delta_y12,delta_y13,delta_y23,delta_ymin,delta_ymax
    REAL(KIND(1d0))::etapsi1,etapsi2,etapsi3
    REAL(KIND(1d0))::delta_eta12,delta_eta13,delta_eta23,delta_etamin,&
         delta_etamax
    REAL(KIND(1d0))::delta_phi12,delta_phi13,delta_phi23,&
         delta_phimin,delta_phimax
    REAL(KIND(1d0))::m12,m23,m13,m123,mpsipsi_min,mpsipsi_max,pt_psipsipsi
    LOGICAL,DIMENSION(3)::cuts
    REAL(KIND(1d0))::pt_psi1,pt_psi2,pt_psi3,pt1,pt2,pt3
    REAL(KIND(1d0)),DIMENSION(3)::pt_ordered
    !IF(NDecayChains.GT.0)THEN
    !   CALL plot_fill_Decay_pp_NOnia_mps(wgts)
    !   RETURN
    !ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    IF(mps_number.NE.3)THEN
       WRITE(*,*)"ERROR: the plot_pp_NOnia_mps.f90 only works for triple-parton scatterings"
       STOP
    ENDIF
    ppsi1(1:4)=mps_hadron_pmom(7,1:4) ! (px,py,pz,E)
    ppsi2(1:4)=mps_hadron_pmom(9,1:4)
    ppsi3(1:4)=mps_hadron_pmom(11,1:4)
    ppsi12(1:4)=ppsi1(1:4)+ppsi2(1:4)
    ppsi13(1:4)=ppsi1(1:4)+ppsi3(1:4)
    ppsi23(1:4)=ppsi2(1:4)+ppsi3(1:4)
    ppsi123(1:4)=ppsi1(1:4)+ppsi2(1:4)+ppsi3(1:4)
    ypsi1=rapidity(ppsi1)
    ypsi2=rapidity(ppsi2)
    ypsi3=rapidity(ppsi3)
    cuts(1)=.TRUE.
    cuts(2)=ypsi1.GT.2D0.AND.ypsi2.GT.2D0.AND.ypsi3.GT.2D0
    cuts(2)=cuts(2).AND.ypsi1.LT.4.5D0.AND.&
         ypsi2.LT.4.5D0.AND.ypsi3.LT.4.5D0
    cuts(3)=DABS(ypsi1).LT.2.4D0.AND.DABS(ypsi2).LT.2.4D0&
         .AND.DABS(ypsi3).LT.2.4D0
    delta_y12=DABS(ypsi1-ypsi2)
    delta_y13=DABS(ypsi1-ypsi3)
    delta_y23=DABS(ypsi2-ypsi3)
    delta_ymin=MIN(delta_y12,delta_y13,delta_y23)
    delta_ymax=MAX(delta_y12,delta_y13,delta_y23)
    delta_phi12=DeltaPh4(ppsi1,ppsi2)
    delta_phi13=DeltaPh4(ppsi1,ppsi3)
    delta_phi23=DeltaPh4(ppsi2,ppsi3)
    delta_phimin=MIN(delta_phi12,delta_phi13,delta_phi23)
    delta_phimax=MAX(delta_phi12,delta_phi13,delta_phi23)
    m12=DSQRT(ppsi12(4)**2-ppsi12(1)**2-ppsi12(2)**2-ppsi12(3)**2)
    m13=DSQRT(ppsi13(4)**2-ppsi13(1)**2-ppsi13(2)**2-ppsi13(3)**2)
    m23=DSQRT(ppsi23(4)**2-ppsi23(1)**2-ppsi23(2)**2-ppsi23(3)**2)
    m123=DSQRT(ppsi123(4)**2-ppsi123(1)**2-ppsi123(2)**2-ppsi123(3)**2)
    mpsipsi_min=MIN(m12,m13,m23)
    mpsipsi_max=MAX(m12,m13,m23)
    pt_psipsipsi=DSQRT(ppsi123(1)**2+ppsi123(2)**2)
    etapsi1=prapidity(ppsi1)
    etapsi2=prapidity(ppsi2)
    etapsi3=prapidity(ppsi3)
    delta_eta12=DABS(etapsi1-etapsi2)
    delta_eta13=DABS(etapsi1-etapsi3)
    delta_eta23=DABS(etapsi2-etapsi3)
    delta_etamin=MIN(delta_eta12,delta_eta13,delta_eta23)
    delta_etamax=MAX(delta_eta12,delta_eta13,delta_eta23)
    pt_psi1=DSQRT(ppsi1(1)**2+ppsi1(2)**2)
    pt_psi2=DSQRT(ppsi2(1)**2+ppsi2(2)**2)
    pt_psi3=DSQRT(ppsi3(1)**2+ppsi3(2)**2)
    pt1=MAX(pt_psi1,pt_psi2,pt_psi3)
    pt3=MIN(pt_psi1,pt_psi2,pt_psi3)
    pt_ordered(1)=pt_psi1
    pt_ordered(2)=pt_psi2
    pt_ordered(3)=pt_psi3
    CALL GetPTOrdered(3,pt_ordered) ! get the pt-ordered results
    pt2=pt_ordered(2)
    IF(pt_ordered(1).NE.pt1)THEN
       WRITE(*,*)"ERROR: do not get the correct pt ordered results #1"
       STOP
    ENDIF
    IF(pt_ordered(3).NE.pt3)THEN
       WRITE(*,*)"ERROR: do not get the correct pt ordered results #2"
       STOP
    ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=1,3
       IF(.NOT.cuts(i))CYCLE
       l=(i-1)*19
       CALL HISTOGRAM_FILL(l+1,var,wgts)
       CALL HISTOGRAM_FILL(l+2,delta_ymin,wgts)
       CALL HISTOGRAM_FILL(l+3,delta_ymax,wgts)
       CALL HISTOGRAM_FILL(l+4,delta_phimin,wgts)
       CALL HISTOGRAM_FILL(l+5,delta_phimax,wgts)
       CALL HISTOGRAM_FILL(l+6,mpsipsi_min,wgts)
       CALL HISTOGRAM_FILL(l+7,mpsipsi_max,wgts)
       CALL HISTOGRAM_FILL(l+8,m123,wgts)
       CALL HISTOGRAM_FILL(l+9,pt_psipsipsi,wgts)
       CALL HISTOGRAM_FILL(l+10,delta_etamin,wgts)
       CALL HISTOGRAM_FILL(l+11,delta_etamax,wgts)
       CALL HISTOGRAM_FILL(l+12,pt_psi1,wgts)
       CALL HISTOGRAM_FILL(l+13,pt_psi2,wgts)
       CALL HISTOGRAM_FILL(l+14,pt_psi3,wgts)
       CALL HISTOGRAM_FILL(l+15,pt1,wgts)
       CALL HISTOGRAM_FILL(l+16,pt2,wgts)
       CALL HISTOGRAM_FILL(l+17,pt3,wgts)
       CALL HISTOGRAM_FILL(l+18,pt_psi1,wgts)
       CALL HISTOGRAM_FILL(l+18,pt_psi2,wgts)
       CALL HISTOGRAM_FILL(l+18,pt_psi3,wgts)
       CALL HISTOGRAM_FILL3D(l+19,delta_ymin,delta_phimin,wgts)
    ENDDO
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_NOnia_mps

  SUBROUTINE GetPTOrdered(npt,pt_ordered)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::npt
    REAL(KIND(1d0)),DIMENSION(npt),INTENT(INOUT)::pt_ordered
    INTEGER::i,j
    REAL(KIND(1d0)),DIMENSION(npt)::pt_orig
    LOGICAL,DIMENSION(npt)::used
    REAL(KIND(1d0))::ptmax
    INTEGER::indexmax
    pt_orig(1:npt)=pt_ordered(1:npt)
    used(1:npt)=.FALSE.
    DO i=1,npt
       ptmax=-1d0
       indexmax=0
       DO j=1,npt
          IF(used(j))CYCLE
          IF(pt_orig(j).GT.ptmax)THEN
             ptmax=pt_orig(j)
             indexmax=j
          ENDIF
       ENDDO
       pt_ordered(i)=ptmax
       used(indexmax)=.TRUE.
    ENDDO
    RETURN
  END SUBROUTINE GetPTOrdered

  FUNCTION DeltaPh4(pmom1,pmom2)
    IMPLICIT NONE
    REAL(KIND(1d0))::DeltaPh4
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pmom1,pmom2 ! px,py,pz,E
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    DeltaPh4=ph4(pmom1(1),pmom1(2),pmom1(3)) ! ph4 is defined in Kinematic_Func.f90
    DeltaPh4=DeltaPh4-ph4(pmom2(1),pmom2(2),pmom2(3))
    IF(DeltaPh4.GT.pi)DeltaPh4=2d0*pi-DeltaPh4
    RETURN
  END FUNCTION DeltaPh4

  SUBROUTINE HwU_write_file_pp_NOnia_mps
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0))::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//'pp_nonia_mps.HwU',&
         status='unknown')
    xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_NOnia_mps

  SUBROUTINE open_topdrawer_file_pp_NOnia_mps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_nonia_mps.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_NOnia_mps

  SUBROUTINE close_topdrawer_file_pp_NOnia_mps
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_NOnia_mps

  SUBROUTINE open_gnuplot_file_pp_NOnia_mps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_nonia_mps.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_NOnia_mps

  SUBROUTINE close_gnuplot_file_pp_NOnia_mps
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_NOnia_mps

  SUBROUTINE open_root_file_pp_NOnia_mps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_nonia_mps.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_NOnia_mps

  SUBROUTINE close_root_file_pp_NOnia_mps
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_NOnia_mps

  SUBROUTINE plot_fill_Decay_pp_NOnia_mps(wgts)
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "stdhep_pp_nonia_mps.inc"
    REAL(KIND(1d0))::var,Q,www,dy_psipsi,dphi_psipsi,M_psipsi,Pt_psipsi,Pt1Pt2_psipsi,deta_psipsi
    REAL(KIND(1d0))::q12_psipsi,cosalpha,ptpsi1,ptpsi2,ptpsimax,ptpsimin,ptpsidiff
    INTEGER::i,m,l,kk
    REAL(KIND(1d0)),DIMENSION(4)::PBOO
    REAL(KIND(1d0)),DIMENSION(2,4)::PJpsi,PJpsiCMS
    REAL(KIND(1d0)),DIMENSION(4,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    REAL(KIND(1d0)),DIMENSION(3)::Pplain1,Pplain2
    REAL(KIND(1d0)),DIMENSION(3)::Plepvec1,Plepvec2
    REAL(KIND(1d0)),DIMENSION(2,3)::PJpsi1l,PJpsi2l
    REAL(KIND(1d0))::PP1,PP2
    REAL(KIND(1d0))::costh1,costh2
    INTEGER,DIMENSION(4)::lep2jpsi
    INTEGER,DIMENSION(2)::newmoth
    INTEGER::i_jpsi,i_lep,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton 
    i_jpsi=0
    i_lep=0
    lep2jpsi(1:4)=0
    newmoth(1:2)=0
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
       IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
       i_lep=i_lep+1
       Plep(i_lep,1:4)=PHEP(1:4,i)
       m=JMOHEP(1,i)
       definemoth=0
       DO kk=1,i_jpsi
          IF(newmoth(kk).EQ.m)THEN
             definemoth=kk
             EXIT
          END IF
       ENDDO
       IF(definemoth.NE.0)THEN
          ! the mother has been defined
          lep2jpsi(i_lep)=definemoth
       ELSE
          ! create the new mother
          i_jpsi=i_jpsi+1
          PJpsi(i_jpsi,1:4)=PHEP(1:4,m)
          newmoth(i_jpsi)=m
          lep2jpsi(i_lep)=i_jpsi
          IF(i_lep.EQ.4)EXIT
       ENDIF
    ENDDO
    IF(literature_cutoffs.EQ.14080000)THEN
       ! LHCb Kinematics Region
       ptpsi1=transverse(PJpsi(1,1:4))
       IF(ptpsi1.GT.10d0)THEN
          RETURN
       ENDIF
       ptpsi2=transverse(PJpsi(2,1:4))
       IF(ptpsi2.GT.10d0)THEN
          RETURN
       ENDIF
    ENDIF
    dy_psipsi=DABS(rapidity(PJpsi(1,1:4))-rapidity(PJpsi(2,1:4)))
    deta_psipsi=DABS(prapidity(PJpsi(1,1:4))-prapidity(PJpsi(2,1:4)))
    dphi_psipsi=ph4(PJpsi(1,1),PJpsi(1,2),PJpsi(1,3))
    dphi_psipsi=dphi_psipsi-ph4(PJpsi(2,1),PJpsi(2,2),PJpsi(2,3))
    dphi_psipsi=DABS(dphi_psipsi)
    IF(dphi_psipsi.GT.pi)dphi_psipsi=2*pi-dphi_psipsi
    M_psipsi=scalar_product(PJpsi(1,1:4)+PJpsi(2,1:4),PJpsi(1,1:4)+PJpsi(2,1:4))
    M_psipsi=DSQRT(M_psipsi)
    Q=scalar_product(PJpsi(1,1:4),PJpsi(1,1:4))
    Q=DSQRT(Q)
    ptpsi1=transverse(PJpsi(1,1:4))
    ptpsi2=transverse(PJpsi(2,1:4))
    ptpsimax=MAX(ptpsi1,ptpsi2)
    ptpsimin=MIN(ptpsi1,ptpsi2)
    ptpsidiff=ptpsimax-ptpsimin
    Pt_psipsi=transverse(PJpsi(1,1:4)+PJpsi(2,1:4))
    Pt1Pt2_psipsi=transverse(PJpsi(1,1:4)-PJpsi(2,1:4))
    q12_psipsi=M_psipsi**2-4*Q**2
    q12_psipsi=DSQRT(DABS(q12_psipsi))
    PBOO(1:3)=-PJpsi(1,1:3)-PJpsi(2,1:3)
    PBOO(4)=PJpsi(1,4)+PJpsi(2,4)
    DO i=1,4
       CALL BOOSTL(M_psipsi,PBOO,Plep(i,1:4))
    ENDDO
    DO i=1,2
       PJpsiCMS(i,1:4)=PJpsi(i,1:4)
       CALL BOOSTL(M_psipsi,PBOO,PJpsiCMS(i,1:4))
    ENDDO
    kk1=0
    kk2=0
    DO i=1,4
       IF(lep2jpsi(i).EQ.1)THEN
          kk1=kk1+1
          PJpsi1l(kk1,1:3)=Plep(i,1:3)
          IF(kk1.EQ.1)THEN
             Plep1(1:4)=Plep(i,1:4)
          ENDIF
       ELSEIF(lep2jpsi(i).EQ.2)THEN
          kk2=kk2+1
          PJpsi2l(kk2,1:3)=Plep(i,1:3)
          IF(kk2.EQ.1)THEN
             Plep2(1:4)=Plep(i,1:4)
          ENDIF
       ENDIF
    ENDDO
    Pplain1(1:3)=vec_crossprod(PJpsi1l(1,1:3),PJpsi1l(2,1:3))
    Pplain2(1:3)=vec_crossprod(PJpsi2l(1,1:3),PJpsi2l(2,1:3))
    PP1=Pplain1(1)**2+Pplain1(2)**2+Pplain1(3)**2
    PP1=DSQRT(PP1)
    PP2=Pplain2(1)**2+Pplain2(2)**2+Pplain2(3)**2
    PP2=DSQRT(PP2)
    IF(PP1*PP2.EQ.0d0)THEN
       cosalpha=0d0
    ELSE
       cosalpha=Pplain1(1)*Pplain2(1)+Pplain1(2)*Pplain2(2) &
            +Pplain1(3)*Pplain2(3)
       cosalpha=DABS(cosalpha/(PP1*PP2))
    ENDIF
    ! costh1
    Q=scalar_product(PJpsiCMS(1,1:4),PJpsiCMS(1,1:4))
    Q=DSQRT(Q)
    PBOO(1:3)=-PJpsiCMS(1,1:3)
    PBOO(4)=PJpsiCMS(1,4)
    CALL BOOSTL(Q,PBOO,Plep1)
    costh1=cosij(Plep1(1:3),PJpsiCMS(1,1:3))
    ! costh2
    Q=scalar_product(PJpsiCMS(2,1:4),PJpsiCMS(2,1:4))
    Q=DSQRT(Q)
    PBOO(1:3)=-PJpsiCMS(2,1:3)
    PBOO(4)=PJpsiCMS(2,4)
    CALL BOOSTL(Q,PBOO,Plep2)
    costh2=cosij(Plep2(1:3),PJpsiCMS(2,1:3))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,dy_psipsi,wgts)
    CALL HISTOGRAM_fill(l+3,dphi_psipsi,wgts)
    CALL HISTOGRAM_fill(l+4,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+5,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+6,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill(l+7,q12_psipsi,wgts)
    CALL HISTOGRAM_fill(l+8,q12_psipsi,wgts)
    CALL HISTOGRAM_fill(l+9,cosalpha,wgts)
    CALL HISTOGRAM_fill(l+10,deta_psipsi,wgts)
    CALL HISTOGRAM_fill(l+11,ptpsi1,wgts)
    CALL HISTOGRAM_fill(l+12,ptpsi2,wgts)
    CALL HISTOGRAM_fill(l+13,ptpsimax,wgts)
    CALL HISTOGRAM_fill(l+14,ptpsimin,wgts)
    CALL HISTOGRAM_fill(l+15,ptpsidiff,wgts)
    CALL HISTOGRAM_fill(l+16,Pt1Pt2_psipsi,wgts)
    CALL HISTOGRAM_fill3d(l+17,costh1,costh2,wgts)
    CALL HISTOGRAM_fill3d(l+18,dy_psipsi,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill3d(l+19,dy_psipsi,dphi_psipsi,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay_pp_NOnia_mps
END MODULE plot_pp_NOnia_mps
