MODULE plot_pp_oniumquark_dps
  USE HELAC_Global
  USE pp_oniumquark_dps_global
  USE Kinetic_Func 
  IMPLICIT NONE
  INTEGER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=28
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_oniumquark_dps
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(:),ALLOCATABLE::weights_info
    INTEGER::i
    max_weight=1+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
    IF(ALLOCATED(weights_info))THEN
       DEALLOCATE(weights_info)
    ENDIF
    ALLOCATE(weights_info(max_weight))
    nwgt=1
    weights_info(nwgt)="central value  "
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'PDF=',ipdf_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_npdf
       IF(ho_npdf2.GT.0)THEN
          DO i=1,ho_npdf2
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'PDF2=',ipdf2_set_min-1+i,'  '
          ENDDO
          nwgt=nwgt+ho_npdf2
       ENDIF
    ENDIF
    IF(reweight_npdf)THEN
       IF(nPDF_id.EQ.1000)THEN
          DO i=1,ho_nnpdf
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'NPDF=',inpdf_set_min+i-1-1000+iPDFSUP1,'  '
          ENDDO
       ELSE
          DO i=1,ho_nnpdf
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'NPDF=',inpdf_set_min-1+i,'  '
          ENDDO
       ENDIF
       nwgt=nwgt+ho_nnpdf
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'REF=',iref_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_nref
    ENDIF
    ! output plot files
    CALL plot_begin_pp_oniumquark_dps(nwgt,max_weight,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_oniumquark_dps

  SUBROUTINE plotout_pp_oniumquark_dps
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_oniumquark_dps(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_oniumquark_dps

  SUBROUTINE outfun_pp_oniumquark_dps(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::wgts
    INTEGER::init=0
    SAVE init,wgts
    IF(www.EQ.0d0)RETURN
    IF(init.EQ.0)THEN
       max_weight=1+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
       IF(ALLOCATED(wgts))THEN
          DEALLOCATE(wgts)
       ENDIF
       ALLOCATE(wgts(max_weight))
       init=1
    ENDIF
    nwgt=1
    wgts(1)=www
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf+ho_npdf2
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecpdf(i)
       ENDDO
    ENDIF
    IF(reweight_npdf)THEN
       DO i=1,ho_nnpdf
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecnpdf(i)
       ENDDO
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecref(i)
       ENDDO
    ENDIF
    ! output plot file
    CALL plot_fill_pp_oniumquark_dps(wgts)
    RETURN
  END SUBROUTINE outfun_pp_oniumquark_dps

  SUBROUTINE plot_begin_pp_oniumquark_dps(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_oniumquark_dps: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    l=0
    CALL histogram_book(l+ 1,'total rate, -5.0<yCM<-2.5 ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2,'total rate, -4.7<yCM<-2.7 ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 3,'total rate, 1.5<yCM<4.0 ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 4,'total rate, 1.7<yCM<3.7 ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 5,'total rate, -3.7<yCM<-2.7 ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 6,'total rate, 2.7<yCM<3.7 ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 7,'J/psi+C pT, -4.7<yCM<-2.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 8,'J/psi+C pT, 1.7<yCM<3.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 9,'J/psi+C pT, -3.7<yCM<-2.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+10,'J/psi+C pT, 2.7<yCM<3.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+11,'|Delta phi(J/psi,C)|, -4.7<yCM<-2.7 ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+12,'|Delta phi(J/psi,C)|, 1.7<yCM<3.7 ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+13,'|Delta phi(J/psi,C)|, -3.7<yCM<-2.7 ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+14,'|Delta phi(J/psi,C)|, 2.7<yCM<3.7 ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+15,'Delta y(J/psi,C), -4.7<yCM<-2.7 ',&
         100,-2.5d0,2.5d0,.TRUE.)
    CALL histogram_book(l+16,'Delta y(J/psi,C), 1.7<yCM<3.7 ',&
         100,-2.5d0,2.5d0,.TRUE.)
    CALL histogram_book(l+17,'Delta y(J/psi,C), -3.7<yCM<-2.7 ',&
         100,-2.5d0,2.5d0,.TRUE.)
    CALL histogram_book(l+18,'Delta y(J/psi,C), 2.7<yCM<3.7 ',&
         100,-2.5d0,2.5d0,.TRUE.)
    CALL histogram_book(l+19,'m(J/psi,C), -4.7<yCM<-2.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+20,'m(J/psi,C), 1.7<yCM<3.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+21,'m(J/psi,C), -3.7<yCM<-2.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+22,'m(J/psi,C), 2.7<yCM<3.7 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+23,'y(J/psi,C), -4.7<yCM<-2.7  ',&
         2,-4.7d0,-2.7d0,.TRUE.)
    CALL histogram_book(l+24,'y(J/psi,C), 1.7<yCM<3.7  ',&
         2,1.7d0,3.7d0,.TRUE.)
    CALL histogram_book(l+25,'y(J/psi+C), -4.7<yCM<-2.7  ',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+26,'y(J/psi+C), 1.7<yCM<3.7  ',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+27,'y(J/psi+C), -3.7<yCM<-2.7  ',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+28,'y(J/psi+C), 2.7<yCM<3.7  ',&
         100,-5d0,5d0,.TRUE.)
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_oniumquark_dps

  SUBROUTINE plot_end_pp_oniumquark_dps(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_oniumquark_dps
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_oniumquark_dps
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_oniumquark_dps
       psfile="pp_oniumquark_dps.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_oniumquark_dps
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_oniumquark_dps
       rootfile="pp_oniumquark_dps.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_oniumquark_dps
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_oniumquark_dps
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_oniumquark_dps

  SUBROUTINE plot_fill_pp_oniumquark_dps(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4)::p_jpsi,p_C,p_psiC
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l
    REAL(KIND(1d0)),PARAMETER::pipi=3.1415926535897932384626433832795d0
    REAL(KIND(1d0))::www,pt_jpsi,pt_C,y_psi,y_C,pt_psiC,y_psiC
    REAL(KIND(1d0))::M_psiC,dy_psiC,dphi_psiC
    IF(NDecayChains.GT.0)THEN
       CALL plot_fill_Decay_pp_oniumquark_dps(wgts)
       RETURN
    ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    p_jpsi(1:4)=dps_hadron_pmom(5,1:4)
    y_psi=rapidity(p_jpsi(1:4))
    p_C(1:4)=dps_hadron_pmom(7,1:4)
    y_C=rapidity(p_C(1:4))
    p_psiC(1:4)=p_jpsi(1:4)+p_C(1:4)
    y_psiC=rapidity(p_psiC(1:4))
    pt_jpsi=DSQRT(p_jpsi(1)**2+p_jpsi(2)**2)
    pt_C=DSQRT(p_C(1)**2+p_C(2)**2)
    pt_psiC=DSQRT(p_psiC(1)**2+p_psiC(2)**2)
    M_psiC=DSQRT(p_psiC(4)**2-p_psiC(1)**2-p_psiC(2)**2-p_psiC(3)**2)
    dy_psiC=y_psi-y_C
    dphi_psiC=ph4(p_jpsi(1),p_jpsi(2),p_jpsi(3))
    dphi_psiC=dphi_psiC-ph4(p_C(1),p_C(2),p_C(3))
    dphi_psiC=DABS(dphi_psiC)
    IF(dphi_psiC.GT.pipi)dphi_psiC=2D0*pipi-dphi_psiC
    ! LHCb has a upper pt cuts on two mesons (from Yanxi Zhang)
    IF(pt_C.GT.12d0.OR.pt_jpsi.GT.12d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    IF(y_C.GE.-5d0.AND.y_C.LE.-2.5d0.AND.&
         y_psi.GE.-5d0.AND.y_psi.LE.-2.5d0)THEN
       CALL HISTOGRAM_FILL(l+1,var,wgts)
    ENDIF
    IF(y_C.GE.1.5d0.AND.y_C.LE.4.0d0.AND.&
         y_psi.GE.1.5d0.AND.y_psi.LE.4.0d0)THEN
       CALL HISTOGRAM_FILL(l+3,var,wgts)
    ENDIF
    IF(y_C.GE.-4.7d0.AND.y_C.LE.-2.7d0.AND.&
         y_psi.GE.-4.7d0.AND.y_psi.LE.-2.7d0)THEN
       CALL HISTOGRAM_FILL(l+2,var,wgts)
       CALL HISTOGRAM_FILL(l+7,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+11,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+15,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+19,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+25,y_psiC,wgts)
    ENDIF
    IF(y_C.GE.1.7d0.AND.y_C.LE.3.7d0.AND.&
         y_psi.GE.1.7d0.AND.y_psi.LE.3.7d0)THEN
       CALL HISTOGRAM_FILL(l+4,var,wgts)
       CALL HISTOGRAM_FILL(l+8,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+12,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+16,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+20,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+26,y_psiC,wgts)
    ENDIF
    IF(y_C.GE.-4.7d0.AND.y_C.LE.-3.7d0.AND.&
         y_psi.GE.-4.7d0.AND.y_psi.LE.-3.7d0)THEN
       CALL HISTOGRAM_FILL(l+23,y_C,wgts)
    ENDIF
    IF(y_C.GE.-3.7d0.AND.y_C.LE.-2.7d0.AND.&
         y_psi.GE.-3.7d0.AND.y_psi.LE.-2.7d0)THEN
       CALL HISTOGRAM_FILL(l+5,var,wgts)
       CALL HISTOGRAM_FILL(l+9,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+13,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+17,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+21,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+23,y_C,wgts)
       CALL HISTOGRAM_FILL(l+27,y_psiC,wgts)
    ENDIF
    IF(y_C.GE.1.7d0.AND.y_C.LE.2.7d0.AND.&
         y_psi.GE.1.7d0.AND.y_psi.LE.2.7d0)THEN
       CALL HISTOGRAM_FILL(l+24,y_C,wgts)
    ENDIF
    IF(y_C.GE.2.7d0.AND.y_C.LE.3.7d0.AND.&
         y_psi.GE.2.7d0.AND.y_psi.LE.3.7d0)THEN
       CALL HISTOGRAM_FILL(l+6,var,wgts)
       CALL HISTOGRAM_FILL(l+10,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+14,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+18,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+22,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+24,y_C,wgts)
       CALL HISTOGRAM_FILL(l+28,y_psiC,wgts)
    ENDIF
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_oniumquark_dps

  SUBROUTINE HwU_write_file_pp_oniumquark_dps
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0))::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//'pp_oniumquark_dps.HwU',&
         status='unknown')
    xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_oniumquark_dps

  SUBROUTINE open_topdrawer_file_pp_oniumquark_dps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_oniumquark_dps.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_oniumquark_dps

  SUBROUTINE close_topdrawer_file_pp_oniumquark_dps
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_oniumquark_dps

  SUBROUTINE open_gnuplot_file_pp_oniumquark_dps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_oniumquark_dps.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_oniumquark_dps

  SUBROUTINE close_gnuplot_file_pp_oniumquark_dps
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_oniumquark_dps

  SUBROUTINE open_root_file_pp_oniumquark_dps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_oniumquark_dps.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_oniumquark_dps

  SUBROUTINE close_root_file_pp_oniumquark_dps
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_oniumquark_dps

  SUBROUTINE plot_fill_Decay_pp_oniumquark_dps(wgts)
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "stdhep_pp_oniumquark_dps.inc"
    REAL(KIND(1d0))::var,Q,www
    REAL(KIND(1d0))::pt_psi,pt_C,y_psi,y_C,M_psiC,dy_psiC,dphi_psiC
    REAL(KIND(1d0))::y_psiC,pt_psiC
    INTEGER::i,m,l,kk
    REAL(KIND(1d0)),DIMENSION(4)::PJpsi,PJpsiCMS,PCharm,P_psiC
    REAL(KIND(1d0)),DIMENSION(4,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    REAL(KIND(1d0))::PP1,PP2
    REAL(KIND(1d0))::costh1,costh2
    INTEGER,DIMENSION(4)::lep2jpsi
    INTEGER,DIMENSION(2)::newmoth
    INTEGER::i_jpsi,i_lep,i_C,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton 
    i_jpsi=0
    i_C=0
    i_lep=0
    lep2jpsi(1:4)=0
    newmoth(1:2)=0
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(IDHEP(i).EQ.411.OR.IDHEP(i).EQ.413.OR.IDHEP(i).EQ.421&
            .OR.IDHEP(i).EQ.431.OR.IDHEP(i).EQ.4122.OR.IDHEP(i).EQ.4)THEN
          ! charm quark, charm meson or charm baryon
          i_C=i_C+1
          PCharm(1:4)=PHEP(1:4,i)
       ELSE
          ! looking for the leptons from J/psi decay
          IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
          IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
          i_lep=i_lep+1
          Plep(i_lep,1:4)=PHEP(1:4,i)
          m=JMOHEP(1,i)
          definemoth=0
          DO kk=1,i_jpsi
             IF(newmoth(kk).EQ.m)THEN
                definemoth=kk
                EXIT
             END IF
          ENDDO
          IF(definemoth.NE.0)THEN
             ! the mother has been defined
             lep2jpsi(i_lep)=definemoth
          ELSE
             ! create the new mother
             i_jpsi=i_jpsi+1
             PJpsi(1:4)=PHEP(1:4,m)
             newmoth(i_jpsi)=m
             lep2jpsi(i_lep)=i_jpsi
          ENDIF
       ENDIF
    ENDDO
    IF(i_jpsi.NE.1)THEN
       WRITE(*,*)"ERROR: the number of J/psi =/= 1. N_Jpsi=",i_jpsi
       STOP
    ENDIF
    IF(i_C.NE.1)THEN
       WRITE(*,*)"ERROR: the number of charm quark/meson/baryon =/= 1. N_C=",i_C
       STOP
    ENDIF
    pt_psi=transverse(PJpsi(1:4))
    pt_C=transverse(PCharm(1:4))
    y_psi=rapidity(PJpsi(1:4))
    y_C=rapidity(PCharm(1:4))
    P_psiC(1:4)=PJpsi(1:4)+PCharm(1:4)
    pt_psiC=transverse(P_psiC(1:4))
    y_psiC=rapidity(P_psiC(1:4))
    M_psiC=DSQRT(p_psiC(4)**2-p_psiC(1)**2-p_psiC(2)**2-p_psiC(3)**2)
    dy_psiC=y_psi-y_C
    dphi_psiC=ph4(PJpsi(1),PJpsi(2),PJpsi(3))
    dphi_psiC=dphi_psiC-ph4(PCharm(1),PCharm(2),PCharm(3))
    dphi_psiC=DABS(dphi_psiC)
    IF(dphi_psiC.GT.pi)dphi_psiC=2D0*pi-dphi_psiC
    ! LHCb has a upper pt cuts on two mesons (from Yanxi Zhang)
    IF(pt_psi.GT.12d0.OR.pt_C.GT.12d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    IF(y_C.GE.-5d0.AND.y_C.LE.-2.5d0.AND.&
         y_psi.GE.-5d0.AND.y_psi.LE.-2.5d0)THEN
       CALL HISTOGRAM_FILL(l+1,var,wgts)
    ENDIF
    IF(y_C.GE.1.5d0.AND.y_C.LE.4d0.AND.&
         y_psi.GE.1.5d0.AND.y_psi.LE.4d0)THEN
       CALL HISTOGRAM_FILL(l+3,var,wgts)
    ENDIF
    IF(y_C.GE.-4.7d0.AND.y_C.LE.-2.7d0.AND.&
         y_psi.GE.-4.7d0.AND.y_psi.LE.-2.7d0)THEN
       CALL HISTOGRAM_FILL(l+2,var,wgts)
       CALL HISTOGRAM_FILL(l+7,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+11,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+15,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+19,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+25,y_psiC,wgts)
    ENDIF
    IF(y_C.GE.1.7d0.AND.y_C.LE.3.7d0.AND.&
         y_psi.GE.1.7d0.AND.y_psi.LE.3.7d0)THEN
       CALL HISTOGRAM_FILL(l+4,var,wgts)
       CALL HISTOGRAM_FILL(l+8,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+12,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+16,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+20,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+26,y_psiC,wgts)
    ENDIF
    IF(y_C.GE.-4.7d0.AND.y_C.LE.-3.7d0.AND.&
         y_psi.GE.-4.7d0.AND.y_psi.LE.-3.7d0)THEN
       CALL HISTOGRAM_FILL(l+23,y_C,wgts)
    ENDIF
    IF(y_C.GE.-3.7d0.AND.y_C.LE.-2.7d0.AND.&
         y_psi.GE.-3.7d0.AND.y_psi.LE.-2.7d0)THEN
       CALL HISTOGRAM_FILL(l+5,var,wgts)
       CALL HISTOGRAM_FILL(l+9,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+13,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+17,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+21,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+23,y_C,wgts)
       CALL HISTOGRAM_FILL(l+27,y_psiC,wgts)
    ENDIF
    IF(y_C.GE.1.7d0.AND.y_C.LE.2.7d0.AND.&
         y_psi.GE.1.7d0.AND.y_psi.LE.2.7d0)THEN
       CALL HISTOGRAM_FILL(l+24,y_C,wgts)
    ENDIF
    IF(y_C.GE.2.7d0.AND.y_C.LE.3.7d0.AND.&
         y_psi.GE.2.7d0.AND.y_psi.LE.3.7d0)THEN
       CALL HISTOGRAM_FILL(l+6,var,wgts)
       CALL HISTOGRAM_FILL(l+10,pt_psiC,wgts)
       CALL HISTOGRAM_FILL(l+14,dphi_psiC,wgts)
       CALL HISTOGRAM_FILL(l+18,dy_psiC,wgts)
       CALL HISTOGRAM_FILL(l+22,M_psiC,wgts)
       CALL HISTOGRAM_FILL(l+24,y_C,wgts)
       CALL HISTOGRAM_FILL(l+28,y_psiC,wgts)
    ENDIF
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay_pp_oniumquark_dps
END MODULE plot_pp_oniumquark_dps
