MODULE plot_pp_QQ_cb
  USE HELAC_Global
  USE pp_QQ_cb_global
  USE Kinetic_Func 
  IMPLICIT NONE
  INTEGER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=45
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_QQ_cb
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(:),ALLOCATABLE::weights_info
    INTEGER::i
    max_weight=1+ho_npdf+ho_nnpdf+ho_nref
    IF(ALLOCATED(weights_info))THEN
       DEALLOCATE(weights_info)
    ENDIF
    ALLOCATE(weights_info(max_weight))
    nwgt=1
    weights_info(nwgt)="central value  "
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'PDF=',ipdf_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_npdf
    ENDIF
    IF(reweight_npdf)THEN
       IF(nPDF_id.EQ.1000)THEN
          DO i=1,ho_nnpdf
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'NPDF=',inpdf_set_min+i-1-1000+iPDFSUP1,'  '
          ENDDO
       ELSE
          DO i=1,ho_nnpdf
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'NPDF=',inpdf_set_min-1+i,'  '
          ENDDO
       ENDIF
       nwgt=nwgt+ho_nnpdf
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'REF=',iref_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_nref
    ENDIF
    ! output plot files
    CALL plot_begin_pp_QQ_cb(nwgt,max_weight,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_QQ_cb

  SUBROUTINE plotout_pp_QQ_cb
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_QQ_cb(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_QQ_cb

  SUBROUTINE outfun_pp_QQ_cb(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::wgts
    INTEGER::init=0
    SAVE init,wgts
    ! it is necessary for using Phegas
    IF(www.EQ.0d0)RETURN
    IF(init.EQ.0)THEN
       max_weight=1+ho_npdf+ho_nnpdf+ho_nref
       IF(ALLOCATED(wgts))THEN
          DEALLOCATE(wgts)
       ENDIF
       ALLOCATE(wgts(max_weight))
       init=1
    ENDIF
    nwgt=1
    wgts(1)=www
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecpdf(i)
       ENDDO
    ENDIF
    IF(reweight_npdf)THEN
       DO i=1,ho_nnpdf
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecnpdf(i)
       ENDDO
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecref(i)
       ENDDO
    ENDIF
    ! output plot file 
    CALL plot_fill_pp_QQ_cb(wgts)
    RETURN
  END SUBROUTINE outfun_pp_QQ_cb

  SUBROUTINE plot_begin_pp_QQ_cb(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    !CALL INIHIST
    !CALL INIHIST_3D
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_QQ_cb: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    l=0
    CALL histogram_book(l+ 1,'total rate    ',&
            5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2,'B+ Pt ',&
            100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 3,'B+ yCM ',&
            100,-5d0,5d0,.TRUE.)
    ! LHCb fiducial
    ! LHCb-CONF-2016-003 for D0
    CALL histogram_book(l+ 4,'B+ Pt, 1.5<yCM<2.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 5,'B+ Pt, 2.0<yCM<2.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 6,'B+ Pt, 2.5<yCM<3.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 7,'B+ Pt, 3.0<yCM<3.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 8,'B+ Pt, 3.5<yCM<4.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 9,'B+ Pt, -3.0<yCM<-2.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+10,'B+ Pt, -3.5<yCM<-3.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+11,'B+ Pt, -4.0<yCM<-3.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+12,'B+ Pt, -4.5<yCM<-4.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+13,'B+ Pt, -5.0<yCM<-4.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+14,'B+ Pt, 1.5<yCM<4.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+15,'B+ Pt, 2.5<yCM<4.0 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+16,'B+ Pt, -5.0<yCM<-2.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+17,'B+ Pt, -4.0<yCM<-2.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+18,'B+ yCM, Pt< 8 ',&
         100,-5d0,5d0,.TRUE.)
    ! end of LHCb
    ! begin of CMS
    ! arXiv:1508.06678
    CALL histogram_book(l+19,'B+ Pt, -2.865<yCM<1.935 ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+20,'B+ Pt, -2.86<yCM<1.93 ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+21,'B+ yCM, 10<Pt<60, -2.86<yCM<-1.0 ',&
         2,-2.86d0,-1d0,.TRUE.)
    CALL histogram_book(l+22,'B+ yCM, 10<Pt<60, -1.0<yCM<1.0 ',&
         2,-1d0,1d0,.TRUE.)
    CALL histogram_book(l+23,'B+ yCM, 10<Pt<60, 1.0<yCM<2.86 ',&
         2,1d0,2.86d0,.TRUE.)
    ! end of CMS
    ! begin of R. Vogt
    CALL histogram_book(l+24,'B+ Pt, -0.1<yCM<0.1 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+25,'B+ Pt, 3.9<yCM<4.1 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+26,'B+ Pt, -4.1<yCM<-3.9 ',&
         100,0d0,50d0,.TRUE.)
    ! end of R. Vogt
    ! begin of LHCb for Yanxi (April 2018)
    CALL histogram_book(l+27,'B+ Pt, 1.5<yCM<2.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+28,'B+ Pt, 2.5<yCM<3.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+29,'B+ Pt, -3.5<yCM<-2.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+30,'B+ Pt, -4.5<yCM<-3.5 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+31,'B+ yCM, 2<Pt<20 ',&
         100,-5d0,5d0,.TRUE.)
    ! end of LHCb for Yanxi (April 2018)
    ! begin of AFTER, 1
    CALL histogram_book(l+32,'B+ Pt, -2.8<yCM<0.2 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+33,'B+ Pt, -0.2<yCM<2.8 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+34,'B+ Pt, -2.8<yCM<-2.3 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+35,'B+ Pt, -2.3<yCM<-1.8 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+36,'B+ Pt, -1.8<yCM<-1.3 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+37,'B+ Pt, -1.3<yCM<-0.8 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+38,'B+ Pt, -0.8<yCM<-0.3 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+39,'B+ Pt, -0.3<yCM<0.2 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+40,'B+ Pt, -0.2<yCM<0.3 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+41,'B+ Pt, 0.3<yCM<0.8 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+42,'B+ Pt, 0.8<yCM<1.3 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+43,'B+ Pt, 1.3<yCM<1.8 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+44,'B+ Pt, 1.8<yCM<2.3 ',&
          100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+45,'B+ Pt, 2.3<yCM<2.8 ',&
          100,0d0,50d0,.TRUE.)
    ! end of AFTER, 1
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_QQ_cb

  SUBROUTINE plot_end_pp_QQ_cb(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       !CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          !CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          !CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_QQ_cb
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_QQ_cb
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_QQ_cb
       psfile="pp_QQ_crystalball.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_QQ_cb
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_QQ_cb
       rootfile="pp_QQ_crystalball.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_QQ_cb
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_QQ_cb(xnorm)
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_QQ_cb

  SUBROUTINE plot_fill_pp_QQ_cb(wgts)
    USE Helac_Global
    IMPLICIT NONE
    !INCLUDE "pp_psipsi_dps.inc"
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2,pt
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,yD0,yD0bar
    IF(wgts(1).EQ.0d0)RETURN
    !IF(NDecayChains.GT.0)THEN
    !   CALL plot_fill_Decay_pp_psiX_cb(wgts)
    !   RETURN
    !ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    pini(1:4)=cb_hadron_pmom(1,1:4)+cb_hadron_pmom(2,1:4) ! (px,py,pz,E)
    yD0=rapidity(cb_hadron_pmom(3,1:4))
    pt=DSQRT(cb_hadron_pmom(3,1)**2+cb_hadron_pmom(3,2)**2)
    yD0bar=rapidity(cb_hadron_pmom(4,1:4))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,pt,wgts)
    CALL HISTOGRAM_fill(l+3,yD0,wgts)
    IF(yD0.GT.1.5d0.AND.yD0.LE.2d0)THEN
       CALL HISTOGRAM_fill(l+4,pt,wgts)
    ENDIF
    IF(yD0.GT.2d0.AND.yD0.LE.2.5d0)THEN
       CALL HISTOGRAM_fill(l+5,pt,wgts)
    ENDIF
    IF(yD0.GT.2.5d0.AND.yD0.LE.3d0)THEN
       CALL HISTOGRAM_fill(l+6,pt,wgts)
    ENDIF
    IF(yD0.GT.3d0.AND.yD0.LE.3.5d0)THEN
       CALL HISTOGRAM_fill(l+7,pt,wgts)
    ENDIF
    IF(yD0.GT.3.5d0.AND.yD0.LE.4d0)THEN
       CALL HISTOGRAM_fill(l+8,pt,wgts)
    ENDIF
    IF(yD0.GT.-3.0d0.AND.yD0.LE.-2.5d0)THEN
       CALL HISTOGRAM_fill(l+9,pt,wgts)
    ENDIF
    IF(yD0.GT.-3.5d0.AND.yD0.LE.-3.0d0)THEN
       CALL HISTOGRAM_fill(l+10,pt,wgts)
    ENDIF
    IF(yD0.GT.-4d0.AND.yD0.LE.-3.5d0)THEN
       CALL HISTOGRAM_fill(l+11,pt,wgts)
    ENDIF
    IF(yD0.GT.-4.5d0.AND.yD0.LE.-4d0)THEN
       CALL HISTOGRAM_fill(l+12,pt,wgts)
    ENDIF
    IF(yD0.GT.-5d0.AND.yD0.LE.-4.5d0)THEN
       CALL HISTOGRAM_fill(l+13,pt,wgts)
    ENDIF
    IF(yD0.GT.1.5d0.AND.yD0.LE.4d0)THEN
       CALL HISTOGRAM_fill(l+14,pt,wgts)
    ENDIF
    IF(yD0.GT.2.5d0.AND.yD0.LE.4d0)THEN
       CALL HISTOGRAM_fill(l+15,pt,wgts)
    ENDIF
    IF(yD0.GT.-5d0.AND.yD0.LE.-2.5d0)THEN
       CALL HISTOGRAM_fill(l+16,pt,wgts)
    ENDIF
    IF(yD0.GT.-4d0.AND.yD0.LE.-2.5d0)THEN
       CALL HISTOGRAM_fill(l+17,pt,wgts)
    ENDIF
    IF(pt.LE.8d0)THEN
       CALL HISTOGRAM_fill(l+18,yD0,wgts)
    ENDIF
    IF(yD0.GT.-2.865d0.AND.yD0.LE.1.935d0)THEN
       CALL HISTOGRAM_fill(l+19,pt,wgts)
    ENDIF
    IF(yD0.GT.-2.86d0.AND.yD0.LE.1.93d0)THEN
       CALL HISTOGRAM_fill(l+20,pt,wgts)
    ENDIF
    IF(pt.LT.60d0.AND.pt.GT.10d0)THEN
       CALL HISTOGRAM_fill(l+21,yD0,wgts)
       CALL HISTOGRAM_fill(l+22,yD0,wgts)
       CALL HISTOGRAM_fill(l+23,yD0,wgts)
    ENDIF
    IF(yD0.GT.-0.1d0.AND.yD0.LT.0.1d0)THEN
       CALL HISTOGRAM_fill(l+24,pt,wgts)
    ENDIF
    IF(yD0.GT.3.9d0.AND.yD0.LT.4.1d0)THEN
       CALL HISTOGRAM_fill(l+25,pt,wgts)
    ENDIF
    IF(yD0.GT.-4.1d0.AND.yD0.LT.-3.9d0)THEN
       CALL HISTOGRAM_fill(l+26,pt,wgts)
    ENDIF
    IF(yD0.GT.1.5d0.AND.yD0.LT.2.5d0)THEN
       CALL HISTOGRAM_fill(l+27,pt,wgts)
    ENDIF
    IF(yD0.GT.2.5d0.AND.yD0.LT.3.5d0)THEN
       CALL HISTOGRAM_fill(l+28,pt,wgts)
    ENDIF
    IF(yD0.GT.-3.5d0.AND.yD0.LT.-2.5d0)THEN
       CALL HISTOGRAM_fill(l+29,pt,wgts)
    ENDIF
    IF(yD0.GT.-4.5d0.AND.yD0.LT.-3.5d0)THEN
       CALL HISTOGRAM_fill(l+30,pt,wgts)
    ENDIF
    IF(pt.GT.2d0.AND.pt.LT.20d0)THEN
       CALL HISTOGRAM_fill(l+31,yD0,wgts)
    ENDIF
    IF(yD0.GT.-2.8d0.AND.yD0.LT.0.2d0)THEN
       CALL HISTOGRAM_fill(l+32,pt,wgts)
    ENDIF
    IF(yD0.GT.-0.2d0.AND.yD0.LT.2.8d0)THEN
       CALL HISTOGRAM_fill(l+33,pt,wgts)
    ENDIF
    IF(yD0.GT.-2.8d0.AND.yD0.LT.-2.3d0)THEN
       CALL HISTOGRAM_fill(l+34,pt,wgts)
    ENDIF
    IF(yD0.GT.-2.3d0.AND.yD0.LT.-1.8d0)THEN
       CALL HISTOGRAM_fill(l+35,pt,wgts)
    ENDIF
    IF(yD0.GT.-1.8d0.AND.yD0.LT.-1.3d0)THEN
       CALL HISTOGRAM_fill(l+36,pt,wgts)
    ENDIF
    IF(yD0.GT.-1.3d0.AND.yD0.LT.-0.8d0)THEN
       CALL HISTOGRAM_fill(l+37,pt,wgts)
    ENDIF
    IF(yD0.GT.-0.8d0.AND.yD0.LT.-0.3d0)THEN
       CALL HISTOGRAM_fill(l+38,pt,wgts)
    ENDIF
    IF(yD0.GT.-0.3d0.AND.yD0.LT.0.2d0)THEN
       CALL HISTOGRAM_fill(l+39,pt,wgts)
    ENDIF
    IF(yD0.GT.-0.2d0.AND.yD0.LT.0.3d0)THEN
       CALL HISTOGRAM_fill(l+40,pt,wgts)
    ENDIF
    IF(yD0.GT.0.3d0.AND.yD0.LT.0.8d0)THEN
       CALL HISTOGRAM_fill(l+41,pt,wgts)
    ENDIF
    IF(yD0.GT.0.8d0.AND.yD0.LT.1.3d0)THEN
       CALL HISTOGRAM_fill(l+42,pt,wgts)
    ENDIF
    IF(yD0.GT.1.3d0.AND.yD0.LT.1.8d0)THEN
       CALL HISTOGRAM_fill(l+43,pt,wgts)
    ENDIF
    IF(yD0.GT.1.8d0.AND.yD0.LT.2.3d0)THEN
       CALL HISTOGRAM_fill(l+44,pt,wgts)
    ENDIF
    IF(yD0.GT.2.3d0.AND.yD0.LT.2.8d0)THEN
       CALL HISTOGRAM_fill(l+45,pt,wgts)
    ENDIF

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_QQ_cb

  SUBROUTINE HwU_write_file_pp_QQ_cb(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,FILE=TRIM(output_dir)//'pp_QQ_crystalball.HwU',&
         status='unknown')
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_QQ_cb

  SUBROUTINE open_topdrawer_file_pp_QQ_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_QQ_crystalball.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_QQ_cb

  SUBROUTINE close_topdrawer_file_pp_QQ_cb
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_QQ_cb

  SUBROUTINE open_gnuplot_file_pp_QQ_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_QQ_crystalball.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_QQ_cb

  SUBROUTINE close_gnuplot_file_pp_QQ_cb
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_QQ_cb

  SUBROUTINE open_root_file_pp_QQ_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_QQ_crystalball.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_QQ_cb

  SUBROUTINE close_root_file_pp_QQ_cb
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_QQ_cb

END MODULE plot_pp_QQ_cb
