MODULE NLO_pp_oniumX
  ! 0: leading order term (Dirac delta term)
  ! 1: NLO Dirac delta term
  ! 2: NLO (1/(1-z))_{z0+} term
  ! 3: NLO (1/(1-z))_{z0+} term with z=1
  ! 4: NLO (log(1-z)/(1-z))_{z0+} term
  ! 5: NLO (log(1-z)/(1-z))_{z0+} term with z=1
  ! 6: NLO normal term
  ! check formula in /Users/erdissshaw/Works/Manuscript/ResumOnia
  ! and Mathematica code in /Users/erdissshaw/Works/FLibatM/jpsi_resummation/CSS/Untitled-2.nb
  USE pp_oniumX_Global
  IMPLICIT NONE
  PRIVATE
  REAL(KIND(1d0)),PUBLIC::onium_muR=3d0,onium_muF=3d0,onium_muNRQCD=1.5d0
  REAL(KIND(1d0)),PUBLIC::z0=1d-2 ! it is MQQ**2/s
  ! spin-summed matrix elements
  PUBLIC::NLO_gg_1S01X,NLO_gq_1S01X,NLO_qq_1S01X
  PUBLIC::NLO_gg_1S08X,NLO_gq_1S08X,NLO_qq_1S08X
  PUBLIC::NLO_gg_3S11X
  PUBLIC::NLO_gg_3S18X,NLO_gq_3S18X,NLO_qq_3S18X
  PUBLIC::NLO_gg_3P01X,NLO_gq_3P01X,NLO_qq_3P01X
  PUBLIC::NLO_gg_3P11X,NLO_gq_3P11X,NLO_qq_3P11X
  PUBLIC::NLO_gg_3P21X,NLO_gq_3P21X,NLO_qq_3P21X
  PUBLIC::NLO_gg_3P08X,NLO_gq_3P08X,NLO_qq_3P08X
  PUBLIC::NLO_gg_3P18X,NLO_gq_3P18X,NLO_qq_3P18X
  PUBLIC::NLO_gg_3P28X,NLO_gq_3P28X,NLO_qq_3P28X
  PUBLIC::NLO_gg_1P11X
  PUBLIC::NLO_gg_1P18X,NLO_gq_1P18X,NLO_qq_1P18X
CONTAINS

  SUBROUTINE Pggbar(z,plus,noplus)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z
    REAL(KIND(1d0)),INTENT(OUT)::plus,noplus
    REAL(KIND(1d0)),PARAMETER::CA=3d0,TWO=2d0,ONE=1d0
    plus=TWO*CA*z
    noplus=TWO*CA*((ONE-z)/z+z*(ONE-z))
    RETURN
  END SUBROUTINE Pggbar

  SUBROUTINE Pgg(z,res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z
    REAL(KIND(1d0)),INTENT(OUT)::res
    REAL(KIND(1d0)),PARAMETER::CA=3d0,TWO=2d0,ONE=1d0
    res=TWO*CA*(z/(ONE-z)+(ONE-z)/z+z*(ONE-z))
    RETURN
  END SUBROUTINE Pgg

  SUBROUTINE Pqqbar(z,plus)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z
    REAL(KIND(1d0)),INTENT(OUT)::plus
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,ONE=1d0
    plus=CF*(ONE+z**2)
    RETURN
  END SUBROUTINE Pqqbar

  SUBROUTINE Pqq(z,res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z
    REAL(KIND(1d0)),INTENT(OUT)::res
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,ONE=1d0
    res=CF*(ONE+z**2)/(ONE-z)
    RETURN
  END SUBROUTINE Pqq

  ! Pqgbar=Pqg
  SUBROUTINE Pqg(z,res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z
    REAL(KIND(1d0)),INTENT(OUT)::res
    REAL(KIND(1d0)),PARAMETER::TF=0.5d0,ONE=1d0
    res=TF*(z**2+(ONE-z)**2)
    RETURN
  END SUBROUTINE Pqg

  ! Pgqbar=Pgq
  SUBROUTINE Pgq(z,res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z
    REAL(KIND(1d0)),INTENT(OUT)::res
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,ONE=1d0
    res=CF*(ONE+(ONE-z)**2)/z
    RETURN
  END SUBROUTINE Pgq

  ! 1S0[1]
  SUBROUTINE NLO_gg_1S01X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=2d0/9d0*as**2*pipi3/MQQ**5*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=CF*(-5d0+pipi2/4d0)+CA*(ONE+pipi2/12d0)+2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
    fgg=-CA/(6d0*(ONE-z)*(ONE+z)**3)*(12d0+11d0*z**2+24d0*z**3-21d0*z**4&
         -24d0*z**5+9d0*z**6-11d0*z**8+12d0*(-1d0+5d0*z**2+2d0*z**3+z**4&
         +3d0*z**6+2d0*z**7)*DLOG(z))
    fgg_z1=0d0
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_1S01X

  SUBROUTINE NLO_gq_1S01X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=2d0/9d0*as**2*pipi3/MQQ**5*LDME
    me(0)=0d0
    fgq=-CF*(ONE-z)*(ONE-DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(1:5)=0d0
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_1S01X

  SUBROUTINE NLO_qq_1S01X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq
    SAVE sigma0,fqq
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*fqq
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=2d0/9d0*as**2*pipi3/MQQ**5*LDME
    me(0)=0d0
    fqq=8d0/9d0*CF*(ONE-z)*z**2
    asopi=as/pipi
    me(1:5)=0d0
    me(6)=sigma0*asopi*fqq
    RETURN
  END SUBROUTINE NLO_qq_1S01X

  ! 1S0[8]
  SUBROUTINE NLO_gg_1S08X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=5d0/12d0*as**2*pipi3/MQQ**5*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=CF*(-5d0+pipi2/4d0)+CA*(3d0-pipi2/24d0)-CA*DLOG(ONE-z0)+2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
    fgg=-CA/(6d0*(ONE-z)*(ONE+z)**3)*(12d0+23d0*z**2+30d0*z**3-21d0*z**4&
         -24d0*z**5+9d0*z**6-6d0*z**7-23d0*z**8+12d0*(-1d0+5d0*z**2+2d0*z**3+3d0*z**4&
         +5d0*z**6+2d0*z**7)*DLOG(z))
    fgg_z1=-CA
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_1S08X


  SUBROUTINE NLO_gq_1S08X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=5d0/12d0*as**2*pipi3/MQQ**5*LDME
    me(0)=0d0
    fgq=-CF*(ONE-z)*(ONE-DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(1:5)=0d0
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_1S08X


  SUBROUTINE NLO_qq_1S08X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq
    SAVE sigma0,fqq
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*fqq
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=5d0/12d0*as**2*pipi3/MQQ**5*LDME
    me(0)=0d0
    fqq=8d0/9d0*CF*(ONE-z)*z**2
    asopi=as/pipi
    me(1:5)=0d0
    me(6)=sigma0*asopi*fqq
    RETURN
  END SUBROUTINE NLO_qq_1S08X

  ! 3S1[1]
  ! It is LO accuracy
  SUBROUTINE NLO_gg_3S11X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgg
    SAVE sigma0,fgg
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fgg
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**5*LDME
    fgg=20d0/81d0*z**2/((ONE-z)**2*(ONE+z)**3)*((2d0+z+2d0*z**2-4d0*z**4-z**5)&
         +2d0*z**2*(5d0+2d0*z+z**2)*DLOG(z))
    me(6)=sigma0*fgg
    RETURN
  END SUBROUTINE NLO_gg_3S11X

  ! 3S1[8]
  SUBROUTINE NLO_gg_3S18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgg
    SAVE sigma0,fgg
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fgg
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**5*LDME
    fgg=1d0/(36d0*(ONE-z)**2*(ONE+z)**3)*(108d0+153d0*z+400d0*z**2+65d0*z**3-356d0*z**4&
         -189d0*z**5-152d0*z**6-29d0*z**7+(108d0+756d0*z+432d0*z**2+704d0*z**3&
         +260d0*z**4+76d0*z**5)*z*DLOG(z))
    me(6)=sigma0*fgg
    RETURN
  END SUBROUTINE NLO_gg_3S18X

   SUBROUTINE NLO_gq_3S18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pqg_norm
    SAVE Pqg_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pqg_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=16d0/27d0*as**2*pipi3/MQQ**5*LDME
    fgq=TF/4d0*(ONE+3d0*z)*(ONE-z)*z+3d0*CA/8d0*(0.5d0*(ONE-z)*(2d0+z+2d0*z**2)&
         +z*(one+z)*DLOG(z))
    CALL Pqg(z,Pqg_norm)
    asopi=as/pipi
    me(6)=sigma0*asopi*(Pqg_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+TF*z**2*(ONE-z)+fgq)
    RETURN
  END SUBROUTINE NLO_gq_3S18X

  SUBROUTINE NLO_qq_3S18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dqq_nomu,dqq,fqq,fqq_z1,beta0
    SAVE sigma0,dqq_nomu,fqq,fqq_z1,beta0
    REAL(KIND(1d0))::Pqqbar_plus, Pqq_norm, Pqqbar_plus_z1
    SAVE Pqqbar_plus,Pqq_norm, Pqqbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dqq=dqq_nomu+beta0*DLOG(onium_muR**2/MQQ**2)&
            +CF*(3d0+4d0*DLOG(ONE-z0))**DLOG(MQQ/onium_muF)
       me(1)=sigma0*asopi*dqq
       me(2)=sigma0*asopi*(Pqqbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fqq)
       me(3)=sigma0*asopi*(Pqqbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fqq_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pqq_norm)
       me(5)=sigma0*asopi*4d0*CF
       me(6)=sigma0*asopi*(CF*z*(one-z))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=16d0/27d0*as**2*pipi3/MQQ**5*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dqq_nomu=-10d0/9d0*TF*nlq+CA*(59d0/9d0-pipi2/4d0+2d0/3d0*DLOG(2d0)-DLOG(ONE-z0))&
         +CF*(-8d0+pipi2/3d0+2d0*DLOG(one-z0)**2)
    dqq=dqq_nomu+beta0*DLOG(onium_muR**2/MQQ**2)&
         +CF*(3d0+4d0*DLOG(ONE-z0))*DLOG(MQQ/onium_muF)
    fqq=-CF*z*(one-z)**2-CA/3d0*z*(one+z+z**2)
    fqq_z1=-CA
    CALL Pqqbar(ONE,Pqqbar_plus_z1)
    CALL Pqqbar(z,Pqqbar_plus)
    CALL Pqq(z,Pqq_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dqq
    me(2)=sigma0*asopi*(Pqqbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fqq)
    me(3)=sigma0*asopi*(Pqqbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fqq_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pqq_norm)
    me(5)=sigma0*asopi*4d0*CF
    me(6)=sigma0*asopi*(CF*z*(one-z))
    RETURN
  END SUBROUTINE NLO_qq_3S18X

  ! 3P0[1]
  SUBROUTINE NLO_gg_3P01X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=8d0/3d0*as**2*pipi3/MQQ**7*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=CF*(-7d0/3d0+pipi2/4d0)+CA*(1d0/3d0+pipi2/12d0)+2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
    fgg=-CA/(54d0*(ONE-z)**3*(ONE+z)**5)*(172d0-56d0*z-789d0*z**2+244d0*z**3+1721d0*z**4&
         -696d0*z**5-802d0*z**6+560d0*z**7-210d0*z**8+80d0*z**9+7d0*z**10-132d0*z**11-99d0*z**12&
         +12d0*(-9d0+31d0*z**2-14d0*z**3-40d0*z**4+10d0*z**5&
         +176d0*z**6-42d0*z**7-7d0*z**8-10d0*z**9+41d0*z**10+24d0*z**11)*DLOG(z))
    fgg_z1=0d0
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_3P01X

  SUBROUTINE NLO_gq_3P01X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=8d0/3d0*as**2*pipi3/MQQ**7*LDME
    fgq=CF/9d0*(-1d0/3d0*(one-z)*(43d0-14d0*z+4d0*z**2)+(9d0-9d0*z+4d0*z**2)*DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_3P01X

  SUBROUTINE NLO_qq_3P01X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq,fqq_z1,dqq,dqq_nomu
    SAVE sigma0,fqq,fqq_z1,dqq_nomu
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:6)=0d0
       dqq=dqq_nomu+1024d0/729d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dqq
       me(2)=sigma0*fqq
       me(3)=sigma0*fqq_z1
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dqq_nomu=1024d0/729d0*(DLOG(one-z0)-5d0/6d0)
    dqq=dqq_nomu+1024d0/729d0*(-DLOG(onium_muNRQCD/MQQ))
    fqq=256d0/729d0*z**2*(one-3d0*z)**2
    fqq_z1=1024d0/729d0
    me(1)=sigma0*dqq
    me(2)=sigma0*fqq
    me(3)=sigma0*fqq_z1
    RETURN
  END SUBROUTINE NLO_qq_3P01X

  ! 3P1[1]
  SUBROUTINE NLO_gg_3P11X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgg
    SAVE sigma0,fgg
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fgg
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    fgg=8d0/27d0/((ONE-z)**4*(ONE+z)**5)*((10d0-8d0*z-157d0*z**2-9d0*z**3-6d0*z**4&
         +136d0*z**5+404d0*z**6+26d0*z**7-212d0*z**8-144d0*z**9-39d0*z**10-z**11)&
         +(-48d0*z**2-48d0*z**3-480d0*z**4+84d0*z**5+204d0*z**6+336d0*z**7&
              +312d0*z**8+108d0*z**9+12d0*z**10)*DLOG(z))
    me(6)=sigma0*fgg
    RETURN
  END SUBROUTINE NLO_gg_3P11X

  SUBROUTINE NLO_gq_3P11X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fgq
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    fgq=-32d0/81d0*(1d0/3d0*(one-z)*(-5d0+4d0*z+4d0*z**2)+z**2*DLOG(z))
    me(6)=sigma0*fgq
    RETURN
  END SUBROUTINE NLO_gq_3P11X

  SUBROUTINE NLO_qq_3P11X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dqq_nomu,dqq,fqq,fqq_z1
    SAVE sigma0,dqq_nomu,fqq,fqq_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:6)=0d0
       dqq=dqq_nomu+1024d0/729d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dqq
       me(2)=sigma0*fqq
       me(3)=sigma0*fqq_z1
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dqq_nomu=1024d0/729d0*(DLOG(one-z0)-5d0/6d0)
    dqq=dqq_nomu+1024d0/729d0*(-DLOG(onium_muNRQCD/MQQ))
    fqq=512d0/729d0*z**2*(one+z)
    fqq_z1=1024d0/729d0
    me(1)=sigma0*dqq
    me(2)=sigma0*fqq
    me(3)=sigma0*fqq_z1
    RETURN
  END SUBROUTINE NLO_qq_3P11X

  ! 3P2[1]
  SUBROUTINE NLO_gg_3P21X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=32d0/45d0*as**2*pipi3/MQQ**7*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=-4d0*CF+CA*(1d0/3d0-pipi2/6d0+5d0/3d0*DLOG(2d0))+2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
    fgg=-CA/(36d0*(ONE-z)**3*(ONE+z)**5)*(106d0-32d0*z-207d0*z**2+271d0*z**3+752d0*z**4&
         -1032d0*z**5-256d0*z**6+266d0*z**7-360d0*z**8+728d0*z**9+31d0*z**10-201d0*z**11&
         -66d0*z**12+12d0*(-6d0+22d0*z**2-8d0*z**3+74d0*z**4+31d0*z**5+11d0*z**6&
         -204d0*z**7+86d0*z**8+17d0*z**9+5d0*z**10+12d0*z**11)*DLOG(z))
    fgg_z1=0d0
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_3P21X

  SUBROUTINE NLO_gq_3P21X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=32d0/45d0*as**2*pipi3/MQQ**7*LDME
    fgq=CF/12d0*(-1d0/3d0*(one-z)*(53d0-16d0*z+20d0*z**2)+(12d0-12d0*z+5d0*z**2)*DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_3P21X

  SUBROUTINE NLO_qq_3P21X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq,fqq_z1,dqq,dqq_nomu
    SAVE sigma0,fqq,fqq_z1,dqq_nomu
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:6)=0d0
       dqq=dqq_nomu+1024d0/729d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dqq
       me(2)=sigma0*fqq
       me(3)=sigma0*fqq_z1
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dqq_nomu=1024d0/729d0*(DLOG(one-z0)-5d0/6d0)
    dqq=dqq_nomu+1024d0/729d0*(-DLOG(onium_muNRQCD/MQQ))
    fqq=512d0/3645d0*z**2*(one+3d0*z+6d0*z**2)
    fqq_z1=1024d0/729d0
    me(1)=sigma0*dqq
    me(2)=sigma0*fqq
    me(3)=sigma0*fqq_z1
    RETURN
  END SUBROUTINE NLO_qq_3P21X

  ! 3P0[8]
  SUBROUTINE NLO_gg_3P08X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=5d0*as**2*pipi3/MQQ**7*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=CF*(-7d0/3d0+pipi2/4d0)+CA*(71d0/54d0-pipi2/24d0+35d0/27d0*DLOG(2d0))&
         -CA*DLOG(one-z0)+2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
    fgg=-CA/(54d0*(ONE-z)**3*(ONE+z)**5)*(172d0-56d0*z-785d0*z**2+254d0*z**3+1881d0*z**4&
         -948d0*z**5-334d0*z**6+632d0*z**7-790d0*z**8+268d0*z**9+63d0*z**10-150d0*z**11&
         -207d0*z**12+12d0*(-9d0+31d0*z**2-14d0*z**3-45d0*z**4-15d0*z**5+279d0*z**6&
         -67d0*z**7+26d0*z**8-21d0*z**9+70d0*z**10+21d0*z**11)*DLOG(z))
    fgg_z1=-CA
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_3P08X

  SUBROUTINE NLO_gq_3P08X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=5d0*as**2*pipi3/MQQ**7*LDME
    fgq=CF/9d0*(-1d0/3d0*(one-z)*(43d0-14d0*z+4d0*z**2)+(9d0-9d0*z+4d0*z**2)*DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_3P08X

  SUBROUTINE NLO_qq_3P08X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq,fqq_z1,dqq,dqq_nomu
    SAVE sigma0,fqq,fqq_z1,dqq_nomu
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:6)=0d0
       dqq=dqq_nomu+640d0/243d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dqq
       me(2)=sigma0*fqq
       me(3)=sigma0*fqq_z1
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dqq_nomu=640d0/243d0*(DLOG(one-z0)-5d0/6d0)
    dqq=dqq_nomu+640d0/243d0*(-DLOG(onium_muNRQCD/MQQ))
    fqq=160d0/243d0*z**2*(one-3d0*z)**2
    fqq_z1=640d0/243d0
    me(1)=sigma0*dqq
    me(2)=sigma0*fqq
    me(3)=sigma0*fqq_z1
    RETURN
  END SUBROUTINE NLO_qq_3P08X

  ! 3P1[8]
  SUBROUTINE NLO_gg_3P18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgg
    SAVE sigma0,fgg
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fgg
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    fgg=5d0/9d0/((ONE-z)**4*(ONE+z)**5)*((10d0-8d0*z-165d0*z**2-8d0*z**3-182d0*z**4&
         +220d0*z**5+692d0*z**6+44d0*z**7-276d0*z**8-244d0*z**9-79d0*z**10-4d0*z**11)&
         +(-48d0*z**2-48d0*z**3-660d0*z**4+144d0*z**5+48d0*z**6+576d0*z**7&
              +540d0*z**8+192d0*z**9+24d0*z**10)*DLOG(z))
    me(6)=sigma0*fgg
    RETURN
  END SUBROUTINE NLO_gg_3P18X

  SUBROUTINE NLO_gq_3P18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fgq
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    fgq=-20d0/27d0*(1d0/3d0*(one-z)*(-5d0+4d0*z+4d0*z**2)+z**2*DLOG(z))
    me(6)=sigma0*fgq
    RETURN
  END SUBROUTINE NLO_gq_3P18X

  SUBROUTINE NLO_qq_3P18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dqq_nomu,dqq,fqq,fqq_z1
    SAVE sigma0,dqq_nomu,fqq,fqq_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:6)=0d0
       dqq=dqq_nomu+640d0/243d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dqq
       me(2)=sigma0*fqq
       me(3)=sigma0*fqq_z1
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dqq_nomu=640d0/243d0*(DLOG(one-z0)-5d0/6d0)
    dqq=dqq_nomu+640d0/243d0*(-DLOG(onium_muNRQCD/MQQ))
    fqq=320d0/243d0*z**2*(one+z)
    fqq_z1=640d0/243d0
    me(1)=sigma0*dqq
    me(2)=sigma0*fqq
    me(3)=sigma0*fqq_z1
    RETURN
  END SUBROUTINE NLO_qq_3P18X

  ! 3P2[8]
  SUBROUTINE NLO_gg_3P28X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=4d0/3d0*as**2*pipi3/MQQ**7*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=-4d0*CF+CA*(59d0/36d0+pipi2/12d0+7d0/9d0*DLOG(2d0))&
         -CA*DLOG(one-z0)+2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)
    fgg=-CA/(36d0*(ONE-z)**3*(ONE+z)**5)*(106d0-32d0*z-215d0*z**2+260d0*z**3+1224d0*z**4&
         -1284d0*z**5-400d0*z**6-76d0*z**7-424d0*z**8+1252d0*z**9-153d0*z**10-120d0*z**11&
         -138d0*z**12+12d0*(-6d0+22d0*z**2-8d0*z**3+90d0*z**4+3d0*z**5+117d0*z**6&
         -265d0*z**7+110d0*z**8-51d0*z**9+43d0*z**10+9d0*z**11)*DLOG(z))
    fgg_z1=-CA
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_3P28X

  SUBROUTINE NLO_gq_3P28X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=4d0/3d0*as**2*pipi3/MQQ**7*LDME
    fgq=CF/12d0*(-1d0/3d0*(one-z)*(53d0-16d0*z+20d0*z**2)+(12d0-12d0*z+5d0*z**2)*DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_3P28X

  SUBROUTINE NLO_qq_3P28X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq,fqq_z1,dqq,dqq_nomu
    SAVE sigma0,fqq,fqq_z1,dqq_nomu
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:6)=0d0
       dqq=dqq_nomu+640d0/243d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dqq
       me(2)=sigma0*fqq
       me(3)=sigma0*fqq_z1
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dqq_nomu=640d0/243d0*(DLOG(one-z0)-5d0/6d0)
    dqq=dqq_nomu+640d0/243d0*(-DLOG(onium_muNRQCD/MQQ))
    fqq=64d0/243d0*z**2*(one+3d0*z+6d0*z**2)
    fqq_z1=640d0/243d0
    me(1)=sigma0*dqq
    me(2)=sigma0*fqq
    me(3)=sigma0*fqq_z1
    RETURN
  END SUBROUTINE NLO_qq_3P28X

  ! 1P1[1]
  ! It is LO accuracy
  SUBROUTINE NLO_gg_1P11X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgg,dgg,dgg_nomu
    SAVE sigma0,fgg,dgg_nomu
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       dgg=dgg_nomu+80d0/81d0*(-DLOG(onium_muNRQCD/MQQ))
       me(1)=sigma0*dgg
       me(6)=sigma0*fgg
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    dgg_nomu=80d0/81d0*(DLOG(one-z0)-5d0/6d0)
    dgg=dgg_nomu+80d0/81d0*(-DLOG(onium_muNRQCD/MQQ))
    fgg=160d0/81d0/((ONE-z)**3*(ONE+z)**5)*((1d0+z+10d0*z**2+4d0*z**3+4d0*z**4&
         -6d0*z**5-14d0*z**6-z**8+z**9)*z**2&
         +(10d0+5d0*z+29d0*z**2+6d0*z**3+16d0*z**4-3d0*z**5+z**6)*z**4*DLOG(z))
    me(1)=sigma0*dgg
    me(6)=sigma0*fgg
    RETURN
  END SUBROUTINE NLO_gg_1P11X

  ! 1P1[8]
  SUBROUTINE NLO_gg_1P18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,dgg_nomu,dgg,fgg,fgg_z1,beta0
    SAVE sigma0,dgg_nomu,fgg,fgg_z1,beta0
    REAL(KIND(1d0))::Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    SAVE Pggbar_plus, Pggbar_noplus, Pgg_norm, Pggbar_plus_z1
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0)=sigma0
       dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
            +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)-38d0/9d0*DLOG(onium_muNRQCD/MQQ)
       me(1)=sigma0*asopi*dgg
       me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
       me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
       me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
       me(5)=sigma0*asopi*4d0*CA
       me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**2*pipi3/MQQ**7*LDME
    me(0)=sigma0
    beta0=11d0/6d0*CA-2d0/3d0*TF*nlq
    dgg_nomu=32d0/9d0+pipi2/6d0+17d0/6d0*DLOG(2d0)-43d0/18d0*CA*DLOG(one-z0)&
         +2d0*CA*DLOG(ONE-z0)**2
    dgg=dgg_nomu+beta0*DLOG(onium_muR**2/onium_muF**2)&
         +2d0*CA*DLOG(ONE-z0)*DLOG(MQQ**2/onium_muF**2)-38d0/9d0*DLOG(onium_muNRQCD/MQQ)
    fgg=CA/(54d0*(ONE-z)**2*(ONE+z)**5)*(-(108d0+108d0*z-863d0*z**2-971d0*z**3-6397d0*z**4&
         -3109d0*z**5-5601d0*z**6-685d0*z**7-1007d0*z**8-271d0*z**9)*(one-z**2)&
         +4d0*(27d0+27d0*z-27d0*z**2+54d0*z**3+1892d0*z**4+1324d0*z**5+3316d0*z**6&
         +714d0*z**7+1553d0*z**8+345d0*z**9+119d0*z**10)*DLOG(z))
    fgg_z1=-43d0/27d0*CA
    CALL Pggbar(ONE,Pggbar_plus_z1, Pggbar_noplus)
    CALL Pggbar(z,Pggbar_plus,Pggbar_noplus)
    CALL Pgg(z,Pgg_norm)
    asopi=as/pipi
    me(1)=sigma0*asopi*dgg
    me(2)=sigma0*asopi*(Pggbar_plus*z*DLOG(MQQ**2/z/onium_muF**2)+fgg)
    me(3)=sigma0*asopi*(Pggbar_plus_z1*DLOG(MQQ**2/onium_muF**2)+fgg_z1)
    me(4)=sigma0*asopi*(2d0*z*(one-z)*Pgg_norm)
    me(5)=sigma0*asopi*4d0*CA
    me(6)=sigma0*asopi*(Pggbar_noplus*z*DLOG(MQQ**2/z/onium_muF**2))
    RETURN
  END SUBROUTINE NLO_gg_1P18X

  SUBROUTINE NLO_gq_1P18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fgq
    SAVE sigma0,fgq
    REAL(KIND(1d0))::Pgq_norm
    SAVE Pgq_norm
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    REAL(KIND(1d0))::asopi
    SAVE asopi
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**2*pipi3/MQQ**7*LDME
    fgq=-CF*(ONE-z)*(ONE-DLOG(z))
    CALL Pgq(z,Pgq_norm)
    asopi=as/pipi
    me(6)=sigma0*asopi*(Pgq_norm*z/2d0*DLOG(MQQ**2*(ONE-z)**2/z/onium_muF**2)+CF*z**2/2d0+fgq)
    RETURN
  END SUBROUTINE NLO_gq_1P18X

  SUBROUTINE NLO_qq_1P18X(z,as,MQQ,LDME,me,change_mu_only)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,as,MQQ,LDME
    REAL(KIND(1d0)),DIMENSION(0:6),INTENT(OUT)::me
    LOGICAL,INTENT(IN),OPTIONAL::change_mu_only
    REAL(KIND(1d0))::sigma0,fqq
    SAVE sigma0,fqq
    REAL(KIND(1d0)),PARAMETER::CF=4d0/3d0,CA=3d0,TF=0.5d0,ONE=1d0
    REAL(KIND(1d0)),PARAMETER::pipi=3.141592653589793238462643383279502884197d0
    REAL(KIND(1d0)),PARAMETER::pipi2=9.869604401089358618834490999876151135314d0
    REAL(KIND(1d0)),PARAMETER::pipi3=31.00627668029982017547631506710139520223d0
    IF(PRESENT(change_mu_only).AND.change_mu_only)THEN
       me(0:5)=0d0
       me(6)=sigma0*fqq
       RETURN
    ENDIF
    me(0:6)=0d0
    sigma0=as**3*pipi2/MQQ**7*LDME
    fqq=32d0/27d0*(ONE-z)*z**2
    me(6)=sigma0*fqq
    RETURN
  END SUBROUTINE NLO_qq_1P18X

END MODULE NLO_pp_oniumX
