MODULE plot_pp_oniumX
  USE HELAC_Global
  USE pp_oniumX_Global
  USE Kinetic_Func
  IMPLICIT NONE
  INTEGER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=240
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_oniumX
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(:),ALLOCATABLE::weights_info
    INTEGER::i
    max_weight=1+ho_nscale+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
    IF(ALLOCATED(weights_info))THEN
       DEALLOCATE(weights_info)
    ENDIF
    ALLOCATE(weights_info(max_weight))
    nwgt=1
    weights_info(nwgt)="central value  "
    IF(reweight_scale)THEN
       nwgt=nwgt+ho_nscale
       IF(ho_nscale.NE.8)THEN
          WRITE(*,*) 'ERROR #1 in initplot_pp_oniumX:',ho_nscale
          STOP
       ENDIF
       WRITE(weights_info(nwgt-7),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",1.0,"muF=",rw_Fscale_up
       WRITE(weights_info(nwgt-6),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",1.0,"muF=",rw_Fscale_down
       WRITE(weights_info(nwgt-5),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_up,"muF=",1d0
       WRITE(weights_info(nwgt-4),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_up,"muF=",rw_Fscale_up
       WRITE(weights_info(nwgt-3),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_up,"muF=",rw_Fscale_down
       WRITE(weights_info(nwgt-2),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_down,"muF=",1d0
       WRITE(weights_info(nwgt-1),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_down,"muF=",rw_Fscale_up
       WRITE(weights_info(nwgt  ),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_down,"muF=",rw_Fscale_down
    END IF
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'PDF=',ipdf_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_npdf
       IF(ho_npdf2.GT.0)THEN
          DO i=1,ho_npdf2
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'PDF2=',ipdf2_set_min-1+i,'  '
          ENDDO
          nwgt=nwgt+ho_npdf2
       ENDIF
    ENDIF
    IF(reweight_npdf)THEN
       IF(nPDF_id.EQ.1000)THEN
          DO i=1,ho_nnpdf
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'NPDF=',inpdf_set_min+i-1-1000+iPDFSUP1,'  '
          ENDDO
       ELSE
          DO i=1,ho_nnpdf
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'NPDF=',inpdf_set_min-1+i,'  '
          ENDDO
       ENDIF
       nwgt=nwgt+ho_nnpdf
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'REF=',iref_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_nref
    ENDIF
    ! output plot files
    CALL plot_begin_pp_oniumX(nwgt,max_weight,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_oniumX

  SUBROUTINE plotout_pp_oniumX
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files
    CALL plot_end_pp_oniumX(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_oniumX

  SUBROUTINE outfun_pp_oniumX(www,ibody)
    ! ibody=1: (n+1)-body contribution
    ! ibody=2: n-body contribution (excluding Born)
    ! ibody=3: Born contribution
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ibody
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::wgts !,wgtden
    INTEGER::init=0
    SAVE init,wgts
    IF(www.EQ.0d0)RETURN
    IF(init.EQ.0)THEN
       max_weight=1+ho_nscale+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
       IF(ALLOCATED(wgts))THEN
          DEALLOCATE(wgts)
       ENDIF
       ALLOCATE(wgts(max_weight))
       init=1
    ENDIF
    nwgt=1
    wgts(1)=www
    IF(reweight_scale)THEN
       DO i=1,3
          DO j=1,3
             IF(i.EQ.1.AND.j.EQ.1)CYCLE
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecmu(i,j)
          ENDDO
       ENDDO
    ENDIF
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf+ho_npdf2
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecpdf(i)
       ENDDO
    ENDIF
    IF(reweight_npdf)THEN
       DO i=1,ho_nnpdf
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecnpdf(i)
       ENDDO
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecref(i)
       ENDDO
    ENDIF
    ! output plot file 
    CALL plot_fill_pp_oniumX(wgts,ibody)
    RETURN
  END SUBROUTINE outfun_pp_oniumX

  SUBROUTINE plot_begin_pp_oniumX(nwgt,max_weight0,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight0
    CHARACTER(len=*),DIMENSION(max_weight0),INTENT(IN)::weights_info
    INTEGER::i,j,kk,l
    CHARACTER(len=4),DIMENSION(12)::states=(/'1S01','1S08','3S11','3S18','3P01','3P11',&
         '3P21','3P08','3P18','3P28','1P11','1P18'/)
    CHARACTER(len=3),DIMENSION(2)::iord=(/'NLO',' LO'/)
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_oniumX: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    DO j=1,2
       DO i=1,12
          l=(i-1)*10+(j-1)*10*12
          CALL histogram_book(l+ 1,'total rate '//states(i)//' '//iord(j),&
               5,0.5d0,5.5d0,.FALSE.)
          CALL histogram_book(l+ 2,'y(onium)   '//states(i)//' '//iord(j),&
               100,-5d0,5d0,.TRUE.)
          CALL histogram_book(l+ 3,'Pt(onium)  '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+ 4,'Pt(onium) |y|<0.1 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+ 5,'Pt(onium) |y|<0.5 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+ 6,'Pt(onium) 2.0<y<2.5 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+ 7,'Pt(onium) 2.5<y<3.0 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+ 8,'Pt(onium) 3.0<y<3.5 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+ 9,'Pt(onium) 3.5<y<4.0 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
          CALL histogram_book(l+10,'Pt(onium) 4.0<y<4.5 '//states(i)//' '//iord(j),&
               100,0d0,25d0,.TRUE.)
       ENDDO
    ENDDO
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_oniumX

  SUBROUTINE plot_end_pp_oniumX(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_oniumX
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_oniumX
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_oniumX
       psfile="pp_oniumX.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_oniumX
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_oniumX
       rootfile="pp_oniumX.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_oniumX
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_oniumX
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_oniumX

  SUBROUTINE plot_fill_pp_oniumX(wgts,ibody)
    ! ibody=1: (n+1)-body contribution
    ! ibody=2: n-body contribution (excluding Born)
    ! ibody=3: Born contribution
    USE Helac_Global
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ibody
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INTEGER::i,j,kk,l
    REAL(KIND(1d0))::pt_onium,y_onium,var
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    IF(ibody.EQ.1)THEN
       pt_onium=DSQRT(onium_pmom(3,1)**2+onium_pmom(3,2)**2)
    ELSE
       pt_onium=1d-2
    ENDIF
    y_onium=rapidity(onium_pmom(3,1:4))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO j=1,2
       IF(j.EQ.2.AND.ibody.NE.3)CYCLE
       l=(onium_state-1)*10+(j-1)*10*12
       CALL HISTOGRAM_FILL(l+1,var,wgts)
       CALL HISTOGRAM_FILL(l+2,y_onium,wgts)
       CALL HISTOGRAM_FILL(l+3,pt_onium,wgts)
       IF(DABS(y_onium).LT.0.1d0)CALL HISTOGRAM_FILL(l+4,pt_onium,wgts)
       IF(DABS(y_onium).LT.0.5d0)CALL HISTOGRAM_FILL(l+5,pt_onium,wgts)
       IF(y_onium.GT.2.0d0.AND.y_onium.LT.2.5d0)CALL HISTOGRAM_FILL(l+6,pt_onium,wgts)
       IF(y_onium.GT.2.5d0.AND.y_onium.LT.3.0d0)CALL HISTOGRAM_FILL(l+7,pt_onium,wgts)
       IF(y_onium.GT.3.0d0.AND.y_onium.LT.3.5d0)CALL HISTOGRAM_FILL(l+8,pt_onium,wgts)
       IF(y_onium.GT.3.5d0.AND.y_onium.LT.4.0d0)CALL HISTOGRAM_FILL(l+9,pt_onium,wgts)
       IF(y_onium.GT.4.0d0.AND.y_onium.LT.4.5d0)CALL HISTOGRAM_FILL(l+10,pt_onium,wgts)
    ENDDO
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_oniumX

  SUBROUTINE GetPTOrdered(npt,pt_ordered)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::npt
    REAL(KIND(1d0)),DIMENSION(npt),INTENT(INOUT)::pt_ordered
    INTEGER::i,j
    REAL(KIND(1d0)),DIMENSION(npt)::pt_orig
    LOGICAL,DIMENSION(npt)::used
    REAL(KIND(1d0))::ptmax
    INTEGER::indexmax
    pt_orig(1:npt)=pt_ordered(1:npt)
    used(1:npt)=.FALSE.
    DO i=1,npt
       ptmax=-1d0
       indexmax=0
       DO j=1,npt
          IF(used(j))CYCLE
          IF(pt_orig(j).GT.ptmax)THEN
             ptmax=pt_orig(j)
             indexmax=j
          ENDIF
       ENDDO
       pt_ordered(i)=ptmax
       used(indexmax)=.TRUE.
    ENDDO
    RETURN
  END SUBROUTINE GetPTOrdered

  FUNCTION DeltaPh4(pmom1,pmom2)
    IMPLICIT NONE
    REAL(KIND(1d0))::DeltaPh4
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pmom1,pmom2 ! px,py,pz,E
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    DeltaPh4=ph4(pmom1(1),pmom1(2),pmom1(3)) ! ph4 is defined in Kinematic_Func.f90
    DeltaPh4=DeltaPh4-ph4(pmom2(1),pmom2(2),pmom2(3))
    IF(DeltaPh4.GT.pi)DeltaPh4=2d0*pi-DeltaPh4
    RETURN
  END FUNCTION DeltaPh4

  SUBROUTINE HwU_write_file_pp_oniumX
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0))::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//'pp_oniumX.HwU',&
         status='unknown')
    xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_oniumX

  SUBROUTINE open_topdrawer_file_pp_oniumX
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_oniumX.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_oniumX

  SUBROUTINE close_topdrawer_file_pp_oniumX
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_oniumX

  SUBROUTINE open_gnuplot_file_pp_oniumX
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_oniumX.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_oniumX

  SUBROUTINE close_gnuplot_file_pp_oniumX
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_oniumX

  SUBROUTINE open_root_file_pp_oniumX
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_oniumX.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_oniumX

  SUBROUTINE close_root_file_pp_oniumX
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_oniumX
END MODULE plot_pp_oniumX
