MODULE plot_pp_psiX_cb
  USE HELAC_Global
  USE pp_psiX_cb_global
  USE Kinetic_Func 
  USE OpticalGlauber_Geometry
  IMPLICIT NONE
  INTEGER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=79
  INTEGER::pp_ref=-1,pB_ref=-1,Ap_ref=-1
  INTEGER,PARAMETER::n_width=1,n_height=1
  INTEGER::n_centrality
  REAL(KIND(1d0)),DIMENSION(20,4)::centrality_bins
  ! it uses the factorised form of RAB (Eq.(4.9) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber)
  ! assume G(x) is a polynomial in x
  ! seen the defition of G(x) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber
  INTEGER,PARAMETER::igmaxA=4,igmaxB=4 ! the maximal power in G(x) is x**igmax
  ! G(x)=N*(gcoeff(0)+gcoeff(1)*x+gcoeff(2)*x**2+gcoeff(3)*x**3+...)
  ! N is to be determined !
  REAL(KIND(1d0)),DIMENSION(0:igmaxA)::gcoeffA=(/0d0,1d0,0d0,0d0,0d0/)
  REAL(KIND(1d0)),DIMENSION(0:igmaxB)::gcoeffB=(/0d0,1d0,0d0,0d0,0d0/)
  REAL(KIND(1d0))::NGNormA,NGNormB ! the normalisation factor N in G(x)
  ! A is the nuclei
  CHARACTER(len=7)::A_name="Au197  ",B_name="Au197  "
  REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TABhat_integ ! the integration of TABhat via bin by bin
  REAL(KIND(1d0)),DIMENSION(:,:,:),ALLOCATABLE::TABhatInG_integ ! the integration of TAhat**(1+i)*TBhat**(1+j) if (gcoeffA(i)=!=0 or i==1) and (gcoeffB(j)=!=0 or j==1)
  REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::TABhat0InG ! TAhat(0)**i*TBhat(0)**j if (gcoeffA(i)=!=0 or i==1) and (gcoeffB(j)=!=0 or j==1)
  LOGICAL::initial_centrality=.FALSE.
  SAVE
CONTAINS
  SUBROUTINE initplot_pp_psiX_cb
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(:),ALLOCATABLE::weights_info
    INTEGER::i
    IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
       IF(reweight_npdf)THEN
          ! it must be the nuclear PDF
          max_weight=1+ho_npdf+N_MCPDFs+ho_nref
       ELSEIF(reweight_pdf)THEN
          ! it must be the proton PDF
          max_weight=1+N_MCPDFs+ho_nnpdf+ho_nref
       ELSE
          WRITE(*,*)"ERROR: no reweighting for both PDF and nPDF but using Hessian to MC"
          STOP
       ENDIF
    ELSE
       max_weight=1+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
    ENDIF
    IF(ALLOCATED(weights_info))THEN
       DEALLOCATE(weights_info)
    ENDIF
    ALLOCATE(weights_info(max_weight))
    nwgt=1
    weights_info(nwgt)="central value  "
    IF(reweight_pdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             WRITE(weights_info(nwgt+i),'(a4,i10,a1)')&
                  'PDF=',900000000+i,' '
          ENDDO
          nwgt=nwgt+N_MCPDFs
       ELSE
          DO i=1,ho_npdf
             WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
                  'PDF=',ipdf_set_min-1+i,'   '
          ENDDO
          nwgt=nwgt+ho_npdf
          IF(ho_npdf2.GT.0)THEN
             DO i=1,ho_npdf2
                WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                     'PDF2=',ipdf2_set_min-1+i,'  '
             ENDDO
             nwgt=nwgt+ho_npdf2
          ENDIF
       ENDIF
    ENDIF
    IF(reweight_npdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             WRITE(weights_info(nwgt+i),'(a5,i9,a1)')&
                     'NPDF=',900000000+i,' '
          ENDDO
          nwgt=nwgt+N_MCPDFs
       ELSE
          IF(nPDF_id.EQ.1000)THEN
             DO i=1,ho_nnpdf
                WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                     'NPDF=',inpdf_set_min+i-1-1000+iPDFSUP1,'  '
             ENDDO
          ELSE
             DO i=1,ho_nnpdf
                WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                     'NPDF=',inpdf_set_min-1+i,'  '
             ENDDO
          ENDIF
          nwgt=nwgt+ho_nnpdf
       ENDIF
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'REF=',iref_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_nref
       IF(ho_nref.NE.3)THEN
          WRITE(*,*)"ERROR: the plot subroutine in plot_psiX_cb.f90 is only used for AB collisions with ho_nref"
          STOP
       ENDIF
       pp_ref=nwgt-2 ! to get the reference pp one
       pB_ref=nwgt-1 ! to get the reference pB one
       Ap_ref=nwgt   ! to get the reference Ap one
    ENDIF
    ! output plot files
    CALL plot_begin_pp_psiX_cb(nwgt,max_weight,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_psiX_cb

  SUBROUTINE plotout_pp_psiX_cb
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_psiX_cb(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_psiX_cb

  SUBROUTINE outfun_pp_psiX_cb(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::wgts
    !REAL(KIND(1d0)),DIMENSION(max_weight)::wgts,wgtden
    INTEGER::init=0
    SAVE init,wgts
    ! it is necessary for using Phegas
    IF(www.EQ.0d0)RETURN
    IF(init.EQ.0)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          IF(reweight_npdf)THEN
             max_weight=1+ho_npdf+N_MCPDFs+ho_nref
          ELSEIF(reweight_pdf)THEN
             max_weight=1+N_MCPDFs+ho_nnpdf+ho_nref
          ELSE
             WRITE(*,*)"ERROR: no reweighting for both PDF and nPDF but using Hessian to MC"
             STOP
          ENDIF
       ELSE
          max_weight=1+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
       ENDIF
       IF(ALLOCATED(wgts))THEN
          DEALLOCATE(wgts)
       ENDIF
       ALLOCATE(wgts(max_weight))
       init=1
    ENDIF
    nwgt=1
    wgts(1)=www
    IF(reweight_pdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecpdf(i)
          ENDDO
       ELSE
          DO i=1,ho_npdf+ho_npdf2
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecpdf(i)
          ENDDO
       ENDIF
    ENDIF
    IF(reweight_npdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecnpdf(i)
          ENDDO
       ELSE
          DO i=1,ho_nnpdf
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecnpdf(i)
          ENDDO
       ENDIF
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecref(i)
       ENDDO
    ENDIF
    ! output plot file 
    CALL plot_fill_pp_psiX_cb(wgts)
    RETURN
  END SUBROUTINE outfun_pp_psiX_cb

  SUBROUTINE plot_begin_pp_psiX_cb(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    CHARACTER(len=3)::cen_tag
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'


    IF(pp_ref.LT.0.OR.pB_ref.LT.0.OR.Ap_ref.LT.0)THEN
       WRITE(*,*)"ERROR: Do not have pp/pB/Ap reference(s)!"
       STOP
    ENDIF
    IF(.NOT.initial_centrality)THEN
       CALL InitialCentrality_AB
       initial_centrality=.TRUE.
    ENDIF
    
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    !CALL INIHIST
    !CALL INIHIST_3D
    nwgt_analysis=nwgt
    num_plots=79*(1+n_centrality)
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_psiX_cb: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    DO i=0,n_centrality
       WRITE(cen_tag,'(A1,I2)')'c',i
       l=79*i
       CALL histogram_book(l+ 1,'total rate  '//cen_tag,&
            5,0.5d0,5.5d0,.FALSE.)
       CALL histogram_book(l+ 2,'psi Pt '//cen_tag,&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 3,'psi yCM '//cen_tag,&
            100,-5d0,5d0,.TRUE.)
       ! LHCb fiducial
       ! arXiv:1308.6729
       CALL histogram_book(l+ 4,'psi Pt,1.5<yCM<4.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+ 5,'psi Pt,-5.0<yCM<-2.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+ 6,'psi Pt,1.5<yCM<2.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+ 7,'psi Pt,2.0<yCM<2.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+ 8,'psi Pt,2.5<yCM<3.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+ 9,'psi Pt,3.0<yCM<3.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+10,'psi Pt,3.5<yCM<4.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+11,'psi yCM, pT<14 '//cen_tag,&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+12,'psi Pt, 2.5<yCM<4.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+13,'psi Pt, -4.0<yCM<-2.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       ! end of LHCb
       ! ALICE fiducial 1
       ! arXiv:1308.6726
       CALL histogram_book(l+14,'psi yCM,-4.46<yCM<-2.96,pT<15 '//cen_tag,&
            6,-4.46d0,-2.96d0,.TRUE.)
       CALL histogram_book(l+15,'psi yCM,2.03<yCM<3.53,pT<15 '//cen_tag,&
            6,2.03d0,3.53d0,.TRUE.)
       CALL histogram_book(l+16,'psi yCM,2.96<yCM<3.53,pT<15 '//cen_tag,&
            57,2.96d0,3.53d0,.TRUE.)
       CALL histogram_book(l+17,'psi yCM,-3.53<yCM<-2.96,pT<15 '//cen_tag,&
            57,-3.53d0,-2.96d0,.TRUE.)
       CALL histogram_book(l+18,'psi Pt, 2.96<yCM<3.53 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+19,'psi Pt, -3.53<yCM<-2.96 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       ! end of ALICE 1
       ! ALICE fiducial 2
       ! arXiv:1503.07179
       CALL histogram_book(l+20,'psi Pt, -4.46<yCM<-2.96 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+21,'psi Pt, -1.37<yCM<0.43 '//cen_tag,&
            100,0d0,10d0,.TRUE.)
       CALL histogram_book(l+22,'psi Pt, 2.03<yCM<3.53 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+23,'psi yCM, -1.37<yCM<0.43 '//cen_tag,&
            18,-1.37d0,0.43d0,.TRUE.)
       ! end of ALICE 2
       ! PHENIX fiducial 1
       ! arXiv:1010.1246
       CALL histogram_book(l+24,'psi yCM, -2.5<yCM<2.5 '//cen_tag,&
            100,-2.5d0,2.5d0,.TRUE.)
       ! end of PHENIX 1
       ! PHENIX fiducial 2
       ! arXiv:1204.0777
       CALL histogram_book(l+25,'psi yCM, -2.5<yCM<2.5,pT>4 '//cen_tag,&
            100,-2.5d0,2.5d0,.TRUE.)
       CALL histogram_book(l+26,'psi Pt, -2.2<yCM<-1.2 '//cen_tag,&
            100,0d0,25d0,.TRUE.)
       CALL histogram_book(l+27,'psi Pt,-0.35<yCM<0.35 '//cen_tag,&
            100,0d0,25d0,.TRUE.)
       CALL histogram_book(l+28,'psi Pt,1.2<yCM<2.2 '//cen_tag,&
            100,0d0,25d0,.TRUE.)
       ! end of PHENIX 2
       ! ATLAS fiducial 1
       ! arXiv:1505.08141
       CALL histogram_book(l+29,'psi Pt, -1.94<yCM<0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+30,'psi Pt, 0<yCM<1.94 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+31,'psi yCM,8<pT<30 '//cen_tag,&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+32,'psi yCM,-2.87<yCM<-1.94,8<pT<30 '//cen_tag,&
            10,-2.87d0,-1.94d0,.TRUE.)
       CALL histogram_book(l+33,'psi yCM,-1.94<yCM<-1.3,8<pT<30 '//cen_tag,&
            10,-1.94d0,-1.3d0,.TRUE.)
       CALL histogram_book(l+34,'psi yCM,-1.3<yCM<1.3,8<pT<30 '//cen_tag,&
            4,-1.3d0,1.3d0,.TRUE.)
       CALL histogram_book(l+35,'psi yCM,1.3<yCM<1.94,8<pT<30 '//cen_tag,&
            10,1.3d0,1.94d0,.TRUE.)
       CALL histogram_book(l+36,'psi yCM,0<yCM<1.94,8<pT<30 '//cen_tag,&
          10,0d0,1.94d0,.TRUE.)
       CALL histogram_book(l+37,'psi yCM,-1.94<yCM<0,8<pT<30 '//cen_tag,&
            10,-1.94d0,0d0,.TRUE.)
       CALL histogram_book(l+38,'psi Pt,0<yCM<1.94 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+39,'psi Pt,-1.94<yCM<0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       ! end of ATLAS 1
       ! ATLAS fiducial 2
       ! ATLAS-CONF-2015-023
       CALL histogram_book(l+40,'psi Pt, -1.5<yCM<1.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+41,'psi yCM, -1.5<yCM<1.5, 10<pT<30 '//cen_tag,&
            4,-1.5d0,1.5d0,.TRUE.)
       ! end of ATLAS 2
       ! STAR fiducial 1
       ! Request from Lijuan Ruan (BNL)
       CALL histogram_book(l+42,'psi Pt, -1.0<yCM<1.0 '//cen_tag,&
            100,0d0,25d0,.TRUE.)
       ! end of STAR fiducail 1
       ! LHCb fiducial 2
       ! Request from Michael Winn
       CALL histogram_book(l+43,'psi Pt, -5.0<yCM<-4.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+44,'psi Pt, -4.5<yCM<-4.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+45,'psi Pt, -4.0<yCM<-3.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+46,'psi Pt, -3.5<yCM<-3.0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+47,'psi Pt, -3.0<yCM<-2.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       ! end of LHCb fiducial 2
       ! CMS fiducial 1, arXiv:1702.01462
       CALL histogram_book(l+48,'psi Pt,1.5<yCM<1.93 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+49,'psi Pt,0.9<yCM<1.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+50,'psi Pt,0<yCM<0.9 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+51,'psi Pt,-0.9<yCM<0 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+52,'psi Pt,-1.5<yCM<-0.9 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+53,'psi Pt,-1.93<yCM<-1.5 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+54,'psi Pt,-2.4<yCM<-1.93 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+55,'psi Pt,-2.87<yCM<-2.4 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+56,'psi yCM,6.5<Pt<10 '//cen_tag,&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+57,'psi yCM,10<Pt<30 '//cen_tag,&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+58,'psi yCM, -2.87<yCM<-1.93,6.5<Pt<10 '//cen_tag,&
            2,-2.87d0,-1.93d0,.TRUE.)
       CALL histogram_book(l+59,'psi yCM, -2.87<yCM<-1.93,10<Pt<30 '//cen_tag,&
            2,-2.87d0,-1.93d0,.TRUE.)
       CALL histogram_book(l+60,'psi yCM, -1.93<yCM<-1.5,6.5<Pt<10 '//cen_tag,&
            2,-1.93d0,-1.5d0,.TRUE.)
       CALL histogram_book(l+61,'psi yCM, -1.93<yCM<-1.5,10<Pt<30 '//cen_tag,&
            2,-1.93d0,-1.5d0,.TRUE.)
       CALL histogram_book(l+62,'psi yCM, -1.5<yCM<1.5,6.5<Pt<10 '//cen_tag,&
            10,-1.5d0,1.5d0,.TRUE.)
       CALL histogram_book(l+63,'psi yCM, -1.5<yCM<1.5,10<Pt<30 '//cen_tag,&
            10,-1.5d0,1.5d0,.TRUE.)
       CALL histogram_book(l+64,'psi yCM, 1.5<yCM<1.93,6.5<Pt<10 '//cen_tag,&
            100,1.5d0,1.93d0,.TRUE.)
       CALL histogram_book(l+65,'psi yCM, 1.5<yCM<1.93,10<Pt<30 '//cen_tag,&
            100,1.5d0,1.93d0,.TRUE.)
       ! end of CMS fiducial 1
       ! begin of AFTER, 1
       CALL histogram_book(l+66,'psi Pt, -2.8<yCM<0.2 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+67,'psi Pt, -0.2<yCM<2.8 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+68,'psi Pt, -2.8<yCM<-2.3 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+69,'psi Pt, -2.3<yCM<-1.8 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+70,'psi Pt, -1.8<yCM<-1.3 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+71,'psi Pt, -1.3<yCM<-0.8 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+72,'psi Pt, -0.8<yCM<-0.3 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+73,'psi Pt, -0.3<yCM<0.2 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+74,'psi Pt, -0.2<yCM<0.3 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+75,'psi Pt, 0.3<yCM<0.8 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+76,'psi Pt, 0.8<yCM<1.3 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+77,'psi Pt, 1.3<yCM<1.8 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+78,'psi Pt, 1.8<yCM<2.3 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
       CALL histogram_book(l+79,'psi Pt, 2.3<yCM<2.8 '//cen_tag,&
            100,0d0,50d0,.TRUE.)
    enddo
       ! end of AFTER, 1
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_psiX_cb

  SUBROUTINE plot_end_pp_psiX_cb(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_psiX_cb
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_psiX_cb
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_psiX_cb
       psfile="pp_psiX_crystalball.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_psiX_cb
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_psiX_cb
       rootfile="pp_psiX_crystalball.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_psiX_cb
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_psiX_cb(xnorm)
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_psiX_cb

  SUBROUTINE plot_fill_pp_psiX_cb(wgts)
    USE Helac_Global
    IMPLICIT NONE
    !INCLUDE "pp_psipsi_dps.inc"
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0)),DIMENSION(SIZE(wgts))::wgts_new
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2,temp
    INTEGER::i,kk,jj,l,mm
    REAL(KIND(1d0))::www,pt_psi,y_psi
    REAL(KIND(1d0))::ppwww,pBwww,Apwww,ABwww,numerator
    IF(NDecayChains.GT.0)THEN
       CALL plot_fill_Decay_pp_psiX_cb(wgts)
       RETURN
    ENDIF
    IF(wgts(1).EQ.0d0)RETURN
    ppwww=wgts(pp_ref)
    pBwww=wgts(pB_ref)
    Apwww=wgts(Ap_ref)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    !pini(1:4)=cb_hadron_pmom(1,1:4)+cb_hadron_pmom(2,1:4) ! (px,py,pz,E)
    !yftot=rapidity(pini)
    y_psi=rapidity(cb_hadron_pmom(3,1:4))
    pt_psi=DSQRT(cb_hadron_pmom(3,1)**2+cb_hadron_pmom(3,2)**2)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=0,n_centrality
       l=79*i
       IF(i.EQ.0)THEN
          wgts_new(1:SIZE(wgts))=wgts(1:SIZE(wgts))
       ELSE
          DO kk=1,SIZE(wgts)
             IF(kk.EQ.pp_ref.OR.kk.EQ.pB_ref.OR.kk.EQ.Ap_ref)THEN
                wgts_new(kk)=wgts(kk)
             ELSE
                ! it is int(TABhat(b) dsigmaNN * \cal{R}_{AB} * 2*pi*b*db) bin by bin in centrality
                ! TABhat(b)*\cal{R}_{AB} = int{TAhat*\cal{R}_A*TBhat*\cal{R}_B}
                ! \cal{R}_A = 1+(R_A-1)*G(TAhat(b)/TAhat(0)=x)
                !           = 1+(R_A-1)*NA*(gcoeffA(0)+gcoeffA(1)*x+...)
                ABwww=wgts(kk)
                numerator=0d0 ! the first term in \cal{R}_A*\cal{R}_B
                ! the first term in \cal{R}_A(=1) times second term in \cal{R}_B
                DO jj=0,igmaxA
                   DO mm=0,igmaxB
                      IF(jj.EQ.0.AND.mm.EQ.0)THEN
                         temp=ppwww*(1d0-1d0*NGNormA*gcoeffA(jj))&
                              *(1d0-1d0*NGNormB*gcoeffB(mm))
                         temp=temp+Apwww*NGNormA*gcoeffA(jj)*(1d0-1d0*NGNormB*gcoeffB(mm))
                         temp=temp+pBwww*(1d0-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                         temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                         numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                      ELSEIF(jj.EQ.0.AND.mm.GT.0)THEN
                         ! 1+(R_A-1)*NA*gcoeffA(0)=(1-NA*gcoeffA(0))+R_A*NA*gcoeffA(0) times
                         ! (R_B-1)*NB*gcoeffB(mm)*TBhat**m/TBhat0**m
                         IF(TABhatInG_integ(jj,mm,i).NE.0d0)THEN
                            temp=ppwww*(1d0-1d0*NGNormA*gcoeffA(jj))*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+Apwww*NGNormA*gcoeffA(jj)*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+pBwww*(1d0-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                            temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                            numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                         ENDIF
                      ELSEIF(jj.GT.0.AND.mm.EQ.0)THEN
                         ! (R_A-1)*NA*gcoeffA(jj)*TAhat**j/TAhat0**j times
                         ! 1+(R_B-1)*NB*gcoeffB(0)=(1-NB*gcoeffB(0))+R_B*NB*gcoeffB(0)
                         IF(TABhatInG_integ(jj,mm,i).NE.0d0)THEN
                            temp=ppwww*(-1d0*NGNormA*gcoeffA(jj))*(1d0-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+Apwww*NGNormA*gcoeffA(jj)*(1d0-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+pBwww*(-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                            temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                            numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                         ENDIF
                      ELSE
                         IF(gcoeffA(jj).NE.0d0.AND.gcoeffB(mm).NE.0d0)THEN
                            ! (R_A-1)*NA*gcoeffA(jj)*TAhat**j/TAhat0**j times
                            ! (R_B-1)*NB*gcoeffB(mm)*TBhat**m/TBhat0**m
                            temp=ppwww*(-1d0*NGNormA*gcoeffA(jj))*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+Apwww*NGNormA*gcoeffA(jj)*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+pBwww*(-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                            temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                            numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                         ENDIF
                      ENDIF
                   ENDDO
                ENDDO
                wgts_new(kk)=numerator
             ENDIF
          ENDDO
       ENDIF
       CALL HISTOGRAM_fill(l+1,var,wgts_new)
       CALL HISTOGRAM_fill(l+2,pt_psi,wgts_new)
       CALL HISTOGRAM_fill(l+3,y_psi,wgts_new)
       IF(y_psi.GT.1.5d0.AND.y_psi.LT.4d0)THEN
          CALL HISTOGRAM_fill(l+4,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-5d0.AND.y_psi.LT.-2.5d0)THEN
          CALL HISTOGRAM_fill(l+5,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.5d0.AND.y_psi.LT.2d0)THEN
          CALL HISTOGRAM_fill(l+6,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2d0.AND.y_psi.LT.2.5d0)THEN
          CALL HISTOGRAM_fill(l+7,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.5d0.AND.y_psi.LT.3d0)THEN
          CALL HISTOGRAM_fill(l+8,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.3d0.AND.y_psi.LT.3.5d0)THEN
          CALL HISTOGRAM_fill(l+9,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.3.5d0.AND.y_psi.LT.4d0)THEN
          CALL HISTOGRAM_fill(l+10,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.LT.14d0)THEN
          CALL HISTOGRAM_fill(l+11,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.5d0.AND.y_psi.LT.4d0)THEN
          CALL HISTOGRAM_fill(l+12,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4d0.AND.y_psi.LT.-2.5d0)THEN
          CALL HISTOGRAM_fill(l+13,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.LT.15d0)THEN
          IF(y_psi.GT.-4.46d0.AND.y_psi.LT.-2.96d0)THEN
             CALL HISTOGRAM_fill(l+14,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.2.03d0.AND.y_psi.LT.3.53d0)THEN
             CALL HISTOGRAM_fill(l+15,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.2.96d0.AND.y_psi.LT.3.53d0)THEN
             CALL HISTOGRAM_fill(l+16,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-3.53d0.AND.y_psi.LT.-2.96d0)THEN
             CALL HISTOGRAM_fill(l+17,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.2.96d0.AND.y_psi.LT.3.53d0)THEN
          CALL HISTOGRAM_fill(l+18,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-3.53d0.AND.y_psi.LT.-2.96d0)THEN
          CALL HISTOGRAM_fill(l+19,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4.46d0.AND.y_psi.LT.-2.96d0)THEN
          CALL HISTOGRAM_fill(l+20,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.37d0.AND.y_psi.LT.0.43d0)THEN
          CALL HISTOGRAM_fill(l+21,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.03d0.AND.y_psi.LT.3.53d0)THEN
          CALL HISTOGRAM_fill(l+22,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.37d0.AND.y_psi.LT.0.43d0)THEN
          CALL HISTOGRAM_fill(l+23,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.5d0.AND.y_psi.LT.2.5d0)THEN
          CALL HISTOGRAM_fill(l+24,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.5d0.AND.y_psi.LT.2.5d0.AND.pt_psi.GT.4d0)THEN
          CALL HISTOGRAM_fill(l+25,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.2d0.AND.y_psi.LT.-1.2d0)THEN
          CALL HISTOGRAM_fill(l+26,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.35d0.AND.y_psi.LT.0.35d0)THEN
          CALL HISTOGRAM_fill(l+27,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.2d0.AND.y_psi.LT.2.2d0)THEN
          CALL HISTOGRAM_fill(l+28,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.94d0.AND.y_psi.LT.0d0)THEN
          CALL HISTOGRAM_fill(l+29,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0d0.AND.y_psi.LT.1.94d0)THEN
          CALL HISTOGRAM_fill(l+30,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.8d0.AND.pt_psi.LT.30d0)THEN
          CALL HISTOGRAM_fill(l+31,y_psi,wgts_new)
          IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-1.94d0)THEN
             CALL HISTOGRAM_fill(l+32,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.94d0.AND.y_psi.LT.-1.3d0)THEN
             CALL HISTOGRAM_fill(l+33,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.3d0.AND.y_psi.LT.1.3d0)THEN
             CALL HISTOGRAM_fill(l+34,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.1.3d0.AND.y_psi.LT.1.94d0)THEN
             CALL HISTOGRAM_fill(l+35,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.0d0.AND.y_psi.LT.1.94d0)THEN
             CALL HISTOGRAM_fill(l+36,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.94d0.AND.y_psi.LT.0d0)THEN
             CALL HISTOGRAM_fill(l+37,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.0d0.AND.y_psi.LT.1.94d0)THEN
          CALL HISTOGRAM_fill(l+38,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.94d0.AND.y_psi.LT.0d0)THEN
          CALL HISTOGRAM_fill(l+39,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.5d0.AND.y_psi.LT.1.5d0)THEN
          CALL HISTOGRAM_fill(l+40,pt_psi,wgts_new)
          IF(pt_psi.GT.10d0.AND.pt_psi.LT.30d0)THEN
             CALL HISTOGRAM_fill(l+41,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.-1d0.AND.y_psi.LT.1d0)THEN
          CALL HISTOGRAM_fill(l+42,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-5.0d0.AND.y_psi.LT.-4.5d0)THEN
          CALL HISTOGRAM_fill(l+43,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4.5d0.AND.y_psi.LT.-4.0d0)THEN
          CALL HISTOGRAM_fill(l+44,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4.0d0.AND.y_psi.LT.-3.5d0)THEN
          CALL HISTOGRAM_fill(l+45,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-3.5d0.AND.y_psi.LT.-3.0d0)THEN
          CALL HISTOGRAM_fill(l+46,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-3.0d0.AND.y_psi.LT.-2.5d0)THEN
          CALL HISTOGRAM_fill(l+47,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.5d0.AND.y_psi.LT.1.93d0)THEN
          CALL HISTOGRAM_fill(l+48,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0.9d0.AND.y_psi.LT.1.5d0)THEN
          CALL HISTOGRAM_fill(l+49,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0d0.AND.y_psi.LT.0.9d0)THEN
          CALL HISTOGRAM_fill(l+50,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.9d0.AND.y_psi.LT.0d0)THEN
          CALL HISTOGRAM_fill(l+51,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.5d0.AND.y_psi.LT.-0.9d0)THEN
          CALL HISTOGRAM_fill(l+52,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.93d0.AND.y_psi.LT.-1.5d0)THEN
          CALL HISTOGRAM_fill(l+53,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.4d0.AND.y_psi.LT.-1.93d0)THEN
          CALL HISTOGRAM_fill(l+54,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-2.4d0)THEN
          CALL HISTOGRAM_fill(l+55,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.6.5d0.AND.pt_psi.LT.10d0)THEN
          CALL HISTOGRAM_fill(l+56,y_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.10d0.AND.pt_psi.LT.30d0)THEN
          CALL HISTOGRAM_fill(l+57,y_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.6.5d0.AND.pt_psi.LT.10d0)THEN
          IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-1.93d0)THEN
             CALL HISTOGRAM_fill(l+58,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.93d0.AND.y_psi.LT.-1.5d0)THEN
             CALL HISTOGRAM_fill(l+60,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.5d0.AND.y_psi.LT.1.5d0)THEN
             CALL HISTOGRAM_fill(l+62,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.1.5d0.AND.y_psi.LT.1.93d0)THEN
             CALL HISTOGRAM_fill(l+64,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(pt_psi.GT.10d0.AND.pt_psi.LT.30d0)THEN
          IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-1.93d0)THEN
             CALL HISTOGRAM_fill(l+59,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.93d0.AND.y_psi.LT.-1.5d0)THEN
             CALL HISTOGRAM_fill(l+61,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.5d0.AND.y_psi.LT.1.5d0)THEN
             CALL HISTOGRAM_fill(l+63,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.1.5d0.AND.y_psi.LT.1.93d0)THEN
             CALL HISTOGRAM_fill(l+65,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.-2.8d0.AND.y_psi.LE.0.2d0)THEN
          CALL HISTOGRAM_fill(l+66,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.2d0.AND.y_psi.LE.2.8d0)THEN
          CALL HISTOGRAM_fill(l+67,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.8d0.AND.y_psi.LE.-2.3d0)THEN
          CALL HISTOGRAM_fill(l+68,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.3d0.AND.y_psi.LE.-1.8d0)THEN
          CALL HISTOGRAM_fill(l+69,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.8d0.AND.y_psi.LE.-1.3d0)THEN
          CALL HISTOGRAM_fill(l+70,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.3d0.AND.y_psi.LE.-0.8d0)THEN
          CALL HISTOGRAM_fill(l+71,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.8d0.AND.y_psi.LE.-0.3d0)THEN
          CALL HISTOGRAM_fill(l+72,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.3d0.AND.y_psi.LE.0.2d0)THEN
          CALL HISTOGRAM_fill(l+73,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.2d0.AND.y_psi.LE.0.3d0)THEN
          CALL HISTOGRAM_fill(l+74,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0.3d0.AND.y_psi.LE.0.8d0)THEN
          CALL HISTOGRAM_fill(l+75,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0.8d0.AND.y_psi.LE.1.3d0)THEN
          CALL HISTOGRAM_fill(l+76,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.3d0.AND.y_psi.LE.1.8d0)THEN
          CALL HISTOGRAM_fill(l+77,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.8d0.AND.y_psi.LE.2.3d0)THEN
          CALL HISTOGRAM_fill(l+78,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.3d0.AND.y_psi.LE.2.8d0)THEN
          CALL HISTOGRAM_fill(l+79,pt_psi,wgts_new)
       ENDIF
    ENDDO

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_psiX_cb

  SUBROUTINE HwU_write_file_pp_psiX_cb(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,FILE=TRIM(output_dir)//'pp_psiX_crystalball.HwU',&
         status='unknown')
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_psiX_cb

  SUBROUTINE open_topdrawer_file_pp_psiX_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_psiX_crystalball.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_psiX_cb

  SUBROUTINE close_topdrawer_file_pp_psiX_cb
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_psiX_cb

  SUBROUTINE open_gnuplot_file_pp_psiX_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_psiX_crystalball.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_psiX_cb

  SUBROUTINE close_gnuplot_file_pp_psiX_cb
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_psiX_cb

  SUBROUTINE open_root_file_pp_psiX_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_psiX_crystalball.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_psiX_cb

  SUBROUTINE close_root_file_pp_psiX_cb
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_psiX_cb

  SUBROUTINE plot_fill_Decay_pp_psiX_cb(wgts)
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0)),DIMENSION(SIZE(wgts))::wgts_new
    INCLUDE "stdhep_pp_psiX_cb.inc"
    REAL(KIND(1d0))::var,Q,www,y_psi,pt_psi,temp
    INTEGER::i,m,l,kk,jj,mm
    REAL(KIND(1d0)),DIMENSION(1,4)::PJpsi
    REAL(KIND(1d0)),DIMENSION(2,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    INTEGER,DIMENSION(2)::lep2jpsi
    INTEGER,DIMENSION(1)::newmoth
    INTEGER::i_jpsi,i_lep,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0))::ppwww,Apwww,pBwww,ABwww,numerator
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    IF(wgts(1).EQ.0d0)RETURN
    ppwww=wgts(pp_ref)
    pBwww=wgts(pB_ref)
    Apwww=wgts(Ap_ref)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!                                               
    !                                                                                                   
    ! DEFINE OBSERVABLES                                                                                
    !                                                                                                   
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton 
    i_jpsi=0
    i_lep=0
    lep2jpsi(1:2)=0
    newmoth(1:1)=0
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
       IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
       i_lep=i_lep+1
       Plep(i_lep,1:4)=PHEP(1:4,i)
       m=JMOHEP(1,i)
       definemoth=0
       DO kk=1,i_jpsi
          IF(newmoth(kk).EQ.m)THEN
             definemoth=kk
             EXIT
          END IF
       ENDDO
       IF(definemoth.NE.0)THEN
          ! the mother has been defined                                                                 
          lep2jpsi(i_lep)=definemoth
       ELSE
          ! create the new mother
          i_jpsi=i_jpsi+1
          PJpsi(i_jpsi,1:4)=PHEP(1:4,m)
          newmoth(i_jpsi)=m
          lep2jpsi(i_lep)=i_jpsi
          IF(i_lep.EQ.2)EXIT
       ENDIF
    ENDDO
    y_psi=rapidity(PJpsi(1,1:4))
    pt_psi=PJpsi(1,1)**2+PJpsi(1,2)**2
    pt_psi=DSQRT(DABS(pt_psi))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!                                          
    !                                                                                              
    ! START TO FILL HISTOGRAM                                                                      
    !                                                                                              
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=0,n_centrality
       IF(i.EQ.0)THEN
          wgts_new(1:SIZE(wgts))=wgts(1:SIZE(wgts))
       ELSE
          DO kk=1,SIZE(wgts)
             IF(kk.EQ.pp_ref.OR.kk.EQ.pB_ref.OR.kk.EQ.Ap_ref)THEN
                wgts_new(kk)=wgts(kk)
             ELSE
                ! it is int(TABhat dsigmaNN * \cal{R}_{AB} * 2*pi*b*db) bin by bin in centrality
                ! TABhat(b)*\cal{R}_{AB} = int{TAhat*\cal{R}_A*TBhat*\cal{R}_B}
                ! \cal{R}_A = 1+(R_A-1)*G(TAhat(b)/TAhat(0)=x)
                !           = 1+(R_A-1)*NA*(gcoeffA(0)+gcoeffA(1)*x+...)
                ABwww=wgts(kk)
                numerator=0d0
                DO jj=0,igmaxA
                   DO mm=0,igmaxB
                      IF(jj.EQ.0.AND.mm.EQ.0)THEN
                         temp=ppwww*(1d0-1d0*NGNormA*gcoeffA(jj))&
                              *(1d0-1d0*NGNormB*gcoeffB(mm))
                         temp=temp+Apwww*NGNormA*gcoeffA(jj)*(1d0-1d0*NGNormB*gcoeffB(mm))
                         temp=temp+pBwww*(1d0-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                         temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                         numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                      ELSEIF(jj.EQ.0.AND.mm.GT.0)THEN
                         ! 1+(R_A-1)*NA*gcoeffA(0)=(1-NA*gcoeffA(0))+R_A*NA*gcoeffA(0) times
                         ! (R_B-1)*NB*gcoeffB(mm)*TBhat**m/TBhat0**m
                         IF(TABhatInG_integ(jj,mm,i).NE.0d0)THEN
                            temp=ppwww*(1d0-1d0*NGNormA*gcoeffA(jj))*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+Apwww*NGNormA*gcoeffA(jj)*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+pBwww*(1d0-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                            temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                            numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                         ENDIF
                      ELSEIF(jj.GT.0.AND.mm.EQ.0)THEN
                         ! (R_A-1)*NA*gcoeffA(jj)*TAhat**j/TAhat0**j times
                         ! 1+(R_B-1)*NB*gcoeffB(0)=(1-NB*gcoeffB(0))+R_B*NB*gcoeffB(0)
                         IF(TABhatInG_integ(jj,mm,i).NE.0d0)THEN
                            temp=ppwww*(-1d0*NGNormA*gcoeffA(jj))*(1d0-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+Apwww*NGNormA*gcoeffA(jj)*(1d0-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+pBwww*(-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                            temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                            numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                         ENDIF
                      ELSE
                         IF(gcoeffA(jj).NE.0d0.AND.gcoeffB(mm).NE.0d0)THEN
                            ! (R_A-1)*NA*gcoeffA(jj)*TAhat**j/TAhat0**j times
                            ! (R_B-1)*NB*gcoeffB(mm)*TBhat**m/TBhat0**m
                            temp=ppwww*(-1d0*NGNormA*gcoeffA(jj))*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+Apwww*NGNormA*gcoeffA(jj)*(-1d0*NGNormB*gcoeffB(mm))
                            temp=temp+pBwww*(-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)
                            temp=temp+ABwww*NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)
                            numerator=numerator+temp*TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                         ENDIF
                      ENDIF
                   ENDDO
                ENDDO
                wgts_new(kk)=numerator
             ENDIF
          ENDDO
       ENDIF
       l=i*79
       CALL HISTOGRAM_fill(l+1,var,wgts_new)
       CALL HISTOGRAM_fill(l+2,pt_psi,wgts_new)
       CALL HISTOGRAM_fill(l+3,y_psi,wgts_new)
       IF(y_psi.GT.1.5d0.AND.y_psi.LT.4d0)THEN
          CALL HISTOGRAM_fill(l+4,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-5d0.AND.y_psi.LT.-2.5d0)THEN
          CALL HISTOGRAM_fill(l+5,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.5d0.AND.y_psi.LT.2d0)THEN
          CALL HISTOGRAM_fill(l+6,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2d0.AND.y_psi.LT.2.5d0)THEN
          CALL HISTOGRAM_fill(l+7,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.5d0.AND.y_psi.LT.3d0)THEN
          CALL HISTOGRAM_fill(l+8,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.3d0.AND.y_psi.LT.3.5d0)THEN
          CALL HISTOGRAM_fill(l+9,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.3.5d0.AND.y_psi.LT.4d0)THEN
          CALL HISTOGRAM_fill(l+10,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.LT.14d0)THEN
          CALL HISTOGRAM_fill(l+11,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.5d0.AND.y_psi.LT.4d0)THEN
          CALL HISTOGRAM_fill(l+12,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4d0.AND.y_psi.LT.-2.5d0)THEN
          CALL HISTOGRAM_fill(l+13,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.LT.15d0)THEN
          IF(y_psi.GT.-4.46d0.AND.y_psi.LT.-2.96d0)THEN
             CALL HISTOGRAM_fill(l+14,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.2.03d0.AND.y_psi.LT.3.53d0)THEN
             CALL HISTOGRAM_fill(l+15,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.2.96d0.AND.y_psi.LT.3.53d0)THEN
             CALL HISTOGRAM_fill(l+16,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-3.53d0.AND.y_psi.LT.-2.96d0)THEN
             CALL HISTOGRAM_fill(l+17,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.2.96d0.AND.y_psi.LT.3.53d0)THEN
          CALL HISTOGRAM_fill(l+18,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-3.53d0.AND.y_psi.LT.-2.96d0)THEN
          CALL HISTOGRAM_fill(l+19,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4.46d0.AND.y_psi.LT.-2.96d0)THEN
          CALL HISTOGRAM_fill(l+20,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.37d0.AND.y_psi.LT.0.43d0)THEN
          CALL HISTOGRAM_fill(l+21,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.03d0.AND.y_psi.LT.3.53d0)THEN
          CALL HISTOGRAM_fill(l+22,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.37d0.AND.y_psi.LT.0.43d0)THEN
          CALL HISTOGRAM_fill(l+23,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.5d0.AND.y_psi.LT.2.5d0)THEN
          CALL HISTOGRAM_fill(l+24,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.5d0.AND.y_psi.LT.2.5d0.AND.pt_psi.GT.4d0)THEN
          CALL HISTOGRAM_fill(l+25,y_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.2d0.AND.y_psi.LT.-1.2d0)THEN
          CALL HISTOGRAM_fill(l+26,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.35d0.AND.y_psi.LT.0.35d0)THEN
          CALL HISTOGRAM_fill(l+27,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.2d0.AND.y_psi.LT.2.2d0)THEN
          CALL HISTOGRAM_fill(l+28,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.94d0.AND.y_psi.LT.0d0)THEN
          CALL HISTOGRAM_fill(l+29,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0d0.AND.y_psi.LT.1.94d0)THEN
          CALL HISTOGRAM_fill(l+30,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.8d0.AND.pt_psi.LT.30d0)THEN
          CALL HISTOGRAM_fill(l+31,y_psi,wgts_new)
          IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-1.94d0)THEN
             CALL HISTOGRAM_fill(l+32,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.94d0.AND.y_psi.LT.-1.3d0)THEN
             CALL HISTOGRAM_fill(l+33,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.3d0.AND.y_psi.LT.1.3d0)THEN
             CALL HISTOGRAM_fill(l+34,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.1.3d0.AND.y_psi.LT.1.94d0)THEN
             CALL HISTOGRAM_fill(l+35,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.0d0.AND.y_psi.LT.1.94d0)THEN
             CALL HISTOGRAM_fill(l+36,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.94d0.AND.y_psi.LT.0d0)THEN
             CALL HISTOGRAM_fill(l+37,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.0d0.AND.y_psi.LT.1.94d0)THEN
          CALL HISTOGRAM_fill(l+38,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.94d0.AND.y_psi.LT.0d0)THEN
          CALL HISTOGRAM_fill(l+39,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.5d0.AND.y_psi.LT.1.5d0)THEN
          CALL HISTOGRAM_fill(l+40,pt_psi,wgts_new)
          IF(pt_psi.GT.10d0.AND.pt_psi.LT.30d0)THEN
             CALL HISTOGRAM_fill(l+41,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.-1d0.AND.y_psi.LT.1d0)THEN
          CALL HISTOGRAM_fill(l+42,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-5.0d0.AND.y_psi.LT.-4.5d0)THEN
          CALL HISTOGRAM_fill(l+43,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4.5d0.AND.y_psi.LT.-4.0d0)THEN
          CALL HISTOGRAM_fill(l+44,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-4.0d0.AND.y_psi.LT.-3.5d0)THEN
          CALL HISTOGRAM_fill(l+45,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-3.5d0.AND.y_psi.LT.-3.0d0)THEN
          CALL HISTOGRAM_fill(l+46,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-3.0d0.AND.y_psi.LT.-2.5d0)THEN
          CALL HISTOGRAM_fill(l+47,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.5d0.AND.y_psi.LT.1.93d0)THEN
          CALL HISTOGRAM_fill(l+48,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0.9d0.AND.y_psi.LT.1.5d0)THEN
          CALL HISTOGRAM_fill(l+49,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0d0.AND.y_psi.LT.0.9d0)THEN
          CALL HISTOGRAM_fill(l+50,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.9d0.AND.y_psi.LT.0d0)THEN
          CALL HISTOGRAM_fill(l+51,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.5d0.AND.y_psi.LT.-0.9d0)THEN
          CALL HISTOGRAM_fill(l+52,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.93d0.AND.y_psi.LT.-1.5d0)THEN
          CALL HISTOGRAM_fill(l+53,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.4d0.AND.y_psi.LT.-1.93d0)THEN
          CALL HISTOGRAM_fill(l+54,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-2.4d0)THEN
          CALL HISTOGRAM_fill(l+55,pt_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.6.5d0.AND.pt_psi.LT.10d0)THEN
          CALL HISTOGRAM_fill(l+56,y_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.10d0.AND.pt_psi.LT.30d0)THEN
          CALL HISTOGRAM_fill(l+57,y_psi,wgts_new)
       ENDIF
       IF(pt_psi.GT.6.5d0.AND.pt_psi.LT.10d0)THEN
          IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-1.93d0)THEN
             CALL HISTOGRAM_fill(l+58,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.93d0.AND.y_psi.LT.-1.5d0)THEN
             CALL HISTOGRAM_fill(l+60,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.5d0.AND.y_psi.LT.1.5d0)THEN
             CALL HISTOGRAM_fill(l+62,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.1.5d0.AND.y_psi.LT.1.93d0)THEN
             CALL HISTOGRAM_fill(l+64,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(pt_psi.GT.10d0.AND.pt_psi.LT.30d0)THEN
          IF(y_psi.GT.-2.87d0.AND.y_psi.LT.-1.93d0)THEN
             CALL HISTOGRAM_fill(l+59,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.93d0.AND.y_psi.LT.-1.5d0)THEN
             CALL HISTOGRAM_fill(l+61,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.-1.5d0.AND.y_psi.LT.1.5d0)THEN
             CALL HISTOGRAM_fill(l+63,y_psi,wgts_new)
          ENDIF
          IF(y_psi.GT.1.5d0.AND.y_psi.LT.1.93d0)THEN
             CALL HISTOGRAM_fill(l+65,y_psi,wgts_new)
          ENDIF
       ENDIF
       IF(y_psi.GT.-2.8d0.AND.y_psi.LE.0.2d0)THEN
          CALL HISTOGRAM_fill(l+66,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.2d0.AND.y_psi.LE.2.8d0)THEN
          CALL HISTOGRAM_fill(l+67,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.8d0.AND.y_psi.LE.-2.3d0)THEN
          CALL HISTOGRAM_fill(l+68,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-2.3d0.AND.y_psi.LE.-1.8d0)THEN
          CALL HISTOGRAM_fill(l+69,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.8d0.AND.y_psi.LE.-1.3d0)THEN
          CALL HISTOGRAM_fill(l+70,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-1.3d0.AND.y_psi.LE.-0.8d0)THEN
          CALL HISTOGRAM_fill(l+71,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.8d0.AND.y_psi.LE.-0.3d0)THEN
          CALL HISTOGRAM_fill(l+72,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.3d0.AND.y_psi.LE.0.2d0)THEN
          CALL HISTOGRAM_fill(l+73,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.-0.2d0.AND.y_psi.LE.0.3d0)THEN
          CALL HISTOGRAM_fill(l+74,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0.3d0.AND.y_psi.LE.0.8d0)THEN
          CALL HISTOGRAM_fill(l+75,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.0.8d0.AND.y_psi.LE.1.3d0)THEN
          CALL HISTOGRAM_fill(l+76,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.3d0.AND.y_psi.LE.1.8d0)THEN
          CALL HISTOGRAM_fill(l+77,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.1.8d0.AND.y_psi.LE.2.3d0)THEN
          CALL HISTOGRAM_fill(l+78,pt_psi,wgts_new)
       ENDIF
       IF(y_psi.GT.2.3d0.AND.y_psi.LE.2.8d0)THEN
          CALL HISTOGRAM_fill(l+79,pt_psi,wgts_new)
       ENDIF
    ENDDO

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay_pp_psiX_cb

  SUBROUTINE InitialCentrality_AB
    IMPLICIT NONE
    INTEGER::i,j
    REAL(KIND(1d0))::TAhat0,TBhat0
    REAL(KIND(1d0)),DIMENSION(0:1)::TAhat_integ,TBhat_integ
    REAL(KIND(1d0)),DIMENSION(1,4)::cent_bins
    REAL(KIND(1d0)),DIMENSION(0:igmaxA,0:1)::TAhatInG_integ
    REAL(KIND(1d0)),DIMENSION(0:igmaxB,0:1)::TBhatInG_integ
    REAL(KIND(1d0)),DIMENSION(0:igmaxA)::TAhat0InG
    REAL(KIND(1d0)),DIMENSION(0:igmaxB)::TBhat0InG
    WRITE(*,*)"INFO: Initialize the centrality in optical Glauber model (will take a few minutes)"
    ! ====================================
    ! get the A and B name
    IF(nuclearA_beam1.GT.0.AND.nuclearA_beam2.GT.0)THEN
       A_name=GetASymbol(nuclearA_beam1)
       B_name=GetASymbol(nuclearA_beam2)
    ELSE
       WRITE(*,*)"ERROR: this plot file is only used for the AB collsion case !"
       STOP
    ENDIF
    ! first to read the information of centralities
    n_centrality=0
    CALL GetCentralityImpactB(n_centrality,centrality_bins)
    ! prepare for determining the normalisations of G(x) in RA and RB only
    cent_bins(1,1)=centrality_bins(1,1)
    cent_bins(1,2)=centrality_bins(n_centrality,2)
    cent_bins(1,3)=centrality_bins(1,3)
    cent_bins(1,4)=centrality_bins(n_centrality,4)
    ! calculate the int_{bmin}^{bmax}{TAhat(b)2pi*bdb} for one bin
    CALL CalculateTAhat_WoodSaxon_centrality(A_name,1,&
         cent_bins(1:1,1:4),TAhat_integ)
    ! calculate the int_{bmin}^{bmax}{TAhat(b)2pi*bdb} for bin by bin
    CALL CalculateTAhat_WoodSaxon_centrality(B_name,1,&
         cent_bins(1:1,1:4),TBhat_integ)

    ! determine the normalisation NGNormA
    ! calculate the int_{bmin}^{bmax}{TAhat(b)**(1+i)*2pi*bdb} for bin by bin
    ! if gcoeffA(i)=!=0
    TAhat0=TAhat0_WoodSaxon(A_name)
    NGNormA=0d0
    DO i=0,igmaxA
       IF(gcoeffA(i).EQ.0d0)THEN
          TAhatInG_integ(i,0:1)=0d0
          TAhat0InG(i)=1d0
       ELSE
          IF(i.EQ.0)THEN
             TAhatInG_integ(i,0:1)=TAhat_integ(0:1)
             TAhat0InG(i)=1d0
          ELSE
             CALL CalculateTAhat_WoodSaxon_centrality(A_name,1,&
                  cent_bins(1:1,1:4),&
                  TAhatInG_integ(i,0:1),1D0+DBLE(i))
             TAhat0InG(i)=TAhat0**i
          ENDIF
          NGNormA=NGNormA+gcoeffA(i)*TAhatInG_integ(i,0)/TAhat0InG(i)
       ENDIF
    ENDDO
    IF(NGNormA.EQ.0d0)THEN
       NGNormA=1d0
    ELSE
       NGNormA=1d0/NGNormA
    ENDIF

    ! determine the normalisation NGNormB
    ! calculate the int_{bmin}^{bmax}{TBhat(b)**(1+i)*2pi*bdb} for bin by bin
    ! if gcoeffB(i)=!=0
    TBhat0=TAhat0_WoodSaxon(B_name)
    NGNormB=0d0
    DO i=0,igmaxB
       IF(gcoeffB(i).EQ.0d0)THEN
          TBhatInG_integ(i,0:1)=0d0
          TBhat0InG(i)=1d0
       ELSE
          IF(i.EQ.0)THEN
             TBhatInG_integ(i,0:1)=TBhat_integ(0:1)
             TBhat0InG(i)=1d0
          ELSE
             CALL CalculateTAhat_WoodSaxon_centrality(B_name,1,&
                  cent_bins(1:1,1:4),&
                  TBhatInG_integ(i,0:1),1D0+DBLE(i))
             TBhat0InG(i)=TBhat0**i
          ENDIF
          NGNormB=NGNormB+gcoeffB(i)*TBhatInG_integ(i,0)/TBhat0InG(i)
       ENDIF
    ENDDO
    IF(NGNormB.EQ.0d0)THEN
       NGNormB=1d0
    ELSE
       NGNormB=1d0/NGNormB
    ENDIF

    ! calculate the int_{bmin}^{bmax}{TABhat(b)2pi*bdb} for bin by bin
    ! it will be used as a denominator
    IF(ALLOCATED(TABhat_integ))THEN
       DEALLOCATE(TABhat_integ)
    ENDIF
    ALLOCATE(TABhat_integ(0:n_centrality))
    CALL CalculateTABhat_WoodSaxon_centrality(A_name,B_name,n_centrality,&
         centrality_bins(1:n_centrality,1:4),TABhat_integ)

    ! write into a file for the normalisation
    OPEN(unit=599,FILE=TRIM(output_dir)//'pp_psiX_crystalball.TABhat',&
         status='unknown')
    DO i=1,n_centrality
       WRITE(599,'(e14.7,2x,e14.7,2x,e14.7,2x,e14.7,2x,e14.7)')centrality_bins(i,1:4),TABhat_integ(i)
    ENDDO
    CLOSE(unit=599)
    
    ! calculate the int_{bmin}^{bmax}{\int_{-inf}^{+inf}{TAhat(\vec{s})**(1+i)*TBhat(\vec{s}-\vec{b})**(1+j)d^2\vec{s}}*2pi*bdb} for bin by bin
    ! if (gcoeffA(i)!=0 or i==0) and (gcoeffB(j)!=0 or j==0)
    IF(ALLOCATED(TABhatInG_integ))THEN
       DEALLOCATE(TABhatInG_integ)
    ENDIF
    ALLOCATE(TABhatInG_integ(0:igmaxA,0:igmaxB,0:n_centrality))
    IF(ALLOCATED(TABhat0InG))THEN
       DEALLOCATE(TABhat0InG)
    ENDIF
    ALLOCATE(TABhat0InG(0:igmaxA,0:igmaxB))
    DO i=0,igmaxA
       DO j=0,igmaxB
          IF((gcoeffA(i).NE.0d0.OR.i.EQ.0).AND.(gcoeffB(j).NE.0d0.OR.j.EQ.0))THEN
             IF(i.EQ.0.AND.j.EQ.0)THEN
                TABhatInG_integ(i,j,0:n_centrality)=TABhat_integ(0:n_centrality)
                TABhat0InG(i,j)=1d0
             ELSE
                CALL CalculateTAhatTBhat_WoodSaxon_centrality(A_name,B_name,n_centrality,&
                     centrality_bins(1:n_centrality,1:4),TABhatInG_integ(i,j,0:n_centrality),&
                     1D0+DBLE(i),1D0+DBLE(j))
                TABhat0InG(i,j)=TAhat0**(i)*TBhat0**(j)
             ENDIF
          ELSE
             TABhatInG_integ(i,j,0:n_centrality)=0d0
             TABhat0InG(i,j)=1d0
          ENDIF
       ENDDO
    ENDDO
    
    WRITE(*,*)"INFO: Done with initialization"
    RETURN
  END SUBROUTINE InitialCentrality_AB

END MODULE plot_pp_psiX_cb
