MODULE plot_pp_psiX_cb
  ! this is for the fiducial acceptance required by David d'Enterria
  ! for triple-J/psi analysis in 23 April 2021
  USE HELAC_Global
  USE pp_psiX_cb_global
  USE Kinetic_Func 
  IMPLICIT NONE
  INTEGER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=3
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_psiX_cb
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(:),ALLOCATABLE::weights_info
    INTEGER::i
    IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
       IF(reweight_npdf)THEN
          ! it must be the nuclear PDF
          max_weight=1+ho_npdf+N_MCPDFs+ho_nref
       ELSEIF(reweight_pdf)THEN
          ! it must be the proton PDF
          max_weight=1+N_MCPDFs+ho_nnpdf+ho_nref
       ELSE
          WRITE(*,*)"ERROR: no reweighting for both PDF and nPDF but using Hessian to MC"
          STOP
       ENDIF
    ELSE
       max_weight=1+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
    ENDIF
    IF(ALLOCATED(weights_info))THEN
       DEALLOCATE(weights_info)
    ENDIF
    ALLOCATE(weights_info(max_weight))
    nwgt=1
    weights_info(nwgt)="central value  "
    IF(reweight_pdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             WRITE(weights_info(nwgt+i),'(a4,i10,a1)')&
                  'PDF=',900000000+i,' '
          ENDDO
          nwgt=nwgt+N_MCPDFs
       ELSE
          DO i=1,ho_npdf
             WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
                  'PDF=',ipdf_set_min-1+i,'   '
          ENDDO
          nwgt=nwgt+ho_npdf
          IF(ho_npdf2.GT.0)THEN
             DO i=1,ho_npdf2
                WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                     'PDF2=',ipdf2_set_min-1+i,'  '
             ENDDO
             nwgt=nwgt+ho_npdf2
          ENDIF
       ENDIF
    ENDIF
    IF(reweight_npdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             WRITE(weights_info(nwgt+i),'(a5,i9,a1)')&
                     'NPDF=',900000000+i,' '
          ENDDO
          nwgt=nwgt+N_MCPDFs
       ELSE
          IF(nPDF_id.EQ.1000)THEN
             DO i=1,ho_nnpdf
                WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                     'NPDF=',inpdf_set_min+i-1-1000+iPDFSUP1,'  '
             ENDDO
          ELSE
             DO i=1,ho_nnpdf
                WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                     'NPDF=',inpdf_set_min-1+i,'  '
             ENDDO
          ENDIF
          nwgt=nwgt+ho_nnpdf
       ENDIF
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'REF=',iref_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_nref
    ENDIF
    ! output plot files
    CALL plot_begin_pp_psiX_cb(nwgt,max_weight,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_psiX_cb

  SUBROUTINE plotout_pp_psiX_cb
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_psiX_cb(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_psiX_cb

  SUBROUTINE outfun_pp_psiX_cb(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::wgts
    !REAL(KIND(1d0)),DIMENSION(max_weight)::wgts,wgtden
    INTEGER::init=0
    SAVE init,wgts
    ! it is necessary for using Phegas
    IF(www.EQ.0d0)RETURN
    IF(init.EQ.0)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          IF(reweight_npdf)THEN
             max_weight=1+ho_npdf+N_MCPDFs+ho_nref
          ELSEIF(reweight_pdf)THEN
             max_weight=1+N_MCPDFs+ho_nnpdf+ho_nref
          ELSE
             WRITE(*,*)"ERROR: no reweighting for both PDF and nPDF but using Hessian to MC"
             STOP
          ENDIF
       ELSE
          max_weight=1+ho_npdf+ho_npdf2+ho_nnpdf+ho_nref
       ENDIF
       IF(ALLOCATED(wgts))THEN
          DEALLOCATE(wgts)
       ENDIF
       ALLOCATE(wgts(max_weight))
       init=1
    ENDIF
    nwgt=1
    wgts(1)=www
    IF(reweight_pdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecpdf(i)
          ENDDO
       ELSE
          DO i=1,ho_npdf+ho_npdf2
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecpdf(i)
          ENDDO
       ENDIF
    ENDIF
    IF(reweight_npdf)THEN
       IF(PDF_Hessian2MC.AND.Way_Hessian2MC.EQ.1)THEN
          DO i=1,N_MCPDFs
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecnpdf(i)
          ENDDO
       ELSE
          DO i=1,ho_nnpdf
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecnpdf(i)
          ENDDO
       ENDIF
    ENDIF
    IF(reweight_ref)THEN
       DO i=1,ho_nref
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecref(i)
       ENDDO
    ENDIF
    ! output plot file 
    CALL plot_fill_pp_psiX_cb(wgts)
    RETURN
  END SUBROUTINE outfun_pp_psiX_cb

  SUBROUTINE plot_begin_pp_psiX_cb(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    !CALL INIHIST
    !CALL INIHIST_3D
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_psiX_cb: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    l=0
    CALL histogram_book(l+ 1,'total rate    ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2,'psi Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 3,'psi y ',&
         100,-5d0,5d0,.TRUE.)

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_psiX_cb

  SUBROUTINE plot_end_pp_psiX_cb(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_psiX_cb
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_psiX_cb
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_psiX_cb
       psfile="pp_psiX_crystalball.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_psiX_cb
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_psiX_cb
       rootfile="pp_psiX_crystalball.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_psiX_cb
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_psiX_cb(xnorm)
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_psiX_cb

  SUBROUTINE plot_fill_pp_psiX_cb(wgts)
    USE Helac_Global
    IMPLICIT NONE
    !INCLUDE "pp_psipsi_dps.inc"
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,pt_psi,y_psi
    IF(NDecayChains.GT.0)THEN
       CALL plot_fill_Decay_pp_psiX_cb(wgts)
       RETURN
    ENDIF
    STOP "Please set J/psi decayed into muon pair first"
    IF(wgts(1).EQ.0d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    var=1.0d0 ! total cross section
    !pini(1:4)=cb_hadron_pmom(1,1:4)+cb_hadron_pmom(2,1:4) ! (px,py,pz,E)
    !yftot=rapidity(pini)
    y_psi=rapidity(cb_hadron_pmom(3,1:4))
    pt_psi=DSQRT(cb_hadron_pmom(3,1)**2+cb_hadron_pmom(3,2)**2)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,pt_psi,wgts)
    CALL HISTOGRAM_fill(l+3,y_psi,wgts)
    
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_psiX_cb

  SUBROUTINE HwU_write_file_pp_psiX_cb(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,FILE=TRIM(output_dir)//'pp_psiX_crystalball.HwU',&
         status='unknown')
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_psiX_cb

  SUBROUTINE open_topdrawer_file_pp_psiX_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_psiX_crystalball.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_psiX_cb

  SUBROUTINE close_topdrawer_file_pp_psiX_cb
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_psiX_cb

  SUBROUTINE open_gnuplot_file_pp_psiX_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_psiX_crystalball.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_psiX_cb

  SUBROUTINE close_gnuplot_file_pp_psiX_cb
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_psiX_cb

  SUBROUTINE open_root_file_pp_psiX_cb
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_psiX_crystalball.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_psiX_cb

  SUBROUTINE close_root_file_pp_psiX_cb
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_psiX_cb

  SUBROUTINE plot_fill_Decay_pp_psiX_cb(wgts)
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "stdhep_pp_psiX_cb.inc"
    REAL(KIND(1d0))::var,Q,www,y_psi,pt_psi,eta_lep1,eta_lep2,pt_lep1,pt_lep2
    INTEGER::i,m,l,kk
    REAL(KIND(1d0)),DIMENSION(1,4)::PJpsi
    REAL(KIND(1d0)),DIMENSION(2,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    INTEGER,DIMENSION(2)::lep2jpsi
    INTEGER,DIMENSION(1)::newmoth
    INTEGER::i_jpsi,i_lep,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    LOGICAL::pass_cmscuts
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton 
    i_jpsi=0
    i_lep=0
    lep2jpsi(1:2)=0
    newmoth(1:1)=0
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
       IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
       i_lep=i_lep+1
       Plep(i_lep,1:4)=PHEP(1:4,i)
       m=JMOHEP(1,i)
       definemoth=0
       DO kk=1,i_jpsi
          IF(newmoth(kk).EQ.m)THEN
             definemoth=kk
             EXIT
          END IF
       ENDDO
       IF(definemoth.NE.0)THEN
          ! the mother has been defined
          lep2jpsi(i_lep)=definemoth
       ELSE
          ! create the new mother
          i_jpsi=i_jpsi+1
          PJpsi(i_jpsi,1:4)=PHEP(1:4,m)
          newmoth(i_jpsi)=m
          lep2jpsi(i_lep)=i_jpsi
          IF(i_lep.EQ.2)EXIT
       ENDIF
    ENDDO
    IF(i_lep.NE.2)THEN
       WRITE(*,*)"Error: do not find two decayed leptons"
       STOP
    ENDIF
    IF(i_jpsi.LT.1)THEN
       WRITE(*,*)"Error: do not find any J/psi"
       STOP
    ENDIF
    y_psi=rapidity(PJpsi(1,1:4))
    pt_psi=PJpsi(1,1)**2+PJpsi(1,2)**2
    pt_psi=DSQRT(DABS(pt_psi))
    ! for J/psi, pT>6 GeV, |y|<2.4
    !            2.9<m(mu+,mu-)<3.3 GeV (not applicable here)
    pass_cmscuts=pt_psi.GT.6d0.AND.DABS(y_psi).LT.2.4d0
    ! for all muons, pT>3.5 GeV when |eta|<=1.2
    !                pT>2.5 GeV when 1.2<|eta|<2.4
    pt_lep1=Plep(1,1)**2+Plep(1,2)**2
    pt_lep1=DSQRT(DABS(pt_lep1))
    pt_lep2=Plep(2,1)**2+Plep(2,2)**2
    pt_lep2=DSQRT(DABS(pt_lep2))
    eta_lep1=prapidity(Plep(1,1:4))
    eta_lep2=prapidity(Plep(2,1:4))
    IF(DABS(eta_lep1).GE.2.4d0.OR.DABS(eta_lep2).GE.2.4d0)THEN
       pass_cmscuts=.FALSE.
    ENDIF
    IF(pass_cmscuts)THEN
       IF(DABS(eta_lep1).LE.1.2d0)THEN
          pass_cmscuts=pass_cmscuts.AND.pt_lep1.GT.3.5d0
       ELSEIF(DABS(eta_lep1).GT.1.2d0.AND.DABS(eta_lep1).LT.2.4d0)THEN
          pass_cmscuts=pass_cmscuts.AND.pt_lep1.GT.2.5d0
       ELSE
          pass_cmscuts=.FALSE.
       ENDIF
       IF(DABS(eta_lep2).LE.1.2d0)THEN
          pass_cmscuts=pass_cmscuts.AND.pt_lep2.GT.3.5d0
       ELSEIF(DABS(eta_lep2).GT.1.2d0.AND.DABS(eta_lep2).LT.2.4d0)THEN
          pass_cmscuts=pass_cmscuts.AND.pt_lep2.GT.2.5d0
       ELSE
          pass_cmscuts=.FALSE.
       ENDIF
    ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    IF(pass_cmscuts)THEN
       l=0
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,pt_psi,wgts)
       CALL HISTOGRAM_fill(l+3,y_psi,wgts)
    ENDIF

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay_pp_psiX_cb
END MODULE plot_pp_psiX_cb
