MODULE plot_pp_psiY_sps
!============================================================ 
!
! PLOT FILE FOR USER
! LHCb fiducial region:
! 2.0<y(psi)<4.5 and pt(psi) < 14 GeV at LHC13
!
!============================================================
  USE HELAC_Global
  USE pp_psiY_global
  USE Kinetic_Func 
  IMPLICIT NONE
  !INTEGER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=56
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_psiY_sps
    IMPLICIT NONE
    INTEGER::nwgt
    !CHARACTER(len=15),DIMENSION(max_weight)::weights_info
    CHARACTER(len=15),DIMENSION(:),ALLOCATABLE::weights_info
    INTEGER::i
    INTEGER::max_weight=1
    max_weight=1+ho_nscale+ho_npdf+ho_npdf2
    IF(ALLOCATED(weights_info))THEN
       DEALLOCATE(weights_info)
    ENDIF
    ALLOCATE(weights_info(max_weight))
    nwgt=1
    weights_info(nwgt)="central value "
    IF(reweight_scale)THEN
       nwgt=nwgt+ho_nscale
       IF(ho_nscale.NE.8)THEN
          WRITE(*,*) 'ERROR #1 in initplot_pp_psiY_sps:',ho_nscale
          STOP
       ENDIF
       WRITE(weights_info(nwgt-7),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",1.0,"muF=",rw_Fscale_up
       WRITE(weights_info(nwgt-6),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",1.0,"muF=",rw_Fscale_down
       WRITE(weights_info(nwgt-5),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_up,"muF=",1d0
       WRITE(weights_info(nwgt-4),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_up,"muF=",rw_Fscale_up
       WRITE(weights_info(nwgt-3),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_up,"muF=",rw_Fscale_down
       WRITE(weights_info(nwgt-2),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_down,"muF=",1d0
       WRITE(weights_info(nwgt-1),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_down,"muF=",rw_Fscale_up
       WRITE(weights_info(nwgt  ),'(a4,f3.1,x,a4,f3.1)')&
            "muR=",rw_Rscale_down,"muF=",rw_Fscale_down
    END IF
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf
          WRITE(weights_info(nwgt+i),'(a4,i8,a3)')&
               'PDF=',ipdf_set_min-1+i,'   '
       ENDDO
       nwgt=nwgt+ho_npdf
       IF(ho_npdf2.GT.0)THEN
          DO i=1,ho_npdf2
             WRITE(weights_info(nwgt+i),'(a5,i8,a2)')&
                  'PDF2=',ipdf2_set_min-1+i,'  '
          ENDDO
          nwgt=nwgt+ho_npdf2
       ENDIF
    ENDIF
    ! output plot files
    CALL plot_begin_pp_psiY_sps(nwgt,max_weight,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_psiY_sps

  SUBROUTINE plotout_pp_psiY_sps
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_psiY_sps(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_psiY_sps

  SUBROUTINE outfun_pp_psiY_sps(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0))::alphasc,alphasu,alphasd
    !REAL(KIND(1d0)),DIMENSION(max_weight)::wgts,wgtden
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::wgts
    INTEGER::init=0
    INTEGER::max_weight
    SAVE init,wgts
    IF(init.EQ.0)THEN
       max_weight=1+ho_nscale+ho_npdf+ho_npdf2
       IF(ALLOCATED(wgts))THEN
          DEALLOCATE(wgts)
       ENDIF
       ALLOCATE(wgts(max_weight))
       init=1
    ENDIF
    nwgt=1
    wgts(1)=www
    IF(reweight_scale)THEN
       DO i=1,3
          DO j=1,3
             IF(i.EQ.1.AND.j.EQ.1)CYCLE
             nwgt=nwgt+1
             wgts(nwgt)=www*wgtxsecmu(i,j)
          ENDDO
       ENDDO
    ENDIF
    IF(reweight_pdf)THEN
       DO i=1,ho_npdf+ho_npdf2
          nwgt=nwgt+1
          wgts(nwgt)=www*wgtxsecpdf(i)
       ENDDO
    ENDIF
    ! output plot file 
    CALL plot_fill_pp_psiY_sps(wgts)
    RETURN
  END SUBROUTINE outfun_pp_psiY_sps

  SUBROUTINE plot_begin_pp_psiY_sps(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_psiY_sps: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    l=0
    CALL histogram_book(l+ 1,'total rate    ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2,'|Delta y|  ',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+ 3,'|Delta phi| ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+ 4,'PT(psi+psi) ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 5,'y(psi+psi)  ',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+ 6,'PT(psi) incl ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 7,'y(psi) incl ',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+ 8,'|pt(1)-pt(2)|/|pt(1)+pt(2)| ',&
         100,0d0,1d0,.TRUE.)
    CALL histogram_book(l+ 9,'M(psi+psi) ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+10,'PT(psi+psi) + sm 0.5',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+11,'PT(psi+psi) + sm 1.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+12,'PT(psi+psi) + sm 2.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+13,'PT(psi+psi) + sm 3.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+14,'|Delta phi| + sm 0.5',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+15,'|Delta phi| + sm 1.0',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+16,'|Delta phi| + sm 2.0',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+17,'|Delta phi| + sm 3.0',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+18,'|Delta y| + sm 0.5',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+19,'|Delta y| + sm 1.0',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+20,'|Delta y| + sm 2.0',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+21,'|Delta y| + sm 3.0',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+22,'y(psi+psi) + sm 0.5',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+23,'y(psi+psi) + sm 1.0',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+24,'y(psi+psi) + sm 2.0',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+25,'y(psi+psi) + sm 3.0',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+26,'PT(psi) + sm 0.5',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+27,'PT(psi) + sm 1.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+28,'PT(psi) + sm 2.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+29,'PT(psi) + sm 3.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+30,'y(psi) + sm 0.5',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+31,'y(psi) + sm 1.0',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+32,'y(psi) + sm 2.0',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+33,'y(psi) + sm 3.0',&
         100,-5d0,5d0,.TRUE.)
    CALL histogram_book(l+34,'|pt(1)-pt(2)|/|pt(1)+pt(2)| + sm 0.5',&
         100,0d0,1d0,.TRUE.)
    CALL histogram_book(l+35,'|pt(1)-pt(2)|/|pt(1)+pt(2)| + sm 1.0',&
         100,0d0,1d0,.TRUE.)
    CALL histogram_book(l+36,'|pt(1)-pt(2)|/|pt(1)+pt(2)| + sm 2.0',&
         100,0d0,1d0,.TRUE.)
    CALL histogram_book(l+37,'|pt(1)-pt(2)|/|pt(1)+pt(2)| + sm 3.0',&
         100,0d0,1d0,.TRUE.)
    CALL histogram_book(l+38,'M(psi+psi) + sm 0.5',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+39,'M(psi+psi) + sm 1.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+40,'M(psi+psi) + sm 2.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+41,'M(psi+psi) + sm 3.0',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+42,'M(psi+psi) zoom',&
         100,5d0,15d0,.TRUE.)
    CALL histogram_book(l+43,'M(psi+psi) zoom + sm 0.5',&
         100,5d0,15d0,.TRUE.)
    CALL histogram_book(l+44,'M(psi+psi) zoom + sm 1.0',&
         100,5d0,15d0,.TRUE.)
    CALL histogram_book(l+45,'M(psi+psi) zoom + sm 2.0',&
         100,5d0,15d0,.TRUE.)
    CALL histogram_book(l+46,'M(psi+psi) zoom + sm 3.0',&
         100,5d0,15d0,.TRUE.)
    CALL histogram_book(l+47,'PT(psi+psi) zoom',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+48,'PT(psi+psi) zoom + sm 0.5',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+49,'PT(psi+psi) zoom + sm 1.0',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+50,'PT(psi+psi) zoom + sm 2.0',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+51,'PT(psi+psi) zoom + sm 3.0',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+52,'PT(psi) zoom',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+53,'PT(psi) zoom + sm 0.5',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+54,'PT(psi) zoom + sm 1.0',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+55,'PT(psi) zoom + sm 2.0',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+56,'PT(psi) zoom + sm 3.0',&
         100,0d0,50d0,.TRUE.)

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_psiY_sps

  SUBROUTINE plot_end_pp_psiY_sps(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_psiY_sps
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_psiY_sps
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_psiY_sps
       psfile="pp_psiY_sps.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_psiY_sps
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_psiY_sps
       rootfile="pp_psiY_sps.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_psiY_sps
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_psiY_sps
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_psiY_sps

  SUBROUTINE plot_fill_pp_psiY_sps(wgts)
    USE Helac_Global
    IMPLICIT NONE
    !INCLUDE "pp_psipsi_dps.inc"
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,yftot
    IF(NDecayChains.GT.0)THEN
       CALL plot_fill_Decay_pp_psiY_sps(wgts)
       RETURN
    ENDIF
    WRITE(*,*)"Please change the analysis file plot_pp_psiY_sps.f90"
    WRITE(*,*)"We do not implement the analysis without leptonic decay of J/psi!"
    STOP
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    pini(1:4)=sps_pmom(1,1:4)+sps_pmom(2,1:4) ! (px,py,pz,E)
    yftot=rapidity(pini)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_FILL(l+1,var,wgts)
    CALL HISTOGRAM_FILL(l+2,yftot,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_psiY_sps

  SUBROUTINE HwU_write_file_pp_psiY_sps
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0))::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//'pp_psiY_sps.HwU',&
         status='unknown')
    xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN  
  END SUBROUTINE HwU_write_file_pp_psiY_sps

  SUBROUTINE open_topdrawer_file_pp_psiY_sps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_psiY_sps.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_psiY_sps

  SUBROUTINE close_topdrawer_file_pp_psiY_sps
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_psiY_sps

  SUBROUTINE open_gnuplot_file_pp_psiY_sps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_psiY_sps.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_psiY_sps

  SUBROUTINE close_gnuplot_file_pp_psiY_sps
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_psiY_sps

  SUBROUTINE open_root_file_pp_psiY_sps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_psiY_sps.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_psiY_sps

  SUBROUTINE close_root_file_pp_psiY_sps
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_psiY_sps

  SUBROUTINE plot_fill_Decay_pp_psiY_sps(wgts)
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "stdhep_pp_psiY_sps.inc"
    REAL(KIND(1d0))::var,Q,www,dy_psipsi,dphi_psipsi,M_psipsi,Pt_psipsi,deta_psipsi,Pt1Pt2_psipsi
    REAL(KIND(1d0))::q12_psipsi,cosalpha,ptpsi1,ptpsi2,ptpsimax,ptpsimin,ptpsidiff
    REAL(KIND(1d0))::Pt_psipsi_sm05,Pt_psipsi_sm10,Pt_psipsi_sm20,Pt_psipsi_sm30
    REAL(KIND(1d0))::y_psipsi,y_psipsi_sm05,y_psipsi_sm10,y_psipsi_sm20,y_psipsi_sm30
    REAL(KIND(1d0))::dy_psipsi_sm05,dy_psipsi_sm10,dy_psipsi_sm20,dy_psipsi_sm30
    REAL(KIND(1d0))::dphi_psipsi_sm05,dphi_psipsi_sm10,dphi_psipsi_sm20,dphi_psipsi_sm30
    REAL(KIND(1d0))::M_psipsi_sm05,M_psipsi_sm10,M_psipsi_sm20,M_psipsi_sm30
    REAL(KIND(1d0))::ypsi1,ypsi2
    REAL(KIND(1d0))::ypsi1_sm05,ypsi1_sm10,ypsi1_sm20,ypsi1_sm30
    REAL(KIND(1d0))::ypsi2_sm05,ypsi2_sm10,ypsi2_sm20,ypsi2_sm30
    INTEGER::i,m,l,kk
    REAL(KIND(1d0)),DIMENSION(4,NMXHEP)::PHEPSM05,PHEPSM10,PHEPSM20,PHEPSM30
    REAL(KIND(1d0)),DIMENSION(4)::PBOO
    REAL(KIND(1d0)),DIMENSION(2,4)::PJpsi,PJpsiCMS,PJpsiSM05,PJpsiSM10,PJpsiSM20,PJpsiSM30
    REAL(KIND(1d0)),DIMENSION(4,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    REAL(KIND(1d0)),DIMENSION(3)::Pplain1,Pplain2
    REAL(KIND(1d0)),DIMENSION(3)::Plepvec1,Plepvec2
    REAL(KIND(1d0)),DIMENSION(2,3)::PJpsi1l,PJpsi2l
    REAL(KIND(1d0))::PP1,PP2
    REAL(KIND(1d0))::costh1,costh2
    REAL(KIND(1d0))::ptpsi1_sm05,ptpsi1_sm10,ptpsi1_sm20,ptpsi1_sm30
    REAL(KIND(1d0))::ptpsi2_sm05,ptpsi2_sm10,ptpsi2_sm20,ptpsi2_sm30
    REAL(KIND(1d0))::ptpsimax_sm05,ptpsimax_sm10,ptpsimax_sm20,ptpsimax_sm30
    REAL(KIND(1d0))::ptpsimin_sm05,ptpsimin_sm10,ptpsimin_sm20,ptpsimin_sm30
    INTEGER,DIMENSION(4)::lep2jpsi
    INTEGER,DIMENSION(2)::newmoth
    INTEGER::i_jpsi,i_lep,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    REAL(KIND(1d0))::ptrate
    IF(wgts(1).EQ.0d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton 
    i_jpsi=0
    i_lep=0
    lep2jpsi(1:4)=0
    newmoth(1:2)=0
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM05(1:4,1:NHEP),0.5d0,2)
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM10(1:4,1:NHEP),1d0,2)
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM20(1:4,1:NHEP),2d0,2)
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM30(1:4,1:NHEP),3d0,2)
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
       IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
       i_lep=i_lep+1
       Plep(i_lep,1:4)=PHEP(1:4,i)
       m=JMOHEP(1,i)
       definemoth=0
       DO kk=1,i_jpsi
          IF(newmoth(kk).EQ.m)THEN
             definemoth=kk
             EXIT
          END IF
       ENDDO
       IF(definemoth.NE.0)THEN
          ! the mother has been defined
          lep2jpsi(i_lep)=definemoth
       ELSE
          ! create the new mother
          i_jpsi=i_jpsi+1
          PJpsi(i_jpsi,1:4)=PHEP(1:4,m)
          PJpsiSM05(i_jpsi,1:4)=PHEPSM05(1:4,m)
          PJpsiSM10(i_jpsi,1:4)=PHEPSM10(1:4,m)
          PJpsiSM20(i_jpsi,1:4)=PHEPSM20(1:4,m)
          PJpsiSM30(i_jpsi,1:4)=PHEPSM30(1:4,m)
          newmoth(i_jpsi)=m
          lep2jpsi(i_lep)=i_jpsi
          IF(i_lep.EQ.4)EXIT
       ENDIF
    ENDDO
    dy_psipsi=DABS(rapidity(PJpsi(1,1:4))-rapidity(PJpsi(2,1:4)))
    dy_psipsi_sm05=DABS(rapidity(PJpsiSM05(1,1:4))-rapidity(PJpsiSM05(2,1:4)))
    IF(isnan(dy_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    dy_psipsi_sm10=DABS(rapidity(PJpsiSM10(1,1:4))-rapidity(PJpsiSM10(2,1:4)))
    IF(isnan(dy_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    dy_psipsi_sm20=DABS(rapidity(PJpsiSM20(1,1:4))-rapidity(PJpsiSM20(2,1:4)))
    IF(isnan(dy_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    dy_psipsi_sm30=DABS(rapidity(PJpsiSM30(1,1:4))-rapidity(PJpsiSM30(2,1:4)))
    IF(isnan(dy_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    deta_psipsi=DABS(prapidity(PJpsi(1,1:4))-prapidity(PJpsi(2,1:4)))
    dphi_psipsi=ph4(PJpsi(1,1),PJpsi(1,2),PJpsi(1,3))
    dphi_psipsi=dphi_psipsi-ph4(PJpsi(2,1),PJpsi(2,2),PJpsi(2,3))
    dphi_psipsi=DABS(dphi_psipsi)
    IF(dphi_psipsi.GT.pi)dphi_psipsi=2*pi-dphi_psipsi
    dphi_psipsi_sm05=ph4(PJpsiSM05(1,1),PJpsiSM05(1,2),PJpsiSM05(1,3))
    dphi_psipsi_sm05=dphi_psipsi_sm05-ph4(PJpsiSM05(2,1),PJpsiSM05(2,2),PJpsiSM05(2,3))
    dphi_psipsi_sm05=DABS(dphi_psipsi_sm05)
    IF(dphi_psipsi_sm05.GT.pi)dphi_psipsi_sm05=2*pi-dphi_psipsi_sm05
    IF(isnan(dphi_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    dphi_psipsi_sm10=ph4(PJpsiSM10(1,1),PJpsiSM10(1,2),PJpsiSM10(1,3))
    dphi_psipsi_sm10=dphi_psipsi_sm10-ph4(PJpsiSM10(2,1),PJpsiSM10(2,2),PJpsiSM10(2,3))
    dphi_psipsi_sm10=DABS(dphi_psipsi_sm10)
    IF(dphi_psipsi_sm10.GT.pi)dphi_psipsi_sm10=2*pi-dphi_psipsi_sm10
    IF(isnan(dphi_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    dphi_psipsi_sm20=ph4(PJpsiSM20(1,1),PJpsiSM20(1,2),PJpsiSM20(1,3))
    dphi_psipsi_sm20=dphi_psipsi_sm20-ph4(PJpsiSM20(2,1),PJpsiSM20(2,2),PJpsiSM20(2,3))
    dphi_psipsi_sm20=DABS(dphi_psipsi_sm20)
    IF(dphi_psipsi_sm20.GT.pi)dphi_psipsi_sm20=2*pi-dphi_psipsi_sm20
    IF(isnan(dphi_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    dphi_psipsi_sm30=ph4(PJpsiSM30(1,1),PJpsiSM30(1,2),PJpsiSM30(1,3))
    dphi_psipsi_sm30=dphi_psipsi_sm30-ph4(PJpsiSM30(2,1),PJpsiSM30(2,2),PJpsiSM30(2,3))
    dphi_psipsi_sm30=DABS(dphi_psipsi_sm30)
    IF(dphi_psipsi_sm30.GT.pi)dphi_psipsi_sm30=2*pi-dphi_psipsi_sm30
    IF(isnan(dphi_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    M_psipsi=scalar_product(PJpsi(1,1:4)+PJpsi(2,1:4),PJpsi(1,1:4)+PJpsi(2,1:4))
    M_psipsi=DSQRT(M_psipsi)
    M_psipsi_sm05=scalar_product(PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4),&
         PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4))
    M_psipsi_sm05=DSQRT(M_psipsi_sm05)
    IF(isnan(M_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    M_psipsi_sm10=scalar_product(PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4),&
         PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4))
    M_psipsi_sm10=DSQRT(M_psipsi_sm10)
    IF(isnan(M_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    M_psipsi_sm20=scalar_product(PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4),&
         PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4))
    M_psipsi_sm20=DSQRT(M_psipsi_sm20)
    IF(isnan(M_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    M_psipsi_sm30=scalar_product(PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4),&
         PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4))
    M_psipsi_sm30=DSQRT(M_psipsi_sm30)
    IF(isnan(M_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF    
    Q=scalar_product(PJpsi(1,1:4),PJpsi(1,1:4))
    Q=DSQRT(Q)
    ptpsi1=transverse(PJpsi(1,1:4))
    ptpsi2=transverse(PJpsi(2,1:4))
    ptpsimax=MAX(ptpsi1,ptpsi2)
    ptpsimin=MIN(ptpsi1,ptpsi2)
    ptpsi1_sm05=transverse(PJpsiSM05(1,1:4))
    ptpsi2_sm05=transverse(PJpsiSM05(2,1:4))
    ptpsimax_sm05=MAX(ptpsi1_sm05,ptpsi2_sm05)
    ptpsimin_sm05=MIN(ptpsi1_sm05,ptpsi2_sm05)
    IF(isnan(ptpsi1_sm05).OR.isnan(ptpsi2_sm05))THEN
       WRITE(*,*)"ERROR:ptpsi1_sm05 or ptpsi2_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    ptpsi1_sm10=transverse(PJpsiSM10(1,1:4))
    ptpsi2_sm10=transverse(PJpsiSM10(2,1:4))
    ptpsimax_sm10=MAX(ptpsi1_sm10,ptpsi2_sm10)
    ptpsimin_sm10=MIN(ptpsi1_sm10,ptpsi2_sm10)
    IF(isnan(ptpsi1_sm10).OR.isnan(ptpsi2_sm10))THEN
       WRITE(*,*)"ERROR:ptpsi1_sm10 or ptpsi2_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    ptpsi1_sm20=transverse(PJpsiSM20(1,1:4))
    ptpsi2_sm20=transverse(PJpsiSM20(2,1:4))
    ptpsimax_sm20=MAX(ptpsi1_sm20,ptpsi2_sm20)
    ptpsimin_sm20=MIN(ptpsi1_sm20,ptpsi2_sm20)
    IF(isnan(ptpsi1_sm20).OR.isnan(ptpsi2_sm20))THEN
       WRITE(*,*)"ERROR:ptpsi1_sm20 or ptpsi2_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    ptpsi1_sm30=transverse(PJpsiSM30(1,1:4))
    ptpsi2_sm30=transverse(PJpsiSM30(2,1:4))
    ptpsimax_sm30=MAX(ptpsi1_sm30,ptpsi2_sm30)
    ptpsimin_sm30=MIN(ptpsi1_sm30,ptpsi2_sm30)
    IF(isnan(ptpsi1_sm30).OR.isnan(ptpsi2_sm30))THEN
       WRITE(*,*)"ERROR:ptpsi1_sm30 or ptpsi2_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    ypsi1=rapidity(PJpsi(1,1:4))
    ypsi2=rapidity(PJpsi(2,1:4))
    ypsi1_sm05=rapidity(PJpsiSM05(1,1:4))
    ypsi2_sm05=rapidity(PJpsiSM05(2,1:4))
    IF(isnan(ypsi1_sm05).OR.isnan(ypsi2_sm05))THEN
       WRITE(*,*)"ERROR:ypsi1_sm05 or ypsi2_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    ypsi1_sm10=rapidity(PJpsiSM10(1,1:4))
    ypsi2_sm10=rapidity(PJpsiSM10(2,1:4))
    IF(isnan(ypsi1_sm10).OR.isnan(ypsi2_sm10))THEN
       WRITE(*,*)"ERROR:ypsi1_sm10 or ypsi2_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    ypsi1_sm20=rapidity(PJpsiSM20(1,1:4))
    ypsi2_sm20=rapidity(PJpsiSM20(2,1:4))
    IF(isnan(ypsi1_sm20).OR.isnan(ypsi2_sm20))THEN
       WRITE(*,*)"ERROR:ypsi1_sm20 or ypsi2_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    ypsi1_sm30=rapidity(PJpsiSM30(1,1:4))
    ypsi2_sm30=rapidity(PJpsiSM30(2,1:4))
    IF(isnan(ypsi1_sm30).OR.isnan(ypsi2_sm30))THEN
       WRITE(*,*)"ERROR:ypsi1_sm30 or ypsi2_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    Pt_psipsi=transverse(PJpsi(1,1:4)+PJpsi(2,1:4))
    Pt_psipsi_sm05=transverse(PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4))
    IF(isnan(Pt_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    Pt_psipsi_sm10=transverse(PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4))
    IF(isnan(Pt_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    Pt_psipsi_sm20=transverse(PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4))
    IF(isnan(Pt_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    Pt_psipsi_sm30=transverse(PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4))
    IF(isnan(Pt_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    y_psipsi=rapidity(PJpsi(1,1:4)+PJpsi(2,1:4))
    y_psipsi_sm05=rapidity(PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4))
    IF(isnan(y_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:y_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    y_psipsi_sm10=rapidity(PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4))
    IF(isnan(y_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:y_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    y_psipsi_sm20=rapidity(PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4))
    IF(isnan(y_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:y_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    y_psipsi_sm30=rapidity(PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4))
    IF(isnan(y_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:y_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,dy_psipsi,wgts)
    CALL HISTOGRAM_fill(l+3,dphi_psipsi,wgts)
    CALL HISTOGRAM_fill(l+4,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill(l+47,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill(l+5,y_psipsi,wgts)

    CALL HISTOGRAM_fill(l+6,ptpsi1,wgts)
    CALL HISTOGRAM_fill(l+6,ptpsi2,wgts)
    CALL HISTOGRAM_fill(l+52,ptpsi1,wgts)
    CALL HISTOGRAM_fill(l+52,ptpsi2,wgts)

    CALL HISTOGRAM_fill(l+7,ypsi1,wgts)
    CALL HISTOGRAM_fill(l+7,ypsi2,wgts)

    IF(ptpsi1.EQ.0d0.AND.ptpsi2.EQ.0d0)THEN
       CALL HISTOGRAM_fill(l+8,0d0,wgts)
    ELSE
       CALL HISTOGRAM_fill(l+8,DABS(ptpsi1-ptpsi2)/(ptpsi1+ptpsi2),wgts)
    ENDIF

    CALL HISTOGRAM_fill(l+9,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+42,M_psipsi,wgts)

    CALL HISTOGRAM_fill(l+10,Pt_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+11,Pt_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+12,Pt_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+13,Pt_psipsi_sm30,wgts)
    ! zoomed histograms
    CALL HISTOGRAM_fill(l+48,Pt_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+49,Pt_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+50,Pt_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+51,Pt_psipsi_sm30,wgts)

    CALL HISTOGRAM_fill(l+14,dphi_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+15,dphi_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+16,dphi_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+17,dphi_psipsi_sm30,wgts)

    CALL HISTOGRAM_fill(l+18,dy_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+19,dy_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+20,dy_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+21,dy_psipsi_sm30,wgts)

    CALL HISTOGRAM_fill(l+22,y_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+23,y_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+24,y_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+25,y_psipsi_sm30,wgts)

    CALL HISTOGRAM_fill(l+26,ptpsi1_sm05,wgts)
    CALL HISTOGRAM_fill(l+26,ptpsi2_sm05,wgts)
    CALL HISTOGRAM_fill(l+27,ptpsi1_sm10,wgts)
    CALL HISTOGRAM_fill(l+27,ptpsi2_sm10,wgts)
    CALL HISTOGRAM_fill(l+28,ptpsi1_sm20,wgts)
    CALL HISTOGRAM_fill(l+28,ptpsi2_sm20,wgts)
    CALL HISTOGRAM_fill(l+29,ptpsi1_sm30,wgts)
    CALL HISTOGRAM_fill(l+29,ptpsi2_sm30,wgts)
    ! zoomed histograms
    CALL HISTOGRAM_fill(l+53,ptpsi1_sm05,wgts)
    CALL HISTOGRAM_fill(l+53,ptpsi2_sm05,wgts)
    CALL HISTOGRAM_fill(l+54,ptpsi1_sm10,wgts)
    CALL HISTOGRAM_fill(l+54,ptpsi2_sm10,wgts)
    CALL HISTOGRAM_fill(l+55,ptpsi1_sm20,wgts)
    CALL HISTOGRAM_fill(l+55,ptpsi2_sm20,wgts)
    CALL HISTOGRAM_fill(l+56,ptpsi1_sm30,wgts)
    CALL HISTOGRAM_fill(l+56,ptpsi2_sm30,wgts)

    CALL HISTOGRAM_fill(l+30,ypsi1_sm05,wgts)
    CALL HISTOGRAM_fill(l+30,ypsi2_sm05,wgts)
    CALL HISTOGRAM_fill(l+31,ypsi1_sm10,wgts)
    CALL HISTOGRAM_fill(l+31,ypsi2_sm10,wgts)
    CALL HISTOGRAM_fill(l+32,ypsi1_sm20,wgts)
    CALL HISTOGRAM_fill(l+32,ypsi2_sm20,wgts)
    CALL HISTOGRAM_fill(l+33,ypsi1_sm30,wgts)
    CALL HISTOGRAM_fill(l+33,ypsi2_sm30,wgts)

    IF(ptpsi1_sm05.EQ.0d0.AND.ptpsi2_sm05.EQ.0d0)THEN
       CALL HISTOGRAM_fill(l+34,0d0,wgts)
    ELSE
       ptrate=DABS(ptpsi1_sm05-ptpsi2_sm05)/(ptpsi1_sm05+ptpsi2_sm05)
       CALL HISTOGRAM_fill(l+34,ptrate,wgts)
    ENDIF
    IF(ptpsi1_sm10.EQ.0d0.AND.ptpsi2_sm10.EQ.0d0)THEN
       CALL HISTOGRAM_fill(l+35,0d0,wgts)
    ELSE
       ptrate=DABS(ptpsi1_sm10-ptpsi2_sm10)/(ptpsi1_sm10+ptpsi2_sm10)
       CALL HISTOGRAM_fill(l+35,ptrate,wgts)
    ENDIF
    IF(ptpsi1_sm20.EQ.0d0.AND.ptpsi2_sm20.EQ.0d0)THEN
       CALL HISTOGRAM_fill(l+36,0d0,wgts)
    ELSE
       ptrate=DABS(ptpsi1_sm20-ptpsi2_sm20)/(ptpsi1_sm20+ptpsi2_sm20)
       CALL HISTOGRAM_fill(l+36,ptrate,wgts)
    ENDIF
    IF(ptpsi1_sm30.EQ.0d0.AND.ptpsi2_sm30.EQ.0d0)THEN
       CALL HISTOGRAM_fill(l+37,0d0,wgts)
    ELSE
       ptrate=DABS(ptpsi1_sm30-ptpsi2_sm30)/(ptpsi1_sm30+ptpsi2_sm30)
       CALL HISTOGRAM_fill(l+37,ptrate,wgts)
    ENDIF

    CALL HISTOGRAM_fill(l+38,M_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+39,M_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+40,M_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+41,M_psipsi_sm30,wgts)
    ! zoomed histograms
    CALL HISTOGRAM_fill(l+43,M_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+44,M_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+45,M_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+46,M_psipsi_sm30,wgts)

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay_pp_psiY_sps
END MODULE plot_pp_psiY_sps
