MODULE plot_pp_psipsi_dps
  USE HELAC_Global
  USE pp_psipsi_dps_global
  USE Kinetic_Func 
  IMPLICIT NONE
  INTEGER,PARAMETER::max_weight=1
  INTEGER::nwgt_analysis
  INTEGER::num_plots=19
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE initplot_pp_psipsi_dps
    IMPLICIT NONE
    INTEGER::nwgt
    CHARACTER(len=15),DIMENSION(max_weight)::weights_info
    INTEGER::i
    nwgt=1
    weights_info(nwgt)="central value"
    ! output plot files
    CALL plot_begin_pp_psipsi_dps(nwgt,weights_info)
    RETURN
  END SUBROUTINE initplot_pp_psipsi_dps

  SUBROUTINE plotout_pp_psipsi_dps
    USE MC_VEGAS
    IMPLICIT NONE
    LOGICAL::usexinteg=.FALSE.,mint=.FALSE. ! I have not implemented MINT 
    LOGICAL::useitmax=.TRUE.
    REAL(KIND(1d0))::xnorm
    xnorm=1.d0
    IF(useitmax.AND.gener.EQ.3)xnorm=xnorm/float(itmx) ! use vegas with itmax
    ! output plot files  
    CALL plot_end_pp_psipsi_dps(xnorm)
    RETURN
  END SUBROUTINE plotout_pp_psipsi_dps

  SUBROUTINE outfun_pp_psipsi_dps(www)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::www
    INTEGER::i,j
    INTEGER::nwgt
    REAL(KIND(1d0)),DIMENSION(max_weight)::wgts,wgtden
    nwgt=1
    wgts(1)=www
    ! output plot file 
    CALL plot_fill_pp_psipsi_dps(wgts)
    RETURN
  END SUBROUTINE outfun_pp_psipsi_dps

  SUBROUTINE plot_begin_pp_psipsi_dps(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    INCLUDE '../../../analysis/hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin_pp_psipsi_dps: ',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook
       STOP
    ENDIF
    l=0
    CALL histogram_book(l+ 1,'total rate    ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2,'|Delta y|  ',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+ 3,'|Delta phi| ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+ 4,'psi psi inv. mass ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 5,'psi psi inv. mass zoom ',&
         100,5.5d0,10.5d0,.TRUE.)
    CALL histogram_book(l+ 6,'psi psi trans mom ',&
         100,0d0,100d0,.TRUE.)
    ! q12^2 = (Ejpsi1+Ejpsi2)^2-(pvecjpsi1+pvecjpsi2)^2-(2mjpsi)^2 
    CALL histogram_book(l+ 7,'q12 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 8,'q12 zoom ',&
         100,0d0,4d0,.TRUE.)
    ! alpha is the angle between the decay planes of Jpsi in the Di-Jpsi rest frame                     
    CALL histogram_book(l+ 9,'Cos(alpha) ',&
         100,0d0,1d0,.FALSE.)
    CALL histogram_book(l+ 10,'|Delta eta| ',&
         100,0d0,4d0,.TRUE.)
    CALL histogram_book(l+ 11,'random Pt 1',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 12,'random Pt 2',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 13,'leading Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 14,'subleading Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 15,'Pt diff ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 16,'|Pt1-Pt2| ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book3d(l+ 17,'Cos(th1)-Cos(th2) ',&
         100,-1d0,1d0,100,-1d0,1d0,.FALSE.)
    CALL histogram_book3d(l+ 18,'|Delta y| vs. Pt(psi-psi)',&
         100,0d0,5d0,100,0d0,100d0,.TRUE.)
    CALL histogram_book3d(l+ 19,'|Delta y| vs. |Delta phi|',&
         100,0d0,5d0,100,-0.0001d0,3.1416d0,.FALSE.)
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin_pp_psipsi_dps

  SUBROUTINE plot_end_pp_psipsi_dps(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../../analysis/hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file_pp_psipsi_dps
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file_pp_psipsi_dps
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file_pp_psipsi_dps
       psfile="pp_psipsi_dps.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file_pp_psipsi_dps
    ENDIF
    IF(root_output)THEN
       CALL open_root_file_pp_psipsi_dps
       rootfile="pp_psipsi_dps.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file_pp_psipsi_dps
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file_pp_psipsi_dps
    ENDIF
    RETURN
  END SUBROUTINE plot_end_pp_psipsi_dps

  SUBROUTINE plot_fill_pp_psipsi_dps(wgts)
    USE Helac_Global
    IMPLICIT NONE
    !INCLUDE "pp_psipsi_dps.inc"
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,yftot
    IF(NDecayChains.GT.0)THEN
       CALL plot_fill_Decay_pp_psipsi_dps(wgts)
       RETURN
    ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    pini(1:4)=dps_hadron_pmom(1,1:4)+dps_hadron_pmom(2,1:4) ! (px,py,pz,E)
    yftot=rapidity(pini)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!                                               
    !                                                                                                   
    ! START TO FILL HISTOGRAM                                                                           
    !                                                                                                   
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_FILL(l+1,var,wgts)
    CALL HISTOGRAM_FILL(l+2,yftot,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    ! boost back to c.m. frame
    RETURN
  END SUBROUTINE plot_fill_pp_psipsi_dps

  SUBROUTINE HwU_write_file_pp_psipsi_dps
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0))::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//'pp_psipsi_dps.HwU',&
         status='unknown')
    xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file_pp_psipsi_dps

  SUBROUTINE open_topdrawer_file_pp_psipsi_dps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//'pp_psipsi_dps.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file_pp_psipsi_dps

  SUBROUTINE close_topdrawer_file_pp_psipsi_dps
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file_pp_psipsi_dps

  SUBROUTINE open_gnuplot_file_pp_psipsi_dps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//'pp_psipsi_dps.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file_pp_psipsi_dps

  SUBROUTINE close_gnuplot_file_pp_psipsi_dps
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file_pp_psipsi_dps

  SUBROUTINE open_root_file_pp_psipsi_dps
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//'pp_psipsi_dps.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file_pp_psipsi_dps

  SUBROUTINE close_root_file_pp_psipsi_dps
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file_pp_psipsi_dps

  SUBROUTINE plot_fill_Decay_pp_psipsi_dps(wgts)
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "stdhep_pp_psipsi_dps.inc"
    REAL(KIND(1d0))::var,Q,www,dy_psipsi,dphi_psipsi,M_psipsi,Pt_psipsi,Pt1Pt2_psipsi,deta_psipsi
    REAL(KIND(1d0))::q12_psipsi,cosalpha,ptpsi1,ptpsi2,ptpsimax,ptpsimin,ptpsidiff
    INTEGER::i,m,l,kk
    REAL(KIND(1d0)),DIMENSION(4)::PBOO
    REAL(KIND(1d0)),DIMENSION(2,4)::PJpsi,PJpsiCMS
    REAL(KIND(1d0)),DIMENSION(4,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    REAL(KIND(1d0)),DIMENSION(3)::Pplain1,Pplain2
    REAL(KIND(1d0)),DIMENSION(3)::Plepvec1,Plepvec2
    REAL(KIND(1d0)),DIMENSION(2,3)::PJpsi1l,PJpsi2l
    REAL(KIND(1d0))::PP1,PP2
    REAL(KIND(1d0))::costh1,costh2
    INTEGER,DIMENSION(4)::lep2jpsi
    INTEGER,DIMENSION(2)::newmoth
    INTEGER::i_jpsi,i_lep,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton 
    i_jpsi=0
    i_lep=0
    lep2jpsi(1:4)=0
    newmoth(1:2)=0
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
       IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
       i_lep=i_lep+1
       Plep(i_lep,1:4)=PHEP(1:4,i)
       m=JMOHEP(1,i)
       definemoth=0
       DO kk=1,i_jpsi
          IF(newmoth(kk).EQ.m)THEN
             definemoth=kk
             EXIT
          END IF
       ENDDO
       IF(definemoth.NE.0)THEN
          ! the mother has been defined
          lep2jpsi(i_lep)=definemoth
       ELSE
          ! create the new mother
          i_jpsi=i_jpsi+1
          PJpsi(i_jpsi,1:4)=PHEP(1:4,m)
          newmoth(i_jpsi)=m
          lep2jpsi(i_lep)=i_jpsi
          IF(i_lep.EQ.4)EXIT
       ENDIF
    ENDDO
    IF(literature_cutoffs.EQ.14080000)THEN
       ! LHCb Kinematics Region
       ptpsi1=transverse(PJpsi(1,1:4))
       IF(ptpsi1.GT.10d0)THEN
          RETURN
       ENDIF
       ptpsi2=transverse(PJpsi(2,1:4))
       IF(ptpsi2.GT.10d0)THEN
          RETURN
       ENDIF
    ENDIF
    dy_psipsi=DABS(rapidity(PJpsi(1,1:4))-rapidity(PJpsi(2,1:4)))
    deta_psipsi=DABS(prapidity(PJpsi(1,1:4))-prapidity(PJpsi(2,1:4)))
    dphi_psipsi=ph4(PJpsi(1,1),PJpsi(1,2),PJpsi(1,3))
    dphi_psipsi=dphi_psipsi-ph4(PJpsi(2,1),PJpsi(2,2),PJpsi(2,3))
    dphi_psipsi=DABS(dphi_psipsi)
    IF(dphi_psipsi.GT.pi)dphi_psipsi=2*pi-dphi_psipsi
    M_psipsi=scalar_product(PJpsi(1,1:4)+PJpsi(2,1:4),PJpsi(1,1:4)+PJpsi(2,1:4))
    M_psipsi=DSQRT(M_psipsi)
    Q=scalar_product(PJpsi(1,1:4),PJpsi(1,1:4))
    Q=DSQRT(Q)
    ptpsi1=transverse(PJpsi(1,1:4))
    ptpsi2=transverse(PJpsi(2,1:4))
    ptpsimax=MAX(ptpsi1,ptpsi2)
    ptpsimin=MIN(ptpsi1,ptpsi2)
    ptpsidiff=ptpsimax-ptpsimin
    Pt_psipsi=transverse(PJpsi(1,1:4)+PJpsi(2,1:4))
    Pt1Pt2_psipsi=transverse(PJpsi(1,1:4)-PJpsi(2,1:4))
    q12_psipsi=M_psipsi**2-4*Q**2
    q12_psipsi=DSQRT(DABS(q12_psipsi))
    PBOO(1:3)=-PJpsi(1,1:3)-PJpsi(2,1:3)
    PBOO(4)=PJpsi(1,4)+PJpsi(2,4)
    DO i=1,4
       CALL BOOSTL(M_psipsi,PBOO,Plep(i,1:4))
    ENDDO
    DO i=1,2
       PJpsiCMS(i,1:4)=PJpsi(i,1:4)
       CALL BOOSTL(M_psipsi,PBOO,PJpsiCMS(i,1:4))
    ENDDO
    kk1=0
    kk2=0
    DO i=1,4
       IF(lep2jpsi(i).EQ.1)THEN
          kk1=kk1+1
          PJpsi1l(kk1,1:3)=Plep(i,1:3)
          IF(kk1.EQ.1)THEN
             Plep1(1:4)=Plep(i,1:4)
          ENDIF
       ELSEIF(lep2jpsi(i).EQ.2)THEN
          kk2=kk2+1
          PJpsi2l(kk2,1:3)=Plep(i,1:3)
          IF(kk2.EQ.1)THEN
             Plep2(1:4)=Plep(i,1:4)
          ENDIF
       ENDIF
    ENDDO
    Pplain1(1:3)=vec_crossprod(PJpsi1l(1,1:3),PJpsi1l(2,1:3))
    Pplain2(1:3)=vec_crossprod(PJpsi2l(1,1:3),PJpsi2l(2,1:3))
    PP1=Pplain1(1)**2+Pplain1(2)**2+Pplain1(3)**2
    PP1=DSQRT(PP1)
    PP2=Pplain2(1)**2+Pplain2(2)**2+Pplain2(3)**2
    PP2=DSQRT(PP2)
    IF(PP1*PP2.EQ.0d0)THEN
       cosalpha=0d0
    ELSE
       cosalpha=Pplain1(1)*Pplain2(1)+Pplain1(2)*Pplain2(2) &
            +Pplain1(3)*Pplain2(3)
       cosalpha=DABS(cosalpha/(PP1*PP2))
    ENDIF
    ! costh1
    Q=scalar_product(PJpsiCMS(1,1:4),PJpsiCMS(1,1:4))
    Q=DSQRT(Q)
    PBOO(1:3)=-PJpsiCMS(1,1:3)
    PBOO(4)=PJpsiCMS(1,4)
    CALL BOOSTL(Q,PBOO,Plep1)
    costh1=cosij(Plep1(1:3),PJpsiCMS(1,1:3))
    ! costh2
    Q=scalar_product(PJpsiCMS(2,1:4),PJpsiCMS(2,1:4))
    Q=DSQRT(Q)
    PBOO(1:3)=-PJpsiCMS(2,1:3)
    PBOO(4)=PJpsiCMS(2,4)
    CALL BOOSTL(Q,PBOO,Plep2)
    costh2=cosij(Plep2(1:3),PJpsiCMS(2,1:3))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,dy_psipsi,wgts)
    CALL HISTOGRAM_fill(l+3,dphi_psipsi,wgts)
    CALL HISTOGRAM_fill(l+4,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+5,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+6,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill(l+7,q12_psipsi,wgts)
    CALL HISTOGRAM_fill(l+8,q12_psipsi,wgts)
    CALL HISTOGRAM_fill(l+9,cosalpha,wgts)
    CALL HISTOGRAM_fill(l+10,deta_psipsi,wgts)
    CALL HISTOGRAM_fill(l+11,ptpsi1,wgts)
    CALL HISTOGRAM_fill(l+12,ptpsi2,wgts)
    CALL HISTOGRAM_fill(l+13,ptpsimax,wgts)
    CALL HISTOGRAM_fill(l+14,ptpsimin,wgts)
    CALL HISTOGRAM_fill(l+15,ptpsidiff,wgts)
    CALL HISTOGRAM_fill(l+16,Pt1Pt2_psipsi,wgts)
    CALL HISTOGRAM_fill3d(l+17,costh1,costh2,wgts)
    CALL HISTOGRAM_fill3d(l+18,dy_psipsi,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill3d(l+19,dy_psipsi,dphi_psipsi,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay_pp_psipsi_dps
END MODULE plot_pp_psipsi_dps
