MODULE pp_psipsi_dps2_functions
  IMPLICIT NONE
CONTAINS

  SUBROUTINE DiracDot(ubar,gamma,new_ubar)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),DIMENSION(4),INTENT(IN)::ubar
    COMPLEX(KIND(1d0)),DIMENSION(4,4),INTENT(IN)::gamma
    COMPLEX(KIND(1d0)),DIMENSION(4),INTENT(OUT)::new_ubar
    INTEGER::i,j
    DO i=1,4
      new_ubar(i)=dcmplx(0d0,0d0)
      DO j=1,4
        new_ubar(i)=new_ubar(i)+ubar(j)*gamma(j,i)
      ENDDO
    ENDDO
    RETURN
  END SUBROUTINE DiracDot

  SUBROUTINE Gamma5(ga5)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),DIMENSION(4,4),INTENT(OUT)::ga5
    COMPLEX(KIND(1d0)),PARAMETER::ONE=dcmplx(1d0,0d0)
    COMPLEX(KIND(1d0)),PARAMETER::ZERO=dcmplx(0d0,0d0)
    INTEGER::i,j
    DO i=1,4
       DO j=1,4
          IF(i.NE.j)THEN
             ga5(i,j)=ZERO
          ELSE
             IF(i.LE.2)THEN
                ga5(i,j)=ONE
             ELSE
                ga5(i,j)=-ONE
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    RETURN
  END SUBROUTINE Gamma5

  SUBROUTINE GammaSlash_R(pmom,gs)
    REAL(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::pmom
    COMPLEX(KIND(1d0)),DIMENSION(4,4),INTENT(OUT)::gs
    COMPLEX(KIND(1d0)),PARAMETER::im=dcmplx(0d0,1d0)
    COMPLEX(KIND(1d0)),PARAMETER::zero=dcmplx(0d0,0d0)
    INTEGER::i,j
    DO i=1,4
      DO j=1,4
      IF((i.LE.2.AND.j.GE.3).OR.(i.GE.3.AND.j.LE.2))THEN
        SELECT CASE(i)
        CASE(1)
          IF(j.EQ.3)THEN
            gs(i,j)=-dcmplx(pmom(0)+pmom(3),0d0)
          ELSE
            gs(i,j)=dcmplx(-pmom(1),pmom(2))
          ENDIF
        CASE(2)
          IF(j.EQ.3)THEN
            gs(i,j)=dcmplx(-pmom(1),-pmom(2))
          ELSE
            gs(i,j)=dcmplx(pmom(3)-pmom(0),0d0)
          ENDIF
        CASE(3)
          IF(j.EQ.1)THEN
            gs(i,j)=dcmplx(pmom(3)-pmom(0),0d0)
          ELSE
            gs(i,j)=dcmplx(pmom(1),-pmom(2))
          ENDIF
        CASE(4)
          IF(j.EQ.1)THEN
            gs(i,j)=dcmplx(pmom(1),pmom(2))
          ELSE
            gs(i,j)=dcmplx(-pmom(0)-pmom(3),0d0)
          ENDIF
        END SELECT
      ELSE
        gs(i,j)=zero
      ENDIF
      ENDDO
    ENDDO
    RETURN
  END SUBROUTINE GammaSlash_R

  SUBROUTINE GammaSlash_C(pmom,gs)
    COMPLEX(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::pmom
    COMPLEX(KIND(1d0)),DIMENSION(4,4),INTENT(OUT)::gs
    COMPLEX(KIND(1d0)),PARAMETER::im=dcmplx(0d0,1d0)
    COMPLEX(KIND(1d0)),PARAMETER::zero=dcmplx(0d0,0d0)
    INTEGER::i,j
    DO i=1,4
      DO j=1,4
      IF((i.LE.2.AND.j.GE.3).OR.(i.GE.3.AND.j.LE.2))THEN
        SELECT CASE(i)
        CASE(1)
          IF(j.EQ.3)THEN
            gs(i,j)=-pmom(0)-pmom(3)
          ELSE
            gs(i,j)=-pmom(1)+im*pmom(2)
          ENDIF
        CASE(2)
          IF(j.EQ.3)THEN
            gs(i,j)=-pmom(1)-im*pmom(2)
          ELSE
            gs(i,j)=pmom(3)-pmom(0)
          ENDIF
        CASE(3)
          IF(j.EQ.1)THEN
            gs(i,j)=pmom(3)-pmom(0)
          ELSE
            gs(i,j)=pmom(1)-im*pmom(2)
          ENDIF
        CASE(4)
          IF(j.EQ.1)THEN
            gs(i,j)=pmom(1)+im*pmom(2)
          ELSE
            gs(i,j)=-pmom(0)-pmom(3)
          ENDIF
        END SELECT
      ELSE
        gs(i,j)=zero
      ENDIF
      ENDDO
    ENDDO
    RETURN
  END SUBROUTINE GammaSlash_C

  SUBROUTINE FermionContract(ubar,v,fc)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),DIMENSION(4),INTENT(IN)::ubar,v
    COMPLEX(KIND(1d0)),INTENT(OUT)::fc
    INTEGER::i
    fc=dcmplx(0d0,0d0)
    DO i=1,4
      fc=fc+ubar(i)*v(i)
    ENDDO
    RETURN
  END SUBROUTINE FermionContract

  SUBROUTINE zq_generate(pmom,mass,zzqq)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::pmom
    REAL(KIND(1d0)),INTENT(IN)::mass
    COMPLEX(KIND(1d0)),DIMENSION(5),INTENT(OUT)::zzqq
    zzqq(1)=dcmplx(pmom(0)+pmom(3),pmom(3))
    zzqq(2)=dcmplx(pmom(0)-pmom(3),dsqrt(pmom(1)**2+pmom(2)**2))
    zzqq(3)=dcmplx(pmom(1),pmom(2))
    zzqq(4)=dcmplx(pmom(1),-pmom(2))
    zzqq(5)=dcmplx(mass,dsqrt(pmom(1)**2+pmom(2)**2+pmom(3)**2))
    RETURN
  END SUBROUTINE zq_generate

  SUBROUTINE lightcone2normal(zy,eps)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),DIMENSION(4),INTENT(IN)::zy
    COMPLEX(KIND(1d0)),DIMENSION(0:3),INTENT(OUT)::eps
    COMPLEX(KIND(1d0)),PARAMETER::im=dcmplx(0d0,1d0)
    eps(0)=(zy(1)+zy(2))/2d0
    eps(1)=(zy(3)+zy(4))/2d0
    eps(2)=(zy(3)-zy(4))/2d0/im
    eps(3)=(zy(1)-zy(2))/2d0
    RETURN
  END SUBROUTINE lightcone2normal

  SUBROUTINE LORENTZDOT_RR(vec1,vec2,sij)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::vec1,vec2
    COMPLEX(KIND(1d0)),INTENT(OUT)::sij
    sij=dcmplx(vec1(0)*vec2(0)-vec1(1)*vec2(1)-vec1(2)*vec2(2)-vec1(3)*vec2(3),0d0)
    RETURN
  END SUBROUTINE LORENTZDOT_RR

  SUBROUTINE LORENTZDOT_RC(vec1,vec2,sij)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::vec1
    COMPLEX(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::vec2
    COMPLEX(KIND(1d0)),INTENT(OUT)::sij
    sij=dcmplx(vec1(0),0d0)*vec2(0)-dcmplx(vec1(1),0d0)*vec2(1)-dcmplx(vec1(2),0d0)*vec2(2)-dcmplx(vec1(3),0d0)*vec2(3)
    RETURN
  END SUBROUTINE LORENTZDOT_RC

  SUBROUTINE LORENTZDOT_CC(vec1,vec2,sij)
    IMPLICIT NONE
    COMPLEX(KIND(1d0)),DIMENSION(0:3),INTENT(IN)::vec1,vec2
    COMPLEX(KIND(1d0)),INTENT(OUT)::sij
    sij=vec1(0)*vec2(0)-vec1(1)*vec2(1)-vec1(2)*vec2(2)-vec1(3)*vec2(3)
    RETURN
  END SUBROUTINE LORENTZDOT_CC

END MODULE pp_psipsi_dps2_functions
