MODULE hwu_hepmc_user
!============================================================
!
! GENERATE HWU FILE FROM HepMC FOR USER
! This is an example file for monojet, p p > xd xd~ j [QCD]
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=12
  INTEGER,PARAMETER::n_width=1,n_height=1
  INTEGER::N_PSPoints=0,N_PSPoints_m=0,N_PSPoints_p=0
  SAVE N_PSPoints,N_PSPoints_m,N_PSPoints_p
CONTAINS
  SUBROUTINE hwu_hepmc_begin(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    INTEGER,PARAMETER::max_weight=500
    CHARACTER(len=15),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    CHARACTER(len=6)::cc=' NLOPS'

    call HwU_inithist(nwgt,weights_info)
    ! Set method for error estimation to '0', i.e., use Poisson statistics 
    ! for the uncertainty estimate
    call set_error_estimation(0)
    l=0
    call HwU_book(l+ 1,'P_T^{j1} '//cc,100,0d0,2000d0)
    call HwU_book(l+ 2,'log(P_T^{j1}) '//cc, 100,0d0,5d0)
    call HwU_book(l+ 3,'eta(j1) '//cc,100,-5d0,5d0)
    call HwU_book(l+ 4,'P_T^{j2} '//cc,100,0d0,1500d0)
    call HwU_book(l+ 5,'log(P_T^{j2}) '//cc, 100,0d0,5d0)
    call HwU_book(l+ 6,'eta(j2) '//cc,100,-5d0,5d0)
    call HwU_book(l+ 7,'E_{miss} '//cc,100,0d0,2000d0)
    call HwU_book(l+ 8,'log(E_{miss}) '//cc,100,0d0,5d0)
    call HwU_book(l+ 9,'XS MET > 150 '//cc,2,0d0,4d0)
    call HwU_book(l+10,'XS MET > 300 '//cc,2,0d0,4d0)
    call HwU_book(l+11,'XS MET > 500 '//cc,2,0d0,4d0)
    call HwU_book(l+12,'XS MET > 0   '//cc,2,0d0,4d0)
    RETURN
  END SUBROUTINE hwu_hepmc_begin

  SUBROUTINE hwu_hepmc_end(dummy)
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::dummy
    INTEGER::i
    INTEGER::kk,l
    ! the following is for the fixed order after each iteration
    ! first argument = .TRUE., taking into account the prevous iteration 
    ! call HwU_accum_iter(.FALSE.,N_PSPoints)
    WRITE(*,*)"INFO:WE HAVE ACCUMULATED THE NUMBER OF THE VALID UNWEIGHTED EVENTS=",N_PSPoints
    IF(N_PSPoints_m.GT.0)THEN
       WRITE(*,*)"INFO:THE NUMBER OF NEGATIVE-UNWEIGHTED EVENTS=",N_PSPoints_m
       WRITE(*,*)"INFO:THE NUMBER OF POSITIVE-UNWEIGHTED EVENTS=",N_PSPoints_p
    ENDIF
    call finalize_histograms(1)
    call HwU_write_file
    RETURN                
  END SUBROUTINE hwu_hepmc_end

  SUBROUTINE hwu_hepmc_fill(wgts)
    IMPLICIT NONE
    INCLUDE '../include/HEPMC90.INC'
    !INCLUDE "../hbook/HwU90.inc"
    REAL(KIND(1d0)),DIMENSION(4)::psyst
    REAL(KIND(1d0)),DIMENSION(250),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,j,l,kk,IHEP,NB,NDM,NBJET,NTRACKS,NJET,IST,ID,ID1
    INTEGER::count_j,count_bj
    REAL(KIND(1d0))::www,JET_KTRADIUS,JET_KTPTMIN,PALG,JET_KTETA
    INTEGER,PARAMETER::MAXTRACK=2048,MAXJET=2048,MAXNUM=30
    REAL(KIND(1d0)),DIMENSION(4,MAXNUM)::P_DM,P_B,P_BJET
    REAL(KIND(1d0)),DIMENSION(4,MAXTRACK)::PJET,PTRACK
    REAL(KIND(1d0)),DIMENSION(4)::P_MISS
    INTEGER,DIMENSION(MAXNUM)::IB,BTRACK
    LOGICAL,DIMENSION(MAXNUM)::IS_B_JET
    INTEGER,DIMENSION(MAXTRACK)::JETVEC
    REAL(KIND(1d0))::Etmiss,ptj1,etaj1
    REAL(KIND(1d0))::ptj2,etaj2
    ! INCOMING PARTONS MAY TRAVEL IN THE SAME DIRECTION: IT''S 
    ! A POWER-SUPPRESSED EFFECT, SO THROW THE EVENT AWAY
    IF(SIGN(1.D0,PHEP(3,1)).EQ.SIGN(1.D0,PHEP(3,2)))THEN
       WRITE(*,*)"WARNING 1 in hwu_hepmc_fill"
       GOTO 999
    ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! IMPORT INFORMATION AND DEFINE JETS
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    NDM=0 ! number of dark matter
    NB=0 ! number of b
    NBJET=0 ! number of b jet
    NTRACKS=0 ! number of tracks
    NJET=0 ! number of non-b jet
    
    DO IHEP=1,NHEP
       IST=ISTHEP(IHEP) ! status
       ID=IDHEP(IHEP) ! pdg
       ID1=IHADR(ID) ! the massive quark's PDG in hadron, defined later

       ! dark matter
       IF(ABS(ID).EQ.52)THEN
          NDM=NDM+1
          DO i=1,4
             P_DM(i,NDM)=PHEP(i,IHEP)
          ENDDO
       ENDIF

       ! define particles that to into jet.
       IF(IST.EQ.1.AND.ABS(ID).GE.100)THEN ! final + hadron
          NTRACKS=NTRACKS+1
          IF(ABS(ID1).EQ.5)THEN
             ! FOUND a stable B hadron
             NB=NB+1
             IB(NB)=IHEP ! the index for b hadron
             DO i=1,4
                P_B(i,NB)=PHEP(i,IHEP)
             ENDDO
             BTRACK(NB)=NTRACKS ! the index of tracks (with b hadron)
          ENDIF
          DO i=1,4
             PTRACK(i,NTRACKS)=PHEP(i,IHEP)
          ENDDO
          IF(NTRACKS.EQ.MAXTRACK)THEN
             WRITE(*,*)"ERROR: Too many particles in hwu_hepmc_fill, please increase MATRACK !"
             STOP
          ENDIF
       ENDIF
    ENDDO
    P_MISS(1:4)=0d0
    DO i=1,NDM
       P_MISS(1:4)=P_MISS(1:4)+P_DM(1:4,i)
    ENDDO

    IF(NTRACKS.EQ.0)THEN
       WRITE(*,*)'WARNING:NO TRACKS FOUND, DROP THIS EVENT'
       GOTO 999
    ENDIF

    ! KT Algorithm with fastjet
    NJET=0
    JET_KTRADIUS = 0.4d0 ! R0
    JET_KTPTMIN = 30d0  ! pt cut on the jet
    JET_KTETA = 4.5d0   ! eta cut on the jet
    PALG=-1d0 ! 1d0: kt algorithm, 0d0: C/A, -1d0: anti-kt algorithm
    CALL fastjetppgenkt_etamax(PTRACK,NTRACKS,JET_KTRADIUS,JET_KTPTMIN,JET_KTETA,&
         PALG,PJET,NJET,JETVEC) ! fastjet internal function: the returned 
                           ! PJET (momentum),NJET(number of jets)
                           ! JETVEC(itrack) is the jet index for itrack TRACK

    ! check that jets are orderd in pt
    IF(njet.LT.1)THEN
       ! skip the event if there is no jet passed the jet cut
       RETURN
    ENDIF
    DO i=1,NJET-1
       IF(GETPT(PJET(1:4,i)).LT.GETPT(PJET(1:4,i+1)))THEN
          WRITE(*,*)"ERROR: Jets are not ordered in hwu_hepmc_fill"
          STOP
       ENDIF
    ENDDO

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    Etmiss=GETPT(P_MISS(1:4))

    count_j=0 ! number of jets
    DO i=1,NJET
       count_j=count_j+1
       IF(count_j.EQ.1)THEN
          ! leading jet
          ptj1=GETPT(PJET(1:4,i))
          etaj1=prapidity(PJET(1:4,i))
       ELSEIF(count_j.EQ.2)THEN
          ! subleading jet
          ptj2 = GETPT(PJET(1:4,i))
          etaj2 = prapidity(PJET(1:4,i))
       ENDIF
    ENDDO
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    N_PSPoints=N_PSPoints+1
    IF(wgts(1).GE.0d0)THEN
       N_PSPoints_p=N_PSPoints_p+1
    ELSE
       N_PSPoints_m=N_PSPoints_m+1
    ENDIF
    l=0
    if(njet.ge.1)then
       call HwU_fill(l+1,ptj1,wgts)
       if (ptj1.gt.0d0) call HwU_fill(l+2,log10(ptj1),wgts)
       call HwU_fill(l+3,etaj1,wgts)
    endif
    if(njet.ge.2)then
       call HwU_fill(l+4,ptj2,wgts)
       if(ptj2.gt.0d0) call HwU_fill(l+5,log10(ptj2),wgts)
       call HwU_fill(l+6,etaj2,wgts)
    endif
    call HwU_fill(l+7,Etmiss,wgts)
    if(Etmiss.gt.0d0) call HwU_fill(l+8,log10(Etmiss),wgts)
    if(Etmiss.gt.150d0) call HwU_fill(l+9,1d0,wgts)
    if(Etmiss.gt.300d0) call HwU_fill(l+10,1d0,wgts)
    if(Etmiss.gt.500d0) call HwU_fill(l+11,1d0,wgts)
    call HwU_fill(l+12,1d0,wgts)
    ! add the points to the histogram
    call HwU_add_points ! add it after alll correlated contribution has been added, i.e. virt+couterevent+born+real
999 RETURN      
  END SUBROUTINE hwu_hepmc_fill

  SUBROUTINE HwU_write_file
    IMPLICIT NONE
    REAL(KIND(1d0))::xnorm
    OPEN(unit=599,file='helaconia_hepmc.HwU',status='unknown')
    xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

! Kinematic functions
  FUNCTION rapidity(p)
    ! rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::rapidity,c
    IF(p(4).EQ.0d0)THEN
       rapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/ABS(p(4))
    IF(ABS(c).GE.1d0)THEN
       rapidity =0d0
    ELSE
       rapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION rapidity

  FUNCTION prapidity(p)
    ! psudo-rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::prapidity,c
    IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
       prapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
    IF(ABS(c).EQ.1d0)THEN
       prapidity = 0d0
    ELSE
       prapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION prapidity

  FUNCTION GETPT(p)
    ! pt
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::GETPT
    GETPT=DSQRT(p(1)**2+p(2)**2)
    RETURN
  END FUNCTION GETPT

  FUNCTION MINV(p)
    ! invariant mass (NOT mass squared)
    ! timelike : positive
    ! spacelike : negative
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::MINV
    MINV=p(4)**2-p(1)**2-p(2)**2-p(3)**2
    IF(MINV.GE.0d0)THEN
       MINV=DSQRT(MINV)
    ELSE
       MINV=-DSQRT(ABS(MINV))
    ENDIF
    RETURN
  END FUNCTION MINV

  ! returns the PDG code of the heavier quark in the hadron of PDG code ID
  FUNCTION IHADR(ID)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ID
    INTEGER::IHADR,ID1
    IF(ID.NE.0)THEN
       ID1=ABS(ID)
       IF(ID1.GT.10000)ID1=ID1-1000*INT(ID1/1000)
       IHADR=ID1/(10**INT(LOG10(DFLOAT(ID1))))
    ELSE
       IHADR=0
    ENDIF
    RETURN
  END FUNCTION IHADR

END MODULE hwu_hepmc_user
