MODULE analysis_plot
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=9
  INTEGER,PARAMETER::n_width=1,n_height=1
  LOGICAL,PARAMETER::topdrawer_output=.TRUE.
  LOGICAL,PARAMETER::gnuplot_output=.TRUE.
  LOGICAL,PARAMETER::root_output=.TRUE.
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    INCLUDE '../../hbook/dbookf90.inc'
    CALL INIHIST
    CALL INIHIST_3D
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    DO kk=1,nwgt_analysis
       l=(kk-1)*num_plots
       ! bookup is defined in analysis/hbook/dbook.f
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       !
       ! enlarge num_plots at the begining
       !
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       CALL bookup(l+ 1,'total rate    '//weights_info(kk),&
            1.0d0,0.5d0,5.5d0)
       CALL bookup(l+ 2, 'total rate BW '//weights_info(kk),&
            1.0d0,0.5d0,5.5d0)
       CALL bookup(l+ 3,'vtx displacement [micrometer] '//weights_info(kk),&
            100,0d0,50d0)
       CALL bookup(l+ 4,'log10(vtx displacement) [log10(m)] '//weights_info(kk),&
            100,-20d0,0d0)
       CALL bookup(l+ 5,'vtx displacement BW [micrometer] '//weights_info(kk),&
            100,0d0,50d0)
       CALL bookup(l+ 6,'log10(vtx displacement BW) [log10(m)] '//weights_info(kk),&
            100,-20d0,0d0)
       CALL bookup(l+ 7,'m(aa) delta '//weights_info(kk),&
            100,0d0,10d0)
       CALL bookup(l+ 8,'m(aa) BW '//weights_info(kk),&
            100,0d0,10d0)
       CALL bookup(l+ 9,'m(aa) BW zoom in '//weights_info(kk),&
            40, 2.8d0, 4d0)
    ENDDO
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    CALL mclear
    CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'+',i,i,xnorm,0.d0)
       CALL mfinal(i)
       CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
       CALL mfinal_3d(i)
    ENDDO
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       DO kk=1,nwgt_analysis
          l=(kk-1)*num_plots
          CALL multitop(l+ 1,n_width,n_height,'total rate   ',ytit,'LIN')
       END DO
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile="plot_lhe.ps"
       DO kk=1,nwgt_analysis
          l=(kk-1)*num_plots
          CALL MGNUPLOT(l+ 1,'total rate   ',ytit,'LIN',psfile)
       ENDDO
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile="plot_lhe.root"
       DO kk=1,nwgt_analysis
          l=(kk-1)*num_plots
          CALL MROOTPLOT(l+ 1,'total rate  ',ytit,'LIN',rootfile)
       ENDDO
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2,maa,maa_new
    INTEGER::i,kk,l,NP
    REAL(KIND(1d0))::www,wwwnew,rrr,tt,dvtx,dvtxBW
    INTEGER::ipdg_resonance,numFa,numIa
    REAL(KIND(1d0))::mass_res,width_res
    REAL(KIND(1d0)),DIMENSION(2,4)::xpI_a,xpF_a,pD1
    REAL(KIND(1d0)),DIMENSION(4)::xpF_aa
    REAL(KIND(1d0)),DIMENSION(3,4)::PA,PO
    REAL(KIND(1d0)),DIMENSION(3)::MA
    REAL(KIND(1d0))::xss,xu,xl,pmax2,pmin2,bwcuts
    REAL(KIND(1d0))::xa,xb,xanew,xbnew,compensatefact
    ! User process initialization common block
    INCLUDE 'init_common_block_f90.inc'
    ! User process event common block.   
    INCLUDE 'event_common_block_f90.inc'
    ! MAXNUP: max number of particles in each event
    ! NUP: number of particles in the event
    ! IDPRUP: the id of the process
    ! IDUP(MAXNUP): the pdg of each particle in the event
    ! ISTUP(MAXUP): the final state (1) or the initial state (-1)
    ! MOTHUP(2,MAXNUP): the mother of each particle in the event
    ! ICOLUP(2,MAXNUP): the color flow of each particle in the event
    ! XWGTUP: the weight of the event
    ! SCALUP: the scale of the event
    ! AQEDUP: alpha_EW value in the event
    ! AQCDUP: alpha_s value in the event
    ! PUP(5,MAXNUP): (px,py,pz,E,m) of the particle in the event
    ! VTIMUP(MAXNUP): life of the particle in the event
    ! SPINUP(MAXNUP): spin of the particle in the event
    IF(wgts(1).EQ.0d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    numIa=0
    numFa=0
    ipdg_resonance=0
    DO i=1,NUP
       IF(ISTUP(i).EQ.2)THEN
          ! it is a resonance
          ipdg_resonance=IDUP(i)
       ELSEIF(ISTUP(i).EQ.-1.AND.IDUP(i).EQ.22)THEN
          ! initial state photon
          numIa=numIa+1
          xpI_a(numIa,1:4)=PUP(1:4,i)
       ELSEIF(ISTUP(i).EQ.1.AND.IDUP(i).EQ.22)THEN
          ! final state photon
          numFa=numFa+1
          xpF_a(numFa,1:4)=PUP(1:4,i)
       ELSE
          CYCLE
       ENDIF
    ENDDO
    IF(numIa.NE.2)THEN
       WRITE(*,*)"ERROR: number of initial photon is not equal to 2 !"
       STOP
    ENDIF
    IF(numFa.NE.2)THEN
       WRITE(*,*)"ERROR: number of final photon is not equal to 2 !"
       STOP
    ENDIF
    xpF_aa(1:4)=xpF_a(1,1:4)+xpF_a(2,1:4)
    maa=PARTMINV(xpF_aa)
    ! physical mass and total width (in GeV) of the resonance
    CALL GETRESMASSWIDTH(ipdg_resonance,mass_res,width_res)
    ! the undecayed particle at t is
    ! dN[t]/dt=-Ga*N[t],  N[t] = N[0]*Exp[-Ga*t]
    ! Let us assume the probability density for a particle decayed at time t as p[t]
    ! p[t]=Ga*Exp[-Ga*t], which is 1 when integrating t from 0 to +infty
    ! At time t, the decayed fraction is
    ! Int[p[tt],{tt,0,t}] = 1 - Exp[-Ga*t]
    ! we can sample over undecayed particle fraction r = fd[t] = Exp[-Ga*t], then
    ! Int[p[tt],{tt,0,t}] = Int[1, {fd, Exp[-Ga*t], 1}]
    rrr=rand()
    ! decayed proper time of the resonance
    ! in unit of GeV-1
    tt=-DLOG(rrr)/width_res
    ! vertex displacement wrt the primary vertex (at origin)
    ! dvtx is in unit of meter
    CALL RESVERTEXDISPLACEMENT(xpF_aa,tt,dvtx)
    ! generate the m(aa) according to the BW distribution
    ! BW(p^2)=1/((p^2-M^2)^2+Ga^2*M^2)
    ! the probability density is f(p^2)=(Ga*M)/Pi*BW(p^2),
    ! which leads to Int[f[p^2],{p^2,-infty,+infty}] = 1
    ! If we only allow p^2 between pmin^2 and pmax^2 (instead of -infty and +infty)
    ! we can take p^2 = M^2+Ga*M*Tan[xss], which means
    ! xss = ArcTan[(p^2-M^2)/(Ga*M)] and dp^2 = dxss/f(p^2)/Pi
    bwcuts=20d0
    pmax2=(mass_res+width_res*bwcuts)**2
    pmin2=MAX(1d-2,mass_res-width_res*bwcuts)
    pmin2=pmin2**2
    xu=DATAN((pmax2-mass_res**2)/(mass_res*width_res))
    xl=DATAN((pmin2-mass_res**2)/(mass_res*width_res))
    rrr=rand()
    xss=rrr*xu+(1d0-rrr)*xl
    maa_new=mass_res**2+mass_res*width_res*TAN(xss)
    maa_new=DSQRT(maa_new)
    ! performing momentum reshuffling according to the new m(aa)
    NP=3
    PA(1,1:4)=xpI_a(1,1:4)
    PA(2,1:4)=xpI_a(2,1:4)
    PA(3,1:4)=xpF_aa(1:4)
    MA(1:2)=-1d0
    MA(3)=maa_new
    xa=xpI_a(1,4)/EBMUP(1)
    xb=xpI_a(2,4)/EBMUP(2)
    CALL InitialStateReshuffling2(NP,PA,MA,xa,xb,PO,xanew,xbnew)
    ! obtain the new momenta of the decayed photons
    CALL DecayedMomenta_ISReshuffling(pA(3,1:4),pO(3,1:4),xpF_a,pD1)
    ! the compensate factor from Muller flux and partonic luminosity (the second is still missing)
    compensatefact=(xa*xb)/(xanew*xbnew)
    ! vertex displacement wrt the primary vertex (at origin)
    ! dvtxBW is in unit of meter
    CALL RESVERTEXDISPLACEMENT(pO(3,1:4),tt,dvtxBW)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO kk=1,nwgt_analysis
       www=wgts(kk)
       wwwnew=www*compensatefact
       l=(kk-1)*num_plots
       CALL mfill(l+1,var,www)
       CALL mfill(l+2,var,wwwnew)
       ! in unit of micrometer
       CALL mfill(l+3,dvtx*1D6,www)
       CALL mfill(l+4,DLOG10(dvtx),www)
       ! in unit of micrometer
       CALL mfill(l+5,dvtxBW*1D6,wwwnew)
       CALL mfill(l+6,DLOG10(dvtxBW),wwwnew)
       CALL mfill(l+7,maa,www)
       CALL mfill(l+8,maa_new,wwwnew)
       CALL mfill(l+9,maa_new,wwwnew)
    ENDDO
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax                                                  
    OPEN(unit=99,FILE='plot_lhe.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='plot_lhe.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='plot_lhe.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE GETRESMASSWIDTH(ipdg,mass,width)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ipdg
    REAL(KIND(1d0)),INTENT(OUT)::mass,width
    SELECT CASE(ipdg)
    CASE(100441)
       ! etac(2S)
       mass=3.6375d0
       width=0.0113d0
    CASE(441)
       ! etac(1S)
       mass=2.9839d0
       width=0.032d0
    CASE(10441)
       ! chic0(1P)
       mass=3.41471d0
       width=0.0108d0
    CASE(445)
       ! chic2(1P)
       mass=3.55617d0
       width=0.00197d0
    CASE(15151)
       ! di-tau onium(1S01)
       mass=3.55d0
       width=1.84d-11
    CASE DEFAULT
       WRITE(*,*)"ERROR: do not know the particle = ",ipdg
       STOP
    END SELECT
    RETURN
  END SUBROUTINE GETRESMASSWIDTH

  SUBROUTINE RESVERTEXDISPLACEMENT(pres,tres,dvtx)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pres ! px,py,pz,E GeV
    REAL(KIND(1d0)),INTENT(IN)::tres ! in GeV-1
    REAL(KIND(1d0)),INTENT(OUT)::dvtx
    REAL(KIND(1d0))::mass,pp
    ! from http : // www.saha.ac.in/theory/palashbaran.pal/conv.html
    REAL(KIND(1d0)),PARAMETER::conv=1.9762845849802373d-16 ! from GeV-1 to meter
    pp=pres(1)**2+pres(2)**2+pres(3)**2
    mass=pres(4)**2-pp
    pp=DSQRT(pp)
    IF(mass.LT.0d0)mass=0d0
    mass=DSQRT(mass)
    ! in unit of meter
    dvtx=tres*pp/mass*conv
    RETURN
  END SUBROUTINE RESVERTEXDISPLACEMENT

  ! the following are momentum reshufflings
  ! refer to arXiv:1907.04898 (Option A)
  SUBROUTINE InitialStateReshuffling2(NP,PA,MA,xa,xb,PO,xanew,xbnew)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::NP
    REAL(KIND(1d0)),DIMENSION(NP,4),INTENT(IN)::PA ! original external momenta
    REAL(KIND(1d0)),DIMENSION(NP),INTENT(IN)::MA ! on-shell masses to be ajusted (negative, no need to ajust)
    REAL(KIND(1d0)),DIMENSION(NP,4),INTENT(OUT)::PO ! new external momenta after reshuffling
    REAL(KIND(1d0)),DIMENSION(4)::PSUM
    REAL(KIND(1d0)),INTENT(IN)::xa,xb ! original xa and xb
    REAL(KIND(1d0)),INTENT(OUT)::xanew,xbnew ! new xa and xb
    REAL(KIND(1d0))::snew,sold,stot
    REAL(KIND(1d0)),DIMENSION(4)::PIN
    REAL(KIND(1d0)),DIMENSION(NP,4)::PA_cms
    REAL(KIND(1d0)),DIMENSION(4)::PBOO
    REAL(KIND(1d0))::Q,EBEAM1,EBEAM2
    INTEGER::i
    IF(NP.GT.3)THEN
       ! always assume the first two momenta are the initial state
       ! boost to the rest frame of the parton-parton collisions
       DO I=1,NP
          PA_cms(I,1:4)=PA(I,1:4)
       ENDDO
       PIN(1:4)=PA(1,1:4)+PA(2,1:4)
       sold=PIN(4)**2-PIN(1)**2-PIN(2)**2-PIN(3)**2
       EBEAM1=PA(1,4)/xa
       EBEAM2=PA(2,4)/xb
       Q=DSQRT(sold)
       PBOO(4)=PA(1,4)+PA(2,4)
       PBOO(3)=-(PA(1,3)+PA(2,3))
       PBOO(1:2)=0d0
       DO I=1,NP
          !     When PBOO(4)=P(4) and PBOO(1:3)=-P(1:3), it boosts P to its rest frame (0,0,0,Q)                           
          CALL BOOSTL5(Q,PBOO(1:4),PA_cms(I,1:4))
       ENDDO
       ! always assume the first two momenta are the initial state
       PSUM(1:4)=0d0
       DO i=3,NP
          IF(MA(i).GT.0d0)THEN
             PO(i,1:3)=PA(i,1:3)
             PO(i,4)=DSQRT(MA(i)**2+PO(i,1)**2+PO(i,2)**2+PO(i,3)**2)
          ELSE
             PO(i,1:4)=PA(i,1:4)
          ENDIF
          PSUM(1:4)=PSUM(1:4)+PO(i,1:4)
       ENDDO
       snew=PSUM(4)**2-PSUM(1)**2-PSUM(2)**2-PSUM(3)**2
       stot=sold/xa/xb
       xanew=MIN(DSQRT(xa*snew/xb/stot),0.99999999999999999D0)
       xbnew=MIN(DSQRT(xb*snew/xa/stot),0.99999999999999999D0)
       PO(1,4)=DSQRT(snew)/2d0
       PO(1,3)=PO(1,4)
       PO(1,1:2)=0d0
       PO(2,4)=DSQRT(snew)/2d0
       PO(2,3)=-PO(2,4)
       PO(2,1:2)=0d0
       Q=DSQRT(snew)
       PBOO(4)=xanew*EBEAM1+xbnew*EBEAM2
       PBOO(3)=xanew*EBEAM1-xbnew*EBEAM2
       PBOO(1:2)=0d0
       DO I=1,NP
          CALL BOOSTL5(Q,PBOO(1:4),PO(I,1:4))
       ENDDO
    ELSE
       !  avoid roundingoff errors
       PIN(1:4)=PA(1,1:4)+PA(2,1:4)
       sold=PIN(4)**2-PIN(1)**2-PIN(2)**2-PIN(3)**2
       EBEAM1=PA(1,4)/xa
       EBEAM2=PA(2,4)/xb
       snew=MA(3)**2
       stot=sold/xa/xb
       xanew=MIN(DSQRT(xa*snew/xb/stot),0.9999999999999999D0)
       xbnew=MIN(DSQRT(xb*snew/xa/stot),0.9999999999999999D0)
       PO(1,4)=xanew*EBEAM1
       PO(1,3)=PO(1,4)
       PO(1,1:2)=0d0
       PO(2,4)=xbnew*EBEAM2
       PO(2,3)=-PO(2,4)
       PO(2,1:2)=0d0
       PO(3,1:4)=PO(1,1:4)+PO(2,1:4)
    ENDIF
    RETURN
  END SUBROUTINE InitialStateReshuffling2

  FUNCTION PARTMINV(P)
    IMPLICIT NONE
    REAL(KIND(1d0))::PARTMINV
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::P
    PARTMINV=P(4)**2-P(1)**2-P(2)**2-P(3)**2
    IF(PARTMINV.LT.0d0)PARTMINV=0d0
    PARTMINV=DSQRT(PARTMINV)
    RETURN
  END FUNCTION PARTMINV

  ! Eqs.(5.38,5.40) in arXiv:1907.04898
  FUNCTION PARTPION(m,m1,m2)
    IMPLICIT NONE
    REAL(KIND(1d0))::PARTPION
    REAL(KIND(1d0)),INTENT(IN)::m,m1,m2
    PARTPION=DSQRT(1d0-(m1+m2)**2/m**2)*DSQRT(1d0-(m1-m2)**2/m**2)
    PARTPION=PARTPION*m/2d0
    RETURN
  END FUNCTION PARTPION

  ! Eq.(5.39) in arXiv:1907.04898
  FUNCTION PARTVAREPSILON(m,m1,m2)
    IMPLICIT NONE
    REAL(KIND(1d0))::PARTVAREPSILON
    REAL(KIND(1d0)),INTENT(IN)::m,m1,m2
    PARTVAREPSILON=m/2d0*(1d0+(m1**2-m2**2)/m**2)
    RETURN
  END FUNCTION PARTVAREPSILON

  ! Eqs.(5.46-5.49) in arXiv:1907.04898
  SUBROUTINE DecayedMomenta_ISReshuffling(pRes0,pRes1,pD0,pD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pRes0 ! the original resonance momentum k_beta
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pRes1 ! the reshuffled resonace momentum kbar_beta
    REAL(KIND(1d0)),DIMENSION(2,4),INTENT(IN)::pD0 ! the original momenta of decayed particles, k_delta, k_gamma
    REAL(KIND(1d0)),DIMENSION(2,4),INTENT(OUT)::pD1 ! the reshuffled resonace menenta of decayed particles, kbar_delta, kbar_gamma
    REAL(KIND(1d0))::m1,m2,mres0,mres1,Delta,ppD0,ppD1
    REAL(KIND(1d0)),DIMENSION(4)::PBOO,pDD1 ! ,pDD2
    m1=PARTMINV(pD0(1,1:4))
    m2=PARTMINV(pD0(2,1:4))
    mres0=PARTMINV(pRes0)
    mres1=PARTMINV(pRes1)
    PBOO(4)=pRes0(4)
    PBOO(1:3)=-pRes0(1:3)
    ! first get Eqs.(5.46-5.47)
    pDD1(1:4)=pD0(1,1:4)
    CALL BOOSTL5(mres0,PBOO,pDD1)
    !pDD2(1:4)=pD0(2,1:4)
    !CALL BOOSTL5(mres0,PBOO,pDD2)
    ppD0=DSQRT(pDD1(1)**2+pDD1(2)**2+pDD1(3)**2)
    Delta=DABS(ppD0-PARTPION(mres0,m1,m2))/ppD0
    IF(Delta.GT.1D-3)THEN
       WRITE(*,*)"ERROR: momentum is not conserved !"
       WRITE(*,*)pRes0(1:4)
       WRITE(*,*)pD0(1,1:4)
       WRITE(*,*)pD0(2,1:4)
       STOP
    ENDIF
    Delta=DABS(pDD1(4)-PARTVAREPSILON(mres0,m1,m2))/DABS(pDD1(4))
    IF(Delta.GT.1D-3)THEN
       WRITE(*,*)"ERROR: energy is not conserved !"
       WRITE(*,*)pRes0(1:4)
       WRITE(*,*)pD0(1,1:4)
       WRITE(*,*)pD0(2,1:4)
       STOP
    ENDIF
    ppD1=PARTPION(mres1,m1,m2)
    pD1(1,1:3)=pDD1(1:3)*ppD1/ppD0
    pD1(1,4)=PARTVAREPSILON(mres1,m1,m2)
    pD1(2,4)=pD1(1,4)
    pD1(2,1:3)=-pD1(1,1:3)
    ! boost back to kbar_delta in lab frame
    PBOO(4)=pRes1(4)
    PBOO(1:3)=pRes1(1:3)
    CALL BOOSTL5(mres1,PBOO,pD1(1,1:4))
    CALL BOOSTL5(mres1,PBOO,pD1(2,1:4))
    RETURN
  END SUBROUTINE DecayedMomenta_ISReshuffling

  ! When PBOO(4)=P(4) and PBOO(1:3)=-P(1:3), it boosts P to its rest frame (0,0,0,Q)
  SUBROUTINE BOOSTL5(Q,PBOO,P)
    ! momentums are in normal representation with fourth comp is the zero comp
    ! Boost P via PBOO(PBOO^2=Q^2) to PLB,and set to P
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PBOO
    REAL(KIND(1d0)),DIMENSION(4),INTENT(INOUT)::P
    REAL(KIND(1d0)),DIMENSION(4)::PCM,PLB
    REAL(KIND(1d0))::FACT
    INTEGER::J
    PCM(1:4)=P(1:4)
    PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)+PBOO(1)*PCM(1))/Q
    FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
    DO J=1,3
       PLB(J)=PCM(J)+FACT*PBOO(J)
    ENDDO
    P(1:4)=PLB(1:4)
  END SUBROUTINE BOOSTL5

END MODULE analysis_plot
