MODULE analysis_plot
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=16 ! num_plots=inum_plots*4
  INTEGER::inum_plots=4
  INTEGER,PARAMETER::n_width=1,n_height=1
  CHARACTER(len=12),DIMENSION(4)::char_cuts=(/'            ',&
       '(ATLAS incl)','(ATLAS fidu)','(CMS   fidu)'/)
  LOGICAL,PARAMETER::topdrawer_output=.TRUE.
  LOGICAL,PARAMETER::gnuplot_output=.TRUE.
  LOGICAL,PARAMETER::root_output=.TRUE.
  LOGICAL,PARAMETER::hwufile_output=.TRUE.
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    INCLUDE '../../hbook/dbookf90.inc'
    INCLUDE '../../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    DO i=1,4
       l=(i-1)*inum_plots
       ! bookup is defined in analysis/hbook/dbook.f
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       !
       ! enlarge num_plots at the begining
       !
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       CALL histogram_book(l+ 1,'total rate'//ADJUSTL(TRIM(char_cuts(i))),&
            2,0d0,4d0,.FALSE.)
       CALL histogram_book(l+ 2,'psi pt'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 3,'psi y'//ADJUSTL(TRIM(char_cuts(i))),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 4,'psi m'//ADJUSTL(TRIM(char_cuts(i))),&
            100,2.8d0,4d0,.TRUE.)
    ENDDO

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile="plot_lhe.ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile="plot_lhe.root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE HwU_write_file(xnorm)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file='plot_lhe.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax
    OPEN(unit=99,FILE='plot_lhe.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='plot_lhe.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='plot_lhe.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE plot_fill(wgts)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::var,Q,www
    REAL(KIND(1d0))::exp1,exp2,mpsi,ypsi,ptpsi
    REAL(KIND(1d0))::e,ptpsilp,ptpsilm,etapsilp,etapsilm
    !REAL(KIND(1d0))::Pt_psipsi_sm05,Pt_psipsi_sm10,Pt_psipsi_sm20,Pt_psipsi_sm30
    !REAL(KIND(1d0))::dy_psipsi_sm05,dy_psipsi_sm10,dy_psipsi_sm20,dy_psipsi_sm30
    !REAL(KIND(1d0))::dphi_psipsi_sm05,dphi_psipsi_sm10,dphi_psipsi_sm20,dphi_psipsi_sm30
    !REAL(KIND(1d0))::M_psipsi_sm05,M_psipsi_sm10,M_psipsi_sm20,M_psipsi_sm30
    INTEGER::i,m,l,kk
    !REAL(KIND(1d0)),DIMENSION(4,NMXHEP)::PHEPSM05,PHEPSM10,PHEPSM20,PHEPSM30
    REAL(KIND(1d0)),DIMENSION(4)::xponium,xplponium,xplmonium
    LOGICAL::atlaspsiselection,atlaspsiselection2,cmspsiselection
    LOGICAL::atlaszselection
    LOGICAL,DIMENSION(4)::atlascuts
    INTEGER::numonium,numoniuml
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
    INCLUDE 'event_common_block_f90.inc'
    IF(wgts(1).EQ.0d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    numonium=0
    numoniuml=0
    DO i=1,NUP
       IF(ISTUP(i).EQ.-1)CYCLE
       IF(MOTHUP(1,i).LE.0.OR.MOTHUP(1,i).GT.NUP)CYCLE
       IF(lepton_pdg(IDUP(i)))THEN
          IF(ISTUP(MOTHUP(1,i)).NE.2)CYCLE
          numoniuml=numoniuml+1
          IF(IDUP(i).GT.0)THEN
             xplponium(1:4)=PUP(1:4,i)
          ELSE
             xplmonium(1:4)=PUP(1:4,i)
          ENDIF
       ENDIF
    ENDDO
    IF(.NOT.((numonium.EQ.0.AND.numoniuml.EQ.2)))THEN
       WRITE(*,*)"ERROR: number of onium is not equal to 1 !"
       STOP
    ENDIF
    xponium(1:4)=xplponium(1:4)+xplmonium(1:4)
    mpsi=DSQRT(xponium(4)**2-xponium(1)**2-xponium(2)**2-xponium(3)**2)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1d0
    ptpsi = DSQRT(xponium(1)**2+xponium(2)**2)
    ypsi = rapidity(xponium)
    ! ATLAS cuts
    ! EPJC 75 (2015) 229
    atlaspsiselection=ptpsi.GT.8.5d0.AND.ptpsi.LT.100d0
    atlaspsiselection=atlaspsiselection.AND.ABS(ypsi).LT.2.1d0
    cmspsiselection=ptpsi.GT.8.5d0.AND.ABS(ypsi).LT.2.1d0
    atlaspsiselection2=atlaspsiselection
    ptpsilp=DSQRT(xplponium(1)**2+xplponium(2)**2)
    ptpsilm=DSQRT(xplmonium(1)**2+xplmonium(2)**2)
    etapsilp=prapidity(xplponium(1:4))
    etapsilm=prapidity(xplmonium(1:4))
    atlaspsiselection=atlaspsiselection.AND.MAX(ptpsilp,ptpsilm).GT.4d0
    IF(ptpsilp.GE.ptpsilm)THEN
       atlaspsiselection=atlaspsiselection.AND.ABS(etapsilp).LT.2.5d0
       atlaspsiselection=atlaspsiselection.AND.((ptpsilm.GT.2.5d0.AND.&
            ABS(etapsilm).GT.1.3d0.AND.ABS(etapsilm).LT.2.5d0).OR.&
            (ptpsilm.GT.3.5d0.AND.ABS(etapsilm).LT.1.3d0))
    ELSE
       atlaspsiselection=atlaspsiselection.AND.ABS(etapsilm).LT.2.5d0
       atlaspsiselection=atlaspsiselection.AND.((ptpsilp.GT.2.5d0.AND.&
            ABS(etapsilp).GT.1.3d0.AND.ABS(etapsilp).LT.2.5d0).OR.&
            (ptpsilp.GT.3.5d0.AND.ABS(etapsilp).LT.1.3d0))
    ENDIF
    cmspsiselection=cmspsiselection.AND.ABS(etapsilp).LT.2.5d0.AND.&
         ABS(etapsilm).LT.2.5d0
    atlascuts(1)=.TRUE.
    atlascuts(3)=atlaspsiselection
    atlascuts(2)=atlaspsiselection2
    atlascuts(4)=cmspsiselection
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    !
    ! START TO FILL HISTOGRAM 
    !  
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    DO i=1,4
       IF(.NOT.atlascuts(i))CYCLE
       l=(i-1)*inum_plots
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,ptpsi,wgts)
       CALL HISTOGRAM_fill(l+3,ypsi,wgts)
       CALL HISTOGRAM_fill(l+4,mpsi,wgts)
    ENDDO
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill


  FUNCTION rapidity(p)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::p
    REAL(KIND(1d0))::rapidity,c
    IF(p(4).EQ.0d0)THEN
       rapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/ABS(p(4))
    IF(ABS(c).GE.1d0)THEN
       rapidity =0d0
    ELSE
       rapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
    RETURN
  END FUNCTION rapidity

  FUNCTION prapidity(p)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::p
    REAL(KIND(1d0))::prapidity,c
    IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
       prapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
    IF(ABS(c).EQ.1d0)THEN
       prapidity = 0d0
    ELSE
       prapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
    RETURN
  END FUNCTION prapidity

  FUNCTION lepton_pdg(ipdg)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ipdg
    LOGICAL::lepton_pdg
    lepton_pdg=.FALSE.
    IF(ABS(ipdg).EQ.11.OR.ABS(ipdg).EQ.13.OR.ABS(ipdg).EQ.15)lepton_pdg=.TRUE.
    RETURN
  END FUNCTION lepton_pdg

END MODULE analysis_plot
