MODULE analysis_plot
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=7
  INTEGER,PARAMETER::n_width=1,n_height=1
  LOGICAL,PARAMETER::topdrawer_output=.TRUE.
  LOGICAL,PARAMETER::gnuplot_output=.TRUE.
  LOGICAL,PARAMETER::root_output=.TRUE.
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    INCLUDE '../../hbook/dbookf90.inc'
    CALL INIHIST
    CALL INIHIST_3D
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    DO kk=1,nwgt_analysis
       l=(kk-1)*num_plots
       ! bookup is defined in analysis/hbook/dbook.f
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       !
       ! enlarge num_plots at the begining
       !
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       CALL bookup(l+ 1,'total rate    '//weights_info(kk),&
            1.0d0,0.5d0,5.5d0)
       CALL bookup(l+ 2,'pt(boson)   '//weights_info(kk),&
            5d0,0d0,500d0)
       CALL bookup(l+ 3,'accum pt(boson) '//weights_info(kk),&
            5d0,0d0,500d0)
       CALL bookup(l+ 4,'pt(top)   '//weights_info(kk),&
            5d0,0d0,500d0)
       CALL bookup(l+ 5,'accum pt(top)   '//weights_info(kk),&
            5d0,0d0,500d0)
       CALL bookup_3d(l+ 6,'pt(boson)-pt(top)'//weights_info(kk),&
            5d0,0d0,500d0,5d0,0d0,500d0)
       CALL bookup_3d(l+ 7,'accum pt(boson)-pt(top)'//weights_info(kk),&
            5d0,0d0,500d0,5d0,0d0,500d0)
    ENDDO
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    CALL mclear
    CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'+',i,i,xnorm,0.d0)
       CALL mfinal(i)
       CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
       CALL mfinal_3d(i)
    ENDDO
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       DO kk=1,nwgt_analysis
          l=(kk-1)*num_plots
          CALL multitop(l+ 1,n_width,n_height,'total rate   ',ytit,'LIN')
          CALL multitop(l+ 2,n_width,n_height,'pt(boson) ',ytit,'LOG')
          CALL multitop(l+ 3,n_width,n_height,'accum pt(boson) ',ytit,'LOG')
          CALL multitop(l+ 4,n_width,n_height,'pt(top) ',ytit,'LOG')
          CALL multitop(l+ 5,n_width,n_height,'accum pt(top) ',ytit,'LOG')
       END DO
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile="plot_lhe.ps"
       DO kk=1,nwgt_analysis
          l=(kk-1)*num_plots
          CALL MGNUPLOT(l+ 1,'total rate   ',ytit,'LIN',psfile)
          CALL MGNUPLOT(l+ 2,'pt(boson) ',ytit,'LOG',psfile)
          CALL MGNUPLOT(l+ 3,'accum pt(boson) ',ytit,'LOG',psfile)
          CALL MGNUPLOT(l+ 4,'pt(top) ',ytit,'LOG',psfile)
          CALL MGNUPLOT(l+ 5,'accum pt(top) ',ytit,'LOG',psfile)
          CALL MGNUPLOT_3D(l+ 6,'pt(boson)','pt(top)',ytit,'LOG',psfile)
          CALL MGNUPLOT_3D(l+ 7,'accum pt(boson)','accum pt(top)',ytit,'LOG',psfile)
       ENDDO
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile="plot_lhe.root"
       DO kk=1,nwgt_analysis
          l=(kk-1)*num_plots
          CALL MROOTPLOT(l+ 1,'total rate  ',ytit,'LIN',rootfile)
          CALL MROOTPLOT(l+ 2,'pt(boson) ',ytit,'LOG',rootfile)
          CALL MROOTPLOT(l+ 3,'accum pt(boson) ',ytit,'LOG',rootfile)
          CALL MROOTPLOT(l+ 4,'pt(top) ',ytit,'LOG',rootfile)
          CALL MROOTPLOT(l+ 5,'accum pt(top) ',ytit,'LOG',rootfile)
          CALL MROOTPLOT_3D(l+ 6,'pt(boson)','pt(top)',ytit,'LOG',rootfile)
          CALL MROOTPLOT_3D(l+ 7,'accum pt(boson)','accum pt(top)',ytit,'LOG',rootfile)
       ENDDO
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::pt_boson,pt_top
    REAL(KIND(1d0)),DIMENSION(1:4)::pboson,ptop
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l,nboson,ntop
    REAL(KIND(1d0))::www,yftot
    ! User process event common block.   
    INCLUDE 'event_common_block_f90.inc'
    ! MAXNUP: max number of particles in each event
    ! NUP: number of particles in the event
    ! IDPRUP: the id of the process
    ! IDUP(MAXNUP): the pdg of each particle in the event
    ! ISTUP(MAXUP): the final state (1) or the initial state (-1)
    ! MOTHUP(2,MAXNUP): the mother of each particle in the event
    ! ICOLUP(2,MAXNUP): the color flow of each particle in the event
    ! XWGTUP: the weight of the event
    ! SCALUP: the scale of the event
    ! AQEDUP: alpha_EW value in the event
    ! AQCDUP: alpha_s value in the event
    ! PUP(5,MAXNUP): (px,py,pz,E,m) of the particle in the event
    ! VTIMUP(MAXNUP): life of the particle in the event
    ! SPINUP(MAXNUP): spin of the particle in the event
    ntop=0
    nboson=0
    DO i=1,NUP
       IF(IDUP(i).EQ.6)THEN
          ptop(1:4)=PUP(1:4,i)
          ntop=ntop+1
       ELSEIF(IDUP(i).EQ.22.OR.IDUP(i).EQ.23.OR.IDUP(i).EQ.25&
            .OR.ABS(IDUP(i)).EQ.24)THEN
          pboson(1:4)=PUP(1:4,i)
          nboson=nboson+1
       ENDIF
    ENDDO
    IF(ntop.NE.1)THEN
       WRITE(*,*)"ERROR:the number of top != 1, #top =",ntop
       STOP
    ENDIF
    IF(nboson.NE.1)THEN
       WRITE(*,*)"ERROR:the number of boson != 1, #boson =",nboson
       STOP
    ENDIF
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    pt_top=DSQRT(ptop(1)**2+ptop(2)**2)
    pt_boson=DSQRT(pboson(1)**2+pboson(2)**2)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO kk=1,nwgt_analysis
       www=wgts(kk)
       l=(kk-1)*num_plots
       CALL mfill(l+1,var,www)
       CALL mfill(l+2,pt_boson,www)
       CALL accum_mfill(l+3,pt_boson,5d0,0d0,500d0,www)
       CALL mfill(l+4,pt_top,www)
       CALL accum_mfill(l+5,pt_top,5d0,0d0,500d0,www)
       CALL mfill_3d(l+6,pt_boson,pt_top,www)
       CALL accum_mfill_3d(l+7,pt_boson,pt_top,5d0,0d0,500d0,&
            5d0,0d0,500d0,www)
    ENDDO
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE accum_mfill(lnum,y,DEL,XMIN,XMAX,www)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::lnum
    REAL(KIND(1d0)),INTENT(IN)::y,www,DEL,XMAX,XMIN
    INTEGER::i,NBIN,ibin
    REAL(KIND(1d0))::HMIN,HMAX,yy
    NBIN = INT((XMAX-XMIN)/(DEL*0.999999d0))
    HMIN = XMIN
    HMAX = NBIN*DEL+XMIN
    CALL find_bin(y,NBIN,HMIN,HMAX,DEL,ibin)
    IF(ibin.LE.0)RETURN
    DO i=1,ibin
       yy=y+DEL*(i-ibin)
       CALL mfill(lnum,yy,www)
    ENDDO
    RETURN
  END SUBROUTINE accum_mfill

  SUBROUTINE accum_mfill_3d(lnum,y1,y2,DEL1,XMIN1,XMAX1,DEL2,XMIN2,XMAX2,www)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::lnum
    REAL(KIND(1d0)),INTENT(IN)::y1,y2,www,DEL1,XMAX1,XMIN1,DEL2,XMAX2,XMIN2
    INTEGER::i1,i2,NBIN1,NBIN2,ibin1,ibin2
    REAL(KIND(1d0))::HMIN1,HMAX1,HMIN2,HMAX2,yy1,yy2
    NBIN1 = INT((XMAX1-XMIN1)/(DEL1*0.999999d0))
    HMIN1 = XMIN1
    HMAX1 = NBIN1*DEL1+XMIN1
    NBIN2 = INT((XMAX2-XMIN2)/(DEL2*0.999999d0))
    HMIN2 = XMIN2
    HMAX2 = NBIN2*DEL2+XMIN2
    CALL find_bin(y1,NBIN1,HMIN1,HMAX1,DEL1,ibin1)
    CALL find_bin(y2,NBIN2,HMIN2,HMAX2,DEL2,ibin2)
    IF(ibin1.LE.0.OR.ibin2.LE.0)RETURN
    DO i1=1,ibin1
       yy1=y1+DEL1*(i1-ibin1)
       DO i2=1,ibin2
          yy2=y2+DEL2*(i2-ibin2)
          CALL mfill_3d(lnum,yy1,yy2,www)
       ENDDO
    ENDDO
    RETURN
  END SUBROUTINE accum_mfill_3d

  SUBROUTINE find_bin(y,nbin,hmin,hmax,del,ibin)
    IMPLICIT NONE
    INTEGER,INTENT(OUT)::ibin
    INTEGER,INTENT(IN)::nbin
    REAL(KIND(1d0)),INTENT(IN)::y,hmin,hmax,del
    INTEGER::i
    REAL(KIND(1d0))::binmin,binmax
    ibin=0
    IF(y.LT.hmin)THEN
       ibin=-1
       RETURN
    ENDIF
    IF(y.GT.hmax)THEN
       ibin=-2
       RETURN
    ENDIF
    binmin=hmin
    binmax=binmin+del
    DO i=1,nbin
       IF(y.GE.binmin.AND.y.LT.binmax)THEN
          ibin=i
          RETURN
       ENDIF
       IF(i.EQ.nbin.AND.y.EQ.binmax)THEN
          ibin=i
          RETURN
       ENDIF
       binmin=binmax
       binmax=binmin+del
    ENDDO
    RETURN
  END SUBROUTINE find_bin

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax                                                  
    OPEN(unit=99,FILE='plot_lhe.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='plot_lhe.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='plot_lhe.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

END MODULE analysis_plot
