MODULE plot_py8_user
!============================================================
!
! PLOT FILE WITH PYTHIA8 FOR USER
! The cuts are according to H1 paper - EPJC 68 (2010) 401
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=13
  INTEGER,PARAMETER::n_width=1,n_height=1
  INTEGER::IEVENT=0,USED_EVENT=0
  REAL(KIND(1d0)),DIMENSION(2),PARAMETER::ebeam=(/27.5d0,920d0/)
  SAVE IEVENT,USED_EVENT
CONTAINS
  SUBROUTINE plot_py8_begin(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    INTEGER,PARAMETER::max_weight=1
    CHARACTER(len=15),DIMENSION(max_weight),INTENT(IN)::weights_info
    CHARACTER(len=15)::weights_info2
    INTEGER::i,kk,l
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    INCLUDE 'histogram_common90.inc'
    top_yes=.TRUE.
    gnu_yes=.TRUE.
    root_yes=.TRUE.
    hwu_yes=.TRUE.
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    l=0
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    CALL histogram_book(l+ 1,'J/{/Symbol y} p_T^2 b1',&
         2,1d0,2.1d0,.TRUE.)
    CALL histogram_book(l+ 2,'J/{/Symbol y} p_T^2 b2',&
         2,2.1d0,3.5d0,.TRUE.)
    CALL histogram_book(l+ 3,'J/{/Symbol y} p_T^2 b3',&
         2,3.5d0,5.4d0,.TRUE.)
    CALL histogram_book(l+ 4,'J/{/Symbol y} p_T^2 b4',&
         2,5.4d0,7.6d0,.TRUE.)
    CALL histogram_book(l+ 5,'J/{/Symbol y} p_T^2 b5',&
         2,7.6d0,10.0d0,.TRUE.)
    CALL histogram_book(l+ 6,'J/{/Symbol y} p_T^2 b6',&
         2,10.0d0,13.5d0,.TRUE.)
    CALL histogram_book(l+ 7,'J/{/Symbol y} p_T^2 b7',&
         2,13.5d0,20.0d0,.TRUE.)
    CALL histogram_book(l+ 8,'J/{/Symbol y} p_T^2 b8',&
         2,20.0d0,26.5d0,.TRUE.)
    CALL histogram_book(l+ 9,'J/{/Symbol y} p_T^2 b9',&
         2,26.5d0,40.0d0,.TRUE.)
    CALL histogram_book(l+10,'J/{/Symbol y} p_T^2 b10',&
         2,40.0d0,60.0d0,.TRUE.)
    CALL histogram_book(l+11,'J/{/Symbol y} p_T^2 b11',&
         2,60.0d0,100d0,.TRUE.)
    CALL histogram_book(l+12,'J/{/Symbol y} p_T^2 b12',&
         2,100d0,150d0,.TRUE.)
    CALL histogram_book(l+13,'J/{/Symbol y} p_T^2 b13',&
         2,150d0,250d0,.TRUE.)
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_py8_begin

  SUBROUTINE plot_py8_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=18)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    CHARACTER(len=400)::psfile,rootfile
    CALL mclear
    CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'F',i,i,xnorm,0.d0)
       CALL mfinal(i)
       CALL mopera_3d(i,'F',i,i,xnorm,0.d0)
       CALL mfinal_3d(i)
    ENDDO

    WRITE(*,*)"INFO:total number of events:",IEVENT
    WRITE(*,*)"INFO:total number of used events:",USED_EVENT

    ytit='sigma per bin [nb]'
    CALL open_topdrawer_file
    CALL HISTOGRAM_END(1,ytit,' ',' ')
    CALL close_topdrawer_file

    CALL open_gnuplot_file
    psfile="helaconia_py8.ps"
    CALL HISTOGRAM_END(2,ytit,psfile,' ')
    CALL close_gnuplot_file

    CALL open_root_file
    rootfile="helaconia_py8.root"
    CALL HISTOGRAM_END(3,ytit,' ',rootfile)
    ! wirte the end of the .C file for root
    WRITE (96, 100)
100 FORMAT(/1X,&
         ' hohisto -> cd();',/1X,&
         ' if (histos -> GetEntries() > 0 ) then {',/1X,&
         '  histos->Write();',/1X,&
         '  hohisto -> Close();',/1X,&
         ' }',/1X,'}')
    CALL close_root_file

    CALL HISTOGRAM_END(4,ytit,' ',' ')
    CALL HwU_write_file(xnorm)
    RETURN                
  END SUBROUTINE plot_py8_end

  SUBROUTINE plot_py8_fill(wgts)
    IMPLICIT NONE
    !INCLUDE '../include/HEPMC90.INC'
    INCLUDE 'HEPMC90.INC' ! it will be copied to shower directory
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(250),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    INTEGER::i,kk,jj,l,IHEP,IMOTH,IST,ID,ID1
    REAL(KIND(1d0))::www
    INTEGER::N_PSI,this_psi,imax,i_psi_b,i_psi_bbar
    REAL(KIND(1d0))::ptmax,ptmax_psi,pt_psi,pt2_psi,y_psi
    LOGICAL::FOUNDPSI,FOUNDPHOTON
    LOGICAL,DIMENSION(10)::PSIFromB
    REAL(KIND(1d0)),DIMENSION(2,4)::p_muon_psi_b,p_muon_psi_bbar
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonp_psi
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonm_psi
    REAL(KIND(1d0))::sgammap
    REAL(KIND(1d0))::W,z,znum,zden  !W = SQRT((p_gamma + p_p)^2) = SQRT(s_gamma-p), z=(p_psi,p_p)/(p_gamma,p_p)
    REAL(KIND(1d0)),DIMENSION(4)::p_psi,p_a,p_p
    INTEGER,DIMENSION(10)::INDEX_psi
    LOGICAL::H1cuts
    ! INCOMING PARTONS MAY TRAVEL IN THE SAME DIRECTION: IT''S 
    ! A POWER-SUPPRESSED EFFECT, SO THROW THE EVENT AWAY
    !IF(SIGN(1.D0,PHEP(3,1)).EQ.SIGN(1.D0,PHEP(3,2)))THEN
    !   WRITE(*,*)"WARNING 1 in plot_py8_fill"
    !   GOTO 999
    !ENDIF
    IEVENT=IEVENT+1
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    N_PSI=0
    INDEX_psi(1:10)=0
    PSIFromB(1:10)=.FALSE.
    p_a(1:4)=0d0
    this_psi=0
    FOUNDPHOTON=.FALSE.
    DO IHEP=1,NHEP
       IST=ISTHEP(IHEP) ! status
       ID=IDHEP(IHEP) ! pdg
       ID1=IHADR(ID) ! the massive quark's PDG in hadron, defined later
       IF(IST.EQ.1.AND.(ABS(ID).EQ.13.OR.ABS(ID).EQ.11))THEN
          IF(N_PSI.LE.10)THEN
             ! try to find whether it is coming from J/psi decay
             IMOTH=JMOHEP(1,IHEP)
             FOUNDPSI=.FALSE.
             DO jj=1,NHEP
                IF(IDHEP(IMOTH).EQ.443)THEN
                   ! it is from J/psi decay 
                   FOUNDPSI=.TRUE.
                   this_psi=0
                   DO i=1,N_PSI
                      IF(IMOTH.EQ.INDEX_psi(i))THEN
                         this_psi=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_psi.EQ.0)THEN
                      N_PSI=N_PSI+1
                      IF(N_PSI.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY PSI !"
                         STOP
                      ENDIF
                      this_psi=N_PSI
                      INDEX_psi(N_PSI)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(ABS(IDHEP(IMOTH)).NE.13.AND.&
                     ABS(IDHEP(IMOTH)).NE.11.AND.IMOTH.NE.3)THEN
                   FOUNDPSI=.FALSE.
                   EXIT
                ENDIF
                IMOTH=JMOHEP(1,IMOTH)
             ENDDO
             IF(FOUNDPSI)THEN
                IF(ID.GT.0)THEN
                   all_p_muonm_psi(this_psi,1:4)=PHEP(1:4,IHEP)
                ELSE
                   all_p_muonp_psi(this_psi,1:4)=PHEP(1:4,IHEP)
                ENDIF
                jj=INDEX_psi(this_psi)
                IMOTH=JMOHEP(1,jj)
                DO WHILE(BHADRON(IDHEP(IMOTH)).EQ.0.AND.IMOTH.GT.0)
                   jj=IMOTH
                   IMOTH=JMOHEP(1,jj)
                ENDDO
                IF(IMOTH.LE.0)THEN
                   PSIFromB(this_psi)=.FALSE.
                ELSE
                   PSIFromB(this_psi)=BHADRON(IDHEP(IMOTH)).EQ.1
                ENDIF
             ENDIF
          ENDIF
       ELSEIF(IST.EQ.1.AND.ID.EQ.443)THEN
          ! not decayed psi 
          WRITE(*,*)"ERROR:Found an undecayed J/psi"
          STOP
       ELSEIF(ID.EQ.22.AND..NOT.FOUNDPHOTON)THEN
          FOUNDPHOTON=.TRUE.
          p_a(1:4)=PHEP(1:4,IHEP)
       ENDIF
    ENDDO
    IF(N_PSI.EQ.0.OR.N_PSI.EQ.1)THEN
       WRITE(*,*)"=============================================="
       PRINT *, IEVENT,N_PSI
       DO IHEP=1,NHEP
          WRITE(*,*)IHEP,ISTHEP(IHEP),IDHEP(IHEP),JMOHEP(1,IHEP),JMOHEP(2,IHEP)
       ENDDO
       WRITE(*,*)"=============================================="
       WRITE(*,*)"WARNING:Cannot find two PSI"
       RETURN
    ENDIF
    sgammap=4d0*p_a(4)*ebeam(2)
    IF(sgammap.LT.0d0)RETURN
    W=DSQRT(sgammap)
    ! H1 cuts
    H1cuts=W.GT.60d0.AND.W.LT.240d0
    IF(.NOT.H1cuts)RETURN
    p_p(1)=0d0
    p_p(2)=0d0
    p_p(3)=ebeam(2)*(-1d0)
    p_p(4)=ebeam(2)
    IF(N_PSI.GE.2)THEN
       ! psi from B
       i_psi_b=0
       ptmax_psi=0d0
       DO i=1,N_PSI
          IF(.NOT.PSIFromB(i))CYCLE
          p_psi(1:4)=all_p_muonp_psi(i,1:4)+all_p_muonm_psi(i,1:4)
          pt_psi=DSQRT(p_psi(1)**2+p_psi(2)**2)
          y_psi=rapidity(p_psi(1:4))
          znum=p_psi(4)*p_p(4)-p_psi(1)*p_p(1)-p_psi(2)*p_p(2)-p_psi(3)*p_p(3)
          zden=p_a(4)*p_p(4)-p_a(1)*p_p(1)-p_a(2)*p_p(2)-p_a(3)*p_p(3)
          IF(zden.EQ.0d0)CYCLE
          z=znum/zden
          IF(z.LT.0.3d0.OR.z.GT.0.9d0)CYCLE
          IF(ptmax_psi.LT.pt_psi)THEN
             ptmax_psi=pt_psi
             i_psi_b=i
          ENDIF
       ENDDO
       IF(i_psi_b.GT.0)THEN
          p_muon_psi_b(1,1:4)=all_p_muonp_psi(i_psi_b,1:4)
          p_muon_psi_b(2,1:4)=all_p_muonm_psi(i_psi_b,1:4)
       ENDIF
       ! psi from Bbar
       i_psi_bbar=0
       ptmax_psi=0d0
       DO i=1,N_PSI
          IF(PSIFromB(i))CYCLE
          p_psi(1:4)=all_p_muonp_psi(i,1:4)+all_p_muonm_psi(i,1:4)
          pt_psi=DSQRT(p_psi(1)**2+p_psi(2)**2)
          y_psi=rapidity(p_psi(1:4))
          znum=p_psi(4)*p_p(4)-p_psi(1)*p_p(1)-p_psi(2)*p_p(2)-p_psi(3)*p_p(3)
          zden=p_a(4)*p_p(4)-p_a(1)*p_p(1)-p_a(2)*p_p(2)-p_a(3)*p_p(3)
          IF(zden.EQ.0d0)CYCLE
          z=znum/zden
          IF(z.LT.0.3d0.OR.z.GT.0.9d0)CYCLE
          IF(ptmax_psi.LT.pt_psi)THEN
             ptmax_psi=pt_psi
             i_psi_bbar=i
          ENDIF
       ENDDO
       IF(i_psi_bbar.GT.0)THEN
          p_muon_psi_bbar(1,1:4)=all_p_muonp_psi(i_psi_bbar,1:4)
          p_muon_psi_bbar(2,1:4)=all_p_muonm_psi(i_psi_bbar,1:4)
       ENDIF
    ENDIF
    IF(i_psi_b.GT.0.OR.i_psi_bbar.GT.0)USED_EVENT=USED_EVENT+1
    l=0
    DO i=1,2
       IF(i.EQ.1)THEN
          IF(i_psi_b.EQ.0)CYCLE
          p_psi(1:4)=p_muon_psi_b(1,1:4)+p_muon_psi_b(2,1:4)
       ELSE
          IF(i_psi_bbar.EQ.0)CYCLE
          p_psi(1:4)=p_muon_psi_bbar(1,1:4)+p_muon_psi_bbar(2,1:4)
       ENDIF
       znum=p_psi(4)*p_p(4)-p_psi(1)*p_p(1)-p_psi(2)*p_p(2)-p_psi(3)*p_p(3)
       zden=p_a(4)*p_p(4)-p_a(1)*p_p(1)-p_a(2)*p_p(2)-p_a(3)*p_p(3)
       IF(zden.EQ.0d0)CYCLE
       z=znum/zden
       H1cuts=H1cuts.AND.z.GT.0.3d0.AND.z.LT.0.9d0
       IF(.NOT.H1cuts)CYCLE
       pt2_psi=p_psi(1)**2+p_psi(2)**2
       pt_psi=DSQRT(pt2_psi)
       y_psi=rapidity(p_psi(1:4))
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       !
       ! START TO FILL HISTOGRAM
       !
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       IF(H1cuts.AND.pt2_psi.GT.1.0d0.AND.pt2_psi.LT.2.1d0)CALL HISTOGRAM_fill(l+1,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.2.1d0.AND.pt2_psi.LT.3.5d0)CALL HISTOGRAM_fill(l+2,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.3.5d0.AND.pt2_psi.LT.5.4d0)CALL HISTOGRAM_fill(l+3,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.5.4d0.AND.pt2_psi.LT.7.6d0)CALL HISTOGRAM_fill(l+4,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.7.6d0.AND.pt2_psi.LT.10.0d0)CALL HISTOGRAM_fill(l+5,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.10.0d0.AND.pt2_psi.LT.13.5d0)CALL HISTOGRAM_fill(l+6,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.13.5d0.AND.pt2_psi.LT.20.0d0)CALL HISTOGRAM_fill(l+7,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.20.0d0.AND.pt2_psi.LT.26.5d0)CALL HISTOGRAM_fill(l+8,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.26.5d0.AND.pt2_psi.LT.40.0d0)CALL HISTOGRAM_fill(l+9,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.40.0d0.AND.pt2_psi.LT.60.0d0)CALL HISTOGRAM_fill(l+10,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.60.0d0.AND.pt2_psi.LT.100d0)CALL HISTOGRAM_fill(l+11,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.100d0.AND.pt2_psi.LT.150d0)CALL HISTOGRAM_fill(l+12,pt2_psi,wgts)
       IF(H1cuts.AND.pt2_psi.GT.150d0.AND.pt2_psi.LT.250d0)CALL HISTOGRAM_fill(l+13,pt2_psi,wgts)
    ENDDO

    call HwU_add_points
999 RETURN      
  END SUBROUTINE plot_py8_fill

  FUNCTION BHADRON(ID)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ID
    INTEGER::BHADRON
    IF(ID.EQ.511.OR.ID.EQ.521.OR.ID.EQ.531.OR.ID.EQ.541.OR.&
         ID.EQ.5122.OR.ID.EQ.5132.OR.ID.EQ.5232.OR.ID.EQ.5332)THEN
       BHADRON=1
    ELSEIF(ID.EQ.-511.OR.ID.EQ.-521.OR.ID.EQ.-531.OR.ID.EQ.-541.OR.&
          ID.EQ.-5122.OR.ID.EQ.-5132.OR.ID.EQ.-5232.OR.ID.EQ.-5332)THEN
       BHADRON=-1
    ELSE
       BHADRON=0
    ENDIF
  END FUNCTION BHADRON

  SUBROUTINE HwU_write_file(xnorm)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file='helaconia_py8.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE='helaconia_py8.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='helaconia_py8.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='helaconia_py8.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

! Kinematic functions
  FUNCTION rapidity(p)
    ! rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::rapidity,c
    IF(p(4).EQ.0d0)THEN
       rapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/ABS(p(4))
    IF(ABS(c).GE.1d0)THEN
       rapidity =0d0
    ELSE
       rapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION rapidity

  FUNCTION prapidity(p)
    ! psudo-rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::prapidity,c
    IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
       prapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
    IF(ABS(c).EQ.1d0)THEN
       prapidity = 0d0
    ELSE
       prapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION prapidity

  ! returns the PDG code of the heavier quark in the hadron of PDG code ID
  FUNCTION IHADR(ID)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ID
    INTEGER::IHADR,ID1
    IF(ID.NE.0)THEN
       ID1=ABS(ID)
       IF(ID1.GT.10000)ID1=ID1-1000*INT(ID1/1000)
       IHADR=ID1/(10**INT(LOG10(DFLOAT(ID1))))
    ELSE
       IHADR=0
    ENDIF
    RETURN
  END FUNCTION IHADR

  FUNCTION ph4(px,py,pz)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::px,py,pz
    REAL(KIND(1d0))::ph4,S2,S
    INTEGER::init=0
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    SAVE init
    IF(px**2+py**2.LE.0d0)THEN
       IF(pz.GE.0d0)ph4=0d0
       IF(pz.LT.0)ph4=pi
       RETURN
    ENDIF
    S2=px**2/(px**2+py**2)
    S=DSQRT(S2)
    IF(S.GT.1d0)THEN
       WRITE(*,*)'PH4(X) WARNING S=',S
       ph4=0
       IF(px.LT.0d0)ph4=pi
       RETURN
    ENDIF
    IF(px.LT.0)S=-DSQRT(S2)
    ph4=DACOS(S)
    IF(py.LT.0)ph4=2*pi-ph4
    RETURN
  END FUNCTION ph4

  FUNCTION scalar_product(p1,p2)
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::p1,p2
    REAL(KIND(1d0))::scalar_product
    scalar_product=p1(4)*p2(4)-p1(3)*p2(3)-p1(2)*p2(2)-p1(1)*p2(1)
  END FUNCTION scalar_product

  FUNCTION transverse(p)
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::transverse
    transverse=SQRT(p(1)**2+p(2)**2)
  END FUNCTION transverse

  ! vector product
  FUNCTION vec_crossprod(vec1,vec2)
  ! vec1 crosstime vec2
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(3),INTENT(IN)::vec1,vec2
    REAL(KIND(1d0)),DIMENSION(3)::vec_crossprod
    vec_crossprod(1)=vec1(2)*vec2(3)-vec1(3)*vec2(2)
    vec_crossprod(2)=vec1(3)*vec2(1)-vec1(1)*vec2(3)
    vec_crossprod(3)=vec1(1)*vec2(2)-vec1(2)*vec2(1)
    RETURN
  END FUNCTION vec_crossprod

  FUNCTION cosij(a,b)
    REAL(KIND(1d0)),DIMENSION(3),INTENT(IN)::a,b
    REAL(KIND(1d0))::cosij,den
    cosij=a(1)*b(1)+a(2)*b(2)+a(3)*b(3)
    den=DSQRT(a(1)**2+a(2)**2+a(3)**2)*&
         DSQRT(b(1)**2+b(2)**2+b(3)**2)
    IF(den.EQ.0d0)THEN
       cosij=1d0
       RETURN
    ENDIF
    cosij=cosij/DSQRT(a(1)**2+a(2)**2+a(3)**2)
    cosij=cosij/DSQRT(b(1)**2+b(2)**2+b(3)**2)
  END FUNCTION cosij

  SUBROUTINE BOOSTL(Q,PBOO,P)
    ! momentums are in normal representation with fourth comp is the zero comp
    ! Boost P via PBOO(PBOO^2=Q^2) to PLB,and set to P
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PBOO
    REAL(KIND(1d0)),DIMENSION(4),INTENT(INOUT)::P
    REAL(KIND(1d0)),DIMENSION(4)::PCM,PLB
    REAL(KIND(1d0))::FACT
    INTEGER::J
    PCM(1:4)=P(1:4)
    PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)+PBOO(1)*PCM(1))/Q
    FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
    DO J=1,3
       PLB(J)=PCM(J)+FACT*PBOO(J)
    ENDDO
    P(1:4)=PLB(1:4)
  END SUBROUTINE BOOSTL

  SUBROUTINE psi_ll_kin(ppsi,mpsi,plp,plm)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::ppsi
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::plp,plm
    REAL(KIND(1d0)),INTENT(IN)::mpsi
    REAL(KIND(1d0)),PARAMETER::pipi=3.1415926535897932384626433832795d0
    REAL(KIND(1d0))::theta,phi,p0
    REAL(KIND(1d0)),DIMENSION(4)::plpres,plmres
    theta=rand()*2d0*pipi
    phi=rand()*2d0*pipi
    p0=mpsi/2d0
    plpres(3)=p0*DCOS(theta)
    plpres(4)=p0
    plpres(1)=p0*DSIN(theta)*DCOS(phi)
    plpres(2)=p0*DSIN(theta)*DSIN(phi)
    plmres(4)=p0
    plmres(1:3)=-plpres(1:3)
    call boost(mpsi,ppsi,plpres,plp)
    call boost(mpsi,ppsi,plmres,plm)
    RETURN
  END SUBROUTINE psi_ll_kin

  SUBROUTINE BOOST(Q,PBOO,PCM,PLB)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PBOO,PCM
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PLB
    REAL(KIND(1d0))::Fact
    INTEGER::j
    PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)&
         +PBOO(1)*PCM(1))/Q
    FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
    DO J=1,3
       PLB(J)=PCM(J)+FACT*PBOO(J)
    ENDDO
    RETURN
  END SUBROUTINE BOOST

END MODULE plot_py8_user
