MODULE plot_py8_user
!============================================================
!
! PLOT FILE WITH PYTHIA8 FOR USER
! It is designed for p p > W Y(1S)
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=18 ! num_plots=inum_plots*2
  INTEGER::inum_plots=9
  INTEGER,PARAMETER::n_width=1,n_height=1
  CHARACTER(len=12),DIMENSION(2)::char_cuts=(/'            ','(CMS   fidu)'/)
  INTEGER::IEVENT=0,USED_EVENT=0
  SAVE IEVENT,USED_EVENT
CONTAINS
  SUBROUTINE plot_py8_begin(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    INTEGER,PARAMETER::max_weight=1
    CHARACTER(len=15),DIMENSION(max_weight),INTENT(IN)::weights_info
    CHARACTER(len=15)::weights_info2
    INTEGER::i,kk,l
    REAL(KIND(1d0)),PARAMETER::PI=3.1416D0
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    INCLUDE 'histogram_common90.inc'
    top_yes=.TRUE.
    gnu_yes=.TRUE.
    root_yes=.TRUE.
    hwu_yes=.TRUE.
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    DO i=1,2
       l=(i-1)*inum_plots
       ! bookup is defined in analysis/hbook/dbook.f
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       !
       ! enlarge num_plots at the begining
       !
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       CALL histogram_book(l+ 1,'total rate '//ADJUSTL(TRIM(char_cuts(i))),&
            2,0d0,4d0,.FALSE.)
       CALL histogram_book(l+ 2,'Y(1S) pt '//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 3,'Y(1S) y '//ADJUSTL(TRIM(char_cuts(i))),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 4,'Y(1S) m '//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,20d0,.TRUE.)
       CALL histogram_book(l+ 5,'W pt '//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 6,'W y '//ADJUSTL(TRIM(char_cuts(i))),&
               100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 7,'Y(1S)-W m '//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,1000d0,.TRUE.)
       CALL histogram_book(l+ 8,'Delta phi(W,Y) '//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
       CALL histogram_book(l+ 9,'Delta phi(W,Y), rebin '//ADJUSTL(TRIM(char_cuts(i))),&
            12,0d0,pi,.TRUE.)
    ENDDO

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_py8_begin

  SUBROUTINE plot_py8_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=18)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    CHARACTER(len=400)::psfile,rootfile
    CALL mclear
    CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'F',i,i,xnorm,0.d0)
       CALL mfinal(i)
       CALL mopera_3d(i,'F',i,i,xnorm,0.d0)
       CALL mfinal_3d(i)
    ENDDO

    WRITE(*,*)"INFO:total number of events:",IEVENT
    WRITE(*,*)"INFO:total number of used events:",USED_EVENT

    ytit='sigma per bin [nb]'
    CALL open_topdrawer_file
    CALL HISTOGRAM_END(1,ytit,' ',' ')
    CALL close_topdrawer_file

    CALL open_gnuplot_file
    psfile="helaconia_py8.ps"
    CALL HISTOGRAM_END(2,ytit,psfile,' ')
    CALL close_gnuplot_file

    CALL open_root_file
    rootfile="helaconia_py8.root"
    CALL HISTOGRAM_END(3,ytit,' ',rootfile)
    ! wirte the end of the .C file for root
    WRITE (96, 100)
100 FORMAT(/1X,&
         ' hohisto -> cd();',/1X,&
         ' if (histos -> GetEntries() > 0 ) then {',/1X,&
         '  histos->Write();',/1X,&
         '  hohisto -> Close();',/1X,&
         ' }',/1X,'}')
    CALL close_root_file

    CALL HISTOGRAM_END(4,ytit,' ',' ')
    CALL HwU_write_file(xnorm)
    RETURN                
  END SUBROUTINE plot_py8_end

  SUBROUTINE plot_py8_fill(wgts)
    IMPLICIT NONE
    !INCLUDE '../include/HEPMC90.INC'
    INCLUDE 'HEPMC90.INC' ! it will be copied to shower directory
    REAL(KIND(1d0)),PARAMETER::mass_zboson=91.1876d0
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(250),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e,ptYlp,ptYlm,etaYlp,etaYlm
    REAL(KIND(1d0))::ptzlp,ptzlm,etazlp,etazlm
    REAL(KIND(1d0))::exp1,exp2,mY,yY,ptY,yw,ptw,mYw,dphiYw
    LOGICAL,DIMENSION(2)::cmscuts
    INTEGER::i,kk,jj,l,IHEP,IMOTH,IST,ID,ID1
    INTEGER,DIMENSION(100)::BHADRON
    LOGICAL::NEWBHADRON,FOUNDY,FOUNDW
    INTEGER::NTYPE_BHADRON=0,N_W_MUON,N_Y_MUON,N_Y,N_W,this_Y,this_w
    SAVE NTYPE_BHADRON,BHADRON
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonp_Y,all_p_muon_w
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonm_Y,all_p_neut_w
    INTEGER,DIMENSION(10)::charge_w
    INTEGER,DIMENSION(10)::INDEX_Y,INDEX_W
    REAL(KIND(1d0)),DIMENSION(2,2,4)::p_muon_Y,p_muon_Y2
    REAL(KIND(1d0)),DIMENSION(2,4)::p_muon_w
    REAL(KIND(1d0)),DIMENSION(4)::p_Y,p_w,p_Yw
    REAL(KIND(1d0)),DIMENSION(4)::p_miss,p_lplm
    REAL(KIND(1d0))::invm_lplm
    INTEGER,DIMENSION(2)::i_Y,i2_Y
    INTEGER::i_W
    LOGICAL::Y_passcut,w_passcut
    REAL(KIND(1d0))::ptmax_Y,ptmax2_Y,ptmax_w,pt_muon1,pt_muon2,eta_muon1,eta_muon2
    REAL(KIND(1d0))::www,yftot
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
    INTEGER,DIMENSION(2)::IY_PASSCUT
    ! INCOMING PARTONS MAY TRAVEL IN THE SAME DIRECTION: IT''S 
    ! A POWER-SUPPRESSED EFFECT, SO THROW THE EVENT AWAY
    IF(SIGN(1.D0,PHEP(3,1)).EQ.SIGN(1.D0,PHEP(3,2)))THEN
       WRITE(*,*)"WARNING 1 in plot_py8_fill"
       GOTO 999
    ENDIF
    IEVENT=IEVENT+1
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    N_W_MUON=0
    N_Y_MUON=0
    N_Y=0
    N_W=0
    INDEX_Y(1:10)=0
    INDEX_w(1:10)=0
    this_Y=0
    this_w=0
    charge_w(1:10)=0
    p_miss(1:4)=0d0
    DO IHEP=1,NHEP
       IST=ISTHEP(IHEP) ! status
       ID=IDHEP(IHEP) ! pdg
       ID1=IHADR(ID) ! the massive quark's PDG in hadron, defined later
       IF(IST.EQ.1.AND.(ABS(ID).EQ.12.OR.ABS(ID).EQ.14.OR.ABS(ID).EQ.16))THEN
          p_miss(1:4)=p_miss(1:4)+PHEP(1:4,IHEP)
       ENDIF
       IF(IST.EQ.1.AND.(ABS(ID).EQ.13.OR.ABS(ID).EQ.11.OR.&
            ABS(ID).EQ.12.OR.ABS(ID).EQ.14))THEN
          IF(N_Y.LE.10.OR.N_W.LE.10)THEN
             ! try to find whether it is coming from Y(1S) decay
             ! or from W decay
             IMOTH=JMOHEP(1,IHEP)
             FOUNDY=.FALSE.
             FOUNDW=.FALSE.
             DO jj=1,NHEP
                IF(IDHEP(IMOTH).EQ.553)THEN
                   ! it is from Y(1S) decay
                   FOUNDY=.TRUE.
                   this_Y=0
                   DO i=1,N_Y
                      IF(IMOTH.EQ.INDEX_Y(i))THEN
                         this_Y=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_Y.EQ.0)THEN
                      N_Y=N_Y+1
                      IF(N_Y.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY Y !"
                         STOP
                      ENDIF
                      this_Y=N_Y
                      INDEX_Y(N_Y)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(ABS(IDHEP(IMOTH)).EQ.24)THEN
                   FOUNDW=.TRUE.
                   this_w=0
                   DO i=1,N_W
                      IF(IMOTH.EQ.INDEX_W(i))THEN
                         this_w=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_w.EQ.0)THEN
                      N_W=N_W+1
                      IF(N_W.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY W !"
                         STOP
                      ENDIF
                      this_w=N_W
                      INDEX_w(N_W)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(ABS(IDHEP(IMOTH)).NE.13.AND.&
                     ABS(IDHEP(IMOTH)).NE.11.AND.&
                     ABS(IDHEP(IMOTH)).NE.12.AND.&
                     ABS(IDHEP(IMOTH)).NE.14.AND.IMOTH.NE.3)THEN
                   FOUNDY=.FALSE.
                   FOUNDW=.FALSE.
                   EXIT
                ENDIF
                IMOTH=JMOHEP(1,IMOTH)
             ENDDO
             IF(FOUNDY)THEN
                IF(ID.GT.0)THEN
                   all_p_muonm_Y(this_Y,1:4)=PHEP(1:4,IHEP)
                ELSE
                   all_p_muonp_Y(this_Y,1:4)=PHEP(1:4,IHEP)
                ENDIF
             ELSEIF(FOUNDW)THEN
                IF(ABS(ID).EQ.13.OR.ABS(ID).EQ.11)THEN
                   all_p_muon_w(this_w,1:4)=PHEP(1:4,IHEP)
                   IF(charge_w(this_w).EQ.0)THEN
                      IF(ID.GT.0)THEN
                         charge_w(this_w)=-1 ! w-
                      ELSE
                         charge_w(this_w)=1 ! w+
                      ENDIF
                   ENDIF
                ELSE
                   all_p_neut_w(this_w,1:4)=PHEP(1:4,IHEP)
                ENDIF
             ENDIF
          ENDIF
       ELSEIF(IST.EQ.1.AND.ID.EQ.553)THEN
          ! not decayed Y(1S)
          WRITE(*,*)"ERROR:Found an undecayed Y(1S)"
          STOP
          IF(N_Y.EQ.1.OR.N_Y_MUON.GE.1)THEN
             WRITE(*,*)"WARNING:Found two Y !"
             WRITE(*,*)IEVENT,USED_EVENT
          ENDIF
          IF(N_Y_MUON.LE.1.AND.N_Y.EQ.0)THEN
             N_Y=N_Y+1
             p_Y(1:4)=PHEP(1:4,IHEP)
          ENDIF
       ENDIF
    ENDDO
    IF(N_Y.EQ.0)THEN
       RETURN
    ENDIF
    IF(N_W.EQ.0)THEN
       WRITE(*,*)"ERROR:Cannot find the two decayed leptons from W boson"
       STOP
    ENDIF
    USED_EVENT=USED_EVENT+1
    IY_PASSCUT(1:2)=0
    IF(N_Y.GE.1)THEN
       ! in the case that there are several Y(1S)
       DO jj=1,2
          i_Y(jj)=0
          i2_Y(jj)=0
          ptmax_Y=0d0
          ptmax2_Y=0d0
          DO i=1,N_Y
             pt_muon1=DSQRT(all_p_muonp_Y(i,1)**2+all_p_muonp_Y(i,2)**2)
             pt_muon2=DSQRT(all_p_muonm_Y(i,1)**2+all_p_muonm_Y(i,2)**2)
             eta_muon1=prapidity(all_p_muonp_Y(i,1:4))
             eta_muon2=prapidity(all_p_muonm_Y(i,1:4))
             p_Y(1:4)=all_p_muonp_Y(i,1:4)+all_p_muonm_Y(i,1:4)
             ptY=DSQRT(p_Y(1)**2+p_Y(2)**2)
             ! CMS cuts
             ! communication with Stefanos Leontsinis <Stefanos.Leontsinis@cern.ch>
             CALL CMSYCUTS(Y_passcut,jj,p_Y,all_p_muonp_Y(i,1:4),&
                  all_p_muonm_Y(i,1:4))
             IF(Y_passcut)THEN
                IY_PASSCUT(jj)=IY_PASSCUT(jj)+1
                IF(ptmax_Y.LT.ptY)THEN
                   ptmax2_Y=ptmax_Y
                   ptmax_Y=ptY
                   i2_Y(jj)=i_Y(jj)
                   i_Y(jj)=i
                ELSEIF(ptmax2_Y.LT.ptY)THEN
                   ptmax2_Y=ptY
                   i2_Y(jj)=i
                ENDIF
             ENDIF
          ENDDO
          IF(i_Y(jj).NE.0)THEN
             p_muon_Y(jj,1,1:4)=all_p_muonp_Y(i_Y(jj),1:4)
             p_muon_Y(jj,2,1:4)=all_p_muonm_Y(i_Y(jj),1:4)
          ENDIF
          IF(i2_Y(jj).NE.0)THEN
             p_muon_Y2(jj,1,1:4)=all_p_muonp_Y(i2_Y(jj),1:4)
             p_muon_Y2(jj,2,1:4)=all_p_muonm_Y(i2_Y(jj),1:4)
          ENDIF
       ENDDO
    ENDIF
    p_muon_w(1,1:4)=all_p_muon_w(1,1:4)
    p_muon_w(2,1:4)=all_p_neut_w(1,1:4)
    IF(N_W.GT.1)THEN
       WRITE(*,*)"=============================================="
       PRINT *, IEVENT,N_W
       DO IHEP=1,NHEP
          WRITE(*,*)IHEP,ISTHEP(IHEP),IDHEP(IHEP),JMOHEP(1,IHEP),JMOHEP(2,IHEP)
       ENDDO
       WRITE(*,*)"=============================================="
       STOP "TOO MANY W boson"
    ENDIF
    DO jj=1,2
       CALL CMSWCUTS(w_passcut,jj,p_muon_w(1,1:4),p_miss(1:4))
       cmscuts(jj)=w_passcut.AND.(i_Y(jj).NE.0)
    ENDDO
    p_w(1:4)=p_muon_w(1,1:4)+p_muon_w(2,1:4)
    ptw=DSQRT(p_w(1)**2+p_w(2)**2)
    yw=rapidity(p_w(1:4))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=1,2
       IF(.NOT.cmscuts(i))CYCLE
       l=(i-1)*inum_plots
       p_Y(1:4)=p_muon_Y(i,1,1:4)+p_muon_Y(i,2,1:4)
       ptY=DSQRT(p_Y(1)**2+p_Y(2)**2)
       yY=rapidity(p_Y(1:4))
       mY=DSQRT(p_Y(4)**2-p_Y(1)**2-p_Y(2)**2-p_Y(3)**2)
       p_Yw(1:4)=p_Y(1:4)+p_w(1:4)
       mYw=DSQRT(p_Yw(4)**2-p_Yw(1)**2-p_Yw(2)**2-p_Yw(3)**2)
       dphiYw=ph4(p_Y(1),p_Y(2),p_Y(3))
       dphiYw = dphiYw-ph4(p_w(1),p_w(2),p_w(3))
       dphiYw = DABS(dphiYw)
       IF(dphiYw.GT.pi)dphiYw=2d0*pi-dphiYw
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,ptY,wgts)
       CALL HISTOGRAM_fill(l+3,yY,wgts)
       CALL HISTOGRAM_fill(l+4,mY,wgts)
       CALL HISTOGRAM_fill(l+5,ptw,wgts)
       CALL HISTOGRAM_fill(l+6,yw,wgts)
       CALL HISTOGRAM_fill(l+7,mYw,wgts)
       CALL HISTOGRAM_fill(l+8,dphiYw,wgts)
       CALL HISTOGRAM_fill(l+9,dphiYw,wgts)
    ENDDO

    call HwU_add_points
999 RETURN      
  END SUBROUTINE plot_py8_fill

  SUBROUTINE HwU_write_file(xnorm)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file='helaconia_py8.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE='helaconia_py8.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='helaconia_py8.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='helaconia_py8.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

! Kinematic functions
  FUNCTION rapidity(p)
    ! rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::rapidity,c
    IF(p(4).EQ.0d0)THEN
       rapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/ABS(p(4))
    IF(ABS(c).GE.1d0)THEN
       rapidity =0d0
    ELSE
       rapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION rapidity

  FUNCTION prapidity(p)
    ! psudo-rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::prapidity,c
    IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
       prapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
    IF(ABS(c).EQ.1d0)THEN
       prapidity = 0d0
    ELSE
       prapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION prapidity

  ! returns the PDG code of the heavier quark in the hadron of PDG code ID
  FUNCTION IHADR(ID)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ID
    INTEGER::IHADR,ID1
    IF(ID.NE.0)THEN
       ID1=ABS(ID)
       IF(ID1.GT.10000)ID1=ID1-1000*INT(ID1/1000)
       IHADR=ID1/(10**INT(LOG10(DFLOAT(ID1))))
    ELSE
       IHADR=0
    ENDIF
    RETURN
  END FUNCTION IHADR

  FUNCTION ph4(px,py,pz)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::px,py,pz
    REAL(KIND(1d0))::ph4,S2,S
    INTEGER::init=0
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    SAVE init
    IF(px**2+py**2.LE.0d0)THEN
       IF(pz.GE.0d0)ph4=0d0
       IF(pz.LT.0)ph4=pi
       RETURN
    ENDIF
    S2=px**2/(px**2+py**2)
    S=DSQRT(S2)
    IF(S.GT.1d0)THEN
       WRITE(*,*)'PH4(X) WARNING S=',S
       ph4=0
       IF(px.LT.0d0)ph4=pi
       RETURN
    ENDIF
    IF(px.LT.0)S=-DSQRT(S2)
    ph4=DACOS(S)
    IF(py.LT.0)ph4=2*pi-ph4
    RETURN
  END FUNCTION ph4

  SUBROUTINE CMSYCUTS(passcut,imode,pY,pYmup,pYmum)
    ! CMS Y(1S) cuts
    ! imode: 1, no cuts (only generation cuts)
    ! imode: 2, CMS fiducial cuts (communication with Stefanos Leontsinis <Stefanos.Leontsinis@cern.ch>)
    IMPLICIT NONE
    LOGICAL,INTENT(OUT)::passcut
    INTEGER,INTENT(IN)::imode
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pY,pYmup,pYmum
    REAL(KIND(1d0))::ptY,yY,ptYlp,ptYlm,etaYlp,etaYlm
    passcut=.TRUE.
    IF(imode.LE.1)RETURN
    ptYlp=DSQRT(pYmup(1)**2+pYmup(2)**2)
    ptYlm=DSQRT(pYmum(1)**2+pYmum(2)**2)
    etaYlp=prapidity(pYmup)
    etaYlm=prapidity(pYmum)
    ptY=DSQRT(pY(1)**2+pY(2)**2)
    yY=rapidity(pY)
    passcut=DABS(yY).LT.2.4d0
    passcut=passcut.AND.ptY.LT.100d0
    passcut=passcut.AND.ptYlp.GT.4d0
    passcut=passcut.AND.ptYlm.GT.4d0
    passcut=passcut.AND.DABS(etaYlp).LT.2.4d0
    passcut=passcut.AND.DABS(etaYlm).LT.2.4d0
    RETURN
  END SUBROUTINE CMSYCUTS

  SUBROUTINE CMSWCUTS(passcut,imode,xplw,xpmiss)
    ! CMS W cuts
    ! imode: 1, no cuts (only generation cuts)
    ! imode: 2, CMS fiducial cuts (communication with Stefanos Leontsinis <Stefanos.Leontsinis@cern.ch>)
    IMPLICIT NONE
    LOGICAL,INTENT(OUT)::passcut
    INTEGER,INTENT(IN)::imode
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::xplw,xpmiss
    REAL(KIND(1d0))::ptwl,ptwn,etawl,etawn,mtw,phil,phin
    passcut=.TRUE.
    IF(imode.EQ.1)RETURN
    ptwl=DSQRT(xplw(1)**2+xplw(2)**2)
    ptwn=DSQRT(xpmiss(1)**2+xpmiss(2)**2)
    etawl=prapidity(xplw(1:4))
    etawn=prapidity(xpmiss(1:4))
    passcut=DABS(etawl).LT.2.4d0.AND.ptwl.GT.30d0
    RETURN
  END SUBROUTINE CMSWCUTS
END MODULE plot_py8_user
