MODULE plot_py8_user
!============================================================
!
! PLOT FILE WITH PYTHIA8 FOR USER
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=52 ! num_plots=inum_plots*4
  INTEGER::inum_plots=13
  INTEGER,PARAMETER::n_width=1,n_height=1
  CHARACTER(len=12),DIMENSION(4)::char_cuts=(/'            ',&
       '(ATLAS incl)','(ATLAS fidu)','(CMS   fidu)'/)
  INTEGER::IEVENT=0,USED_EVENT=0
  SAVE IEVENT,USED_EVENT
CONTAINS
  SUBROUTINE plot_py8_begin(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    INTEGER,PARAMETER::max_weight=1
    CHARACTER(len=15),DIMENSION(max_weight),INTENT(IN)::weights_info
    CHARACTER(len=15)::weights_info2
    INTEGER::i,kk,l
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    INCLUDE 'histogram_common90.inc'
    top_yes=.TRUE.
    gnu_yes=.TRUE.
    root_yes=.TRUE.
    hwu_yes=.TRUE.
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    DO i=1,4
       l=(i-1)*inum_plots
       ! bookup is defined in analysis/hbook/dbook.f
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       !
       ! enlarge num_plots at the begining
       !
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       CALL histogram_book(l+ 1,'total rate'//ADJUSTL(TRIM(char_cuts(i))),&
            2,0d0,4d0,.FALSE.)
       CALL histogram_book(l+ 2,'psi pt'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 3,'psi y'//ADJUSTL(TRIM(char_cuts(i))),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 4,'psi m'//ADJUSTL(TRIM(char_cuts(i))),&
            100,2.8d0,4d0,.TRUE.)
       CALL histogram_book(l+ 5,'z pt'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 6,'z y'//ADJUSTL(TRIM(char_cuts(i))),&
               100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 7,'psi-z m'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,1000d0,.TRUE.)
       CALL histogram_book(l+ 8,'Delta phi(Z,psi)'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
       CALL histogram_book(l+ 9,'Delta phi(Z,psi),8.5<pt<10'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
       CALL histogram_book(l+10,'Delta phi(Z,psi),10<pt<14'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
       CALL histogram_book(l+11,'Delta phi(Z,psi),14<pt<18'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
       CALL histogram_book(l+12,'Delta phi(Z,psi),18<pt<30'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
       CALL histogram_book(l+13,'Delta phi(Z,psi),30<pt<100'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
    ENDDO

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_py8_begin

  SUBROUTINE plot_py8_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=18)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    CHARACTER(len=400)::psfile,rootfile
    CALL mclear
    CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'F',i,i,xnorm,0.d0)
       CALL mfinal(i)
       CALL mopera_3d(i,'F',i,i,xnorm,0.d0)
       CALL mfinal_3d(i)
    ENDDO

    WRITE(*,*)"INFO:total number of events:",IEVENT
    WRITE(*,*)"INFO:total number of used events:",USED_EVENT

    ytit='sigma per bin [nb]'
    CALL open_topdrawer_file
    CALL HISTOGRAM_END(1,ytit,' ',' ')
    CALL close_topdrawer_file

    CALL open_gnuplot_file
    psfile="helaconia_py8.ps"
    CALL HISTOGRAM_END(2,ytit,psfile,' ')
    CALL close_gnuplot_file

    CALL open_root_file
    rootfile="helaconia_py8.root"
    CALL HISTOGRAM_END(3,ytit,' ',rootfile)
    ! wirte the end of the .C file for root
    WRITE (96, 100)
100 FORMAT(/1X,&
         ' hohisto -> cd();',/1X,&
         ' if (histos -> GetEntries() > 0 ) then {',/1X,&
         '  histos->Write();',/1X,&
         '  hohisto -> Close();',/1X,&
         ' }',/1X,'}')
    CALL close_root_file

    CALL HISTOGRAM_END(4,ytit,' ',' ')
    CALL HwU_write_file(xnorm)
    RETURN                
  END SUBROUTINE plot_py8_end

  SUBROUTINE plot_py8_fill(wgts)
    IMPLICIT NONE
    !INCLUDE '../include/HEPMC90.INC'
    INCLUDE 'HEPMC90.INC' ! it will be copied to shower directory
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(250),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e,ptpsilp,ptpsilm,etapsilp,etapsilm
    REAL(KIND(1d0))::ptzlp,ptzlm,etazlp,etazlm
    REAL(KIND(1d0))::exp1,exp2,mpsi,ypsi,ptpsi,yz,ptz,mpsiz,dphipsiz
    LOGICAL,DIMENSION(4)::atlascuts
    INTEGER::i,kk,jj,l,IHEP,IMOTH,IST,ID,ID1
    INTEGER,DIMENSION(100)::BHADRON
    LOGICAL::NEWBHADRON,FOUNDPSI,FOUNDZ
    INTEGER::NTYPE_BHADRON=0,N_Z_MUON,N_PSI_MUON,N_PSI,N_Z,this_psi,this_z
    SAVE NTYPE_BHADRON,BHADRON
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonp_psi,all_p_muonp_z
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonm_psi,all_p_muonm_z
    INTEGER,DIMENSION(10)::INDEX_psi,INDEX_Z
    REAL(KIND(1d0)),DIMENSION(4,2,4)::p_muon_psi
    REAL(KIND(1d0)),DIMENSION(2,4)::p_muon_z
    REAL(KIND(1d0)),DIMENSION(4)::p_psi,p_z,p_psiz
    INTEGER,DIMENSION(4)::i_psi
    INTEGER::i_Z
    LOGICAL::psi_passcut,z_passcut
    REAL(KIND(1d0))::ptmax_psi,ptmax_z,pt_muon1,pt_muon2,eta_muon1,eta_muon2
    REAL(KIND(1d0))::www,yftot
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
    ! INTEGER::PSI_PASSCUT
    ! INCOMING PARTONS MAY TRAVEL IN THE SAME DIRECTION: IT''S 
    ! A POWER-SUPPRESSED EFFECT, SO THROW THE EVENT AWAY
    IF(SIGN(1.D0,PHEP(3,1)).EQ.SIGN(1.D0,PHEP(3,2)))THEN
       WRITE(*,*)"WARNING 1 in plot_py8_fill"
       GOTO 999
    ENDIF
    IEVENT=IEVENT+1
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    N_Z_MUON=0
    N_PSI_MUON=0
    N_PSI=0
    N_Z=0
    INDEX_psi(1:10)=0
    INDEX_z(1:10)=0
    this_psi=0
    this_z=0
    DO IHEP=1,NHEP
       IST=ISTHEP(IHEP) ! status
       ID=IDHEP(IHEP) ! pdg
       ID1=IHADR(ID) ! the massive quark's PDG in hadron, defined later
       IF(IST.EQ.1.AND.(ABS(ID).EQ.13.OR.ABS(ID).EQ.11))THEN
          IF(N_PSI.LE.10.OR.N_Z.LE.10)THEN
             ! try to find whether it is coming from J/psi decay
             ! or from Z decay
             IMOTH=JMOHEP(1,IHEP)
             FOUNDPSI=.FALSE.
             FOUNDZ=.FALSE.
             DO jj=1,NHEP
                IF(IDHEP(IMOTH).EQ.443)THEN
                   ! it is from J/psi decay
                   FOUNDPSI=.TRUE.
                   this_psi=0
                   DO i=1,N_PSI
                      IF(IMOTH.EQ.INDEX_psi(i))THEN
                         this_psi=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_psi.EQ.0)THEN
                      N_PSI=N_PSI+1
                      IF(N_PSI.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY PSI !"
                         STOP
                      ENDIF
                      this_psi=N_PSI
                      INDEX_psi(N_PSI)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(IDHEP(IMOTH).EQ.23)THEN
                   FOUNDZ=.TRUE.
                   this_z=0
                   DO i=1,N_Z
                      IF(IMOTH.EQ.INDEX_Z(i))THEN
                         this_z=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_z.EQ.0)THEN
                      N_Z=N_Z+1
                      IF(N_Z.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY Z !"
                         STOP
                      ENDIF
                      this_z=N_Z
                      INDEX_z(N_Z)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(ABS(IDHEP(IMOTH)).NE.13.AND.&
                     ABS(IDHEP(IMOTH)).NE.11.AND.IMOTH.NE.3)THEN
                   FOUNDPSI=.FALSE.
                   FOUNDZ=.FALSE.
                   EXIT
                ENDIF
                IMOTH=JMOHEP(1,IMOTH)
             ENDDO
             IF(FOUNDPSI)THEN
                IF(ID.GT.0)THEN
                   all_p_muonm_psi(this_psi,1:4)=PHEP(1:4,IHEP)
                ELSE
                   all_p_muonp_psi(this_psi,1:4)=PHEP(1:4,IHEP)
                ENDIF
             ELSEIF(FOUNDZ)THEN
                !N_Z_MUON=N_Z_MUON+1
                IF(ID.GT.0)THEN
                   all_p_muonm_z(this_z,1:4)=PHEP(1:4,IHEP)
                ELSE
                   all_p_muonp_z(this_z,1:4)=PHEP(1:4,IHEP)
                ENDIF
             ENDIF
          ENDIF
       ELSEIF(IST.EQ.1.AND.ID.EQ.443)THEN
          ! not decayed psi
          WRITE(*,*)"ERROR:Found an undecayed J/psi"
          STOP
          IF(N_PSI.EQ.1.OR.N_PSI_MUON.GE.1)THEN
             WRITE(*,*)"WARNING:Found two psi !"
             WRITE(*,*)IEVENT,USED_EVENT
          ENDIF
          IF(N_PSI_MUON.LE.1.AND.N_PSI.EQ.0)THEN
             N_PSI=N_PSI+1
             p_psi(1:4)=PHEP(1:4,IHEP)
          ENDIF
       ENDIF
    ENDDO
    IF(N_PSI.EQ.0)THEN
       RETURN
    ENDIF
    IF(N_Z.EQ.0)THEN
       WRITE(*,*)"ERROR:Cannot find the two decayed muon from Z boson"
       STOP
    ENDIF
    USED_EVENT=USED_EVENT+1
    IF(N_PSI.GE.1)THEN
       ! in the case that there are several J/psi
       DO jj=1,4
          i_psi(jj)=0
          ptmax_psi=0d0
          DO i=1,N_PSI
             pt_muon1=DSQRT(all_p_muonp_psi(i,1)**2+all_p_muonp_psi(i,2)**2)
             pt_muon2=DSQRT(all_p_muonm_psi(i,1)**2+all_p_muonm_psi(i,2)**2)
             eta_muon1=prapidity(all_p_muonp_psi(i,1:4))
             eta_muon2=prapidity(all_p_muonm_psi(i,1:4))
             p_psi(1:4)=all_p_muonp_psi(i,1:4)+all_p_muonm_psi(i,1:4)
             ptpsi=DSQRT(p_psi(1)**2+p_psi(2)**2)
             ! ATLAS cuts
             ! EPJC 75 (2015) 229
             CALL ATLASPSICUTS(psi_passcut,jj,p_psi,all_p_muonp_psi(i,1:4),&
                  all_p_muonm_psi(i,1:4))
             IF(psi_passcut)THEN
                IF(ptmax_psi.LT.ptpsi)THEN
                   ptmax_psi=ptpsi
                   i_psi(jj)=i
                ENDIF
             ENDIF
          ENDDO
          IF(i_psi(jj).NE.0)THEN
             p_muon_psi(jj,1,1:4)=all_p_muonp_psi(i_psi(jj),1:4)
             p_muon_psi(jj,2,1:4)=all_p_muonm_psi(i_psi(jj),1:4)
          ENDIF
       ENDDO
    ENDIF
    p_muon_z(1,1:4)=all_p_muonp_z(1,1:4)
    p_muon_z(2,1:4)=all_p_muonm_z(1,1:4)
    IF(N_Z.GT.1)THEN
       WRITE(*,*)"=============================================="
       PRINT *, IEVENT,N_Z
       DO IHEP=1,NHEP
          WRITE(*,*)IHEP,ISTHEP(IHEP),IDHEP(IHEP),JMOHEP(1,IHEP),JMOHEP(2,IHEP)
       ENDDO
       WRITE(*,*)"=============================================="
       STOP "TOO MANY Z boson"
    ENDIF
    DO jj=1,4
       CALL ATLASZCUTS(z_passcut,jj,p_muon_z(1,1:4),p_muon_z(2,1:4))
       atlascuts(jj)=z_passcut.AND.(i_psi(jj).NE.0)
    ENDDO
    p_z(1:4)=p_muon_z(1,1:4)+p_muon_z(2,1:4)
    ptz=DSQRT(p_z(1)**2+p_z(2)**2)
    yz=rapidity(p_z(1:4))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=1,4
       IF(.NOT.atlascuts(i))CYCLE
       l=(i-1)*inum_plots
       p_psi(1:4)=p_muon_psi(i,1,1:4)+p_muon_psi(i,2,1:4)
       ptpsi=DSQRT(p_psi(1)**2+p_psi(2)**2)
       ypsi=rapidity(p_psi(1:4))
       mpsi=DSQRT(p_psi(4)**2-p_psi(1)**2-p_psi(2)**2-p_psi(3)**2)
       p_psiz(1:4)=p_psi(1:4)+p_z(1:4)
       mpsiz=DSQRT(p_psiz(4)**2-p_psiz(1)**2-p_psiz(2)**2-p_psiz(3)**2)
       dphipsiz=ph4(p_psi(1),p_psi(2),p_psi(3))
       dphipsiz = dphipsiz-ph4(p_z(1),p_z(2),p_z(3))
       dphipsiz = DABS(dphipsiz)
       IF(dphipsiz.GT.pi)dphipsiz=2d0*pi-dphipsiz
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,ptpsi,wgts)
       CALL HISTOGRAM_fill(l+3,ypsi,wgts)
       CALL HISTOGRAM_fill(l+4,mpsi,wgts)
       CALL HISTOGRAM_fill(l+5,ptz,wgts)
       CALL HISTOGRAM_fill(l+6,yz,wgts)
       CALL HISTOGRAM_fill(l+7,mpsiz,wgts)
       CALL HISTOGRAM_fill(l+8,dphipsiz,wgts)
       IF(ptpsi.GT.8.5d0.AND.ptpsi.LT.10d0)CALL HISTOGRAM_fill(l+9,dphipsiz,wgts)
       IF(ptpsi.GT.10d0.AND.ptpsi.LT.14d0)CALL HISTOGRAM_fill(l+10,dphipsiz,wgts)
       IF(ptpsi.GT.14d0.AND.ptpsi.LT.18d0)CALL HISTOGRAM_fill(l+11,dphipsiz,wgts)
       IF(ptpsi.GT.18d0.AND.ptpsi.LT.30d0)CALL HISTOGRAM_fill(l+12,dphipsiz,wgts)
       IF(ptpsi.GT.30d0.AND.ptpsi.LT.100d0)CALL HISTOGRAM_fill(l+13,dphipsiz,wgts)
    ENDDO

    call HwU_add_points
999 RETURN      
  END SUBROUTINE plot_py8_fill

  SUBROUTINE HwU_write_file(xnorm)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file='helaconia_py8.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE='helaconia_py8.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='helaconia_py8.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='helaconia_py8.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

! Kinematic functions
  FUNCTION rapidity(p)
    ! rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::rapidity,c
    IF(p(4).EQ.0d0)THEN
       rapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/ABS(p(4))
    IF(ABS(c).GE.1d0)THEN
       rapidity =0d0
    ELSE
       rapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION rapidity

  FUNCTION prapidity(p)
    ! psudo-rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::prapidity,c
    IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
       prapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
    IF(ABS(c).EQ.1d0)THEN
       prapidity = 0d0
    ELSE
       prapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION prapidity

  ! returns the PDG code of the heavier quark in the hadron of PDG code ID
  FUNCTION IHADR(ID)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ID
    INTEGER::IHADR,ID1
    IF(ID.NE.0)THEN
       ID1=ABS(ID)
       IF(ID1.GT.10000)ID1=ID1-1000*INT(ID1/1000)
       IHADR=ID1/(10**INT(LOG10(DFLOAT(ID1))))
    ELSE
       IHADR=0
    ENDIF
    RETURN
  END FUNCTION IHADR

  FUNCTION ph4(px,py,pz)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::px,py,pz
    REAL(KIND(1d0))::ph4,S2,S
    INTEGER::init=0
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    SAVE init
    IF(px**2+py**2.LE.0d0)THEN
       IF(pz.GE.0d0)ph4=0d0
       IF(pz.LT.0)ph4=pi
       RETURN
    ENDIF
    S2=px**2/(px**2+py**2)
    S=DSQRT(S2)
    IF(S.GT.1d0)THEN
       WRITE(*,*)'PH4(X) WARNING S=',S
       ph4=0
       IF(px.LT.0d0)ph4=pi
       RETURN
    ENDIF
    IF(px.LT.0)S=-DSQRT(S2)
    ph4=DACOS(S)
    IF(py.LT.0)ph4=2*pi-ph4
    RETURN
  END FUNCTION ph4

  SUBROUTINE ATLASPSICUTS(passcut,imode,ppsi,ppsimup,ppsimum)
    ! ATLAS cuts
    ! EPJC 75 (2015) 229
    ! imode: 1, no cuts (only generation cuts)
    ! imode: 2, ATLAS inclusive cuts (no muon cuts from J/psi decay)
    ! imode: 3, ATLAS fiducial cuts (with muon cuts from J/psi decay)
    ! imode: 4, CMS fiducial cuts
    IMPLICIT NONE
    LOGICAL,INTENT(OUT)::passcut
    INTEGER,INTENT(IN)::imode
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::ppsi,ppsimup,ppsimum
    REAL(KIND(1d0))::ptpsi,ypsi,ptpsilp,ptpsilm,etapsilp,etapsilm
    passcut=.TRUE.
    IF(imode.LE.1)RETURN
    ptpsi=DSQRT(ppsi(1)**2+ppsi(2)**2)
    ypsi=rapidity(ppsi)
    passcut=passcut.AND.ptpsi.GT.8.5d0.AND.ptpsi.LT.100d0
    passcut=passcut.AND.ABS(ypsi).LT.2.1d0
    IF(imode.LE.2)RETURN
    ptpsilp=DSQRT(ppsimup(1)**2+ppsimup(2)**2)
    ptpsilm=DSQRT(ppsimum(1)**2+ppsimum(2)**2)
    etapsilp=prapidity(ppsimup)
    etapsilm=prapidity(ppsimum)
    IF(imode.EQ.4)THEN
       passcut=ptpsi.GT.8.5d0
       passcut=passcut.AND.ABS(ypsi).LT.2.1d0
       passcut=passcut.AND.ABS(etapsilp).LT.2.5d0.AND.&
            ABS(etapsilm).LT.2.5d0
       RETURN
    ENDIF
    passcut=passcut.AND.MAX(ptpsilp,ptpsilm).GT.4d0
    IF(ptpsilp.GE.ptpsilm)THEN
       passcut=passcut.AND.ABS(etapsilp).LT.2.5d0
       passcut=passcut.AND.((ptpsilm.GT.2.5d0.AND.&
            ABS(etapsilm).GT.1.3d0.AND.ABS(etapsilm).LT.2.5d0).OR.&
            (ptpsilm.GT.3.5d0.AND.ABS(etapsilm).LT.1.3d0))
    ELSE
       passcut=passcut.AND.ABS(etapsilm).LT.2.5d0
       passcut=passcut.AND.((ptpsilp.GT.2.5d0.AND.&
            ABS(etapsilp).GT.1.3d0.AND.ABS(etapsilp).LT.2.5d0).OR.&
            (ptpsilp.GT.3.5d0.AND.ABS(etapsilp).LT.1.3d0))
    ENDIF
    RETURN
  END SUBROUTINE ATLASPSICUTS

  SUBROUTINE ATLASZCUTS(passcut,imode,xplpz,xplmz)
    IMPLICIT NONE
    LOGICAL,INTENT(OUT)::passcut
    INTEGER,INTENT(IN)::imode
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::xplpz,xplmz
    REAL(KIND(1d0))::ptzlp,ptzlm,etazlp,etazlm
    passcut=.TRUE.
    IF(imode.EQ.1)RETURN
    ptzlp=DSQRT(xplpz(1)**2+xplpz(2)**2)
    ptzlm=DSQRT(xplmz(1)**2+xplmz(2)**2)
    etazlp=prapidity(xplpz(1:4))
    etazlm=prapidity(xplmz(1:4))
    passcut=ABS(etazlp).LT.2.5d0.AND.ABS(etazlm).LT.2.5d0
    passcut=passcut.AND.MAX(ptzlp,ptzlm).GT.25d0
    passcut=passcut.AND.MIN(ptzlp,ptzlm).GT.15d0
    RETURN
  END SUBROUTINE ATLASZCUTS
END MODULE plot_py8_user
