MODULE plot_py8_user
!============================================================
!
! PLOT FILE WITH PYTHIA8 FOR USER
!
!============================================================
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=17
  INTEGER,PARAMETER::n_width=1,n_height=1
  INTEGER::IEVENT=0,USED_EVENT=0
  SAVE IEVENT,USED_EVENT
CONTAINS
  SUBROUTINE plot_py8_begin(nwgt,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt
    INTEGER,PARAMETER::max_weight=1
    CHARACTER(len=15),DIMENSION(max_weight),INTENT(IN)::weights_info
    CHARACTER(len=15)::weights_info2
    INTEGER::i,kk,l
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    INCLUDE 'histogram_common90.inc'
    top_yes=.TRUE.
    gnu_yes=.TRUE.
    root_yes=.TRUE.
    hwu_yes=.TRUE.
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    l=0
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    CALL histogram_book(l+ 1,'total rate    ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2,'|Delta y|  ',&
         100,0d0,5d0,.TRUE.)
    CALL histogram_book(l+ 3,'|Delta phi| ',&
         100,-0.0001d0,3.1416d0,.TRUE.)
    CALL histogram_book(l+ 4,'psi Y inv. mass ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 5,'psi Y trans mom ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 6,'q12 ',&
         100,0d0,50d0,.TRUE.)
    CALL histogram_book(l+ 7,'Cos(alpha) ',&
         100,0d0,1d0,.FALSE.)
    CALL histogram_book(l+ 8,'|Delta eta| ',&
         100,0d0,4d0,.TRUE.)
    CALL histogram_book(l+ 9,'psi Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 10,'Y Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 11,'leading Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 12,'subleading Pt ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 13,'Pt diff ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book(l+ 14,'|Pt1-Pt2| ',&
         100,0d0,100d0,.TRUE.)
    CALL histogram_book3d(l+ 15,'Cos(th1)-Cos(th2) ',&
         100,-1d0,1d0,100,-1d0,1d0,.FALSE.)
    CALL histogram_book3d(l+ 16,'|Delta y| vs. Pt(psi-Y)',&
         100,0d0,5d0,100,0d0,100d0,.TRUE.)
    CALL histogram_book3d(l+ 17,'|Delta y| vs. |Delta phi|',&
         100,0d0,5d0,100,-0.0001d0,3.1416d0,.FALSE.)

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_py8_begin

  SUBROUTINE plot_py8_end(xnorm)
    IMPLICIT NONE
    CHARACTER(len=18)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    !INCLUDE '../hbook/dbookf90.inc'
    INCLUDE 'dbookf90.inc' ! it will be copied to shower directory
    CHARACTER(len=400)::psfile,rootfile
    CALL mclear
    CALL mclear_3d
    DO i=1,NPLOTS
       CALL mopera(i,'F',i,i,xnorm,0.d0)
       CALL mfinal(i)
       CALL mopera_3d(i,'F',i,i,xnorm,0.d0)
       CALL mfinal_3d(i)
    ENDDO

    WRITE(*,*)"INFO:total number of events:",IEVENT
    WRITE(*,*)"INFO:total number of used events:",USED_EVENT

    ytit='sigma per bin [nb]'
    CALL open_topdrawer_file
    CALL HISTOGRAM_END(1,ytit,' ',' ')
    CALL close_topdrawer_file

    CALL open_gnuplot_file
    psfile="helaconia_py8.ps"
    CALL HISTOGRAM_END(2,ytit,psfile,' ')
    CALL close_gnuplot_file

    CALL open_root_file
    rootfile="helaconia_py8.root"
    CALL HISTOGRAM_END(3,ytit,' ',rootfile)
    ! wirte the end of the .C file for root
    WRITE (96, 100)
100 FORMAT(/1X,&
         ' hohisto -> cd();',/1X,&
         ' if (histos -> GetEntries() > 0 ) then {',/1X,&
         '  histos->Write();',/1X,&
         '  hohisto -> Close();',/1X,&
         ' }',/1X,'}')
    CALL close_root_file

    CALL HISTOGRAM_END(4,ytit,' ',' ')
    CALL HwU_write_file(xnorm)
    RETURN                
  END SUBROUTINE plot_py8_end

  SUBROUTINE plot_py8_fill(wgts)
    IMPLICIT NONE
    !INCLUDE '../include/HEPMC90.INC'
    INCLUDE 'HEPMC90.INC' ! it will be copied to shower directory
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(250),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,mpsi
    REAL(KIND(1d0))::exp1,exp2,pt_muon1,pt_muon2,eta_muon1,eta_muon2
    REAL(KIND(1d0))::dy_psipsi,dphi_psipsi,M_psipsi,Q,ptpsi1,ptpsi2,ptpsimax,ptpsimin
    REAL(KIND(1d0))::ptpsidiff,Pt_psipsi,Pt1Pt2_psipsi,q12_psipsi,cosalpha,costh1,costh2
    REAL(KIND(1d0))::deta_psipsi
    REAL(KIND(1d0)),DIMENSION(4)::PBOO2
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    INTEGER::i,kk,jj,l,IHEP,IMOTH,IST,ID,ID1
    INTEGER,DIMENSION(100)::BHADRON
    LOGICAL::NEWBHADRON,FOUNDPSI,FOUNDY
    INTEGER::NTYPE_BHADRON=0,N_Y_MUON,N_PSI_MUON,N_PSI,N_Y,this_psi,this_y
    SAVE NTYPE_BHADRON,BHADRON
    REAL(KIND(1d0))::www,yftot,PP1,PP2
    REAL(KIND(1d0)),DIMENSION(3)::Pplain1,Pplain2
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonp_psi,all_p_muonp_y
    REAL(KIND(1d0)),DIMENSION(10,4)::all_p_muonm_psi,all_p_muonm_y
    LOGICAL,DIMENSION(10)::psi_passcut,Y_passcut
    INTEGER,DIMENSION(10)::INDEX_psi,INDEX_Y
    REAL(KIND(1d0)),DIMENSION(2,4)::p_muon_psi,p_muon_y
    REAL(KIND(1d0)),DIMENSION(4)::p_psi,p_y,PJpsiCMS,PYCMS,Plep1,Plep2
    INTEGER::i_psi,i_Y
    REAL(KIND(1d0))::ptmax_psi,ptmax_y
    ! INCOMING PARTONS MAY TRAVEL IN THE SAME DIRECTION: IT''S 
    ! A POWER-SUPPRESSED EFFECT, SO THROW THE EVENT AWAY
    IF(SIGN(1.D0,PHEP(3,1)).EQ.SIGN(1.D0,PHEP(3,2)))THEN
       WRITE(*,*)"WARNING 1 in plot_py8_fill"
       GOTO 999
    ENDIF
    IEVENT=IEVENT+1
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    N_Y_MUON=0
    N_PSI_MUON=0
    N_PSI=0
    N_Y=0
    INDEX_psi(1:10)=0
    INDEX_Y(1:10)=0
    this_psi=0
    this_y=0
    DO IHEP=1,NHEP
       IST=ISTHEP(IHEP) ! status
       ID=IDHEP(IHEP) ! pdg
       ID1=IHADR(ID) ! the massive quark's PDG in hadron, defined later
       IF(IST.EQ.1.AND.(ABS(ID).EQ.13.OR.ABS(ID).EQ.11))THEN
          IF(N_PSI.LE.10.OR.N_Y.LE.10)THEN
             ! try to find whether it is coming from J/psi decay
             ! or from Upsilon decay
             IMOTH=JMOHEP(1,IHEP)
             FOUNDPSI=.FALSE.
             FOUNDY=.FALSE.
             DO jj=1,NHEP
                IF(IDHEP(IMOTH).EQ.443)THEN
                   ! it is from J/psi decay
                   FOUNDPSI=.TRUE.
                   this_psi=0
                   DO i=1,N_PSI
                      IF(IMOTH.EQ.INDEX_psi(i))THEN
                         this_psi=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_psi.EQ.0)THEN
                      N_PSI=N_PSI+1
                      IF(N_PSI.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY PSI !"
                         STOP
                      ENDIF
                      this_psi=N_PSI
                      INDEX_psi(N_PSI)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(IDHEP(IMOTH).EQ.553)THEN
                   FOUNDY=.TRUE.
                   this_y=0
                   DO i=1,N_Y
                      IF(IMOTH.EQ.INDEX_Y(i))THEN
                         this_y=i
                         EXIT
                      ENDIF
                   ENDDO
                   IF(this_y.EQ.0)THEN
                      N_Y=N_Y+1
                      IF(N_Y.GT.10)THEN
                         WRITE(*,*)"FOUND TOO MANY Y !"
                         STOP
                      ENDIF
                      this_y=N_Y
                      INDEX_y(N_Y)=IMOTH
                   ENDIF
                   EXIT
                ELSEIF(ABS(IDHEP(IMOTH)).NE.13.AND.&
                     ABS(IDHEP(IMOTH)).NE.11.AND.IMOTH.NE.3)THEN
                   FOUNDPSI=.FALSE.
                   FOUNDY=.FALSE.
                   EXIT
                ENDIF
                IMOTH=JMOHEP(1,IMOTH)
             ENDDO
             IF(FOUNDPSI)THEN
                IF(ID.GT.0)THEN
                   all_p_muonm_psi(this_psi,1:4)=PHEP(1:4,IHEP)
                ELSE
                   all_p_muonp_psi(this_psi,1:4)=PHEP(1:4,IHEP)
                ENDIF
             ELSEIF(FOUNDY)THEN
                !N_Y_MUON=N_Y_MUON+1
                IF(ID.GT.0)THEN
                   all_p_muonm_y(this_y,1:4)=PHEP(1:4,IHEP)
                ELSE
                   all_p_muonp_y(this_y,1:4)=PHEP(1:4,IHEP)
                ENDIF
             ENDIF
          ENDIF
       ELSEIF(IST.EQ.1.AND.ID.EQ.443)THEN
          ! not decayed psi
          WRITE(*,*)"ERROR:Found an undecayed J/psi"
          STOP
          IF(N_PSI.EQ.1.OR.N_PSI_MUON.GE.1)THEN
             WRITE(*,*)"WARNING:Found two psi !"
             WRITE(*,*)IEVENT,USED_EVENT
          ENDIF
          IF(N_PSI_MUON.LE.1.AND.N_PSI.EQ.0)THEN
             N_PSI=N_PSI+1
             p_psi(1:4)=PHEP(1:4,IHEP)
          ENDIF
       ENDIF
    ENDDO
    IF(N_PSI.EQ.0)THEN
       RETURN
    ENDIF
    IF(N_Y.EQ.0)THEN
       WRITE(*,*)"ERROR:Cannot find the two decayed muon from Y(1S)"
       STOP
    ENDIF
    IF(N_PSI.GE.1)THEN
       i_psi=0
       ptmax_psi=0d0
       DO i=1,N_PSI
          pt_muon1=DSQRT(all_p_muonp_psi(i,1)**2+all_p_muonp_psi(i,2)**2)
          pt_muon2=DSQRT(all_p_muonm_psi(i,1)**2+all_p_muonm_psi(i,2)**2)
          eta_muon1=prapidity(all_p_muonp_psi(i,1:4))
          eta_muon2=prapidity(all_p_muonm_psi(i,1:4))
          psi_passcut(i)=pt_muon1.GT.2d0.AND.pt_muon2.GT.2d0.AND.&
               ABS(eta_muon1).LT.2d0.AND.ABS(eta_muon2).LT.2d0
          IF(psi_passcut(i))THEN
             p_psi(1:4)=all_p_muonp_psi(i,1:4)+all_p_muonm_psi(i,1:4)
             IF(ptmax_psi.LT.DSQRT(p_psi(1)**2+p_psi(2)**2))THEN
                ptmax_psi=DSQRT(p_psi(1)**2+p_psi(2)**2)
                i_psi=i
             ENDIF
          ENDIF
       ENDDO
       IF(i_psi.EQ.0)THEN
          RETURN
       ELSE
          p_muon_psi(1,1:4)=all_p_muonp_psi(i_psi,1:4)
          p_muon_psi(2,1:4)=all_p_muonm_psi(i_psi,1:4)
       ENDIF
    ENDIF
    p_muon_y(1,1:4)=all_p_muonp_y(1,1:4)
    p_muon_y(2,1:4)=all_p_muonm_y(1,1:4)
    IF(N_Y.GT.1)THEN
       WRITE(*,*)"=============================================="
       PRINT *, IEVENT,N_Y
       DO IHEP=1,NHEP
          WRITE(*,*)IHEP,ISTHEP(IHEP),IDHEP(IHEP),JMOHEP(1,IHEP),JMOHEP(2,IHEP)
       ENDDO
       WRITE(*,*)"=============================================="
       STOP "TOO MANY Y(1S)"
    ENDIF
    p_psi(1:4)=p_muon_psi(1,1:4)+p_muon_psi(2,1:4)
    p_y(1:4)=p_muon_y(1,1:4)+p_muon_y(2,1:4)
    pt_muon1=DSQRT(p_muon_psi(1,1)**2+p_muon_psi(1,2)**2)
    pt_muon2=DSQRT(p_muon_psi(2,1)**2+p_muon_psi(2,2)**2)
    eta_muon1=prapidity(p_muon_psi(1,1:4))
    eta_muon2=prapidity(p_muon_psi(2,1:4))
    dy_psipsi=DABS(rapidity(p_psi(1:4))-rapidity(p_y(1:4)))
    deta_psipsi=DABS(prapidity(p_psi(1:4))-prapidity(p_y(1:4)))
    dphi_psipsi=ph4(p_psi(1),p_psi(2),p_psi(3))
    dphi_psipsi=dphi_psipsi-ph4(p_y(1),p_y(2),p_y(3))
    dphi_psipsi=DABS(dphi_psipsi)
    IF(dphi_psipsi.GT.pi)dphi_psipsi=2*pi-dphi_psipsi
    M_psipsi=scalar_product(p_psi(1:4)+p_y(1:4),p_psi(1:4)+p_y(1:4))
    M_psipsi=DSQRT(M_psipsi)
    Q=scalar_product(p_psi(1:4),p_psi(1:4))
    Q=DSQRT(Q)
    ptpsi1=transverse(p_psi(1:4))
    ptpsi2=transverse(p_y(1:4))
    ptpsimax=MAX(ptpsi1,ptpsi2)
    ptpsimin=MIN(ptpsi1,ptpsi2)
    ptpsidiff=ptpsimax-ptpsimin
    Pt_psipsi=transverse(p_psi(1:4)+p_y(1:4))
    Pt1Pt2_psipsi=transverse(p_psi(1:4)-p_y(1:4))
    q12_psipsi=M_psipsi**2-4*Q**2
    q12_psipsi=DSQRT(DABS(q12_psipsi))
    PBOO2(1:3)=-p_psi(1:3)-p_y(1:3)
    PBOO2(4)=p_psi(4)+p_y(4)
    DO i=1,2
       CALL BOOSTL(M_psipsi,PBOO2,p_muon_psi(i,1:4))
       CALL BOOSTL(M_psipsi,PBOO2,p_muon_y(i,1:4))
    ENDDO
    PJpsiCMS(1:4)=p_psi(1:4)
    CALL BOOSTL(M_psipsi,PBOO2,PJpsiCMS(1:4))
    PYCMS(1:4)=p_y(1:4)
    CALL BOOSTL(M_psipsi,PBOO2,PYCMS(1:4))

    Pplain1(1:3)=vec_crossprod(p_muon_psi(1,1:3),p_muon_psi(2,1:3))
    Pplain2(1:3)=vec_crossprod(p_muon_y(1,1:3),p_muon_y(2,1:3))
    PP1=Pplain1(1)**2+Pplain1(2)**2+Pplain1(3)**2
    PP1=DSQRT(PP1)
    PP2=Pplain2(1)**2+Pplain2(2)**2+Pplain2(3)**2
    PP2=DSQRT(PP2)
    IF(PP1*PP2.EQ.0d0)THEN
       cosalpha=0d0
    ELSE
       cosalpha=Pplain1(1)*Pplain2(1)+Pplain1(2)*Pplain2(2) &
            +Pplain1(3)*Pplain2(3)
       cosalpha=DABS(cosalpha/(PP1*PP2))
    ENDIF
    ! costh1
    Q=scalar_product(PJpsiCMS(1:4),PJpsiCMS(1:4))
    Q=DSQRT(Q)
    PBOO2(1:3)=-PJpsiCMS(1:3)
    PBOO2(4)=PJpsiCMS(4)
    Plep1(1:4)=p_muon_psi(1,1:4)
    CALL BOOSTL(Q,PBOO2,Plep1)
    costh1=cosij(Plep1(1:3),PJpsiCMS(1:3))
    ! costh2
    Q=scalar_product(PYCMS(1:4),PYCMS(1:4))
    Q=DSQRT(Q)
    PBOO2(1:3)=-PYCMS(1:3)
    PBOO2(4)=PYCMS(4)
    Plep2(1:4)=p_muon_y(1,1:4)
    CALL BOOSTL(Q,PBOO2,Plep2)
    costh2=cosij(Plep2(1:3),PYCMS(1:3))
    USED_EVENT=USED_EVENT+1
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_FILL(l+1,var,wgts)
    CALL HISTOGRAM_FILL(l+2,dy_psipsi,wgts)
    CALL HISTOGRAM_FILL(l+3,dphi_psipsi,wgts)
    CALL HISTOGRAM_FILL(l+4,M_psipsi,wgts)
    CALL HISTOGRAM_FILL(l+5,Pt_psipsi,wgts)
    CALL HISTOGRAM_FILL(l+6,q12_psipsi,wgts)
    CALL HISTOGRAM_FILL(l+7,cosalpha,wgts)
    CALL HISTOGRAM_FILL(l+8,deta_psipsi,wgts)
    CALL HISTOGRAM_FILL(l+9,ptpsi1,wgts)
    CALL HISTOGRAM_FILL(l+10,ptpsi2,wgts)
    CALL HISTOGRAM_FILL(l+11,ptpsimax,wgts)
    CALL HISTOGRAM_FILL(l+12,ptpsimin,wgts)
    CALL HISTOGRAM_FILL(l+13,ptpsidiff,wgts)
    CALL HISTOGRAM_FILL(l+14,Pt1Pt2_psipsi,wgts)
    CALL HISTOGRAM_FILL3D(l+15,costh1,costh2,wgts)
    CALL HISTOGRAM_FILL3D(l+16,dy_psipsi,Pt_psipsi,wgts)
    CALL HISTOGRAM_FILL3D(l+17,dy_psipsi,dphi_psipsi,wgts)

    call HwU_add_points
999 RETURN      
  END SUBROUTINE plot_py8_fill

  SUBROUTINE HwU_write_file(xnorm)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file='helaconia_py8.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=99,FILE='helaconia_py8.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE='helaconia_py8.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE='helaconia_py8.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

! Kinematic functions
  FUNCTION rapidity(p)
    ! rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::rapidity,c
    IF(p(4).EQ.0d0)THEN
       rapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/ABS(p(4))
    IF(ABS(c).GE.1d0)THEN
       rapidity =0d0
    ELSE
       rapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION rapidity

  FUNCTION prapidity(p)
    ! psudo-rapidity
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::prapidity,c
    IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
       prapidity = 0d0
       RETURN
    ENDIF
    c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
    IF(ABS(c).EQ.1d0)THEN
       prapidity = 0d0
    ELSE
       prapidity=0.5d0*LOG((1+c)/(1-c))
    ENDIF
  END FUNCTION prapidity

  ! returns the PDG code of the heavier quark in the hadron of PDG code ID
  FUNCTION IHADR(ID)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ID
    INTEGER::IHADR,ID1
    IF(ID.NE.0)THEN
       ID1=ABS(ID)
       IF(ID1.GT.10000)ID1=ID1-1000*INT(ID1/1000)
       IHADR=ID1/(10**INT(LOG10(DFLOAT(ID1))))
    ELSE
       IHADR=0
    ENDIF
    RETURN
  END FUNCTION IHADR

  FUNCTION ph4(px,py,pz)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::px,py,pz
    REAL(KIND(1d0))::ph4,S2,S
    INTEGER::init=0
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    SAVE init
    IF(px**2+py**2.LE.0d0)THEN
       IF(pz.GE.0d0)ph4=0d0
       IF(pz.LT.0)ph4=pi
       RETURN
    ENDIF
    S2=px**2/(px**2+py**2)
    S=DSQRT(S2)
    IF(S.GT.1d0)THEN
       WRITE(*,*)'PH4(X) WARNING S=',S
       ph4=0
       IF(px.LT.0d0)ph4=pi
       RETURN
    ENDIF
    IF(px.LT.0)S=-DSQRT(S2)
    ph4=DACOS(S)
    IF(py.LT.0)ph4=2*pi-ph4
    RETURN
  END FUNCTION ph4

  FUNCTION scalar_product(p1,p2)
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::p1,p2
    REAL(KIND(1d0))::scalar_product
    scalar_product=p1(4)*p2(4)-p1(3)*p2(3)-p1(2)*p2(2)-p1(1)*p2(1)
  END FUNCTION scalar_product

  FUNCTION transverse(p)
    REAL(KIND(1d0)),DIMENSION(4)::p
    REAL(KIND(1d0))::transverse
    transverse=SQRT(p(1)**2+p(2)**2)
  END FUNCTION transverse

  ! vector product
  FUNCTION vec_crossprod(vec1,vec2)
  ! vec1 crosstime vec2
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(3),INTENT(IN)::vec1,vec2
    REAL(KIND(1d0)),DIMENSION(3)::vec_crossprod
    vec_crossprod(1)=vec1(2)*vec2(3)-vec1(3)*vec2(2)
    vec_crossprod(2)=vec1(3)*vec2(1)-vec1(1)*vec2(3)
    vec_crossprod(3)=vec1(1)*vec2(2)-vec1(2)*vec2(1)
    RETURN
  END FUNCTION vec_crossprod

  FUNCTION cosij(a,b)
    REAL(KIND(1d0)),DIMENSION(3),INTENT(IN)::a,b
    REAL(KIND(1d0))::cosij,den
    cosij=a(1)*b(1)+a(2)*b(2)+a(3)*b(3)
    den=DSQRT(a(1)**2+a(2)**2+a(3)**2)*&
         DSQRT(b(1)**2+b(2)**2+b(3)**2)
    IF(den.EQ.0d0)THEN
       cosij=1d0
       RETURN
    ENDIF
    cosij=cosij/DSQRT(a(1)**2+a(2)**2+a(3)**2)
    cosij=cosij/DSQRT(b(1)**2+b(2)**2+b(3)**2)
  END FUNCTION cosij

  SUBROUTINE BOOSTL(Q,PBOO,P)
    ! momentums are in normal representation with fourth comp is the zero comp
    ! Boost P via PBOO(PBOO^2=Q^2) to PLB,and set to P
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PBOO
    REAL(KIND(1d0)),DIMENSION(4),INTENT(INOUT)::P
    REAL(KIND(1d0)),DIMENSION(4)::PCM,PLB
    REAL(KIND(1d0))::FACT
    INTEGER::J
    PCM(1:4)=P(1:4)
    PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)+PBOO(1)*PCM(1))/Q
    FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
    DO J=1,3
       PLB(J)=PCM(J)+FACT*PBOO(J)
    ENDDO
    P(1:4)=PLB(1:4)
  END SUBROUTINE BOOSTL

  SUBROUTINE psi_ll_kin(ppsi,mpsi,plp,plm)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::ppsi
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::plp,plm
    REAL(KIND(1d0)),INTENT(IN)::mpsi
    REAL(KIND(1d0)),PARAMETER::pipi=3.1415926535897932384626433832795d0
    REAL(KIND(1d0))::theta,phi,p0
    REAL(KIND(1d0)),DIMENSION(4)::plpres,plmres
    theta=rand()*2d0*pipi
    phi=rand()*2d0*pipi
    p0=mpsi/2d0
    plpres(3)=p0*DCOS(theta)
    plpres(4)=p0
    plpres(1)=p0*DSIN(theta)*DCOS(phi)
    plpres(2)=p0*DSIN(theta)*DSIN(phi)
    plmres(4)=p0
    plmres(1:3)=-plpres(1:3)
    call boost(mpsi,ppsi,plpres,plp)
    call boost(mpsi,ppsi,plmres,plm)
    RETURN
  END SUBROUTINE psi_ll_kin

  SUBROUTINE BOOST(Q,PBOO,PCM,PLB)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PBOO,PCM
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PLB
    REAL(KIND(1d0))::Fact
    INTEGER::j
    PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)&
         +PBOO(1)*PCM(1))/Q
    FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
    DO J=1,3
       PLB(J)=PCM(J)+FACT*PBOO(J)
    ENDDO
    RETURN
  END SUBROUTINE BOOST

END MODULE plot_py8_user
