#!/usr/bin/python

import os

eta_from_psi={'1s08':('3s18',1./3.),'1s01':('3s11',1./3.),'1p18':('3p08',3.0),'3s18':('1s08',1.0)}

psi_from_eta={'3s18':('1s08',3.0),'3s11':('1s01',3.0),'3p08':('1p18',1.0/3.0),'3p18':('1p18',1.0),'3p28':('1p18',5.0/3.0),'1s08':('3s18',1.0)}

h_from_chi0={'1s08':('3s18',3.0),'1p11':('3p01',3.0)}

h_from_chi1={'1s08':('3s18',1.0),'1p11':('3p11',1.0)}

h_from_chi2={'1s08':('3s18',3.0/5.0),'1p11':('3p21',3.0/5.0)}

chi0_from_h={'3s18':('1s08',1.0/3.0),'3p01':('1p11',1.0/3.0)}

chi1_from_h={'3s18':('1s08',1.0),'3p11':('1p11',1.0)}

chi2_from_h={'3s18':('1s08',5.0/3.0),'3p21':('1p11',5.0/3.0)}

def read_LDMEfile(config_path):
    LDMEs={}
    config_file=open(config_path,'r')
    for line in config_file:
        if '#' in line:
            line = line.split('#',1)[0]
        line = line.replace('\n','').replace('\r\n','')
        line = line.replace('LDME[','').replace(']','')
        try:
            name, value = line.split('=')
        except ValueError:
            pass
        else:
            name = name.strip().lower()
            value = value.strip().lower()
            value = value.replace('d','e')
            value = float(value)
            LDMEs[name]=value
    config_file.close()

    return LDMEs

def convert_LDME(in_path,out_path,rule):
    inLDMEs = read_LDMEfile(in_path)
    outLDMEs={}
    for fo in inLDMEs.keys():
        tag=fo[:3]
        break
    text=""
    for fo in rule.keys():
        info,fact=rule[fo]
        info=tag+'('+info+')'
        outLDMEs[tag+'('+fo+')']=inLDMEs[info]*fact
        text+="LDME[%s] = %17.15e\n"%(tag+'('+fo.upper()+')',inLDMEs[info]*fact)

    config_file=open(out_path,'w')
    config_file.write(text)
    config_file.close()


if __name__ == '__main__':
    rules=[eta_from_psi,h_from_chi0,h_from_chi1,h_from_chi2,psi_from_eta,chi0_from_h,chi1_from_h,chi2_from_h]
    hqpairs=[('etac1S','jpsi',1),('etac2S','psi2s',1),('hc1P','chic0',2),\
                 ('etab1S','Y1S',1),('etab2S','Y2S',1),('etab3S','Y3S',1),\
                 ('hb1P','chib01P',2),('hb2P','chib02P',2),('hb3P','chib03P',2)]
    # the rule to rename file
    def new_name(old_name,old_hadron,new_hadron):
        res=old_name
        if not ('arXiv' in old_name or 'hepph' in old_name):
            return res.replace(old_hadron,new_hadron)
        else:
            res=new_hadron+'_from_'+res
            return res
    
    for hqpair in hqpairs:
        new_hadron,old_hadron,irule=hqpair
        rule=rules[irule-1]
        # update the demo ones
        os.chdir(old_hadron)
        allfils=os.listdir(".")
        alldats=[f for f in allfils if f.endswith('.dat')]
        os.chdir("../")
        for dat in alldats:
            newdat=new_name(dat,old_hadron,new_hadron)
            convert_LDME(os.path.join('./',old_hadron,dat),\
                             os.path.join('./',new_hadron,newdat),
                         rule)

        # update the onia main directory file
        #for dat in ['_LDME.dat','_LDME_default.dat']:
        #    newdat=new_hadron+dat
        #    olddat=old_hadron+dat
        #    convert_LDME(os.path.join('../',olddat),\
        #                     os.path.join('../',newdat),\
        #                     rule)
