"""A class for dealing with topdrawer/gnuplot/root plot file"""

import sys
import re
import math
import os
import shutil
from types import FunctionType as function
import copy

#===============================================================================
# Plot
#===============================================================================
class Plot(object):
        """ A simple storage class to represent a plot """

        data = []
	dimension = 2

        def __init__(self, data=[],dimension=2):
            """ Initialize a plot with data which is just 
	    a list of tuples (x,y,dy) when dimension = 2 or
	    a list of tuples (x1,x2,y) when dimension = 3"""

            self.data = data
	    self.dimension = dimension

	def append(self,data):
            """append data in self.data"""
	    self.data.append(data)
	
	def extend(self,datas):
            self.data.extend(datas)

        def write_plot(self,outfile):
            """ Print ou a raw data file for gnuplot to import"""

            outFile = open(outfile,'w') 
            for aBin in self.data:
                outFile.write(' '.join('%.5e'%v for v in aBin)+'\n')
            outFile.close()
	    
	@classmethod
	def write_plots(cls,plots,outfile,gnuplots_hisband=True):
            """write out the plots"""
	    
	    if len(plots)==0:return
            if any(not isinstance(plot,Plot) for plot in plots):
                raise Exception, "All plots specified in the write_plots() function must be instances of Plot"

	    outFile = open(outfile,'w')
	    for i,aBin in enumerate(plots[0].data):
                newBin=[aBin[0],aBin[1]]
		for plot in plots[1:]:
                    newBin.append(plot.data[i][1])
                outFile.write(' '.join('%.5e'%v for v in newBin)+'\n')
	    outFile.close()

	    if len(plots)>0 and gnuplots_hisband and self.dimension == 2:
                Plot.write_gnuplots_hisband(plots,outfile="rebin_"+outfile)

	@classmethod
	def write_gnuplots_hisband(cls,plots,outfile):

            if len(plots)==0:return
	    if any(not isinstance(plot,Plot) for plot in plots):
		raise Exception, "All plots specified in the write_gnuplots_hisband() function must be instances of Plot"
	    outFile = open(outfile,'w')
	    if len(plots[0].data) <=1:return
	    binsize=plots[0].data[1][0]-plots[0].data[0][0]
	    for i,aBin in enumerate(plots[0].data):
                newBin1=[aBin[0]-binsize/2,aBin[1]]
		newBin2=[aBin[0]+binsize/2,aBin[1]]
		for plot in plots[1:]:
                    newBin1.append(plot.data[i][1])
		    newBin2.append(plot.data[i][1])
		outFile.write(' '.join('%.5e'%v for v in newBin1)+'\n')
		outFile.write(' '.join('%.5e'%v for v in newBin2)+'\n')
	    outFile.close

	@classmethod
	def integrate(cls,plot):
            """ integrate the bins via the lower bin """

	    if not isinstance(plot,Plot):
                raise Exception, "The plot specified in integrate is not a Plot"
	    if plot.dimension != 2:
		raise Exception, "integrate can only be applied to 2-dimension Plot"    

            new_plot=Plot(data=[])
	    for dat in plot.data:
                ysum,dysum=Plot.summation(plot,xrange=(dat[0],float("inf")))
		new_plot.data.append((dat[0],ysum,dysum))

	    return new_plot

        @classmethod
        def envelope(cls,plots):
            """ multiply two plot together or a plot and a float """
            
            if any(not isinstance(plot,Plot) for plot in plots):
                raise Exception, "All plots specified in the envelope() function must be instances of Plot"
	    if any(plot.dimension!=2 for plot in plots):
                raise Exception, "envelope can only be applied to 2-dimensional Plot"

            new_plot_up = Plot(data=[])
	    new_plot_down = Plot(data=[])

            if len(plots)==0:
                return new_plot_up,new_plot_down
	    
	    new_plot_up.data = copy.copy(plots[0].data)
	    new_plot_down.data = copy.copy(plots[0].data)
	    if len(plots)<2: 
                return new_plot_up,new_plot_down

            for plot in plots[1:]:
                i=0
		for x,y,dy in plot.data:
                    if not (x==new_plot_up.data[i][0] and x==new_plot_down.data[i][0]):
                        raise Exception,"The bins are not matched in envolope"
		    if new_plot_up.data[i][1]<y:
                        new_plot_up.data[i]=(new_plot_up.data[i][0],y,dy)
		    if new_plot_down.data[i][1]>y:
                        new_plot_down.data[i]=(new_plot_down.data[i][0],y,dy)
		    i+=1

	    return new_plot_up,new_plot_down

	@classmethod
	def estimate_error(cls,plot0,plots):
            """ estmate the error via quar. summation or Hessian method."""
	    if (not isinstance(plot0,Plot)) or any(not isinstance(plot,Plot) for plot in plots):
                raise Exception,"All plots specified in the estimate_error function must be instances of Plot"
	    if plot0.dimension!=2 or any(plot.dimension!=2 for plot in plots):
                raise Exception,"estimate_error can only be applied to 2-dimension Plot"

	    new_plot_up=Plot(data=[])
	    new_plot_down=Plot(data=[])
	    new_plot_up.data=copy.copy(plot0.data)
	    new_plot_down.data=copy.copy(plot0.data)
	    if len(plots)==0:
                return new_plot_up,new_plot_down
	    error_plus=[0.0 for dat in plot0.data]
	    error_minus=[0.0 for dat in plot0.data]
	    for plot in plots:
                i=0
                for x,y,dy in plot.data:
                    if not x==plot0.data[i][0]:
			raise Exception,"The bins are not matched in estimate_error"
		    if plot0.data[i][1]>y:
                        error_minus[i]=error_minus[i]+(y-plot0.data[i][1])**2
		    if plot0.data[i][1]<y:
                        error_plus[i]=error_plus[i]+(y-plot0.data[i][1])**2
		    i+=1
	    for i,dat in enumerate(new_plot_up.data):
                new_plot_up.data[i]=(dat[0],dat[1]+math.sqrt(error_plus[i]),dat[2])
	    for i,dat in enumerate(new_plot_down.data):
                new_plot_down.data[i]=(dat[0],dat[1]-math.sqrt(error_minus[i]),dat[2])

	    return new_plot_up,new_plot_down

        @classmethod
        def Hessian_error(cls,plot0,plots):
            """estimate the error via Hessian method"""
	    return estimate_error(cls,plot0,plots)

	@classmethod
	def PDF4LHC_error(cls,plot0,plots):
            """ estmate the error via  PDF4LHC15 Hessian Hessian method."""
	    if (not isinstance(plot0,Plot)) or any(not isinstance(plot,Plot) for plot in plots):
                raise Exception,"All plots specified in the PDF4LHC_error function must be instances of Plot"
            new_plot_up=Plot(data=[])
            new_plot_down=Plot(data=[])
            new_plot_up.data=copy.copy(plot0.data)
            new_plot_down.data=copy.copy(plot0.data)
            if len(plots)==0:
                return new_plot_up,new_plot_down
            error_std=[0.0 for dat in plot0.data]
	    for plot in plots:
                i=0
                for x,y,dy in plot.data:
                    if not x==plot0.data[i][0]:
                        raise Exception,"The bins are not matched in PDF4LHC_error"
                    error_std[i]=error_std[i]+(y-plot0.data[i][1])**2
                    i+=1
	    
	    for i,dat in enumerate(new_plot_up.data):
                new_plot_up.data[i]=(dat[0],dat[1]+math.sqrt(error_std[i]),dat[2])
            for i,dat in enumerate(new_plot_down.data):
                new_plot_down.data[i]=(dat[0],dat[1]-math.sqrt(error_std[i]),dat[2])

            return new_plot_up,new_plot_down

	@classmethod
	def Gaussian_error(cls,plot0,plots):
            """estimate the error via Gaussian method"""
	    if (not isinstance(plot0,Plot)) or any(not isinstance(plot,Plot) for plot in plots):
                raise Exception,"All plots specified in the Gaussian_error function must be instances of Plot"
	    if plot0.dimension!=2 or any(plot.dimension!=2 for plot in plots):
                raise Exception,"Gaussian_error can only be applied to 2-dimensional Plot"

            new_plot_up=Plot(data=[])
            new_plot_down=Plot(data=[])
            new_plot_up.data=copy.copy(plot0.data)
            new_plot_down.data=copy.copy(plot0.data)
            if len(plots)==0:
                return new_plot_up,new_plot_down
	    error_std=[0.0 for dat in plot0.data]
	    numofpdf=len(plots)
            for plot in plots:
                i=0
                for x,y,dy in plot.data:
                    if not x==plot0.data[i][0]:
                        raise Exception,"The bins are not matched in Gaussian_error"
		    error_std[i]=error_std[i]+(y-plot0.data[i][1])**2/numofpdf
                    i+=1

            for i,dat in enumerate(new_plot_up.data):
                new_plot_up.data[i]=(dat[0],dat[1]+math.sqrt(error_std[i]),dat[2])
            for i,dat in enumerate(new_plot_down.data):
                new_plot_down.data[i]=(dat[0],dat[1]-math.sqrt(error_std[i]),dat[2])

            return new_plot_up,new_plot_down

        @classmethod
        def combine(cls,plot1,plot2,operator,error_operator, error_float_operator=None):
            """ multiply two plot together or a plot and a float """

            if not isinstance(operator,function):
                raise Exception, "The operator in argument of function Combine must be a function"
            if not isinstance(error_operator,function):
                raise Exception, "The error_operator in argument of function Combine must be a function"

            if error_float_operator is None:
                error_float_operator = error_operator

            plots = [p for p in [plot1, plot2] if isinstance(p, Plot)]
            floats = [p for p in [plot1, plot2] if isinstance(p, float)]
            if (len(plots),len(floats)) not in [(1,1),(2,0)]:
                raise Exception, "One can multiply only either two plots together or a plot and a float"
	    if (len(plots),len(floats)) == (1,1) and plot1.dimension != plot2.dimension:
                raise Exception, "The dimensions of the two plots are not the same"
                

            new_plot = Plot(data=[],dimension=plots[0].dimension)
            for i, dat in enumerate(plots[0].data):
                if len(plots)>1:
                    if dat[0]!=plots[1].data[i][0]:
                        raise Exception, 'The bins of the two plots which must be multiplied do not match.'
		    if plots[0].dimension == 2:
                        new_plot.data.append((
					    dat[0],
					    operator(dat[1],plots[1].data[i][1]),
					    error_operator(dat[2],plots[1].data[i][2])
					    ))
		    else:
                        if dat[1]!=plots[1].data[i][1]:
                            raise Exception, 'The bins of the two plots which must be match #2'
			new_plot.data.append((
					    dat[0],dat[1],
					    operator(dat[2],plots[1].data[i][2])
					    ))
                else:
                    if plots[0].dimension == 2:
                        new_plot.data.append((
					dat[0],
					operator(dat[1],floats[0]),
					error_float_operator(dat[2],floats[0])
					))
		    else:
                        new_plot.data.append((
					dat[0],dat[1],
					operator(dat[2],floats[0])
					))

            return new_plot

        @classmethod
	def absolute(cls,plot):
            """ Take the absolute of the x value"""
	    if not isinstance(plot,Plot):
                raise Exception, "plot is not a Plot"
	    if plot.dimension!=2:
                raise Exception, "absolute can only be applied to 2-dimensional plot"
	    data=[]
	    if len(plot.data)<=1:return
	    binsize=abs(plot.data[1][0]-plot.data[0][0])
	    for dat in plot.data:
                find=False
                for i,dat0 in enumerate(data):
                    if abs(dat0[0]-abs(dat[0]))/binsize < 1e-2:
                       data[i]=(dat0[0],dat0[1]+dat[1],math.sqrt(dat0[2]**2+dat[2]**2))
		       find=True
		       break
		    if abs(dat[0]) < dat0[0]:
                       data.insert(i,(abs(dat[0]),dat[1],dat[2]))
                       find=True
                       break
		if not find:
                    data.append((abs(dat[0]),dat[1],dat[2]))
	    plot.data=data
	    return plot

        @classmethod
        def multiply(cls,plot1,plot2):
            """ A wrapper of the function combine to implement multiplication """

            return cls.combine(plot1,plot2, 
                               operator=(lambda a, b: a*b),
                               error_operator = (lambda a, b: math.sqrt(a**2+b**2)),
                               error_float_operator = (lambda a, b: a*b))

        @classmethod
        def divide(cls,plot1,plot2):
            """ A wrapper of the function combine to implement multiplication """

            return cls.combine(plot1,plot2, 
                               operator=(lambda a, b: (a/b if b!=0.0 else 0.0) ),
                               error_operator = (lambda a, b: math.sqrt(a**2+b**2)),
                               error_float_operator = (lambda a, b: (a/b if b!=0.0 else 0.0)) )

        @classmethod
        def add(cls,plot1,plot2):
            """ A wrapper of the function combine to implement multiplication """

            return cls.combine(plot1,plot2, 
                               operator=(lambda a, b: a+b),
                               error_operator = (lambda a, b: math.sqrt(a**2+b**2)),
                               error_float_operator = (lambda a, b: a))

        @classmethod
        def subtract(cls,plot1,plot2):
            """ A wrapper of the function combine to implement multiplication """

            return cls.combine(plot1,plot2, 
                               operator=(lambda a, b: a-b),
                               error_operator = (lambda a, b: math.sqrt(a**2+b**2)),
                               error_float_operator = (lambda a, b: a))


	@classmethod
	def summation(cls,plot,xrange=None):
            """ sum over all the weights """
	    if not isinstance(plot,Plot):
                raise Exception, "The argument plot is not a Plot."
	    if xrange and not isinstance(xrange,tuple):
                raise Exception, "The xrange is not a tuple or None."
	    if plot.dimension!=2:
                raise Exception, "summation can only be applied to 2-dimesional plot"
	    if not xrange:
                ysum=math.fsum([wgt[1] for wgt in plot.data])
		dysum=math.sqrt(math.fsum([wgt[2] for wgt in plot.data]))
	    else:
                ysum=math.fsum([wgt[1] for wgt in plot.data \
					if wgt[0]>=xrange[0] and wgt[0]<=xrange[1]])
		dysum=math.sqrt(math.fsum([wgt[2] for wgt in plot.data \
					if wgt[0]>=xrange[0] and wgt[0]<=xrange[1]]))
	    return ysum,dysum

#===============================================================================
# PlotList
#===============================================================================
class PlotList(list):
    """A class to store lists of Plot"""
    
    class PlotListError(Exception):
        """Exception raised if an error occurs in the definition
	or execution of a plot list"""
	pass
    
    def __init__(self, init_list=None):
        """Creates a new plot list."""

	list.__init__(self)
	
	if init_list is not None:
            for plot in init_list:
                self.append(plot)

    def append(self, plot):
        """Appends an element, but test if valid before"""

	assert self.is_valid_element(plot),\
	    "Plot is not a vlid object for the current list"

	list.append(self, plot)

    def is_valid_element(self, plot):
        """Test if plot is a valid element for the list"""
	return isinstance(plot,Plot)

    
#===============================================================================
# TopdrawerPlotsReader
#===============================================================================
class TopdrawerPlotsReader(dict):
    """TopdrawerPlotsReader: Contains several routines for reading topdrawer files from HELAC-Onia"""

    plot_text = ''
    plots = {}
    dim_type_list = []

    class TopdrawerPlotsReaderError(Exception):
        pass

    def __init__(self,plot_path=None):
	self.plot_path=plot_path
	self.plot_text=''
	self.plots={}
	self.dim_type_list=[]
	self.read_td_plots()

    def read_td_plots(self):
        """read the topdrawer file from the plot_path it is sepcified"""
	if not self.plot_path:
           return
	if not os.path.isfile(self.plot_path):
           raise TopdrawerPlotsReaderError,"the plot_path is not a valid path"
	td_file = open(self.plot_path)
	store_res = 0
	i = 0
	for line in td_file:
           if self.plot_start(line):
              store_res = 1
              i = i + 1
	      self.dim_type_list.append(self.plot_dimtype(line))
	   elif self.plot_end(line):
              store_res = 0

	   if store_res == 0:
              if '\n' in line:
                  self.plot_text +='%s'%line
	      else:
                  self.plot_text +='%s\n'%line
	   elif self.plot_start(line):
              rep_str = 'plot_%i'%i
	      if '\n' in line:
                  self.plot_text +='%s'%line
	      else:
		  self.plot_text +='%s\n'%line
	      self.plot_text +='%('+rep_str+')s'
	   else:
              # read the data in the plot
              rep_str = 'plot_%i'%i
	      if not rep_str in self.plots:
                 self.plots[rep_str]=Plot(data=[],dimension=self.dim_type_list[i-1])
	      data = line.split()
	      data = [float(val) for val in data]
	      if len(data)>2:
                 self.plots[rep_str].append(data)
	      else:
                 self.plots[rep_str].append(None)

	td_file.close()

    def plot_dimtype(self,line):
        # only 2-dim for topdrawer
        return 2

    def plot_start(self,line):
        if 'SET ORDER X Y DY' in line:
            return True
	else:
            return False

    def plot_end(self,line):
        if 'HISTO' in line:
            return True
	else:
            return False

    def plot_print(self,value_list):
        """output topdrawer file"""
	text=""
	for data in value_list:
            text += "       %5.4e       %5.4e       %5.4e\n"%tuple(data)

	return text

    def write_data(self):
        """write the data into plot_text"""

	store_plots_str={}
	for key,value in self.plots.items():
            store_plots_str[key]=self.plot_print(value.data)

	self.plot_text=self.plot_text%store_plots_str

	return self.plot_text

class GnuplotPlotsReader(dict):
    """GnuplotPlotsReader: Contains several routines for reading gnuplot files from HELAC-Onia"""

    plot_text = ''
    plots = {}
    dim_type_list = []

    class GnuplotPlotsReaderError(Exception):
        pass

    def __init__(self,plot_path=None):
        self.plot_path=plot_path
	self.plot_text=''
	self.plots={}
	self.dim_type_list=[]
	self.read_gnu_plots()

    def read_gnu_plots(self):
        """read the topdrawer file from the plot_path it is sepcified"""
	if not self.plot_path:
           return
	if not os.path.isfile(self.plot_path):
           raise GnuplotPlotsReaderError,"the plot_path is not a valid path"
	gnu_file = open(self.plot_path)
	store_res = 0
	i = 0
	for line in gnu_file:
           if self.plot_start(line):
              store_res = 1
	      i = i + 1
	      self.dim_type_list.append(self.plot_dimtype(line))
	   elif self.plot_end(line):
              store_res = 0

	   if store_res == 0:
              if '\n' in line:
                  self.plot_text +='%s'%line
	      else:
                  self.plot_text +='%s\n'%line
	   elif self.plot_start(line):
              rep_str = 'plot_%i'%i
	      if '\n' in line:
                  self.plot_text +='%s'%line
	      else:
                  self.plot_text +='%s\n'%line
	      self.plot_text +='%('+rep_str+')s'
	   else:
              # read the data in the plot
              rep_str = 'plot_%i'%i
	      if not rep_str in self.plots:
                 self.plots[rep_str]=Plot(data=[],dimension=self.dim_type_list[i-1])
	      data = line.split()
	      data = [float(val) for val in data]
	      if len(data)>2:
                 self.plots[rep_str].append(data)
	      else:
                 self.plots[rep_str].append(None)

	gnu_file.close()
	
	self.plot_text=self.plot_text.replace('set format y "10^{%T}"','set format y "10^{%%T}"')

    def plot_dimtype(self,line):
        if 'splot "-" with pm3d' in line:
            return 3
	else:
            return 2

    def plot_start(self,line):
        if 'plot "-" with histeps' in line or 'splot "-" with pm3d' in line:
            return True
	else:
            return False

    def plot_end(self,line):
        newline=line.split()
	if len(newline) == 1 and newline[0] == 'e':
            return True
	else:
            return False

    def plot_print(self,value_list):
        """output gnuplot file"""
	text=""
	for data in value_list:
            if data==None:
                text +="     \n"
            else:
                text +="     %7.6e       %7.6e    %7.6e\n"%tuple(data)

	return text

    def write_data(self):
        """write the data into plot_text"""
	
	store_plots_str={}
        for key,value in self.plots.items():
            store_plots_str[key]=self.plot_print(value.data)

	self.plot_text=self.plot_text%store_plots_str

	return self.plot_text

#===============================================================================
# RootPlotsReader
#===============================================================================
class RootPlotsReader(dict):
    """RootPlotsReader: Contains several routines for reading topdrawer files from HELAC-Onia"""

    plot_text = ''
    plots = {}
    dim_type_list = []
    store_index = {}
    
    class RootPlotsReaderError(Exception):
	pass

    def __init__(self,plot_path=None):
        self.plot_path=plot_path
	self.plot_text=''
	self.plots={}
	self.dim_type_list=[]
	self.store_index={}
	self.read_root_plots()

    def read_root_plots(self):
        """read the root file from the plot_path it is specified"""
	if not self.plot_path:
           return
	if not os.path.isfile(self.plot_path):
           raise RootPlotsReaderError,"the plot_path is not a valid path"
	root_file = open(self.plot_path)
	store_res = 0
	i = 0
	updata = []
	dim_type=2
	for line in root_file:
           if self.plot_start(line):
               store_res = 1
               i = i + 1
	   elif self.plot_end(line):
               store_res = 0
               # following are stable for empty plot
               if 'hist_3d -> SetOption("colz");' in line:
                   dim_type=3
	       else:
                   dim_type=2
               if len(self.dim_type_list)<i:
                   self.dim_type_list.append(dim_type)

	   if store_res == 0:
               if '\n' in line:
                   self.plot_text +='%s'%line
               else:
                   self.plot_text +='%s\n'%line
           elif self.plot_start(line):
               rep_str = 'plot_%i'%i
	       if '\n' in line:
                   self.plot_text += '%s'%line
	       else:
                   self.plot_text += '%s\n'%line
	       self.plot_text +='%('+rep_str+')s'
	       self.store_index[rep_str]=i
	   else:
               rep_str = 'plot_%i'%i
	       if 'xybin' in line:
                   dim_type=3
	       else:
                   dim_type=2
	       if len(self.dim_type_list)<i:
                   self.dim_type_list.append(dim_type)
	       if not rep_str in self.plots:
                   self.plots[rep_str]=Plot(data=[],dimension=dim_type)
               data = line.split()
	       data.pop()
	       if 'FindBin' in line:
                   if dim_type==2:
                       updata.append(float(data[-1]))
		   else:
                       updata.append([float(data[-3]),float(data[-1])])
	       if 'SetBinContent' in line:
                   updata.append(float(data[-1]))
               if 'SetBinError' in line:
                   updata.append(float(data[-1]))
                   self.plots[rep_str].append(updata)
		   updata=[]

	root_file.close()

    def plot_start(self,line):
        if 'SetStats(false)' in line:
            return True
	else:
            return False

    def plot_end(self,line):
        if 'histos -> Add(hist);' in line or 'hist_3d -> SetOption("colz");' in line:
            return True
	else:
            return False

    def plot_print(self,value_list,i=1):
        """output root file"""
	text=""
	for data in value_list:
            text +='   int xbin = id%i->FindBin(   %18.17e      );\n'%(i,data[0])
            text +='  id%i -> SetBinContent( xbin,   %18.17e );\n'%(i,data[1])
            text +='  id%i -> SetBinError( xbin,    %18.17e      );\n'%(i,data[2])

	return text

    def plot_print_3d(self,value_list,i=1):
        """output root file"""
	text=""
	for data in value_list:
            text +='   int xybin = id3d%i->FindBin(   %17.16e      ,   %17.16e      );\n'%(i,data[0][0],data[0][1])
	    text +='  id3d%i -> SetBinContent( xybin,   %17.16e );\n'%(i,data[1])
	    text +='  id3d%i -> SetBinError( xybin,    %17.16e      );\n'%(i,data[2])

	return text

    def write_data(self):
        """write the data into plot_text"""
	
	store_plots_str={}
	for key,value in self.plots.items():
            if self.dim_type_list[self.store_index[key]-1]==2:
                store_plots_str[key]=self.plot_print(value.data,i=self.store_index[key])
	    else:
                store_plots_str[key]=self.plot_print_3d(value.data,i=self.store_index[key])

	self.plot_text=self.plot_text%store_plots_str

	return self.plot_text

#===============================================================================
# HwUPlotsReader
#===============================================================================
class HwUPlotsReader(dict):
    """HwUPlotsReader: Contains several routines for reading HwU files from HELAC-Onia"""

    plot_text = ''
    plots = {}
    dim_type_list = []

    class HwUPlotsReaderError(Exception):
        pass

    def __init__(self,plot_path=None):
        self.plot_path=plot_path
        self.plot_text=''
        self.plots={}
        self.dim_type_list=[]
        self.read_hwu_plots()

    def read_hwu_plots(self):
        """read the HwU file from the plot_path it is sepcified"""
        if not self.plot_path:
           return
        if not os.path.isfile(self.plot_path):
           raise HwUPlotsReaderError,"the plot_path is not a valid path"
        hwu_file = open(self.plot_path)
        store_res = 0
        i = 0
        for line in hwu_file:
           if self.plot_start(line):
              store_res = 1
              i = i + 1
              self.dim_type_list.append(self.plot_dimtype(line))
           elif self.plot_end(line):
              store_res = 0

           if store_res == 0:
              if '\n' in line:
                  self.plot_text +='%s'%line
              else:
                  self.plot_text +='%s\n'%line
           elif self.plot_start(line):
              rep_str = 'plot_%i'%i
              if '\n' in line:
                  self.plot_text +='%s'%line
              else:
                  self.plot_text +='%s\n'%line
              self.plot_text +='%('+rep_str+')s'
           else:
              # read the data in the plot 
              rep_str = 'plot_%i'%i
              if not rep_str in self.plots:
                 self.plots[rep_str]=Plot(data=[],dimension=self.dim_type_list[i-1])
              data = line.split()
              data = [float(val) for val in data]
              if len(data)>2:
                 self.plots[rep_str].append(data)
              else:
                 self.plots[rep_str].append(None)

        hwu_file.close()

    def plot_dimtype(self,line):
        # only 2-dim for HwU
        return 2

    def plot_start(self,line):
        if line.startswith('<histogram>'):
            return True
        else:
            return False

    def plot_end(self,line):
        if line.startswith('<\histogram>'):
            return True
        else:
            return False

    def plot_print(self,value_list):
        """output HwU file"""
        text=""
        for data in value_list:
            for val in data:
                text += "   %14.7e"%val
	    text +="\n"

        return text

    def write_data(self):
        """write the data into plot_text"""

        store_plots_str={}
        for key,value in self.plots.items():
            store_plots_str[key]=self.plot_print(value.data)

        self.plot_text=self.plot_text%store_plots_str

        return self.plot_text

#===============================================================================
# PlotTopExtractor
#===============================================================================
class PlotTopExtractor(dict):
    """PlotTopExtractor: Contains several routines for dealing with topdrawer files."""

    top_titles = []
    top_xranges = []

    class PlotTopExtractorError(Exception):
        pass

    def __init__(self):
        pass

    def toFloat(self,strNumber):
        return float(strNumber.replace('d','e').replace('D','e'))

    def single_plottop_exctractor(self, topfile, plottitle, outfile = None, plot_name = None, matchindex = 1):
        """Read the data with the plot title in the topfile. 
        The data is outputed into outfile"""

        assert isinstance(topfile,str)
        if not outfile is None:assert isinstance(outfile,str)
        assert isinstance(plottitle,str)
        # copy from Valentin's SinglePlotTopExctractor.py
        try:
            try:
                sourceTop=open(topfile,'r')
            except:
                print "The target file %s could not be found/opened"%topfile
            titleToPick=plottitle
            titleToPick = titleToPick.replace('(','\(').replace(')','\)')
            titleToPick = titleToPick.replace('[','\[').replace(']','\]')
            titleToPick = titleToPick.replace('{','\{').replace('}','\}')
            titleToPick = titleToPick.replace('*','\*').replace('^','\^')
            titleToPick = titleToPick.replace('.','\.').replace('$','\$')
            titleToPick = titleToPick.replace('+','\+').replace('?','\?')
        except:
            raise self.PlotTopExtractorError, "The arguments are: self.single_plottop_exctractor(topfile, plottitle, outfile)"

        titleFinder=re.compile("^\s*TITLE\s*TOP\s*\".*%s"%titleToPick)

        entry = "-?[0-9]*.[0-9]*[DEde][+-][0-9]*"
        ReadData=re.compile(r"^\s*(%(numb)s)\s*(%(numb)s)\s*(%(numb)s).*"%{'numb':entry})
        entry2 = "-?[0-9]*.[0-9]*"
        ReadXRange=re.compile(r"^\s*SET\s*LIMITS\s*X\s*(%(numb)s)\s*(%(numb)s).*"%{'numb':entry2})
        SourceTopReader = (line for line in sourceTop.xreadlines())

        # Find the correct plot
        matchtime = 0
        try:
            title = titleFinder.match(SourceTopReader.next())
            while title is None:
                title = titleFinder.match(SourceTopReader.next())
                if title:
                    matchtime = matchtime + 1
                    if matchtime < matchindex:title = None
        except StopIteration:
            raise self.PlotTopExtractorError, \
                "Error, could not find the plot with title starting with %s"%titleToPick
        # Find the x range
        xrange = ReadXRange.match(SourceTopReader.next())
        while xrange is None:
            xrange = ReadXRange.match(SourceTopReader.next())

        self.top_xranges.append((self.toFloat(xrange.group(1)),\
                                    self.toFloat(xrange.group(2))))
        # Find the beginning of data in the plot
        dat = ReadData.match(SourceTopReader.next())
        while dat is None:
            dat = ReadData.match(SourceTopReader.next())

        # The data structure Histogram below is a list of tuple (x,y,dy)
        myHistogram = Plot(data=[])

        # Now store the data in our abstract histo
        while not dat is None:
            myHistogram.data.append((
                    self.toFloat(dat.group(1)),
                    self.toFloat(dat.group(2)),
                    self.toFloat(dat.group(3))))
            dat = ReadData.match(SourceTopReader.next())

        # Close the source top
        sourceTop.close()

	# uncomment when using absolute x
	#Plot.absolute(myHistogram)
        
        if plot_name is None:
            plot_name = plottitle

        self[plot_name] = myHistogram
        
        if not outfile is None:
            myHistogram.write_plot(outfile)

    def toptitles_exctractor(self,topfile):
        """Extract all the titles of the plots in the topfile."""
        titleFinder=re.compile("^\s*TITLE\s*TOP\s*\"(.*)\"")
        try:
            sourceTop=open(topfile,'r')
        except:
            print "The target file %s could not be found/opened"%topfile
        SourceTopReader = (line for line in sourceTop.xreadlines())
        try:
            while True:
                title=titleFinder.match(SourceTopReader.next())
                if not title is None:self.top_titles.append(title.group(1))
        except StopIteration:
            pass

        sourceTop.close()

        return self.top_titles

    def plotstop_exctractor(self,topfile):
        """Extracte useful informations for all the plots in the topdrawer file"""
        assert isinstance(topfile,str)
        self.toptitles_exctractor(topfile)
        for title in self.top_titles:
            self.single_plottop_exctractor(topfile,title,outfile=None)

    def clear(self):
        #self.histograms={}
        self.top_titles=[]
        self.top_xranges=[]
