MODULE plot_user
!============================================================
!
! PLOT FILE FOR gamma gamma -> W+ W- (UPC)
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=24
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    character*5 cc(2)
    data cc/'     ',' CMS '/
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(emep_ISR_shower.EQ.1)num_plots=num_plots+2
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    DO i=1,2
       l=(i-1)*12
       CALL histogram_book(l+ 1,'total rate    '//cc(i),&
            2,0.5d0,2.5d0,.FALSE.)
       CALL histogram_book(l+ 2,'eta(W+)       '//cc(i),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 3,'eta(W-)       '//cc(i),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 4,'y(W+)         '//cc(i),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 5,'y(W-)         '//cc(i),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 6,'pT(W+)        '//cc(i),&
            100, 0d0,3000d0,.TRUE.)
       CALL histogram_book(l+ 7,'pT(W-)        '//cc(i),&
            100, 0d0,3000d0,.TRUE.)
       CALL histogram_book(l+ 8,'|y(W+W-)|     '//cc(i),&
            100,0d0,5d0,.TRUE.)
       CALL histogram_book(l+ 9,'m(W+W-)        '//cc(i),&
            100,0d0,5000d0,.TRUE.)
       CALL histogram_book(l+10,'m(W+W-) zoomin '//cc(i),&
            100,1000d0,2000d0,.TRUE.)
       CALL histogram_book(l+11,'xmax(gamma)    '//cc(i),&
            100,0d0,1d0,.TRUE.)
       CALL histogram_book(l+12,'xmin(gamma)    '//cc(i),&
            100,0d0,1d0,.TRUE.)
    ENDDO

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::pWp, pWm, pWpWm, pWp_cms, pWm_cms, pboo
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,etaWp,etaWm,yWp,yWm,ptWp,ptWm
    REAL(KIND(1d0))::yWpWm,mWpWm,ptWpWm,xmax,xmin
    REAL(KIND(1d0))::exp1,exp2,q
    INTEGER::i,kk,l,nWp,nWm
    REAL(KIND(1d0))::www,yftot
    LOGICAL,DIMENSION(2)::cuts
    IF(emep_ISR_shower.EQ.1)THEN
       CALL plot_fill_QEDPS(wgts)
       RETURN
    ENDIF
    IF(wgts(1).EQ.0d0)RETURN
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    ! 
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    IF(Momenta_Frame.EQ.2)THEN
       CALL cmstolab()
    ELSEIF(Momenta_Frame.EQ.3.AND..NOT.labeqcoll)THEN
       CALL collisiontolab()
    ENDIF
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    kk=1
    nWp=0
    nWm=0
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          kk=kk+1
       ENDIF
       IF(iflh(i).EQ.34)THEN
          pWm(1:4)=phad(i,1:4)
          nWm=nWm+1
       ELSEIF(iflh(i).EQ.33)THEN
          pWp(1:4)=phad(i,1:4)
          nWp=nWp+1
       ENDIF
    ENDDO
    IF(nWm.NE.1.OR.nWp.NE.1)THEN
       WRITE(*,*)"Error: cannot find W+W-"
       STOP
    ENDIF
    DO i=1,4
       pWpWm(i)=pWp(i)+pWm(i)
       pWp_cms(i)=pWp(i)
       pWm_cms(i)=pWm(i)
    ENDDO
    exp1=ebeam(1)*xp1
    exp2=ebeam(2)*xp2
    q=DSQRT(exp1*exp2)*2d0
    pboo(4)=(exp1+exp2)   ! the zero component
    pboo(3)=-(exp1-exp2)   ! the z component
    pboo(1:2)=0d0
    ! boost to the center-of-mass frame of gamma-gamma
    CALL BOOSTL(q,pboo,pWp_cms)
    CALL BOOSTL(q,pboo,pWm_cms)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    etaWp=prapidity(pWp)
    etaWm=prapidity(pWm)
    yWp=rapidity(pWp)
    yWm=rapidity(pWm)
    ptWp=transverse(pWp)
    ptWm=transverse(pWm)
    ptWpWm=transverse(pWpWm)
    yWpWm=rapidity(pWpWm)
    yWpWm=DABS(yWpWm) ! I just take the absolute value
    mWpWm=DSQRT(pWpWm(4)**2-pWpWm(1)**2-pWpWm(2)**2-pWpWm(3)**2)
    xmax=MAX(xp1,xp2)
    xmin=MIN(xp1,xp2)
    cuts(1)=.TRUE.
    ! CMS cuts https://inspirehep.net/files/1f048f88bb52286e8578932b3d73c356
    cuts(2)=mWpWm.GT.1000d0.AND.xmax.GT.0.04d0.AND.xmax.LT.0.2d0&
         .AND.xmin.GT.0.04d0.AND.xmin.LT.0.2d0
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=1,2
       IF(.NOT.cuts(i))CYCLE
       l=(i-1)*12
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,etaWp,wgts)
       CALL HISTOGRAM_fill(l+3,etaWm,wgts)
       CALL HISTOGRAM_fill(l+4,yWp,wgts)
       CALL HISTOGRAM_fill(l+5,yWm,wgts)
       CALL HISTOGRAM_fill(l+6,ptWp,wgts)
       CALL HISTOGRAM_fill(l+7,ptWm,wgts)
       CALL HISTOGRAM_fill(l+8,DABS(yWpWm),wgts)
       CALL HISTOGRAM_fill(l+9,mWpWm,wgts)
       CALL HISTOGRAM_fill(l+10,mWpWm,wgts)
       CALL HISTOGRAM_fill(l+11,xmax,wgts)
       CALL HISTOGRAM_fill(l+12,xmin,wgts)
    ENDDO
    ! boost back to c.m. frame
    IF(Momenta_Frame.EQ.2)THEN
       CALL labtocms()
    ELSEIF(Momenta_Frame.EQ.3.AND..NOT.labeqcoll)THEN
       CALL labtocollision()
    ENDIF
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax                                                  
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE plot_fill_Decay(wgts)
    USE Decay_interface
    USE QEDPS_interface
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "../../decay/stdhep.inc"
    REAL(KIND(1d0))::var,costhl,Q
    INTEGER::i,m
    REAL(KIND(1d0)),DIMENSION(1:4)::Pl,PV,PBOO
    RETURN
  END SUBROUTINE plot_fill_Decay

  SUBROUTINE plot_fill_QEDPS(wgts)
    USE QEDPS_interface
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    ! QEDPS parameters                                                      
    REAL(KIND(1d0)),DIMENSION(10,1000)::PLPTN
    INTEGER,DIMENSION(10,1000)::NLPTN
    INTEGER::NPTCL
    COMMON/QPLIST/PLPTN,NLPTN,NPTCL
    REAL(KIND(1d0)),DIMENSION(4)::pini
    REAL(KIND(1d0))::wgt,var,frache
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l,nphoton
    REAL(KIND(1d0))::www,yftot
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    pini(1:4)=PLPTN(4:7,1)+PLPTN(4:7,2)
    yftot=rapidity(pini)
    nphoton=0
    DO i=1,NPTCL
       IF(NLPTN(1,i).EQ.22)THEN
          nphoton=nphoton+1
       ENDIF
    ENDDO
    frache=MIN(Q2OUT_QEDPS/Q2MAX_QEDPS,1d0)
    frache=DSQRT(frache)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,yftot,wgts)
    CALL HISTOGRAM_fill(l+3,DBLE(nphoton),wgts)
    CALL HISTOGRAM_fill(l+4,frache,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_QEDPS
END MODULE plot_user
