MODULE plot_user
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=5 ! num_plots=inum_plots*4
  INTEGER::inum_plots=5
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
!     open(200, file = "/home/charlie/check.dat", status = "unknown")
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    l=0
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    CALL histogram_book(l+ 1,'J/{/Symbol y} p_T',&
         58,1d0,30d0,.TRUE.)   
    CALL histogram_book(l+ 2,'z',&
         17,0.05d0,0.9d0,.TRUE.)
    CALL histogram_book(l+ 3,'W',&
         90,60.d0,240.0d0,.TRUE.)
    CALL histogram_book(l+ 4,'y(b)',&
         50,-2.5d0,2.5d0,.TRUE.)
    CALL histogram_book(l+ 5,'y(bbar)',&
         50,-2.5d0,2.5d0,.TRUE.)
    !WRITE(*,*) "CHECK:", i, l, inum_plots
    !     ENDDO
    !     close(200)
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=30)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin'
!     ytit='d{/Symbol s}/dp_T per bin  '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::xponium,xpz,xponiumz
    REAL(KIND(1d0)),DIMENSION(4)::xplpz,xplmz,xplponium,xplmonium
    INTEGER::numonium,numz
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e,ptpsilp,ptpsilm,etapsilp,etapsilm,sgammap,shat
    REAL(KIND(1d0))::ptzlp,ptzlm,etazlp,etazlm
    REAL(KIND(1d0))::exp1,exp2,mb,ptb,ptb2,ptbbar,yz,ptz,mpsiz,dphipsiz,mtb,ptpsi2,yb,ybbar,yh,avptb
    REAL(KIND(1d0))::W,z,Q2     !W = SQRT((p_gamma + p_p)^2) = SQRT(s_gamma-p), z=(p_psi,p_p)/(p_gamma,p_p)
    LOGICAL::zeuscuts, zeuscutsz, H1cuts, H1cutspsi, H1Wcuts, Pierrecuts
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www,yftot
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
    IF(wgts(1).EQ.0d0)RETURN
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    !
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    xponium(1:4)=0d0
    xpz(1:4)=0d0
    numonium=0
    numz=0
    kk=1
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          xponium(1:4)=phad(i,1:4)
          numonium=numonium+1
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          IF(iflh(i).EQ.32)THEN
             xpz(1:4)=phad(i,1:4)
             numz=numz+1
          ENDIF
          kk=kk+1
       ENDIF
    ENDDO
!     IF(numonium.NE.1)THEN
!        WRITE(*,*)"ERROR:the number of onium is not equal to 1 !"
!        STOP
!     ENDIF
    xponiumz(1:4)=xponium(1:4)+xpz(1:4)
!     write(*,*) 'check: ', xponium(1), xponium(2), xponium(3), xponium(4), xp1, xp2, W, Q2
    ! trivial decay (without spin correlation)
    mb=DSQRT(DABS(phad(3,4)**2-phad(3,1)**2-phad(3,2)**2-phad(3,3)**2))
!     WRITE(*,*) "mb:", mb
!     DO i=1,4
!         write(*,*) xponium(i)
!     END DO
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    q=ehat
    e=q/SQRT(xp1*xp2)
!     exp1=xp1*ebeam(1)
!     exp2=xp2*ebeam(2)
    sgammap = 4.d0*xp1*ebeam(1)*ebeam(2)!CF
    W = DSQRT(sgammap)
    shat = 4.d0*xp1*xp2*ebeam(1)*ebeam(2)  !CF -- ehat should be equal to shat
    Q2 = -scalar_product(p(1,1:4),p(1,1:4))
    ptb = DSQRT(phad(3,1)**2+phad(3,2)**2)
    ptb2 = ptb**2
!     write(*,*) "ptb =", ptb, "ptb2 =",ptb2
!     ptbbar = DSQRT(phad(4,1)**2+phad(4,2)**2)
!     avptb = DSQRT((ptb**2 + ptbbar**2)/2.0)
!     ptb = transverse(phad(3,1:4) !CF
!     ptpsi2 = (ptpsi**2)
!     ptz = DSQRT(xpz(1)**2+xpz(2)**2)
    yb = rapidity(phad(3,1:4))
    ybbar = rapidity(phad(4,1:4))
!     write(*,*) "ptb =", ptb
    mtb = DSQRT(ptb**2 + mb**2)
    z = scalar_product(phad(3,1:4),phad(2,1:4)) / scalar_product(phad(1,1:4),phad(2,1:4))
!     write(*,*) "W =", W, "z =", z
    yh = 2.d0 * ebeam(2) * mtb * exp(-yb) / (z * 4.d0 * ebeam(1) * ebeam(2))
!     z = 2.0d0 * mtpsi * EXP(-ypsi) / W
    IF(z.GT.1.d0) WRITE(*,*) "WARNING: z > 1"
!     WRITE(*,*) e,xp1,xp2,z,ptpsi,mtpsi,sqrt(sgammap),sqrt(shat)
!     close(200)
    H1cuts = W.GT.71.3d0.AND.W.LT.257.2d0.AND.yb.GT.(-2d0).AND.yb.LT.2.0d0.AND.Q2.LT.1.d0.AND.yh.GT.0.05d0.AND.yh.LT.0.65d0
!     H1cutspsi = W.GT.60.d0.AND.W.LT.240.d0.AND.Q2.LT.1.d0.AND.yb.GT.(-2d0).AND.yb.LT.2.0d0.AND.yh.GT.0.05d0.AND.yh.LT.0.65d0
    H1cutspsi = W.GT.60.d0.AND.W.LT.240.d0.AND.Q2.LT.1.d0.AND.z.GT.0.3d0.AND.z.LT.0.9d0
!     zeuscutsz = Q2.LT.1.d0 
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!     DO i=1,4
!        l=(i-1)*inum_plots
    l=0
!        IF(.NOT.zeuscutsz) CYCLE
! 500    WRITE(*,*) "GOTO 500" 
!        IF(zeuscutsz) CALL HISTOGRAM_fill(l+3,z,wgts)
!        IF(H1cutsz) CALL HISTOGRAM_fill(l+3,z,wgts)
!        IF(Pierrecuts) CALL HISTOGRAM_fill(l+3,z,wgts)
!        IF(.NOT.zeuscuts) CYCLE
!        IF(.NOT.H1cuts) CYCLE
!        IF(.NOT.EIC100cuts) CYCLE
!     CALL HISTOGRAM_fill(l+1,var,wgts)
    IF(H1cuts.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+1,ptb,wgts)
!     IF(H1cutspsi.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+1,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.1.d0.AND.ptb2.LT.2.1d0) CALL HISTOGRAM_fill(l+1,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.2.1d0.AND.ptb2.LT.3.5d0) CALL HISTOGRAM_fill(l+2,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.3.5d0.AND.ptb2.LT.5.4d0) CALL HISTOGRAM_fill(l+3,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.5.4d0.AND.ptb2.LT.7.6d0) CALL HISTOGRAM_fill(l+4,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.7.6d0.AND.ptb2.LT.10.d0) CALL HISTOGRAM_fill(l+5,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.10.d0.AND.ptb2.LT.13.5d0) CALL HISTOGRAM_fill(l+6,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.13.5d0.AND.ptb2.LT.20.d0) CALL HISTOGRAM_fill(l+7,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.20.d0.AND.ptb2.LT.26.d0) CALL HISTOGRAM_fill(l+8,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.26.5d0.AND.ptb2.LT.40.0d0) CALL HISTOGRAM_fill(l+9,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.40.0d0.AND.ptb2.LT.60.0d0) CALL HISTOGRAM_fill(l+10,ptb2,wgts)
!     IF(H1cutspsi.AND.ptb2.GT.60.0d0.AND.ptb2.LT.100.d0) CALL HISTOGRAM_fill(l+11,ptb2,wgts)    
    IF(H1cutspsi.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+2,z,wgts)
    IF(H1cutspsi.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+3,W,wgts)
    IF(H1cutspsi.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+4,yb,wgts)
    IF(H1cutspsi.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+5,ybbar,wgts)
!     IF(H1cuts.AND.ptb.GT.1.d0) CALL HISTOGRAM_fill(l+6,avptb,wgts)
!     ENDDO

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file
  
END MODULE plot_user
