MODULE plot_user
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=24 ! num_plots=inum_plots*3
  INTEGER::inum_plots=8
  INTEGER,PARAMETER::n_width=1,n_height=1
  CHARACTER(len=12),DIMENSION(3)::char_cuts=(/'            ',&
       '(ATLAS incl)','(ATLAS fidu)'/)
  REAL(KIND(1d0)),PARAMETER::mass_zboson=91.1876d0
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    DO i=1,3
       l=(i-1)*inum_plots
       ! bookup is defined in analysis/hbook/dbook.f
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       !
       ! enlarge num_plots at the begining
       !
       ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
       CALL histogram_book(l+ 1,'total rate'//ADJUSTL(TRIM(char_cuts(i))),&
            2,0d0,4d0,.FALSE.)
       CALL histogram_book(l+ 2,'psi pt'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 3,'psi y'//ADJUSTL(TRIM(char_cuts(i))),&
            100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 4,'psi m'//ADJUSTL(TRIM(char_cuts(i))),&
            100,2.8d0,4d0,.TRUE.)
       CALL histogram_book(l+ 5,'w pt'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 6,'w y'//ADJUSTL(TRIM(char_cuts(i))),&
               100,-5d0,5d0,.TRUE.)
       CALL histogram_book(l+ 7,'psi-w m'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,1000d0,.TRUE.)
       CALL histogram_book(l+ 8,'Delta phi(W,psi)'//ADJUSTL(TRIM(char_cuts(i))),&
            100,0d0,pi,.TRUE.)
    ENDDO

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::xponium,xpw,xponiumw
    REAL(KIND(1d0)),DIMENSION(4)::xplw,xpnw,xplponium,xplmonium,p_lplm
    INTEGER::numonium,numw
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e,ptpsilp,ptpsilm,etapsilp,etapsilm
    REAL(KIND(1d0))::ptwl,ptwn,etawl,etawn,mtw,phil,phin,invm_lplm
    REAL(KIND(1d0))::exp1,exp2,mpsi,ypsi,ptpsi,yw,ptw,mpsiw,dphipsiw
    LOGICAL::atlaspsiselection,atlaspsiselection2
    LOGICAL::atlaswselection
    LOGICAL,DIMENSION(3)::atlascuts
    INTEGER::i,kk,l,charge_w
    REAL(KIND(1d0))::www,yftot
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
    IF(wgts(1).EQ.0d0)RETURN
    IF(NDecayChains.GT.0)THEN
       CALL plot_fill_Decay(wgts)
       RETURN
    ENDIF
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    !
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    xponium(1:4)=0d0
    xpw(1:4)=0d0
    numonium=0
    numw=0
    kk=1
    charge_w=0
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          xponium(1:4)=phad(i,1:4)
          numonium=numonium+1
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          IF(iflh(i).EQ.33.OR.iflh(i).EQ.34)THEN
             xpw(1:4)=phad(i,1:4)
             numw=numw+1
             IF(charge_w.EQ.0)THEN
                IF(iflh(i).EQ.33)THEN
                   ! w+
                   charge_w=1
                ELSE
                   ! w-
                   charge_w=-1
                ENDIF
             ENDIF
          ENDIF
          kk=kk+1
       ENDIF
    ENDDO
    IF(numw.NE.1)THEN
       WRITE(*,*)"ERROR:the number of w is not equal to 1 !"
       STOP
    ENDIF
    IF(numonium.NE.1)THEN
       WRITE(*,*)"ERROR:the number of onium is not equal to 1 !"
       STOP
    ENDIF
    xponiumw(1:4)=xponium(1:4)+xpw(1:4)
    ! trivial decay (without spin correlation)
    mpsi=DSQRT(xponium(4)**2-xponium(1)**2-xponium(2)**2-xponium(3)**2)
    CALL w_ln_kin(xpw,xplw,xpnw)
    CALL psi_ll_kin(xponium,mpsi,xplponium,xplmonium)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ptpsi = DSQRT(xponium(1)**2+xponium(2)**2)
    ptw = DSQRT(xpw(1)**2+xpw(2)**2)
    ypsi = rapidity(xponium)
    yw = rapidity(xpw)
    mpsiw = DSQRT(xponiumw(4)**2-xponiumw(1)**2-xponiumw(2)**2-xponiumw(3)**2)
    dphipsiw = ph4(xponium(1),xponium(2),xponium(3))
    dphipsiw = dphipsiw-ph4(xpw(1),xpw(2),xpw(3))
    dphipsiw = DABS(dphipsiw)
    IF(dphipsiw.GT.pi)dphipsiw=2d0*pi-dphipsiw
    ! ATLAS cuts
    ! arXiv:1401.2831
    atlaspsiselection=ptpsi.GT.8.5d0.AND.ptpsi.LT.30d0
    atlaspsiselection=atlaspsiselection.AND.ABS(ypsi).LT.2.1d0
    atlaspsiselection2=atlaspsiselection
    ptpsilp=DSQRT(xplponium(1)**2+xplponium(2)**2)
    ptpsilm=DSQRT(xplmonium(1)**2+xplmonium(2)**2)
    etapsilp=prapidity(xplponium(1:4))
    etapsilm=prapidity(xplmonium(1:4))
    atlaspsiselection=atlaspsiselection.AND.MAX(ptpsilp,ptpsilm).GT.4d0
    IF(ptpsilp.GE.ptpsilm)THEN
       atlaspsiselection=atlaspsiselection.AND.ABS(etapsilp).LT.2.5d0
       atlaspsiselection=atlaspsiselection.AND.((ptpsilm.GT.2.5d0.AND.&
            ABS(etapsilm).GT.1.3d0.AND.ABS(etapsilm).LT.2.5d0).OR.&
            (ptpsilm.GT.3.5d0.AND.ABS(etapsilm).LT.1.3d0))
    ELSE
       atlaspsiselection=atlaspsiselection.AND.ABS(etapsilm).LT.2.5d0
       atlaspsiselection=atlaspsiselection.AND.((ptpsilp.GT.2.5d0.AND.&
            ABS(etapsilp).GT.1.3d0.AND.ABS(etapsilp).LT.2.5d0).OR.&
            (ptpsilp.GT.3.5d0.AND.ABS(etapsilp).LT.1.3d0))
    ENDIF
    IF(charge_w.GT.0)THEN
       ! w+
       p_lplm(1:4)=xplw(1:4)+xplmonium(1:4)
    ELSE
       ! w-
       p_lplm(1:4)=xplw(1:4)+xplponium(1:4)
    ENDIF
    invm_lplm=DSQRT(p_lplm(4)**2-p_lplm(1)**2-p_lplm(2)**2-p_lplm(3)**2)
    atlaspsiselection=atlaspsiselection.AND.&
         (DABS(invm_lplm-mass_zboson).GT.10d0)
    ptwl=DSQRT(xplw(1)**2+xplw(2)**2)
    ptwn=DSQRT(xpnw(1)**2+xpnw(2)**2)
    etawl=prapidity(xplw(1:4))
    etawn=prapidity(xpnw(1:4))
    phil=ph4(xplw(1),xplw(2),xplw(3))
    phin=ph4(xpnw(1),xpnw(2),xpnw(3))
    mtw=2d0*ptwl*ptwn*MAX((1d0-DCOS(phil-phin)),0d0)
    mtw=DSQRT(mtw)
    atlaswselection=ABS(etawl).LT.2.4d0.AND.ptwl.GT.25d0
    atlaswselection=atlaswselection.AND.ptwn.GT.20d0
    atlaswselection=atlaswselection.AND.mtw.GT.40d0
    atlascuts(1)=.TRUE.
    atlascuts(3)=atlaspsiselection.AND.atlaswselection
    atlascuts(2)=atlaspsiselection2.AND.atlaswselection
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    DO i=1,3
       IF(.NOT.atlascuts(i))CYCLE
       l=(i-1)*inum_plots
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,ptpsi,wgts)
       CALL HISTOGRAM_fill(l+3,ypsi,wgts)
       CALL HISTOGRAM_fill(l+4,mpsi,wgts)
       CALL HISTOGRAM_fill(l+5,ptw,wgts)
       CALL HISTOGRAM_fill(l+6,yw,wgts)
       CALL HISTOGRAM_fill(l+7,mpsiw,wgts)
       CALL HISTOGRAM_fill(l+8,dphipsiw,wgts)
    ENDDO

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE psi_ll_kin(ppsi,mpsi,plp,plm)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::ppsi
    REAL(KIND(1d0)),INTENT(IN)::mpsi
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::plp,plm
    REAL(KIND(1d0)),DIMENSION(4)::plpres,plmres
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::p0,phi,costh,sinth
    costh=2d0*(rand()-0.5d0)
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=rand()*2d0*pipi
    p0=mpsi/2d0
    plpres(3)=p0*costh
    plpres(4)=p0
    plpres(1)=p0*sinth*DCOS(phi)
    plpres(2)=p0*sinth*DSIN(phi)
    plmres(4)=p0
    plmres(1:3)=-plpres(1:3)
    CALL boost(mpsi,ppsi,plpres,plp)
    CALL boost(mpsi,ppsi,plmres,plm)
    RETURN
  END SUBROUTINE psi_ll_kin

  SUBROUTINE w_ln_kin(pw,pl,pn)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pw
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::pl,pn
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::mw,phi,p0,costh,sinth
    REAL(KIND(1d0)),DIMENSION(4)::plpres,plmres
    mw=DSQRT(pw(4)**2-pw(1)**2-pw(2)**2-pw(3)**2)
    costh=2d0*(rand()-0.5d0)
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=rand()*2d0*pipi
    p0=mw/2d0
    plpres(3)=p0*costh
    plpres(4)=p0
    plpres(1)=p0*sinth*DCOS(phi)
    plpres(2)=p0*sinth*DSIN(phi)
    plmres(4)=p0
    plmres(1:3)=-plpres(1:3)
    CALL boost(mw,pw,plpres,pl)
    CALL boost(mw,pw,plmres,pn)
    RETURN
  END SUBROUTINE w_ln_kin

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE plot_fill_Decay(wgts)
    USE Decay_interface
    USE QEDPS_interface
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "../../decay/stdhep.inc"! decay variables are defined in stdhep.inc
    REAL(KIND(1d0))::var,Q,www
    REAL(KIND(1d0))::exp1,exp2,mpsi,ypsi,ptpsi,yw,ptw,mpsiw,dphipsiw
    REAL(KIND(1d0))::ptwl,ptwn,etawl,etawn,e,ptpsilp,ptpsilm,etapsilp,etapsilm
    !REAL(KIND(1d0))::Pt_psipsi_sm05,Pt_psipsi_sm10,Pt_psipsi_sm20,Pt_psipsi_sm30
    !REAL(KIND(1d0))::dy_psipsi_sm05,dy_psipsi_sm10,dy_psipsi_sm20,dy_psipsi_sm30
    !REAL(KIND(1d0))::dphi_psipsi_sm05,dphi_psipsi_sm10,dphi_psipsi_sm20,dphi_psipsi_sm30
    !REAL(KIND(1d0))::M_psipsi_sm05,M_psipsi_sm10,M_psipsi_sm20,M_psipsi_sm30
    INTEGER::i,m,l,kk,charge_w
    !REAL(KIND(1d0)),DIMENSION(4,NMXHEP)::PHEPSM05,PHEPSM10,PHEPSM20,PHEPSM30
    REAL(KIND(1d0)),DIMENSION(4)::xpw,xponium,xponiumw,xplw,xpnw,xplponium,xplmonium
    REAL(KIND(1d0)),DIMENSION(4)::p_lplm
    REAL(KIND(1d0))::invm_lplm,phil,phin,mtw
    LOGICAL::atlaspsiselection,atlaspsiselection2
    LOGICAL::atlaswselection
    LOGICAL,DIMENSION(3)::atlascuts
    INTEGER::numw,numonium,numwl,numoniuml
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979312D0
    IF(wgts(1).EQ.0d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton
    !CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM05(1:4,1:NHEP),0.5d0,2)
    !CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM10(1:4,1:NHEP),1d0,2)
    !CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM20(1:4,1:NHEP),2d0,2)
    !CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM30(1:4,1:NHEP),3d0,2)
    numw=0
    numonium=0
    numoniuml=0
    numwl=0
    charge_w=0
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(lepton_pdg(IDHEP(i)))THEN
          IF(ISHEP(JMOHEP(1,i)).NE.1)CYCLE
          IF(ISTHEP(JMOHEP(1,i)).NE.1)CYCLE
          IF(ABS(IDHEP(JMOHEP(1,i))).EQ.24)THEN
             numwl=numwl+1
             IF(charge_w.EQ.0)THEN
                IF(IDHEP(JMOHEP(1,i)).EQ.24)THEN
                   ! w+
                   charge_w=1
                ELSE
                   ! w- 
                   charge_w=-1
                ENDIF
             ENDIF
             IF(ABS(IDHEP(i)).EQ.13.OR.ABS(IDHEP(i)).EQ.11)THEN
                xplw(1:4)=PHEP(1:4,i)
             ELSEIF(ABS(IDHEP(i)).EQ.12.OR.ABS(IDHEP(i)).EQ.14)THEN
                xpnw(1:4)=PHEP(1:4,i)
             ELSE
                WRITE(*,*)"ERROR: this analysis file only supports w > l v, with l=e,mu"
                STOP
             ENDIF
          ELSE
             numoniuml=numoniuml+1
             IF(IDHEP(i).GT.0)THEN
                xplponium(1:4)=PHEP(1:4,i)
             ELSE
                xplmonium(1:4)=PHEP(1:4,i)
             ENDIF
          ENDIF
       ELSEIF(ABS(IDHEP(i)).EQ.24.AND.ISTHEP(i).EQ.0)THEN
          numw=numw+1
          IF(charge_w.EQ.0)THEN
             IF(IDHEP(i).EQ.24)THEN
                ! w+
                charge_w=1
             ELSE
                ! w-
                charge_w=-1
             ENDIF
          ENDIF
          xpw(1:4)=PHEP(1:4,i)
       ELSEIF(ABS(rpdgt(IDHEP(i))).GT.100.AND.ISTHEP(i).EQ.0)THEN
          numonium=numonium+1
          xponium(1:4)=PHEP(1:4,i)
       ENDIF
    ENDDO
    IF(.NOT.((numw.EQ.0.AND.numwl.EQ.2).OR.(numw.EQ.1.AND.numwl.EQ.0)))THEN
       WRITE(*,*)"ERROR: number of w is not equal to 1 !"
       STOP
    ENDIF
    IF(.NOT.((numonium.EQ.0.AND.numoniuml.EQ.2).OR.&
         (numonium.EQ.1.AND.numoniuml.EQ.0)))THEN
       WRITE(*,*)"ERROR: number of onium is not equal to 1 !"
       STOP
    ENDIF
    IF(numw.EQ.0)THEN
       xpw(1:4)=xplw(1:4)+xpnw(1:4)
    ELSE
       CALL w_ln_kin(xpw,xplw,xpnw)
    ENDIF
    IF(numonium.EQ.0)THEN
       xponium(1:4)=xplponium(1:4)+xplmonium(1:4)
       mpsi=DSQRT(xponium(4)**2-xponium(1)**2-xponium(2)**2-xponium(3)**2)
    ELSE
       mpsi=DSQRT(xponium(4)**2-xponium(1)**2-xponium(2)**2-xponium(3)**2)
       CALL psi_ll_kin(xponium,mpsi,xplponium,xplmonium)
    ENDIF
    xponiumw(1:4)=xponium(1:4)+xpw(1:4)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1d0
    ptpsi = DSQRT(xponium(1)**2+xponium(2)**2)
    ptw = DSQRT(xpw(1)**2+xpw(2)**2)
    ypsi = rapidity(xponium)
    yw = rapidity(xpw)
    mpsiw = DSQRT(xponiumw(4)**2-xponiumw(1)**2-xponiumw(2)**2-xponiumw(3)**2)
    dphipsiw = ph4(xponium(1),xponium(2),xponium(3))
    dphipsiw = dphipsiw-ph4(xpw(1),xpw(2),xpw(3))
    dphipsiw = DABS(dphipsiw)
    IF(dphipsiw.GT.pi)dphipsiw=2d0*pi-dphipsiw
    ! ATLAS cuts
    ! arXiv:1401.2831
    atlaspsiselection=ptpsi.GT.8.5d0.AND.ptpsi.LT.30d0
    atlaspsiselection=atlaspsiselection.AND.ABS(ypsi).LT.2.1d0
    atlaspsiselection2=atlaspsiselection
    ptpsilp=DSQRT(xplponium(1)**2+xplponium(2)**2)
    ptpsilm=DSQRT(xplmonium(1)**2+xplmonium(2)**2)
    etapsilp=prapidity(xplponium(1:4))
    etapsilm=prapidity(xplmonium(1:4))
    atlaspsiselection=atlaspsiselection.AND.MAX(ptpsilp,ptpsilm).GT.4d0
    IF(ptpsilp.GE.ptpsilm)THEN
       atlaspsiselection=atlaspsiselection.AND.ABS(etapsilp).LT.2.5d0
       atlaspsiselection=atlaspsiselection.AND.((ptpsilm.GT.2.5d0.AND.&
            ABS(etapsilm).GT.1.3d0.AND.ABS(etapsilm).LT.2.5d0).OR.&
            (ptpsilm.GT.3.5d0.AND.ABS(etapsilm).LT.1.3d0))
    ELSE
       atlaspsiselection=atlaspsiselection.AND.ABS(etapsilm).LT.2.5d0
       atlaspsiselection=atlaspsiselection.AND.((ptpsilp.GT.2.5d0.AND.&
            ABS(etapsilp).GT.1.3d0.AND.ABS(etapsilp).LT.2.5d0).OR.&
            (ptpsilp.GT.3.5d0.AND.ABS(etapsilp).LT.1.3d0))
    ENDIF
    IF(charge_w.GT.0)THEN
       ! w+
       p_lplm(1:4)=xplw(1:4)+xplmonium(1:4)
    ELSE
       ! w-
       p_lplm(1:4)=xplw(1:4)+xplponium(1:4)
    ENDIF
    invm_lplm=DSQRT(p_lplm(4)**2-p_lplm(1)**2-p_lplm(2)**2-p_lplm(3)**2)
    atlaspsiselection=atlaspsiselection.AND.&
         (DABS(invm_lplm-mass_zboson).GT.10d0)
    ptwl=DSQRT(xplw(1)**2+xplw(2)**2)
    ptwn=DSQRT(xpnw(1)**2+xpnw(2)**2)
    etawl=prapidity(xplw(1:4))
    etawn=prapidity(xpnw(1:4))
    phil=ph4(xplw(1),xplw(2),xplw(3))
    phin=ph4(xpnw(1),xpnw(2),xpnw(3))
    mtw=2d0*ptwl*ptwn*MAX((1d0-DCOS(phil-phin)),0d0)
    mtw=DSQRT(mtw)
    atlaswselection=ABS(etawl).LT.2.4d0.AND.ptwl.GT.25d0
    atlaswselection=atlaswselection.AND.ptwn.GT.20d0
    atlaswselection=atlaswselection.AND.mtw.GT.40d0
    atlascuts(1)=.TRUE.
    atlascuts(3)=atlaspsiselection.AND.atlaswselection
    atlascuts(2)=atlaspsiselection2.AND.atlaswselection
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    !
    ! START TO FILL HISTOGRAM 
    !  
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    DO i=1,3
       IF(.NOT.atlascuts(i))CYCLE
       l=(i-1)*inum_plots
       CALL HISTOGRAM_fill(l+1,var,wgts)
       CALL HISTOGRAM_fill(l+2,ptpsi,wgts)
       CALL HISTOGRAM_fill(l+3,ypsi,wgts)
       CALL HISTOGRAM_fill(l+4,mpsi,wgts)
       CALL HISTOGRAM_fill(l+5,ptw,wgts)
       CALL HISTOGRAM_fill(l+6,yw,wgts)
       CALL HISTOGRAM_fill(l+7,mpsiw,wgts)
       CALL HISTOGRAM_fill(l+8,dphipsiw,wgts)
    ENDDO
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay

  SUBROUTINE plot_fill_QEDPS(wgts)
    USE QEDPS_interface
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    ! QEDPS parameters                                                      
    REAL(KIND(1d0)),DIMENSION(10,1000)::PLPTN
    INTEGER,DIMENSION(10,1000)::NLPTN
    INTEGER::NPTCL
    COMMON/QPLIST/PLPTN,NLPTN,NPTCL
    REAL(KIND(1d0)),DIMENSION(4)::pini
    REAL(KIND(1d0))::wgt,var,frache
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l,nphoton
    REAL(KIND(1d0))::www,yftot
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    WRITE(*,*)"Please choose the correct analysis file plot_user.f90"
    STOP
    var=1.0d0 ! total cross section
    pini(1:4)=PLPTN(4:7,1)+PLPTN(4:7,2)
    yftot=rapidity(pini)
    nphoton=0
    DO i=1,NPTCL
       IF(NLPTN(1,i).EQ.22)THEN
          nphoton=nphoton+1
       ENDIF
    ENDDO
    frache=MIN(Q2OUT_QEDPS/Q2MAX_QEDPS,1d0)
    frache=DSQRT(frache)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,yftot,wgts)
    CALL HISTOGRAM_fill(l+3,DBLE(nphoton),wgts)
    CALL HISTOGRAM_fill(l+4,frache,wgts)

    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_QEDPS
END MODULE plot_user
