MODULE plot_user
!============================================================
!
! PLOT FILE FOR USER
! p p > psi + n jet in 0.0<y<4.5
! STOP cuts (arXiv:1809.02369)
! with Non-perturbative effects
!
! WARNING WARNING WARNING WARNING WARNING WARNING WARNING
!
! To change max_plots in analysis/hbook/HwU.inc
! To change max_histograms in analysis/hbook/histograms_common.inc
! To change max_histograms in analysis/hbook/histograms_common90.inc
! They should be at least larger than num_plots below
! Recompile from the scratch after changing above parameters 
!
! WARNING WARNING WARNING WARNING WARNING WARNING WARNING
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=1729 ! note to change max_plots in analysis/hbook/HwU.inc and analysis/hbook/HwU90.inc
  ! to change max_histograms in analysis/hbook/histograms_common.inc and analysis/hbook/histograms_common90.inc
  INTEGER,PARAMETER::n_width=1,n_height=1
  ! stop cuts for analysis
  INTEGER::stop_min_n_jet_ana,stop_max_n_jet_ana,stop_jetalg_ana
  INTEGER::stop_n_frag_gluon_ana,stop_n_frag_quark_ana
  REAL(KIND(1d0))::stop_maxrapjet_ana,stop_jet_radius_ana,stop_minptjet_ana
  REAL(KIND(1d0))::stop_zcut_ana,stop_beta_ana,stop_zsoftcut_ana
  REAL(KIND(1d0))::stop_zasymcut_ana,stop_jet_dyn_radius_ana
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,jj,kk,l,ll,mm,nn
    ! J/psi momentum from different final states
    CHARACTER(len=7),DIMENSION(6)::fstates=(/'FO     ','Psi(2S)','Chic0  ',&
         'Chic1  ','Chic2  ','J/Psi  '/)
    ! stop cuts dependence
    ! arXiv:1809.02369
    CHARACTER(len=4),DIMENSION(2)::minjpt=(/'ptj3,zasym01','ptj6,zasym07'/)
    ! radiation function dependence
    ! Eq.(2.22) in hep-ph/0001062
    CHARACTER(len=4),DIMENSION(4)::radiations=(/'L0.0','L0.2','L0.3','L0.5'/)
    CHARACTER(len=4),DIMENSION(3)::momreshuf=(/'NORS','ISRS','FSRS'/)
    ! chic or psi(2S) decay pattern
    ! D1: p[chic/psi(2S)] = p[J/psi]
    ! D2: p[chic/psi(2S)]*M[J/psi]/M[chic/psi(2S)] = p[J/psi]
    ! D3: p[J/psi] is from p[chic/psi(2S)] via spin uncorrelated decay
    ! D4: p[J/psi] is from p[chic/psi(2S)] via spin correlated decay
    CHARACTER(len=2),DIMENSION(4)::decayway=(/'D1','D2','D3','D4'/)
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(emep_ISR_shower.EQ.1)THEN
       WRITE(*,*)"Please change a plot_user.f90 !!"
       STOP
    ENDIF
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    l=0
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    ! enlarge max_plots in ../hbook/HwU.inc
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    CALL HISTOGRAM_book(l+ 1,'total rate  ',&
         5,0.5d0,5.5d0,.FALSE.)
    l=1
    DO i=1,2
       ! pt cut on the jet, stop_minptjet
       DO kk=1,4
          ! Lambda values in radiation function
          DO ll=1,3
             ! perfoming momentum reshuffling (or not)
             DO mm=1,4
                ! shifting momentum from excited states
                DO nn=1,6
                   ! J/psi momentum from different final states
                   l=1+(nn-1)*3+(mm-1)*6*3+&
                        (ll-1)*4*6*3+(kk-1)*3*4*6*3+&
                        (i-1)*4*3*4*6*3
                   CALL HISTOGRAM_book(l+ 1,'pt of '//fstates(nn)//' in |y|<0.6 '//minjpt(i)&
                        //' '//radiations(kk)//' '//momreshuf(ll)//' '//decayway(mm),&
                        100,5d0,105d0,.TRUE.)
                   CALL HISTOGRAM_book(l+ 2,'pt of '//fstates(nn)//' in |y|<0.75 '//minjpt(i)&
                        //' '//radiations(kk)//' '//momreshuf(ll)//' '//decayway(mm),&
                        100,5d0,105d0,.TRUE.)
                   CALL HISTOGRAM_book(l+ 3,'pt of '//fstates(nn)//' in 0.75<|y|<1.5 '//minjpt(i)&
                        //' '//radiations(kk)//' '//momreshuf(ll)//' '//decayway(mm),&
                        100,5d0,105d0,.TRUE.)
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    
    CALL HISTOGRAM_BOOK_FINISH

10203 FORMAT(E9.3,"<pt<",E9.3)
10204 FORMAT(E9.3)
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    USE Helac_Global
    USE Structf_PDFs
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0)),DIMENSION(SIZE(wgts))::wgts_Final
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l,j,jj,ll
    REAL(KIND(1d0))::www,yy,pt
    INTEGER::n_track,n_jet,n_track2,n_jet2
    REAL(KIND(1d0)),DIMENSION(4,20)::p_track,p_track2
    REAL(KIND(1d0)),DIMENSION(4,20)::p_jet
    INTEGER,DIMENSION(20)::jetvec
    INTEGER::NJET
    REAL(KIND(1d0)),DIMENSION(20,4)::PJET
    INTEGER,DIMENSION(3)::index2
    INTEGER::fockstate,imoth,isign,JzJz
    REAL(KIND(1d0))::rrr
    LOGICAL,DIMENSION(2)::pass_stop_cuts
    INTEGER,DIMENSION(20)::pid_track
    REAL(KIND(1d0)),DIMENSION(2)::new_stop_minptjet
    REAL(KIND(1d0)),DIMENSION(2)::new_stop_zasymcut
    REAL(KIND(1d0)),DIMENSION(4)::radiation_lam
    ! the masses of psi(2S), chi_c0, chi_c1, chi_c2, J/psi
    REAL(KIND(1d0)),DIMENSION(2:6),PARAMETER::masses=(/3.686d0,3.415d0,3.511d0,3.556d0, 3.097d0/)
    ! the branching ratios into J/psi
    REAL(KIND(1d0)),DIMENSION(2:5),PARAMETER::BRList=(/0.574d0,0.0128d0,0.36d0,0.2d0/)
    LOGICAL,DIMENSION(2:6)::fo2state ! to judge whether the Fock state will contribute to the charmonium
    REAL(KIND(1d0)),DIMENSION(2:6,4)::P_JPsi ! the J/psi momentum from different charmonium
    REAL(KIND(1d0)),DIMENSION(nhad+1,4)::POUT1,POUT2
    REAL(KIND(1d0)),DIMENSION(2:6,nhad+1,4)::POUT_RS
    INTEGER::NPtotal
    LOGICAL::coloroctet,twomomkeep
    REAL(KIND(1d0))::rad_bn,rad_c,rad_c2,x1new,x2new,wwsf,wwsf0,x1orig,x2orig
    REAL(KIND(1d0)),DIMENSION(2:6)::wgt_RS,wgt_Final
    REAL(KIND(1d0)),DIMENSION(nhad+1)::MA_IN
    REAL(KIND(1d0)),DIMENSION(4)::Ponium,PX
    IF(wgts(1).EQ.0d0)RETURN
    !IF(NDecayChains.GT.0)THEN
    !   CALL plot_fill_Decay(wgts)
    !   RETURN
    !ENDIF
    !WRITE(*,*)"ERROR:events without decay is not supported !"
    !STOP
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    ! 
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    ! preparing the setup
    new_stop_minptjet(1)=3d0
    new_stop_zasymcut(1)=0.1d0
    new_stop_minptjet(2)=6d0
    new_stop_zasymcut(2)=0.7d0
    radiation_lam(1)=0d0
    radiation_lam(2)=0.2d0
    radiation_lam(3)=0.3d0
    radiation_lam(4)=0.5d0
    fo2state(2:6)=.FALSE.

    kk=1
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          ! get the last fock state
          IF(calc_LDME_RG_CT)THEN
             fockstate=iflh_CT(i)
          ELSE
             fockstate=iflh(i)
          ENDIF
          imoth=i
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          kk=kk+1
       ENDIF
    ENDDO
    coloroctet=MOD(fockstate,10).EQ.8
    twomomkeep=.FALSE.
    ! same as Eq.(2.30), Eq.(2.31) in hep-ph/0001062
    IF(fockstate.EQ.441008)THEN
       rad_bn=2d0 ! because 1S0[8] -> J/psi, psi(2S) has one M1 transition
       rad_c=1d0
       rad_c2=-1d0
    ELSEIF(fockstate.EQ.443018)THEN
       rad_bn=0d0
       rad_c=1.5d0 ! the first choice is 3S1[8] -> J/psi,psi(2S), which has two E1 transitions
       rad_c2=1d0 ! the second choice is for 3S1[8] -> chi_c, which only has one E1 transition
       twomomkeep=.TRUE.
    ELSEIF(coloroctet)THEN
       rad_bn=0d0 ! because others to J/psi has one E1 transition
       rad_c=1d0
       rad_c2=-1d0
    ENDIF
    IF(fockstate.EQ.443018)THEN
       fo2state(2:6)=.TRUE.
    ELSEIF(fockstate.EQ.443011.OR.fockstate.EQ.441008.OR.&
         fockstate.EQ.443108.OR.fockstate.EQ.443118.OR.&
         fockstate.EQ.443128)THEN
       fo2state(2)=.TRUE.
       fo2state(6)=.TRUE.
    ELSEIF(fockstate.EQ.443101)THEN
       fo2state(3)=.TRUE.
    ELSEIF(fockstate.EQ.443111)THEN
       fo2state(4)=.TRUE.
    ELSEIF(fockstate.EQ.443121)THEN
       fo2state(5)=.TRUE.
    ELSE
       WRITE(*,*)"ERROR: the analysis file cannot deal with fock state = ",fockstate
       STOP
    ENDIF
    CALL strf_pdf(wwsf0)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! CHECK WHETHER PASS THE NEW STOP CUTS
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    n_track=0
    n_jet=0
    p_track(1:4,1:20)=0d0
    DO i=3,nhad
       IF(is_a_jet_ana(iflh(i)))THEN
          n_track=n_track+1
          p_track(1:4,n_track)=phad(i,1:4)
          pid_track(n_track)=pdgt_ana(iflh(i))
       ENDIF
    ENDDO
    ! always put J/psi at the last
    n_track2=n_track+1
    n_jet2=0
    p_track2(1:4,1:20)=0d0
    p_track2(1:4,1:n_track)=p_track(1:4,1:n_track)
    p_track2(1:4,n_track2)=phad(imoth,1:4)
    pid_track(n_track2)=iflh(imoth)
    pass_stop_cuts(1:2)=.TRUE.
    IF(use_stop_cut)THEN
       ! check the stop cuts
       stop_min_n_jet_ana=stop_min_n_jet
       stop_max_n_jet_ana=stop_max_n_jet
       stop_jetalg_ana=stop_jetalg
       stop_n_frag_gluon_ana=stop_n_frag_gluon
       stop_n_frag_quark_ana=stop_n_frag_quark
       stop_maxrapjet_ana=stop_maxrapjet
       stop_jet_radius_ana=stop_jet_radius
       stop_jet_dyn_radius_ana=stop_jet_dyn_radius
       stop_minptjet_ana=stop_minptjet
       stop_zcut_ana=stop_zcut
       stop_beta_ana=stop_beta
       stop_zsoftcut_ana=stop_zsoftcut
       stop_zasymcut_ana=stop_zasymcut
       DO i=1,2
          stop_minptjet_ana=new_stop_minptjet(i)
          IF(i.EQ.1)THEN
             pass_stop_cuts(i)=.TRUE.
          ELSE
             stop_zasymcut_ana=new_stop_zasymcut(i)
             CALL check_stop_cuts(n_track2,p_track2,pid_track,pass_stop_cuts(i))
          ENDIF
       ENDDO
    ENDIF

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    CALL HISTOGRAM_fill(1,1d0,wgts) ! total cross sections

    DO i=1,2
       ! pt cut on the jet, new_stop_minptjet(i) and new_stop_zasymcut(i)
       ! check whether it passes the (new) stop cuts
       IF(.NOT.pass_stop_cuts(i))CYCLE
       DO kk=1,4
          ! Lambda values radiation_lam(kk) in radiation function
          IF(radiation_lam(kk).LE.0d0.OR..NOT.coloroctet)THEN
             ! we will not apply the radiation function
             NPtotal=nhad
             POUT1(1:nhad,1:4)=phad(1:nhad,1:4)
             IF(twomomkeep)THEN
                POUT2(1:nhad,1:4)=phad(1:nhad,1:4)
             ENDIF
          ELSE
             ! it is a subroutine defined in Kinetic_Func.f90
             ! it tries to perform the radiation function via Eq.(2.22) in hep-ph/0001062
             CALL OniumRadiationFunction(rad_c*radiation_lam(kk),rad_bn,&
                  phad(imoth,1:4),Ponium,PX)
             NPtotal=nhad+1
             POUT1(1:nhad,1:4)=phad(1:nhad,1:4)
             POUT1(imoth,1:4)=Ponium(1:4)
             POUT1(nhad+1,1:4)=PX(1:4)
             IF(twomomkeep)THEN
                ! second radiation for 3S1[8] -> chi_c
                CALL OniumRadiationFunction(rad_c2*radiation_lam(kk),rad_bn,&
                  phad(imoth,1:4),Ponium,PX)
                POUT2(1:nhad,1:4)=phad(1:nhad,1:4)
                POUT2(imoth,1:4)=Ponium(1:4)
                POUT2(nhad+1,1:4)=PX(1:4)
             ENDIF
          ENDIF
          DO l=1,3
             ! perform momentum reshuffling
             ! No, ISR, FSR
             IF(l.EQ.1)THEN
                ! no reshuffling
                DO jj=2,6
                   IF(twomomkeep.AND.fo2state(jj))THEN
                      POUT_RS(jj,1:NPtotal,1:4)=POUT2(1:NPtotal,1:4)
                   ELSE
                      POUT_RS(jj,1:NPtotal,1:4)=POUT1(1:NPtotal,1:4)
                   ENDIF
                   wgt_RS(jj)=1d0
                ENDDO
             ELSEIF(l.EQ.2.OR.l.EQ.3)THEN
                ! Initial/Final state reshuffling
                MA_IN(1:nhad+1)=-1d0
                DO jj=2,6
                   IF(fo2state(jj))THEN
                      MA_IN(imoth)=masses(jj)
                      IF(twomomkeep.AND.jj.GE.3.AND.jj.LE.5)THEN
                         ! it is 3S1[8] -> chi_c
                         IF(l.EQ.2)THEN
                            ! Initial state reshuffling
                            CALL InitialStateReshuffling(NPtotal,POUT2(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ELSE
                            ! Final state reshuffling
                            CALL FinalStateReshuffling(NPtotal,POUT2(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ENDIF
                      ELSE
                         IF(l.EQ.2)THEN
                            ! Initial state reshuffling
                            CALL InitialStateReshuffling(NPtotal,POUT1(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ELSE
                            ! Final state reshuffling
                            CALL FinalStateReshuffling(NPtotal,POUT1(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ENDIF
                      ENDIF
                      IF(x1new.NE.xp1.OR.x2new.NE.xp2)THEN
                         x1orig=xp1
                         x2orig=xp2
                         xp1=x1new
                         xp2=x2new
                         CALL strf_pdf(wwsf)
                         wgt_RS(jj)=x1orig*x2orig/(x1new*x2new)*wwsf/wwsf0
                         xp1=x1orig
                         xp2=x2orig
                      ENDIF
                   ELSE
                      wgt_RS(jj)=1d0
                   ENDIF
                ENDDO                
             ENDIF
             DO j=1,4
                ! shifting momentum or not, i.e. decayway(j)
                ! chic or psi(2S) decay pattern
                IF(j.EQ.1)THEN
                   ! D1: p[chic/psi(2S)] = p[J/psi]
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)
                         IF(jj.LT.6)THEN
                            wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                         ELSE
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ELSEIF(j.EQ.2)THEN
                   ! D2: p[chic/psi(2S)]*M[J/psi]/M[chic/psi(2S)] = p[J/psi]
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)*masses(6)/masses(jj)
                         IF(jj.LT.6)THEN
                            wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                         ELSE
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ELSEIF(j.EQ.3)THEN
                   ! D3: p[J/psi] is from p[chic/psi(2S)] via spin uncorrelated decay
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         IF(jj.LT.6)THEN
                            ! a true decay
                            CALL TwoBodyDecay_unpol_ana(POUT_RS(jj,imoth,1:4),&
                                 masses(jj),masses(6),P_JPsi(jj,1:4))
                            wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                         ELSE
                            P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ELSE
                   ! D4: p[J/psi] is from p[chic/psi(2S)] via spin correlated decay
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         IF(jj.LT.6)THEN
                            ! a true decay
                            IF(jj.EQ.2.OR.jj.EQ.3)THEN
                               ! for psi(2S) -> J/psi+pion+pion
                               ! We do not know the exact decay pattern
                               ! Let us use a two-body unpolarized decay
                               ! for chi_c0 -> J/psi+a, it must be an isotropic decay
                               CALL TwoBodyDecay_unpol_ana(POUT_RS(jj,imoth,1:4),&
                                 masses(jj),masses(6),P_JPsi(jj,1:4))
                               wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                            ELSE
                               ! they must be chi_c1, chi_c2 -> J/psi+gamma
                               IF(MCoHelicity)THEN
                                  CALL Qeveryhel_ana(index2,ipol(imoth),imoth)
                               ENDIF
                               IF(MCoHelicity)THEN
                                  IF(calc_LDME_RG_CT.OR.fockstate.EQ.443018)THEN
                                     ! use Eq.(29) in arXiv:1209.4610
                                     isign=MOD(2*index2(3)-3,3)
                                     isign=-isign
                                     IF(jj.EQ.4)THEN
                                        ! chi_c1
                                        CALL GetJz_ana(isign,1,JzJz)
                                     ELSE
                                        ! chi_c2
                                        CALL GetJz_ana(isign,2,JzJz)
                                     ENDIF
                                     isign=JzJz
                                  ELSE
                                     isign=MOD(2*index2(3)-3,3)
                                     isign=-isign
                                     isign=isign-MOD(2*index2(2)-3,3)
                                  ENDIF
                               ELSE
                                  rrr=rand()
                                  IF(jj.EQ.4)THEN
                                     ! chi_c1
                                     isign=FLOOR(3d0*rrr)-1
                                  ELSE
                                     ! chi_c2
                                     isign=FLOOR(5d0*rrr)-2
                                  ENDIF
                               ENDIF
                               IF(jj.EQ.4)THEN
                                  ! chi_c1
                                  CALL HO_chi12psia_hel_ana(POUT_RS(jj,imoth,1:4),&
                                       masses(jj),masses(6),isign,P_JPsi(jj,1:4))
                               ELSE
                                  ! chi_c2
                                  CALL HO_chi22psia_hel_ana(POUT_RS(jj,imoth,1:4),&
                                       masses(jj),masses(6),isign,P_JPsi(jj,1:4))
                               ENDIF
                               wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                            ENDIF
                         ELSE
                            P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ENDIF
                ! filling the histograms
                DO jj=1,6
                   ! J/psi from various final states
                   pt=-1d0
                   IF(jj.EQ.1)THEN
                      pt=DSQRT(phad(imoth,1)**2+phad(imoth,2)**2)
                      yy=rapidity(phad(imoth,1:4))
                      wgts_Final(1:SIZE(wgts))=wgts(1:SIZE(wgts))
                   ELSE
                      IF(fo2state(jj))THEN
                         pt=DSQRT(P_JPsi(jj,1)**2+P_JPsi(jj,2)**2)
                         yy=rapidity(P_JPsi(jj,1:4))
                         wgts_Final(1:SIZE(wgts))=wgts(1:SIZE(wgts))*wgt_Final(jj)
                      ENDIF
                   ENDIF
                   IF(pt.GE.0d0)THEN
                      ll=1+(jj-1)*3+(j-1)*6*3+&
                           (l-1)*4*6*3+(kk-1)*3*4*6*3+&
                           (i-1)*4*3*4*6*3
                      IF(DABS(yy).LT.0.6d0)CALL HISTOGRAM_fill(ll+1,pt,wgts_Final)
                      IF(DABS(yy).LT.0.75d0)CALL HISTOGRAM_fill(ll+2,pt,wgts_Final)
                      IF(DABS(yy).LT.1.5d0.AND.DABS(yy).GT.0.75d0)CALL HISTOGRAM_fill(ll+3,pt,wgts_Final)
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE psi_ll_kin(ppsi,mpsi,plp,plm)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::ppsi
    REAL(KIND(1d0)),INTENT(IN)::mpsi
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::plp,plm
    REAL(KIND(1d0)),DIMENSION(4)::plpres,plmres
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::p0,phi,costh,sinth
    costh=2d0*(rand()-0.5d0)
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=rand()*2d0*pipi
    p0=mpsi/2d0
    plpres(3)=p0*costh
    plpres(4)=p0
    plpres(1)=p0*sinth*DCOS(phi)
    plpres(2)=p0*sinth*DSIN(phi)
    plmres(4)=p0
    plmres(1:3)=-plpres(1:3)
    CALL boost(mpsi,ppsi,plpres,plp)
    CALL boost(mpsi,ppsi,plmres,plm)
    RETURN
  END SUBROUTINE psi_ll_kin

  FUNCTION is_a_jet_ana(ifh)
    IMPLICIT NONE
    LOGICAL::is_a_jet_ana
    INTEGER,INTENT(IN)::ifh
    INTEGER::iff,init=0
    SAVE init
    LOGICAL,DIMENSION(12)::is_fermion_a_jet
    SAVE is_fermion_a_jet
    IF(ABS(ifh).GT.12.AND.ABS(ifh).NE.35)THEN
       is_a_jet_ana=.FALSE.
       RETURN
    ENDIF
    IF(ifh.EQ.35)THEN
       is_a_jet_ana=.TRUE.
       RETURN
    ENDIF
    iff=ABS(ifh)
    IF(MOD(iff,4).NE.0.AND.MOD(iff,4).NE.3)THEN
       is_a_jet_ana=.FALSE.
       RETURN
    ENDIF
    IF(init.EQ.0)THEN
       is_fermion_a_jet(1:12)=.FALSE.
       IF(jet_n_flavour.GE.1)is_fermion_a_jet(4)=.TRUE.
       IF(jet_n_flavour.GE.2)is_fermion_a_jet(3)=.TRUE.
       IF(jet_n_flavour.GE.3)is_fermion_a_jet(8)=.TRUE.
       IF(jet_n_flavour.GE.4)is_fermion_a_jet(7)=.TRUE.
       IF(jet_n_flavour.GE.5)is_fermion_a_jet(12)=.TRUE.
       IF(jet_n_flavour.GE.6)is_fermion_a_jet(11)=.TRUE.
       init=1
    ENDIF
    is_a_jet_ana=is_fermion_a_jet(iff)
    RETURN
  END FUNCTION is_a_jet_ana

  FUNCTION is_a_jet_ana_PDG(pdg)
    IMPLICIT NONE
    LOGICAL::is_a_jet_ana_PDG
    INTEGER,INTENT(IN)::pdg
    INTEGER::iff,init=0
    SAVE init
    LOGICAL,DIMENSION(6)::is_fermion_a_jet
    SAVE is_fermion_a_jet
    IF(ABS(pdg).GT.6.AND.ABS(pdg).NE.21)THEN
       is_a_jet_ana_PDG=.FALSE.
       RETURN
    ENDIF
    IF(pdg.EQ.21)THEN
       is_a_jet_ana_PDG=.TRUE.
       RETURN
    ENDIF
    IF(init.EQ.0)THEN
       is_fermion_a_jet(1:6)=.FALSE.
       IF(jet_n_flavour.GE.1)is_fermion_a_jet(1)=.TRUE.
       IF(jet_n_flavour.GE.2)is_fermion_a_jet(2)=.TRUE.
       IF(jet_n_flavour.GE.3)is_fermion_a_jet(3)=.TRUE.
       IF(jet_n_flavour.GE.4)is_fermion_a_jet(4)=.TRUE.
       IF(jet_n_flavour.GE.5)is_fermion_a_jet(5)=.TRUE.
       IF(jet_n_flavour.GE.6)is_fermion_a_jet(6)=.TRUE.
       init=1
    ENDIF
    is_a_jet_ana_PDG=is_fermion_a_jet(ABS(pdg))
    RETURN
  END FUNCTION is_a_jet_ana_PDG

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                  !
! The following are just copies of functions in    !
! Helac_Func_1.f                                   !
!                                                  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  FUNCTION SubInteger_ana(int,n1,n2)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::int,n1,n2
    INTEGER::nmax,nmin,mod11,int2
    INTEGER::SubInteger_ana
    int2=ABS(int)
    nmax=MAX(n1,n2)
    nmin=MIN(n1,n2)
    IF(int2.LT.10**(nmin-1))THEN
       SubInteger_ana=0
       RETURN
    ELSEIF(nmin.GT.0)THEN
       mod11=MOD(int2,10**nmax)
       SubInteger_ana=mod11/10**(nmin-1)
       RETURN
    ELSE
       PRINT *,"Error in SubInteger_ana ! STOP!"
       STOP
    ENDIF
  END FUNCTION SubInteger_ana

  FUNCTION QN3PJF_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN3PJF_ana
    QN3PJF_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.31)RETURN
    QN3PJF_ana=.TRUE.
    RETURN
  END FUNCTION QN3PJF_ANA

  FUNCTION QN1P1F_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN1P1F_ana
    QN1P1F_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.11)RETURN
    QN1P1F_ana=.TRUE.
    RETURN
  END FUNCTION QN1P1F_ANA

  FUNCTION QN3S1F_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN3S1F_ana
    QN3S1F_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.30)RETURN
    QN3S1F_ana=.TRUE.
    RETURN
  END FUNCTION QN3S1F_ANA

  FUNCTION QN1S0F_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN1S0F_ana
    QN1S0F_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.10)RETURN
    QN1S0F_ana=.TRUE.
    RETURN
  END FUNCTION QN1S0F_ANA

  ! for every number, it generates the helicity for (U_lamda,eps(L_z),eps(S_z))
  !*********************************************************
  !     ipol(k)=helicity,ipol(k)=2:rigth-handed            *
  !                      ipol(k)=1:left-handed             *
  !                      ipol(k)=3:long.                   *
  !*********************************************************
  SUBROUTINE Qeveryhel_ana(ihela,num,kk)  !num=ipol(kk)
    IMPLICIT NONE
    INTEGER,DIMENSION(3),INTENT(OUT)::ihela
    INTEGER,INTENT(IN)::num,kk
    INTEGER::ii1,jj1,kk1
    IF(kk.GT.nhad.OR.kk.LE.2)THEN
       PRINT *,"Error 1 in Qeveryhel_ana ! STOP !"
       STOP
    ENDIF
    IF(ABS(iflh(kk)).LE.100)THEN
       PRINT *,"Error 2 in Qeveryhel_ana ! STOP !"
       STOP
    ENDIF
    IF(iranhel.EQ.3)THEN
       ihela(1)=num
       ihela(2:3)=0
       RETURN
    ENDIF
    IF(QN1S0F_ana(kk))THEN
       ihela(1)=num
       ihela(2:3)=0
       RETURN
    ENDIF
    IF(QN3S1F_ana(kk))THEN
       ihela(2)=0
       ii1=MOD(num-1,2)
       jj1=(num-1)/2
       ihela(1)=ii1+1
       ihela(3)=jj1+1
       RETURN
    ENDIF
    IF(QN1P1F_ana(kk))THEN
       IF(iranhel.EQ.2)THEN
          ihela(1)=num
          ihela(2:3)=0
          RETURN
       ELSE
          ihela(3)=0
          ii1=MOD(num-1,2)
          jj1=(num-1)/2
          ihela(1)=ii1+1
          ihela(2)=jj1+1
          RETURN
       ENDIF
    ENDIF
    IF(QN3PJF_ana(kk))THEN
       IF(iranhel.EQ.2)THEN
          ihela(2)=0
          ii1=MOD(num-1,2)
          jj1=(num-1)/2
          ihela(1)=ii1+1
          ihela(3)=jj1+1
          RETURN
       ELSE
          ii1=MOD(num-1,2)
          jj1=MOD((num-1)/2,3)
          kk1=(num-1)/6
          ihela(1)=ii1+1
          ihela(2)=jj1+1
          ihela(3)=kk1+1
          RETURN
       ENDIF
    ENDIF
    ihela(1:3)=0
    RETURN
  END SUBROUTINE Qeveryhel_ana

  SUBROUTINE check_stop_cuts(n_track,p_track,pid_track,passcuts)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::n_track
    REAL(KIND(1d0)),DIMENSION(4,20),INTENT(IN)::p_track
    INTEGER,DIMENSION(20),INTENT(IN)::pid_track
    LOGICAL,INTENT(OUT)::passcuts
    INTEGER::ioniumjet,n_oniumjet_track,n_oniumjet_gluon,n_oniumjet_quark
    INTEGER,DIMENSION(-6:6)::n_oniumjet_quarkflavor
    INTEGER::n_oniumjet_gluonandqqbar,n_oniumjet_nonpairquark
    REAL(KIND(1d0)),DIMENSION(4,20)::p_oniumjet_track
    INTEGER::n_jet,n_onium
    REAL(KIND(1d0)),DIMENSION(4,20)::p_jet
    INTEGER,DIMENSION(20)::jetvec
    REAL(KIND(1d0))::pt1_oniumjet,pt2_oniumjet,z12,z1,d1,d2,dr,pi
    REAL(KIND(1d0))::ptjet_min,ptjet_max,ptjet,RRstop,ptonium,monium
    INTEGER::i,l
    passcuts=.TRUE.
    pi=DACOS(-1d0)
    n_onium=1
    ! make sure the last track is the onium
    IF(stop_jet_dyn_radius_ana.GT.0d0)THEN
       monium=p_track(4,n_track)**2-p_track(1,n_track)**2&
            -p_track(2,n_track)**2-p_track(3,n_track)**2
       monium=DSQRT(monium)
       ptonium=p_track(1,n_track)**2+p_track(2,n_track)**2
       ptonium=DSQRT(ptonium)
       RRstop=MAX(stop_jet_radius_ana,stop_jet_dyn_radius_ana*monium/ptonium)
    ELSE
       RRstop=stop_jet_radius_ana
    ENDIF
    IF(stop_maxrapjet_ana.GT.0d0)THEN
       CALL fastjetppgenkt_etamax(p_track,n_track,RRstop,stop_minptjet_ana,stop_maxrapjet_ana,&
            DBLE(stop_jetalg_ana),p_jet,n_jet,jetvec)
    ELSE
       CALL fastjetppgenkt(p_track,n_track,RRstop,stop_minptjet_ana,DBLE(stop_jetalg_ana),&
            p_jet,n_jet,jetvec)
    ENDIF
    IF(n_jet.LT.stop_min_n_jet_ana)THEN
       passcuts=.FALSE.
       RETURN
    ELSEIF(stop_max_n_jet_ana.GE.0.AND.n_jet.GT.stop_max_n_jet_ana)THEN
       passcuts=.FALSE.
       RETURN
    ENDIF
    ioniumjet=jetvec(n_track)
    IF(ioniumjet.LE.0)THEN
       passcuts=.FALSE.
       RETURN
    ENDIF
    n_oniumjet_track=0
    p_oniumjet_track(1:4,1:20)=0d0
    n_oniumjet_gluon=0
    n_oniumjet_quark=0
    n_oniumjet_quarkflavor(-6:6)=0
    DO i=1,n_track
       IF(jetvec(i).EQ.ioniumjet.AND.is_a_jet_ana_PDG(pid_track(i)))THEN
          n_oniumjet_track=n_oniumjet_track+1
          p_oniumjet_track(1:4,n_oniumjet_track)=p_track(1:4,i)
          IF(pid_track(i).EQ.21)THEN
             n_oniumjet_gluon=n_oniumjet_gluon+1
          ELSEIF(ABS(pid_track(i)).LE.6)THEN
             n_oniumjet_quark=n_oniumjet_quark+1
             n_oniumjet_quarkflavor(pid_track(i))=&
                          n_oniumjet_quarkflavor(pid_track(i))+1
          ENDIF
       ENDIF
    ENDDO
    n_oniumjet_gluonandqqbar=n_oniumjet_gluon
    n_oniumjet_nonpairquark=0
    DO i=1,6
       n_oniumjet_gluonandqqbar=n_oniumjet_gluonandqqbar+&
            MIN(n_oniumjet_quarkflavor(-i),n_oniumjet_quarkflavor(i))
       n_oniumjet_nonpairquark=n_oniumjet_nonpairquark+&
            ABS(n_oniumjet_quarkflavor(-i)-n_oniumjet_quarkflavor(i))
    ENDDO
    ! I do not know the flavours of the stop_n_frag_quark_ana
    ! choose the way to maximize n_oniumjet_gluonandqqbar
    ! subtract the ones needed by stop_n_frag_quark_ana
    IF(n_oniumjet_nonpairquark.LT.stop_n_frag_quark_ana)THEN
       n_oniumjet_gluonandqqbar=n_oniumjet_gluonandqqbar&
            -(stop_n_frag_quark_ana-n_oniumjet_nonpairquark+1)/2
    ENDIF
    IF(n_oniumjet_track.LT.(stop_n_frag_gluon_ana+stop_n_frag_quark_ana).OR.&
         n_oniumjet_gluonandqqbar.LT.stop_n_frag_gluon_ana.OR.&
         n_oniumjet_quark.LT.stop_n_frag_quark_ana)THEN
       passcuts=.FALSE.
       RETURN
    ELSE
       IF(n_oniumjet_track.GE.2)THEN
          ! check the soft drop cut
          DO i=1,n_oniumjet_track-1
             pt1_oniumjet=DSQRT(p_oniumjet_track(1,i)**2+p_oniumjet_track(2,i)**2)
             DO l=i+1,n_oniumjet_track
                pt2_oniumjet=DSQRT(p_oniumjet_track(1,l)**2+p_oniumjet_track(2,l)**2)
                z12=MIN(pt1_oniumjet,pt2_oniumjet)/MAX((pt1_oniumjet+pt2_oniumjet),1D-99)
                d1=prapidity(p_oniumjet_track(1:4,i))&
                     -prapidity(p_oniumjet_track(1:4,l))
                d2=ph4(p_oniumjet_track(1,i),p_oniumjet_track(2,i),p_oniumjet_track(3,i))&
                     -ph4(p_oniumjet_track(1,l),p_oniumjet_track(2,l),p_oniumjet_track(3,l))
                d2=MIN(DABS(d2),2*pi-DABS(d2))
                IF(d2/pi.GT.1.d0)WRITE(*,*)d2/pi
                dr=DSQRT(d1**2+d2**2)
                IF(z12.LE.stop_zcut_ana*(dr/RRstop)**stop_beta_ana)THEN
                   passcuts=.FALSE.
                   RETURN
                ENDIF
             ENDDO
          ENDDO
       ENDIF
       ! n_track-n_onium=i+k+o
       ! assume there is only one onium and one onium jet
       ! stop_min_n_jet-1 is the minimal number of parton jets
       ! If there are extra o radiations, the condition is to require
       ! i+k+o-m=n_track-n_onium-n_oniumjet_track
       IF(stop_min_n_jet_ana-1.LT.(n_track-n_onium-n_oniumjet_track))THEN
          IF(n_jet-1.LT.(n_track-n_onium-n_oniumjet_track))THEN
             passcuts=.FALSE.
             RETURN
          ENDIF
          IF(n_jet-1.GE.2.AND.stop_zasymcut_ana.GT.0d0)THEN
             ! apply the asymmetric cut                                                                                                               
             ptjet_min=-1d0
             ptjet_max=-1d0
             DO i=1,n_jet
                IF(i.EQ.ioniumjet)CYCLE
                ptjet=DSQRT(p_jet(1,i)**2+p_jet(2,i)**2)
                IF(ptjet_min.LT.0d0.OR.ptjet_min.GT.ptjet)THEN
                   ptjet_min=ptjet
                ENDIF
                IF(ptjet_max.LT.0d0.OR.ptjet_max.LT.ptjet)THEN
                   ptjet_max=ptjet
                ENDIF
             ENDDO
             IF(ptjet_min/ptjet_max.LE.stop_zasymcut_ana)THEN
                passcuts=.FALSE.
                RETURN
             ENDIF
          ENDIF
       ENDIF
       ! to pass the soft cut for 3S1[8]
       ! k=stop_n_frag_gluon+stop_n_frag_quark
       IF(n_oniumjet_track.GT.0.AND.(stop_n_frag_gluon_ana+stop_n_frag_quark_ana).EQ.0)THEN
          DO i=1,n_oniumjet_track
             ! Take energy fraction
             z1=p_oniumjet_track(4,i)/p_jet(4,ioniumjet)
             ! zsoftcut is divided by the number of partons inside the onium jet
             IF(z1.LE.stop_zsoftcut_ana/DBLE(n_oniumjet_track))THEN
                passcuts=.FALSE.
                RETURN
             ENDIF
          ENDDO
       ENDIF
    ENDIF
    RETURN
  END SUBROUTINE check_stop_cuts

  FUNCTION pdgt_ana(ifif)
    ! from HELAC-Onia notation to PDG notation
    INTEGER,INTENT(IN)::ifif
    INTEGER::pdgt_ana,init=0,ifac
    INTEGER,DIMENSION(-50:50)::ipdt
    SAVE ipdt,init,ifac
    IF(init.EQ.0)THEN
       ipdt( 1)= 12
       ipdt(-1)=-12
       ipdt( 5)= 14
       ipdt(-5)=-14
       ipdt( 9)= 16
       ipdt(-9)=-16
       ipdt( 2)= 11
       ipdt(-2)=-11
       ipdt( 6)= 13
       ipdt(-6)=-13
       ipdt( 10)= 15
       ipdt(-10)=-15
       ipdt( 3)= 2
       ipdt(-3)=-2
       ipdt( 4)= 1
       ipdt(-4)=-1
       ipdt( 7)= 4
       ipdt(-7)=-4
       ipdt( 8)= 3
       ipdt(-8)=-3
       ipdt( 11)= 6
       ipdt(-11)=-6
       ipdt( 12)= 5
       ipdt(-12)=-5
       ipdt(35)=21
       ipdt(31)=22
       ipdt(32)=23
       ipdt(33)=24
       ipdt(34)=-24
       ipdt(41)=25
       init=1
    ENDIF
    IF(ABS(ifif).LT.100)THEN
       pdgt_ana=ipdt(ifif)
       RETURN
    ENDIF
    ifac=1
    IF(ifif.LT.0)ifac=-1
    SELECT CASE(ABS(ifif))
       ! J/psi
    CASE(443011)
       pdgt_ana=443
       ! etac
    CASE(441001)
       pdgt_ana=441
       ! hc
    CASE(441111)
       pdgt_ana=10443
       ! chic0
    CASE(443101)
       pdgt_ana=10441
       ! chic1
    CASE(443111)
       pdgt_ana=20443
       ! chic2
    CASE(443121)
       pdgt_ana=445
       ! upsilon
    CASE(553011)
       pdgt_ana=553
       ! etab
    CASE(551001)
       pdgt_ana=551
       ! hb
    CASE(551111)
       pdgt_ana=10553
       ! chib0
    CASE(553101)
       pdgt_ana=10551
       ! chib1
    CASE(553111)
       pdgt_ana=20553
       ! chib2
    CASE(553121)
       pdgt_ana=555
       ! Bc*+-
    CASE(453011)
       pdgt_ana=543*ifac
       ! Bc+-
    CASE(451001)
       pdgt_ana=541*ifac
       ! Bc2*+-
    CASE(453121)
       pdgt_ana=545*ifac
       ! Bc0*+-
    CASE(453101)
       pdgt_ana=10541*ifac
       ! Bc1(H)+-
    CASE(453111)
       pdgt_ana=20543*ifac
       ! ccbar(3P0(8))
    CASE(443108)
       pdgt_ana=9910441
       ! bbbar(3P0(8))
    CASE(553108)
       pdgt_ana=9910551
       ! bbbar(3S1(8))
    CASE(553018)
       pdgt_ana=9900553
       ! bbbar(1S0(8))
    CASE(551008)
       pdgt_ana=9900551
       ! ccbar(3S1(8))
    CASE(443018)
       pdgt_ana=9900443
       ! ccbar(1S0(8))
    CASE(441008)
       pdgt_ana=9900441
    CASE DEFAULT
       pdgt_ana=ifif
    END SELECT
    RETURN
  END FUNCTION pdgt_ana

  SUBROUTINE GetJz_ana(sz,JJ,Jz)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::sz,JJ
    INTEGER,INTENT(OUT)::Jz
    REAL(KIND(1d0))::rrr
    ! use Eq.(29) in arXiv:1209.4610
    IF(ABS(sz).GT.1)THEN
       WRITE(*,*)"ERROR: invalid sz (0,+-1) !"
       STOP
    ENDIF
    rrr=rand()
    IF(JJ.EQ.1)THEN
       IF(sz.EQ.-1)THEN
          IF(rrr.GT.0.5d0)THEN
             Jz=0
          ELSE
             Jz=-1
          ENDIF
       ELSEIF(sz.EQ.1)THEN
          IF(rrr.GT.0.5d0)THEN
             Jz=1
          ELSE
             Jz=0
          ENDIF
       ELSEIF(sz.EQ.0)THEN
          IF(rrr.GT.0.5d0)THEN
             Jz=1
          ELSE
             Jz=-1
          ENDIF
       ENDIF
    ELSEIF(JJ.EQ.2)THEN
       IF(sz.EQ.-1)THEN
          IF(rrr.LT.0.6d0)THEN
             Jz=-2
          ELSEIF(rrr.GT.0.6d0.AND.rrr.LT.0.9d0)THEN
             Jz=-1
          ELSE
             Jz=0
          ENDIF
       ELSEIF(sz.EQ.1)THEN
          IF(rrr.LT.0.6d0)THEN
             Jz=2
          ELSEIF(rrr.GT.0.6d0.AND.rrr.LT.0.9d0)THEN
             Jz=1
          ELSE
             Jz=0
          ENDIF
       ELSE
          IF(rrr.LT.0.3d0)THEN
             Jz=-1
          ELSEIF(rrr.GT.0.3d0.AND.rrr.LT.0.7d0)THEN
             Jz=0
          ELSE
             Jz=1
          ENDIF
       ENDIF
    ELSE
       WRITE(*,*)"ERROR: invalid J (1 or 2) !"
       STOP
    ENDIF
    RETURN
  END SUBROUTINE GetJz_ana

  ! unpolarized two body decay
  ! particle one (MM) -> daugther one (MD1)  daughter two (M=0) 
  SUBROUTINE TwoBodyDecay_unpol_ana(PM,MM,MD1,PD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PM
    REAL(KIND(1d0)),INTENT(IN)::MM,MD1
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PD1
    REAL(KIND(1d0))::mass,mass2,ppp,sinth,costh,phi
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    mass=DSQRT(PM(4)**2-PM(1)**2-PM(2)**2-PM(3)**2)
    mass2=mass*MD1/MM
    ppp=(mass**2-mass2**2)/(2d0*mass)
    costh=2d0*(rand()-0.5d0)
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=2d0*pipi*rand()
    PD1(4)=DSQRT(ppp**2+mass2**2)
    PD1(1)=-ppp*DCOS(phi)*sinth
    PD1(2)=-ppp*DSIN(phi)*sinth
    PD1(3)=-ppp*costh
    ! boost to the lab frame
    CALL BOOSTL(mass,PM,PD1)
    RETURN
  END SUBROUTINE TwoBodyDecay_unpol_ana

  ! spin-correlated decay of chi_c1 -> J/psi+a
  ! mother particle one (MM) -> daugther one (MD1)  daughter two (M=0)
  ! Jz of mother partical is "spin"
  ! check the formalism in arXiv:1412.2576
  SUBROUTINE HO_chi12psia_hel_ana(PM,MM,MD1,spin,PD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PM
    REAL(KIND(1d0)),INTENT(IN)::MM,MD1
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PD1
    INTEGER,INTENT(IN)::spin ! Jz in the helicity frame
    ! use CLEO measurement arXiv:0910.0046
    REAL(KIND(1d0)),PARAMETER::aJ10=0.998038696644574d0,aJ11=-0.0626d0
    REAL(KIND(1d0))::rrr,rrr2,costh,sinth,phi
    INTEGER::init=0,i,j,k
    SAVE init
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
    REAL(KIND(1d0)),DIMENSION(4)::fac1
    SAVE fac1
    REAL(KIND(1d0)),DIMENSION(4)::fac2
    SAVE fac2
    REAL(KIND(1d0)),DIMENSION(4)::fac3
    SAVE fac3
    REAL(KIND(1d0))::mass,mass2,ppp
    REAL(KIND(1d0)),DIMENSION(3)::frame
    IF(init.EQ.0)THEN
       fac1(1)=(1 + 2*aJ10*aJ11)/4.
       fac1(2)=0.5
       fac1(3)=(3 - 2*aJ10*aJ11)/4.
       fac1(4)=1
       fac2(1)=(1 + 2*aJ10*aJ11)/4.
       fac2(2)=0.5
       fac2(3)=(3 - 2*aJ10*aJ11)/4.
       fac2(4)=1
       fac3(1)=(1 + 2*aJ10*aJ11)/4.
       fac3(2)=0.5
       fac3(3)=(3 - 2*aJ10*aJ11)/4.
       fac3(4)=1
       init=1
     ENDIF
     SELECT CASE(spin)
     CASE(-1)
        rrr=rand()
        k=1
        DO i=1,4
           IF(rrr.LE.fac1(i))THEN
              k=i
              EXIT
           ENDIF
           IF(i.EQ.4)THEN
              k=4
              EXIT
           ENDIF
        ENDDO
        SELECT CASE(k)
        CASE(1)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(2)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=1-rrr2**(1d0/3d0)
        CASE(3)
           ! decay via -((-1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=rrr2**(1d0/3d0)-1
        CASE(4)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        END SELECT
     CASE(0)
        rrr=rand()
        k=1
        DO i=1,4
           IF(rrr.LE.fac2(i))THEN
              k=i
              EXIT
           ENDIF
           IF(i.EQ.4)THEN
              k=4
              EXIT
           ENDIF
        ENDDO
        SELECT CASE(k)
        CASE(1)
           ! decay via ((1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13))**2)/4.
           rrr2=2d0/3d0*rand()-1d0/3d0
           IF(rrr2.GE.0d0)THEN
              costh=(rrr2*3d0)**(1d0/3d0)
           ELSE
              costh=-(-rrr2*3d0)**(1d0/3d0)
           ENDIF
        CASE(2)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(3)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(4)
           ! decay via ((1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13))**2)/4.
           rrr2=2d0/3d0*rand()-1d0/3d0
           IF(rrr2.GE.0d0)THEN
              costh=(rrr2*3d0)**(1d0/3d0)
           ELSE
              costh=-(-rrr2*3d0)**(1d0/3d0)
           ENDIF
        END SELECT
     CASE(1)
        rrr=rand()
        k=1
        DO i=1,4
           IF(rrr.LE.fac3(i))THEN
              k=i
              EXIT
           ENDIF
           IF(i.EQ.4)THEN
              k=4
              EXIT
           ENDIF
        ENDDO
        SELECT CASE(k)
        CASE(1)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(2)
           ! decay via -((-1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=rrr2**(1d0/3d0)-1
        CASE(3)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=1-rrr2**(1d0/3d0)
        CASE(4)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        END SELECT
     END SELECT
     sinth=DSQRT(ABS(1d0-costh**2))
     phi=2*pi*rand()
     mass=DSQRT(PM(4)**2-PM(1)**2-PM(2)**2-PM(3)**2)
     mass2=mass*MD1/MM
     ppp=(mass**2-mass2**2)/(2d0*mass)
     PD1(4)=DSQRT(ppp**2+mass2**2)
     PD1(1)=-ppp*DCOS(phi)*sinth
     PD1(2)=-ppp*DSIN(phi)*sinth
     PD1(3)=-ppp*costh
     frame(1:3)=PM(1:3)
     ! rotate to make the z axis is the flight of mother particle
     CALL ROTATEL(frame,PD1)
     CALL BOOSTL(mass,PM,PD1)
     RETURN
  END SUBROUTINE HO_chi12psia_hel_ana


  ! spin-correlated decay of chi_c2 -> J/psi+a
  ! mother particle one (MM) -> daugther one (MD1)  daughter two (M=0)
  ! Jz of mother partical is "spin"
  ! check the formalism in arXiv:1412.2576
  SUBROUTINE HO_chi22psia_hel_ana(PM,MM,MD1,spin,PD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PM
    REAL(KIND(1d0)),INTENT(IN)::MM,MD1
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PD1
    INTEGER,INTENT(IN)::spin ! Jz in the helicity frame
    ! use CLEO measurement arXiv:0910.0046
    REAL(KIND(1d0)),PARAMETER::aJ21=0.9956661086930698d0,aJ22=-0.093d0,aJ23=0d0
    REAL(KIND(1d0))::mass,mass2,ppp
    REAL(KIND(1d0)),DIMENSION(3)::frame
    REAL(KIND(1d0))::rrr,rrr2,costh,sinth,phi,prob,delta0,delta1,fmax
    INTEGER::init=0,i,j,k
    SAVE init
    LOGICAL::lexit
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
    REAL(KIND(1d0)),DIMENSION(6)::fac1
    SAVE fac1
    REAL(KIND(1d0)),DIMENSION(6)::fac2
    SAVE fac2
    REAL(KIND(1d0)),DIMENSION(6)::fac3
    SAVE fac3
    REAL(KIND(1d0)),DIMENSION(6)::fac4
    SAVE fac4
    REAL(KIND(1d0)),DIMENSION(6)::fac5
    SAVE fac5
    IF(init.EQ.0)THEN
       delta0=1d0+2d0*aJ21*(DSQRT(5d0)*aJ22+2d0*aJ23)&
            +4d0*aJ22*(aJ22+DSQRT(5d0)*aJ23)+3d0*aJ23**2
       delta0=delta0/10d0
       delta1=9d0+6d0*aJ21*(DSQRT(5d0)*aJ22-4d0*aJ23)&
            -4d0*aJ22*(aJ22+2d0*DSQRT(5d0)*aJ23)+7d0*aJ23**2
       delta1=delta1/30d0
       fac1(1)=delta0/2.
       fac1(2)=(1 - delta1)/2.
       fac1(3)=0.5
       fac1(4)=(1 + delta1)/2.
       fac1(5)=1 - delta0/2.
       fac1(6)=1
       fac2(1)=delta0/2.
       fac2(2)=(1 - delta1)/2.
       fac2(3)=0.5
       fac2(4)=(1 + delta1)/2.
       fac2(5)=1 - delta0/2.
       fac2(6)=1
       fac3(1)=delta0/2.
       fac3(2)=(1 - delta1)/2.
       fac3(3)=0.5
       fac3(4)=(1 + delta1)/2.
       fac3(5)=1 - delta0/2.
       fac3(6)=1
       fac4(1)=delta0/2.
       fac4(2)=(1 - delta1)/2.
       fac4(3)=0.5
       fac4(4)=(1 + delta1)/2.
       fac4(5)=1 - delta0/2.
       fac4(6)=1
       fac5(1)=delta0/2.
       fac5(2)=(1 - delta1)/2.
       fac5(3)=0.5
       fac5(4)=(1 + delta1)/2.
       fac5(5)=1 - delta0/2.
       fac5(6)=1
       init=1
    ENDIF
    SELECT CASE(spin)
    CASE(-2)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac1(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Cos(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(-1)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac2(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*(1 + 2*Cos(Subscript(\[Theta],13)))**2*Sin(Subscript(\[Theta],13)/2.)**4)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+2d0*costh)**2*(1d0-costh)**2/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*Cos(Subscript(\[Theta],13)/2.)**4*(1 - 2*Cos(Subscript(\[Theta],13)))**2)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**2/4d0*(1d0-2d0*costh)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(0)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac3(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (delta0*(1 + 3*Cos(2*Subscript(\[Theta],13)))**2)/32.
          fmax=16d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(-2d0+6d0*costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via (-3*(-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (3*delta1*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (3*delta1*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via (-3*(-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (delta0*(1 + 3*Cos(2*Subscript(\[Theta],13)))**2)/32.
          fmax=16d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(-2d0+6d0*costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(1)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac4(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*Cos(Subscript(\[Theta],13)/2.)**4*(1 - 2*Cos(Subscript(\[Theta],13)))**2)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**2/4d0*(1d0-2d0*costh)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*(1 + 2*Cos(Subscript(\[Theta],13)))**2*Sin(Subscript(\[Theta],13)/2.)**4)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+2d0*costh)**2*(1d0-costh)**2/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(2)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac5(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Cos(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    END SELECT
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=2*pi*rand()
    mass=DSQRT(PM(4)**2-PM(1)**2-PM(2)**2-PM(3)**2)
    mass2=mass*MD1/MM
    ppp=(mass**2-mass2**2)/(2d0*mass)
    PD1(4)=DSQRT(ppp**2+mass2**2)
    PD1(1)=-ppp*DCOS(phi)*sinth
    PD1(2)=-ppp*DSIN(phi)*sinth
    PD1(3)=-ppp*costh
    frame(1:3)=PM(1:3)
    ! rotate to make the z axis is the flight of mother particle
    CALL ROTATEL(frame,PD1)
    CALL BOOSTL(mass,PM,PD1)
    RETURN
  END SUBROUTINE HO_chi22psia_hel_ana

  FUNCTION SOLVEVll00_ana(rrr)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::rrr
    REAL(KIND(1d0))::SOLVEVll00_ana
    REAL(KIND(1d0))::xr,xi
    COMPLEX(KIND(1d0))::x
    x=DCMPLX(rrr)+DCMPLX(0d0,1d0)*SQRT(1d0-rrr**2)
    x=x**(1d0/3d0)
    xr=DREAL(x)
    xi=DIMAG(x)
    IF(ABS(xr).LE.0.5d0+1d-99)THEN
       SOLVEVll00_ana=-2d0*xr
       SOLVEVll00_ana=DSIGN(MIN(ABS(SOLVEVll00_ana),1d0),SOLVEVll00_ana)
    ELSEIF(ABS(xr+DSQRT(3d0)*xi).LE.1d0+1d-99)THEN
       SOLVEVll00_ana=xr+DSQRT(3d0)*xi
       SOLVEVll00_ana=DSIGN(MIN(ABS(SOLVEVll00_ana),1d0),SOLVEVll00_ana)
    ELSE
       SOLVEVll00_ana=xr-DSQRT(3d0)*xi
       SOLVEVll00_ana=DSIGN(MIN(ABS(SOLVEVll00_ana),1d0),SOLVEVll00_ana)
    ENDIF
    RETURN
  END FUNCTION SOLVEVll00_ana

END MODULE plot_user
