MODULE plot_user
!============================================================
!
! PLOT FILE FOR USER
! p p > psi + n jet in 0.0<y<4.5
! STOP cuts (arXiv:1809.02369)
! with Non-perturbative effects
!
! WARNING WARNING WARNING WARNING WARNING WARNING WARNING
!
! To change max_plots in analysis/hbook/HwU.inc
! To change max_histograms in analysis/hbook/histograms_common.inc
! To change max_histograms in analysis/hbook/histograms_common90.inc
! They should be at least larger than num_plots below
! Recompile from the scratch after changing above parameters 
!
! WARNING WARNING WARNING WARNING WARNING WARNING WARNING
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=649 ! note to change max_plots in analysis/hbook/HwU.inc and analysis/hbook/HwU90.inc
  ! to change max_histograms in analysis/hbook/histograms_common.inc and analysis/hbook/histograms_common90.inc
  INTEGER,PARAMETER::n_width=1,n_height=1
  ! stop cuts for analysis
  INTEGER::stop_min_n_jet_ana,stop_max_n_jet_ana,stop_jetalg_ana
  INTEGER::stop_n_frag_gluon_ana,stop_n_frag_quark_ana
  REAL(KIND(1d0))::stop_maxrapjet_ana,stop_jet_radius_ana,stop_minptjet_ana
  REAL(KIND(1d0))::stop_zcut_ana,stop_beta_ana,stop_zsoftcut_ana
  REAL(KIND(1d0))::stop_zasymcut_ana,stop_jet_dyn_radius_ana
  LOGICAL,DIMENSION(4,3,4)::need_histogram
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,jj,kk,l,ll,mm,nn
    ! J/psi momentum from different final states
    CHARACTER(len=7),DIMENSION(6)::fstates=(/'FO     ','Psi(2S)','Chic0  ',&
         'Chic1  ','Chic2  ','J/Psi  '/)
    ! stop cuts dependence
    ! arXiv:1809.02369
    CHARACTER(len=4),DIMENSION(2)::minjpt=(/'ptj3,zasym01','ptj6,zasym07'/)
    ! radiation function dependence
    ! Eq.(2.22) in hep-ph/0001062
    CHARACTER(len=4),DIMENSION(4)::radiations=(/'L0.0','L0.2','L0.3','L0.5'/)
    CHARACTER(len=4),DIMENSION(3)::momreshuf=(/'NORS','ISRS','FSRS'/)
    ! chic or psi(2S) decay pattern
    ! D1: p[chic/psi(2S)] = p[J/psi]
    ! D2: p[chic/psi(2S)]*M[J/psi]/M[chic/psi(2S)] = p[J/psi]
    ! D3: p[J/psi] is from p[chic/psi(2S)] via spin uncorrelated decay
    ! D4: p[J/psi] is from p[chic/psi(2S)] via spin correlated decay
    CHARACTER(len=2),DIMENSION(4)::decayway=(/'D1','D2','D3','D4'/)
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(emep_ISR_shower.EQ.1)THEN
       WRITE(*,*)"Please change a plot_user.f90 !!"
       STOP
    ENDIF
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    l=0
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    ! enlarge max_plots in ../hbook/HwU.inc
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    CALL HISTOGRAM_book(l+ 1,'total rate  ',&
         5,0.5d0,5.5d0,.FALSE.)
    need_histogram(1:4,1:3,1:4)=.FALSE.
    DO kk=1,4
       ! Lambda values in radiation function 
       DO ll=1,3
          ! perfoming momentum reshuffling (or not)
          DO mm=1,4
             ! shifting momentum from excited states
             IF(mm.EQ.1)THEN
                ! D1: L != 0 + ISRS/FSRS; L = 0 + NORS/ISRS/FSRS
                IF(kk.GT.1.AND.ll.GT.1)THEN
                   need_histogram(kk,ll,mm)=.TRUE.
                ELSEIF(kk.EQ.1)THEN
                   need_histogram(kk,ll,mm)=.TRUE.
                ENDIF
             ELSEIF(mm.EQ.2)THEN
                ! D2: L = 0 + NORS
                IF(kk.EQ.1.AND.ll.EQ.1)THEN
                   need_histogram(kk,ll,mm)=.TRUE.
                ENDIF
             ELSEIF(mm.EQ.4)THEN
                ! D4: any L + ISRS/FSRS
                IF(ll.GT.1)THEN
                   need_histogram(kk,ll,mm)=.TRUE.
                ENDIF
             ENDIF
          ENDDO
       ENDDO
    ENDDO
    l=1
    DO i=1,2
       ! pt cut on the jet, stop_minptjet
       DO kk=1,4
          ! Lambda values in radiation function
          DO ll=1,3
             ! perfoming momentum reshuffling (or not)
             DO mm=1,4
                ! shifting momentum from excited states
                IF(.NOT.need_histogram(kk,ll,mm))CYCLE
                DO nn=1,6
                   ! J/psi momentum from different final states
                   CALL HISTOGRAM_book(l+ 1,'pt of '//fstates(nn)//' in |y|<0.6 '//minjpt(i)&
                        //' '//radiations(kk)//' '//momreshuf(ll)//' '//decayway(mm),&
                        100,5d0,105d0,.TRUE.)
                   CALL HISTOGRAM_book(l+ 2,'pt of '//fstates(nn)//' in |y|<0.75 '//minjpt(i)&
                        //' '//radiations(kk)//' '//momreshuf(ll)//' '//decayway(mm),&
                        100,5d0,105d0,.TRUE.)
                   CALL HISTOGRAM_book(l+ 3,'pt of '//fstates(nn)//' in 0.75<|y|<1.5 '//minjpt(i)&
                        //' '//radiations(kk)//' '//momreshuf(ll)//' '//decayway(mm),&
                        100,5d0,105d0,.TRUE.)
                   l=l+3
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    
    CALL HISTOGRAM_BOOK_FINISH

10203 FORMAT(E9.3,"<pt<",E9.3)
10204 FORMAT(E9.3)
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts0)
    USE Helac_Global
    USE Structf_PDFs
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts0
    REAL(KIND(1d0)),DIMENSION(SIZE(wgts0))::wgts,wgts_Final
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l,j,jj,ll
    REAL(KIND(1d0))::www,yy,pt
    INTEGER::n_track,n_jet,n_track2,n_jet2
    REAL(KIND(1d0)),DIMENSION(4,20)::p_track,p_track2
    REAL(KIND(1d0)),DIMENSION(4,20)::p_jet
    INTEGER,DIMENSION(20)::jetvec
    INTEGER::NJET
    REAL(KIND(1d0)),DIMENSION(20,4)::PJET
    INTEGER,DIMENSION(3)::index2
    INTEGER::fockstate,imoth,isign,JzJz
    REAL(KIND(1d0))::rrr
    LOGICAL,DIMENSION(2)::pass_stop_cuts
    INTEGER,DIMENSION(20)::pid_track
    REAL(KIND(1d0)),DIMENSION(2)::new_stop_minptjet
    REAL(KIND(1d0)),DIMENSION(2)::new_stop_zasymcut
    REAL(KIND(1d0)),DIMENSION(4)::radiation_lam
    ! the masses of psi(2S), chi_c0, chi_c1, chi_c2, J/psi
    REAL(KIND(1d0)),DIMENSION(2:6),PARAMETER::masses=(/3.686d0,3.415d0,3.511d0,3.556d0, 3.097d0/)
    ! the branching ratios into J/psi
    REAL(KIND(1d0)),DIMENSION(2:5),PARAMETER::BRList=(/0.574d0,0.0128d0,0.36d0,0.2d0/)
    LOGICAL,DIMENSION(2:6)::fo2state ! to judge whether the Fock state will contribute to the charmonium
    REAL(KIND(1d0)),DIMENSION(2:6,4)::P_JPsi ! the J/psi momentum from different charmonium
    REAL(KIND(1d0)),DIMENSION(nhad+1,4)::POUT1,POUT2
    REAL(KIND(1d0)),DIMENSION(2:6,nhad+1,4)::POUT_RS
    INTEGER::NPtotal
    LOGICAL::coloroctet,twomomkeep
    REAL(KIND(1d0))::rad_bn,rad_c,rad_c2,x1new,x2new,wwsf,wwsf0,x1orig,x2orig
    REAL(KIND(1d0)),DIMENSION(2:6)::wgt_RS,wgt_Final
    REAL(KIND(1d0)),DIMENSION(nhad+1)::MA_IN
    REAL(KIND(1d0)),DIMENSION(4)::Ponium,PX
    REAL(KIND(1d0))::ptFO,KFactor
    IF(wgts0(1).EQ.0d0)RETURN
    !IF(NDecayChains.GT.0)THEN
    !   CALL plot_fill_Decay(wgts)
    !   RETURN
    !ENDIF
    !WRITE(*,*)"ERROR:events without decay is not supported !"
    !STOP
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    ! 
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    ! preparing the setup
    new_stop_minptjet(1)=3d0
    new_stop_zasymcut(1)=0.1d0
    new_stop_minptjet(2)=6d0
    new_stop_zasymcut(2)=0.7d0
    radiation_lam(1)=0d0
    radiation_lam(2)=0.2d0
    radiation_lam(3)=0.3d0
    radiation_lam(4)=0.5d0
    fo2state(2:6)=.FALSE.

    kk=1
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          ! get the last fock state
          IF(calc_LDME_RG_CT)THEN
             fockstate=iflh_CT(i)
          ELSE
             fockstate=iflh(i)
          ENDIF
          imoth=i
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          kk=kk+1
       ENDIF
    ENDDO
    coloroctet=MOD(fockstate,10).EQ.8
    twomomkeep=.FALSE.
    ! same as Eq.(2.30), Eq.(2.31) in hep-ph/0001062
    IF(fockstate.EQ.441008)THEN
       rad_bn=2d0 ! because 1S0[8] -> J/psi, psi(2S) has one M1 transition
       rad_c=1d0
       rad_c2=-1d0
    ELSEIF(fockstate.EQ.443018)THEN
       rad_bn=0d0
       rad_c=1.5d0 ! the first choice is 3S1[8] -> J/psi,psi(2S), which has two E1 transitions
       rad_c2=1d0 ! the second choice is for 3S1[8] -> chi_c, which only has one E1 transition
       twomomkeep=.TRUE.
    ELSEIF(coloroctet)THEN
       rad_bn=0d0 ! because others to J/psi has one E1 transition
       rad_c=1d0
       rad_c2=-1d0
    ENDIF
    IF(fockstate.EQ.443018)THEN
       fo2state(2:6)=.TRUE.
    ELSEIF(fockstate.EQ.443011.OR.fockstate.EQ.441008.OR.&
         fockstate.EQ.443108.OR.fockstate.EQ.443118.OR.&
         fockstate.EQ.443128)THEN
       fo2state(2)=.TRUE.
       fo2state(6)=.TRUE.
    ELSEIF(fockstate.EQ.443101)THEN
       fo2state(3)=.TRUE.
    ELSEIF(fockstate.EQ.443111)THEN
       fo2state(4)=.TRUE.
    ELSEIF(fockstate.EQ.443121)THEN
       fo2state(5)=.TRUE.
    ELSE
       WRITE(*,*)"ERROR: the analysis file cannot deal with fock state = ",fockstate
       STOP
    ENDIF
    CALL strf_pdf(wwsf0)
    ! apply the K factors
    ! istate:
    ! 1: 3S11
    ! 2: 3S18
    ! 3: 1S08
    ! 4: 3PJ8
    ! 5: 3P01
    ! 6: 3P11
    ! 7: 3P21
    ptFO=DSQRT(phad(imoth,1)**2+phad(imoth,2)**2)
    IF(fockstate.EQ.443011)THEN
       CALL KFactorLPNLO(1,ptFO,KFactor) 
    ELSEIF(fockstate.EQ.443018)THEN
       CALL KFactorLPNLO(2,ptFO,KFactor)
    ELSEIF(fockstate.EQ.441008)THEN
       CALL KFactorLPNLO(3,ptFO,KFactor)
    ELSEIF(fockstate.EQ.443108.OR.fockstate.EQ.443118&
         .OR.fockstate.EQ.443128)THEN
       CALL KFactorLPNLO(4,ptFO,KFactor)
    ELSEIF(fockstate.EQ.443101)THEN
       CALL KFactorLPNLO(5,ptFO,KFactor)
    ELSEIF(fockstate.EQ.443111)THEN
       CALL KFactorLPNLO(6,ptFO,KFactor)
    ELSEIF(fockstate.EQ.443121)THEN
       CALL KFactorLPNLO(7,ptFO,KFactor)
    ELSE
       KFactor=1d0
    ENDIF
    wgts(1:SIZE(wgts0))=wgts0(1:SIZE(wgts0))*KFactor

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! CHECK WHETHER PASS THE NEW STOP CUTS
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    n_track=0
    n_jet=0
    p_track(1:4,1:20)=0d0
    DO i=3,nhad
       IF(is_a_jet_ana(iflh(i)))THEN
          n_track=n_track+1
          p_track(1:4,n_track)=phad(i,1:4)
          pid_track(n_track)=pdgt_ana(iflh(i))
       ENDIF
    ENDDO
    ! always put J/psi at the last
    n_track2=n_track+1
    n_jet2=0
    p_track2(1:4,1:20)=0d0
    p_track2(1:4,1:n_track)=p_track(1:4,1:n_track)
    p_track2(1:4,n_track2)=phad(imoth,1:4)
    pid_track(n_track2)=iflh(imoth)
    pass_stop_cuts(1:2)=.TRUE.
    IF(use_stop_cut)THEN
       ! check the stop cuts
       stop_min_n_jet_ana=stop_min_n_jet
       stop_max_n_jet_ana=stop_max_n_jet
       stop_jetalg_ana=stop_jetalg
       stop_n_frag_gluon_ana=stop_n_frag_gluon
       stop_n_frag_quark_ana=stop_n_frag_quark
       stop_maxrapjet_ana=stop_maxrapjet
       stop_jet_radius_ana=stop_jet_radius
       stop_jet_dyn_radius_ana=stop_jet_dyn_radius
       stop_minptjet_ana=stop_minptjet
       stop_zcut_ana=stop_zcut
       stop_beta_ana=stop_beta
       stop_zsoftcut_ana=stop_zsoftcut
       stop_zasymcut_ana=stop_zasymcut
       DO i=1,2
          stop_minptjet_ana=new_stop_minptjet(i)
          IF(i.EQ.1)THEN
             pass_stop_cuts(i)=.TRUE.
          ELSE
             stop_zasymcut_ana=new_stop_zasymcut(i)
             CALL check_stop_cuts(n_track2,p_track2,pid_track,pass_stop_cuts(i))
          ENDIF
       ENDDO
    ENDIF

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    CALL HISTOGRAM_fill(1,1d0,wgts) ! total cross sections

    ll=1
    DO i=1,2
       ! pt cut on the jet, new_stop_minptjet(i) and new_stop_zasymcut(i)
       ! check whether it passes the (new) stop cuts
       IF(.NOT.pass_stop_cuts(i))CYCLE
       DO kk=1,4
          ! Lambda values radiation_lam(kk) in radiation function
          IF(radiation_lam(kk).LE.0d0.OR..NOT.coloroctet)THEN
             ! we will not apply the radiation function
             NPtotal=nhad
             POUT1(1:nhad,1:4)=phad(1:nhad,1:4)
             IF(twomomkeep)THEN
                POUT2(1:nhad,1:4)=phad(1:nhad,1:4)
             ENDIF
          ELSE
             ! it is a subroutine defined in Kinetic_Func.f90
             ! it tries to perform the radiation function via Eq.(2.22) in hep-ph/0001062
             CALL OniumRadiationFunction(rad_c*radiation_lam(kk),rad_bn,&
                  phad(imoth,1:4),Ponium,PX)
             NPtotal=nhad+1
             POUT1(1:nhad,1:4)=phad(1:nhad,1:4)
             POUT1(imoth,1:4)=Ponium(1:4)
             POUT1(nhad+1,1:4)=PX(1:4)
             IF(twomomkeep)THEN
                ! second radiation for 3S1[8] -> chi_c
                CALL OniumRadiationFunction(rad_c2*radiation_lam(kk),rad_bn,&
                  phad(imoth,1:4),Ponium,PX)
                POUT2(1:nhad,1:4)=phad(1:nhad,1:4)
                POUT2(imoth,1:4)=Ponium(1:4)
                POUT2(nhad+1,1:4)=PX(1:4)
             ENDIF
          ENDIF
          DO l=1,3
             ! perform momentum reshuffling
             ! No, ISR, FSR
             IF(l.EQ.1)THEN
                ! no reshuffling
                DO jj=2,6
                   IF(twomomkeep.AND.fo2state(jj))THEN
                      POUT_RS(jj,1:NPtotal,1:4)=POUT2(1:NPtotal,1:4)
                   ELSE
                      POUT_RS(jj,1:NPtotal,1:4)=POUT1(1:NPtotal,1:4)
                   ENDIF
                   wgt_RS(jj)=1d0
                ENDDO
             ELSEIF(l.EQ.2.OR.l.EQ.3)THEN
                ! Initial/Final state reshuffling
                MA_IN(1:nhad+1)=-1d0
                DO jj=2,6
                   IF(fo2state(jj))THEN
                      MA_IN(imoth)=masses(jj)
                      IF(twomomkeep.AND.jj.GE.3.AND.jj.LE.5)THEN
                         ! it is 3S1[8] -> chi_c
                         IF(l.EQ.2)THEN
                            ! Initial state reshuffling
                            CALL InitialStateReshuffling(NPtotal,POUT2(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ELSE
                            ! Final state reshuffling
                            CALL FinalStateReshuffling(NPtotal,POUT2(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ENDIF
                      ELSE
                         IF(l.EQ.2)THEN
                            ! Initial state reshuffling
                            CALL InitialStateReshuffling(NPtotal,POUT1(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ELSE
                            ! Final state reshuffling
                            CALL FinalStateReshuffling(NPtotal,POUT1(1:NPtotal,1:4),MA_IN(1:NPtotal),&
                                 xp1,xp2,POUT_RS(jj,1:NPtotal,1:4),x1new,x2new)
                         ENDIF
                      ENDIF
                      IF(x1new.NE.xp1.OR.x2new.NE.xp2)THEN
                         x1orig=xp1
                         x2orig=xp2
                         xp1=x1new
                         xp2=x2new
                         CALL strf_pdf(wwsf)
                         wgt_RS(jj)=x1orig*x2orig/(x1new*x2new)*wwsf/wwsf0
                         xp1=x1orig
                         xp2=x2orig
                      ENDIF
                   ELSE
                      wgt_RS(jj)=1d0
                   ENDIF
                ENDDO                
             ENDIF
             DO j=1,4
                ! shifting momentum or not, i.e. decayway(j)
                ! chic or psi(2S) decay pattern
                IF(.NOT.need_histogram(kk,l,j))CYCLE
                IF(j.EQ.1)THEN
                   ! D1: p[chic/psi(2S)] = p[J/psi]
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)
                         IF(jj.LT.6)THEN
                            wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                         ELSE
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ELSEIF(j.EQ.2)THEN
                   ! D2: p[chic/psi(2S)]*M[J/psi]/M[chic/psi(2S)] = p[J/psi]
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)*masses(6)/masses(jj)
                         IF(jj.LT.6)THEN
                            wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                         ELSE
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ELSEIF(j.EQ.3)THEN
                   ! D3: p[J/psi] is from p[chic/psi(2S)] via spin uncorrelated decay
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         IF(jj.LT.6)THEN
                            ! a true decay
                            CALL TwoBodyDecay_unpol_ana(POUT_RS(jj,imoth,1:4),&
                                 masses(jj),masses(6),P_JPsi(jj,1:4))
                            wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                         ELSE
                            P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ELSE
                   ! D4: p[J/psi] is from p[chic/psi(2S)] via spin correlated decay
                   DO jj=2,6
                      IF(fo2state(jj))THEN
                         IF(jj.LT.6)THEN
                            ! a true decay
                            IF(jj.EQ.2.OR.jj.EQ.3)THEN
                               ! for psi(2S) -> J/psi+pion+pion
                               ! We do not know the exact decay pattern
                               ! Let us use a two-body unpolarized decay
                               ! for chi_c0 -> J/psi+a, it must be an isotropic decay
                               CALL TwoBodyDecay_unpol_ana(POUT_RS(jj,imoth,1:4),&
                                 masses(jj),masses(6),P_JPsi(jj,1:4))
                               wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                            ELSE
                               ! they must be chi_c1, chi_c2 -> J/psi+gamma
                               IF(MCoHelicity)THEN
                                  CALL Qeveryhel_ana(index2,ipol(imoth),imoth)
                               ENDIF
                               IF(MCoHelicity)THEN
                                  IF(calc_LDME_RG_CT.OR.fockstate.EQ.443018)THEN
                                     ! use Eq.(29) in arXiv:1209.4610
                                     isign=MOD(2*index2(3)-3,3)
                                     isign=-isign
                                     IF(jj.EQ.4)THEN
                                        ! chi_c1
                                        CALL GetJz_ana(isign,1,JzJz)
                                     ELSE
                                        ! chi_c2
                                        CALL GetJz_ana(isign,2,JzJz)
                                     ENDIF
                                     isign=JzJz
                                  ELSE
                                     isign=MOD(2*index2(3)-3,3)
                                     isign=-isign
                                     isign=isign-MOD(2*index2(2)-3,3)
                                  ENDIF
                               ELSE
                                  rrr=rand()
                                  IF(jj.EQ.4)THEN
                                     ! chi_c1
                                     isign=FLOOR(3d0*rrr)-1
                                  ELSE
                                     ! chi_c2
                                     isign=FLOOR(5d0*rrr)-2
                                  ENDIF
                               ENDIF
                               IF(jj.EQ.4)THEN
                                  ! chi_c1
                                  CALL HO_chi12psia_hel_ana(POUT_RS(jj,imoth,1:4),&
                                       masses(jj),masses(6),isign,P_JPsi(jj,1:4))
                               ELSE
                                  ! chi_c2
                                  CALL HO_chi22psia_hel_ana(POUT_RS(jj,imoth,1:4),&
                                       masses(jj),masses(6),isign,P_JPsi(jj,1:4))
                               ENDIF
                               wgt_Final(jj)=wgt_RS(jj)*BRList(jj)
                            ENDIF
                         ELSE
                            P_JPsi(jj,1:4)=POUT_RS(jj,imoth,1:4)
                            wgt_Final(jj)=wgt_RS(jj)
                         ENDIF
                      ENDIF
                   ENDDO
                ENDIF
                ! filling the histograms
                DO jj=1,6
                   ! J/psi from various final states
                   pt=-1d0
                   IF(jj.EQ.1)THEN
                      pt=DSQRT(phad(imoth,1)**2+phad(imoth,2)**2)
                      yy=rapidity(phad(imoth,1:4))
                      wgts_Final(1:SIZE(wgts))=wgts(1:SIZE(wgts))
                   ELSE
                      IF(fo2state(jj))THEN
                         pt=DSQRT(P_JPsi(jj,1)**2+P_JPsi(jj,2)**2)
                         yy=rapidity(P_JPsi(jj,1:4))
                         wgts_Final(1:SIZE(wgts))=wgts(1:SIZE(wgts))*wgt_Final(jj)
                      ENDIF
                   ENDIF
                   IF(pt.GE.0d0)THEN
                      IF(DABS(yy).LT.0.6d0)CALL HISTOGRAM_fill(ll+1,pt,wgts_Final)
                      IF(DABS(yy).LT.0.75d0)CALL HISTOGRAM_fill(ll+2,pt,wgts_Final)
                      IF(DABS(yy).LT.1.5d0.AND.DABS(yy).GT.0.75d0)CALL HISTOGRAM_fill(ll+3,pt,wgts_Final)
                   ENDIF
                   ll=ll+3
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE psi_ll_kin(ppsi,mpsi,plp,plm)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::ppsi
    REAL(KIND(1d0)),INTENT(IN)::mpsi
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::plp,plm
    REAL(KIND(1d0)),DIMENSION(4)::plpres,plmres
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::p0,phi,costh,sinth
    costh=2d0*(rand()-0.5d0)
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=rand()*2d0*pipi
    p0=mpsi/2d0
    plpres(3)=p0*costh
    plpres(4)=p0
    plpres(1)=p0*sinth*DCOS(phi)
    plpres(2)=p0*sinth*DSIN(phi)
    plmres(4)=p0
    plmres(1:3)=-plpres(1:3)
    CALL boost(mpsi,ppsi,plpres,plp)
    CALL boost(mpsi,ppsi,plmres,plm)
    RETURN
  END SUBROUTINE psi_ll_kin

  FUNCTION is_a_jet_ana(ifh)
    IMPLICIT NONE
    LOGICAL::is_a_jet_ana
    INTEGER,INTENT(IN)::ifh
    INTEGER::iff,init=0
    SAVE init
    LOGICAL,DIMENSION(12)::is_fermion_a_jet
    SAVE is_fermion_a_jet
    IF(ABS(ifh).GT.12.AND.ABS(ifh).NE.35)THEN
       is_a_jet_ana=.FALSE.
       RETURN
    ENDIF
    IF(ifh.EQ.35)THEN
       is_a_jet_ana=.TRUE.
       RETURN
    ENDIF
    iff=ABS(ifh)
    IF(MOD(iff,4).NE.0.AND.MOD(iff,4).NE.3)THEN
       is_a_jet_ana=.FALSE.
       RETURN
    ENDIF
    IF(init.EQ.0)THEN
       is_fermion_a_jet(1:12)=.FALSE.
       IF(jet_n_flavour.GE.1)is_fermion_a_jet(4)=.TRUE.
       IF(jet_n_flavour.GE.2)is_fermion_a_jet(3)=.TRUE.
       IF(jet_n_flavour.GE.3)is_fermion_a_jet(8)=.TRUE.
       IF(jet_n_flavour.GE.4)is_fermion_a_jet(7)=.TRUE.
       IF(jet_n_flavour.GE.5)is_fermion_a_jet(12)=.TRUE.
       IF(jet_n_flavour.GE.6)is_fermion_a_jet(11)=.TRUE.
       init=1
    ENDIF
    is_a_jet_ana=is_fermion_a_jet(iff)
    RETURN
  END FUNCTION is_a_jet_ana

  FUNCTION is_a_jet_ana_PDG(pdg)
    IMPLICIT NONE
    LOGICAL::is_a_jet_ana_PDG
    INTEGER,INTENT(IN)::pdg
    INTEGER::iff,init=0
    SAVE init
    LOGICAL,DIMENSION(6)::is_fermion_a_jet
    SAVE is_fermion_a_jet
    IF(ABS(pdg).GT.6.AND.ABS(pdg).NE.21)THEN
       is_a_jet_ana_PDG=.FALSE.
       RETURN
    ENDIF
    IF(pdg.EQ.21)THEN
       is_a_jet_ana_PDG=.TRUE.
       RETURN
    ENDIF
    IF(init.EQ.0)THEN
       is_fermion_a_jet(1:6)=.FALSE.
       IF(jet_n_flavour.GE.1)is_fermion_a_jet(1)=.TRUE.
       IF(jet_n_flavour.GE.2)is_fermion_a_jet(2)=.TRUE.
       IF(jet_n_flavour.GE.3)is_fermion_a_jet(3)=.TRUE.
       IF(jet_n_flavour.GE.4)is_fermion_a_jet(4)=.TRUE.
       IF(jet_n_flavour.GE.5)is_fermion_a_jet(5)=.TRUE.
       IF(jet_n_flavour.GE.6)is_fermion_a_jet(6)=.TRUE.
       init=1
    ENDIF
    is_a_jet_ana_PDG=is_fermion_a_jet(ABS(pdg))
    RETURN
  END FUNCTION is_a_jet_ana_PDG

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                  !
! The following are just copies of functions in    !
! Helac_Func_1.f                                   !
!                                                  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  FUNCTION SubInteger_ana(int,n1,n2)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::int,n1,n2
    INTEGER::nmax,nmin,mod11,int2
    INTEGER::SubInteger_ana
    int2=ABS(int)
    nmax=MAX(n1,n2)
    nmin=MIN(n1,n2)
    IF(int2.LT.10**(nmin-1))THEN
       SubInteger_ana=0
       RETURN
    ELSEIF(nmin.GT.0)THEN
       mod11=MOD(int2,10**nmax)
       SubInteger_ana=mod11/10**(nmin-1)
       RETURN
    ELSE
       PRINT *,"Error in SubInteger_ana ! STOP!"
       STOP
    ENDIF
  END FUNCTION SubInteger_ana

  FUNCTION QN3PJF_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN3PJF_ana
    QN3PJF_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.31)RETURN
    QN3PJF_ana=.TRUE.
    RETURN
  END FUNCTION QN3PJF_ANA

  FUNCTION QN1P1F_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN1P1F_ana
    QN1P1F_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.11)RETURN
    QN1P1F_ana=.TRUE.
    RETURN
  END FUNCTION QN1P1F_ANA

  FUNCTION QN3S1F_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN3S1F_ana
    QN3S1F_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.30)RETURN
    QN3S1F_ana=.TRUE.
    RETURN
  END FUNCTION QN3S1F_ANA

  FUNCTION QN1S0F_ana(kk)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::kk
    LOGICAL::QN1S0F_ana
    QN1S0F_ana=.FALSE.
    IF(kk.GT.nhad.OR.kk.LE.0)RETURN
    IF(SubInteger_ana(iflh(kk),3,4).NE.10)RETURN
    QN1S0F_ana=.TRUE.
    RETURN
  END FUNCTION QN1S0F_ANA

  ! for every number, it generates the helicity for (U_lamda,eps(L_z),eps(S_z))
  !*********************************************************
  !     ipol(k)=helicity,ipol(k)=2:rigth-handed            *
  !                      ipol(k)=1:left-handed             *
  !                      ipol(k)=3:long.                   *
  !*********************************************************
  SUBROUTINE Qeveryhel_ana(ihela,num,kk)  !num=ipol(kk)
    IMPLICIT NONE
    INTEGER,DIMENSION(3),INTENT(OUT)::ihela
    INTEGER,INTENT(IN)::num,kk
    INTEGER::ii1,jj1,kk1
    IF(kk.GT.nhad.OR.kk.LE.2)THEN
       PRINT *,"Error 1 in Qeveryhel_ana ! STOP !"
       STOP
    ENDIF
    IF(ABS(iflh(kk)).LE.100)THEN
       PRINT *,"Error 2 in Qeveryhel_ana ! STOP !"
       STOP
    ENDIF
    IF(iranhel.EQ.3)THEN
       ihela(1)=num
       ihela(2:3)=0
       RETURN
    ENDIF
    IF(QN1S0F_ana(kk))THEN
       ihela(1)=num
       ihela(2:3)=0
       RETURN
    ENDIF
    IF(QN3S1F_ana(kk))THEN
       ihela(2)=0
       ii1=MOD(num-1,2)
       jj1=(num-1)/2
       ihela(1)=ii1+1
       ihela(3)=jj1+1
       RETURN
    ENDIF
    IF(QN1P1F_ana(kk))THEN
       IF(iranhel.EQ.2)THEN
          ihela(1)=num
          ihela(2:3)=0
          RETURN
       ELSE
          ihela(3)=0
          ii1=MOD(num-1,2)
          jj1=(num-1)/2
          ihela(1)=ii1+1
          ihela(2)=jj1+1
          RETURN
       ENDIF
    ENDIF
    IF(QN3PJF_ana(kk))THEN
       IF(iranhel.EQ.2)THEN
          ihela(2)=0
          ii1=MOD(num-1,2)
          jj1=(num-1)/2
          ihela(1)=ii1+1
          ihela(3)=jj1+1
          RETURN
       ELSE
          ii1=MOD(num-1,2)
          jj1=MOD((num-1)/2,3)
          kk1=(num-1)/6
          ihela(1)=ii1+1
          ihela(2)=jj1+1
          ihela(3)=kk1+1
          RETURN
       ENDIF
    ENDIF
    ihela(1:3)=0
    RETURN
  END SUBROUTINE Qeveryhel_ana

  SUBROUTINE check_stop_cuts(n_track,p_track,pid_track,passcuts)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::n_track
    REAL(KIND(1d0)),DIMENSION(4,20),INTENT(IN)::p_track
    INTEGER,DIMENSION(20),INTENT(IN)::pid_track
    LOGICAL,INTENT(OUT)::passcuts
    INTEGER::ioniumjet,n_oniumjet_track,n_oniumjet_gluon,n_oniumjet_quark
    INTEGER,DIMENSION(-6:6)::n_oniumjet_quarkflavor
    INTEGER::n_oniumjet_gluonandqqbar,n_oniumjet_nonpairquark
    REAL(KIND(1d0)),DIMENSION(4,20)::p_oniumjet_track
    INTEGER::n_jet,n_onium
    REAL(KIND(1d0)),DIMENSION(4,20)::p_jet
    INTEGER,DIMENSION(20)::jetvec
    REAL(KIND(1d0))::pt1_oniumjet,pt2_oniumjet,z12,z1,d1,d2,dr,pi
    REAL(KIND(1d0))::ptjet_min,ptjet_max,ptjet,RRstop,ptonium,monium
    INTEGER::i,l
    passcuts=.TRUE.
    pi=DACOS(-1d0)
    n_onium=1
    ! make sure the last track is the onium
    IF(stop_jet_dyn_radius_ana.GT.0d0)THEN
       monium=p_track(4,n_track)**2-p_track(1,n_track)**2&
            -p_track(2,n_track)**2-p_track(3,n_track)**2
       monium=DSQRT(monium)
       ptonium=p_track(1,n_track)**2+p_track(2,n_track)**2
       ptonium=DSQRT(ptonium)
       RRstop=MAX(stop_jet_radius_ana,stop_jet_dyn_radius_ana*monium/ptonium)
    ELSE
       RRstop=stop_jet_radius_ana
    ENDIF
    IF(stop_maxrapjet_ana.GT.0d0)THEN
       CALL fastjetppgenkt_etamax(p_track,n_track,RRstop,stop_minptjet_ana,stop_maxrapjet_ana,&
            DBLE(stop_jetalg_ana),p_jet,n_jet,jetvec)
    ELSE
       CALL fastjetppgenkt(p_track,n_track,RRstop,stop_minptjet_ana,DBLE(stop_jetalg_ana),&
            p_jet,n_jet,jetvec)
    ENDIF
    IF(n_jet.LT.stop_min_n_jet_ana)THEN
       passcuts=.FALSE.
       RETURN
    ELSEIF(stop_max_n_jet_ana.GE.0.AND.n_jet.GT.stop_max_n_jet_ana)THEN
       passcuts=.FALSE.
       RETURN
    ENDIF
    ioniumjet=jetvec(n_track)
    IF(ioniumjet.LE.0)THEN
       passcuts=.FALSE.
       RETURN
    ENDIF
    n_oniumjet_track=0
    p_oniumjet_track(1:4,1:20)=0d0
    n_oniumjet_gluon=0
    n_oniumjet_quark=0
    n_oniumjet_quarkflavor(-6:6)=0
    DO i=1,n_track
       IF(jetvec(i).EQ.ioniumjet.AND.is_a_jet_ana_PDG(pid_track(i)))THEN
          n_oniumjet_track=n_oniumjet_track+1
          p_oniumjet_track(1:4,n_oniumjet_track)=p_track(1:4,i)
          IF(pid_track(i).EQ.21)THEN
             n_oniumjet_gluon=n_oniumjet_gluon+1
          ELSEIF(ABS(pid_track(i)).LE.6)THEN
             n_oniumjet_quark=n_oniumjet_quark+1
             n_oniumjet_quarkflavor(pid_track(i))=&
                          n_oniumjet_quarkflavor(pid_track(i))+1
          ENDIF
       ENDIF
    ENDDO
    n_oniumjet_gluonandqqbar=n_oniumjet_gluon
    n_oniumjet_nonpairquark=0
    DO i=1,6
       n_oniumjet_gluonandqqbar=n_oniumjet_gluonandqqbar+&
            MIN(n_oniumjet_quarkflavor(-i),n_oniumjet_quarkflavor(i))
       n_oniumjet_nonpairquark=n_oniumjet_nonpairquark+&
            ABS(n_oniumjet_quarkflavor(-i)-n_oniumjet_quarkflavor(i))
    ENDDO
    ! I do not know the flavours of the stop_n_frag_quark_ana
    ! choose the way to maximize n_oniumjet_gluonandqqbar
    ! subtract the ones needed by stop_n_frag_quark_ana
    IF(n_oniumjet_nonpairquark.LT.stop_n_frag_quark_ana)THEN
       n_oniumjet_gluonandqqbar=n_oniumjet_gluonandqqbar&
            -(stop_n_frag_quark_ana-n_oniumjet_nonpairquark+1)/2
    ENDIF
    IF(n_oniumjet_track.LT.(stop_n_frag_gluon_ana+stop_n_frag_quark_ana).OR.&
         n_oniumjet_gluonandqqbar.LT.stop_n_frag_gluon_ana.OR.&
         n_oniumjet_quark.LT.stop_n_frag_quark_ana)THEN
       passcuts=.FALSE.
       RETURN
    ELSE
       IF(n_oniumjet_track.GE.2)THEN
          ! check the soft drop cut
          DO i=1,n_oniumjet_track-1
             pt1_oniumjet=DSQRT(p_oniumjet_track(1,i)**2+p_oniumjet_track(2,i)**2)
             DO l=i+1,n_oniumjet_track
                pt2_oniumjet=DSQRT(p_oniumjet_track(1,l)**2+p_oniumjet_track(2,l)**2)
                z12=MIN(pt1_oniumjet,pt2_oniumjet)/MAX((pt1_oniumjet+pt2_oniumjet),1D-99)
                d1=prapidity(p_oniumjet_track(1:4,i))&
                     -prapidity(p_oniumjet_track(1:4,l))
                d2=ph4(p_oniumjet_track(1,i),p_oniumjet_track(2,i),p_oniumjet_track(3,i))&
                     -ph4(p_oniumjet_track(1,l),p_oniumjet_track(2,l),p_oniumjet_track(3,l))
                d2=MIN(DABS(d2),2*pi-DABS(d2))
                IF(d2/pi.GT.1.d0)WRITE(*,*)d2/pi
                dr=DSQRT(d1**2+d2**2)
                IF(z12.LE.stop_zcut_ana*(dr/RRstop)**stop_beta_ana)THEN
                   passcuts=.FALSE.
                   RETURN
                ENDIF
             ENDDO
          ENDDO
       ENDIF
       ! n_track-n_onium=i+k+o
       ! assume there is only one onium and one onium jet
       ! stop_min_n_jet-1 is the minimal number of parton jets
       ! If there are extra o radiations, the condition is to require
       ! i+k+o-m=n_track-n_onium-n_oniumjet_track
       IF(stop_min_n_jet_ana-1.LT.(n_track-n_onium-n_oniumjet_track))THEN
          IF(n_jet-1.LT.(n_track-n_onium-n_oniumjet_track))THEN
             passcuts=.FALSE.
             RETURN
          ENDIF
          IF(n_jet-1.GE.2.AND.stop_zasymcut_ana.GT.0d0)THEN
             ! apply the asymmetric cut                                                                                                               
             ptjet_min=-1d0
             ptjet_max=-1d0
             DO i=1,n_jet
                IF(i.EQ.ioniumjet)CYCLE
                ptjet=DSQRT(p_jet(1,i)**2+p_jet(2,i)**2)
                IF(ptjet_min.LT.0d0.OR.ptjet_min.GT.ptjet)THEN
                   ptjet_min=ptjet
                ENDIF
                IF(ptjet_max.LT.0d0.OR.ptjet_max.LT.ptjet)THEN
                   ptjet_max=ptjet
                ENDIF
             ENDDO
             IF(ptjet_min/ptjet_max.LE.stop_zasymcut_ana)THEN
                passcuts=.FALSE.
                RETURN
             ENDIF
          ENDIF
       ENDIF
       ! to pass the soft cut for 3S1[8]
       ! k=stop_n_frag_gluon+stop_n_frag_quark
       IF(n_oniumjet_track.GT.0.AND.(stop_n_frag_gluon_ana+stop_n_frag_quark_ana).EQ.0)THEN
          DO i=1,n_oniumjet_track
             ! Take energy fraction
             z1=p_oniumjet_track(4,i)/p_jet(4,ioniumjet)
             ! zsoftcut is divided by the number of partons inside the onium jet
             IF(z1.LE.stop_zsoftcut_ana/DBLE(n_oniumjet_track))THEN
                passcuts=.FALSE.
                RETURN
             ENDIF
          ENDDO
       ENDIF
    ENDIF
    RETURN
  END SUBROUTINE check_stop_cuts

  FUNCTION pdgt_ana(ifif)
    ! from HELAC-Onia notation to PDG notation
    INTEGER,INTENT(IN)::ifif
    INTEGER::pdgt_ana,init=0,ifac
    INTEGER,DIMENSION(-50:50)::ipdt
    SAVE ipdt,init,ifac
    IF(init.EQ.0)THEN
       ipdt( 1)= 12
       ipdt(-1)=-12
       ipdt( 5)= 14
       ipdt(-5)=-14
       ipdt( 9)= 16
       ipdt(-9)=-16
       ipdt( 2)= 11
       ipdt(-2)=-11
       ipdt( 6)= 13
       ipdt(-6)=-13
       ipdt( 10)= 15
       ipdt(-10)=-15
       ipdt( 3)= 2
       ipdt(-3)=-2
       ipdt( 4)= 1
       ipdt(-4)=-1
       ipdt( 7)= 4
       ipdt(-7)=-4
       ipdt( 8)= 3
       ipdt(-8)=-3
       ipdt( 11)= 6
       ipdt(-11)=-6
       ipdt( 12)= 5
       ipdt(-12)=-5
       ipdt(35)=21
       ipdt(31)=22
       ipdt(32)=23
       ipdt(33)=24
       ipdt(34)=-24
       ipdt(41)=25
       init=1
    ENDIF
    IF(ABS(ifif).LT.100)THEN
       pdgt_ana=ipdt(ifif)
       RETURN
    ENDIF
    ifac=1
    IF(ifif.LT.0)ifac=-1
    SELECT CASE(ABS(ifif))
       ! J/psi
    CASE(443011)
       pdgt_ana=443
       ! etac
    CASE(441001)
       pdgt_ana=441
       ! hc
    CASE(441111)
       pdgt_ana=10443
       ! chic0
    CASE(443101)
       pdgt_ana=10441
       ! chic1
    CASE(443111)
       pdgt_ana=20443
       ! chic2
    CASE(443121)
       pdgt_ana=445
       ! upsilon
    CASE(553011)
       pdgt_ana=553
       ! etab
    CASE(551001)
       pdgt_ana=551
       ! hb
    CASE(551111)
       pdgt_ana=10553
       ! chib0
    CASE(553101)
       pdgt_ana=10551
       ! chib1
    CASE(553111)
       pdgt_ana=20553
       ! chib2
    CASE(553121)
       pdgt_ana=555
       ! Bc*+-
    CASE(453011)
       pdgt_ana=543*ifac
       ! Bc+-
    CASE(451001)
       pdgt_ana=541*ifac
       ! Bc2*+-
    CASE(453121)
       pdgt_ana=545*ifac
       ! Bc0*+-
    CASE(453101)
       pdgt_ana=10541*ifac
       ! Bc1(H)+-
    CASE(453111)
       pdgt_ana=20543*ifac
       ! ccbar(3P0(8))
    CASE(443108)
       pdgt_ana=9910441
       ! bbbar(3P0(8))
    CASE(553108)
       pdgt_ana=9910551
       ! bbbar(3S1(8))
    CASE(553018)
       pdgt_ana=9900553
       ! bbbar(1S0(8))
    CASE(551008)
       pdgt_ana=9900551
       ! ccbar(3S1(8))
    CASE(443018)
       pdgt_ana=9900443
       ! ccbar(1S0(8))
    CASE(441008)
       pdgt_ana=9900441
    CASE DEFAULT
       pdgt_ana=ifif
    END SELECT
    RETURN
  END FUNCTION pdgt_ana

  SUBROUTINE GetJz_ana(sz,JJ,Jz)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::sz,JJ
    INTEGER,INTENT(OUT)::Jz
    REAL(KIND(1d0))::rrr
    ! use Eq.(29) in arXiv:1209.4610
    IF(ABS(sz).GT.1)THEN
       WRITE(*,*)"ERROR: invalid sz (0,+-1) !"
       STOP
    ENDIF
    rrr=rand()
    IF(JJ.EQ.1)THEN
       IF(sz.EQ.-1)THEN
          IF(rrr.GT.0.5d0)THEN
             Jz=0
          ELSE
             Jz=-1
          ENDIF
       ELSEIF(sz.EQ.1)THEN
          IF(rrr.GT.0.5d0)THEN
             Jz=1
          ELSE
             Jz=0
          ENDIF
       ELSEIF(sz.EQ.0)THEN
          IF(rrr.GT.0.5d0)THEN
             Jz=1
          ELSE
             Jz=-1
          ENDIF
       ENDIF
    ELSEIF(JJ.EQ.2)THEN
       IF(sz.EQ.-1)THEN
          IF(rrr.LT.0.6d0)THEN
             Jz=-2
          ELSEIF(rrr.GT.0.6d0.AND.rrr.LT.0.9d0)THEN
             Jz=-1
          ELSE
             Jz=0
          ENDIF
       ELSEIF(sz.EQ.1)THEN
          IF(rrr.LT.0.6d0)THEN
             Jz=2
          ELSEIF(rrr.GT.0.6d0.AND.rrr.LT.0.9d0)THEN
             Jz=1
          ELSE
             Jz=0
          ENDIF
       ELSE
          IF(rrr.LT.0.3d0)THEN
             Jz=-1
          ELSEIF(rrr.GT.0.3d0.AND.rrr.LT.0.7d0)THEN
             Jz=0
          ELSE
             Jz=1
          ENDIF
       ENDIF
    ELSE
       WRITE(*,*)"ERROR: invalid J (1 or 2) !"
       STOP
    ENDIF
    RETURN
  END SUBROUTINE GetJz_ana

  ! unpolarized two body decay
  ! particle one (MM) -> daugther one (MD1)  daughter two (M=0) 
  SUBROUTINE TwoBodyDecay_unpol_ana(PM,MM,MD1,PD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PM
    REAL(KIND(1d0)),INTENT(IN)::MM,MD1
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PD1
    REAL(KIND(1d0))::mass,mass2,ppp,sinth,costh,phi
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    mass=DSQRT(PM(4)**2-PM(1)**2-PM(2)**2-PM(3)**2)
    mass2=mass*MD1/MM
    ppp=(mass**2-mass2**2)/(2d0*mass)
    costh=2d0*(rand()-0.5d0)
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=2d0*pipi*rand()
    PD1(4)=DSQRT(ppp**2+mass2**2)
    PD1(1)=-ppp*DCOS(phi)*sinth
    PD1(2)=-ppp*DSIN(phi)*sinth
    PD1(3)=-ppp*costh
    ! boost to the lab frame
    CALL BOOSTL(mass,PM,PD1)
    RETURN
  END SUBROUTINE TwoBodyDecay_unpol_ana

  ! spin-correlated decay of chi_c1 -> J/psi+a
  ! mother particle one (MM) -> daugther one (MD1)  daughter two (M=0)
  ! Jz of mother partical is "spin"
  ! check the formalism in arXiv:1412.2576
  SUBROUTINE HO_chi12psia_hel_ana(PM,MM,MD1,spin,PD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PM
    REAL(KIND(1d0)),INTENT(IN)::MM,MD1
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PD1
    INTEGER,INTENT(IN)::spin ! Jz in the helicity frame
    ! use CLEO measurement arXiv:0910.0046
    REAL(KIND(1d0)),PARAMETER::aJ10=0.998038696644574d0,aJ11=-0.0626d0
    REAL(KIND(1d0))::rrr,rrr2,costh,sinth,phi
    INTEGER::init=0,i,j,k
    SAVE init
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
    REAL(KIND(1d0)),DIMENSION(4)::fac1
    SAVE fac1
    REAL(KIND(1d0)),DIMENSION(4)::fac2
    SAVE fac2
    REAL(KIND(1d0)),DIMENSION(4)::fac3
    SAVE fac3
    REAL(KIND(1d0))::mass,mass2,ppp
    REAL(KIND(1d0)),DIMENSION(3)::frame
    IF(init.EQ.0)THEN
       fac1(1)=(1 + 2*aJ10*aJ11)/4.
       fac1(2)=0.5
       fac1(3)=(3 - 2*aJ10*aJ11)/4.
       fac1(4)=1
       fac2(1)=(1 + 2*aJ10*aJ11)/4.
       fac2(2)=0.5
       fac2(3)=(3 - 2*aJ10*aJ11)/4.
       fac2(4)=1
       fac3(1)=(1 + 2*aJ10*aJ11)/4.
       fac3(2)=0.5
       fac3(3)=(3 - 2*aJ10*aJ11)/4.
       fac3(4)=1
       init=1
     ENDIF
     SELECT CASE(spin)
     CASE(-1)
        rrr=rand()
        k=1
        DO i=1,4
           IF(rrr.LE.fac1(i))THEN
              k=i
              EXIT
           ENDIF
           IF(i.EQ.4)THEN
              k=4
              EXIT
           ENDIF
        ENDDO
        SELECT CASE(k)
        CASE(1)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(2)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=1-rrr2**(1d0/3d0)
        CASE(3)
           ! decay via -((-1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=rrr2**(1d0/3d0)-1
        CASE(4)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        END SELECT
     CASE(0)
        rrr=rand()
        k=1
        DO i=1,4
           IF(rrr.LE.fac2(i))THEN
              k=i
              EXIT
           ENDIF
           IF(i.EQ.4)THEN
              k=4
              EXIT
           ENDIF
        ENDDO
        SELECT CASE(k)
        CASE(1)
           ! decay via ((1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13))**2)/4.
           rrr2=2d0/3d0*rand()-1d0/3d0
           IF(rrr2.GE.0d0)THEN
              costh=(rrr2*3d0)**(1d0/3d0)
           ELSE
              costh=-(-rrr2*3d0)**(1d0/3d0)
           ENDIF
        CASE(2)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(3)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(4)
           ! decay via ((1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13))**2)/4.
           rrr2=2d0/3d0*rand()-1d0/3d0
           IF(rrr2.GE.0d0)THEN
              costh=(rrr2*3d0)**(1d0/3d0)
           ELSE
              costh=-(-rrr2*3d0)**(1d0/3d0)
           ENDIF
        END SELECT
     CASE(1)
        rrr=rand()
        k=1
        DO i=1,4
           IF(rrr.LE.fac3(i))THEN
              k=i
              EXIT
           ENDIF
           IF(i.EQ.4)THEN
              k=4
              EXIT
           ENDIF
        ENDDO
        SELECT CASE(k)
        CASE(1)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        CASE(2)
           ! decay via -((-1 + 2*aJ10*aJ11)*Cos(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=rrr2**(1d0/3d0)-1
        CASE(3)
           ! decay via -((-1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13)/2.)**4)/4.
           rrr2=8d0*rand()
           costh=1-rrr2**(1d0/3d0)
        CASE(4)
           ! decay via ((1 + 2*aJ10*aJ11)*Sin(Subscript(\[Theta],13))**2)/8.
           rrr2=2d0*(rand()-0.5d0)
           IF(rrr2.LT.-1d0)THEN
              rrr2=-1d0
           ELSEIF(rrr2.GT.1d0)THEN
              rrr2=1d0
           ENDIF
           costh=SOLVEVll00_ana(rrr2)
        END SELECT
     END SELECT
     sinth=DSQRT(ABS(1d0-costh**2))
     phi=2*pi*rand()
     mass=DSQRT(PM(4)**2-PM(1)**2-PM(2)**2-PM(3)**2)
     mass2=mass*MD1/MM
     ppp=(mass**2-mass2**2)/(2d0*mass)
     PD1(4)=DSQRT(ppp**2+mass2**2)
     PD1(1)=-ppp*DCOS(phi)*sinth
     PD1(2)=-ppp*DSIN(phi)*sinth
     PD1(3)=-ppp*costh
     frame(1:3)=PM(1:3)
     ! rotate to make the z axis is the flight of mother particle
     CALL ROTATEL(frame,PD1)
     CALL BOOSTL(mass,PM,PD1)
     RETURN
  END SUBROUTINE HO_chi12psia_hel_ana


  ! spin-correlated decay of chi_c2 -> J/psi+a
  ! mother particle one (MM) -> daugther one (MD1)  daughter two (M=0)
  ! Jz of mother partical is "spin"
  ! check the formalism in arXiv:1412.2576
  SUBROUTINE HO_chi22psia_hel_ana(PM,MM,MD1,spin,PD1)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PM
    REAL(KIND(1d0)),INTENT(IN)::MM,MD1
    REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::PD1
    INTEGER,INTENT(IN)::spin ! Jz in the helicity frame
    ! use CLEO measurement arXiv:0910.0046
    REAL(KIND(1d0)),PARAMETER::aJ21=0.9956661086930698d0,aJ22=-0.093d0,aJ23=0d0
    REAL(KIND(1d0))::mass,mass2,ppp
    REAL(KIND(1d0)),DIMENSION(3)::frame
    REAL(KIND(1d0))::rrr,rrr2,costh,sinth,phi,prob,delta0,delta1,fmax
    INTEGER::init=0,i,j,k
    SAVE init
    LOGICAL::lexit
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
    REAL(KIND(1d0)),DIMENSION(6)::fac1
    SAVE fac1
    REAL(KIND(1d0)),DIMENSION(6)::fac2
    SAVE fac2
    REAL(KIND(1d0)),DIMENSION(6)::fac3
    SAVE fac3
    REAL(KIND(1d0)),DIMENSION(6)::fac4
    SAVE fac4
    REAL(KIND(1d0)),DIMENSION(6)::fac5
    SAVE fac5
    IF(init.EQ.0)THEN
       delta0=1d0+2d0*aJ21*(DSQRT(5d0)*aJ22+2d0*aJ23)&
            +4d0*aJ22*(aJ22+DSQRT(5d0)*aJ23)+3d0*aJ23**2
       delta0=delta0/10d0
       delta1=9d0+6d0*aJ21*(DSQRT(5d0)*aJ22-4d0*aJ23)&
            -4d0*aJ22*(aJ22+2d0*DSQRT(5d0)*aJ23)+7d0*aJ23**2
       delta1=delta1/30d0
       fac1(1)=delta0/2.
       fac1(2)=(1 - delta1)/2.
       fac1(3)=0.5
       fac1(4)=(1 + delta1)/2.
       fac1(5)=1 - delta0/2.
       fac1(6)=1
       fac2(1)=delta0/2.
       fac2(2)=(1 - delta1)/2.
       fac2(3)=0.5
       fac2(4)=(1 + delta1)/2.
       fac2(5)=1 - delta0/2.
       fac2(6)=1
       fac3(1)=delta0/2.
       fac3(2)=(1 - delta1)/2.
       fac3(3)=0.5
       fac3(4)=(1 + delta1)/2.
       fac3(5)=1 - delta0/2.
       fac3(6)=1
       fac4(1)=delta0/2.
       fac4(2)=(1 - delta1)/2.
       fac4(3)=0.5
       fac4(4)=(1 + delta1)/2.
       fac4(5)=1 - delta0/2.
       fac4(6)=1
       fac5(1)=delta0/2.
       fac5(2)=(1 - delta1)/2.
       fac5(3)=0.5
       fac5(4)=(1 + delta1)/2.
       fac5(5)=1 - delta0/2.
       fac5(6)=1
       init=1
    ENDIF
    SELECT CASE(spin)
    CASE(-2)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac1(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Cos(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(-1)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac2(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*(1 + 2*Cos(Subscript(\[Theta],13)))**2*Sin(Subscript(\[Theta],13)/2.)**4)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+2d0*costh)**2*(1d0-costh)**2/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*Cos(Subscript(\[Theta],13)/2.)**4*(1 - 2*Cos(Subscript(\[Theta],13)))**2)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**2/4d0*(1d0-2d0*costh)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(0)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac3(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (delta0*(1 + 3*Cos(2*Subscript(\[Theta],13)))**2)/32.
          fmax=16d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(-2d0+6d0*costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via (-3*(-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (3*delta1*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (3*delta1*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via (-3*(-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (delta0*(1 + 3*Cos(2*Subscript(\[Theta],13)))**2)/32.
          fmax=16d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(-2d0+6d0*costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(1)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac4(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*Cos(Subscript(\[Theta],13)/2.)**4*(1 - 2*Cos(Subscript(\[Theta],13)))**2)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**2/4d0*(1d0-2d0*costh)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*(1 + 2*Cos(Subscript(\[Theta],13)))**2*Sin(Subscript(\[Theta],13)/2.)**4)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+2d0*costh)**2*(1d0-costh)**2/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(2*Subscript(\[Theta],13))**2)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh**2)*costh**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    CASE(2)
       rrr=rand()
       k=1
       DO i=1,6
          IF(rrr.LE.fac5(i))THEN
             k=i
             EXIT
          ENDIF
          IF(i.EQ.6)THEN
             k=6
             EXIT
          ENDIF
       ENDDO
       SELECT CASE(k)
       CASE(1)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(2)
          ! decay via -((-1 + delta0 + delta1)*Sin(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(3)
          ! decay via (delta1*Csc(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**6)/32.
          fmax=6.75d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=4d0*(1d0-costh)*(1d0+costh)**3/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(4)
          ! decay via (delta1*Sin(Subscript(\[Theta],13)/2.)**4*Sin(Subscript(\[Theta],13))**2)/2.
          fmax=0.43d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh)**2*(1d0-costh**2)/4d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(5)
          ! decay via -((-1 + delta0 + delta1)*Cos(Subscript(\[Theta],13)/2.)**8)/2.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0+costh)**4/16d0/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       CASE(6)
          ! decay via (3*delta0*Sin(Subscript(\[Theta],13))**4)/16.
          fmax=1d0
          lexit=.FALSE.
          DO WHILE(.NOT.lexit)
             costh=2d0*rand()-1d0
             prob=(1d0-costh**2)**2/fmax
             rrr2=rand()
             IF(prob.GT.rrr2)lexit=.TRUE.
          ENDDO
       END SELECT
    END SELECT
    sinth=DSQRT(ABS(1d0-costh**2))
    phi=2*pi*rand()
    mass=DSQRT(PM(4)**2-PM(1)**2-PM(2)**2-PM(3)**2)
    mass2=mass*MD1/MM
    ppp=(mass**2-mass2**2)/(2d0*mass)
    PD1(4)=DSQRT(ppp**2+mass2**2)
    PD1(1)=-ppp*DCOS(phi)*sinth
    PD1(2)=-ppp*DSIN(phi)*sinth
    PD1(3)=-ppp*costh
    frame(1:3)=PM(1:3)
    ! rotate to make the z axis is the flight of mother particle
    CALL ROTATEL(frame,PD1)
    CALL BOOSTL(mass,PM,PD1)
    RETURN
  END SUBROUTINE HO_chi22psia_hel_ana

  FUNCTION SOLVEVll00_ana(rrr)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::rrr
    REAL(KIND(1d0))::SOLVEVll00_ana
    REAL(KIND(1d0))::xr,xi
    COMPLEX(KIND(1d0))::x
    x=DCMPLX(rrr)+DCMPLX(0d0,1d0)*SQRT(1d0-rrr**2)
    x=x**(1d0/3d0)
    xr=DREAL(x)
    xi=DIMAG(x)
    IF(ABS(xr).LE.0.5d0+1d-99)THEN
       SOLVEVll00_ana=-2d0*xr
       SOLVEVll00_ana=DSIGN(MIN(ABS(SOLVEVll00_ana),1d0),SOLVEVll00_ana)
    ELSEIF(ABS(xr+DSQRT(3d0)*xi).LE.1d0+1d-99)THEN
       SOLVEVll00_ana=xr+DSQRT(3d0)*xi
       SOLVEVll00_ana=DSIGN(MIN(ABS(SOLVEVll00_ana),1d0),SOLVEVll00_ana)
    ELSE
       SOLVEVll00_ana=xr-DSQRT(3d0)*xi
       SOLVEVll00_ana=DSIGN(MIN(ABS(SOLVEVll00_ana),1d0),SOLVEVll00_ana)
    ENDIF
    RETURN
  END FUNCTION SOLVEVll00_ana

  ! K factors (LP+NLO)/NLO in 1509.07904
  ! istate:
  ! 1: 3S11
  ! 2: 3S18
  ! 3: 1S08
  ! 4: 3PJ8
  ! 5: 3P01
  ! 6: 3P11
  ! 7: 3P21
  SUBROUTINE KFactorLPNLO(istate,ptFO0,KFactor)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ptFO0
    REAL(KIND(1d0)),INTENT(OUT)::KFactor
    REAL(KIND(1d0))::ptFO
    INTEGER,INTENT(IN)::istate
    REAL(KIND(1d0))::ptmin=10d0,ptmax=150d0
    INTEGER,PARAMETER::NARRAY=115
    REAL(KIND(1d0)),DIMENSION(NARRAY)::xarray,KF3S18,KF1S08,&
       KF3PJ8,KF3P11,KF3P21
    ptFO=ptFO0
    IF(ptFO.LT.ptmin.OR.ptFO.GT.ptmax)THEN
       WRITE(*,*)"WARNING: ptFO is outside the range in KFactorLPNLO"
       IF(ptFO.LT.ptmin)THEN
          KFactor=1d0
          RETURN
       ELSE
          ptFO=ptmax ! larger to take the ptmax K factor as it is flat
       ENDIF
    ENDIF
    ! the input values
    xarray=(/&
       10.,&
       10.1,&
       10.200000000000001,&
       10.3,&
       10.4,&
       10.5,&
       10.6,&
       10.700000000000001,&
       10.8,&
       10.9,&
       11.,&
       11.100000000000001,&
       11.200000000000001,&
       11.3,&
       11.4,&
       11.5,&
       11.600000000000001,&
       11.700000000000001,&
       11.8,&
       11.9,&
       12.,&
       12.100000000000001,&
       12.200000000000001,&
       12.3,&
       12.4,&
       12.5,&
       12.600000000000001,&
       12.700000000000001,&
       12.8,&
       12.9,&
       13.,&
       13.100000000000001,&
       13.200000000000001,&
       13.3,&
       13.4,&
       13.5,&
       13.600000000000001,&
       13.700000000000001,&
       13.8,&
       13.9,&
       14.,&
       14.100000000000001,&
       14.2,&
       14.3,&
       14.4,&
       14.5,&
       14.600000000000001,&
       14.700000000000001,&
       14.8,&
       14.9,&
       15.,&
       15.5,&
       16.,&
       16.5,&
       17.,&
       17.5,&
       18.,&
       18.5,&
       19.,&
       19.5,&
       20.,&
       21.,&
       22.,&
       23.,&
       24.,&
       25.,&
       26.,&
       27.,&
       28.,&
       29.,&
       30.,&
       31.,&
       32.,&
       33.,&
       34.,&
       35.,&
       36.,&
       37.,&
       38.,&
       39.,&
       40.,&
       42.,&
       44.,&
       46.,&
       48.,&
       50.,&
       52.,&
       54.,&
       56.,&
       58.,&
       60.,&
       62.,&
       64.,&
       66.,&
       68.,&
       70.,&
       72.,&
       74.,&
       76.,&
       78.,&
       80.,&
       85.,&
       90.,&
       95.,&
       100.,&
       105.,&
       110.,&
       115.,&
       120.,&
       125.,&
       130.,&
       135.,&
       140.,&
       145.,&
       150.&
       /)

    KF3S18=(/&
       0.693878,&
       0.6912808159520484,&
       0.6890168956255517,&
       0.6870862390205099,&
       0.6854888461369232,&
       0.6842247169747914,&
       0.6832938515341146,&
       0.6826962498148926,&
       0.6824319118171257,&
       0.6825008375408138,&
       0.6829030269859567,&
       0.6836384801525547,&
       0.6834605335160182,&
       0.6831742944842802,&
       0.6828214092743771,&
       0.6824018778863089,&
       0.6819157003200756,&
       0.6813628765756773,&
       0.680743406653114,&
       0.6800572905523855,&
       0.679304528273492,&
       0.6784851198164334,&
       0.6775990651812098,&
       0.6766463643678211,&
       0.6756270173762673,&
       0.6745410242065485,&
       0.6733883848586646,&
       0.6721690993326157,&
       0.6708831676284016,&
       0.6695305897460225,&
       0.6681113656854785,&
       0.6666254954467692,&
       0.6650729790298949,&
       0.6634538164348556,&
       0.6628739600637217,&
       0.6624115516515447,&
       0.6619268165349227,&
       0.6614197547138557,&
       0.6608903661883436,&
       0.6603386509583864,&
       0.6597646090239843,&
       0.6591682403851369,&
       0.6585495450418446,&
       0.6579085229941073,&
       0.6572451742419249,&
       0.6565594987852974,&
       0.6558514966242248,&
       0.6551211677587072,&
       0.6543685121887446,&
       0.653593529914337,&
       0.6528454555464436,&
       0.6488999497401919,&
       0.6436976335897013,&
       0.6352540620093644,&
       0.632653,&
       0.632653,&
       0.6321587520002302,&
       0.6274201324663695,&
       0.6207374484865076,&
       0.615994324093759,&
       0.6119593960330126,&
       0.6120749723249775,&
       0.6064294245954833,&
       0.596970347310116,&
       0.5891707020466452,&
       0.5912593616857952,&
       0.5874071890142102,&
       0.5809704918365355,&
       0.5729420392837767,&
       0.566433324891848,&
       0.561444348660749,&
       0.5630069195376209,&
       0.557898883828643,&
       0.5490334649953471,&
       0.5473202016556267,&
       0.5508964206861453,&
       0.544206798124103,&
       0.540944591414114,&
       0.5399007238162351,&
       0.5354227723582251,&
       0.531158815938656,&
       0.524113580198258,&
       0.5163921121446574,&
       0.5055107158881914,&
       0.5003162608169667,&
       0.49984680100127105,&
       0.49592834893655263,&
       0.4919518776211259,&
       0.48789019413942714,&
       0.48378155006605605,&
       0.4796729392976219,&
       0.47556441091511037,&
       0.47145585087009517,&
       0.46734722465255335,&
       0.4632385816732494,&
       0.4591299719988824,&
       0.4550213956294524,&
       0.45091285256495933,&
       0.44685057784155063,&
       0.4428737321909122,&
       0.4389548725182821,&
       0.4321747019825248,&
       0.4285815429621134,&
       0.4250219953870521,&
       0.4182495262415722,&
       0.4082743600886228,&
       0.4080874052524719,&
       0.40480877105485696,&
       0.39798124390354234,&
       0.39283645641232134,&
       0.38770205441106415,&
       0.38268352722331467,&
       0.3776181757588875,&
       0.3725060000177826,&
       0.367347&
       /)

KF1S08=(/&
       1.11224,&
       1.114340958125744,&
       1.116391728710981,&
       1.1183923117557106,&
       1.1203427072599332,&
       1.1222429152236484,&
       1.1240929356468565,&
       1.1258927685295572,&
       1.127642413871751,&
       1.1293418716734371,&
       1.1309911419346164,&
       1.1325902246552884,&
       1.134139119835453,&
       1.1356378274751104,&
       1.1370863475742607,&
       1.1384846801329036,&
       1.1398328251510395,&
       1.141130782628668,&
       1.1423785525657895,&
       1.1437434575158325,&
       1.1452242019981589,&
       1.1467244609005047,&
       1.1482442342228703,&
       1.1497835219652555,&
       1.1513423241276601,&
       1.1529206407100843,&
       1.1545184717125283,&
       1.1561358171349918,&
       1.1577726769774748,&
       1.1594290512399774,&
       1.1611049399224995,&
       1.1628003430250413,&
       1.1645959534574672,&
       1.1664558000192782,&
       1.168351525121572,&
       1.1702831287643485,&
       1.1722506109476076,&
       1.1742539716713498,&
       1.1762932109355746,&
       1.1783683287402822,&
       1.1804793250854726,&
       1.1826261999711458,&
       1.1848089533973016,&
       1.1870275853639405,&
       1.189282095871062,&
       1.1915724849186662,&
       1.193886908958946,&
       1.1947730101917766,&
       1.195703717306742,&
       1.1966790303038422,&
       1.1976989491830772,&
       1.2034676318112738,&
       1.2103514614928397,&
       1.2183504382277752,&
       1.22746456201608,&
       1.236682446444376,&
       1.2439630960952155,&
       1.2516027627493538,&
       1.2596014464067908,&
       1.2679591470675269,&
       1.2767249115654133,&
       1.295215549960618,&
       1.3137039996471802,&
       1.3349765082514193,&
       1.3573307755498059,&
       1.3689174321046504,&
       1.3828969012132026,&
       1.3992925408055155,&
       1.4156134432130791,&
       1.429138595074957,&
       1.4402377551020407,&
       1.4518426419485113,&
       1.467983285217835,&
       1.486287990319818,&
       1.5005558106466297,&
       1.5085887751814624,&
       1.520534556699491,&
       1.5349588310281643,&
       1.5463426358865378,&
       1.5537786615866085,&
       1.5613953843136374,&
       1.583116409222122,&
       1.5976626650109753,&
       1.6164483489119048,&
       1.6321467472490565,&
       1.644116959713612,&
       1.6523801197757668,&
       1.6649536636727005,&
       1.6794602864236758,&
       1.691653398565897,&
       1.6985228614737469,&
       1.704047077484737,&
       1.7079637555776963,&
       1.7210985214708399,&
       1.7323332939371883,&
       1.73469,&
       1.73469,&
       1.7365788224424736,&
       1.7438078184384225,&
       1.7466305882600728,&
       1.7450620338894027,&
       1.7449,&
       1.7449,&
       1.7449,&
       1.7448336807097726,&
       1.7381029384373374,&
       1.7347166643748078,&
       1.734736794310539,&
       1.7245300093567009,&
       1.7176694920280982,&
       1.714236996497655,&
       1.709214260150249,&
       1.7041471819515044,&
       1.6990357619014214,&
       1.69388&
       /)

KF3PJ8=(/&
       5.,&
       4.915559153917042,&
       4.834912337276911,&
       4.758004881412208,&
       4.678106798227831,&
       4.595218087723783,&
       4.51003066442737,&
       4.427028310085174,&
       4.34701658306265,&
       4.269995483359797,&
       4.239156588347126,&
       4.205681987314483,&
       4.15711855678053,&
       4.093466296745266,&
       4.014725207208693,&
       3.9301058774988995,&
       3.8734114400838653,&
       3.8323032457808326,&
       3.8067812945898,&
       3.7968455865107678,&
       3.8024961215437356,&
       3.77381048074119,&
       3.7335914013942086,&
       3.6957844527416777,&
       3.660389634783598,&
       3.6274069475199693,&
       3.5968363909507906,&
       3.5686779650760636,&
       3.5429316698957876,&
       3.5195975054099624,&
       3.495106354048268,&
       3.4686544879043915,&
       3.4425381507954156,&
       3.4167573427213394,&
       3.391312063682163,&
       3.366339426431821,&
       3.3451061735325096,&
       3.325279684903332,&
       3.3068599605442883,&
       3.2898470004553784,&
       3.274240804636602,&
       3.2600413730879594,&
       3.247248705809451,&
       3.2358628028010754,&
       3.2258836640628337,&
       3.212798916265643,&
       3.1993332570037274,&
       3.1862149961024206,&
       3.173444133561724,&
       3.1610206693816356,&
       3.1489446035621573,&
       3.0937752498739037,&
       3.034045828334273,&
       2.97395180306374,&
       2.925084846236005,&
       2.8886940282587403,&
       2.8623547751956706,&
       2.8447581394959496,&
       2.816071891023601,&
       2.7881395493698338,&
       2.7609611145346475,&
       2.712262137026882,&
       2.671305995693088,&
       2.6372718857566646,&
       2.6102419549864564,&
       2.589406192981238,&
       2.5651263947604264,&
       2.5415817921396564,&
       2.5223240799606637,&
       2.5064796907239106,&
       2.4906335550812244,&
       2.473597111593001,&
       2.457993523353654,&
       2.443929311258993,&
       2.4314044753090185,&
       2.42041901550373,&
       2.408608615687807,&
       2.3974169137313908,&
       2.3875402907944805,&
       2.380771950305046,&
       2.3724642933731084,&
       2.351231030181725,&
       2.330348167461203,&
       2.315330680055931,&
       2.2994812917927296,&
       2.2836323813811235,&
       2.2705272323859,&
       2.26334892881108,&
       2.2481163715686616,&
       2.2396054764597983,&
       2.2318669469570644,&
       2.224295926255972,&
       2.217723833006074,&
       2.202192884098845,&
       2.1910482519154066,&
       2.1836090264065473,&
       2.176362209142672,&
       2.161517408811007,&
       2.1511615426860335,&
       2.1422018329680754,&
       2.128089792210611,&
       2.1047568100053384,&
       2.0974719476345576,&
       2.078290970282714,&
       2.0589699197144045,&
       2.0286530929864166,&
       2.001004639787983,&
       1.9818656326254234,&
       1.9625477472865895,&
       1.94898,&
       1.94898,&
       1.94898,&
       1.9476072721546795,&
       1.9316779167364588,&
       1.92857&
       /)

KF3P11=(/&
       47.5338,&
       45.840193500000005,&
       44.18066399999998,&
       42.555211499999984,&
       40.96383599999999,&
       39.4065375,&
       37.88331600000001,&
       36.394171499999985,&
       34.939103999999986,&
       33.5181135,&
       32.1312,&
       30.77836349999998,&
       29.459603999999985,&
       28.17492149999999,&
       26.924315999999994,&
       25.7077875,&
       24.52533599999998,&
       23.376961499999986,&
       22.26266399999999,&
       21.182443499999994,&
       20.1363,&
       19.124233499999985,&
       18.14624399999999,&
       17.202331499999993,&
       16.292495999999996,&
       15.416737499999998,&
       14.575055999999988,&
       13.767451499999991,&
       12.993923999999993,&
       12.254473499999996,&
       11.5491,&
       10.877807058024665,&
       10.240590957287203,&
       9.637451697787604,&
       9.068389279525864,&
       8.533403702501989,&
       8.032494966715964,&
       7.565663072167811,&
       7.13290801885752,&
       6.734229806785089,&
       6.3696284359505215,&
       6.03910390635381,&
       5.74265621799497,&
       5.4802853708739825,&
       5.251991364990861,&
       5.057774200345601,&
       4.897633808480343,&
       4.771570372371375,&
       4.679583957073638,&
       4.621674562587132,&
       4.597842188911855,&
       4.360354645676852,&
       4.1887220938479945,&
       3.9916095762766277,&
       3.8375131162312126,&
       3.7197979377852923,&
       3.6288756703691556,&
       3.531847479196137,&
       3.4423540238297927,&
       3.365996557300649,&
       3.304586611972859,&
       3.2034392949429935,&
       3.1090686924114763,&
       3.0396766597211764,&
       2.9835934942968634,&
       2.9156052344404984,&
       2.8591793622128803,&
       2.8062515077232697,&
       2.762580388987821,&
       2.722600381460017,&
       2.686950981393711,&
       2.6652021372518155,&
       2.6495803617829377,&
       2.6292351083030163,&
       2.6071468277187435,&
       2.581914150950402,&
       2.560285893313375,&
       2.543448008940488,&
       2.5257158683104786,&
       2.5051075022162137,&
       2.4884149474987796,&
       2.4609147668818974,&
       2.4362868748598587,&
       2.4125979735181313,&
       2.3956267589310003,&
       2.3762465240001434,&
       2.3543534554133143,&
       2.3448283568244714,&
       2.3296759731239836,&
       2.308315195513616,&
       2.2831346087833047,&
       2.2592351103649153,&
       2.2396002261830907,&
       2.222257204437371,&
       2.2045678286340813,&
       2.1877641789206743,&
       2.1719627573518823,&
       2.162010198648145,&
       2.1582111012072462,&
       2.1599927600349136,&
       2.1616409478649574,&
       2.148271263585623,&
       2.1267104993407906,&
       2.1048332847656006,&
       2.085359749848312,&
       2.0677018708826105,&
       2.0492109229997926,&
       2.022755845201161,&
       2.013027904095117,&
       2.0073003684798634,&
       1.9996634593284035,&
       1.9910433856890053,&
       1.9831374323120812,&
       1.975421775069983,&
       1.96538&
       /)

KF3P21=(/&
       3.48799,&
       3.473829333333333,&
       3.4597099999999994,&
       3.445632,&
       3.431595333333333,&
       3.4176,&
       3.40356,&
       3.38952,&
       3.37548,&
       3.36144,&
       3.3474,&
       3.3333599999999994,&
       3.3193199999999994,&
       3.3052819999999996,&
       3.2912453333333334,&
       3.27721,&
       3.2631716476254913,&
       3.2491324523970615,&
       3.235092414314711,&
       3.2210515333784393,&
       3.2072974409090054,&
       3.1971598707493945,&
       3.1839294899249135,&
       3.167606298435563,&
       3.1466690506177124,&
       3.1242530084654425,&
       3.1038980815827633,&
       3.085604269969675,&
       3.069371573626178,&
       3.0551999925522715,&
       3.0429298218714322,&
       3.0314421481974043,&
       3.0207292778620007,&
       3.010791210865221,&
       3.0016279472070653,&
       2.993239486887534,&
       2.985625829906626,&
       2.9767337283731687,&
       2.9678600259564156,&
       2.959760166198294,&
       2.952717678858185,&
       2.9480374298085557,&
       2.9425833381002953,&
       2.936355403733403,&
       2.9293536267078792,&
       2.921578007023724,&
       2.9130285446809374,&
       2.9037052396795193,&
       2.8917814751159034,&
       2.880193132652671,&
       2.86937863284807,&
       2.8267193599807183,&
       2.800252288398062,&
       2.781865253452023,&
       2.748164399891733,&
       2.734224420663859,&
       2.7077616064950716,&
       2.6890123832873374,&
       2.6705438043935894,&
       2.651829107361674,&
       2.637713147153337,&
       2.6039145357046327,&
       2.585905797650186,&
       2.5540815608475707,&
       2.5386350213156974,&
       2.5258051275726943,&
       2.5030236897914087,&
       2.483235078846355,&
       2.469579795830313,&
       2.459734289332655,&
       2.4482051745668474,&
       2.433580689166514,&
       2.4158608331316556,&
       2.4066859317657636,&
       2.397804549430907,&
       2.3868430235594342,&
       2.3757484679256495,&
       2.365897065514934,&
       2.3572888163272867,&
       2.3493034642244566,&
       2.340505245036854,&
       2.322675271519396,&
       2.305567224073476,&
       2.2900951682944313,&
       2.280925429043226,&
       2.270203832006018,&
       2.2575314899897885,&
       2.245288209398956,&
       2.2340782574422935,&
       2.224690968205806,&
       2.216691207309199,&
       2.2099230167422608,&
       2.1980155912907406,&
       2.1840248856389515,&
       2.1722640525093073,&
       2.166036975631064,&
       2.1577714864968303,&
       2.144467838181955,&
       2.131630610360838,&
       2.1192598030334775,&
       2.1113199114697734,&
       2.0972646759896523,&
       2.083790500575251,&
       2.0555189823053195,&
       2.0448643009626646,&
       2.0333106220116517,&
       2.0116172031444868,&
       1.9876510687351134,&
       1.9666742497259189,&
       1.956761238442503,&
       1.9490687565184255,&
       1.9383918242875093,&
       1.9322703249192226,&
       1.9220312252409237,&
       1.90428&
       /)

    ! istate
    ! 1: 3S11
    ! 2: 3S18
    ! 3: 1S08
    ! 4: 3PJ8
    ! 5: 3P01
    ! 6: 3P11
    ! 7: 3P21
    IF(istate.EQ.2)THEN
       CALL SPLINE_INTERPOLATE_ana(xarray,KF3S18,NARRAY,ptFO,KFactor)
    ELSEIF(istate.EQ.3)THEN
       CALL SPLINE_INTERPOLATE_ana(xarray,KF1S08,NARRAY,ptFO,KFactor)
    ELSEIF(istate.EQ.4)THEN
       CALL SPLINE_INTERPOLATE_ana(xarray,KF3PJ8,NARRAY,ptFO,KFactor)
    ELSEIF(istate.EQ.6)THEN
       CALL SPLINE_INTERPOLATE_ana(xarray,KF3P11,NARRAY,ptFO,KFactor)
    ELSEIF(istate.EQ.7)THEN
       CALL SPLINE_INTERPOLATE_ana(xarray,KF3P21,NARRAY,ptFO,KFactor)
    ELSE
       KFactor=1d0
    ENDIF
    RETURN
  END SUBROUTINE KFactorLPNLO

  SUBROUTINE SPLINE_INTERPOLATE_ana(XI,YI,N,X,Y)
    !====================================================================
    ! Spline interpolation
    ! Comments: values of function f(x) are calculated in n base points
    ! then: spline coefficients are computed
    !       spline interpolation is computed in 2n-1 points, 
    !       a difference sum|f(u)-ispline(u)| 
    !====================================================================
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N ! base points for interpolation
    REAL(KIND(1d0)),DIMENSION(N),INTENT(IN)::XI,YI
    REAL(KIND(1d0)),DIMENSION(N)::b,c,d
    REAL(KIND(1d0)),INTENT(IN)::x
    REAL(KIND(1d0)),INTENT(OUT)::y
    REAL(KIND(1d0))::error,errav
    INTEGER::i
    ! call spline to calculate spline coefficients
    CALL SPLINE_ana(XI,YI,b,c,d,N)
    ! interpolation at ninit points
    Y=ISPLINE_ana(X,XI,YI,b,c,d,N)
    RETURN
  END SUBROUTINE SPLINE_INTERPOLATE_ANA

  subroutine spline_ana (x, y, b, c, d, n)
    !======================================================================
    !  Calculate the coefficients b(i), c(i), and d(i), i=1,2,...,n
    !  for cubic spline interpolation
    !  s(x) = y(i) + b(i)*(x-x(i)) + c(i)*(x-x(i))**2 + d(i)*(x-x(i))**3
    !  for  x(i) <= x <= x(i+1)
    !  Alex G: January 2010
    !----------------------------------------------------------------------
    !  input..
    !  x = the arrays of data abscissas (in strictly increasing order)
    !  y = the arrays of data ordinates
    !  n = size of the arrays xi() and yi() (n>=2)
    !  output..
    !  b, c, d  = arrays of spline coefficients
    !  comments ...
    !  spline.f90 program is based on fortran version of program spline.f
    !  the accompanying function fspline can be used for interpolation
    !======================================================================
    implicit none
    integer n
    double precision x(n), y(n), b(n), c(n), d(n)
    integer i, j, gap
    double precision h
    
    gap = n-1
    ! check input
    if ( n < 2 ) return
    if ( n < 3 ) then
       b(1) = (y(2)-y(1))/(x(2)-x(1))   ! linear interpolation
       c(1) = 0.
       d(1) = 0.
       b(2) = b(1)
       c(2) = 0.
       d(2) = 0.
       return
    end if
    !
    ! step 1: preparation
    !
    d(1) = x(2) - x(1)
    c(2) = (y(2) - y(1))/d(1)
    do i = 2, gap
       d(i) = x(i+1) - x(i)
       b(i) = 2.0*(d(i-1) + d(i))
       c(i+1) = (y(i+1) - y(i))/d(i)
       c(i) = c(i+1) - c(i)
    end do
    !
    ! step 2: end conditions 
    !
    b(1) = -d(1)
    b(n) = -d(n-1)
    c(1) = 0.0
    c(n) = 0.0
    if(n /= 3) then
       c(1) = c(3)/(x(4)-x(2)) - c(2)/(x(3)-x(1))
       c(n) = c(n-1)/(x(n)-x(n-2)) - c(n-2)/(x(n-1)-x(n-3))
       c(1) = c(1)*d(1)**2/(x(4)-x(1))
       c(n) = -c(n)*d(n-1)**2/(x(n)-x(n-3))
    end if
    !
    ! step 3: forward elimination 
    !
    do i = 2, n
       h = d(i-1)/b(i-1)
       b(i) = b(i) - h*d(i-1)
       c(i) = c(i) - h*c(i-1)
    end do
    !
    ! step 4: back substitution
    !
    c(n) = c(n)/b(n)
    do j = 1, gap
       i = n-j
       c(i) = (c(i) - d(i)*c(i+1))/b(i)
    end do
    !
    ! step 5: compute spline coefficients
    !
    b(n) = (y(n) - y(gap))/d(gap) + d(gap)*(c(gap) + 2.0*c(n))
    do i = 1, gap
       b(i) = (y(i+1) - y(i))/d(i) - d(i)*(c(i+1) + 2.0*c(i))
       d(i) = (c(i+1) - c(i))/d(i)
       c(i) = 3.*c(i)
    end do
    c(n) = 3.0*c(n)
    d(n) = d(n-1)
  end subroutine spline_ana

  function ispline_ana(u, x, y, b, c, d, n)
    !======================================================================
    ! function ispline evaluates the cubic spline interpolation at point z
    ! ispline = y(i)+b(i)*(u-x(i))+c(i)*(u-x(i))**2+d(i)*(u-x(i))**3
    ! where  x(i) <= u <= x(i+1)
    !----------------------------------------------------------------------
    ! input..
    ! u       = the abscissa at which the spline is to be evaluated
    ! x, y    = the arrays of given data points
    ! b, c, d = arrays of spline coefficients computed by spline
    ! n       = the number of data points
    ! output:
    ! ispline = interpolated value at point u
    !=======================================================================
    implicit none
    double precision ispline_ana
    integer n
    double precision  u, x(n), y(n), b(n), c(n), d(n)
    integer i, j, k
    double precision dx
    
    ! if u is ouside the x() interval take a boundary value (left or right)
    if(u <= x(1)) then
       ispline_ana = y(1)
       return
    end if
    if(u >= x(n)) then
       ispline_ana = y(n)
       return
    end if
    
    !*
    !  binary search for for i, such that x(i) <= u <= x(i+1)
    !*
    i = 1
    j = n+1
    do while (j > i+1)
       k = (i+j)/2
       if(u < x(k)) then
          j=k
       else
          i=k
       end if
    end do
    !*
    !  evaluate spline interpolation
    !*
    dx = u - x(i)
    ispline_ana = y(i) + dx*(b(i) + dx*(c(i) + dx*d(i)))
  end function ispline_ana
END MODULE plot_user
