MODULE plot_user
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=18*3*5
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,j,kk,l
    CHARACTER(len=4),DIMENSION(3)::cuts=(/'    ','LHCb','CMS '/)
    CHARACTER(len=5),DIMENSION(5)::smears=(/'     ','sm0.5','sm1.0',&
         'sm2.0','sm3.0'/)
!    character*5 cc(2)
!      data cc/'     ',' Born'/
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    DO j=1,5
       DO i=1,3
          l=(i-1)*18+(j-1)*18*3
          CALL histogram_book(l+ 1,'total rate    '//cuts(i)//' '//smears(j),&
               5,0.5d0,5.5d0,.FALSE.)
          CALL histogram_book(l+ 2,'|Delta y| min '//cuts(i)//' '//smears(j),&
               100,0d0,5d0,.TRUE.)
          CALL histogram_book(l+ 3,'|Delta y| max '//cuts(i)//' '//smears(j),&
               100,0d0,5d0,.TRUE.)
          CALL histogram_book(l+ 4,'|Delta phi| min '//cuts(i)//' '//smears(j),&
               100,-0.0001d0,3.1416d0,.TRUE.)
          CALL histogram_book(l+ 5,'|Delta phi| max '//cuts(i)//' '//smears(j),&
               100,-0.0001d0,3.1416d0,.TRUE.)
          CALL histogram_book(l+ 6,'psi psi inv. mass min '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 7,'psi psi inv. mass max '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 8,'psi psi psi inv. mass '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 9,'psi psi psi trans mom '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 10,'|Delta eta| min '//cuts(i)//' '//smears(j),&
               100,0d0,4d0,.TRUE.)
          CALL histogram_book(l+ 11,'|Delta eta| max '//cuts(i)//' '//smears(j),&
               100,0d0,4d0,.TRUE.)
          CALL histogram_book(l+ 12,'random Pt 1 '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 13,'random Pt 2 '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 14,'random Pt 3 '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 15,'leading Pt '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 16,'subleading Pt '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 17,'subsubleading Pt '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
          CALL histogram_book(l+ 18,'Pt inclusive '//cuts(i)//' '//smears(j),&
               100,0d0,100d0,.TRUE.)
       ENDDO
    ENDDO
    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::pboo
    REAL(KIND(1d0)),DIMENSION(4)::ppsi1,ppsi2,ppsi3
    REAL(KIND(1d0)),DIMENSION(4)::ppsi12,ppsi13,ppsi23,ppsi123
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,j,kk,l
    REAL(KIND(1d0))::www,ypsi1,ypsi2,ypsi3
    REAL(KIND(1d0))::delta_y12,delta_y13,delta_y23,delta_ymin,delta_ymax
    REAL(KIND(1d0))::etapsi1,etapsi2,etapsi3
    REAL(KIND(1d0))::delta_eta12,delta_eta13,delta_eta23,delta_etamin,&
         delta_etamax
    REAL(KIND(1d0))::delta_phi12,delta_phi13,delta_phi23,&
         delta_phimin,delta_phimax
    REAL(KIND(1d0))::m12,m23,m13,m123,mpsipsi_min,mpsipsi_max,pt_psipsipsi
    LOGICAL,DIMENSION(3)::cuts
    REAL(KIND(1d0))::pt_psi1,pt_psi2,pt_psi3,pt1,pt2,pt3
    REAL(KIND(1d0)),DIMENSION(3)::pt_ordered
    REAL(KIND(1d0)),DIMENSION(4,20)::phep_psi,phep_psi_sm
    REAL(KIND(1d0)),DIMENSION(2:5)::smear_value
    IF(wgts(1).EQ.0d0)RETURN
    !IF(NDecayChains.GT.0)THEN
    !   CALL plot_fill_Decay(wgts)
    !   RETURN
    !ENDIF
    !IF(emep_ISR_shower.EQ.1)THEN
    !   CALL plot_fill_QEDPS(wgts)
    !   RETURN
    !ENDIF
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    ! 
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    kk=1
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          kk=kk+1
       ENDIF
    ENDDO
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    smear_value(2)=0.5d0
    smear_value(3)=1d0
    smear_value(4)=2d0
    smear_value(5)=3d0
    DO j=1,5
       IF(j.EQ.1)THEN
          ppsi1(1:4)=phad(3,1:4)
          ppsi2(1:4)=phad(4,1:4)
          ppsi3(1:4)=phad(5,1:4)
       ELSE
          DO kk=1,nhad
             phep_psi(1:4,kk)=phad(kk,1:4) ! (px,py,pz,E)
          ENDDO
          CALL InitKtSmearing(nhad,phep_psi(1:4,1:nhad),phep_psi_sm(1:4,1:nhad),&
               smear_value(j),2)
          ppsi1(1:4)=phep_psi_sm(1:4,3)
          ppsi2(1:4)=phep_psi_sm(1:4,4)
          ppsi3(1:4)=phep_psi_sm(1:4,5)
       ENDIF
       ppsi12(1:4)=ppsi1(1:4)+ppsi2(1:4)
       ppsi13(1:4)=ppsi1(1:4)+ppsi3(1:4)
       ppsi23(1:4)=ppsi2(1:4)+ppsi3(1:4)
       ppsi123(1:4)=ppsi1(1:4)+ppsi2(1:4)+ppsi3(1:4)
       ypsi1=rapidity(ppsi1)
       ypsi2=rapidity(ppsi2)
       ypsi3=rapidity(ppsi3)
       cuts(1)=.TRUE.
       cuts(2)=ypsi1.GT.2D0.AND.ypsi2.GT.2D0.AND.ypsi3.GT.2D0
       cuts(2)=cuts(2).AND.ypsi1.LT.4.5D0.AND.&
            ypsi2.LT.4.5D0.AND.ypsi3.LT.4.5D0
       cuts(3)=DABS(ypsi1).LT.2.4D0.AND.DABS(ypsi2).LT.2.4D0&
            .AND.DABS(ypsi3).LT.2.4D0
       delta_y12=DABS(ypsi1-ypsi2)
       delta_y13=DABS(ypsi1-ypsi3)
       delta_y23=DABS(ypsi2-ypsi3)
       delta_ymin=MIN(delta_y12,delta_y13,delta_y23)
       delta_ymax=MAX(delta_y12,delta_y13,delta_y23)
       delta_phi12=DeltaPh4(ppsi1,ppsi2)
       delta_phi13=DeltaPh4(ppsi1,ppsi3)
       delta_phi23=DeltaPh4(ppsi2,ppsi3)
       delta_phimin=MIN(delta_phi12,delta_phi13,delta_phi23)
       delta_phimax=MAX(delta_phi12,delta_phi13,delta_phi23)
       m12=DSQRT(ppsi12(4)**2-ppsi12(1)**2-ppsi12(2)**2-ppsi12(3)**2)
       m13=DSQRT(ppsi13(4)**2-ppsi13(1)**2-ppsi13(2)**2-ppsi13(3)**2)
       m23=DSQRT(ppsi23(4)**2-ppsi23(1)**2-ppsi23(2)**2-ppsi23(3)**2)
       m123=DSQRT(ppsi123(4)**2-ppsi123(1)**2-ppsi123(2)**2-ppsi123(3)**2)
       mpsipsi_min=MIN(m12,m13,m23)
       mpsipsi_max=MAX(m12,m13,m23)
       pt_psipsipsi=DSQRT(ppsi123(1)**2+ppsi123(2)**2)
       etapsi1=prapidity(ppsi1)
       etapsi2=prapidity(ppsi2)
       etapsi3=prapidity(ppsi3)
       delta_eta12=DABS(etapsi1-etapsi2)
       delta_eta13=DABS(etapsi1-etapsi3)
       delta_eta23=DABS(etapsi2-etapsi3)
       delta_etamin=MIN(delta_eta12,delta_eta13,delta_eta23)
       delta_etamax=MAX(delta_eta12,delta_eta13,delta_eta23)
       pt_psi1=DSQRT(ppsi1(1)**2+ppsi1(2)**2)
       pt_psi2=DSQRT(ppsi2(1)**2+ppsi2(2)**2)
       pt_psi3=DSQRT(ppsi3(1)**2+ppsi3(2)**2)
       pt1=MAX(pt_psi1,pt_psi2,pt_psi3)
       pt3=MIN(pt_psi1,pt_psi2,pt_psi3)
       pt_ordered(1)=pt_psi1
       pt_ordered(2)=pt_psi2
       pt_ordered(3)=pt_psi3
       CALL GetPTOrdered(3,pt_ordered) ! get the pt-ordered results
       pt2=pt_ordered(2)
       IF(pt_ordered(1).NE.pt1)THEN
          WRITE(*,*)"ERROR: do not get the correct pt ordered results #1"
          STOP
       ENDIF
       IF(pt_ordered(3).NE.pt3)THEN
          WRITE(*,*)"ERROR: do not get the correct pt ordered results #2"
          STOP
       ENDIF
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       !
       ! START TO FILL HISTOGRAM
       !
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       DO i=1,3
          IF(.NOT.cuts(i))CYCLE
          l=(i-1)*18+(j-1)*18*3
          CALL HISTOGRAM_FILL(l+1,var,wgts)
          CALL HISTOGRAM_FILL(l+2,delta_ymin,wgts)
          CALL HISTOGRAM_FILL(l+3,delta_ymax,wgts)
          CALL HISTOGRAM_FILL(l+4,delta_phimin,wgts)
          CALL HISTOGRAM_FILL(l+5,delta_phimax,wgts)
          CALL HISTOGRAM_FILL(l+6,mpsipsi_min,wgts)
          CALL HISTOGRAM_FILL(l+7,mpsipsi_max,wgts)
          CALL HISTOGRAM_FILL(l+8,m123,wgts)
          CALL HISTOGRAM_FILL(l+9,pt_psipsipsi,wgts)
          CALL HISTOGRAM_FILL(l+10,delta_etamin,wgts)
          CALL HISTOGRAM_FILL(l+11,delta_etamax,wgts)
          CALL HISTOGRAM_FILL(l+12,pt_psi1,wgts)
          CALL HISTOGRAM_FILL(l+13,pt_psi2,wgts)
          CALL HISTOGRAM_FILL(l+14,pt_psi3,wgts)
          CALL HISTOGRAM_FILL(l+15,pt1,wgts)
          CALL HISTOGRAM_FILL(l+16,pt2,wgts)
          CALL HISTOGRAM_FILL(l+17,pt3,wgts)
          CALL HISTOGRAM_FILL(l+18,pt_psi1,wgts)
          CALL HISTOGRAM_FILL(l+18,pt_psi2,wgts)
          CALL HISTOGRAM_FILL(l+18,pt_psi3,wgts)
       ENDDO
    ENDDO
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE GetPTOrdered(npt,pt_ordered)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::npt
    REAL(KIND(1d0)),DIMENSION(npt),INTENT(INOUT)::pt_ordered
    INTEGER::i,j
    REAL(KIND(1d0)),DIMENSION(npt)::pt_orig
    LOGICAL,DIMENSION(npt)::used
    REAL(KIND(1d0))::ptmax
    INTEGER::indexmax
    pt_orig(1:npt)=pt_ordered(1:npt)
    used(1:npt)=.FALSE.
    DO i=1,npt
       ptmax=-1d0
       indexmax=0
       DO j=1,npt
          IF(used(j))CYCLE
          IF(pt_orig(j).GT.ptmax)THEN
             ptmax=pt_orig(j)
             indexmax=j
          ENDIF
       ENDDO
       pt_ordered(i)=ptmax
       used(indexmax)=.TRUE.
    ENDDO
    RETURN
  END SUBROUTINE GetPTOrdered

  FUNCTION DeltaPh4(pmom1,pmom2)
    IMPLICIT NONE
    REAL(KIND(1d0))::DeltaPh4
    REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::pmom1,pmom2 ! px,py,pz,E
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    DeltaPh4=ph4(pmom1(1),pmom1(2),pmom1(3)) ! ph4 is defined in Kinematic_Func.f90
    DeltaPh4=DeltaPh4-ph4(pmom2(1),pmom2(2),pmom2(3))
    IF(DeltaPh4.GT.pi)DeltaPh4=2d0*pi-DeltaPh4
    RETURN
  END FUNCTION DeltaPh4

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax                                                  
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

  SUBROUTINE plot_fill_Decay(wgts)
    USE Decay_interface
    USE QEDPS_interface
    USE Helac_Func_1
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    INCLUDE "../../decay/stdhep.inc"! decay variables are defined in stdhep.inc
    REAL(KIND(1d0))::var,Q,www,dy_psipsi,dphi_psipsi,M_psipsi,Pt_psipsi,deta_psipsi,Pt1Pt2_psipsi
    REAL(KIND(1d0))::q12_psipsi,cosalpha,ptpsi1,ptpsi2,ptpsimax,ptpsimin,ptpsidiff
    REAL(KIND(1d0))::Pt_psipsi_sm05,Pt_psipsi_sm10,Pt_psipsi_sm20,Pt_psipsi_sm30
    REAL(KIND(1d0))::dy_psipsi_sm05,dy_psipsi_sm10,dy_psipsi_sm20,dy_psipsi_sm30
    REAL(KIND(1d0))::dphi_psipsi_sm05,dphi_psipsi_sm10,dphi_psipsi_sm20,dphi_psipsi_sm30
    REAL(KIND(1d0))::M_psipsi_sm05,M_psipsi_sm10,M_psipsi_sm20,M_psipsi_sm30
    INTEGER::i,m,l,kk
    REAL(KIND(1d0)),DIMENSION(4,NMXHEP)::PHEPSM05,PHEPSM10,PHEPSM20,PHEPSM30
    REAL(KIND(1d0)),DIMENSION(4)::PBOO
    REAL(KIND(1d0)),DIMENSION(2,4)::PJpsi,PJpsiCMS,PJpsiSM05,PJpsiSM10,PJpsiSM20,PJpsiSM30
    REAL(KIND(1d0)),DIMENSION(4,4)::Plep
    REAL(KIND(1d0)),DIMENSION(4)::plep1,Plep2
    REAL(KIND(1d0)),DIMENSION(3)::Pplain1,Pplain2
    REAL(KIND(1d0)),DIMENSION(3)::Plepvec1,Plepvec2
    REAL(KIND(1d0)),DIMENSION(2,3)::PJpsi1l,PJpsi2l
    REAL(KIND(1d0))::PP1,PP2
    REAL(KIND(1d0))::costh1,costh2
    INTEGER,DIMENSION(4)::lep2jpsi
    INTEGER,DIMENSION(2)::newmoth
    INTEGER::i_jpsi,i_lep,kk1,kk2
    INTEGER::definemoth
    REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932384626433832795d0
    WRITE(*,*)"PLEASE UPDATE THIS FUNCTION FIRST !"
    STOP
    IF(wgts(1).EQ.0d0)RETURN
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    ! select the first four decayed stable lepton
    i_jpsi=0
    i_lep=0
    lep2jpsi(1:4)=0
    newmoth(1:2)=0
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM05(1:4,1:NHEP),0.5d0,2)
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM10(1:4,1:NHEP),1d0,2)
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM20(1:4,1:NHEP),2d0,2)
    CALL InitKtSmearing(NHEP,PHEP(1:4,1:NHEP),PHEPSM30(1:4,1:NHEP),3d0,2)
    DO i=1,NHEP
       IF(ISHEP(i).EQ.-1)CYCLE
       IF(ISTHEP(i).EQ.1)CYCLE
       IF(JMOHEP(1,i).LE.0.OR.JMOHEP(1,i).GT.NHEP)CYCLE
       IF(ISTHEP(JMOHEP(1,i)).NE.1.OR.ISHEP(JMOHEP(1,i)).EQ.-1)CYCLE
       IF(.NOT.lepton_pdg(IDHEP(i)))CYCLE
       i_lep=i_lep+1
       Plep(i_lep,1:4)=PHEP(1:4,i)
       m=JMOHEP(1,i)
       definemoth=0
       DO kk=1,i_jpsi
          IF(newmoth(kk).EQ.m)THEN
             definemoth=kk
             EXIT
          END IF
       ENDDO
       IF(definemoth.NE.0)THEN
          ! the mother has been defined
          lep2jpsi(i_lep)=definemoth
       ELSE
          ! create the new mother
          i_jpsi=i_jpsi+1
          PJpsi(i_jpsi,1:4)=PHEP(1:4,m)
          PJpsiSM05(i_jpsi,1:4)=PHEPSM05(1:4,m)
          PJpsiSM10(i_jpsi,1:4)=PHEPSM10(1:4,m)
          PJpsiSM20(i_jpsi,1:4)=PHEPSM20(1:4,m)
          PJpsiSM30(i_jpsi,1:4)=PHEPSM30(1:4,m)
          newmoth(i_jpsi)=m
          lep2jpsi(i_lep)=i_jpsi
          IF(i_lep.EQ.4)EXIT
       ENDIF
    ENDDO
    IF(literature_cutoffs.EQ.14080000)THEN
       ! LHCb Kinematics Region
       ptpsi1=transverse(PJpsi(1,1:4))
       IF(ptpsi1.GT.10d0)THEN
          RETURN
       ENDIF
       ptpsi2=transverse(PJpsi(2,1:4))
       IF(ptpsi2.GT.10d0)THEN
          RETURN
       ENDIF
    ENDIF
    dy_psipsi=DABS(rapidity(PJpsi(1,1:4))-rapidity(PJpsi(2,1:4)))
    dy_psipsi_sm05=DABS(rapidity(PJpsiSM05(1,1:4))-rapidity(PJpsiSM05(2,1:4)))
    IF(isnan(dy_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    dy_psipsi_sm10=DABS(rapidity(PJpsiSM10(1,1:4))-rapidity(PJpsiSM10(2,1:4)))
    IF(isnan(dy_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    dy_psipsi_sm20=DABS(rapidity(PJpsiSM20(1,1:4))-rapidity(PJpsiSM20(2,1:4)))
    IF(isnan(dy_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    dy_psipsi_sm30=DABS(rapidity(PJpsiSM30(1,1:4))-rapidity(PJpsiSM30(2,1:4)))
    IF(isnan(dy_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:dy_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    deta_psipsi=DABS(prapidity(PJpsi(1,1:4))-prapidity(PJpsi(2,1:4)))
    dphi_psipsi=ph4(PJpsi(1,1),PJpsi(1,2),PJpsi(1,3))
    dphi_psipsi=dphi_psipsi-ph4(PJpsi(2,1),PJpsi(2,2),PJpsi(2,3))
    dphi_psipsi=DABS(dphi_psipsi)
    IF(dphi_psipsi.GT.pi)dphi_psipsi=2*pi-dphi_psipsi
    dphi_psipsi_sm05=ph4(PJpsiSM05(1,1),PJpsiSM05(1,2),PJpsiSM05(1,3))
    dphi_psipsi_sm05=dphi_psipsi_sm05-ph4(PJpsiSM05(2,1),PJpsiSM05(2,2),PJpsiSM05(2,3))
    dphi_psipsi_sm05=DABS(dphi_psipsi_sm05)
    IF(dphi_psipsi_sm05.GT.pi)dphi_psipsi_sm05=2*pi-dphi_psipsi_sm05
    IF(isnan(dphi_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    dphi_psipsi_sm10=ph4(PJpsiSM10(1,1),PJpsiSM10(1,2),PJpsiSM10(1,3))
    dphi_psipsi_sm10=dphi_psipsi_sm10-ph4(PJpsiSM10(2,1),PJpsiSM10(2,2),PJpsiSM10(2,3))
    dphi_psipsi_sm10=DABS(dphi_psipsi_sm10)
    IF(dphi_psipsi_sm10.GT.pi)dphi_psipsi_sm10=2*pi-dphi_psipsi_sm10
    IF(isnan(dphi_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    dphi_psipsi_sm20=ph4(PJpsiSM20(1,1),PJpsiSM20(1,2),PJpsiSM20(1,3))
    dphi_psipsi_sm20=dphi_psipsi_sm20-ph4(PJpsiSM20(2,1),PJpsiSM20(2,2),PJpsiSM20(2,3))
    dphi_psipsi_sm20=DABS(dphi_psipsi_sm20)
    IF(dphi_psipsi_sm20.GT.pi)dphi_psipsi_sm20=2*pi-dphi_psipsi_sm20
    IF(isnan(dphi_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    dphi_psipsi_sm30=ph4(PJpsiSM30(1,1),PJpsiSM30(1,2),PJpsiSM30(1,3))
    dphi_psipsi_sm30=dphi_psipsi_sm30-ph4(PJpsiSM30(2,1),PJpsiSM30(2,2),PJpsiSM30(2,3))
    dphi_psipsi_sm30=DABS(dphi_psipsi_sm30)
    IF(dphi_psipsi_sm30.GT.pi)dphi_psipsi_sm30=2*pi-dphi_psipsi_sm30
    IF(isnan(dphi_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:dphi_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    M_psipsi=scalar_product(PJpsi(1,1:4)+PJpsi(2,1:4),PJpsi(1,1:4)+PJpsi(2,1:4))
    M_psipsi=DSQRT(M_psipsi)
    M_psipsi_sm05=scalar_product(PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4),PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4))
    M_psipsi_sm05=DSQRT(M_psipsi_sm05)
    IF(isnan(M_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    M_psipsi_sm10=scalar_product(PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4),PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4))
    M_psipsi_sm10=DSQRT(M_psipsi_sm10)
    IF(isnan(M_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    M_psipsi_sm20=scalar_product(PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4),PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4))
    M_psipsi_sm20=DSQRT(M_psipsi_sm20)
    IF(isnan(M_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    M_psipsi_sm30=scalar_product(PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4),PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4))
    M_psipsi_sm30=DSQRT(M_psipsi_sm30)
    IF(isnan(M_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:M_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    Q=scalar_product(PJpsi(1,1:4),PJpsi(1,1:4))
    Q=DSQRT(Q)
    ptpsi1=transverse(PJpsi(1,1:4))
    ptpsi2=transverse(PJpsi(2,1:4))
    ptpsimax=MAX(ptpsi1,ptpsi2)
    ptpsimin=MIN(ptpsi1,ptpsi2)
    ptpsidiff=ptpsimax-ptpsimin
    Pt_psipsi=transverse(PJpsi(1,1:4)+PJpsi(2,1:4))
    Pt_psipsi_sm05=transverse(PJpsiSM05(1,1:4)+PJpsiSM05(2,1:4))
    IF(isnan(Pt_psipsi_sm05))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm05 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM05(1,1:4)=",PJpsiSM05(1,1:4)
       WRITE(*,*)"PJpsiSM05(2,1:4)=",PJpsiSM05(2,1:4)
       STOP
    ENDIF
    Pt_psipsi_sm10=transverse(PJpsiSM10(1,1:4)+PJpsiSM10(2,1:4))
    IF(isnan(Pt_psipsi_sm10))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm10 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM10(1,1:4)=",PJpsiSM10(1,1:4)
       WRITE(*,*)"PJpsiSM10(2,1:4)=",PJpsiSM10(2,1:4)
       STOP
    ENDIF
    Pt_psipsi_sm20=transverse(PJpsiSM20(1,1:4)+PJpsiSM20(2,1:4))
    IF(isnan(Pt_psipsi_sm20))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm20 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM20(1,1:4)=",PJpsiSM20(1,1:4)
       WRITE(*,*)"PJpsiSM20(2,1:4)=",PJpsiSM20(2,1:4)
       STOP
    ENDIF
    Pt_psipsi_sm30=transverse(PJpsiSM30(1,1:4)+PJpsiSM30(2,1:4))
    IF(isnan(Pt_psipsi_sm30))THEN
       WRITE(*,*)"ERROR:Pt_psipsi_sm30 is NaN !"
       WRITE(*,*)"wgt=",wgts(1),i_jpsi,NHEP
       DO i=1,NHEP
          WRITE(*,*)i,IDHEP(i),PHEP(1:4,i)
       ENDDO
       WRITE(*,*)"PJpsi(1,1:4)=",PJpsi(1,1:4)
       WRITE(*,*)"PJpsi(2,1:4)=",PJpsi(2,1:4)
       WRITE(*,*)"PJpsiSM30(1,1:4)=",PJpsiSM30(1,1:4)
       WRITE(*,*)"PJpsiSM30(2,1:4)=",PJpsiSM30(2,1:4)
       STOP
    ENDIF
    Pt1Pt2_psipsi=transverse(PJpsi(1,1:4)-PJpsi(2,1:4))
    q12_psipsi=M_psipsi**2-4*Q**2
    q12_psipsi=DSQRT(DABS(q12_psipsi))
    PBOO(1:3)=-PJpsi(1,1:3)-PJpsi(2,1:3)
    PBOO(4)=PJpsi(1,4)+PJpsi(2,4)
    DO i=1,4
       CALL BOOSTL(M_psipsi,PBOO,Plep(i,1:4))
    ENDDO
    DO i=1,2
       PJpsiCMS(i,1:4)=PJpsi(i,1:4)
       CALL BOOSTL(M_psipsi,PBOO,PJpsiCMS(i,1:4))
    ENDDO
    kk1=0
    kk2=0
    DO i=1,4
       IF(lep2jpsi(i).EQ.1)THEN
          kk1=kk1+1
          PJpsi1l(kk1,1:3)=Plep(i,1:3)
          IF(kk1.EQ.1)THEN
             Plep1(1:4)=Plep(i,1:4)
          ENDIF
       ELSEIF(lep2jpsi(i).EQ.2)THEN
          kk2=kk2+1
          PJpsi2l(kk2,1:3)=Plep(i,1:3)
          IF(kk2.EQ.1)THEN
             Plep2(1:4)=Plep(i,1:4)
          ENDIF
       ENDIF
    ENDDO
    Pplain1(1:3)=vec_crossprod(PJpsi1l(1,1:3),PJpsi1l(2,1:3))
    Pplain2(1:3)=vec_crossprod(PJpsi2l(1,1:3),PJpsi2l(2,1:3))
    PP1=Pplain1(1)**2+Pplain1(2)**2+Pplain1(3)**2
    PP1=DSQRT(PP1)
    PP2=Pplain2(1)**2+Pplain2(2)**2+Pplain2(3)**2
    PP2=DSQRT(PP2)
    IF(PP1*PP2.EQ.0d0)THEN
       cosalpha=0d0
    ELSE
       cosalpha=Pplain1(1)*Pplain2(1)+Pplain1(2)*Pplain2(2) &
       +Pplain1(3)*Pplain2(3)
       cosalpha=DABS(cosalpha/(PP1*PP2))
    ENDIF
    ! costh1
    Q=scalar_product(PJpsiCMS(1,1:4),PJpsiCMS(1,1:4))
    Q=DSQRT(Q)
    PBOO(1:3)=-PJpsiCMS(1,1:3)
    PBOO(4)=PJpsiCMS(1,4)
    CALL BOOSTL(Q,PBOO,Plep1)
    costh1=cosij(Plep1(1:3),PJpsiCMS(1,1:3))
    ! costh2
    Q=scalar_product(PJpsiCMS(2,1:4),PJpsiCMS(2,1:4))
    Q=DSQRT(Q)
    PBOO(1:3)=-PJpsiCMS(2,1:3)
    PBOO(4)=PJpsiCMS(2,4)
    CALL BOOSTL(Q,PBOO,Plep2)
    costh2=cosij(Plep2(1:3),PJpsiCMS(2,1:3))
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    !
    ! START TO FILL HISTOGRAM 
    !  
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,dy_psipsi,wgts)
    CALL HISTOGRAM_fill(l+3,dphi_psipsi,wgts)
    CALL HISTOGRAM_fill(l+4,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+5,M_psipsi,wgts)
    CALL HISTOGRAM_fill(l+6,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill(l+7,q12_psipsi,wgts)
    CALL HISTOGRAM_fill(l+8,q12_psipsi,wgts)
    CALL HISTOGRAM_fill(l+9,cosalpha,wgts)
    CALL HISTOGRAM_fill(l+10,deta_psipsi,wgts)
    CALL HISTOGRAM_fill(l+11,ptpsi1,wgts)
    CALL HISTOGRAM_fill(l+12,ptpsimax,wgts)
    CALL HISTOGRAM_fill(l+13,ptpsimin,wgts)
    CALL HISTOGRAM_fill(l+14,ptpsidiff,wgts)
    CALL HISTOGRAM_fill(l+15,Pt1Pt2_psipsi,wgts)
    CALL HISTOGRAM_fill3d(l+16,costh1,costh2,wgts)
    CALL HISTOGRAM_fill3d(l+17,dy_psipsi,Pt_psipsi,wgts)
    CALL HISTOGRAM_fill3d(l+18,dy_psipsi,dphi_psipsi,wgts)
    CALL HISTOGRAM_fill(l+19,Pt_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+20,Pt_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+21,Pt_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+22,Pt_psipsi_sm30,wgts)
    CALL HISTOGRAM_fill(l+23,dphi_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+24,dphi_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+25,dphi_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+26,dphi_psipsi_sm30,wgts)
    CALL HISTOGRAM_fill(l+27,dy_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+28,dy_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+29,dy_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+30,dy_psipsi_sm30,wgts)
    CALL HISTOGRAM_fill(l+31,M_psipsi_sm05,wgts)
    CALL HISTOGRAM_fill(l+32,M_psipsi_sm10,wgts)
    CALL HISTOGRAM_fill(l+33,M_psipsi_sm20,wgts)
    CALL HISTOGRAM_fill(l+34,M_psipsi_sm30,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_Decay

  SUBROUTINE plot_fill_QEDPS(wgts)
    USE QEDPS_interface
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    ! QEDPS parameters                                                      
    REAL(KIND(1d0)),DIMENSION(10,1000)::PLPTN
    INTEGER,DIMENSION(10,1000)::NLPTN
    INTEGER::NPTCL
    COMMON/QPLIST/PLPTN,NLPTN,NPTCL
    REAL(KIND(1d0)),DIMENSION(4)::pini
    REAL(KIND(1d0))::wgt,var,frache
    REAL(KIND(1d0))::exp1,exp2
    INTEGER::i,kk,l,nphoton
    REAL(KIND(1d0))::www,yftot
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! 
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    WRITE(*,*)"Please choose the correct analysis file plot_user.f90"
    STOP
    var=1.0d0 ! total cross section
    pini(1:4)=PLPTN(4:7,1)+PLPTN(4:7,2)
    yftot=rapidity(pini)
    nphoton=0
    DO i=1,NPTCL
       IF(NLPTN(1,i).EQ.22)THEN
          nphoton=nphoton+1
       ENDIF
    ENDDO
    frache=MIN(Q2OUT_QEDPS/Q2MAX_QEDPS,1d0)
    frache=DSQRT(frache)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,yftot,wgts)
    CALL HISTOGRAM_fill(l+3,DBLE(nphoton),wgts)
    CALL HISTOGRAM_fill(l+4,frache,wgts)
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN
  END SUBROUTINE plot_fill_QEDPS
END MODULE plot_user
