MODULE plot_user
!============================================================
!
! PLOT FILE FOR USER
!
!============================================================
  USE Helac_Global
  USE Kinetic_Func ! provides kinematic functions
  IMPLICIT NONE
  INTEGER::nwgt_analysis
  INTEGER::num_plots=2
  INTEGER,PARAMETER::n_width=1,n_height=1
CONTAINS
  SUBROUTINE plot_begin(nwgt,max_weight,weights_info)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::nwgt,max_weight
    !INTEGER,PARAMETER::num_plots=1
    CHARACTER(len=*),DIMENSION(max_weight),INTENT(IN)::weights_info
    INTEGER::i,kk,l
    CHARACTER(len=2)::index_string
    INCLUDE '../hbook/dbookf90.inc'
    INCLUDE '../hbook/histogram_common90.inc'
    top_yes=topdrawer_output
    gnu_yes=gnuplot_output
    root_yes=root_output
    hwu_yes=hwufile_output
    CALL HISTOGRAM_INIT(nwgt,weights_info)
    nwgt_analysis=nwgt
    IF(emep_ISR_shower.EQ.1)num_plots=num_plots+2
    IF(nwgt_analysis*16.GT.nplots/4)THEN
       WRITE(*,*) 'error in plot_begin: '//&
            'too many histograms, increase NPLOTS to',&
            nwgt_analysis*num_plots*4 ! 4 is internal in dbook,each plot has 4 histogram to estimate error
       STOP
    ENDIF
    ! bookup is defined in analysis/hbook/dbook.f
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    !
    ! enlarge num_plots at the begining
    !
    ! NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE**NOTE
    l=0
    CALL histogram_book(l+ 1,'total rate    ',&
         5,0.5d0,5.5d0,.FALSE.)
    CALL histogram_book(l+ 2, "m ",&
            100,2d0,5d0,.TRUE.)
    CALL histogram_book(l+ 3, "y ",&
         100,-5d0,5d0,.TRUE.)
    DO i=3,nhad
       l=3+(i-3)*2
       WRITE(index_string,'(I2)')i-2
       CALL histogram_book(l+ 1,'pt('//index_string//') ',&
            100,0d0,100d0,.TRUE.)
       CALL histogram_book(l+ 2,'y('//index_string//') ',&
            100,-5d0,5d0,.TRUE.)
    ENDDO

    CALL HISTOGRAM_BOOK_FINISH
    RETURN
  END SUBROUTINE plot_begin

  SUBROUTINE plot_end(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    CHARACTER(len=14)::ytit
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    INTEGER::i
    INTEGER::kk,l
    INCLUDE '../hbook/dbookf90.inc'
    CHARACTER(len=400)::psfile,rootfile
    !IF(topdrawer_output)CALL open_topdrawer_file
    !IF(gnuplot_output)CALL open_gnuplot_file
    !IF(root_output)CALL open_root_file
    IF(topdrawer_output.OR.gnuplot_output.OR.root_output)THEN
       CALL mclear
       CALL mclear_3d
       DO i=1,NPLOTS
          CALL mopera(i,'+',i,i,xnorm,0.d0)
          CALL mfinal(i)
          CALL mopera_3d(i,'+',i,i,xnorm,0.d0)
          CALL mfinal_3d(i)
       ENDDO
    ENDIF
    ytit='sigma per bin '
    IF(topdrawer_output)THEN
       CALL open_topdrawer_file
       CALL HISTOGRAM_END(1,ytit,' ',' ')
       CALL close_topdrawer_file
    ENDIF
    IF(gnuplot_output)THEN
       CALL open_gnuplot_file
       psfile=TRIM(process(2:10*nhad+1))//".ps"
       CALL HISTOGRAM_END(2,ytit,psfile,' ')
       CALL close_gnuplot_file
    ENDIF
    IF(root_output)THEN
       CALL open_root_file
       rootfile=TRIM(process(2:10*nhad+1))//".root"
       CALL HISTOGRAM_END(3,ytit,' ',rootfile)
       ! wirte the end of the .C file for root
       WRITE (96, 100)
100    FORMAT(/1X,&
            ' hohisto -> cd();',/1X,&
            ' if (histos -> GetEntries() > 0 ) then {',/1X,&
            '  histos->Write();',/1X,&
            '  hohisto -> Close();',/1X,&
            ' }',/1X,'}')
       CALL close_root_file
    ENDIF
    IF(hwufile_output)THEN
       CALL HISTOGRAM_END(4,ytit,' ',' ')
       CALL HwU_write_file(xnorm)
    ENDIF
    RETURN                
  END SUBROUTINE plot_end

  SUBROUTINE plot_fill(wgts)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(20,4)::p
    REAL(KIND(1d0)),DIMENSION(20,4)::phad
    REAL(KIND(1d0)),DIMENSION(4)::pboo,pini
    REAL(KIND(1d0)),DIMENSION(:),INTENT(IN)::wgts
    REAL(KIND(1d0))::wgt,var,q,e
    REAL(KIND(1d0))::exp1,exp2,yi,pti,mFS,yFS
    INTEGER::i,kk,l
    REAL(KIND(1d0))::www
    IF(wgts(1).EQ.0d0)RETURN
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    !
    ! phegas_pmom is the partonic momenta,i.e. if there is a final QQ~
    ! bound state, the hadronic momenta is pQ+pQ~
    ! 
    ! WARNING*WARNING*WARNING*WARNING*WARNING*WARNING*WARNING
    DO i=1,n
       p(i,1:4)=phegas_pmom(i,1:4)
    ENDDO
    kk=1
    DO i=1,nhad
       IF(ABS(iflh(i)).GT.100)THEN
          phad(i,1:4)=p(kk,1:4)+p(kk+1,1:4)
          kk=kk+2
       ELSE
          phad(i,1:4)=p(kk,1:4)
          kk=kk+1
       ENDIF
    ENDDO
    pini(1:4)=phad(1,1:4)+phad(2,1:4)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! DEFINE OBSERVABLES
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    var=1.0d0 ! total cross section
    mFS=DSQRT(pini(4)**2-pini(3)**2-pini(2)**2-pini(1)**2)
    yFS=rapidity(pini)
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !
    ! START TO FILL HISTOGRAM
    !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    l=0    
    CALL HISTOGRAM_fill(l+1,var,wgts)
    CALL HISTOGRAM_fill(l+2,mFS,wgts)
    CALL HISTOGRAM_fill(l+3,yFS,wgts)
    DO i=3,nhad
       l=3+(i-3)*2
       pti=DSQRT(phad(i,1)**2+phad(i,2)**2)
       yi=rapidity(phad(i,1:4))
       CALL HISTOGRAM_fill(l+1,pti,wgts)
       CALL HISTOGRAM_fill(l+2,yi,wgts)
    ENDDO
    IF(hwufile_output)THEN
       call HwU_add_points
    ENDIF
    RETURN      
  END SUBROUTINE plot_fill

  SUBROUTINE HwU_write_file(xnorm)
    USE Helac_Global
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::xnorm
    OPEN(unit=599,file=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.HwU',&
         status='unknown')
    !xnorm=1d0
    CALL HwU_output(599,xnorm)
    CLOSE(599)
    RETURN
  END SUBROUTINE HwU_write_file

  SUBROUTINE open_topdrawer_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    !common/cuseitmax/useitmax                                                  
    OPEN(unit=99,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.top',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_topdrawer_file

  SUBROUTINE close_topdrawer_file
    IMPLICIT NONE
    CLOSE(99)
    RETURN
  END SUBROUTINE close_topdrawer_file

  SUBROUTINE open_gnuplot_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=97,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.gnu',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_gnuplot_file

  SUBROUTINE close_gnuplot_file
    IMPLICIT NONE
    CLOSE(97)
    RETURN
  END SUBROUTINE close_gnuplot_file

  SUBROUTINE open_root_file
    USE Helac_Global
    IMPLICIT NONE
    LOGICAL::useitmax
    OPEN(unit=96,FILE=TRIM(output_dir)//TRIM(process(2:10*nhad+1))//'.C',&
         status='unknown')
    useitmax=.FALSE.
  END SUBROUTINE open_root_file

  SUBROUTINE close_root_file
    IMPLICIT NONE
    CLOSE(96)
    RETURN
  END SUBROUTINE close_root_file

END MODULE plot_user
