################################################################################
# 
# An ad hoc code for judging identical processes in HELAC-Onia
#
################################################################################

from __future__ import division

import atexit
import cmath
import cmd
import glob
import logging
import logging.config
import coloring_logging
import math
import optparse
import os
import pydoc
import random
import re
import shutil
import signal
import stat
import subprocess
import sys
import time
import traceback
import tarfile
import copy
import datetime
import itertools

try:
    import readline
    GNU_SPLITTING = ('GNU' in readline.__doc__)
except:
    GNU_SPLITTING = True

root_path = os.path.split(os.path.dirname(os.path.realpath( __file__ )))[0]
root_path = os.path.split(root_path)[0]
sys.path.insert(0, os.path.join(root_path,'bin'))

# usefull shortcut
pjoin = os.path.join
# Special logger for the Cmd Interface
logger = logging.getLogger('helaconia.stdout') # -> stdout
logger_stderr = logging.getLogger('helaconia.stderr') # ->stderr

import extended_cmd as cmd
import misc as misc
import files as files
import cluster as cluster
import helaconia_run_interface as run_interface
from __init__ import InvalidCmd, HELACOniaError, HODIR

#===============================================================================
# QCDIdenticalProcess
#===============================================================================
class QCDIdenticalProcess(object):

    def __init__(self, *arg, **opt):
        """ Special init tasks for the Loop Interface """
        logging.config.fileConfig(os.path.join(root_path, 'cluster', 'pythoncode', \
                                                   '.ho_logging.conf'))
        logging.root.setLevel(eval('logging.INFO'))
        info = misc.get_pkg_info()
        info_line = ""
        if info.has_key('version') and  info.has_key('date'):
            len_version = len(info['version'])
            len_date = len(info['date'])
            if len_version + len_date < 30:
                info_line = "#=         VERSION %s %s %s         =\n" % \
                            (info['version'],
                            (30 - len_version - len_date) * ' ',
                            info['date'])
        if info_line:
            info_line = info_line[1:]
        logger.info(\
        "\n============================================================\n" + \
        "=                                                          =\n" + \
        "=                     H E L A C - O n i a                  =\n" + \
        "=                                                          =\n" + \
        info_line + \
        "=                                                          =\n" + \
        "=                                                          =\n" + \
        "=       Author:        Hua-Sheng Shao                      =\n" + \
        "=   Affliation:       Peking University                    =\n" + \
        "=      Contact:      erdissshaw@gmail.com                  =\n" + \
        "=    Reference:   Comput.Phys.Commun. 184 (2013) 2562      =\n" + \
        "=        arXiv:           1212.5293                        =\n" + \
        "=        arXiv:           1507.03435                       =\n" + \
        "=    Webpage: http://helac-phegas.web.cern.ch/helac-phegas =\n" + \
        "=                                                          =\n" + \
        "============================================================")

        self.set_addon_process() # read addon information

        self.set_options() # read options from default.inp and user.inp

        cmd.Cmd.__init__(self, *arg, **opt)

    def initialstates(self,init1,init2):
        if (not isinstance(init1,list)) or (not isinstance(init2,list)):
            raise HELACOniaError("The two arguments should be two lists")
        if (not len(init1) == 2) or (not len(init2) == 2):
            raise HELACOniaError("The number of initial states is not equal 2")
        if init1 == init2:
            return True
        # q q~
        # q q
        # q g
        # g g
