#! /usr/bin/env python

import os, sys
import ROOT as r
from ROOT import TTree, TFile, AddressOf, gROOT


class RootTTree(object):
    """A classs for the implementations of TTree conventions 
    """
        
    @classmethod
    def LHE2RootTTree(cls,LHEF,ROOTF):
        """Convert LHE file to a Root TTree file"""

        try:
            #input_file = file(LHEF,'r')
            input_file = open(LHEF,'r')
        except:
            raise Exception, "The LHE file cannot be opened, please enter a vaild .lhe file. Exiting."
        

        output_file_name = ROOTF

        try:
            output_file = TFile(output_file_name, "RECREATE")
        except:
            raise Exception, "Cannot open output file named: " + output_file_name + "Exiting"

        output_tree = TTree("helaconia", "helaconia")

        # Setup output branches
        PID_v = r.vector('Int_t')()
        STATUS_v = r.vector('Int_t')()
        P_X_v = r.vector('Double_t')()
        P_Y_v =r.vector('Double_t')()
        P_Z_v =r.vector('Double_t')()
        E_v =r.vector('Double_t')()
        M_v =r.vector('Double_t')()
        

        # Create a struct which acts as the TBranch for non-vectors
        gROOT.ProcessLine( "struct MyStruct{ Int_t n_particles; Int_t process_ID; Double_t weight; Double_t scale; Double_t aem; Double_t astrong; };")
        from ROOT import MyStruct

        # Assign the variables to the struct
        s = MyStruct() 
        output_tree.Branch('n_particles',AddressOf(s,'n_particles'),'n_particles/I')
        output_tree.Branch('process_ID',AddressOf(s,'process_ID'),'process_ID/I')
        output_tree.Branch('weight',AddressOf(s,'weight'),'weight/D')
        output_tree.Branch('scale',AddressOf(s,'scale'),'scale/D')
        output_tree.Branch('aem',AddressOf(s,'aem'),'aem/D')
        output_tree.Branch('astrong',AddressOf(s,'astrong'),'astrong/D')
        output_tree.Branch("PID",PID_v)
        output_tree.Branch("P_X",P_X_v)
        output_tree.Branch("P_Y",P_Y_v)
        output_tree.Branch("P_Z",P_Z_v)
        output_tree.Branch("E",E_v)
        output_tree.Branch("M",M_v)
        output_tree.Branch("Status",STATUS_v) # 1, final state; -1, initial state; 2, intermediate resonace

        skippedLines = []
        in_ev = 0 # To know when to look for particles we must know when we are inside an event
        in_ev_1 = 0 # The first line after <event> is information so we must skip that as well
        s.n_particles = 0
        s.weight = 0
        for line in input_file:
            
            if line[:1] == "#":
                continue
    
            if in_ev_1 == 1:
                in_ev_1 = 0
                sline=line.split()
                s.n_particles=int(sline[0])
                s.process_ID=int(sline[1])
                s.weight = float(sline[2])
                s.scale = float(sline[3])
                s.aem = float(sline[4])
                s.astrong = float(sline[5])
                in_ev = 1
                continue
    
            if line.strip().startswith("<event>"):
                in_ev_1 = 1
                continue
    
            # Some versions of les houches have a pdf line that we don't care about here
            if line.startswith("#pdf"):
                continue
    
            if in_ev == 1 and (line.strip().startswith("</event>") or line.strip().startswith("<rwgt>")):
                output_tree.Fill()
                # Reset variables
                s.n_particles = 0
                s.process_ID = 0
                s.weight = 0
                s.scale = 0
                s.aem = 0
                s.astrong = 0
                PID_v.clear()
                STATUS_v.clear()
                P_X_v.clear()
                P_Y_v.clear()
                P_Z_v.clear()
                E_v.clear()
                M_v.clear()
                in_ev = 0
                continue
    
            if in_ev == 1:
                # Check the status of this particle
                try:
                    sline=line.split()
                    PID_v.push_back(int(sline[0]))
                    STATUS_v.push_back(int(sline[1]))
                    P_X_v.push_back(float(sline[6]))
                    P_Y_v.push_back(float(sline[7]))
                    P_Z_v.push_back(float(sline[8]))
                    E_v.push_back(float(sline[9]))
                    M_v.push_back(float(sline[10]))
                    pass
                except:
                    if line not in skippedLines:
                        print "Problem with line: ", line.replace("\n","")
                        print "Skipping..."
                        skippedLines.append( line )
                        pass
                    pass
                pass
            pass

        output_tree.Write()
        output_file.Close()
        input_file.close()


if __name__ == "__main__":
    LHEF="/Users/erdissshaw/Works/HELAC-Onia/test-onia2/PROC_HO_0/P0_calc_0/output/sampleggpsi1g.lhe"
    ROOTF="helaconia_lhe.root"
    RootTTree.LHE2RootTTree(LHEF,ROOTF)
