MODULE FFKretzer_interface
  IMPLICIT NONE
CONTAINS
  FUNCTION CallFFKretzer(z,scale,PDG_parton,PDG_hadron,ifforder,iffset)
    ! Call FF provided by Kretzer
    ! transfer the number density to the momentum density
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,scale
    INTEGER,INTENT(IN)::PDG_parton,PDG_hadron
    REAL(KIND(1d0))::CallFFKretzer,Q2
    INTEGER,OPTIONAL,INTENT(IN)::ifforder,iffset
    INTEGER::ipi,iparton,ihadron,icp,icharge,fforder,ffset
    REAL(KIND(1d0)),DIMENSION(-5:5)::dff
    INTEGER::print1=0,print2=0,print3=0,print4=0,print5=0,print6=0,print7=0
    SAVE print1,print2,print3,print4,print5,print6,print7
    IF(PRESENT(ifforder))THEN
       fforder=ifforder
       IF(fforder.LT.0.OR.fforder.GT.1)THEN
          IF(print1.EQ.0)THEN
             WRITE(*,*)"WARNING:order is invalid in CallFFKretzer ! Set it to be 0 (LO)"
             print1=1
          ENDIF
          fforder=0
       ENDIF
    ELSE
       fforder=0
    ENDIF
    IF(PRESENT(iffset))THEN
       ffset=iffset
       IF(ffset.LT.1.OR.ffset.GT.3)THEN
          IF(print2.EQ.0)THEN
             WRITE(*,*)"WARNING: set is invalid in CallFFKretzer ! Set it to be 2 (KKP)"
             print2=1
          ENDIF
          ffset=2
       ENDIF
    ELSE
       ffset=2
    ENDIF
    IF(ffset.EQ.3.AND.fforder.EQ.0)THEN
       IF(print3.EQ.0)THEN
          WRITE(*,*)"WARNING: No LO set for BFGW"
          WRITE(*,*)"WARNING: Choose NLO(MSbar) set instead"
          print3=1
       ENDIF
       ffset=1
    ENDIF
    Q2=scale**2
    SELECT CASE(ABS(PDG_parton))
    CASE(5,4,3)
       iparton=PDG_parton
    CASE(2)
       iparton=SIGN(1,PDG_parton)
    CASE(1)
       iparton=SIGN(2,PDG_parton)
    CASE(21)
       iparton=0
    CASE DEFAULT
       WRITE(*,*)"ERROR:Unknown PDG_parton in CallFFKretzer !"
       STOP 1
    END SELECT
    icp=1
    SELECT CASE(ABS(PDG_hadron))
    CASE(111,211)
       ! pions
       IF(ffset.EQ.3)THEN
          IF(print4.EQ.0)THEN
             WRITE(*,*)"WARNING: No pion set for BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print4=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=1
       SELECT CASE(PDG_hadron)
       CASE(111)
          icharge=0
       CASE(211)
          icharge=1
       CASE(-211)
          icharge=2
       CASE DEFAULT
          WRITE(*,*)"ERROR:Unknown PDG_hadron in CallFFKretzer!"
          STOP 2
       END SELECT
    CASE(311,321)
       ! kaon
       IF(ffset.EQ.3)THEN
          IF(print5.EQ.0)THEN
             WRITE(*,*)"WARNING: No kaon set for BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print5=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=2
       SELECT CASE(PDG_hadron)
       CASE(311)
          ! K0
          icharge=0
          icp=1
       CASE(-311)
          ! K0bar
          icharge=0
          icp=2
       CASE(321)
          ! K+
          icharge=1
          icp=1
       CASE(-321)
          ! K-
          icharge=2
          icp=1
       CASE DEFAULT
          WRITE(*,*)"ERROR:Unknown PDG_hadron in CallFFKretzer!"
          STOP 3
       END SELECT
    CASE(2212)
       ! proton
       IF(ffset.EQ.3.OR.ffset.EQ.1)THEN
          IF(print6.EQ.0)THEN
             WRITE(*,*)"WARNING: No proton set for K and BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print6=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=4
       IF(PDG_hadron.GT.0)THEN
          icharge=1
          icp=1
       ELSE
          icharge=2
          icp=1
       ENDIF
    CASE(2112)
       ! neutron
       IF(ffset.EQ.3.OR.ffset.EQ.1)THEN
          IF(print7.EQ.0)THEN
             WRITE(*,*)"WARNING: No neutron set for K,BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print7=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=5
       icharge=0
       IF(PDG_hadron.GT.0)THEN
          icp=1
       ELSE
          icp=2
       ENDIF
    CASE DEFAULT
       WRITE(*,*)"ERROR:Unknown PDG_hadron in CallFFKretzer!"
       STOP 4
    END SELECT
    ipi=1 ! a flag for BFGW, inactive for K, KKP
          ! we choose best fit here
    CALL dlib(z,Q2,dff,ffset,fforder,ihadron,icharge,icp,ipi)
    CallFFKretzer=dff(iparton)*z
    RETURN
  END FUNCTION CallFFKretzer

  SUBROUTINE CallingFFKretzer(z,scale,PDG_hadron,ifforder,iffset,dff)
    ! Call FF provided by Kretzer
    ! transfer the number density to the momentum density
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,scale
    INTEGER,INTENT(IN)::PDG_hadron
    REAL(KIND(1d0))::Q2
    INTEGER,OPTIONAL,INTENT(IN)::ifforder,iffset
    INTEGER::ipi,ihadron,icp,icharge,fforder,ffset
    REAL(KIND(1d0)),DIMENSION(-5:5),INTENT(OUT)::dff
    INTEGER::print1=0,print2=0,print3=0,print4=0,print5=0,print6=0,print7=0
    SAVE print1,print2,print3,print4,print5,print6,print7
    IF(PRESENT(ifforder))THEN
       fforder=ifforder
       IF(fforder.LT.0.OR.fforder.GT.1)THEN
          IF(print1.EQ.0)THEN
             WRITE(*,*)"WARNING:order is invalid in CallingFFKretzer ! Set it to be 0 (LO)"
             print1=1
          ENDIF
          fforder=0
       ENDIF
    ELSE
       fforder=0
    ENDIF
    IF(PRESENT(iffset))THEN
       ffset=iffset
       IF(ffset.LT.1.OR.ffset.GT.3)THEN
          IF(print2.EQ.0)THEN
             WRITE(*,*)"WARNING: set is invalid in CallingFFKretzer ! Set it to be 2 (KKP)"
             print2=1
          ENDIF
          ffset=2
       ENDIF
    ELSE
       ffset=2
    ENDIF
    IF(ffset.EQ.3.AND.fforder.EQ.0)THEN
       IF(print3.EQ.0)THEN
          WRITE(*,*)"WARNING: No LO set for BFGW"
          WRITE(*,*)"WARNING: Choose NLO(MSbar) set instead"
          print3=1
       ENDIF
       ffset=1
    ENDIF
    Q2=scale**2
    icp=1
    SELECT CASE(ABS(PDG_hadron))
    CASE(111,211)
       ! pions
       IF(ffset.EQ.3)THEN
          IF(print4.EQ.0)THEN
             WRITE(*,*)"WARNING: No pion set for BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print4=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=1
       SELECT CASE(PDG_hadron)
       CASE(111)
          icharge=0
       CASE(211)
          icharge=1
       CASE(-211)
          icharge=2
       CASE DEFAULT
          WRITE(*,*)"ERROR:Unknown PDG_hadron in CallingFFKretzer!"
          STOP 2
       END SELECT
    CASE(311,321)
       ! kaon
       IF(ffset.EQ.3)THEN
          IF(print5.EQ.0)THEN
             WRITE(*,*)"WARNING: No kaon set for BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print5=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=2
       SELECT CASE(PDG_hadron)
       CASE(311)
          ! K0
          icharge=0
          icp=1
       CASE(-311)
          ! K0bar
          icharge=0
          icp=2
       CASE(321)
          ! K+
          icharge=1
          icp=1
       CASE(-321)
          ! K-
          icharge=2
          icp=1
       CASE DEFAULT
          WRITE(*,*)"ERROR:Unknown PDG_hadron in CallingFFKretzer!"
          STOP 3
       END SELECT
    CASE(2212)
       ! proton
       IF(ffset.EQ.3.OR.ffset.EQ.1)THEN
          IF(print6.EQ.0)THEN
             WRITE(*,*)"WARNING: No proton set for K and BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print6=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=4
       IF(PDG_hadron.GT.0)THEN
          icharge=1
          icp=1
       ELSE
          icharge=2
          icp=1
       ENDIF
    CASE(2112)
       ! neutron
       IF(ffset.EQ.3.OR.ffset.EQ.1)THEN
          IF(print7.EQ.0)THEN
             WRITE(*,*)"WARNING: No neutron set for K,BFGW"
             WRITE(*,*)"WARNING: Using KKP instead"
             print7=1
          ENDIF
          ffset=2
       ENDIF
       ihadron=5
       icharge=0
       IF(PDG_hadron.GT.0)THEN
          icp=1
       ELSE
          icp=2
       ENDIF
    CASE DEFAULT
       WRITE(*,*)"ERROR:Unknown PDG_hadron in CallingFFKretzer!"
       STOP 4
    END SELECT
    ipi=1 ! a flag for BFGW, inactive for K, KKP
          ! we choose best fit here
    CALL dlib(z,Q2,dff,ffset,fforder,ihadron,icharge,icp,ipi)
    dff(-5:5)=dff(-5:5)*z
    RETURN
  END SUBROUTINE CallingFFKretzer

END MODULE FFKretzer_interface
