MODULE fragfun
  USE Helac_Global
  USE Helac_Func_1
  IMPLICIT NONE
CONTAINS
  FUNCTION ff_g_1S01g(z,mQ,mu0)
    IMPLICIT NONE
    REAL(KIND(1d0))::ff_g_1S01g
    REAL(KIND(1d0)),INTENT(IN)::z,mQ,mu0
    !INTEGER,INTENT(IN)::ord ! ord: 0, LO, 1, NLO etc
    REAL(KIND(1d0))::alp
    IF(z.LT.0d0.OR.z.GT.1d0)THEN
       ff_g_1S01g=0d0
       RETURN
    ENDIF
    alp=ALPHAS(mu0)
    ! Eqs.(45,60,61) in hep-ph/9610401
    ff_g_1S01g=2d0*(1d0-z)*DLOG(1d0-z)+z*(3d0-2d0*z)
    ff_g_1S01g=4d0*mQ*alp**2/144d0/mQ**4*ff_g_1S01g ! we should multiply 2*Nc to CS LDME
    RETURN
    !IF(ord.LE.0)RETURN
  END FUNCTION ff_g_1S01g
  
  FUNCTION ff_g_3S18(mQ,mu0)
    ! it is transverse polarized
    IMPLICIT NONE
    REAL(KIND(1d0))::ff_g_3S18
    REAL(KIND(1d0)),INTENT(IN)::mQ,mu0
    REAL(KIND(1d0))::alp
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846d0
    alp=ALPHAS(mu0)
    ! Eqs(62-63) in hep-ph/9610401
    ff_g_3S18=4d0*mQ*pi*alp/96d0/mQ**4 ! one should multiply (Nc^2-1) to CO LDME 
    RETURN
  END FUNCTION ff_g_3S18

  FUNCTION d_g_3S18g_L(z,mQ,mu0)
    IMPLICIT NONE
    REAL(KIND(1d0))::d_g_3S18g_L
    REAL(KIND(1d0)),INTENT(IN)::z,mQ,mu0
    REAL(KIND(1d0))::alp
    alp=ALPHAS(mu0)
    ! Eq.(39) in hep-ph/0004118
    IF(z.GT.1d0.OR.z.LT.0d0)THEN
       d_g_3S18g_L=0d0
       RETURN
    ENDIF
    alp=ALPHAS(mu0)
    d_g_3S18g_L=alp**2/8d0/mQ**3*(1d0-z)/z ! one should multiply (Nc^2-1) to CO LDME
    RETURN
  END FUNCTION d_g_3S18g_L

  FUNCTION d_g_3S18g_T_delta(mQ,mu0)
    ! delta(1-z)
    IMPLICIT NONE
    REAL(KIND(1d0))::d_g_3S18g_T_delta
    REAL(KIND(1d0)),INTENT(IN)::mQ,mu0
    REAL(KIND(1d0))::alp
    alp=ALPHAS(mu0)
    d_g_3S18g_T_delta=alp**2/48d0/mQ**3*A_g_3S18g(mQ,mu0) ! it is 1/2*result in arXiv:1311.7078
    RETURN
  END FUNCTION d_g_3S18g_T_delta

  FUNCTION d_g_3S18g_T_plus(z,mQ,mu0)
    ! plus function 1/(1-z)_+
    IMPLICIT NONE
    REAL(KIND(1d0))::d_g_3S18g_T_plus
    REAL(KIND(1d0)),INTENT(IN)::z,mQ,mu0
    REAL(KIND(1d0))::alp
    IF(z.GT.1d0.OR.z.LT.0d0)THEN
       d_g_3S18g_T_plus=0d0
       RETURN
    ENDIF
    alp=ALPHAS(mu0)
    d_g_3S18g_T_plus=alp**2/48d0/mQ**3*&
         (DLOG(mu0/2d0/mQ)-1d0/2d0)*6d0*z ! it also differs by a factor of 1/2 w.r.t arXiv:1311.7078
    RETURN
  END FUNCTION d_g_3S18g_T_plus

  FUNCTION d_g_3S18g_T_logplus(z,mQ,mu0)
    ! log plus function (log(1-z)/(1-z))_+
    IMPLICIT NONE
    REAL(KIND(1d0))::d_g_3S18g_T_logplus
    REAL(KIND(1d0)),INTENT(IN)::z,mQ,mu0
    REAL(KIND(1d0))::alp
    IF(z.LT.0d0.OR.z.GT.1d0)THEN
       d_g_3S18g_T_logplus=0d0
       RETURN
    ENDIF
    alp=ALPHAS(mu0)
    d_g_3S18g_T_logplus=-6d0/z*alp**2/48d0/mQ**3 ! it differ by a factor of (1-z+z^2)^2/2 wrt arXiv:1311.7078
    RETURN
  END FUNCTION d_g_3S18g_T_logplus

  FUNCTION d_g_3S18g_T(z,mQ,mu0)
    ! normal 1
    IMPLICIT NONE
    REAL(KIND(1d0))::d_g_3S18g_T
    REAL(KIND(1d0)),INTENT(IN)::z,mQ,mu0
    REAL(KIND(1d0))::alp
    IF(z.GT.1d0.OR.z.LT.0d0)THEN
       d_g_3S18g_T=0d0
       RETURN
    ENDIF
    alp=ALPHAS(mu0)
    d_g_3S18g_T=alp**2/48d0/mQ**3*&
         ((DLOG(mu0/2d0/MQ)-1d0/2d0)*&
         6d0*((1d0-z)/z+z*(1d0-z))+&
         6d0*(2d0-z+z**2)*DLOG(1d0-z)) ! it differs by a factor of 1/2 wrt arXiv:1311.7078
                                       ! the result in arXiv:1311.7078 also misses the last piece
    RETURN
  END FUNCTION d_g_3S18g_T
  
  FUNCTION A_g_3S18g(mQ,mu0)
    IMPLICIT NONE
    REAL(KIND(1d0))::A_g_3S18g
    REAL(KIND(1d0)),INTENT(IN)::mQ,mu0
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846d0
    A_g_3S18g=beta0(mu0)*(2d0*DLOG(mu0/2d0/mQ)+5d0/3d0)+&
         2d0/3d0-pi**2/2d0+8d0*DLOG(2d0)+6d0*DLOG(2d0)**2 ! log(2)**2 is different with Eq.(C11) in arXiv:1311.7078
    RETURN
  END FUNCTION A_g_3S18g

  FUNCTION beta0(mu0)
    IMPLICIT NONE
    REAL(KIND(1d0))::beta0
    REAL(KIND(1d0)),INTENT(IN)::mu0
    INTEGER::nf
    nf=2
    IF(mu0.LE.parmas(7))THEN
       nf=nf+1
    ELSEIF(mu0.LE.parmas(12))THEN
       nf=nf+2
    ELSEIF(mu0.LE.parmas(11))THEN
       nf=nf+3
    ELSE
       nf=nf+4
    ENDIF
    beta0=11d0-2d0/3d0*nf
    beta0=beta0/2d0
    RETURN
  END FUNCTION beta0
END MODULE fragfun
