MODULE FF_DGLAP
  USE ff_common
  USE FF_INIT
  IMPLICIT NONE
! ---------------------------------------------------------------------
!  FF_DGLAP.f    Ver 1.6                                   24/NOV/2011
!  Q^2 evolution of fragmentation functions
!  CHANGE TO F90 by Hua-Sheng Shao
!  on 20 May 2016
! ---------------------------------------------------------------------
CONTAINS
  
  SUBROUTINE FF_DGLAP_EVOL()
    IMPLICIT NONE
    ! ---------------------------------------------------------------------
    ! T=LOG(MU**2)
    REAL(KIND(1d0)),DIMENSION(600)::T ! the grid of T
    INTEGER::NGLI ! # of steps for Gauss-Legendre integral
    INTEGER::IORDER ! 1: LO, 2: NLO 
    REAL(KIND(1d0))::Q02,Q2 ! Q2 range for making grid files: Q02 -> Q2max
    REAL(KIND(1d0))::CTHRE,BTHRE ! m_c^2, m_b^2 for the mass threshold
    REAL(KIND(1d0))::ALP2PI ! as/Pi/2
    REAL(KIND(1d0))::SIGN
    REAL(KIND(1d0))::XMIN ! minimum of x
    REAL(KIND(1d0))::F ! real type of nf
    REAL(KIND(1d0))::DT
    REAL(KIND(1d0)),DIMENSION(600)::Q2T ! the grid of Q2 (i.e. the exponential of T)
    INTEGER::NT ! # of t-steps for Q^2 dependence
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX ! # of x-steps for xD(x)
    ! the varaibels defined in Eq.(13) of arXiv:1106.1553
    ! Y: xm/xk
    ! Z: xk
    ! W: (1-xm)/2*wk
    REAL(KIND(1d0)),DIMENSION(600,600)::Y,Z,W
    ! QQ is the x*FF at the initial scale
    REAL(KIND(1d0)),DIMENSION(600,10)::QQ,WQQ,B,C,D
    REAL(KIND(1d0)),DIMENSION(600,600,9)::WFIY ! WFIY = x*D(xm/xk,tl)
    COMMON /NISTEP/NGLI
    COMMON /ORD/IORDER
    COMMON /PRM2/Q02,Q2
    COMMON /HQMTHRE/CTHRE,BTHRE
    COMMON /ALP/ALP2PI
    COMMON /SIGN/SIGN
    COMMON /XMIN/XMIN
    COMMON /FLA/F
    COMMON /DELT/DT
    COMMON /QQT/Q2T,NT
    COMMON /XXX/X,NX
    COMMON /YZW/Y,Z,W
    COMMON /QQQ/QQ
    COMMON /WWW/WQQ
    COMMON /IPQ/B,C,D
    COMMON /WQSG/WFIY
    INTEGER::IERR
    INTEGER::M,IN,I,J,K,ISPFC
    INTEGER::NFI
    REAL(KIND(1d0))::YY

    ! READ INPUT PARAMETERS.
    CALL GETPA(IERR)  ! Reading input parameters 
    IF(IERR.EQ.1) STOP
      
    ! X, T, SPENCE FUNCITON, AND SPLITTING FUNCTIONS ARE CALCULATED.
    CALL GAUSSLEGZ(NGLI)             ! For GAUSSIAN-LEGENDRE
    CALL GETT(Q02,Q2,NT,DT,T,Q2T)    ! Divided T=ln(Q2)
    CALL GETX(NX,XMIN,1.D0,X,Y,Z,W)  ! Divided X,Y,Z, and W  
    CALL GETP(IORDER,SIGN)           ! Calculating splitting functions

    ! INITIAL DISTRIBUTIONS ARE STORED IN QQ(I,IN)
    M=1 ! Including non-singlet terms
    DO IN=M,9             ! Initializing
       DO I=1,NX+1
          QQ(I,IN)=0.D0
          WQQ(I,IN)=0.D0
          DO K=NGLI,1,-1
             WFIY(K,I,IN)=0.D0
          ENDDO
       END DO
    ENDDO

    ! running as/2/pi
    CALL GETALP(DLOG(Q02),IORDER,ALP2PI)
    NFI=3
    CALL GETINI(NFI,QQ) ! storing the x*FF (u,d,s,g only) at the initial scale in QQ
    CALL PUTFFG(1,QQ) ! put QQ into FFG (COMMON /FFG2D/FFG) at the 1st scale T
                      ! it is in different eigenvector of x*FF 

    ! START CALCULATING Q**2 EVOLUTION.
    !      L=1
    ISPFC=0     ! Flag of getting the splitting function of NF=4,5 
    DO J=2,NT+1

       ! DISTRIBUTION AT T=T(J)-DT.
       DO IN=M,4+NFI
          DO I=1,NX+1
             WQQ(I,IN)=QQ(I,IN)
          END DO
          CALL SPLINE(NX+1,X,WQQ,B,C,D,IN)
       ENDDO
       ! ----------------------------------------------------------------
       ! For FUNNS,FUNQ,FUNG,FUNQI in bfp.f
       DO I=1,NX+1
          DO K=NGLI,1,-1
             YY=Y(K,I) ! xm/xk
             DO IN=M,4+NFI
                ! WFIY = x*D(xm/xk,tl)
                WFIY(K,I,IN)=SEVAL(NX+1,YY,X,WQQ,B,C,D,IN)
             ENDDO
          ENDDO
       ENDDO
       !-----------------------------------------------------------------   
       ! Getting splitting functions for NF=4,5 above heavy quark mass thresholds
       IF((Q2T(J-1).LT.CTHRE).AND.(ISPFC.LT.1)) THEN
          F=3.D0;NFI=3;ISPFC=2
          CALL CHNGP(F) ! change flavor for splitting functions 

       ELSE IF((Q2T(J-1).GE.CTHRE).AND.(ISPFC.LT.3)) THEN ! Charm
          F=4.D0;NFI=4;ISPFC=4
          CALL CHANGENF(F,1)
          
       ELSE IF((Q2T(J-1).GE.BTHRE).AND.(ISPFC.LT.5)) THEN ! Bottom
          F=5.D0;NFI=5;ISPFC=6
          CALL CHANGENF(F,2)
       END IF
       !-----------------------------------------------------------------  
       ! DISTRIBUTION(S) AT T=TJ IS CALCULATED.
       CALL GETALP(T(J-1),IORDER,ALP2PI)
       CALL GETQQ(M,NFI,QQ)
       CALL PUTFFG(J,QQ)
    END DO

    RETURN
  END SUBROUTINE FF_DGLAP_EVOL

  SUBROUTINE GETPA(IERR)
    USE FF_DGLAP_GLOBAL
    !-------------------------------------------------------------------------
    !
    ! READ INPUT PARAMETERS.
    !-------------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER::IORDER
    COMMON /ORD/IORDER
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0))::SIGN
    COMMON /SIGN/SIGN
    REAL(KIND(1d0))::XMIN
    COMMON /XMIN/XMIN
    REAL(KIND(1d0))::Q02,Q2
    COMMON /PRM2/Q02,Q2
    REAL(KIND(1d0)),DIMENSION(600)::Q2T
    INTEGER::NT
    COMMON /QQT/Q2T,NT
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    REAL(KIND(1d0))::DLAML,DLAMN
    COMMON /LAM/DLAML,DLAMN
    REAL(KIND(1d0))::CTHRE,BTHRE
    COMMON /HQMTHRE/CTHRE,BTHRE
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::CF,CG,TR ! color factors, CF=4/3,CG=3,TR=0.5
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    INTEGER,INTENT(OUT)::IERR
    CHARACTER(len=76)::ini
    LOGICAL::loop=.TRUE.
    INTEGER,PARAMETER::init_unit=1033
    REAL(KIND(1d0))::THRE4,THRE5
    REAL(KIND(1d0))::DLAM ! Scale parameter in QCD
    INTEGER::NF

    !OPEN(unit=init_unit,file='setup.ini',status='OLD',FORM='formatted') 

    ! Read input prameters from setup.ini
    !Do while(loop)
    !   Read(init_unit,'(a22)',End=10) ini
    !   If(ini(1:1).EQ.'#') Then
    !      ! do nothing
    !   Else If(ini(1:6).EQ.'IORDER') Then
    !      Read(ini(8:22),*) IORDER
    IORDER=FF_DGLAP_IORDER
    !      
    !   Else If(ini(1:2).EQ.'Q2') Then
    !      Read(ini(4:22),*) Q02,Q2
    Q02=FF_DGLAP_Q2min
    Q2=FF_DGLAP_Q2max
    !   Else If(ini(1:4).EQ.'DLAM') Then
    !      Read(ini(6:22),*) DLAM
    !      DLAML=DLAM
    !      DLAMN=DLAM
    DLAML=FF_DGLAP_DLAM
    DLAMN=FF_DGLAP_DLAM
    !   Else If(ini(1:6).EQ.'HQTHRE') Then
    !      Read(ini(8:22),*) THRE4, THRE5
    THRE4=FF_DGLAP_mc_HQTHRE
    THRE5=FF_DGLAP_mb_HQTHRE
    !   Else If(ini(1:2).EQ.'NF') Then
    !      Read(ini(4:22),*) NF
    NF=FF_DGLAP_NF      
    !   Else If(ini(1:4).EQ.'XMIN') Then
    !      Read(ini(6:22),*) XMIN
    XMIN=FF_DGLAP_XMIN      
    !   Else If(ini(1:2).EQ.'NX') Then
    !      Read(ini(4:22),*) NX
    NX=FF_DGLAP_NX      
    !   Else If(ini(1:2).EQ.'NT') Then
    !      Read(ini(4:22),*) NT
    NT=FF_DGLAP_NT      
    !   Else If(ini(1:4).EQ.'NGLI') Then
    !      Read(ini(6:22),*) NGLI
    NGLI=FF_DGLAP_NGLI      
    !   End If
    !Enddo

!10  Continue  
    !Close(init_unit)

    

    ! Check input parameters
    IERR=0
    IF((IORDER.LT.1).OR.(IORDER.GT.2)) IERR=1
    IF((NX.LT.1).OR.(NX.GE.600)) IERR=1
    IF((NT.LT.1).OR.(NT.GE.600)) IERR=1
    IF((XMIN.LT.0).OR.(XMIN.GT.1.D0)) IERR=1

    IF(IERR.EQ.1) THEN
       WRITE(*,*)  'INPUT PARAMETER ERROR'
    END IF

    IF(IERR.EQ.1) GO TO 30
    
    ! Get other prameters
    SIGN=-1.D0
    
    PI=4.D0*DATAN(1.D0)
    F=DBLE(NF)
    
    CF=4.D0/3.D0
    CG=3.D0
    TR=1.D0/2.D0
    
    CTHRE=THRE4*THRE4
    BTHRE=THRE5*THRE5
    
30  CONTINUE

    Return
  End SUBROUTINE GETPA

  SUBROUTINE GETT(Q02,Q2,NT,DT,T,Q2T)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::NT
    REAL(KIND(1d0)),INTENT(IN)::Q02,Q2
    REAL(KIND(1d0)),INTENT(OUT)::DT
    REAL(KIND(1d0)),DIMENSION(600),INTENT(OUT)::T,Q2T
    INTEGER::J
    REAL(KIND(1d0))::TMIN,TMAX
    
    TMIN=DLOG(Q02)
    TMAX=DLOG(Q2)
    DT=(TMAX-TMIN)/DBLE(NT)

    DO J=1,NT+1
       T(J)=TMIN+DBLE(J-1)*DT
       Q2T(J)=DEXP(T(J))
    ENDDO

    RETURN
  END SUBROUTINE GETT

  SUBROUTINE GETX(NX,XMIN,XMAX,X,Y,Z,W)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::NX
    REAL(KIND(1d0)),INTENT(IN)::XMIN,XMAX
    INTEGER::NXOS,NTX,NXGL
    REAL(KIND(1d0)),DIMENSION(600),INTENT(OUT)::X
    REAL(KIND(1d0)),DIMENSION(600,600),INTENT(OUT)::Y,Z,W
    REAL(KIND(1d0)),DIMENSION(600)::B,C
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0))::XMIDL=0.1D0,XESP=1.D-11
    INTEGER::I,K
    REAL(KIND(1d0))::ZZ
    REAL(KIND(1d0))::DLMAX,DLMIN,XLSTEP,DLOGX,DX
    REAL(KIND(1d0))::DXMIN

    DXMIN=-DLOG10(XMIN)
    NXOS=IDINT(DXMIN+XESP) ! XMIN=10^-10, NXOS=10, XMIN=2*10^-10, NXOS=9
    NTX=NX/NXOS ! NTX grid for 10^-(i+1) to 10^(-i), i=0,...,NXOS-1
    NXGL=(NXOS-1)*NTX ! the total number of points from XMIN - 0.1

    ! setting up the x grid
    ! Log scale step below x=0.1
    DLMAX=DLOG10(XMIDL) ! -1
    DLMIN=DLOG10(XMIN)  ! -10 when XMIN=10^-10
    XLSTEP=(DLMAX-DLMIN)/DBLE(NXGL)
    DO I=1,NXGL+1
       DLOGX=DLMIN+DBLE(I-1)*XLSTEP
       X(I)=10.D0**(DLOGX)
    END DO

    ! Liner scale step avobe x=0.1
    DX=(XMAX-XMIDL)/DBLE(NTX)
    DO I=1, NTX
       X(NXGL+I)=XMIDL+DBLE(I-1)*DX
    END DO
    X(NX+1)=XMAX

    ! Caluculation of Y,Z,W for Gauss-Legendre integral
    DO I=1,NX
       IF(X(I).LT.XMIDL) THEN
          ! C = wi*(DLOG10(XMAX)-DLOG10(X(I)))*0.5
          ! i.e. C is the prefactor of rhs of Eq.(12) in arXiv:1106.1553
          ! B = (DLOG10(XMAX)+DLOG10(X(I)))*0.5-(DLOG10(XMAX)-DLOG10(X(I)))*0.5*z
          ! where z is i-th root of Pn[z]==0
          ! i.e. B=xk in Eq.(12) of arXiv:1106.1553
          CALL GAULEG(NGLI,DLOG10(X(I)),DLOG10(XMAX),B,C)
          DO K=1,NGLI
             ZZ=10.D0**B(K) ! xk=10^(log(xk))
             Y(K,I)=X(I)/ZZ ! xm/xk
             Z(K,I)=ZZ ! xk
             W(K,I)=C(K) ! (1-xm)/2*wk
          ENDDO
       ELSE
          CALL GAULEG(NGLI,X(I),XMAX,B,C)
          DO K=1,NGLI
             Y(K,I)=X(I)/B(K) ! xm/xk in Eq.(13) of arXiv:1106.1553
             Z(K,I)=B(K) ! xk
             W(K,I)=C(K) ! (1-xm)/2*wk in Eq.(13) of arXiv:1106.1553
          ENDDO
       ENDIF
    ENDDO
    
    RETURN
  END SUBROUTINE GETX

  SUBROUTINE GETP(IORDER,SIGN)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::IORDER
    REAL(KIND(1d0)),INTENT(IN)::SIGN
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON/XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,600)::Y,ZZ,W
    COMMON /YZW/Y,ZZ,W
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0,PNS1
    COMMON /PNS/PNS0,PNS1
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0P,PNS1P
    COMMON /PNSP/PNS0P,PNS1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PGG0P,PGG1P
    COMMON /PSIP/PGG0P,PGG1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PPS1
    REAL(KIND(1d0)),DIMENSION(600,600,4)::P0,P1
    COMMON /PSI/PPS1,P0,P1
    REAL(KIND(1d0)),DIMENSION(600,600,3)::PNS1F,PNS1PF
    COMMON /PNS1F/PNS1F,PNS1PF
    REAL(KIND(1d0)),DIMENSION(600,600,3)::PGG1PF
    REAL(KIND(1d0)),DIMENSION(600,600,4,3)::P1F
    COMMON /PSIF/PGG1PF,P1F
    INTEGER::I,K,N

    DO I=1,NX
       DO K=1,NGLI
          ! ZZ(K,I) is xk in grid of x0
          PNS0(K,I)=PNSLO(ZZ(K,I)) ! LO nonregularized Pqq=CF*(1+x^2)/(1-x)
          PNS0P(K,I)=PNS0PL(ZZ(K,I)) ! -2*CF/(1-x)
          ! LO (nonregularized) splitting functions (w/o plus and delta inEqs.(21-24))
          ! in orders PQQ0,PQG0,PGQ0,PGG0
          ! Matrix is transposed in DGLAP eq for FF: PQG -> 2*PGQ, PGQ -> PQG 
          CALL PLO(ZZ(K,I),P0(K,I,1),P0(K,I,2),P0(K,I,3),P0(K,I,4))
          PGG0P(K,I)=PGG0PL(ZZ(K,I)) ! -2*CG/(1-x)
       END DO
    END DO
      
    IF(IORDER.EQ.2) THEN
       DO I=1,NX
          DO K=1,NGLI
             PPS1(K,I)=PPSNLO(ZZ(K,I))
             DO N=1,3
                F=DBLE(N+2)
                PNS1F(K,I,N)=PNSNLO(ZZ(K,I),SIGN)
                PNS1PF(K,I,N)=PNS1PL(ZZ(K,I))
                CALL PNLO(ZZ(K,I),P1F(K,I,1,N),P1F(K,I,2,N) &
                     ,P1F(K,I,3,N),P1F(K,I,4,N))
                PGG1PF(K,I,N)=PGG1PL(ZZ(K,I))
             ENDDO
          END DO
       END DO
    END IF
    
    RETURN
  END SUBROUTINE GETP

  SUBROUTINE CHNGP(F)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::F
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON/XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0,PNS1
    COMMON /PNS/PNS0,PNS1
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0P,PNS1P
    COMMON /PNSP/PNS0P,PNS1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PGG0P,PGG1P
    COMMON /PSIP/PGG0P,PGG1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PPS1
    REAL(KIND(1d0)),DIMENSION(600,600,4)::P0,P1
    COMMON /PSI/PPS1,P0,P1
    REAL(KIND(1d0)),DIMENSION(600,600,3)::PNS1F,PNS1PF
    COMMON /PNS1F/PNS1F,PNS1PF
    REAL(KIND(1d0)),DIMENSION(600,600,3)::PGG1PF
    REAL(KIND(1d0)),DIMENSION(600,600,4,3)::P1F
    COMMON /PSIF/PGG1PF,P1F
    INTEGER::N,I,K,L

    N=DINT(F-2)
    DO I=1,NX
       DO K=1,NGLI
          PNS1(K,I)=PNS1F(K,I,N)
          PNS1P(K,I)=PNS1PF(K,I,N)
          PGG1P(K,I)=PGG1PF(K,I,N)
          DO L=1,4
             P1(K,I,L)=P1F(K,I,L,N)
          ENDDO
       ENDDO
    ENDDO
    
    RETURN
  END SUBROUTINE CHNGP


  SUBROUTINE GETQQ(M,NFI,QQ)
    ! ---------------------------------------------------------------------
    ! 
    ! DISTRIBUTION(S) AT T=TJ IS CALCULATED.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(OUT)::QQ
    INTEGER,INTENT(IN)::M,NFI
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0)),DIMENSION(600,3)::WNS
    COMMON /WQNS/WNS
    REAL(KIND(1d0)),DIMENSION(600)::WQY,WGY,WIY
    COMMON /WQGI/WQY,WGY,WIY
    REAL(KIND(1d0)),DIMENSION(600,600,9)::WFIY
    COMMON /WQSG/WFIY
    INTEGER::I,IN,K
    
    DO I=1,NX
       IF(M.EQ.1) THEN
          DO IN=1,2         ! IN=1:uv, 2:dv
             DO K=1,NGLI
                WNS(K,IN)=WFIY(K,I,IN)
             ENDDO
             QQ(I,IN)=QNSXT(I,IN) ! From Eq.(13) we know QQ is xm*D(xm,tl+1) now
          ENDDO
       ENDIF
       
       DO K=1,NGLI
          WGY(K)=WFIY(K,I,3) ! gluon
          WQY(K)=WFIY(K,I,4) ! singlet
       ENDDO
       
       ! it is the coupled evol
       CALL QSGXT(I,QQ(I,3),QQ(I,4))  ! 3:gluon, 4:singlet, 
       
       DO IN=5,4+NFI
          DO K=1,NGLI
             WIY(K)=WFIY(K,I,IN)
          ENDDO
          QQ(I,IN)=QIFXT(I,IN)
       END DO
    END DO

    RETURN
  END SUBROUTINE GETQQ


  FUNCTION QNSXT(I,IN)
    ! ---------------------------------------------------------------------
    ! CALCULATE Q**2 EVOLVED NONSINGLET QUARK DISTRIBUTION.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::I,IN
    REAL(KIND(1d0))::QNSXT
    !REAL(KIND(1d0)),EXTERNAL::FUNNS
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0))::DT
    COMMON /DELT/DT
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ ! xm*D(xm)
    COMMON /WWW/WQQ
    REAL(KIND(1d0))::SUMNS,BFINS

    CALL GLSUM(I,FUNNS,IN,SUMNS)

    ! second term is delta function and one-plus function piece (0,x0)
    BFINS=SUMNS+PNS0DL(X(I))*WQQ(I,IN)

    IF(IORDER.EQ.2) THEN
       BFINS=BFINS+ALP2PI*PNS1DL(X(I))*WQQ(I,IN)
    END IF
    ! rhs of Eq.(13) of arXiv:1106.1553
    QNSXT=WQQ(I,IN)+DT*ALP2PI*BFINS
    
    RETURN
  END FUNCTION QNSXT

  SUBROUTINE QSGXT(I,GGXT,QSXT)
    ! ---------------------------------------------------------------------
    ! CALCULATE Q**2 EVOLVED SINGLET QUARK AND GLUON DISTRIBUTIONS.
    ! --------------------------------------------------------------------- 
    ! coupled one can be seen in Eq.(2.9) of arXiv:1005.1481
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(OUT)::GGXT,QSXT
    INTEGER,INTENT(IN)::I
    !REAL(KIND(1d0)),EXTERNAL::FUNQ,FUNG
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0))::DT
    COMMON /DELT/DT
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ
    COMMON /WWW/WQQ
    REAL(KIND(1d0))::SUMGG,SUMQS
    REAL(KIND(1d0))::BFIGG,BFIQS
    
    CALL GLSUM(I,FUNG,3,SUMGG)
    CALL GLSUM(I,FUNQ,4,SUMQS)
    
    ! second term is delta function and one-plus function piece (0,x0)
    BFIGG=SUMGG+PGG0DL(X(I))*WQQ(I,3)
    BFIQS=SUMQS+PNS0DL(X(I))*WQQ(I,4)
    
    IF(IORDER.EQ.2) THEN
       BFIGG=BFIGG+ALP2PI*PGG1DL(X(I))*WQQ(I,3)
       BFIQS=BFIQS+ALP2PI*PNS1DL(X(I))*WQQ(I,4)
    END IF
    
    GGXT=WQQ(I,3)+DT*ALP2PI*BFIGG
    QSXT=WQQ(I,4)+DT*ALP2PI*BFIQS
    
    RETURN
  END SUBROUTINE QSGXT

  FUNCTION QIFXT(I,IN)
    ! ---------------------------------------------------------------------
    ! CALCULATE Q**2 EVOLVED q_i^+ QUARK DISTRIBUTIONS.
    ! --------------------------------------------------------------------- 
    ! one can be seen in Eq.(2.12) of arXiv:1005.1481
    ! however, Eq.(2.12) is q_i^+-q_j^+
    IMPLICIT NONE
    INTEGER,INTENT(IN)::I,IN
    REAL(KIND(1d0))::QIFXT
    !REAL(KIND(1d0)),EXTERNAL::FUNQI
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0))::DT
    COMMON /DELT/DT
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ
    COMMON /WWW/WQQ
    REAL(KIND(1d0))::SUM,BFIQI

    CALL GLSUM(I,FUNQI,IN,SUM)
    
    ! second term is delta function and one-plus function piece (0,x0)
    BFIQI=SUM+PNS0DL(X(I))*WQQ(I,IN)
    
    IF(IORDER.EQ.2) THEN
       BFIQI=BFIQI+ALP2PI*PNS1DL(X(I))*WQQ(I,IN)
    END IF
    
    QIFXT=WQQ(I,IN)+DT*ALP2PI*BFIQI

    RETURN
  END FUNCTION QIFXT
  
  FUNCTION FUNNS(K,I,IN)
    ! ---------------------------------------------------------------------
    ! it is P(xk)*xm/xk*D(xm/xk,tl) where P(xk) is NON-SINGLET SP
    IMPLICIT NONE
    INTEGER,INTENT(IN)::K,I,IN
    REAL(KIND(1d0))::FUNNS
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ ! xm*D(xm)
    COMMON /WWW/WQQ
    REAL(KIND(1d0)),DIMENSION(600,3)::WNS
    COMMON /WQNS/WNS
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0,PNS1
    COMMON /PNS/PNS0,PNS1
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0P,PNS1P
    COMMON /PNSP/PNS0P,PNS1P
    REAL(KIND(1d0))::WQNSY,FUNNS1
    
    WQNSY=WNS(K,IN) ! xm/xk*D(xm/xk,tl) with fixed xm
    
    ! second term is plus function
    FUNNS=PNS0(K,I)*WQNSY+PNS0P(K,I)*WQQ(I,IN)
    
    IF(IORDER.EQ.2) THEN
       FUNNS1=PNS1(K,I)*WQNSY+PNS1P(K,I)*WQQ(I,IN)
       FUNNS=FUNNS+ALP2PI*FUNNS1
    END IF
    
    RETURN
  END FUNCTION FUNNS

  FUNCTION FUNQ(K,I,IN)
    ! ---------------------------------------------------------------------
    ! it is Sum(P(xk)*xm/xk*D(xm/xk,tl)) where P(xk) is SP
    ! the sum can be seen as rhs of Eq.(2.9) in arXiv:1005.1481
    ! it is for quark at the lhs of Eq.(2.9)
    IMPLICIT NONE
    REAL(KIND(1d0))::FUNQ
    REAL(KIND(1d0))::F
    INTEGER,INTENT(IN)::K,I,IN
    COMMON /FLA/F
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0)),DIMENSION(600)::WQY,WGY,WIY
    COMMON /WQGI/WQY,WGY,WIY
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ
    COMMON /WWW/WQQ
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0P,PNS1P
    COMMON /PNSP/PNS0P,PNS1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PPS1
    REAL(KIND(1d0)),DIMENSION(600,600,4)::P0,P1
    COMMON /PSI/PPS1,P0,P1
    REAL(KIND(1d0))::WGGY,WQSY,FUNQ1

    WGGY=WGY(K) ! xm/xk*Dg(xm/xk,tl)
    WQSY=WQY(K) ! xm/xk*Ds(xm/xk,tl)

    ! P0(_,_,i) in orders PQQ0,PQG0,PGQ0,PGG0
    ! Matrix is transposed in DGLAP eq for FF: PQG -> 2*PGQ, PGQ -> PQG
    ! the second term is a plus function
    FUNQ=P0(K,I,1)*WQSY+PNS0P(K,I)*WQQ(I,IN) &
       +F*P0(K,I,2)*WGGY ! 2 has been multiplied in P0
    ! 2*nf*Pgq, factor 2 has been multiplied in SUBROUTINE PLO

    IF(IORDER.EQ.2) THEN
       FUNQ1=(P1(K,I,1)+F*PPS1(K,I))*WQSY+PNS1P(K,I)*WQQ(I,IN) &
            +F*P1(K,I,2)*WGGY
       FUNQ=FUNQ+ALP2PI*FUNQ1
    END IF
    
    RETURN
  END FUNCTION FUNQ

  FUNCTION FUNG(K,I,IN)
    ! ---------------------------------------------------------------------
    ! it is Sum(P(xk)*xm/xk*D(xm/xk,tl)) where P(xk) is SP
    ! the sum can be seen as rhs of Eq.(2.9) in arXiv:1005.1481
    ! it is for gluon at the lhs of Eq.(2.9)
    IMPLICIT NONE
    REAL(KIND(1d0))::FUNG
    INTEGER,INTENT(IN)::K,I,IN
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0)),DIMENSION(600)::WQY,WGY,WIY
    COMMON /WQGI/WQY,WGY,WIY
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ
    COMMON /WWW/WQQ
    REAL(KIND(1d0)),DIMENSION(600,600)::PGG0P,PGG1P
    COMMON /PSIP/PGG0P,PGG1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PPS1
    REAL(KIND(1d0)),DIMENSION(600,600,4)::P0,P1
    COMMON /PSI/PPS1,P0,P1
    REAL(KIND(1d0))::WGGY,WQSY,FUNG1

    WGGY=WGY(K) ! xm/xk*Dg(xm/xk,tl)
    WQSY=WQY(K) ! xm/xk*Ds(xm/xk,tl)

    ! P0(_,_,i) in orders PQQ0,PQG0,PGQ0,PGG0
    ! Matrix is transposed in DGLAP eq for FF: PQG -> 2*PGQ, PGQ -> PQG
    ! the third term is a plus function
    FUNG=P0(K,I,3)*WQSY &
         +P0(K,I,4)*WGGY+PGG0P(K,I)*WQQ(I,IN)

    IF(IORDER.EQ.2) THEN
       FUNG1=P1(K,I,3)*WQSY &
            +P1(K,I,4)*WGGY+PGG1P(K,I)*WQQ(I,IN)
       FUNG=FUNG+ALP2PI*FUNG1
    END IF

    RETURN
  END FUNCTION FUNG

  FUNCTION FUNQI(K,I,IN)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::FUNQI
    INTEGER,INTENT(IN)::K,I,IN
    INTEGER::IORDER
    COMMON /ORD/IORDER
    REAL(KIND(1d0))::ALP2PI
    COMMON /ALP/ALP2PI
    REAL(KIND(1d0)),DIMENSION(600)::WQY,WGY,WIY
    COMMON /WQGI/WQY,WGY,WIY
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ
    COMMON /WWW/WQQ
    REAL(KIND(1d0)),DIMENSION(600,600)::PNS0P,PNS1P
    COMMON /PNSP/PNS0P,PNS1P
    REAL(KIND(1d0)),DIMENSION(600,600)::PPS1
    REAL(KIND(1d0)),DIMENSION(600,600,4)::P0,P1
    COMMON /PSI/PPS1,P0,P1
    REAL(KIND(1d0))::WGGY,WQSY,WQIY
    REAL(KIND(1d0))::FUNQI1

    WGGY=WGY(K) ! xm/xk*Dg(xm/xk,tl)
    WQSY=WQY(K) ! xm/xk*Ds(xm/xk,tl)
    WQIY=WIY(K) ! xm/xk*D+(xm/xk,tl)

    ! P0(_,_,i) in orders PQQ0,PQG0,PGQ0,PGG0
    ! Matrix is transposed in DGLAP eq for FF: PQG -> 2*PGQ, PGQ -> PQG
    ! the second term is a plus function
    FUNQI=P0(K,I,1)*WQIY+PNS0P(K,I)*WQQ(I,IN) &
         +P0(K,I,2)*WGGY ! 2 has been multiplied in P0

    IF(IORDER.EQ.2) THEN
       FUNQI1=P1(K,I,1)*WQIY+PNS1P(K,I)*WQQ(I,IN) &
            +PPS1(K,I)*WQSY+P1(K,I,2)*WGGY
       FUNQI=FUNQI+ALP2PI*FUNQI1
    END IF
    
    RETURN
  END FUNCTION FUNQI

  SUBROUTINE GLSUM(I,FUN,IN,SUM)
    ! ---------------------------------------------------------------------
    ! the second term of the rhs of Eq.(13)
    ! w/o dt and as/2/pi
    IMPLICIT NONE
    REAL(KIND(1d0)),PARAMETER::XMIDL=0.1D0
    INTEGER::NGLI  
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,600)::Y,Z,W
    COMMON/YZW/Y,Z,W
    INTEGER,INTENT(IN)::I,IN
    REAL(KIND(1d0)),EXTERNAL::FUN
    REAL(KIND(1d0)),INTENT(OUT)::SUM
    INTEGER::K

    SUM=0.D0
    IF(X(I).LT.XMIDL) THEN
       ! LOG SCALE when x < 0.1
       DO K=1,NGLI
          ! W is (1-xm)/2*wk
          SUM=SUM+W(K,I)*FUN(K,I,IN)*Z(K,I) ! xf(x)
       ENDDO
       SUM=SUM*DLOG(10.D0)
    ELSE
       ! LIN SCALE when x > 0.1
       DO K=1,NGLI
          SUM=SUM+W(K,I)*FUN(K,I,IN)
       ENDDO
    ENDIF
    
    RETURN
  END SUBROUTINE GLSUM
  
  SUBROUTINE GAUSSLEGZ(N)
    ! ---------------------------------------------------------------------
    ! Reference: pp. 179-188, Numerical Recipes: 
    !            The Art of Scientific Computing, Third edition,
    !            Cambridge University Press, 2007, W. H. Press et al.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N
    REAL(KIND(1d0))::EPS=1.D-14
    REAL(KIND(1d0)),DIMENSION(600)::GLZ,GLP2
    COMMON/GAUS2L/GLZ,GLP2
    REAL(KIND(1d0))::PI
    INTEGER::M,i,j
    REAL(KIND(1d0))::Z,Z1
    REAL(KIND(1d0))::p1,p2,p3,pp

    PI=4.D0*DATAN(1.D0)

    m=(n+1)/2
    DO i=1,m
       z=DCOS(PI*(DBLE(i)-0.25d0)/(DBLE(n)+0.5d0))
1      CONTINUE
       p1=1.d0
       p2=0.d0
       DO j=1,n
          p3=p2
          p2=p1
          ! the relation
          ! Pn[x]=(Pn-1[x]*x*(2n-1)-(n-1)*Pn-2[x])/n
          p1=((2.d0*DBLE(j)-1.d0)*z*p2-(DBLE(j)-1.d0)*p3)/DBLE(j)
       ENDDO
       ! p3 is Pn-2[z]
       ! p2 is Pn-1[z]
       ! p1 is the LegendreP, i.e. Pn[z]
       ! pp is the derivation of the LegendreP,D[Pn[z],z]
       pp=DBLE(n)*(z*p1-p2)/(z*z-1.d0)
       z1=z
       z=z1-p1/pp
       IF(DABS(z-z1).GT.EPS) GOTO 1
       ! as wi is related to i-th root of Pn
       ! see e.g. https://en.wikipedia.org/wiki/Gaussian_quadrature
       ! wi = 2/(1-zi**2)/(D[Pn[z],z]|z=zi)**2
       GLZ(I)=z ! Pn[z]==0
       GLP2(I)=pp ! D[Pn[z],z] with Pn[z]==0
    ENDDO
    
    RETURN
  END SUBROUTINE GAUSSLEGZ

  SUBROUTINE GAULEG(N,x1,x2,X,W)
    ! ---------------------------------------------------------------------
    ! W=wi*(x2-x1)*0.5
    ! i.e. W is the prefactor of rhs of Eq.(12) of arXiv:1106.1553
    ! X=(x2+x1)*0.5-(x2-x1)*0.5*z
    ! where z is the root of Pn[z]==0
    ! i.e. X=xk in Eq.(12) of arXiv:1106.1553
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N
    REAL(KIND(1d0)),INTENT(IN)::x1,x2
    REAL(KIND(1d0)),DIMENSION(N),INTENT(OUT)::X
    REAL(KIND(1d0)),DIMENSION(N),INTENT(OUT)::W
    REAL(KIND(1d0)),DIMENSION(600)::GLZ,GLP2 ! GLZ: i-th root of Pn[z]==0
                                             ! GLP2: D[Pn[z],z] with Pn[z]==0  
    COMMON/GAUS2L/GLZ,GLP2
    INTEGER::m,i
    REAL(KIND(1d0))::xm,xl,z,pp

    m=(n+1)/2
    xm=0.5d0*(x2+x1)
    xl=0.5d0*(x2-x1)
    DO i=1,m
       z=GLZ(i)
       pp=GLP2(i)
       x(i)=xm-xl*z
       x(n+1-i)=xm+xl*z
       w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
       w(n+1-i)=w(i)
    ENDDO
    
    RETURN
  END SUBROUTINE GAULEG

  FUNCTION GAUSSI(XMIN,XMAX,FUN)
    ! ---------------------------------------------------------------------
    ! integration, i.e.
    ! rhs of Eq.(12) in arXiv:1106.1553
    IMPLICIT NONE
    REAL(KIND(1d0))::GAUSSI
    REAL(KIND(1d0)),INTENT(IN)::XMIN,XMAX
    REAL(KIND(1d0)),EXTERNAL::FUN
    REAL(KIND(1d0)),DIMENSION(600)::X,W
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    INTEGER::I

    CALL GAULEG(NGLI,XMIN,XMAX,X,W)
    GAUSSI=0.D0
    DO I=1,NGLI
       GAUSSI=GAUSSI+FUN(X(I))*W(I)
    ENDDO

    RETURN
  END FUNCTION GAUSSI
  
  FUNCTION SS2GL(XMIN,XMAX)
    ! ---------------------------------------------------------------------
    ! integration of FUN[X]=log((1-x)/x)/x
    ! same as GAUSSI(XMIN,XMAX,FUN) as FUN is sepcific
    IMPLICIT NONE
    REAL(KIND(1d0))::SS2GL
    REAL(KIND(1d0)),INTENT(IN)::XMIN,XMAX
    REAL(KIND(1d0)),DIMENSION(600)::X,W
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    INTEGER::I
    REAL(KIND(1d0))::GAUSSI
    REAL(KIND(1d0))::SS
    
    CALL GAULEG(NGLI,XMIN,XMAX,X,W)
    GAUSSI=0.D0
    DO I=1,NGLI
       SS=DLOG((1.D0-X(I))/X(I))/X(I)  ! For S2 function
       GAUSSI=GAUSSI+SS*W(I)
    ENDDO
    SS2GL=GAUSSI
    
    RETURN
  END FUNCTION SS2GL

  FUNCTION PNSLO(Z)
    ! ---------------------------------------------------------------------
    ! CALCULATE LO NONSINGLET SPLITTING FUNCTION.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PNSLO
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    
    PNSLO=CF*(1.D0+Z*Z)/(1.D0-Z)
    
    RETURN
  END FUNCTION PNSLO

  FUNCTION PNS0PL(Z)
    ! ---------------------------------------------------------------------
    ! CALCULATE LO NONSINGLET SPLITTING FUNCTION (PLUS FUNCTION).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PNS0PL
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR

    PNS0PL=-2.D0*CF/(1.D0-Z)

    RETURN
  END FUNCTION PNS0PL

  FUNCTION PNS0DL(X)
    ! ---------------------------------------------------------------------
    ! CALCULATE LO PQQ SPLITTING FUNCTION (PLUS AND DELTA FUNCTIONS).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PNS0DL
    REAL(KIND(1d0)),INTENT(IN)::X
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR

    ! the plus function means
    ! int(f[x]/(1-x)+,{x,x0,1})=int(f[x]/(1-x),{x,x0,1})-int(f[1]/(1-x),{x,0,1})
    ! =int((f[x]-f[1])/(1-x),{x,x0,1})-int(f[1]/(1-x),{x,0,x0})
    ! where the first term we call it to be int((f[x]-f[1])/(1-x)x0+,{x,x0,1})
    ! and the second term result in f[1]*log(1-x0)
    ! which is the first piece of the following one
    ! see page.10 of http://www.hep.phy.cam.ac.uk/theory/webber/QCDlect3.pdf
    PNS0DL=CF*(2.D0*DLOG(1.D0-X)+3.D0/2.D0)

    RETURN
  END FUNCTION PNS0DL

  FUNCTION PNSNLO(Z,SIGN)
    ! ---------------------------------------------------------------------
    ! CALCULATE NLO NONSINGLET SPLITTING FUNCTION.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PNSNLO
    REAL(KIND(1d0)),INTENT(IN)::Z,SIGN
    !REAL(KIND(1d0)),EXTERNAL::SS2
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::OPZ,OMZ,OPZ2,DLNZ,DLNO,PQQ
    REAL(KIND(1d0))::PF2,Z1,Z2,S2,PFG,PFT,PQQV,PFFG
    REAL(KIND(1d0))::PQQBV

    OPZ=1.D0+Z
    OMZ=1.D0-Z
    OPZ2=1.D0+Z*Z
    DLNZ=DLOG(Z)
    DLNO=DLOG(OMZ)
    PQQ=OPZ2/OMZ

    ! S2 FUNCTION
    Z1=1.D0/(OPZ)
    Z2=Z/(OPZ)
    S2=GAUSSI(Z2,Z1,SS2)
    
    PF2=(2.D0*DLNZ*DLNO+3.D0/2.D0*DLNZ-2.D0*DLNZ*DLNZ)*PQQ &
         -(7.D0+3.D0*Z)*0.5D0*DLNZ+0.5D0*OPZ*DLNZ*DLNZ &
         -5.D0*OMZ
    PFG=(0.5D0*DLNZ*DLNZ+11.D0/6.D0*DLNZ+67.D0/18.D0-PI*PI/6.D0)*PQQ &
         +(1.D0+Z)*DLNZ+20.D0/3.D0*OMZ
    PFT=-(2.D0/3.D0*DLNZ+10.D0/9.D0)*PQQ-4.D0/3.D0*OMZ

    PQQV=CF*CF*PF2+CF*CG*PFG+CF*TR*F*PFT
    
    PFFG=2.D0*OPZ2/OPZ*S2+2.D0*OPZ*DLNZ+4.D0*OMZ
    
    PQQBV=CF*(CF-CG*0.5D0)*PFFG
    
    PNSNLO=PQQV+SIGN*PQQBV
    
    RETURN
  END FUNCTION PNSNLO

  FUNCTION PNS1PL(Z)
    ! ---------------------------------------------------------------------
    ! CALCULATE NLO NONSINGLET SPLITTING FUNCTION (PLUS FUNCTION).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PNS1PL
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::OMZ,PFGPLUS,PFTPLUS

    OMZ=1.D0-Z

    PFGPLUS=2.D0*(67.D0/18.D0-PI*PI/6.D0)
    PFTPLUS=-2.D0*10.D0/9.D0

    PNS1PL=-(CF*CG*PFGPLUS+CF*TR*F*PFTPLUS)/OMZ

    RETURN
  END FUNCTION PNS1PL

  FUNCTION PNS1DL(X)
    ! ---------------------------------------------------------------------
    ! CALCULATE NLO NONSINGLET SPLITTING FUNCTION 
    ! (PLUS AND DELTA FUNCTIONS).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PNS1DL
    REAL(KIND(1d0)),INTENT(IN)::X
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::ZTA,DELF,PGF1,PNFF1,PLUS

    ZTA=1.2020569031595943D0

    DELF=CF*CF*(3.D0/8.D0-PI*PI/2.D0+6.D0*ZTA) &
         +CF*CG*0.5D0*(17.D0/12.D0+11.D0*PI*PI/9.D0-6.D0*ZTA) &
         -CF*TR*F*(1.D0/6.D0+2.D0*PI*PI/9.D0)
    PGF1=2.D0*(67.D0/9.D0-PI*PI/3.D0)
    PNFF1=-2.D0*10.D0/9.D0

    PLUS=(CF*CG/2.D0*PGF1+CF*TR*F*PNFF1)*DLOG(1.D0-X)

    PNS1DL=DELF+PLUS
    
    RETURN
  END FUNCTION PNS1DL

  SUBROUTINE PLO(Z,PQQ0,PQG0,PGQ0,PGG0)
    ! ---------------------------------------------------------------------
    ! CALCULATE LO (NONREGULARIZED) SPLITTING FUNCTIONS.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0)),INTENT(OUT)::PQQ0,PQG0,PGQ0,PGG0
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::OMZ,TEMP

    OMZ=1.D0-Z
    
    PQQ0=PNSLO(Z) ! Eq.(21) (w/o delta and plus) in arXiv:1106.1553
    PQG0=TR*(Z*Z+OMZ*OMZ) ! Eq.(22) in arXiv:1106.1553
    PGQ0=CF*(1.D0+OMZ*OMZ)/Z ! Eq.(23) in arXiv:1106.1553
    PGG0=2.D0*CG*(Z/OMZ+OMZ/Z+Z*OMZ) ! Eq.(24) (w/o delta and plus) in arXiv:1106.1553
    
    ! Matrix is transposed in DGLAP eq for FF: PQG -> PGQ, PGQ -> PQG
    TEMP=PQG0
    PQG0=PGQ0*2.D0
    PGQ0=TEMP
    
    RETURN
  END SUBROUTINE PLO
  
  FUNCTION PGG0PL(Z)
    ! ---------------------------------------------------------------------
    ! CALCULATE LO PGG SPLITTING FUNCTION (PLUS FUNCTION).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PGG0PL
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR

    PGG0PL=-2.D0*CG/(1.D0-Z)
    
    RETURN
  END FUNCTION PGG0PL

  FUNCTION PGG0DL(X)
    ! ---------------------------------------------------------------------
    ! CALCULATE LO PGG SPLITTING FUNCTION (PLUS AND DELTA FUNCTIONS).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PGG0DL
    REAL(KIND(1d0)),INTENT(IN)::X
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    
    ! the plus function means
    ! int(f[x]/(1-x)+,{x,x0,1})=int(f[x]/(1-x),{x,x0,1})-int(f[1]/(1-x),{x,0,1})
    ! =int((f[x]-f[1])/(1-x),{x,x0,1})-int(f[1]/(1-x),{x,0,x0})
    ! where the first term we call it to be int((f[x]-f[1])/(1-x)x0+,{x,x0,1})
    ! and the second term result in f[1]*log(1-x0)
    ! which is the first piece of the following one
    ! see page.10 of http://www.hep.phy.cam.ac.uk/theory/webber/QCDlect3.pdf
    PGG0DL=2.D0*CG*(DLOG(1.D0-X)+(11.D0/12.D0-F*TR/CG/3.D0))
    
    RETURN
  END FUNCTION PGG0DL

  SUBROUTINE PNLO(Z,PQQ1,PQG1,PGQ1,PGG1)
    ! ---------------------------------------------------------------------
    ! CALCULATE NLO (NONREGULARIZED) SPLITTING FUNCTIONS.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(OUT)::PQQ1,PQG1,PGQ1,PGG1
    REAL(KIND(1d0)),INTENT(IN)::Z
    !REAL(KIND(1d0)),EXTERNAL::SS1,SS2
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::OMZ,OPZ,DLNZ,DLNO
    REAL(KIND(1d0))::Z1,Z2,S1,S2,PQGP,PQGM
    REAL(KIND(1d0))::FQG1,FQG2,FQG3
    REAL(KIND(1d0))::PGQP,PGQM
    REAL(KIND(1d0))::FGQ1,FGQ2,FGQ3
    REAL(KIND(1d0))::PGGP,PGGM
    REAL(KIND(1d0))::FGG1,FGG2,FGG3
    REAL(KIND(1d0))::TEMP
    
    OMZ=1.D0-Z
    OPZ=1.D0+Z
    DLNZ=DLOG(Z)
    DLNO=DLOG(OMZ)

    ! S1 FUNCTION
    Z1=1.D0-Z
    S1=GAUSSI(0.D0,Z1,SS1)
    
    ! S2 FUNCTION
    Z1=1.D0/(OPZ)
    Z2=Z/(OPZ)
    S2=GAUSSI(Z2,Z1,SS2)
    
    ! NLO Pqg
    PQGP=Z*Z+OMZ*OMZ
    PQGM=Z*Z+OPZ*OPZ
    
    FQG1=-8.D0/3.D0-(16.D0/3.D0+8.D0*DLNZ+8.D0*DLNO)*PQGP/3.D0
    FQG2=-2.D0+3.D0*Z+(-7.D0+8.D0*Z)*DLNZ-4.D0*DLNO &
         +(1.D0-2.D0*Z)*DLNZ*DLNZ &
         +(-2.D0*DLOG(Z*OMZ)*DLOG(Z*OMZ)-2.D0*DLOG(OMZ/Z) &
         +16.D0*S1+2.D0*PI*PI-10.D0)*PQGP
    FQG3=(-152.D0+166.D0*Z-40.D0/Z)/9.D0 &
         +(-4.D0-76.D0*Z)*DLNZ/3.D0+4.D0*DLNO &
         +(2.D0+8.D0*Z)*DLNZ*DLNZ &
         +(8.D0*DLNZ*DLNO-DLNZ*DLNZ-4.D0/3.D0*DLNZ+10.D0/3.D0*DLNO &
         +2.D0*DLNO*DLNO-16.D0*S1-7.D0*PI*PI/3.D0+178.D0/9.D0)*PQGP &
         +2.D0*PQGM*S2
    
    ! NLO Pgq
    PGQP=(1.D0+OMZ*OMZ)/Z
    PGQM=-(1.D0+OPZ*OPZ)/Z
    
    FGQ1=-0.5D0+9.D0*0.5D0*Z+(-8.D0+0.5D0*Z)*DLNZ+2.D0*Z*DLNO &
         +(1.D0-0.5D0*Z)*DLNZ*DLNZ &
         +(DLNO*DLNO+4.D0*DLNZ*DLNO-8.D0*S1-4.D0*PI*PI/3.D0)*PGQP
    FGQ2=(62.D0-35.D0*0.5D0*Z-44.D0*Z*Z)/9.D0 &
         +(2.D0+12.D0*Z+8.D0/3.D0*Z*Z)*DLNZ-2.D0*Z*DLNO &
         -(4.D0+Z)*DLNZ*DLNZ+PGQM*S2 &
         +(-2.D0*DLNZ*DLNO-3.D0*DLNZ-3.D0*0.5D0*DLNZ*DLNZ-DLNO*DLNO &
         +8.D0*S1+7.D0*PI*PI/6.D0+17.D0/18.D0)*PGQP

    ! NLO Pgg
    PGGP=1.D0/OMZ+1.D0/Z-2.D0+Z*OMZ
    PGGM=1.D0/OPZ-1.D0/Z-2.D0-Z*OPZ
    
    FGG1=-4.D0+12.D0*Z-164.D0/9.D0*Z*Z &
         +(10.D0+14.D0*Z+16.D0/3.D0*Z*Z+16.D0/3.D0/Z)*DLNZ &
         +92.D0/9.D0/Z+2.D0*OPZ*DLNZ*DLNZ
    FGG2=2.D0-2.D0*Z+26.D0/9.D0*(Z*Z-1.D0/Z)-4.D0/3.D0*OPZ*DLNZ &
         -(20.D0/9.D0+8.D0/3.D0*DLNZ)*PGGP
    FGG3=27.D0*0.5D0*OMZ+67.D0/9.D0*(Z*Z-1.D0/Z) &
         +(11.D0-25.D0*Z-44.D0/Z)/3.D0*DLNZ &
         -4.D0*OPZ*DLNZ*DLNZ+2.D0*PGGM*S2 &
         +(4.D0*DLNZ*DLNO-3.D0*DLNZ*DLNZ+22.D0/3.D0*DLNZ &
         -PI*PI/3.D0+67.D0/9.D0)*PGGP
    
    PQQ1=PNSNLO(Z,1.D0)
    PQG1=TR*TR*F*FQG1+CF*TR*FQG2+CG*TR*FQG3
    PGQ1=CF*CF*FGQ1+CF*CG*FGQ2
    PGG1=CF*TR*F*FGG1+CG*TR*F*FGG2+CG*CG*FGG3
    
    ! Matrix is transposed in DGLAP eq for FF: PQG -> PGQ, PGQ -> PQG
    TEMP=PQG1/2.D0
    PQG1=PGQ1*2.D0
    PGQ1=TEMP
    
    RETURN
  END SUBROUTINE PNLO
  
  FUNCTION PPSNLO(Z)
    !---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PPSNLO
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::DLNZ
    REAL(KIND(1d0))::FQQ
    
    DLNZ=DLOG(Z)
    
    FQQ=-20.D0/9.D0/Z-8.D0+4.D0*Z+56.D0/9.D0*Z*Z &
         -(5.D0+9.D0*Z+8.D0/3.D0*Z*Z)*DLNZ &
         +(1.D0+Z)*DLNZ*DLNZ
    PPSNLO=2.D0*CF*TR*FQQ
    
    RETURN
  END FUNCTION PPSNLO
  
  FUNCTION PGG1PL(Z)
    ! ---------------------------------------------------------------------
    ! CALCULATE NLO PGG SPLITTING FUNCTIONS (PLUS FUNCTION).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PGG1PL
    REAL(KIND(1d0)),INTENT(IN)::Z
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::OMZ
    REAL(KIND(1d0))::FGG2PL
    REAL(KIND(1d0))::FGG3PL
    REAL(KIND(1d0))::FGG1PL
    
    OMZ=1.D0-Z
    
    FGG2PL=-20.D0/9.D0
    FGG3PL=-PI*PI/3.D0+67.D0/9.D0
    
    PGG1PL=-(CG*TR*F*FGG2PL+CG*CG*FGG3PL)/OMZ
    
    RETURN
  END FUNCTION PGG1PL
  
  FUNCTION PGG1DL(X)
    ! ---------------------------------------------------------------------
    ! CALCULATE NLO PGG SPLITTING FUNCTIONS (PLUS AND DELTA FUNCTIONS).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::PGG1DL
    REAL(KIND(1d0)),INTENT(IN)::X
    REAL(KIND(1d0))::F
    COMMON /FLA/F
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::ZTA
    REAL(KIND(1d0))::FGG1DL,FGG2DL,FGG3DL
    REAL(KIND(1d0))::DELF
    REAL(KIND(1d0))::FGG2PL,FGG3PL,PLUS
    
    ZTA=1.2020569031595943D0
    
    FGG1DL=-4.D0/3.D0
    FGG2DL=-1.D0
    FGG3DL=3.D0*ZTA+8.D0/3.D0
    
    DELF=CG*TR*F*FGG1DL+CF*TR*F*FGG2DL+CG*CG*FGG3DL
    
    FGG2PL=-20.D0/9.D0
    FGG3PL=-PI*PI/3.D0+67.D0/9.D0
    
    PLUS=(CG*TR*F*FGG2PL+CG*CG*FGG3PL)*DLOG(1.D0-X)
    
    PGG1DL=DELF+PLUS
        
    RETURN
  END FUNCTION PGG1DL

  FUNCTION SS1(Z)
    !---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::SS1
    REAL(KIND(1d0)),INTENT(IN)::Z
    
    
    SS1=DLOG(1.D0-Z)/Z
    
    RETURN
  END FUNCTION SS1
  
  FUNCTION SS2(Z)
    !---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0))::SS2
    REAL(KIND(1d0)),INTENT(IN)::Z
    
    
    SS2=DLOG((1.D0-Z)/Z)/Z
    
    RETURN
  END FUNCTION SS2
  
  SUBROUTINE SPLINE(N,X,Y,B,C,D,IN)
    ! ----
    ! CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
    ! INTERPOLATION SUBROUTINES ARE TAKEN FROM 
    ! G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
    ! COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N
    REAL(KIND(1d0)),DIMENSION(600),INTENT(IN)::X
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(IN)::Y
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(OUT)::B,C,D
    INTEGER,INTENT(IN)::IN
    INTEGER::NM1,I,IB
    REAL(KIND(1d0))::T
    
    NM1=N-1
    IF(N.LT.2) RETURN
    IF(N.LT.3) GO TO 250
    
    D(1,IN)=X(2)-X(1)
    C(2,IN)=(Y(2,IN)-Y(1,IN))/D(1,IN)
    DO I=2,NM1
       D(I,IN)=X(I+1)-X(I)
       B(I,IN)=2.0D0*(D(I-1,IN)+D(I,IN))
       C(I+1,IN)=(Y(I+1,IN)-Y(I,IN))/D(I,IN)
       C(I,IN)=C(I+1,IN)-C(I,IN)
    END DO
    
    B(1,IN)=-D(1,IN)
    B(N,IN)=-D(N-1,IN)
    C(1,IN)=0.0D0
    C(N,IN)=0.0D0
    
    IF(N.EQ.3) GO TO 215
    C(1,IN)=C(3,IN)/(X(4)-X(2))-C(2,IN)/(X(3)-X(1))
    C(1,IN)=C(1,IN)*D(1,IN)**2.0D0/(X(4)-X(1))
    C(N,IN)=C(N-1,IN)/(X(N)-X(N-2))-C(N-2,IN)/(X(N-1)-X(N-3))
    C(N,IN)=-C(N,IN)*D(N-1,IN)**2.0D0/(X(N)-X(N-3))
215 CONTINUE
    
    DO I=2,N
       T=D(I-1,IN)/B(I-1,IN)
       B(I,IN)=B(I,IN)-T*D(I-1,IN)
       C(I,IN)=C(I,IN)-T*C(I-1,IN)
    END DO
    C(N,IN)=C(N,IN)/B(N,IN)
    DO IB=1,NM1
       I=N-IB
       C(I,IN)=(C(I,IN)-D(I,IN)*C(I+1,IN))/B(I,IN)
    END DO
    
    B(N,IN)=(Y(N,IN)-Y(NM1,IN))/D(NM1,IN) &
         +D(NM1,IN)*(C(NM1,IN)+2.0D0*C(N,IN))
    DO I=1,NM1
       B(I,IN)=(Y(I+1,IN)-Y(I,IN))/D(I,IN) &
            -D(I,IN)*(C(I+1,IN)+2.0D0*C(I,IN))
       D(I,IN)=(C(I+1,IN)-C(I,IN))/D(I,IN)
       C(I,IN)=3.0D0*C(I,IN)
    END DO

    C(N,IN)=3.0D0*C(N,IN)
    D(N,IN)=D(N-1,IN)
    RETURN
    
250 CONTINUE
    B(1,IN)=(Y(2,IN)-Y(1,IN))/(X(2)-X(1))
    C(1,IN)=0.0D0
    D(1,IN)=0.0D0
    B(2,IN)=B(1,IN)
    C(2,IN)=0.0D0
    D(2,IN)=0.0D0
    
    RETURN
  END SUBROUTINE SPLINE
  
  FUNCTION SEVAL(N,XX,X,Y,B,C,D,IN)
    ! ---------------------------------------------------------------------
    ! CALCULATE THE DISTRIBUTION AT XX BY CUBIC SPLINE INTERPOLATION.
    ! ---------------------------------------------------------------------
    ! REFER TO EQ.(13) in arXiv:1106.1553
    IMPLICIT NONE
    REAL(KIND(1d0))::SEVAL
    INTEGER,INTENT(IN)::N ! number of xm
    REAL(KIND(1d0)),DIMENSION(600),INTENT(IN)::X ! xm,i.e. x grid
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(IN)::Y ! WQQ, x*FF at tl
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(IN)::B,C,D ! the coefficients from SPLINE
    INTEGER::I=1
    SAVE I
    INTEGER,INTENT(IN)::IN ! which FF, quark or gluon or else
    REAL(KIND(1d0)),INTENT(IN)::XX ! xm/xk
    INTEGER::J,K
    REAL(KIND(1d0))::DX

    IF(I.GE.N) I=1
    IF(XX.LT.X(I)) THEN 
       GO TO 310
    ELSE IF(XX.LE.X(I+1)) THEN
       GO TO 330
    END IF
    J=N+1
    GOTO 320
    
310 CONTINUE
    J=I
    I=1
    
320 CONTINUE
    K=(I+J)/2
    IF(XX.LT.X(K)) J=K
    IF(XX.GE.X(K)) I=K
    IF(J.GT.I+1) GO TO 320
    
330 CONTINUE
    DX=XX-X(I)
    SEVAL=Y(I,IN)+DX*(B(I,IN)+DX*(C(I,IN)+DX*D(I,IN)))
    
    RETURN
  END FUNCTION SEVAL

  SUBROUTINE SPLINE2D(N,X,Y,B,C,D,I,J)
    ! ---------------------------------------------------------------------
    ! CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
    ! INTERPOLATION SUBROUTINES ARE TAKEN FROM
    ! G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
    ! COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
    ! ----------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER,INTENT(IN)::N,I,J
    INTEGER,PARAMETER::NQ=600,NX=600,ND=10
    REAL(KIND(1d0)),DIMENSION(NX,NQ,ND),INTENT(IN)::Y
    REAL(KIND(1d0)),DIMENSION(NX,NQ,ND),INTENT(OUT)::B,C,D
    REAL(KIND(1d0)),DIMENSION(NX),INTENT(IN)::X
    INTEGER::NM1,K,IB
    REAL(KIND(1d0))::T

    NM1=N-1
    IF(N.LT.2) RETURN
    IF(N.LT.3) GO TO 250
    D(1,J,I)=X(2)-X(1)
    C(2,J,I)=(Y(2,J,I)-Y(1,J,I))/D(1,J,I)
    DO K=2,NM1
       D(K,J,I)=X(K+1)-X(K)
       B(K,J,I)=2.0D0*(D(K-1,J,I)+D(K,J,I))
       C(K+1,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
       C(K,J,I)=C(K+1,J,I)-C(K,J,I)
    ENDDO
    B(1,J,I)=-D(1,J,I)
    B(N,J,I)=-D(N-1,J,I)
    C(1,J,I)=0.0D0
    C(N,J,I)=0.0D0
    IF(N.EQ.3) GO TO 215
    C(1,J,I)=C(3,J,I)/(X(4)-X(2))-C(2,J,I)/(X(3)-X(1))
    C(N,J,I)=C(N-1,J,I)/(X(N)-X(N-2))-C(N-2,J,I)/(X(N-1)-X(N-3))
    C(1,J,I)=C(1,J,I)*D(1,J,I)**2.0D0/(X(4)-X(1))
    C(N,J,I)=-C(N,J,I)*D(N-1,J,I)**2.0D0/(X(N)-X(N-3))
215 CONTINUE
    DO K=2,N
       T=D(K-1,J,I)/B(K-1,J,I)
       B(K,J,I)=B(K,J,I)-T*D(K-1,J,I)
       C(K,J,I)=C(K,J,I)-T*C(K-1,J,I)
    ENDDO
    C(N,J,I)=C(N,J,I)/B(N,J,I)
    DO IB=1,NM1
       K=N-IB
       C(K,J,I)=(C(K,J,I)-D(K,J,I)*C(K+1,J,I))/B(K,J,I)
    ENDDO
    B(N,J,I)=(Y(N,J,I)-Y(NM1,J,I))/D(NM1,J,I) &
         +D(NM1,J,I)*(C(NM1,J,I)+2.0D0*C(N,J,I))
    DO K=1,NM1
       B(K,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I) &
            -D(K,J,I)*(C(K+1,J,I)+2.0D0*C(K,J,I))
       D(K,J,I)=(C(K+1,J,I)-C(K,J,I))/D(K,J,I)
       C(K,J,I)=3.0D0*C(K,J,I)
    ENDDO
    C(N,J,I)=3.0D0*C(N,J,I)
    D(N,J,I)=D(N-1,J,I)
    RETURN
250     CONTINUE
    B(1,J,I)=(Y(2,J,I)-Y(1,J,I))/(X(2)-X(1))
    C(1,J,I)=0.0D0
    D(1,J,I)=0.0D0
    B(2,J,I)=B(1,J,I)
    C(2,J,I)=0.0D0
    D(2,J,I)=0.0D0
    RETURN
  END SUBROUTINE SPLINE2D
  
  SUBROUTINE PUTFFG(J,QQ)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(IN)::QQ
    INTEGER,INTENT(IN)::J
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON/XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600)::Q2T ! the grid of Q2 (i.e. the exponential of T)
    INTEGER::NT
    COMMON /QQT/Q2T,NT
    REAL(KIND(1d0)),DIMENSION(600,600,10)::FFG
    COMMON /FFG2D/FFG
    REAL(KIND(1d0)),DIMENSION(600,600,10)::BXG,CXG,DXG
    COMMON /SPL2D/BXG, CXG, DXG
    INTEGER::I,K,M,N,L
    
    IF(J.EQ.1) THEN
       DO K=1,NX+1
          DO M=1,NT+1
             DO N=1,10
                FFG(K,M,N)=0.D0
             ENDDO
          ENDDO
       ENDDO
    ENDIF
    
    DO K=1,NX
       FFG(K,J,1)=QQ(K,3)                    ! gluon
       FFG(K,J,2)=(QQ(K,1)+QQ(K,5))*0.5D0    ! u 
       FFG(K,J,3)=(QQ(K,2)+QQ(K,6))*0.5D0    ! d
       FFG(K,J,4)=QQ(K,7)*0.5D0              ! s
       FFG(K,J,5)=(QQ(K,5)-QQ(K,1))*0.5D0    ! ub
       FFG(K,J,6)=(QQ(K,6)-QQ(K,2))*0.5D0    ! db
       FFG(K,J,7)=QQ(K,8)*0.5D0              ! c
       FFG(K,J,8)=QQ(K,9)*0.5D0              ! b
    ENDDO
    
    IF(J.EQ.NT+1) THEN
       DO I=1,8
          DO L=1,NT+1
             FFG(NX+1,L,I)=0.D0 ! at z=1, FF(z)=0.D0
             CALL SPLINE2D(NX+1,X,FFG,BXG,CXG,DXG,I,L)
          ENDDO
       ENDDO
    ENDIF

    RETURN
  END SUBROUTINE PUTFFG
  
  SUBROUTINE GETFF(Q2,XX,FF)
    ! ---------------------------------------------------------------------
    ! return FF=D at scale Sqrt(Q2) and X=XX
    IMPLICIT NONE
    INTEGER,PARAMETER::ND=10
    REAL(KIND(1d0)),DIMENSION(ND)::PDFJ1,PDFJ2,XFF
    REAL(KIND(1d0)),DIMENSION(-5:5),INTENT(OUT)::FF
    REAL(KIND(1d0)),INTENT(IN)::Q2,XX
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON/XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600)::Q2T
    INTEGER::NT
    COMMON /QQT/Q2T,NT
    REAL(KIND(1d0)),DIMENSION(600,600,10)::FFG
    COMMON /FFG2D/FFG
    REAL(KIND(1d0)),DIMENSION(600,600,10)::BXG,CXG,DXG
    COMMON /SPL2D/BXG, CXG, DXG
    INTEGER::I,J,K
    REAL(KIND(1d0))::T,DX

    DO I=-5,5
       FF(I)=0.D0  ! Initializing arrary
    END DO
    IF(XX.GT.1.D0) RETURN
    
    ! INTERPOLATION.
    ! X: CUBIC SPLINE INTERPOLATION, LOG(Q2): LINEAR INTERPOLATION.
    J=ISERCH(NT+1,Q2T,Q2)
    IF(J.EQ.NT+1) J=NT
    
    K=ISERCH(NX+1,X,XX)
    DX=XX-X(K)
    DO I=1,8
       PDFJ1(I)=FFG(K,J,I) &
            +DX*(BXG(K,J,I)+DX*(CXG(K,J,I)+DX*DXG(K,J,I)))
       PDFJ2(I)=FFG(K,J+1,I) &
                +DX*(BXG(K,J+1,I)+DX*(CXG(K,J+1,I)+DX*DXG(K,J+1,I)))
    ENDDO
    
    T=(DLOG(Q2)-DLOG(Q2T(J)))/(DLOG(Q2T(J+1))-DLOG(Q2T(J)))
    ! I=1:gluon, 2:u, 3:d, 4:s, 5:ub, 6:db, 7:c, 8:b
    DO I=1,8
       XFF(I)=(1.D0-T)*PDFJ1(I)+T*PDFJ2(I)
    ENDDO
    
    ! Changing label of FF array for being consistent with the LHAPDF
    FF(0)=XFF(1)/XX  ! gluon 
    FF(1)=XFF(3)/XX  ! d
    FF(2)=XFF(2)/XX  ! u
    FF(3)=XFF(4)/XX  ! s
    FF(4)=XFF(7)/XX  ! c
    FF(5)=XFF(8)/XX  ! b
    
    FF(-1)=XFF(6)/XX ! db 
    FF(-2)=XFF(5)/XX ! ub
    FF(-3)=FF(3)     ! sb
    FF(-4)=FF(4)     ! cb
    FF(-5)=FF(5)     ! bb
        
    RETURN
  END SUBROUTINE GETFF
  
  SUBROUTINE CHANGENF(F,INF)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER::NGLI
    COMMON /NISTEP/NGLI
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON/XXX/X,NX
    REAL(KIND(1d0)),DIMENSION(600,10)::QQ
    COMMON /QQQ/QQ
    REAL(KIND(1d0)),DIMENSION(600,10)::WQQ
    COMMON /WWW/WQQ
    REAL(KIND(1d0)),DIMENSION(600,600)::Y,Z,W
    COMMON/YZW/Y,Z,W
    REAL(KIND(1d0)),DIMENSION(600,600,9)::WFIY
    COMMON /WQSG/WFIY
    INTEGER::I,K
    REAL(KIND(1d0))::KK
    INTEGER,INTENT(IN)::INF
    REAL(KIND(1d0))::ZDHY,ZDHX
    REAL(KIND(1d0)),INTENT(IN)::F
    REAL(KIND(1d0))::YY
    
    CALL CHNGP(F)
    
    DO I=1,NX ! NX+1 -> NX
       DO K=NGLI,1,-1
          YY=Y(K,I)  ! xm/xk       
          ZDHY=HQFF(YY,INF)*2.D0         ! 2*z*D_c,b
          WFIY(K,I,4)=WFIY(K,I,4)+ZDHY   ! siglet+2*zD_c,b
          WFIY(K,I,7+INF)=ZDHY           ! 2*zD_c,b
       ENDDO
       ZDHX=2.D0*HQFF(X(I),INF)         ! INF=1:charm, 2:bottom
       QQ(I,4)=QQ(I,4)+ZDHX
       QQ(I,7+INF)=ZDHX                 ! 7+INF=8:charm, 9:bottom
       WQQ(I,4)=QQ(I,4)
       WQQ(I,7+INF)=QQ(I,7+INF)
    ENDDO
    
    RETURN
  END SUBROUTINE CHANGENF
  
  SUBROUTINE GETINI(NFI,QQ)
    ! ---------------------------------------------------------------------
    ! get the initial FF values for q,g at the initial scale
    ! they are stored in QQ, z*D(z)
    IMPLICIT NONE
    INTEGER,INTENT(IN)::NFI
    REAL(KIND(1d0)),DIMENSION(600,10),INTENT(OUT)::QQ
    INTEGER::IN,I
    REAL(KIND(1d0)),DIMENSION(600)::X
    INTEGER::NX
    COMMON /XXX/X,NX
    
    DO IN=1,4+NFI
       DO I=1,NX
          QQ(I,IN)=QI0(X(I),IN)
       ENDDO
       QQ(NX+1,IN)=0.D0
    END DO
    
    RETURN
  END SUBROUTINE GETINI
  
  FUNCTION QI0(X,IN)
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::X
    INTEGER,INTENT(IN)::IN
    REAL(KIND(1d0))::QI0
    
    IF (IN.EQ.1) THEN          ! uv=u^-
       QI0=FF_INI(X,1)-FF_INI(X,-1)
       
    ELSE IF (IN.EQ.2) THEN    ! dv=d^-
       QI0=FF_INI(X,2)-FF_INI(X,-2)
       
    ELSE IF (IN.EQ.3) THEN    ! gluon
       QI0=FF_INI(X,0)
           
    ELSE IF (IN.EQ.4) THEN    ! Singlet=Sum(q+qbar)
       QI0=FF_INI(X,1)+FF_INI(X,-1)+FF_INI(X,2)+FF_INI(X,-2) &
            +2.D0*FF_INI(X,3)
       
    ELSE IF (IN.EQ.5) THEN    ! u^+=uv+2*ub=u+ubar 
       QI0=FF_INI(X,1)+FF_INI(X,-1)
       
    ELSE IF (IN.EQ.6) THEN    ! d^+=dv+2*db=d+dbar
       QI0=FF_INI(X,2)+FF_INI(X,-2)
       
    ELSE IF (IN.EQ.7) THEN    ! s^+=2*sb=s+sbar, sbar=s
       QI0=FF_INI(X,3)*2.D0
    ELSE 
       QI0=0.D0
    END IF
        
    RETURN
  END FUNCTION QI0

  !FUNCTION FF_INI(X,Ifl)
  !  ! ---------------------------------------------------------------------
  !  ! x*F at the initial scale
  !  IMPLICIT NONE
  !  REAL(KIND(1d0))::FF_INI
  !  REAL(KIND(1d0)),INTENT(IN)::X
  !  INTEGER,INTENT(IN)::Ifl
  !  

  !  ! HKNS07 initial functions
  !  !    M. Hirai, S. Kumano, T.-H. Nagai, and K. Sudoh, 
  !  !    Phys. Rev. D 75 (2007) 094009.    
  !  !      FF_INI=X*FF(Ifl) ! z*D(x)
  !  ! pi^+
  !  
  !  IF(Ifl.EQ.0) THEN ! Gluon
  !     FF_INI=4.387047D+02*X**( 1.942747D+00)*(1.D0-X)**8.000000D+00
  !     
  !  ELSE IF(Ifl.EQ.1) THEN  ! Up
  !     FF_INI=1.008181D+00*X**(-9.634418D-01)*(1.D0-X)**1.369666D+00
  !     
  !  ELSE IF(Ifl.EQ.2) THEN  ! Down
  !     FF_INI=1.754279D+01*X**( 7.181598D-01)*(1.D0-X)**6.265853D+00
  !     
  !  ELSE IF(Ifl.EQ.3) THEN  ! Strange
  !     FF_INI=1.754279D+01*X**( 7.181598D-01)*(1.D0-X)**6.265853D+00
  !     
  !  ELSE IF(Ifl.EQ.-1) THEN ! ubar
  !     FF_INI=1.754279D+01*X**( 7.181598D-01)*(1.D0-X)**6.265853D+00
  !     
  !  ELSE IF(Ifl.EQ.-2) THEN ! dbar
  !     FF_INI=1.008181D+00*X**(-9.634418D-01)*(1.D0-X)**1.369666D+00
  !     
  !  ELSE IF(Ifl.EQ.-3) THEN ! sbar
  !     FF_INI=1.754279D+01*X**( 7.181598D-01)*(1.D0-X)**6.265853D+00
  !     
  !  ELSE
  !     FF_INI=0.D0      
  !  ENDIF
  !  
  !  FF_INI=X*FF_INI ! Z*D(Z)
  !  RETURN
  !END FUNCTION FF_INI
  
  
  !FUNCTION HQFF(X,IN)
  !  ! ---------------------------------------------------------------------
  !  ! x*F at initial scale for heavy quarks
  !  ! it is relevant above the threshold
  !  IMPLICIT NONE
  !  REAL(KIND(1d0))::HQFF
  !  REAL(KIND(1d0)),INTENT(IN)::X
  !  INTEGER,INTENT(IN)::IN
  !      
  !  IF(IN.EQ.1) THEN ! FF_Charm at initial Q^2
  !     HQFF=X*1.208375D+00*X**(-8.453058D-01)*(1.D0-X)**3.868264D+00
  !     ! pi^+_c of the HKNS NLO
  !     
  !  ELSE              ! FF_Bottom at initial Q^2
  !     HQFF=X*8.655283D-01*X**(-1.219219D+00)*(1.D0-X)**5.667591D+00
  !     ! pi^+_b of the HKNS NLO
  !     
  !  ENDIF
  !  
  !  RETURN
  !END FUNCTION HQFF
  
END MODULE FF_DGLAP
