MODULE FF_INIT
  IMPLICIT NONE
  ! FF_SET: 1, pion in HKNS07
  !         2, ccbar(3S1[1]) from gluon and charm quark
  !         3, ccbar(1S0[1]) from gluon and charm quark
  !         4, ccbar(1S0[8]) from gluon and charm quark
  !         5, ccbar(3PJ[8]) from gluon and charm quark
  !         6, ccbar(3S1[8]) from gluon and charm quark
  INTEGER::FF_SET=1
CONTAINS
  FUNCTION FF_INI(X,Ifl)
    ! Ifl: 0, gluon
    ! Ifl: 1, u quark
    ! Ifl: 2, d quark
    ! Ifl: 3, s quark
    ! Ifl: -1, ubar quark
    ! Ifl: -2, dbar quark
    ! Ifl: -3, sbar quark
    ! others zero
    USE fragfun_HKNS07_ini
    USE fragfun_onia_ini
    ! ---------------------------------------------------------------------
    ! x*F at the initial scale
    IMPLICIT NONE
    REAL(KIND(1d0))::FF_INI
    REAL(KIND(1d0)),INTENT(IN)::X
    INTEGER,INTENT(IN)::Ifl
    IF(FF_SET.EQ.1)THEN
       IF(ABS(Ifl).GE.4)THEN
          FF_INI=0d0
       ELSE
          FF_INI=X*ff_pionp_HKNS07_ini(X,Ifl)
       ENDIF
    ELSEIF(FF_SET.EQ.2)THEN
       IF(Ifl.EQ.0)THEN
          FF_INI=ff_g_3S11gg_ini(X,3d0,3.1d0,0.81d0,2)
          FF_INI=FF_INI*X
       ELSE
          FF_INI=0d0
       ENDIF
    ELSEIF(FF_SET.EQ.3)THEN
       IF(Ifl.EQ.0)THEN
          FF_INI=ff_g_1S01g_ini(X,3d0,3d0,0.81d0)
          FF_INI=FF_INI*X
       ELSE
          FF_INI=0d0
       ENDIF
    ELSEIF(FF_SET.EQ.4)THEN
       IF(Ifl.EQ.0)THEN
          FF_INI=ff_g_1S08g_ini(X,3d0,3d0,0.0382d0)
          FF_INI=FF_INI*X
       ELSE
          FF_INI=0d0
       ENDIF
    ELSEIF(FF_SET.EQ.5)THEN
       IF(Ifl.EQ.0)THEN
          FF_INI=ff_g_3PJ8g_ini(X,3d0,3d0,0.0382d0)
          FF_INI=FF_INI*X
       ELSE
          FF_INI=0d0
       ENDIF
    ELSEIF(FF_SET.EQ.6)THEN
       IF(Ifl.EQ.0)THEN
          FF_INI=ff_g_3S18_ini(X,3d0,3d0,0.0382d0)
          FF_INI=FF_INI*X
       ELSE
          FF_INI=0d0
       ENDIF
    ELSE
       WRITE(*,*)"ERROR: Do not understand FF_SET=",FF_SET
       STOP
    ENDIF
  END FUNCTION FF_INI

  FUNCTION HQFF(X,IN)
    ! ---------------------------------------------------------------------
    ! x*F at initial scale for heavy quarks
    ! it is relevant above the threshold
    ! IN: 1, charm
    ! IN: 2, bottom
    USE fragfun_HKNS07_ini
    USE fragfun_onia_ini
    IMPLICIT NONE
    REAL(KIND(1d0))::HQFF
    REAL(KIND(1d0)),INTENT(IN)::X
    INTEGER,INTENT(IN)::IN
    IF(FF_SET.EQ.1)THEN
       IF(IN.EQ.1)THEN
          HQFF=X*ff_pionp_HKNS07_ini(X,4)
       ELSE
          HQFF=X*ff_pionp_HKNS07_ini(X,5)
       ENDIF
    ELSEIF(FF_SET.EQ.2)THEN
       IF(IN.EQ.1)THEN
          HQFF=ff_c_3S11c_ini(X,4.5d0,3.1d0,0.81d0)
       ELSE
          HQFF=0d0
       ENDIF
    ELSEIF(FF_SET.EQ.3)THEN
       IF(IN.EQ.1)THEN
          HQFF=ff_c_1S01c_ini(X,4.5d0,3d0,0.81d0)
       ELSE
          HQFF=0d0
       ENDIF
    ENDIF
  END FUNCTION HQFF

  FUNCTION FF_INI_Q02MIN(ISET)
    USE FF_DGLAP_GLOBAL
    ! Q02_MIN for the FF
    IMPLICIT NONE
    INTEGER,INTENT(IN)::ISET
    REAL(KIND(1d0))::FF_INI_Q02MIN
    IF(ISET.EQ.1)THEN
       FF_INI_Q02MIN=1D0
       FF_DGLAP_Q2min=FF_INI_Q02MIN
    ELSEIF(ISET.EQ.2)THEN
       FF_INI_Q02MIN=3D0**2
       FF_DGLAP_Q2min=FF_INI_Q02MIN
    ELSEIF(ISET.EQ.3)THEN
       FF_INI_Q02MIN=3D0**2
       FF_DGLAP_Q2min=FF_INI_Q02MIN
    ENDIF
    RETURN
  END FUNCTION FF_INI_Q02MIN

END MODULE FF_INIT
