MODULE ff_common
  IMPLICIT NONE
CONTAINS
  SUBROUTINE GETALP(T,IORDER,ALP2PI)
    ! ---------------------------------------------------------------------
    ! ---------------------------------------------------------------------
    ! RUNNING COUPLING CONSTANTS.
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::T ! T=LOG(Q^2)
    INTEGER,INTENT(IN)::IORDER ! 1: LO, 2: NLO
    REAL(KIND(1d0)),INTENT(OUT)::ALP2PI ! as/2/Pi
    REAL(KIND(1d0))::CF,CG,TR
    COMMON /COL/CF,CG,TR
    REAL(KIND(1d0))::DLAML,DLAMN
    COMMON /LAM/DLAML,DLAMN
    REAL(KIND(1d0))::PI
    COMMON /PI/PI
    REAL(KIND(1d0))::CTHRE,BTHRE
    COMMON /HQMTHRE/CTHRE,BTHRE
    REAL(KIND(1d0))::Q2thr,F,B0,B1
    REAL(KIND(1d0))::DLAMF,DLAMFN
    REAL(KIND(1d0))::DLNLAM,DLNQ2,ALPHA

    ! Changing the number of the quark flavor at heavy quark mass threshold
    Q2thr=DEXP(T)
    IF(Q2thr.LT.CTHRE) F=3.D0
    IF((Q2thr.GE.CTHRE).AND.(Q2thr.LT.BTHRE)) F=4.D0
    IF(Q2thr.GE.BTHRE) F=5.D0

    B0=11.D0/3.D0*CG-4.D0/3.D0*TR*F
    B1=34.D0/3.D0*CG*CG-10.D0/3.D0*CG*F-2.D0*CF*F

    ! Changing Lambda_QCD for connecting alpah_s at the threshold
    IF(Q2thr.LT.CTHRE) then      ! Lambda_QCD (4to3)
       DLAMF=DLAML*(DSQRT(CTHRE)/DLAML)**(2.D0/3.D0/B0)
       DLAMFN=DLAMN*(DSQRT(CTHRE)/DLAMN)**(2.D0/27.D0) &
            *DLOG(CTHRE/(DLAMN*DLAMN))**(107.D0/2025.D0)

    ELSE IF((Q2thr.GE.CTHRE).AND.(Q2thr.LT.BTHRE)) then
       DLAMF=DLAML
       DLAMFN=DLAMN

    ELSE IF(Q2thr.GE.BTHRE) Then ! Lambda_QCD (4to5)
       DLAMF=DLAML*(DSQRT(BTHRE)/DLAML)**(-2.D0/3.D0/B0)
       DLAMFN=DLAMN*(DLAMN/DSQRT(BTHRE))**(2.D0/23.D0) &
            *DLOG(BTHRE/(DLAMN*DLAMN))**(-963.D0/13225.D0)
    END IF
    
    ! Calculating alpha_s(Q^2)
    IF(IORDER.EQ.2) DLAMF=DLAMFN
    DLNLAM=DLOG(DLAMF*DLAMF)
    DLNQ2=T-DLNLAM
    
    ALPHA=4.D0*PI/B0/DLNQ2    ! LO
    IF(IORDER.EQ.2) THEN      ! NLO
       ALPHA=ALPHA*(1.D0-B1*DLOG(DLNQ2)/(B0*B0*DLNQ2))
    END IF
    
    ! Return alpha_s(Q^2)/(2*PI) (LO OR NLO)
    ALP2PI=ALPHA/2.D0/PI

    RETURN
  END SUBROUTINE GETALP

  FUNCTION ISERCH(N,X,Y)
    ! ---------------------------------------------------------------------
    ! THIS FUNCTION SEARCHES "I" WHICH SATISFIES THE RELATION 
    ! X(I) <= Y < X(I+1) BY USING A BINARY SEARCH.
    ! ---------------------------------------------------------------------
    IMPLICIT NONE
    INTEGER::ISERCH
    INTEGER,INTENT(IN)::N
    REAL(KIND(1d0)),DIMENSION(*),INTENT(IN)::X
    REAL(KIND(1d0)),INTENT(IN)::Y
    INTEGER::MIN,MAX,MID

    MIN=1
    MAX=N+1

10  CONTINUE
    MID=(MIN+MAX)/2
    IF(Y.LT.X(MID)) THEN
       MAX=MID
    ELSE
       MIN=MID
    END IF
    IF((MAX-MIN).GT.1) GO TO 10
    
    ISERCH=MIN
    
    RETURN
  END FUNCTION ISERCH

END MODULE ff_common
