MODULE fragfun_HKNS07
  USE ff_common
  USE FF_DGLAP
  USE FF_DGLAP_GLOBAL
  USE FF_INIT
  IMPLICIT NONE
CONTAINS
  SUBROUTINE GETFF_pionp_HKNS07(Q2,XX,FF,IMETH)
    ! return FF=D at scale Sqrt(Q2) and X=XX
    ! IMETH: 0, using the FF_DGLAP.f90
    ! IMETH: 1, using FF_DGLAP.f90 and write into the grid
    ! IMETH: 2, using the grid
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q2,XX
    INTEGER,INTENT(IN)::IMETH
    REAL(KIND(1d0)),DIMENSION(-5:5),INTENT(OUT)::FF
    INCLUDE 'getff_common.inc'
    INCLUDE 'fragfun_grid.inc'
    INTEGER::init=0
    SAVE init
    INTEGER::I,J,K
    REAL(KIND(1d0))::T,DX
    LOGICAL::lexist
    INTEGER::lunit=20345
    INTEGER,DIMENSION(3)::IXXG
    
    IF(IMETH.EQ.2)THEN
       IF(init.EQ.0)THEN
          ! read the grid
          OPEN(UNIT=lunit,FILE='./grid/FF_HKNS07/FF_pionp_Q2T_X.grid')
          INCLUDE 'read_ff_q2t_x.inc'
          CLOSE(UNIT=lunit)
          OPEN(UNIT=lunit,FILE='./grid/FF_HKNS07/FF_pionp_FBCDXG.grid')
          INCLUDE 'read_ff_ffg_bxg_cxg_dxg.inc'
          CLOSE(UNIT=lunit)
          init=1
       ENDIF
       DO I=-5,5
          FF(I)=0.D0  ! Initializing arrary
       END DO
       IF(XX.GT.1.D0) RETURN
       ! INTERPOLATION.
       ! X: CUBIC SPLINE INTERPOLATION, LOG(Q2): LINEAR INTERPOLATION.
       J=ISERCH(NT_GRID+1,Q2T_GRID,Q2)
       IF(J.EQ.NT_GRID+1) J=NT_GRID
       K=ISERCH(NX_GRID+1,X_GRID,XX)
       DX=XX-X_GRID(K)
       DO I=1,8
          PDFJ1(I)=FFG_GRID(K,J,I) &
               +DX*(BXG_GRID(K,J,I)+DX*(CXG_GRID(K,J,I)&
               +DX*DXG_GRID(K,J,I)))
          PDFJ2(I)=FFG_GRID(K,J+1,I) &
               +DX*(BXG_GRID(K,J+1,I)+DX*(CXG_GRID(K,J+1,I)&
               +DX*DXG_GRID(K,J+1,I)))
       ENDDO
       T=(DLOG(Q2)-DLOG(Q2T_GRID(J)))/(DLOG(Q2T_GRID(J+1))-DLOG(Q2T_GRID(J)))
       ! I=1:gluon, 2:u, 3:d, 4:s, 5:ub, 6:db, 7:c, 8:b 
       DO I=1,8
          XFF(I)=(1.D0-T)*PDFJ1(I)+T*PDFJ2(I)
       ENDDO
       
       ! Changing label of FF array for being consistent with the LHAPDF
       FF(0)=XFF(1)/XX  ! gluon
       FF(1)=XFF(3)/XX  ! d
       FF(2)=XFF(2)/XX  ! u
       FF(3)=XFF(4)/XX  ! s
       FF(4)=XFF(7)/XX  ! c
       FF(5)=XFF(8)/XX  ! b
       
       FF(-1)=XFF(6)/XX ! db
       FF(-2)=XFF(5)/XX ! ub
       FF(-3)=FF(3)     ! sb
       FF(-4)=FF(4)     ! cb
       FF(-5)=FF(5)     ! bb
    ELSE
       IF(init.EQ.0)THEN
          FF_SET=1
          FF_DGLAP_IORDER=2
          FF_DGLAP_DLAM=0.323d0
          FF_DGLAP_mc_HQTHRE=1.43d0
          FF_DGLAP_mb_HQTHRE=4.3d0
          FF_DGLAP_Q2min=1D0
          FF_DGLAP_Q2max=MAX(1558.D+5,Q2*1.2D0)
          IF(XX.GE.1D-2)THEN
             FF_DGLAP_XMIN=1D-2
          ELSEIF(XX.GE.1D-3)THEN
             FF_DGLAP_XMIN=1D-3
          ELSEIF(XX.GE.1D-4)THEN
             FF_DGLAP_XMIN=1D-4
          ELSEIF(XX.GE.1D-5)THEN
             FF_DGLAP_XMIN=1D-5
          ELSEIF(XX.GE.1D-6)THEN
             FF_DGLAP_XMIN=1D-6
          ELSEIF(XX.GE.1D-7)THEN
             FF_DGLAP_XMIN=1D-8
          ELSE
             FF_DGLAP_XMIN=1D-16
          ENDIF
          FF_DGLAP_NT=580
          FF_DGLAP_NX=160
          FF_DGLAP_NGLI=32
          FF_DGLAP_NF=3 ! light flavor nf at Q02=Q2min
          CALL FF_DGLAP_EVOL() ! Making arrary of FF(XMIN:1.D0, Q2_ini:Q2_max)
          init=1
          IF(IMETH.EQ.1)THEN
             ! write to the grid
             OPEN(UNIT=lunit,FILE='./grid/FF_HKNS07/FF_pionp_INITIALIZE.grid')
             INCLUDE 'write_ff_initialize.inc'
             CLOSE(UNIT=lunit)
             OPEN(UNIT=lunit,FILE='./grid/FF_HKNS07/FF_pionp_Q2T_X.grid')
             INCLUDE 'write_ff_q2t_x.inc'
             CLOSE(UNIT=lunit)
             OPEN(UNIT=lunit,FILE='./grid/FF_HKNS07/FF_pionp_FBCDXG.grid')
             INCLUDE 'write_ff_ffg_bxg_cxg_dxg.inc'
             CLOSE(UNIT=lunit)
          ENDIF
       ENDIF
       CALL GETFF(Q2,XX,FF)
    ENDIF
  END SUBROUTINE GETFF_PIONP_HKNS07
END MODULE fragfun_HKNS07
