MODULE fragfun_HKNS07_ini
  IMPLICIT NONE
  ! the implementation of FF at the initial scale
  ! it is the pi^+ FF from HKNS07
  ! M. Hirai, S. Kumano, T.-H. Nagai, and K. Sudoh,
  ! Phys. Rev. D 75 (2007) 094009.
CONTAINS
  FUNCTION ff_pionp_HKNS07_ini(x,ifl)
    ! ifl= 0: gluon
    ! ifl= 1: u quark
    ! ifl= 2: d quark
    ! ifl= 3: s quark
    ! ifl= -1: ubar quark
    ! ifl= -2: dbar quark
    ! ifl= -3: sbar quark
    ! ifl= 4 or -4: c/cbar quark
    ! ifl= 5 or -5: b/bbar quark
    ! others are zero
    IMPLICIT NONE
    REAL(KIND(1d0))::ff_pionp_HKNS07_ini
    REAL(KIND(1d0)),INTENT(IN)::x
    INTEGER,INTENT(IN)::ifl
    IF(Ifl.EQ.0) THEN ! Gluon
       ff_pionp_HKNS07_ini=4.387047D+02*X**( 1.942747D+00)*&
            (1.D0-X)**8.000000D+00

    ELSE IF(Ifl.EQ.1) THEN  ! Up
       ff_pionp_HKNS07_ini=1.008181D+00*X**(-9.634418D-01)*&
            (1.D0-X)**1.369666D+00

    ELSE IF(Ifl.EQ.2) THEN  ! Down
       ff_pionp_HKNS07_ini=1.754279D+01*X**( 7.181598D-01)*&
            (1.D0-X)**6.265853D+00

    ELSE IF(Ifl.EQ.3) THEN  ! Strange
       ff_pionp_HKNS07_ini=1.754279D+01*X**( 7.181598D-01)*&
            (1.D0-X)**6.265853D+00

    ELSE IF(Ifl.EQ.-1) THEN ! ubar
       ff_pionp_HKNS07_ini=1.754279D+01*X**( 7.181598D-01)*&
            (1.D0-X)**6.265853D+00

    ELSE IF(Ifl.EQ.-2) THEN ! dbar
       ff_pionp_HKNS07_ini=1.008181D+00*X**(-9.634418D-01)*&
            (1.D0-X)**1.369666D+00

    ELSE IF(Ifl.EQ.-3) THEN ! sbar
       ff_pionp_HKNS07_ini=1.754279D+01*X**( 7.181598D-01)*&
            (1.D0-X)**6.265853D+00

    ELSE IF(ABS(Ifl).EQ.4) THEN ! c or cbar
       ff_pionp_HKNS07_ini=1.208375D+00*X**(-8.453058D-01)*&
            (1.D0-X)**3.868264D+00

    ELSE IF(ABS(Ifl).EQ.5) THEN ! b or bbar
       ff_pionp_HKNS07_ini=8.655283D-01*X**(-1.219219D+00)*&
            (1.D0-X)**5.667591D+00

    ELSE
       ff_pionp_HKNS07_ini=0.D0
    ENDIF
    RETURN
  END FUNCTION Ff_pionp_HKNS07_ini
END MODULE fragfun_HKNS07_ini
