MODULE nFFKretzer_interface
  USE FFKretzer_interface
  USE NM_effects
  IMPLICIT NONE
  REAL(KIND(1d0))::wc_FF=1d0
  REAL(KIND(1d0))::as_FF=0.5d0
CONTAINS
  ! nuclear fragmentation function
  FUNCTION CallnFFKretzer(z,eps,v,scale,PDG_parton,PDG_hadron,ifforder,iffset)
    ! Call FF provided by Kretzer with nuclear modified
    ! see Eq.(5.6) in hep-ph/0210104
    ! transfer the number density to the momentum density
    ! it is D(eps,v)*zstar*Dfh(zstar,Q^2)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,eps,v,scale
    INTEGER,INTENT(IN)::PDG_parton,PDG_hadron
    REAL(KIND(1d0))::CallnFFKretzer,zstar
    INTEGER,OPTIONAL,INTENT(IN)::ifforder,iffset
    REAL(KIND(1d0))::epsbar
    INTEGER::fforder,ffset
    REAL(KIND(1d0))::normalFF,Dval
    INTEGER::print1=0,print2=0,print3=0,print4=0,print5=0,print6=0,print7=0
    SAVE print1,print2,print3,print4,print5,print6,print7
    INTEGER::iquark
    IF(PRESENT(ifforder))THEN
       fforder=ifforder
       IF(fforder.LT.0.OR.fforder.GT.1)THEN
          IF(print1.EQ.0)THEN
             WRITE(*,*)"WARNING:order is invalid in CallFFKretzer ! Set it to be 0 (LO)"
             print1=1
          ENDIF
          fforder=0
       ENDIF
    ELSE
       fforder=0
    ENDIF
    IF(PRESENT(iffset))THEN
       ffset=iffset
       IF(ffset.LT.1.OR.ffset.GT.3)THEN
          IF(print2.EQ.0)THEN
             WRITE(*,*)"WARNING: set is invalid in CallFFKretzer ! Set it to be 2 (KKP)"
             print2=1
          ENDIF
          ffset=2
       ENDIF
    ELSE
       ffset=2
    ENDIF
    IF(ffset.EQ.3.AND.fforder.EQ.0)THEN
       IF(print3.EQ.0)THEN
          WRITE(*,*)"WARNING: No LO set for BFGW"
          WRITE(*,*)"WARNING: Choose NLO(MSbar) set instead"
          print3=1
       ENDIF
       ffset=1
    ENDIF
    zstar=z/(1d0-eps/v)
    ! normal FF,i.e. zstar*D(zsta,scale**2)
    normalFF=CallFFKretzer(zstar,scale,PDG_parton,PDG_hadron,&
         fforder,ffset)
    ! nuclear corrected with BDMPS spectrum
    IF(PDG_parton.EQ.21)THEN
       iquark=2
    ELSE
       iquark=1
    ENDIF
    epsbar=eps/wc_FF
    Ebar=v/wc_FF
    CALL BDMPS_energy_loss(epsbar,iquark,1,as_FF,Dval,.TRUE.)
    CallnFFKretzer=normalFF*Dval/wc_FF
    RETURN
  END FUNCTION CallnFFKretzer

  SUBROUTINE CallingnFFKretzer(z,eps,v,scale,PDG_hadron,ifforder,iffset,dff)
    ! Call FF provided by Kretzer with nuclear modified
    ! see Eq.(5.6) in hep-ph/0210104 
    ! transfer the number density to the momentum density
    ! it is D(eps,v)*zstar*Dfh(zstar,Q^2)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::z,eps,v,scale
    INTEGER,INTENT(IN)::PDG_hadron
    REAL(KIND(1d0))::Q2
    INTEGER,OPTIONAL,INTENT(IN)::ifforder,iffset
    INTEGER::fforder,ffset
    REAL(KIND(1d0)),DIMENSION(-5:5)::normalFF
    REAL(KIND(1d0))::Dval,epsbar,zstar
    INTEGER::iquark
    REAL(KIND(1d0)),DIMENSION(-5:5),INTENT(OUT)::dff
    INTEGER::print1=0,print2=0,print3=0,print4=0,print5=0,print6=0,print7=0
    SAVE print1,print2,print3,print4,print5,print6,print7
    IF(PRESENT(ifforder))THEN
       fforder=ifforder
       IF(fforder.LT.0.OR.fforder.GT.1)THEN
          IF(print1.EQ.0)THEN
             WRITE(*,*)"WARNING:order is invalid in CallingFFKretzer ! Set it to be 0 (LO)"
             print1=1
          ENDIF
          fforder=0
       ENDIF
    ELSE
       fforder=0
    ENDIF
    IF(PRESENT(iffset))THEN
       ffset=iffset
       IF(ffset.LT.1.OR.ffset.GT.3)THEN
          IF(print2.EQ.0)THEN
             WRITE(*,*)"WARNING: set is invalid in CallingFFKretzer ! Set it to be 2 (KKP)"
             print2=1
          ENDIF
          ffset=2
       ENDIF
    ELSE
       ffset=2
    ENDIF
    IF(ffset.EQ.3.AND.fforder.EQ.0)THEN
       IF(print3.EQ.0)THEN
          WRITE(*,*)"WARNING: No LO set for BFGW"
          WRITE(*,*)"WARNING: Choose NLO(MSbar) set instead"
          print3=1
       ENDIF
       ffset=1
    ENDIF
    zstar=z/(1d0-eps/v)
    ! normal FF, i.e. zstar*D(zsta,scale**2)
    CALL CallingFFKretzer(zstar,scale,PDG_hadron,&
         fforder,ffset,normalFF(-5:5))
    ! nuclear corrected with BDMPS spectrum
    epsbar=eps/wc_FF
    Ebar=v/wc_FF
    ! quark first
    iquark=1
    CALL BDMPS_energy_loss(epsbar,iquark,1,as_FF,Dval,.TRUE.)
    dff(-5:-1)=normalFF(-5:-1)*Dval/wc_FF
    dff(1:5)=normalFF(1:5)*Dval/wc_FF
    ! gluon second
    iquark=2
    CALL BDMPS_energy_loss(epsbar,iquark,1,as_FF,Dval,.TRUE.)
    dff(0)=normalFF(0)*Dval/wc_FF
    RETURN
  END SUBROUTINE CallingnFFKretzer

END MODULE NFFKretzer_interface
