MODULE CalculateCentrality
  USE OpticalGlauber_Geometry
  IMPLICIT NONE
  INTEGER::n_centrality,n_bbins
  REAL(KIND(1d0)),DIMENSION(20,2)::centrality_bins,b_bins
  REAL(KIND(1d0)),DIMENSION(20,20)::wmatrix
  CHARACTER(len=20)::out_dir="./"
CONTAINS
  SUBROUTINE CalculateCentrality_Fact_AB(A_name,B_name)
    IMPLICIT NONE
    ! it uses the factorised form of RAB (Eq.(4.9) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber)
    ! assume G(x) is a polynomial in x
    ! seen the defition of G(x) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber
    INTEGER,PARAMETER::igmaxA=4,igmaxB=4 ! the maximal power in G(x) is x**igmax
    ! G(x)=N*(gcoeff(0)+gcoeff(1)*x+gcoeff(2)*x**2+gcoeff(3)*x**3+...)
    ! N is to be determined !
    REAL(KIND(1d0)),DIMENSION(0:igmaxA)::gcoeffA=(/0d0,1d0,0d0,0d0,0d0/)
    REAL(KIND(1d0)),DIMENSION(0:igmaxB)::gcoeffB=(/0d0,1d0,0d0,0d0,0d0/)
    REAL(KIND(1d0))::NGNormA,NGNormB ! the normalisation factor N in G(x)
    ! A is the nuclei 
    CHARACTER(len=7),INTENT(IN)::A_name,B_name
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TABhat_integ ! the integration of TABhat via bin by bin in b, (0 component is the sum of all bins)
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TABhat_integ_cen ! the integration of TABhat via bin by bin in centrality
    REAL(KIND(1d0)),DIMENSION(:,:,:),ALLOCATABLE::TABhatInG_integ ! the integration of TAhat**(1+i)*TBhat**(1+j) if (gcoeffA(i)=!=0 or i==1) and (gcoeffB(j)=!=0 or j==1) 
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::TABhat0InG ! TAhat(0)**i*TBhat(0)**j if (gcoeffA(i)=!=0 or i==1) and (gcoeffB(j)=!=0 or j==1)
    INTEGER::i,j,jj,mm
    REAL(KIND(1d0))::TAhat0,TBhat0
    REAL(KIND(1d0)),DIMENSION(0:1)::TAhat_integ,TBhat_integ
    REAL(KIND(1d0)),DIMENSION(1,2)::btemp_bins
    REAL(KIND(1d0)),DIMENSION(0:igmaxA,0:1)::TAhatInG_integ
    REAL(KIND(1d0)),DIMENSION(0:igmaxB,0:1)::TBhatInG_integ
    REAL(KIND(1d0)),DIMENSION(0:igmaxA)::TAhat0InG
    REAL(KIND(1d0)),DIMENSION(0:igmaxB)::TBhat0InG
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::coeffs ! the coefficients of sigma_pp, sigma_Ap/A, sigma_pB/B, sigma_AB/AB in each b bin
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::coeffs_cen ! the coefficients of sigma_pp, sigma_Ap/A, sigma_pB/B, sigma_AB/AB in each centrality bin
    WRITE(*,*)"INFO: Calculate the centrality in optical Glauber model in AB collisions (will take a few minutes)"
    ! first to read the information of centralities
    n_centrality=0
    n_bbins=0
    CALL GetCentralityImpactB(n_centrality,n_bbins,centrality_bins,b_bins,wmatrix)
    ! prepare for determining the normalisations of G(x) in RA and RB only
    btemp_bins(1,1)=b_bins(1,1)
    btemp_bins(1,2)=b_bins(n_bbins,2)
    ! calculate the int_{bmin}^{bmax}{TAhat(b)2pi*bdb} for one bin
    CALL CalculateTAhat_WoodsSaxon_centrality(A_name,1,&
         btemp_bins(1:1,1:2),TAhat_integ)
    ! calculate the int_{bmin}^{bmax}{TAhat(b)2pi*bdb} for bin by bin
    CALL CalculateTAhat_WoodsSaxon_centrality(B_name,1,&
         btemp_bins(1:1,1:2),TBhat_integ)

    ! determine the normalisation NGNormA
    ! calculate the int_{bmin}^{bmax}{TAhat(b)**(1+i)*2pi*bdb} for bin by bin
    ! if gcoeffA(i)=!=0
    TAhat0=TAhat0_WoodsSaxon(A_name)
    NGNormA=0d0
    DO i=0,igmaxA
       IF(gcoeffA(i).EQ.0d0)THEN
          TAhatInG_integ(i,0:1)=0d0
          TAhat0InG(i)=1d0
       ELSE
          IF(i.EQ.0)THEN
             TAhatInG_integ(i,0:1)=TAhat_integ(0:1)
             TAhat0InG(i)=1d0
          ELSE
             CALL CalculateTAhat_WoodsSaxon_centrality(A_name,1,&
                  btemp_bins(1:1,1:2),&
                  TAhatInG_integ(i,0:1),1D0+DBLE(i))
             TAhat0InG(i)=TAhat0**i
          ENDIF
          NGNormA=NGNormA+gcoeffA(i)*TAhatInG_integ(i,0)/TAhat0InG(i)
       ENDIF
    ENDDO
    IF(NGNormA.EQ.0d0)THEN
       NGNormA=1d0
    ELSE
       NGNormA=1d0/NGNormA
    ENDIF

    ! determine the normalisation NGNormB
    ! calculate the int_{bmin}^{bmax}{TBhat(b)**(1+i)*2pi*bdb} for bin by bin
    ! if gcoeffB(i)=!=0
    TBhat0=TAhat0_WoodsSaxon(B_name)
    NGNormB=0d0
    DO i=0,igmaxB
       IF(gcoeffB(i).EQ.0d0)THEN
          TBhatInG_integ(i,0:1)=0d0
          TBhat0InG(i)=1d0
       ELSE
          IF(i.EQ.0)THEN
             TBhatInG_integ(i,0:1)=TBhat_integ(0:1)
             TBhat0InG(i)=1d0
          ELSE
             CALL CalculateTAhat_WoodsSaxon_centrality(B_name,1,&
                  btemp_bins(1:1,1:2),&
                  TBhatInG_integ(i,0:1),1D0+DBLE(i))
             TBhat0InG(i)=TBhat0**i
          ENDIF
          NGNormB=NGNormB+gcoeffB(i)*TBhatInG_integ(i,0)/TBhat0InG(i)
       ENDIF
    ENDDO
    IF(NGNormB.EQ.0d0)THEN
       NGNormB=1d0
    ELSE
       NGNormB=1d0/NGNormB
    ENDIF

    ! calculate the int_{bmin}^{bmax}{TABhat(b)2pi*bdb} for bin by bin
    ! it will be used as a denominator
    IF(ALLOCATED(TABhat_integ))THEN
       DEALLOCATE(TABhat_integ)
    ENDIF
    ALLOCATE(TABhat_integ(0:n_bbins))
    CALL CalculateTABhat_WoodsSaxon_centrality(A_name,B_name,n_bbins,&
         b_bins(1:n_bbins,1:2),TABhat_integ)

    ! calculate the int_{bmin}^{bmax}{\int_{-inf}^{+inf}{TAhat(\vec{s})**(1+i)*TBhat(\vec{s}-\vec{b})**(1+j)d^2\vec{s}}*2pi*bdb} for bin by bin
    ! if (gcoeffA(i)!=0 or i==0) and (gcoeffB(j)!=0 or j==0)
    IF(ALLOCATED(TABhatInG_integ))THEN
       DEALLOCATE(TABhatInG_integ)
    ENDIF
    ALLOCATE(TABhatInG_integ(0:igmaxA,0:igmaxB,0:n_bbins))
    IF(ALLOCATED(TABhat0InG))THEN
       DEALLOCATE(TABhat0InG)
    ENDIF
    ALLOCATE(TABhat0InG(0:igmaxA,0:igmaxB))
    DO i=0,igmaxA
       DO j=0,igmaxB
          IF((gcoeffA(i).NE.0d0.OR.i.EQ.0).AND.(gcoeffB(j).NE.0d0.OR.j.EQ.0))THEN
             IF(i.EQ.0.AND.j.EQ.0)THEN
                TABhatInG_integ(i,j,0:n_bbins)=TABhat_integ(0:n_bbins)
                TABhat0InG(i,j)=1d0
             ELSE
                CALL CalculateTAhatTBhat_WoodsSaxon_centrality(A_name,B_name,n_bbins,&
                     b_bins(1:n_bbins,1:2),TABhatInG_integ(i,j,0:n_bbins),&
                     1D0+DBLE(i),1D0+DBLE(j))
                TABhat0InG(i,j)=TAhat0**(i)*TBhat0**(j)
             ENDIF
          ELSE
             TABhatInG_integ(i,j,0:n_bbins)=0d0
             TABhat0InG(i,j)=1d0
          ENDIF
       ENDDO
    ENDDO

    ! calculate the coefficients of sigma_pp, sigma_Ap/A, sigma_pB/B, sigma_AB/AB in each b bin
    IF(ALLOCATED(coeffs))THEN
       DEALLOCATE(coeffs)
    ENDIF
    ALLOCATE(coeffs(1:n_bbins,1:4))
    DO i=1,n_bbins
       coeffs(i,1:4)=0d0
       DO jj=0,igmaxA
          DO mm=0,igmaxB
             IF(jj.EQ.0.AND.mm.EQ.0)THEN
                coeffs(i,1)=coeffs(i,1)+(1d0-1d0*NGNormA*gcoeffA(jj))&
                     *(1d0-1d0*NGNormB*gcoeffB(mm))&
                     *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                coeffs(i,2)=coeffs(i,2)+NGNormA*gcoeffA(jj)*(1d0-1d0*NGNormB*gcoeffB(mm))&
                     *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                coeffs(i,3)=coeffs(i,3)+(1d0-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)&
                     *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                coeffs(i,4)=coeffs(i,4)+NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)&
                     *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
             ELSEIF(jj.EQ.0.AND.mm.GT.0)THEN
                ! 1+(R_A-1)*NA*gcoeffA(0)=(1-NA*gcoeffA(0))+R_A*NA*gcoeffA(0) times
                ! (R_B-1)*NB*gcoeffB(mm)*TBhat**m/TBhat0**m
                IF(TABhatInG_integ(jj,mm,i).NE.0d0)THEN
                   coeffs(i,1)=coeffs(i,1)+(1d0-1d0*NGNormA*gcoeffA(jj))*(-1d0*NGNormB*gcoeffB(mm))&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,2)=coeffs(i,2)+NGNormA*gcoeffA(jj)*(-1d0*NGNormB*gcoeffB(mm))&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,3)=coeffs(i,3)+(1d0-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,4)=coeffs(i,4)+NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                ENDIF
             ELSEIF(jj.GT.0.AND.mm.EQ.0)THEN
                ! (R_A-1)*NA*gcoeffA(jj)*TAhat**j/TAhat0**j times
                ! 1+(R_B-1)*NB*gcoeffB(0)=(1-NB*gcoeffB(0))+R_B*NB*gcoeffB(0)
                IF(TABhatInG_integ(jj,mm,i).NE.0d0)THEN
                   coeffs(i,1)=coeffs(i,1)+(-1d0*NGNormA*gcoeffA(jj))*(1d0-1d0*NGNormB*gcoeffB(mm))&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,2)=coeffs(i,2)+NGNormA*gcoeffA(jj)*(1d0-1d0*NGNormB*gcoeffB(mm))&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,3)=coeffs(i,3)+(-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,4)=coeffs(i,4)+NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                ENDIF
             ELSE
                IF(gcoeffA(jj).NE.0d0.AND.gcoeffB(mm).NE.0d0)THEN
                   ! (R_A-1)*NA*gcoeffA(jj)*TAhat**j/TAhat0**j times
                   ! (R_B-1)*NB*gcoeffB(mm)*TBhat**m/TBhat0**m
                   coeffs(i,1)=coeffs(i,1)+(-1d0*NGNormA*gcoeffA(jj))*(-1d0*NGNormB*gcoeffB(mm))&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,2)=coeffs(i,2)+NGNormA*gcoeffA(jj)*(-1d0*NGNormB*gcoeffB(mm))&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,3)=coeffs(i,3)+(-1d0*NGNormA*gcoeffA(jj))*NGNormB*gcoeffB(mm)&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                   coeffs(i,4)=coeffs(i,4)+NGNormA*gcoeffA(jj)*NGNormB*gcoeffB(mm)&
                        *TABhatInG_integ(jj,mm,i)/TABhat0InG(jj,mm)
                ENDIF
             ENDIF
          ENDDO
       ENDDO
    ENDDO
    ! to get the coefficients of sigma_pp, sigma_Ap/A, sigma_pB/B, sigma_AB/AB in each centrality bin
    ! according to the weight
    ! and TABhat_integ needed for the denominator of R_{AB}(cen)
    IF(ALLOCATED(coeffs_cen))THEN
       DEALLOCATE(coeffs_cen)
    ENDIF
    ALLOCATE(coeffs_cen(1:n_centrality,1:4))
    IF(ALLOCATED(TABhat_integ_cen))THEN
       DEALLOCATE(TABhat_integ_cen)
    ENDIF
    ALLOCATE(TABhat_integ_cen(1:n_centrality))
    DO i=1,n_centrality
       coeffs_cen(i,1:4)=0d0
       TABhat_integ_cen(i)=0d0
       DO j=1,n_bbins
          coeffs_cen(i,1)=coeffs_cen(i,1)+wmatrix(i,j)*coeffs(j,1)
          coeffs_cen(i,2)=coeffs_cen(i,2)+wmatrix(i,j)*coeffs(j,2)
          coeffs_cen(i,3)=coeffs_cen(i,3)+wmatrix(i,j)*coeffs(j,3)
          coeffs_cen(i,4)=coeffs_cen(i,4)+wmatrix(i,j)*coeffs(j,4)
          TABhat_integ_cen(i)=TABhat_integ_cen(i)+wmatrix(i,j)*TABhat_integ(j)
       ENDDO
    ENDDO

    ! write into a file for the centrality coefficients
    ! cenmin   cenmax    bmin     bmax     TABhat     coeff_pp     coeff_Ap    coeff_pB    coeff_AB
    ! sigma_{AB}/(AB)(cen)=coeff_pp*sigma_pp+coeff_Ap*sigma_Ap/A+coeff_pB*sigma_pB/B+coeff_AB*sigma_AB/(AB)
    ! to get R_{AB}(cen)=sigma_{AB}/(AB)(cen)/(TABhat*sigma_pp)
    OPEN(unit=5991,FILE=TRIM(out_dir)//'centrality_coefficients.dat',&
         status='unknown')
    WRITE(5991,*)"# cenmin   cenmax  TABhat   coeff_pp   coeff_Ap   coeff_pB   coeff_AB"
    WRITE(5991,*)"# sigma_{AB}/(AB)(cen)=coeff_pp*sigma_pp+coeff_Ap*sigma_Ap/A+coeff_pB*sigma_pB/B+coeff_AB*sigma_AB/(AB)"
    WRITE(5991,*)"# R_{AB}(cen)=sigma_{AB}/(AB)(cen)/(TABhat*sigma_pp)"
    WRITE(5991,*)"# R_{AB}(cen1+cen2)=sigma_{AB}/(AB)(cen1+cen2)/(TABhat(cen1+cen2)*sigma_pp)"
    WRITE(5991,*)"# <Ncoll>=AB*TABhat*sigmainel_pp/(Ab), <sigma_{AB}>=(sigma_{AB}/(AB)(cen))*AB*sigmainel_pp/(Ab)"
    WRITE(5991,*)"# where the area Ab=Pi*(bmax**2-bmin**2), cen <-> [bmin,bmax]"
    WRITE(5991,*)"# then, R_{AB}(cen)=<sigma_{AB}>/(<Ncoll>*sigma_pp)"
    DO i=1,n_centrality
       WRITE(5991,'(6(e14.7,2x),e14.7)')centrality_bins(i,1:2),TABhat_integ_cen(i),coeffs_cen(i,1:4)
    ENDDO
    CLOSE(unit=5991)

    WRITE(*,*)"INFO: Done with centrality calculation"
    RETURN
  END SUBROUTINE CalculateCentrality_Fact_AB

  SUBROUTINE CalculateCentrality_NonFact_AB(A_name,B_name)
    IMPLICIT NONE
    ! it uses the factorised form of RAB (Eq.(4.8) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber)
    ! assume H(x) is a polynomial in x
    ! seen the defition of H(x) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber
    INTEGER,PARAMETER::ihmax=4 ! the maximal power in H(x) is x**ihmax
    ! H(x)=N*(hcoeff(0)+hcoeff(1)*x+hcoeff(2)*x**2+hcoeff(3)*x**3+...)
    ! N is to be determined !
    REAL(KIND(1d0)),DIMENSION(0:ihmax)::hcoeff=(/0d0,1d0,0d0,0d0,0d0/)
    REAL(KIND(1d0))::NHNorm ! the normalisation factor N in H(x)
    ! A is the nuclei
    CHARACTER(len=7),INTENT(IN)::A_name,B_name
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TABhat_integ ! the integration of TABhat via bin by bin in b (0 component is the sum of all bins)
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TABhat_integ_cen ! the integration of TABhat via bin by bin in centrality
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::TABhatInH_integ ! the integration of TABhat**(1+i) if hcoeff(i)=!=0
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TABhat0InH ! TABhat(0)**i if hcoeff(i)=!=0
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::coeffs ! the coefficients of sigma_pp, sigma_AB/(AB) in b bins
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::coeffs_cen ! the coefficients of sigma_pp, sigma_AB/(AB) in centrality bins
    INTEGER::i,j,jj
    REAL(KIND(1d0))::TABhat0,temp
    WRITE(*,*)"INFO: Initialize the centrality in optical Glauber model in AB collisions #2 (will take a few minutes)"
    ! first to read the information of centralities
    n_centrality=0
    n_bbins=0
    CALL GetCentralityImpactB(n_centrality,n_bbins,centrality_bins,b_bins,wmatrix)
    ! calculate the int_{bmin}^{bmax}{TAhat(b)2pi*bdb} for bin by bin
    ! it will be used as a denominator
    IF(ALLOCATED(TABhat_integ))THEN
       DEALLOCATE(TABhat_integ)
    ENDIF
    ALLOCATE(TABhat_integ(0:n_bbins))
    CALL CalculateTABhat_WoodsSaxon_centrality(A_name,B_name,n_bbins,&
         b_bins(1:n_bbins,1:2),TABhat_integ)

    ! calculate the int_{bmin}^{bmax}{TABhat(b)**(1+i)*2pi*bdb} for bin by bin
    ! if hcoeff(i)=!=0
    IF(ALLOCATED(TABhatInH_integ))THEN
       DEALLOCATE(TABhatInH_integ)
    ENDIF
    ALLOCATE(TABhatInH_integ(0:ihmax,0:n_bbins))
    IF(ALLOCATED(TABhat0InH))THEN
       DEALLOCATE(TABhat0InH)
    ENDIF
    ALLOCATE(TABhat0InH(0:ihmax))
    TABhat0=TABhat0_WoodsSaxon(A_name,B_name)
    NHNorm=0d0
    DO i=0,ihmax
       IF(hcoeff(i).EQ.0d0)THEN
          TABhatInH_integ(i,0:n_bbins)=0d0
          TABhat0InH(i)=1d0
       ELSE
          IF(i.EQ.0)THEN
             TABhatInH_integ(i,0:n_bbins)=TABhat_integ(0:n_bbins)
             TABhat0InH(i)=1d0
          ELSE
             CALL CalculateTABhat_WoodsSaxon_centrality(A_name,B_name,n_bbins,&
                  b_bins(1:n_bbins,1:2),&
                  TABhatInH_integ(i,0:n_bbins),1D0+DBLE(i))
             TABhat0InH(i)=TABhat0**i
          ENDIF
          NHNorm=NHNorm+hcoeff(i)*TABhatInH_integ(i,0)/TABhat0InH(i)
       ENDIF
    ENDDO
    IF(NHNorm.EQ.0d0)THEN
       NHNorm=1d0
    ELSE
       NHNorm=1d0/NHNorm
    ENDIF

    ! calculate the coefficients of sigma_pp, sigma_AB/(AB) in each b bin
    IF(ALLOCATED(coeffs))THEN
       DEALLOCATE(coeffs)
    ENDIF
    ALLOCATE(coeffs(1:n_bbins,1:2))
    DO i=1,n_bbins
       coeffs(i,1:2)=0d0
       coeffs(i,1)=coeffs(i,1)+TABhat_integ(i)
       DO jj=0,ihmax
          IF(hcoeff(jj).NE.0d0)THEN
             temp=NHNorm*hcoeff(jj)*TABhatInH_integ(jj,i)/TABhat0InH(jj)
             coeffs(i,1)=coeffs(i,1)-temp
             coeffs(i,2)=coeffs(i,2)+temp
          ENDIF
       ENDDO
    ENDDO

    ! to get the coefficients of sigma_pp, sigma_AB/AB in each centrality bin
    ! according to the weight
    ! and TABhat_integ needed for the denominator of R_{AB}(cen)
    IF(ALLOCATED(coeffs_cen))THEN
       DEALLOCATE(coeffs_cen)
    ENDIF
    ALLOCATE(coeffs_cen(1:n_centrality,1:2))
    IF(ALLOCATED(TABhat_integ_cen))THEN
       DEALLOCATE(TABhat_integ_cen)
    ENDIF
    ALLOCATE(TABhat_integ_cen(1:n_centrality))
    DO i=1,n_centrality
       coeffs_cen(i,1:2)=0d0
       TABhat_integ_cen(i)=0d0
       DO j=1,n_bbins
          coeffs_cen(i,1)=coeffs_cen(i,1)+wmatrix(i,j)*coeffs(j,1)
          coeffs_cen(i,2)=coeffs_cen(i,2)+wmatrix(i,j)*coeffs(j,2)
          TABhat_integ_cen(i)=TABhat_integ_cen(i)+wmatrix(i,j)*TABhat_integ(j)
       ENDDO
    ENDDO

    ! write into a file for the centrality coefficients 
    ! cenmin   cenmax    TABhat     coeff_pp     coeff_AB
    ! sigma_{AB}/(AB)(cen)=coeff_pp*sigma_pp+coeff_AB*sigma_AB/(AB)
    ! to get R_{AB}(cen)=sigma_{AB}/(AB)(cen)/(TABhat*sigma_pp)
    OPEN(unit=5993,FILE=TRIM(out_dir)//'centrality_coefficients.dat',&
         status='unknown')
    WRITE(5993,*)"# cenmin   cenmax  TABhat   coeff_pp   coeff_AB"
    WRITE(5993,*)"# sigma_{AB}/(AB)(cen)=coeff_pp*sigma_pp+coeff_AB*sigma_AB/(AB)"
    WRITE(5993,*)"# R_{AB}(cen)=sigma_{AB}/(AB)(cen)/(TABhat*sigma_pp)"
    WRITE(5991,*)"# R_{AB}(cen1+cen2)=sigma_{AB}/(AB)(cen1+cen2)/(TABhat(cen1+cen2)*sigma_pp)"
    WRITE(5991,*)"# <Ncoll>=AB*TABhat*sigmainel_pp/(Ab), <sigma_{AB}>=(sigma_{AB}/(AB)(cen))*AB*sigmainel_pp/(Ab)"
    WRITE(5991,*)"# where the area Ab=Pi*(bmax**2-bmin**2), cen <-> [bmin,bmax]"
    WRITE(5991,*)"# then, R_{AB}(cen)=<sigma_{AB}>/(<Ncoll>*sigma_pp)"
    DO i=1,n_centrality
       WRITE(5993,'(4(e14.7,2x),e14.7)')centrality_bins(i,1:2),TABhat_integ_cen(i),coeffs_cen(i,1:2)
    ENDDO
    CLOSE(unit=5993)

    WRITE(*,*)"INFO: Done with centrality calculation"
    RETURN
  END SUBROUTINE CalculateCentrality_NonFact_AB

  SUBROUTINE CalculateCentrality_pA(A_name)
    IMPLICIT NONE
    ! assume G(x) is a polynomial in x 
    ! seen the defition of G(x) in /Users/erdissshaw/Works/Manuscript/OpticalGlauber
    INTEGER,PARAMETER::igmax=4 ! the maximal power in G(x) is x**igmax
    ! G(x)=N*(gcoeff(0)+gcoeff(1)*x+gcoeff(2)*x**2+gcoeff(3)*x**3+...)
    ! N is to be determined !
    REAL(KIND(1d0)),DIMENSION(0:igmax)::gcoeff=(/0d0,1d0,0d0,0d0,0d0/)
    REAL(KIND(1d0))::NGNorm ! the normalisation factor N in G(x)
    ! A is the nuclei 
    CHARACTER(len=7),INTENT(IN)::A_name
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TAhat_integ ! the integration of TAhat via bin by bin in b (0 component is the sum of all bins)
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TAhat_integ_cen ! the integration of TAhat via bin by bin in centrality
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::TAhatInG_integ ! the integration of TAhat**(1+i) if gcoeff(i)=!=0
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::TAhat0InG ! TAhat(0)**i if gcoeff(i)=!=0
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::coeffs ! the coefficients of sigma_pp, sigma_pA/A (or sigma_Ap/A) in b bins
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::coeffs_cen ! the coefficients of sigma_pp, sigma_pA/A (or sigma_Ap/A) in centrality bins
    INTEGER::i,j,jj
    REAL(KIND(1d0))::TAhat0,temp
    WRITE(*,*)"INFO: Initialize the centrality in optical Glauber model in pA or Ap collisions (will take a few minutes)"
    ! first to read the information of centralities
    n_centrality=0
    n_bbins=0
    CALL GetCentralityImpactB(n_centrality,n_bbins,centrality_bins,b_bins,wmatrix)
    ! calculate the int_{bmin}^{bmax}{TAhat(b)2pi*bdb} for bin by bin
    ! it will be used as a denominator
    IF(ALLOCATED(TAhat_integ))THEN
       DEALLOCATE(TAhat_integ)
    ENDIF
    ALLOCATE(TAhat_integ(0:n_bbins))
    CALL CalculateTAhat_WoodsSaxon_centrality(A_name,n_bbins,&
         b_bins(1:n_bbins,1:2),TAhat_integ)

    ! calculate the int_{bmin}^{bmax}{TAhat(b)**(1+i)*2pi*bdb} for bin by bin
    ! if gcoeff(i)=!=0
    IF(ALLOCATED(TAhatInG_integ))THEN
       DEALLOCATE(TAhatInG_integ)
    ENDIF
    ALLOCATE(TAhatInG_integ(0:igmax,0:n_bbins))
    IF(ALLOCATED(TAhat0InG))THEN
       DEALLOCATE(TAhat0InG)
    ENDIF
    ALLOCATE(TAhat0InG(0:igmax))
    TAhat0=TAhat0_WoodsSaxon(A_name)
    NGNorm=0d0
    DO i=0,igmax
       IF(gcoeff(i).EQ.0d0)THEN
          TAhatInG_integ(i,0:n_bbins)=0d0
          TAhat0InG(i)=1d0
       ELSE
          IF(i.EQ.0)THEN
             TAhatInG_integ(i,0:n_bbins)=TAhat_integ(0:n_bbins)
             TAhat0InG(i)=1d0
          ELSE
             CALL CalculateTAhat_WoodsSaxon_centrality(A_name,n_bbins,&
                  b_bins(1:n_bbins,1:2),&
                  TAhatInG_integ(i,0:n_bbins),1D0+DBLE(i))
             TAhat0InG(i)=TAhat0**i
          ENDIF
          NGNorm=NGNorm+gcoeff(i)*TAhatInG_integ(i,0)/TAhat0InG(i)
       ENDIF
    ENDDO
    IF(NGNorm.EQ.0d0)THEN
       NGNorm=1d0
    ELSE
       NGNorm=1d0/NGNorm
    ENDIF

    ! calculate the coefficients of sigma_pp, sigma_Ap/A (or sigma_pA/A) in each b bin
    IF(ALLOCATED(coeffs))THEN
       DEALLOCATE(coeffs)
    ENDIF
    ALLOCATE(coeffs(1:n_bbins,1:2))
    DO i=1,n_bbins
       coeffs(i,1:2)=0d0
       coeffs(i,1)=coeffs(i,1)+TAhat_integ(i)
       DO jj=0,igmax
          IF(gcoeff(jj).NE.0d0)THEN
             temp=NGNorm*gcoeff(jj)*TAhatInG_integ(jj,i)/TAhat0InG(jj)
             coeffs(i,1)=coeffs(i,1)-temp
             coeffs(i,2)=coeffs(i,2)+temp
          ENDIF
       ENDDO
    ENDDO

    ! to get the coefficients of sigma_pp, sigma_pA/A (or sigma_Ap/A) in each centrality bin
    ! according to the weight
    ! and TAhat_integ needed for the denominator of R_{pA}(cen) (or R_{Ap}(cen))
    IF(ALLOCATED(coeffs_cen))THEN
       DEALLOCATE(coeffs_cen)
    ENDIF
    ALLOCATE(coeffs_cen(1:n_centrality,1:2))
    IF(ALLOCATED(TAhat_integ_cen))THEN
       DEALLOCATE(TAhat_integ_cen)
    ENDIF
    ALLOCATE(TAhat_integ_cen(1:n_centrality))
    DO i=1,n_centrality
       coeffs_cen(i,1:2)=0d0
       TAhat_integ_cen(i)=0d0
       DO j=1,n_bbins
          coeffs_cen(i,1)=coeffs_cen(i,1)+wmatrix(i,j)*coeffs(j,1)
          coeffs_cen(i,2)=coeffs_cen(i,2)+wmatrix(i,j)*coeffs(j,2)
          TAhat_integ_cen(i)=TAhat_integ_cen(i)+wmatrix(i,j)*TAhat_integ(j)
       ENDDO
    ENDDO

    ! write into a file for the centrality coefficients
    ! cenmin   cenmax    bmin     bmax     TAhat     coeff_pp     coeff_pA
    ! sigma_{pA}/(A)(cen)=coeff_pp*sigma_pp+coeff_pA*sigma_pA/A
    ! to get R_{pA}(cen)=sigma_{pA}/(A)(cen)/(TAhat*sigma_pp)
    OPEN(unit=5992,FILE=TRIM(out_dir)//'centrality_coefficients.dat',&
         status='unknown')
    WRITE(5992,*)"# cenmin   cenmax  TAhat   coeff_pp   coeff_pA"
    WRITE(5992,*)"# sigma_{pA}/(A)(cen)=coeff_pp*sigma_pp+coeff_pA*sigma_pA/A"
    WRITE(5992,*)"# R_{pA}(cen)=sigma_{pA}/(A)(cen)/(TAhat*sigma_pp)"
    WRITE(5991,*)"# R_{pA}(cen1+cen2)=sigma_{pA}/(A)(cen1+cen2)/(TAhat(cen1+cen2)*sigma_pp)"
    WRITE(5991,*)"# <Ncoll>=A*TAhat*sigmainel_pp/(Ab), <sigma_{pA}>=(sigma_{pA}/(A)(cen))*A*sigmainel_pp/(Ab)"
    WRITE(5991,*)"# where the area Ab=Pi*(bmax**2-bmin**2), cen <-> [bmin,bmax]"
    WRITE(5991,*)"# then, R_{pA}(cen)=<sigma_{pA}>/(<Ncoll>*sigma_pp)"
    DO i=1,n_centrality
       WRITE(5992,'(4(e14.7,2x),e14.7)')centrality_bins(i,1:2),TAhat_integ_cen(i),coeffs_cen(i,1:2)
    ENDDO
    CLOSE(unit=5992)

    WRITE(*,*)"INFO: Done with centrality calculation"
    RETURN
  END SUBROUTINE CalculateCentrality_pA
END MODULE CalculateCentrality
