MODULE Nuclear_Cuts_Module
  USE Helac_Global
  USE Kinetic_Func
  USE  Constants
  USE Cuts_Module
  IMPLICIT NONE
CONTAINS

  SUBROUTINE nuclear_cuts_HC(icut)
    IMPLICIT NONE
    REAL(KIND=DBL),DIMENSION(20,4)::p
    REAL(KIND=DBL),DIMENSION(4)::pboo,ponia,pboo2
    REAL(KIND=DBL)::pi,q,e,pt,eta,d1,d2,dr,pmiss_E,pmiss_x
    REAL(KIND=DBL)::pmiss_y,pmiss_z,pt_miss,s,t
    REAL(KIND=DBL)::exp1,exp2,aaa,bbb,ptcut
    INTEGER,INTENT(OUT)::icut
    INTEGER::i,l,flag,l1,l2,i_nu,i1
    pi=DACOS(-1d0)
    DO i=3,n
       p(i,1:4)=nuclear_parton_pmom(i,1:4)
    ENDDO
    icut=0
    ! invariant mass cuts
    DO l1=3,n-1
       DO l2=l1+1,n
          IF(l1.EQ.l2.OR.Quarkonium3(l1).EQ.Quarkonium3(l2).AND.Quarkonium3(l1).NE.0)CYCLE
          s=2*scalar_product(nuclear_parton_pmom(l1,1:4),&
               nuclear_parton_pmom(l2,1:4))&
               +scalar_product(nuclear_parton_pmom(l1,1:4),&
               nuclear_parton_pmom(l1,1:4))&
               +scalar_product(nuclear_parton_pmom(l2,1:4),&
               nuclear_parton_pmom(l2,1:4))
          IF(s.LT.gmas(l1,l2)**2)RETURN
       ENDDO
    ENDDO
    ! special for the beam cuts for NLO* or NNLO*
    DO l1=1,2
       DO l2=3,n
          t=scalar_product(nuclear_parton_pmom(l1,1:4),&
               nuclear_parton_pmom(l1,1:4))&
               +scalar_product(nuclear_parton_pmom(l2,1:4),&
               nuclear_parton_pmom(l2,1:4))&
               -2*scalar_product(nuclear_parton_pmom(l1,1:4),&
               nuclear_parton_pmom(l2,1:4))
          IF(DABS(t).LT.gbeammass(l1,l2)**2)RETURN
       ENDDO
    ENDDO
    
    flag=0
    DO l=3,n
       pt=SQRT(p(l,1)**2+p(l,2)**2)
       IF(pt.LT.ptc(l).AND.nhad.GT.3)THEN
	  flag=1
	  EXIT
       ENDIF
       IF(maxptc(l).GE.0d0.AND.nhad.GT.3)THEN
          IF(pt.GT.maxptc(l))THEN
             flag=1
             EXIT
          ENDIF
       ENDIF
       eta=prapidity(p(l,1:4))
       IF(ABS(eta).GT.etac(l).AND.nhad.GT.3)THEN 
          flag=1
          EXIT
       ENDIF
       eta=rapidity(p(l,1:4))
       IF(absrap)eta=ABS(eta)
       IF(eta.GT.yycut(l).OR.eta.LT.yycutlow(l))THEN
          flag=1
          EXIT
       ENDIF
       ! special for the first particle , which can be used to calculate the y distribution
       IF(l.EQ.3)THEN
          !		eta=rapidity(p(l,1:4))
          IF(eta.GT.y1cup.OR.eta.LT.y1clow)THEN
             flag=1
             EXIT
          ENDIF
       ENDIF
    ENDDO


    IF(flag.EQ.0)THEN
       DO l1=3,n
          DO l2=3,n
             IF(l1.EQ.l2.OR.Quarkonium3(l1).EQ.Quarkonium3(l2).AND.Quarkonium3(l1).NE.0)CYCLE
             d1=prapidity(p(l1,1:4))-prapidity(p(l2,1:4))
             d2=ph4(p(l1,1),p(l1,2),p(l1,3))-ph4(p(l2,1),p(l2,2),p(l2,3))
             d2=MIN(DABS(d2),2*pi-DABS(d2))
             IF(d2/pi.GT.1.d0)WRITE(*,*)d2/pi
             dr=SQRT(d1**2+d2**2)
             IF(dr.LT.drc(l1,l2))THEN 
                flag=1
                EXIT
             ENDIF
          ENDDO
	  IF(flag.EQ.1)EXIT
       ENDDO
       IF(flag.EQ.0)THEN
          i_nu=0
          pmiss_E=0.d0
          pmiss_x=0.d0
          pmiss_y=0.d0
          pmiss_z=0.d0
          DO l=3,n
             IF(ifl(l).LT.13)THEN
                i1=IABS(MOD(ifl(l),4))
             ENDIF
             IF(i1.EQ.1)THEN        !neutrino found
                i_nu=1
                pmiss_E = pmiss_E + p(l,4)
                pmiss_x = pmiss_x + p(l,1)
                pmiss_y = pmiss_y + p(l,2)
                pmiss_z = pmiss_z + p(l,3)
             ENDIF
          ENDDO
          pt_miss=SQRT((pmiss_x)**2+(pmiss_y)**2)
          IF(i_nu.NE.1 .OR. pt_miss.GE.ptmissc.OR.nhad.EQ.3)THEN
             icut=1
	  ENDIF
   ENDIF
ENDIF
! special for the beam cuts for NLO* or NNLO*
! special cutoffs in the literature
IF(icut.EQ.1.AND.flag.EQ.0)THEN
   IF(literature_cutoffs.EQ.14060484.OR.literature_cutoffs.EQ.15090000)THEN
      l1=3
      DO l=3,nhad
         IF(iflh(l).GT.440001.AND.iflh(l).LT.449999)THEN
            ! charmonia
            ponia(1:4)=p(l1,1:4)+p(l1+1,1:4)
            eta=rapidity(ponia(1:4))
            IF(ABS(eta).LT.1.2d0)THEN
               pt=SQRT(ponia(1)**2+ponia(2)**2)
               IF(pt.LE.6.5d0)THEN
                  icut=0
                  EXIT
               ENDIF
            ELSEIF(ABS(eta).GT.1.2d0.AND.ABS(eta).LT.1.43d0)THEN
               pt=SQRT(ponia(1)**2+ponia(2)**2)
               aaa=-8.695652173913045d0
               bbb=16.934782608695656d0
               ptcut=aaa*ABS(eta)+bbb
               IF(pt.LE.ptcut)THEN
                  icut=0
                  EXIT
               ENDIF
            ELSEIF(ABS(eta).GT.1.43d0.AND.ABS(eta).LT.2.2d0)THEN
               pt=SQRT(ponia(1)**2+ponia(2)**2)
               IF(pt.LE.4.5d0)THEN
                  icut=0
                  EXIT
               ENDIF
            ELSE
               icut=0
               EXIT
            ENDIF
            l1=l1+2
         ELSEIF(ABS(iflh(l)).GT.100)THEN
            l1=l1+2
         ELSE
            l1=l1+1
         ENDIF
      ENDDO
   ENDIF
ENDIF
! 1000 continue

!     ---
!     boost from lab to the collision frame for xF cut
!     ---
IF(xFcutflag.AND.icut.EQ.1.AND.flag.EQ.0)THEN
   q=ehat
   e=q/SQRT(xp1*xp2)
   IF(.NOT.labeqcoll)THEN
      ! collider lab frame:
      ! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
      ! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
      ! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
      ! beam2 = (FT_M/2,0,0,-FT_M/2),FT_M=beam(2)
      ! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
      ! collision frame:
      ! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
      ! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
      ! ehat=Sqrt(s);e=Sqrt(S)
      IF(.NOT.fixtarget)THEN
         pboo2(4)=(ebeam(1)+ebeam(2))
         pboo2(3)=-(ebeam(1)-ebeam(2))
      ELSE
         IF(.NOT.fixtargetrev)THEN
            pboo2(4)=(ebeam(1)+ebeam(2))
            pboo2(3)=-ebeam(1)
         ELSE
            pboo2(4)=(ebeam(1)+ebeam(2))
            pboo2(3)=ebeam(2)
         ENDIF
      ENDIF
      pboo2(1:2)=0
      ! boost from the lab frame to the collision frame
      DO i=3,n
         CALL boostl(e,pboo2,p(i,1:4))
      ENDDO
   ENDIF
   DO l=3,n
      eta=xFeynman(p(l,1:4),e)
      IF(eta.GT.xFcut(l).OR.eta.LT.xFcutlow(l))THEN
         icut=0
         EXIT
      ENDIF
   ENDDO
   IF(.NOT.labeqcoll)THEN
      pboo2(3)=-pboo2(3)
      ! boost back to the lab frame
      DO i=3,n
         CALL boostl(e,pboo2,p(i,1:4))
      ENDDO
   ENDIF
ENDIF

END SUBROUTINE nuclear_cuts_HC

END MODULE Nuclear_Cuts_Module
