MODULE integrate_overlappingF
  ! This module is just for the check purpose as we already
  ! derived the analytical expressions
  USE NINTLIB ! for multiple dimensional integrations
  USE nucleon_transverse_profile
  IMPLICIT NONE
CONTAINS

  FUNCTION integrate_F_hardsphere(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_F_hardsphere
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_hardsphere/vx_common,vy_common,r0_common
    vx_common=vx
    vy_common=vy
    r0_common=r0
    amin(1)=-r0
    amax(1)=r0
    amin(2)=-r0
    amax(2)=r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(integrand_F_hardsphere,amin,amax,2,sub_num,1,1d-5,&
         integrate_F_hardsphere,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_F_hardsphere

  FUNCTION integrand_F_hardsphere(dim_num,u)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_F_hardsphere
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::u
    REAL(KIND(1d0)),DIMENSION(dim_num)::umv
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_hardsphere/vx_common,vy_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_F_hardsphere is not a two dimensional function"
       STOP
    ENDIF
    umv(1)=u(1)-vx_common
    umv(2)=u(2)-vy_common
    integrand_F_hardsphere=tprofile_hardsphere(u(1),u(2),r0_common)&
         *tprofile_hardsphere(umv(1),umv(2),r0_common)
    RETURN
  END FUNCTION integrand_F_hardsphere

  FUNCTION integrate_F_gaussian(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_F_gaussian
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_gaussian/vx_common,vy_common,r0_common
    vx_common=vx
    vy_common=vy
    r0_common=r0
    amin(1)=-r0*100
    amax(1)=r0*100
    amin(2)=-r0*100
    amax(2)=r0*100
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_F_gaussian,amin,amax,2,sub_num,1,1d-5,&
         integrate_F_gaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_F_gaussian

  FUNCTION integrand_F_gaussian(dim_num,u)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_F_gaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::u
    REAL(KIND(1d0)),DIMENSION(dim_num)::umv
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_gaussian/vx_common,vy_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_F_gaussian is not a two dimensional function"
       STOP
    ENDIF
    umv(1)=u(1)-vx_common
    umv(2)=u(2)-vy_common
    integrand_F_gaussian=tprofile_gaussian(u(1),u(2),r0_common)&
         *tprofile_gaussian(umv(1),umv(2),r0_common)
    RETURN
  END FUNCTION integrand_F_gaussian

  FUNCTION integrate_F_doublegaussian(vx,vy,beta,r01,r02)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,beta,r01,r02
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_F_doublegaussian,rmax
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,vx_common,vy_common
    COMMON/F_doublegaussian/vx_common,vy_common,beta_common,r01_common,r02_common
    vx_common=vx
    vy_common=vy
    beta_common=beta
    r02_common=r02
    r01_common=r01
    rmax=MAX(DABS(r01),DABS(r02))
    amin(1)=-rmax*100
    amax(1)=rmax*100
    amin(2)=-rmax*100
    amax(2)=rmax*100
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_F_doublegaussian,amin,amax,2,sub_num,1,1d-5,&
         integrate_F_doublegaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_F_doublegaussian

  FUNCTION integrand_F_doublegaussian(dim_num,u)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_F_doublegaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::u
    REAL(KIND(1d0)),DIMENSION(dim_num)::umv
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,vx_common,vy_common
    COMMON/F_doublegaussian/vx_common,vy_common,beta_common,r01_common,r02_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_F_doublegaussian is not a two dimensional function"
       STOP
    ENDIF
    umv(1)=u(1)-vx_common
    umv(2)=u(2)-vy_common
    integrand_F_doublegaussian=tprofile_doublegaussian(u(1),u(2),beta_common,r01_common,r02_common)&
         *tprofile_doublegaussian(umv(1),umv(2),beta_common,r01_common,r02_common)
    RETURN
  END FUNCTION integrand_F_doublegaussian

  FUNCTION integrate_F_tophat(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_F_tophat
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_tophat/vx_common,vy_common,r0_common
    vx_common=vx
    vy_common=vy
    r0_common=r0
    amin(1)=-r0
    amax(1)=r0
    amin(2)=-r0
    amax(2)=r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(integrand_F_tophat,amin,amax,2,sub_num,1,1d-5,&
         integrate_F_tophat,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_F_tophat

  FUNCTION integrand_F_tophat(dim_num,u)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_F_tophat
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::u
    REAL(KIND(1d0)),DIMENSION(dim_num)::umv
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_tophat/vx_common,vy_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_F_tophat is not a two dimensional function"
       STOP
    ENDIF
    umv(1)=u(1)-vx_common
    umv(2)=u(2)-vy_common
    integrand_F_tophat=tprofile_tophat(u(1),u(2),r0_common)&
         *tprofile_tophat(umv(1),umv(2),r0_common)
    RETURN
  END FUNCTION integrand_F_tophat

  FUNCTION integrate_F_dipole(vx,vy,mg)
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,mg
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_F_dipole
    REAL(KIND(1d0))::mg_common,vx_common,vy_common
    COMMON/F_dipole/vx_common,vy_common,mg_common
    vx_common=vx
    vy_common=vy
    mg_common=mg
    amin(1)=-1d0/mg*100
    amax(1)=1d0/mg*100
    amin(2)=-1d0/mg*100
    amax(2)=1d0/mg*100
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_F_dipole,amin,amax,2,sub_num,1,1d-5,&
         integrate_F_dipole,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_F_dipole

  FUNCTION integrand_F_dipole(dim_num,u)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_F_dipole
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::u
    REAL(KIND(1d0)),DIMENSION(dim_num)::umv
    REAL(KIND(1d0))::mg_common,vx_common,vy_common
    COMMON/F_dipole/vx_common,vy_common,mg_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_F_dipole is not a two dimensional function"
       STOP
    ENDIF
    umv(1)=u(1)-vx_common
    umv(2)=u(2)-vy_common
    integrand_F_dipole=tprofile_dipole(u(1),u(2),mg_common)&
         *tprofile_dipole(umv(1),umv(2),mg_common)
    RETURN
  END FUNCTION integrand_F_dipole

  FUNCTION integrate_F_exponential(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_F_exponential
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_exponential/vx_common,vy_common,r0_common
    vx_common=vx
    vy_common=vy
    r0_common=r0
    amin(1)=-r0*100
    amax(1)=r0*100
    amin(2)=-r0*100
    amax(2)=r0*100
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_F_exponential,amin,amax,2,sub_num,1,1d-5,&
         integrate_F_exponential,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_F_exponential

  FUNCTION integrand_F_exponential(dim_num,u)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_F_exponential
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::u
    REAL(KIND(1d0)),DIMENSION(dim_num)::umv
    REAL(KIND(1d0))::r0_common,vx_common,vy_common
    COMMON/F_exponential/vx_common,vy_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_F_exponential is not a two dimensional function"
       STOP
    ENDIF
    umv(1)=u(1)-vx_common
    umv(2)=u(2)-vy_common
    integrand_F_exponential=tprofile_exponential(u(1),u(2),r0_common)&
         *tprofile_exponential(umv(1),umv(2),r0_common)
    RETURN
  END FUNCTION integrand_F_exponential

  FUNCTION integrate_sigmaeffpp_hardsphere(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_sigmaeffpp_hardsphere
    REAL(KIND(1d0))::r0_common
    COMMON/s_hardsphere/r0_common
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0 ! 1 mb = 0.1 fm^2
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(integrand_sigmaeffpp_hardsphere,amin,amax,2,sub_num,1,1d-5,&
         integrate_sigmaeffpp_hardsphere,ind,eval_num)
    integrate_sigmaeffpp_hardsphere=fm2tomb/integrate_sigmaeffpp_hardsphere
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_sigmaeffpp_hardsphere

  FUNCTION integrand_sigmaeffpp_hardsphere(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_sigmaeffpp_hardsphere
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0))::r0_common
    COMMON/s_hardsphere/r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_sigmaeffpp_hardsphere is not a two dimensional function"
       STOP
    ENDIF
    integrand_sigmaeffpp_hardsphere=overlappingF_hardsphere(v(1),v(2),r0_common)**2
    RETURN
  END FUNCTION integrand_sigmaeffpp_hardsphere
  
  FUNCTION integrate_sigmaeffpp_gaussian(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_sigmaeffpp_gaussian
    REAL(KIND(1d0))::r0_common
    COMMON/s_gaussian/r0_common
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0 ! 1 mb = 0.1 fm^2
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_sigmaeffpp_gaussian,amin,amax,2,sub_num,1,1d-5,&
         integrate_sigmaeffpp_gaussian,ind,eval_num)
    integrate_sigmaeffpp_gaussian=fm2tomb/integrate_sigmaeffpp_gaussian
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_sigmaeffpp_gaussian

  FUNCTION integrand_sigmaeffpp_gaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_sigmaeffpp_gaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0))::r0_common
    COMMON/s_gaussian/r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_sigmaeffpp_gaussian is not a two dimensional function"
       STOP
    ENDIF
    integrand_sigmaeffpp_gaussian=overlappingF_gaussian(v(1),v(2),r0_common)**2
    RETURN
  END FUNCTION integrand_sigmaeffpp_gaussian

  FUNCTION integrate_sigmaeffpp_doublegaussian(beta,r01,r02)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::beta,r01,r02
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_sigmaeffpp_doublegaussian,rmax
    REAL(KIND(1d0))::beta_common,r01_common,r02_common
    COMMON/s_doublegaussian/beta_common,r01_common,r02_common
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0 ! 1 mb = 0.1 fm^2
    beta_common=beta
    r01_common=r01
    r02_common=r02
    rmax=MAX(DABS(r01),DABS(r02))
    amin(1)=-200d0*rmax
    amax(1)=200d0*rmax
    amin(2)=-200d0*rmax
    amax(2)=200d0*rmax
    sub_num(1)=10000
    sub_num(2)=10000
    CALL ROMBERG_ND(integrand_sigmaeffpp_doublegaussian,amin,amax,2,sub_num,1,1d-5,&
         integrate_sigmaeffpp_doublegaussian,ind,eval_num)
    integrate_sigmaeffpp_doublegaussian=fm2tomb/integrate_sigmaeffpp_doublegaussian
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_sigmaeffpp_doublegaussian

  FUNCTION integrand_sigmaeffpp_doublegaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_sigmaeffpp_doublegaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0))::beta_common,r01_common,r02_common
    COMMON/s_doublegaussian/beta_common,r01_common,r02_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_sigmaeffpp_doublegaussian is not a two dimensional function"
       STOP
    ENDIF
    integrand_sigmaeffpp_doublegaussian=overlappingF_doublegaussian(v(1),v(2),beta_common,r01_common,r02_common)**2
    RETURN
  END FUNCTION integrand_sigmaeffpp_doublegaussian

  FUNCTION integrate_sigmaeffpp_tophat(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_sigmaeffpp_tophat
    REAL(KIND(1d0))::r0_common
    COMMON/s_tophat/r0_common
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0 ! 1 mb = 0.1 fm^2
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(integrand_sigmaeffpp_tophat,amin,amax,2,sub_num,1,1d-5,&
         integrate_sigmaeffpp_tophat,ind,eval_num)
    integrate_sigmaeffpp_tophat=fm2tomb/integrate_sigmaeffpp_tophat
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_sigmaeffpp_tophat

  FUNCTION integrand_sigmaeffpp_tophat(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_sigmaeffpp_tophat
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0))::r0_common
    COMMON/s_tophat/r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_sigmaeffpp_tophat is not a two dimensional function"
       STOP
    ENDIF
    integrand_sigmaeffpp_tophat=overlappingF_tophat(v(1),v(2),r0_common)**2
    RETURN
  END FUNCTION integrand_sigmaeffpp_tophat

  FUNCTION integrate_sigmaeffpp_dipole(mg)
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::mg
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_sigmaeffpp_dipole
    REAL(KIND(1d0))::mg_common
    COMMON/s_dipole/mg_common
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0 ! 1 mb = 0.1 fm^2
    mg_common=mg
    amin(1)=-200d0/mg
    amax(1)=200d0/mg
    amin(2)=-200d0/mg
    amax(2)=200d0/mg
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_sigmaeffpp_dipole,amin,amax,2,sub_num,1,1d-5,&
         integrate_sigmaeffpp_dipole,ind,eval_num)
    integrate_sigmaeffpp_dipole=fm2tomb/integrate_sigmaeffpp_dipole
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_sigmaeffpp_dipole

  FUNCTION integrand_sigmaeffpp_dipole(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_sigmaeffpp_dipole
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0))::mg_common
    COMMON/s_dipole/mg_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_sigmaeffpp_dipole is not a two dimensional function"
       STOP
    ENDIF
    integrand_sigmaeffpp_dipole=overlappingF_dipole(v(1),v(2),mg_common)**2
    RETURN
  END FUNCTION integrand_sigmaeffpp_dipole

  FUNCTION integrate_sigmaeffpp_exponential(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0))::integrate_sigmaeffpp_exponential
    REAL(KIND(1d0))::r0_common
    COMMON/s_exponential/r0_common
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0 ! 1 mb = 0.1 fm^2
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(integrand_sigmaeffpp_exponential,amin,amax,2,sub_num,1,1d-5,&
         integrate_sigmaeffpp_exponential,ind,eval_num)
    integrate_sigmaeffpp_exponential=fm2tomb/integrate_sigmaeffpp_exponential
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION integrate_sigmaeffpp_exponential

  FUNCTION integrand_sigmaeffpp_exponential(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::integrand_sigmaeffpp_exponential
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0))::r0_common
    COMMON/s_exponential/r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: integrand_sigmaeffpp_exponential is not a two dimensional function"
       STOP
    ENDIF
    integrand_sigmaeffpp_exponential=overlappingF_exponential(v(1),v(2),r0_common)**2
    RETURN
  END FUNCTION integrand_sigmaeffpp_exponential

END MODULE integrate_overlappingF
