MODULE nucleus_transverse_profile
  USE OpticalGlauber_Geometry
  USE nucleon_transverse_profile
  USE NINTLIB ! for multiple dimensional integrations
  IMPLICIT NONE
CONTAINS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                           !
! TnAhat = Int[F(v)*TAhat(v-b),dv]                                          !
!                                                                           !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! F: hardsphere
  FUNCTION TnAhat_WoodsSaxon_hardsphere(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using hardsphere
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_WoodsSaxon_hardsphere
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,w,aa,A,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_HS/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnAhat_fxn_WoodsSaxon_hardsphere,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_WoodsSaxon_hardsphere,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_WoodsSaxon_hardsphere

  FUNCTION TnAhat_fxn_WoodsSaxon_hardsphere(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_WoodsSaxon_hardsphere
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_HS/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_WoodsSaxon_hardsphere is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_WoodsSaxon_hardsphere=TAhat_WoodsSaxon(vmb(1),vmb(2),RA_common,wA_common,aaA_common,AA_common,1)&
         *overlappingF_hardsphere(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_WoodsSaxon_hardsphere

  FUNCTION TnAhat_HardSphere_hardsphere(bx,by,RR,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using hardsphere
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_HardSphere_hardsphere
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_HS/bx_common,by_common,RA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnAhat_fxn_HardSphere_hardsphere,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_HardSphere_hardsphere,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_HardSphere_hardsphere

  FUNCTION TnAhat_fxn_HardSphere_hardsphere(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_HardSphere_hardsphere
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_HS/bx_common,by_common,RA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_HardSphere_hardsphere is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_HardSphere_hardsphere=TAhat_hardsphere(vmb(1),vmb(2),RA_common)&
         *overlappingF_hardsphere(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_HardSphere_hardsphere

! F: gaussian
  FUNCTION TnAhat_WoodsSaxon_gaussian(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using Gaussian
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_WoodsSaxon_gaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,w,aa,A,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_G/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_WoodsSaxon_gaussian,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_WoodsSaxon_gaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_WoodsSaxon_gaussian

  FUNCTION TnAhat_fxn_WoodsSaxon_gaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_WoodsSaxon_gaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_G/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_WoodsSaxon_gaussian is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_WoodsSaxon_gaussian=TAhat_WoodsSaxon(vmb(1),vmb(2),RA_common,wA_common,aaA_common,AA_common,1)&
         *overlappingF_gaussian(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_WoodsSaxon_gaussian

  FUNCTION TnAhat_HardSphere_gaussian(bx,by,RR,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using gaussian
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_HardSphere_gaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_G/bx_common,by_common,RA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_HardSphere_gaussian,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_HardSphere_gaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_HardSphere_gaussian

  FUNCTION TnAhat_fxn_HardSphere_gaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_HardSphere_gaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_G/bx_common,by_common,RA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_HardSphere_gaussian is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_HardSphere_gaussian=TAhat_hardsphere(vmb(1),vmb(2),RA_common)&
         *overlappingF_gaussian(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_HardSphere_gaussian

! F: double gaussian
  FUNCTION TnAhat_WoodsSaxon_doublegaussian(bx,by,RR,w,aa,A,beta,r01,r02)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using double gaussian
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_WoodsSaxon_doublegaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,w,aa,A,beta,r01,r02
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnA_WS_DG/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,beta_common,&
         r01_common,r02_common
    bx_common=bx
    by_common=by
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    beta_common=beta
    r01_common=r01
    r02_common=r02
    amin(1)=-100d0*(r01+r02)
    amax(1)=100d0*(r01+r02)
    amin(2)=-100d0*(r01+r02)
    amax(2)=100d0*(r01+r02)
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_WoodsSaxon_doublegaussian,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_WoodsSaxon_doublegaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_WoodsSaxon_doublegaussian

  FUNCTION TnAhat_fxn_WoodsSaxon_doublegaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_WoodsSaxon_doublegaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnA_WS_DG/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,beta_common,&
         r01_common,r02_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_WoodsSaxon_doublegaussian is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_WoodsSaxon_doublegaussian=TAhat_WoodsSaxon(vmb(1),vmb(2),RA_common,wA_common,aaA_common,AA_common,1)&
         *overlappingF_doublegaussian(v(1),v(2),beta_common,r01_common,r02_common)
    RETURN
  END FUNCTION TnAhat_fxn_WoodsSaxon_doublegaussian

  FUNCTION TnAhat_HardSphere_doublegaussian(bx,by,RR,beta,r01,r02)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using double gaussian
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_HardSphere_doublegaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,beta,r01,r02
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnA_HS_DG/bx_common,by_common,RA_common,beta_common,r01_common,r02_common
    bx_common=bx
    by_common=by
    RA_common=RR
    beta_common=beta
    r01_common=r01
    r02_common=r02
    amin(1)=-100d0*(r01+r02)
    amax(1)=100d0*(r01+r02)
    amin(2)=-100d0*(r01+r02)
    amax(2)=100d0*(r01+r02)
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_HardSphere_doublegaussian,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_HardSphere_doublegaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_HardSphere_doublegaussian

  FUNCTION TnAhat_fxn_HardSphere_doublegaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_HardSphere_doublegaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnA_HS_DG/bx_common,by_common,RA_common,beta_common,r01_common,r02_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_HardSphere_doublegaussian is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_HardSphere_doublegaussian=TAhat_hardsphere(vmb(1),vmb(2),RA_common)&
         *overlappingF_doublegaussian(v(1),v(2),beta_common,r01_common,r02_common)
    RETURN
  END FUNCTION TnAhat_fxn_HardSphere_doublegaussian

! F: top hat
  FUNCTION TnAhat_WoodsSaxon_tophat(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using tophat
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_WoodsSaxon_tophat
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,w,aa,A,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_TH/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnAhat_fxn_WoodsSaxon_tophat,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_WoodsSaxon_tophat,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_WoodsSaxon_tophat

  FUNCTION TnAhat_fxn_WoodsSaxon_tophat(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_WoodsSaxon_tophat
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_TH/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_WoodsSaxon_tophat is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_WoodsSaxon_tophat=TAhat_WoodsSaxon(vmb(1),vmb(2),RA_common,wA_common,aaA_common,AA_common,1)&
         *overlappingF_tophat(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_WoodsSaxon_tophat

  FUNCTION TnAhat_HardSphere_tophat(bx,by,RR,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using tophat
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_HardSphere_tophat
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_TH/bx_common,by_common,RA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnAhat_fxn_HardSphere_tophat,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_HardSphere_tophat,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_HardSphere_tophat

  FUNCTION TnAhat_fxn_HardSphere_tophat(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_HardSphere_tophat
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_TH/bx_common,by_common,RA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_HardSphere_tophat is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_HardSphere_tophat=TAhat_hardsphere(vmb(1),vmb(2),RA_common)&
         *overlappingF_tophat(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_HardSphere_tophat

! F: dipole
  FUNCTION TnAhat_WoodsSaxon_dipole(bx,by,RR,w,aa,A,mg)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using dipole
    ! nucleon density in a nucleus is using WoodsSaxon
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_WoodsSaxon_dipole
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,w,aa,A,mg
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnA_WS_D/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,mg_common
    bx_common=bx
    by_common=by
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    mg_common=mg
    amin(1)=-200d0/mg
    amax(1)=200d0/mg
    amin(2)=-200d0/mg
    amax(2)=200d0/mg
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_WoodsSaxon_dipole,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_WoodsSaxon_dipole,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_WoodsSaxon_dipole

  FUNCTION TnAhat_fxn_WoodsSaxon_dipole(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_WoodsSaxon_dipole
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnA_WS_D/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,mg_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_WoodsSaxon_dipole is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_WoodsSaxon_dipole=TAhat_WoodsSaxon(vmb(1),vmb(2),RA_common,wA_common,aaA_common,AA_common,1)&
         *overlappingF_dipole(v(1),v(2),mg_common)
    RETURN
  END FUNCTION TnAhat_fxn_WoodsSaxon_dipole

  FUNCTION TnAhat_HardSphere_dipole(bx,by,RR,mg)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using dipole
    ! nucleon density in a nucleus is using HardSphere
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_HardSphere_dipole
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,mg
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnA_HS_D/bx_common,by_common,RA_common,mg_common
    bx_common=bx
    by_common=by
    RA_common=RR
    mg_common=mg
    amin(1)=-200d0/mg
    amax(1)=200d0/mg
    amin(2)=-200d0/mg
    amax(2)=200d0/mg
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_HardSphere_dipole,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_HardSphere_dipole,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_HardSphere_dipole

  FUNCTION TnAhat_fxn_HardSphere_dipole(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_HardSphere_dipole
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnA_HS_D/bx_common,by_common,RA_common,mg_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_HardSphere_dipole is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_HardSphere_dipole=TAhat_hardsphere(vmb(1),vmb(2),RA_common)&
         *overlappingF_dipole(v(1),v(2),mg_common)
    RETURN
  END FUNCTION TnAhat_fxn_HardSphere_dipole

! F: exponential
  FUNCTION TnAhat_WoodsSaxon_exponential(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using exponential
    ! nucleon density in a nucleus is using WoodsSaxon                                                                                         
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_WoodsSaxon_exponential
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,w,aa,A,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_E/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_WoodsSaxon_exponential,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_WoodsSaxon_exponential,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_WoodsSaxon_exponential

  FUNCTION TnAhat_fxn_WoodsSaxon_exponential(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_WoodsSaxon_exponential
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_WS_E/bx_common,by_common,RA_common,wA_common,aaA_common,AA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_WoodsSaxon_exponential is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_WoodsSaxon_exponential=TAhat_WoodsSaxon(vmb(1),vmb(2),RA_common,wA_common,aaA_common,AA_common,1)&
         *overlappingF_exponential(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_WoodsSaxon_exponential

  FUNCTION TnAhat_HardSphere_exponential(bx,by,RR,r0)
    ! Int[F(v)*TAhat(v-b),dv]
    ! parton density in a nucleon is using exponential                                                                                              
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_HardSphere_exponential
    REAL(KIND(1d0)),INTENT(IN)::bx,by,RR,r0
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_E/bx_common,by_common,RA_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnAhat_fxn_HardSphere_exponential,amin,amax,2,sub_num,1,1d-5,&
         TnAhat_HardSphere_exponential,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnAhat_HardSphere_exponential

  FUNCTION TnAhat_fxn_HardSphere_exponential(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnAhat_fxn_HardSphere_exponential
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnA_HS_E/bx_common,by_common,RA_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnAhat_fxn_HardSphere_exponential is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    TnAhat_fxn_HardSphere_exponential=TAhat_hardsphere(vmb(1),vmb(2),RA_common)&
         *overlappingF_exponential(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnAhat_fxn_HardSphere_exponential

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                           !
! TnABhat = Int[F(v)*TABhat(v-b),dv]                                        !
!                                                                           !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! F: hard sphere
  FUNCTION TnABhat_WoodsSaxon_hardsphere(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using hardsphere
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_WoodsSaxon_hardsphere
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR,w,aa,A
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_HS/bx_common,by_common,RA_common,RB_common,&
         wA_common,wB_common,aaA_common,aaB_common,AA_common,AB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    wA_common=w(1)
    wB_common=w(2)
    aaA_common=aa(1)
    aaB_common=aa(2)
    AA_common=A(1)
    AB_common=A(2)
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnABhat_fxn_WoodsSaxon_hardsphere,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_WoodsSaxon_hardsphere,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_WoodsSaxon_hardsphere
  
  FUNCTION TnABhat_fxn_WoodsSaxon_hardsphere(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_WoodsSaxon_hardsphere
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR,w,aa,A
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_HS/bx_common,by_common,RA_common,RB_common,&
         wA_common,wB_common,aaA_common,aaB_common,AA_common,AB_common,r0_common
    LOGICAL::storegrid
    COMMON/TAB_WoodsSaxon_Grid/storegrid
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_WoodsSaxon_hardsphere is not a two dimensional function"
       STOP
    ENDIF
    storegrid=.TRUE.
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    w(1)=wA_common
    w(2)=wB_common
    aa(1)=aaA_common
    aa(2)=aaB_common
    A(1)=AA_common
    A(2)=AB_common
    TnABhat_fxn_WoodsSaxon_hardsphere=TABhat_WoodsSaxon(vmb(1),vmb(2),RR,w,aa,A)&
         *overlappingF_hardsphere(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_WoodsSaxon_hardsphere

  FUNCTION TnABhat_HardSphere_hardsphere(bx,by,RR,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using hardsphere
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_HardSphere_hardsphere
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_HS/bx_common,by_common,RA_common,RB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnABhat_fxn_HardSphere_hardsphere,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_HardSphere_hardsphere,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_HardSphere_hardsphere

  FUNCTION TnABhat_fxn_HardSphere_hardsphere(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_HardSphere_hardsphere
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_HS/bx_common,by_common,RA_common,RB_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_HardSphere_hardsphere is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    TnABhat_fxn_HardSphere_hardsphere=TABhat_hardsphere(vmb(1),vmb(2),RR)&
         *overlappingF_hardsphere(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_HardSphere_hardsphere


! F: gaussian
  FUNCTION TnABhat_WoodsSaxon_gaussian(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using Gaussian
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_WoodsSaxon_gaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR,w,aa,A
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_G/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    wA_common=w(1)
    wB_common=w(2)
    aaA_common=aa(1)
    aaB_common=aa(2)
    AA_common=A(1)
    AB_common=A(2)
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_WoodsSaxon_gaussian,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_WoodsSaxon_gaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_WoodsSaxon_gaussian

  FUNCTION TnABhat_fxn_WoodsSaxon_gaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_WoodsSaxon_gaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR,w,aa,A
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_G/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,r0_common
    LOGICAL::storegrid
    COMMON/TAB_WoodsSaxon_Grid/storegrid
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_WoodsSaxon_gaussian is not a two dimensional function"
       STOP
    ENDIF
    storegrid=.TRUE.
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    w(1)=wA_common
    w(2)=wB_common
    aa(1)=aaA_common
    aa(2)=aaB_common
    A(1)=AA_common
    A(2)=AB_common
    TnABhat_fxn_WoodsSaxon_gaussian=TABhat_WoodsSaxon(vmb(1),vmb(2),RR,w,aa,A)&
         *overlappingF_gaussian(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_WoodsSaxon_gaussian

  FUNCTION TnABhat_HardSphere_gaussian(bx,by,RR,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using gaussian
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_HardSphere_gaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_G/bx_common,by_common,RA_common,RB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_HardSphere_gaussian,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_HardSphere_gaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_HardSphere_gaussian

  FUNCTION TnABhat_fxn_HardSphere_gaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_HardSphere_gaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_G/bx_common,by_common,RA_common,RB_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_HardSphere_gaussian is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    TnABhat_fxn_HardSphere_gaussian=TABhat_hardsphere(vmb(1),vmb(2),RR)&
         *overlappingF_gaussian(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_HardSphere_gaussian

! F: double gaussian
  FUNCTION TnABhat_WoodsSaxon_doublegaussian(bx,by,RR,w,aa,A,beta,r01,r02)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using double gaussian
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_WoodsSaxon_doublegaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,beta,r01,r02
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR,w,aa,A
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common,beta_common
    REAL(KIND(1d0))::r01_common,r02_common,bx_common,by_common
    COMMON/TnAB_WS_DG/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,beta_common,r01_common,r02_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    wA_common=w(1)
    wB_common=w(2)
    aaA_common=aa(1)
    aaB_common=aa(2)
    AA_common=A(1)
    AB_common=A(2)
    beta_common=beta
    r01_common=r01
    r02_common=r02
    amin(1)=-100d0*(r01+r02)
    amax(1)=100d0*(r01+r02)
    amin(2)=-100d0*(r01+r02)
    amax(2)=100d0*(r01+r02)
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_WoodsSaxon_doublegaussian,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_WoodsSaxon_doublegaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_WoodsSaxon_doublegaussian

  FUNCTION TnABhat_fxn_WoodsSaxon_doublegaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_WoodsSaxon_doublegaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR,w,aa,A
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnAB_WS_DG/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,beta_common,r01_common,r02_common
    LOGICAL::storegrid
    COMMON/TAB_WoodsSaxon_Grid/storegrid
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_WoodsSaxon_doublegaussian is not a two dimensional function"
       STOP
    ENDIF
    storegrid=.TRUE.
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    w(1)=wA_common
    w(2)=wB_common
    aa(1)=aaA_common
    aa(2)=aaB_common
    A(1)=AA_common
    A(2)=AB_common
    TnABhat_fxn_WoodsSaxon_doublegaussian=TABhat_WoodsSaxon(vmb(1),vmb(2),RR,w,aa,A)&
         *overlappingF_doublegaussian(v(1),v(2),beta_common,r01_common,r02_common)
    RETURN
  END FUNCTION TnABhat_fxn_WoodsSaxon_doublegaussian

  FUNCTION TnABhat_HardSphere_doublegaussian(bx,by,RR,beta,r01,r02)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using double gaussian
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_HardSphere_doublegaussian
    REAL(KIND(1d0)),INTENT(IN)::bx,by,beta,r01,r02
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnAB_HS_DG/bx_common,by_common,RA_common,RB_common,beta_common,r01_common,r02_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    r01_common=r01
    r02_common=r02
    beta_common=beta
    amin(1)=-100d0*(r01+r02)
    amax(1)=100d0*(r01+r02)
    amin(2)=-100d0*(r01+r02)
    amax(2)=100d0*(r01+r02)
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_HardSphere_doublegaussian,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_HardSphere_doublegaussian,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_HardSphere_doublegaussian

  FUNCTION TnABhat_fxn_HardSphere_doublegaussian(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_HardSphere_doublegaussian
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::beta_common,r01_common,r02_common,bx_common,by_common
    COMMON/TnAB_HS_DG/bx_common,by_common,RA_common,RB_common,beta_common,r01_common,r02_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_HardSphere_doublegaussian is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    TnABhat_fxn_HardSphere_doublegaussian=TABhat_hardsphere(vmb(1),vmb(2),RR)&
         *overlappingF_doublegaussian(v(1),v(2),beta_common,r01_common,r02_common)
    RETURN
  END FUNCTION TnABhat_fxn_HardSphere_doublegaussian

! F: top hat
  FUNCTION TnABhat_WoodsSaxon_tophat(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using top hat
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_WoodsSaxon_tophat
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR,w,aa,A
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_TH/bx_common,by_common,RA_common,RB_common,&
         wA_common,wB_common,aaA_common,aaB_common,AA_common,AB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    wA_common=w(1)
    wB_common=w(2)
    aaA_common=aa(1)
    aaB_common=aa(2)
    AA_common=A(1)
    AB_common=A(2)
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnABhat_fxn_WoodsSaxon_tophat,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_WoodsSaxon_tophat,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_WoodsSaxon_tophat

  FUNCTION TnABhat_fxn_WoodsSaxon_tophat(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_WoodsSaxon_tophat
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR,w,aa,A
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_TH/bx_common,by_common,RA_common,RB_common,&
         wA_common,wB_common,aaA_common,aaB_common,AA_common,AB_common,r0_common
    LOGICAL::storegrid
    COMMON/TAB_WoodsSaxon_Grid/storegrid
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_WoodsSaxon_tophat is not a two dimensional function"
       STOP
    ENDIF
    storegrid=.TRUE.
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    w(1)=wA_common
    w(2)=wB_common
    aa(1)=aaA_common
    aa(2)=aaB_common
    A(1)=AA_common
    A(2)=AB_common
    TnABhat_fxn_WoodsSaxon_tophat=TABhat_WoodsSaxon(vmb(1),vmb(2),RR,w,aa,A)&
         *overlappingF_tophat(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_WoodsSaxon_tophat

  FUNCTION TnABhat_HardSphere_tophat(bx,by,RR,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using top hat
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_HardSphere_tophat
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_TH/bx_common,by_common,RA_common,RB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    r0_common=r0
    amin(1)=-2d0*r0
    amax(1)=2d0*r0
    amin(2)=-2d0*r0
    amax(2)=2d0*r0
    sub_num(1)=100
    sub_num(2)=100
    CALL ROMBERG_ND(TnABhat_fxn_HardSphere_tophat,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_HardSphere_tophat,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_HardSphere_tophat

  FUNCTION TnABhat_fxn_HardSphere_tophat(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_HardSphere_tophat
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_TH/bx_common,by_common,RA_common,RB_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_HardSphere_tophat is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    TnABhat_fxn_HardSphere_tophat=TABhat_hardsphere(vmb(1),vmb(2),RR)&
         *overlappingF_tophat(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_HardSphere_tophat

! F: dipole
  FUNCTION TnABhat_WoodsSaxon_dipole(bx,by,RR,w,aa,A,mg)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using dipole
    ! nucleon density in a nucleus is using WoodsSaxon
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_WoodsSaxon_dipole
    REAL(KIND(1d0)),INTENT(IN)::bx,by,mg
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR,w,aa,A
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnAB_WS_D/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,mg_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    wA_common=w(1)
    wB_common=w(2)
    aaA_common=aa(1)
    aaB_common=aa(2)
    AA_common=A(1)
    AB_common=A(2)
    mg_common=mg
    amin(1)=-200d0/mg
    amax(1)=200d0/mg
    amin(2)=-200d0/mg
    amax(2)=200d0/mg
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_WoodsSaxon_dipole,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_WoodsSaxon_dipole,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_WoodsSaxon_dipole

  FUNCTION TnABhat_fxn_WoodsSaxon_dipole(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_WoodsSaxon_dipole
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR,w,aa,A
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnAB_WS_D/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,mg_common
    LOGICAL::storegrid
    COMMON/TAB_WoodsSaxon_Grid/storegrid
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_WoodsSaxon_dipole is not a two dimensional function"
       STOP
    ENDIF
    storegrid=.TRUE.
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    w(1)=wA_common
    w(2)=wB_common
    aa(1)=aaA_common
    aa(2)=aaB_common
    A(1)=AA_common
    A(2)=AB_common
    TnABhat_fxn_WoodsSaxon_dipole=TABhat_WoodsSaxon(vmb(1),vmb(2),RR,w,aa,A)&
         *overlappingF_dipole(v(1),v(2),mg_common)
    RETURN
  END FUNCTION TnABhat_fxn_WoodsSaxon_dipole

  FUNCTION TnABhat_HardSphere_dipole(bx,by,RR,mg)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using dipole
    ! nucleon density in a nucleus is using HardSphere
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_HardSphere_dipole
    REAL(KIND(1d0)),INTENT(IN)::bx,by,mg
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnAB_HS_D/bx_common,by_common,RA_common,RB_common,mg_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    mg_common=mg
    amin(1)=-200d0/mg
    amax(1)=200d0/mg
    amin(2)=-200d0/mg
    amax(2)=200d0/mg
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_HardSphere_dipole,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_HardSphere_dipole,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_HardSphere_dipole

  FUNCTION TnABhat_fxn_HardSphere_dipole(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_HardSphere_dipole
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::mg_common,bx_common,by_common
    COMMON/TnAB_HS_D/bx_common,by_common,RA_common,RB_common,mg_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_HardSphere_dipole is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    TnABhat_fxn_HardSphere_dipole=TABhat_hardsphere(vmb(1),vmb(2),RR)&
         *overlappingF_dipole(v(1),v(2),mg_common)
    RETURN
  END FUNCTION TnABhat_fxn_HardSphere_dipole

! F: exponential
  FUNCTION TnABhat_WoodsSaxon_exponential(bx,by,RR,w,aa,A,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using exponential
    ! nucleon density in a nucleus is using WoodsSaxon
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_WoodsSaxon_exponential
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR,w,aa,A
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_E/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    wA_common=w(1)
    wB_common=w(2)
    aaA_common=aa(1)
    aaB_common=aa(2)
    AA_common=A(1)
    AB_common=A(2)
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_WoodsSaxon_exponential,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_WoodsSaxon_exponential,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_WoodsSaxon_exponential
  
  FUNCTION TnABhat_fxn_WoodsSaxon_exponential(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_WoodsSaxon_exponential
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR,w,aa,A
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::wA_common,wB_common
    REAL(KIND(1d0))::aaA_common,aaB_common
    REAL(KIND(1d0))::AA_common,AB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_WS_E/bx_common,by_common,RA_common,RB_common,wA_common,wB_common,&
         aaA_common,aaB_common,AA_common,AB_common,r0_common
    LOGICAL::storegrid
    COMMON/TAB_WoodsSaxon_Grid/storegrid
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_WoodsSaxon_exponential is not a two dimensional function"
       STOP
    ENDIF
    storegrid=.TRUE.
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    w(1)=wA_common
    w(2)=wB_common
    aa(1)=aaA_common
    aa(2)=aaB_common
    A(1)=AA_common
    A(2)=AB_common
    TnABhat_fxn_WoodsSaxon_exponential=TABhat_WoodsSaxon(vmb(1),vmb(2),RR,w,aa,A)&
         *overlappingF_exponential(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_WoodsSaxon_exponential

  FUNCTION TnABhat_HardSphere_exponential(bx,by,RR,r0)
    ! Int[F(v)*TABhat(v-b),dv]
    ! parton density in a nucleon is using exponential
    ! nucleon density in a nucleus is using HardSphere
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_HardSphere_exponential
    REAL(KIND(1d0)),INTENT(IN)::bx,by,r0
    REAL(KIND(1d0)),DIMENSION(2),INTENT(IN)::RR
    INTEGER,DIMENSION(2)::sub_num
    INTEGER::ind,eval_num
    REAL(KIND(1d0)),DIMENSION(2)::amin,amax
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_E/bx_common,by_common,RA_common,RB_common,r0_common
    bx_common=bx
    by_common=by
    RA_common=RR(1)
    RB_common=RR(2)
    r0_common=r0
    amin(1)=-200d0*r0
    amax(1)=200d0*r0
    amin(2)=-200d0*r0
    amax(2)=200d0*r0
    sub_num(1)=1000
    sub_num(2)=1000
    CALL ROMBERG_ND(TnABhat_fxn_HardSphere_exponential,amin,amax,2,sub_num,1,1d-5,&
         TnABhat_HardSphere_exponential,ind,eval_num)
    IF(ind.EQ.-1)THEN
       WRITE(*,*)"WARNING: the precision 1e-5 is not achieved"
    ENDIF
    RETURN
  END FUNCTION TnABhat_HardSphere_exponential

  FUNCTION TnABhat_fxn_HardSphere_exponential(dim_num,v)
    IMPLICIT NONE
    REAL(KIND(1d0))::TnABhat_fxn_HardSphere_exponential
    INTEGER,INTENT(IN)::dim_num
    REAL(KIND(1d0)),DIMENSION(dim_num),INTENT(IN)::v
    REAL(KIND(1d0)),DIMENSION(dim_num)::vmb
    REAL(KIND(1d0)),DIMENSION(2)::RR
    REAL(KIND(1d0))::RA_common,RB_common
    REAL(KIND(1d0))::r0_common,bx_common,by_common
    COMMON/TnAB_HS_E/bx_common,by_common,RA_common,RB_common,r0_common
    IF(dim_num.NE.2)THEN
       WRITE(*,*)"ERROR: TnABhat_fxn_HardSphere_exponential is not a two dimensional function"
       STOP
    ENDIF
    vmb(1)=v(1)-bx_common
    vmb(2)=v(2)-by_common
    RR(1)=RA_common
    RR(2)=RB_common
    TnABhat_fxn_HardSphere_exponential=TABhat_hardsphere(vmb(1),vmb(2),RR)&
         *overlappingF_exponential(v(1),v(2),r0_common)
    RETURN
  END FUNCTION TnABhat_fxn_HardSphere_exponential

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                           !
! TA11hat = Int[HA(b)*TAhat(b)*b,db]*2*Pi  (2*Pi is the azimuthal angle int)!
! where HA(b) is an arbitary function of b                                  !
! Eq.(20) in arXiv:2001.04256, one uses HA(b)=GA(b)**2                      !
! following the notation of Eq.(20) in the above reference                  !
!                                                                           !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  FUNCTION TA11hat_HardSphere(RA,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::TA11hat_HardSphere
    REAL(KIND(1d0)),INTENT(IN)::RA
    REAL(KIND(1d0)),EXTERNAL::HA ! HA is a function of |b| only
    REAL(KIND(1d0))::bmin,bmax
    INTEGER::ninteg
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0)),PARAMETER::TWOPI=6.28318530717958647692528676656d0
    COMMON/TA11hat_HS/RA_common
    RA_common=RA
    bmin=0d0
    bmax=RA
    ninteg=10000
    CALL simpson2(TA11hat_hardsphere_fxn,HA,bmin,bmax,TA11hat_HardSphere,ninteg)
    TA11hat_HardSphere=TA11hat_HardSphere*TWOPI
    RETURN
  END FUNCTION TA11hat_HardSphere

  FUNCTION TA11hat_hardsphere_fxn(b,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0)),EXTERNAL::HA
    COMMON/TA11hat_HS/RA_common
    REAL(KIND(1d0)),INTENT(IN)::b
    REAL(KIND(1d0))::TA11hat_hardsphere_fxn
    TA11hat_hardsphere_fxn=HA(b)*b*TAhat_hardsphere(b,0d0,RA_common)
    RETURN
  END FUNCTION TA11hat_hardsphere_fxn

  FUNCTION TA11hat_WoodsSaxon(RR,w,aa,A,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::TA11hat_WoodsSaxon
    REAL(KIND(1d0)),INTENT(IN)::RR,w,aa,A
    REAL(KIND(1d0)),EXTERNAL::HA ! HA is a function of |b| only
    REAL(KIND(1d0))::bmin,bmax
    INTEGER::ninteg
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0)),PARAMETER::TWOPI=6.28318530717958647692528676656d0
    COMMON/TA11hat_WS/RA_common,wA_common,aaA_common,AA_common
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    bmin=0d0
    bmax=RR*10d0
    ninteg=10000
    CALL simpson2(TA11hat_WoodsSaxon_fxn,HA,bmin,bmax,TA11hat_WoodsSaxon,ninteg)
    TA11hat_WoodsSaxon=TA11hat_WoodsSaxon*TWOPI
    RETURN
  END FUNCTION TA11hat_WoodsSaxon

  FUNCTION TA11hat_WoodsSaxon_fxn(b,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0)),EXTERNAL::HA
    COMMON/TA11hat_WS/RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0)),INTENT(IN)::b
    REAL(KIND(1d0))::TA11hat_WoodsSaxon_fxn
    TA11hat_WoodsSaxon_fxn=HA(b)*b*TAhat_WoodsSaxon(b,0d0,RA_common,wA_common,aaA_common,AA_common,1)
    RETURN
  END FUNCTION TA11hat_WoodsSaxon_fxn

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                           !
! TA11hat2=Int[HA(b)*TAhat(b)**2*b,db]*2*Pi(2*Pi is the azimuthal angle int)!
! where HA(b) is an arbitary function of b                                  !
! Eq.(20) in arXiv:2001.04256, one uses HA(b)=GA(b)**n, n=0,1,2             !
! with Fpp(v)=delta(v)                                                      !
! following the notation of Eq.(20) in the above reference                  !
!                                                                           !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  FUNCTION TA11hat2_HardSphere(RA,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::TA11hat2_HardSphere
    REAL(KIND(1d0)),INTENT(IN)::RA
    REAL(KIND(1d0)),EXTERNAL::HA ! HA is a function of |b| only
    REAL(KIND(1d0))::bmin,bmax
    INTEGER::ninteg
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0)),PARAMETER::TWOPI=6.28318530717958647692528676656d0
    COMMON/TA11hat2_HS/RA_common
    RA_common=RA
    bmin=0d0
    bmax=RA
    ninteg=10000
    CALL simpson2(TA11hat2_hardsphere_fxn,HA,bmin,bmax,TA11hat2_HardSphere,ninteg)
    TA11hat2_HardSphere=TA11hat2_HardSphere*TWOPI
    RETURN
  END FUNCTION TA11hat2_HardSphere

  FUNCTION TA11hat2_hardsphere_fxn(b,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::RA_common
    REAL(KIND(1d0)),EXTERNAL::HA
    COMMON/TA11hat2_HS/RA_common
    REAL(KIND(1d0)),INTENT(IN)::b
    REAL(KIND(1d0))::TA11hat2_hardsphere_fxn
    TA11hat2_hardsphere_fxn=HA(b)*b*TAhat_hardsphere(b,0d0,RA_common)**2
    RETURN
  END FUNCTION TA11hat2_hardsphere_fxn
  
  FUNCTION TA11hat2_WoodsSaxon(RR,w,aa,A,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::TA11hat2_WoodsSaxon
    REAL(KIND(1d0)),INTENT(IN)::RR,w,aa,A
    REAL(KIND(1d0)),EXTERNAL::HA ! HA is a function of |b| only
    REAL(KIND(1d0))::bmin,bmax
    INTEGER::ninteg
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0)),PARAMETER::TWOPI=6.28318530717958647692528676656d0
    COMMON/TA11hat2_WS/RA_common,wA_common,aaA_common,AA_common
    RA_common=RR
    wA_common=w
    aaA_common=aa
    AA_common=A
    bmin=0d0
    bmax=RR*10d0
    ninteg=10000
    CALL simpson2(TA11hat2_WoodsSaxon_fxn,HA,bmin,bmax,TA11hat2_WoodsSaxon,ninteg)
    TA11hat2_WoodsSaxon=TA11hat2_WoodsSaxon*TWOPI
    RETURN
  END FUNCTION TA11hat2_WoodsSaxon

  FUNCTION TA11hat2_WoodsSaxon_fxn(b,HA)
    IMPLICIT NONE
    REAL(KIND(1d0))::RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0)),EXTERNAL::HA
    COMMON/TA11hat2_WS/RA_common,wA_common,aaA_common,AA_common
    REAL(KIND(1d0)),INTENT(IN)::b
    REAL(KIND(1d0))::TA11hat2_WoodsSaxon_fxn
    TA11hat2_WoodsSaxon_fxn=HA(b)*b*TAhat_WoodsSaxon(b,0d0,RA_common,wA_common,aaA_common,AA_common,1)**2
    RETURN
  END FUNCTION TA11hat2_WoodsSaxon_fxn

END MODULE nucleus_transverse_profile
