!======================================================================
! Module exlcusively for communication between convolution and
! any routine which might supply convolutions
!======================================================================
MODULE convolution_communicator
  IMPLICIT NONE
  ! cc_REAL: the real contribution to splitting function (nonregularized one)
  ! cc_REALVIRT: the real+virtual contribution to splitting function without delta piece (the regular one) when x->1 (excluding delta and plus functions)
  ! cc_DELTA: the Dirac delta contribution to the splitting function
  ! cc_PLUS0: the coefficient  of (1/(1-x))_+ term
  ! cc_PLUS1: the coefficient of (log(1-x)/(1-x))_+ term
  ! cc_PLUS2: the coefficient of (log^2(1-x)/(1-x))_+ term
  ! similar for cc_PLUSi (i>=3)
  INTEGER,PARAMETER,PUBLIC::cc_REAL=1,&
       cc_REALVIRT=2,cc_DELTA=3,cc_PLUS0=4, cc_PLUS1=5,&
       cc_PLUS2=6, cc_PLUS3=7,cc_PLUS4=8,cc_PLUS5=9,cc_PLUS6=10
  INTEGER,PUBLIC,SAVE::cc_piece
  INTEGER,PUBLIC,SAVE::cc_order ! order of the convolution
  ! the following are for the time of two TMD beam functions and a TMD soft function
  ! for qt subtraction and qt resummation
  ! Note that we can always make the coefficient of the plus function independent of z
  ! the coefficient of the PLUS function is independent of z first
  ! PLUS is the sum of all plus functions, i.e. a_0/(1-z)_{x+}+a_1*(log(1-z)/(1-z))_{x+}+...
  ! REALVIRT is non delta term and non plus term 
  ! ss_DELTA_DELTA: Dirac delta * Dirac delta
  ! ss_DELTA_PLUS:  Dirac delta * Plus functions
  ! ss_PLUS_DELTA:  Plus functions * Dirac delta
  ! ss_DELTA_REALVIRT: Dirac delta * REALVIRT
  ! ss_REALVIRT_DELTA: REALVIRT * Dirac delta
  ! ss_PLUS_PLUS: Plus functions * Plus functions
  ! ss_PLUS_REALVIRT: Plus functions * REALVIRT
  ! ss_REALVIRT_PLUS: REALVIRT * Plus functions
  ! ss_REALVIRT_REALVIRT: REALVIRT * REALVIRT
  INTEGER,PARAMETER,PUBLIC::ss_DELTA_DELTA=1,&
       ss_DELTA_PLUS=2,ss_PLUS_DELTA=3,ss_DELTA_REALVIRT=4,&
       ss_REALVIRT_DELTA=5,ss_PLUS_PLUS=6,ss_PLUS_REALVIRT=7,&
       ss_REALVIRT_PLUS=8,ss_REALVIRT_REALVIRT=9
  INTEGER,PUBLIC,SAVE::ss_piece
END MODULE convolution_communicator
