MODULE qcd_tmd_beamsoftbeam_functions_expinas
  ! This is the first TMD beam function times the soft function times the second TMD beam function
  ! where the soft function is for the colourless final states only
  ! it has been expanded in as in order to obtain the ingredients for 
  ! qt subtraction
  USE convolution_communicator
  USE qcd_constants
  USE global_constants
  IMPLICIT NONE
  PRIVATE
  PUBLIC::TMD_BeamSoftBeam_IggSggIgg0,TMD_BeamSoftBeam_IggSggIgg1,TMD_BeamSoftBeam_IggSggIgg2
  PUBLIC::TMD_BeamSoftBeam_IgqSggIgg0,TMD_BeamSoftBeam_IgqSggIgg1,TMD_BeamSoftBeam_IgqSggIgg2
  PUBLIC::TMD_BeamSoftBeam_IggSggIgq0,TMD_BeamSoftBeam_IggSggIgq1,TMD_BeamSoftBeam_IggSggIgq2
  PUBLIC::TMD_BeamSoftBeam_IgqSggIgq0,TMD_BeamSoftBeam_IgqSggIgq1,TMD_BeamSoftBeam_IgqSggIgq2
  PUBLIC::TMD_BeamSoftBeam_IqqSqqIqq0,TMD_BeamSoftBeam_IqqSqqIqq1,TMD_BeamSoftBeam_IqqSqqIqq2
  PUBLIC::TMD_BeamSoftBeam_IqgSqqIqq0,TMD_BeamSoftBeam_IqgSqqIqq1,TMD_BeamSoftBeam_IqgSqqIqq2
  PUBLIC::TMD_BeamSoftBeam_IqqSqqIqg0,TMD_BeamSoftBeam_IqqSqqIqg1,TMD_BeamSoftBeam_IqqSqqIqg2
  PUBLIC::TMD_BeamSoftBeam_IqgSqqIqg0,TMD_BeamSoftBeam_IqgSqqIqg1,TMD_BeamSoftBeam_IqgSqqIqg2
  PUBLIC::TMD_BeamSoftBeam_IqqpSqqIqq0,TMD_BeamSoftBeam_IqqpSqqIqq1,TMD_BeamSoftBeam_IqqpSqqIqq2
  PUBLIC::TMD_BeamSoftBeam_IqqSqqIqqp0,TMD_BeamSoftBeam_IqqSqqIqqp1,TMD_BeamSoftBeam_IqqSqqIqqp2
  PUBLIC::TMD_BeamSoftBeam_IqqpSqqIqqp0,TMD_BeamSoftBeam_IqqpSqqIqqp1,TMD_BeamSoftBeam_IqqpSqqIqqp2
  PUBLIC::TMD_BeamSoftBeam_IqqbSqqIqq0,TMD_BeamSoftBeam_IqqbSqqIqq1,TMD_BeamSoftBeam_IqqbSqqIqq2
  PUBLIC::TMD_BeamSoftBeam_IqqSqqIqqb0,TMD_BeamSoftBeam_IqqSqqIqqb1,TMD_BeamSoftBeam_IqqSqqIqqb2
  PUBLIC::TMD_BeamSoftBeam_IqqbSqqIqqb0,TMD_BeamSoftBeam_IqqbSqqIqqb1,TMD_BeamSoftBeam_IqqbSqqIqqb2
  PUBLIC::TMD_BeamSoftBeam_IqqpSqqIqg0,TMD_BeamSoftBeam_IqqpSqqIqg1,TMD_BeamSoftBeam_IqqpSqqIqg2
  PUBLIC::TMD_BeamSoftBeam_IqgSqqIqqp0,TMD_BeamSoftBeam_IqgSqqIqqp1,TMD_BeamSoftBeam_IqgSqqIqqp2
  PUBLIC::TMD_BeamSoftBeam_IqqbSqqIqg0,TMD_BeamSoftBeam_IqqbSqqIqg1,TMD_BeamSoftBeam_IqqbSqqIqg2
  PUBLIC::TMD_BeamSoftBeam_IqgSqqIqqb0,TMD_BeamSoftBeam_IqgSqqIqqb1,TMD_BeamSoftBeam_IqgSqqIqqb2
  PUBLIC::TMD_BeamSoftBeam_IqqpSqqIqqb0,TMD_BeamSoftBeam_IqqpSqqIqqb1,TMD_BeamSoftBeam_IqqpSqqIqqb2
  PUBLIC::TMD_BeamSoftBeam_IqqbSqqIqqp0,TMD_BeamSoftBeam_IqqbSqqIqqp1,TMD_BeamSoftBeam_IqqbSqqIqqp2
CONTAINS
!===============================================================
! Beam function 1 times the colourless final
! state soft function times beam function 2, 
! and expanding it into as
! ISI*(za,zb,xa,xb,as)=Igg1*Sgg*Igg2
! ISI(za,zb,xa,xb,as)=ISI(za,zb,xa,xb)+as/2/Pi*ISI(za,zb,xa,xb)
!                   +(as/2/Pi)**2*ISI(za,zb,xa,xb)
! The integration range for za is [xa,1]
! The integration range for zb is [xb,1]
! It is rapidity regulator scale v independent !
! Lcut=Log[pTcut**2/mu**2], LS=2*Log[Q**2/mu**2]
!================================================================
  ! The expressions are exported with ExportExpressin.nb in
  ! /Users/huasheng/Physics/FLibatM/TMDBeamFunctions at the new Mac Pro (2017)
  ! I1gg*Sgg*I1gg+I2gg*Sgg*I2gg at (as/2/Pi)**0 
  FUNCTION TMD_BeamSoftBeam_IggSggIgg0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero

    IF(ss_piece.EQ.ss_Delta_Delta)THEN
       res=one
       RETURN
    ENDIF
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IggSggIgg0

  ! I1gg*Sgg*I1gg+I2gg*Sgg*I2gg at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IggSggIgg1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1
    REAL(KIND(1d0))::Lxa,Lxb
    res=zero

    IF(ss_piece.EQ.ss_Delta_Delta)THEN
       Lxa=DLOG(one-xa)
       Lxb=DLOG(one-xb)
       res=-(one*CA*(Lcut**2-one*Lcut*(LS+two*(Lxa+Lxb))+zeta2))
       RETURN
    ENDIF

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_REALVIRT)
       res=-((two*CA*Lcut*(one-zb+zb**2)**2)/((-one+zb)*zb))
    CASE(ss_REALVIRT_DELTA)
       res=-((two*CA*Lcut*(one-za+za**2)**2)/((-one+za)*za))
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_PLUS,ss_DELTA_REALVIRT)
       res1=-((two*CA*Lcut)/(-one+zb))
       IF(ss_piece.EQ.ss_DELTA_REALVIRT)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    CASE(ss_PLUS_DELTA,ss_REALVIRT_DELTA)
       res1=-((two*CA*Lcut)/(-one+za))
       IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IggSggIgg1
  
  ! I1gg*Sgg*I1gg+I2gg*Sgg*I2gg at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IggSggIgg2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1,res0
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1a,HC1b
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2a,HC2b
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3a,HC3b
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4a,HC4b
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1a,HI1a,HR1b,HI1b
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2a,HI2a,HR2b,HI2b
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3a,HI3a,HR3b,HI3b
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4a,HI4a,HR4b,HI4b
    REAL(KIND(1d0))::Lxa,Lxb,Lza,Lzb
    res=zero
    Lxa=DLOG(one-xa)
    Lxb=DLOG(one-xb)

    IF(ss_piece.EQ.ss_Delta_Delta)THEN
       res=(CA*(-(four*nfTF*(36d0*Lcut**3-84*LS-168*Lxa-168*Lxb-  &
            9*Lcut**2*(three*LS+6d0*Lxa+6d0*Lxb+10d0)+164d0-135*zeta2+  &
            18d0*Lcut*(5d0*LS+10d0*Lxa+10d0*Lxb+6d0*zeta2)+18d0*zeta3))+  &
            CA*(162d0*Lcut**4-2424*Lxa-2424*Lxb-36*Lcut**3*(-11+9d0*LS+  &
            18d0*Lxa+18d0*Lxb)+2428d0+9d0*Lcut**2*(18d0*LS**2+LS*(-33+  &
            72d0*Lxa+72d0*Lxb)+two*(-67-33*Lxa+36d0*Lxa**2-33*Lxb+  &
            72d0*Lxa*Lxb+36d0*Lxb**2-36*zeta2))-1809*zeta2+  &
            162d0*zeta2**2-18*Lcut*(LS*(-67+36d0*zeta2)+two*(-67*Lxa-  &
            67*Lxb-33*zeta2+36d0*Lxa*zeta2+36d0*Lxb*zeta2-72*zeta3))-  &
            324*Lxa*zeta3-324*Lxb*zeta3+198d0*zeta3-6*LS*(202d0+  &
            27d0*zeta3)+810d0*zeta4)))/3.24d2
       RETURN
    ENDIF

    IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
       CALL HPLOG(za,NW,HC1a,HC2a,HC3a,HC4a,HR1a,HR2a,HR3a,HR4a,&
            HI1a,HI2a,HI3a,HI4a,N1,N2)
    ENDIF
    IF(ss_piece.EQ.ss_DELTA_REALVIRT)THEN
       CALL HPLOG(zb,NW,HC1b,HC2b,HC3b,HC4b,HR1b,HR2b,HR3b,HR4b,&
            HI1b,HI2b,HI3b,HI4b,N1,N2)
    ENDIF

    Lza=DLOG(one-za)
    Lzb=DLOG(one-zb)

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_REALVIRT)
       res=(-(four*CA*nfTF*(one+zb)*(121d0+139d0*zb**4+  &
            18d0*Lcut**2*(one-one*zb+zb**2)**2-6*Lcut*(23d0+48d0*zb**2+  &
            23d0*zb**4+6d0*zb*(-7+Hr1b(0))-6*zb**3*(7d0+Hr1b(0)))+  &
            zb**2*(276d0-9*Hr1b(0)+9d0*Hr1b(1))+three*zb*(-77+  &
            13d0*Hr1b(0)+6d0*Hr2b(0,0))-three*zb**3*(83d0+10d0*Hr1b(0)+  &
            three*Hr1b(1)+6d0*Hr2b(0,0))))-36*CF*nfTF*(-one+zb**2)*(two-  &
            two*zb**3+Lcut**2*(-four+four*zb**3+zb**2*(three-6*Hr1b(0))-  &
            three*zb*(one+two*Hr1b(0)))-two*Lcut*(two+four*zb**3-  &
            three*zb**2*(-5+three*Hr1b(0)+two*Hr2b(0,0))-three*zb*(7d0+  &
            three*Hr1b(0)+two*Hr2b(0,0)))-three*zb**2*(-16+6d0*Hr1b(0)+  &
            Hr2b(0,0)+two*Hr3b(0,0,0))-three*zb*(16d0+6d0*Hr1b(0)+  &
            three*Hr2b(0,0)+two*Hr3b(0,0,0)))+CA**2*(216d0*Lcut**3*(one+  &
            zb)*(one-one*zb+zb**2)**2-18*Lcut**2*(one+zb)*(-55+  &
            24d0*Lzb*zb+102d0*zb-105*zb**2+102d0*zb**3-55*zb**4+  &
            12d0*LS*(one-one*zb+zb**2)**2+24d0*Lxa*(one-one*zb+zb**2)**2  &
            -12*Hr1b(0)-36*zb**2*Hr1b(0)+48d0*zb**3*Hr1b(0)-  &
            12*zb**4*Hr1b(0)-24*Hr1b(1)+72d0*zb*Hr1b(1)-72*zb**2*Hr1b(1)  &
            +48d0*zb**3*Hr1b(1)-24*zb**4*Hr1b(1))+6d0*Lcut*(zb**4*(-109-  &
            36*zeta2+18d0*Hr1b(1)**2+Hr1b(0)*(66d0+72d0*Hr1b(1))+  &
            72d0*Hr2b(-1,0)+144d0*Hr2b(0,0)-36*Hr2b(1,1))+zb**2*(-25+  &
            36d0*zeta2-18*Hr1b(1)**2-6*Hr1b(0)*(11d0+12d0*Hr1b(1))-  &
            72*Hr2b(-1,0)-144*Hr2b(0,0)+36d0*Hr2b(1,1))+zb**3*(-25+  &
            108d0*zeta2+Hr1b(0)*(114d0-72*Hr1b(1))-18*Hr1b(1)**2+  &
            72d0*Hr2b(-1,0)+72d0*Hr2b(0,0)+36d0*Hr2b(1,1))-one*zb*(109d0  &
            +108d0*zeta2-18*Hr1b(1)**2-6*Hr1b(0)*(25d0+12d0*Hr1b(1))+  &
            72d0*Hr2b(-1,0)+72d0*Hr2b(0,0)+36d0*Hr2b(1,1))+  &
            18d0*(two*zeta2-four*Hr1b(0)*Hr1b(1)-one*Hr1b(1)**2-  &
            four*Hr2b(-1,0)+two*Hr2b(1,1))+6d0*zb**5*(18d0*zeta2-  &
            four*Hr1b(0)*(11d0+three*Hr1b(1))-three*(Hr1b(1)**2-  &
            four*Hr2b(-1,0)+four*Hr2b(0,0)-two*Hr2b(1,1))))+zb**2*(688d0  &
            -432*zeta3-72*zeta2*(one+three*Hr1b(-1))+447d0*Hr1b(0)+  &
            18d0*Hr1b(1)+198d0*Hr2b(0,0)-72*Hr2b(1,0)-432*Hr3b(-1,-1,0)+  &
            216d0*Hr3b(-1,0,0)+432d0*Hr3b(0,-1,0)+432d0*Hr3b(0,0,0)-  &
            432*Hr3b(0,1,0)-216*Hr3b(0,1,1)+216d0*Hr3b(1,0,0)-  &
            216*Hr3b(1,0,1)-216*Hr3b(1,1,0))-one*zb**3*(-364+648d0*zeta3  &
            -72*zeta2*(-one+three*Hr1b(-1))+495d0*Hr1b(0)+  &
            342d0*Hr2b(0,0)+72d0*Hr2b(1,0)-432*Hr3b(-1,-1,0)+  &
            216d0*Hr3b(-1,0,0)+432d0*Hr3b(0,-1,0)+216d0*Hr3b(0,0,0)+  &
            216d0*Hr3b(0,1,1)-216*Hr3b(1,0,0)+216d0*Hr3b(1,0,1)+  &
            216d0*Hr3b(1,1,0))+zb*(-2896+648d0*zeta3-216*zeta2*(-four+  &
            Hr1b(-1))+2103d0*Hr1b(0)-450*Hr2b(0,0)+864d0*Hr2b(1,0)-  &
            432*Hr3b(-1,-1,0)+216d0*Hr3b(-1,0,0)+432d0*Hr3b(0,-1,0)+  &
            216d0*Hr3b(0,0,0)+216d0*Hr3b(0,1,1)-216*Hr3b(1,0,0)+  &
            216d0*Hr3b(1,0,1)+216d0*Hr3b(1,1,0))+zb**4*(-3040+  &
            432d0*zeta3+216d0*zeta2*(four+Hr1b(-1))-447*Hr1b(0)-  &
            18*Hr1b(1)-198*Hr2b(0,0)+864d0*Hr2b(1,0)+432d0*Hr3b(-1,-1,0)  &
            -216*Hr3b(-1,0,0)-432*Hr3b(0,-1,0)-432*Hr3b(0,0,0)+  &
            432d0*Hr3b(0,1,0)+216d0*Hr3b(0,1,1)-216*Hr3b(1,0,0)+  &
            216d0*Hr3b(1,0,1)+216d0*Hr3b(1,1,0))-8*(-395+54d0*zeta3+  &
            9d0*zeta2*(11d0+three*Hr1b(-1))+99d0*Hr2b(1,0)+  &
            54d0*Hr3b(-1,-1,0)-27*Hr3b(-1,0,0)-54*Hr3b(0,-1,0)+  &
            54d0*Hr3b(0,1,0)+27d0*Hr3b(0,1,1)-27*Hr3b(1,0,0)+  &
            27d0*Hr3b(1,0,1)+27d0*Hr3b(1,1,0))-four*zb**5*(-835+  &
            162d0*zeta3+zeta2*(198d0-54*Hr1b(-1))+402d0*Hr1b(0)-  &
            198*Hr2b(0,0)+198d0*Hr2b(1,0)-108*Hr3b(-1,-1,0)+  &
            54d0*Hr3b(-1,0,0)+108d0*Hr3b(0,-1,0)-54*Hr3b(0,0,0)+  &
            54d0*Hr3b(0,1,1)-54*Hr3b(1,0,0)+54d0*Hr3b(1,0,1)+  &
            54d0*Hr3b(1,1,0))))/(1.08d2*zb*(-one+zb**2))
    CASE(ss_REALVIRT_DELTA)
       res=(-(four*CA*nfTF*(one+za)*(121d0+139d0*za**4+  &
            18d0*Lcut**2*(one-one*za+za**2)**2-6*Lcut*(23d0+48d0*za**2+  &
            23d0*za**4+6d0*za*(-7+Hr1a(0))-6*za**3*(7d0+Hr1a(0)))+  &
            za**2*(276d0-9*Hr1a(0)+9d0*Hr1a(1))+three*za*(-77+  &
            13d0*Hr1a(0)+6d0*Hr2a(0,0))-three*za**3*(83d0+10d0*Hr1a(0)+  &
            three*Hr1a(1)+6d0*Hr2a(0,0))))-36*CF*nfTF*(-one+za**2)*(two-  &
            two*za**3+Lcut**2*(-four+four*za**3+za**2*(three-6*Hr1a(0))-  &
            three*za*(one+two*Hr1a(0)))-two*Lcut*(two+four*za**3-  &
            three*za**2*(-5+three*Hr1a(0)+two*Hr2a(0,0))-three*za*(7d0+  &
            three*Hr1a(0)+two*Hr2a(0,0)))-three*za**2*(-16+6d0*Hr1a(0)+  &
            Hr2a(0,0)+two*Hr3a(0,0,0))-three*za*(16d0+6d0*Hr1a(0)+  &
            three*Hr2a(0,0)+two*Hr3a(0,0,0)))+CA**2*(216d0*Lcut**3*(one+  &
            za)*(one-one*za+za**2)**2-18*Lcut**2*(one+za)*(-55+  &
            24d0*Lza*za+102d0*za-105*za**2+102d0*za**3-55*za**4+  &
            12d0*LS*(one-one*za+za**2)**2+24d0*Lxb*(one-one*za+za**2)**2  &
            -12*Hr1a(0)-36*za**2*Hr1a(0)+48d0*za**3*Hr1a(0)-  &
            12*za**4*Hr1a(0)-24*Hr1a(1)+72d0*za*Hr1a(1)-72*za**2*Hr1a(1)  &
            +48d0*za**3*Hr1a(1)-24*za**4*Hr1a(1))+6d0*Lcut*(za**4*(-109-  &
            36*zeta2+18d0*Hr1a(1)**2+Hr1a(0)*(66d0+72d0*Hr1a(1))+  &
            72d0*Hr2a(-1,0)+144d0*Hr2a(0,0)-36*Hr2a(1,1))+za**2*(-25+  &
            36d0*zeta2-18*Hr1a(1)**2-6*Hr1a(0)*(11d0+12d0*Hr1a(1))-  &
            72*Hr2a(-1,0)-144*Hr2a(0,0)+36d0*Hr2a(1,1))+za**3*(-25+  &
            108d0*zeta2+Hr1a(0)*(114d0-72*Hr1a(1))-18*Hr1a(1)**2+  &
            72d0*Hr2a(-1,0)+72d0*Hr2a(0,0)+36d0*Hr2a(1,1))-one*za*(109d0  &
            +108d0*zeta2-18*Hr1a(1)**2-6*Hr1a(0)*(25d0+12d0*Hr1a(1))+  &
            72d0*Hr2a(-1,0)+72d0*Hr2a(0,0)+36d0*Hr2a(1,1))+  &
            18d0*(two*zeta2-four*Hr1a(0)*Hr1a(1)-one*Hr1a(1)**2-  &
            four*Hr2a(-1,0)+two*Hr2a(1,1))+6d0*za**5*(18d0*zeta2-  &
            four*Hr1a(0)*(11d0+three*Hr1a(1))-three*(Hr1a(1)**2-  &
            four*Hr2a(-1,0)+four*Hr2a(0,0)-two*Hr2a(1,1))))+za**2*(688d0  &
            -432*zeta3-72*zeta2*(one+three*Hr1a(-1))+447d0*Hr1a(0)+  &
            18d0*Hr1a(1)+198d0*Hr2a(0,0)-72*Hr2a(1,0)-432*Hr3a(-1,-1,0)+  &
            216d0*Hr3a(-1,0,0)+432d0*Hr3a(0,-1,0)+432d0*Hr3a(0,0,0)-  &
            432*Hr3a(0,1,0)-216*Hr3a(0,1,1)+216d0*Hr3a(1,0,0)-  &
            216*Hr3a(1,0,1)-216*Hr3a(1,1,0))-one*za**3*(-364+648d0*zeta3  &
            -72*zeta2*(-one+three*Hr1a(-1))+495d0*Hr1a(0)+  &
            342d0*Hr2a(0,0)+72d0*Hr2a(1,0)-432*Hr3a(-1,-1,0)+  &
            216d0*Hr3a(-1,0,0)+432d0*Hr3a(0,-1,0)+216d0*Hr3a(0,0,0)+  &
            216d0*Hr3a(0,1,1)-216*Hr3a(1,0,0)+216d0*Hr3a(1,0,1)+  &
            216d0*Hr3a(1,1,0))+za*(-2896+648d0*zeta3-216*zeta2*(-four+  &
            Hr1a(-1))+2103d0*Hr1a(0)-450*Hr2a(0,0)+864d0*Hr2a(1,0)-  &
            432*Hr3a(-1,-1,0)+216d0*Hr3a(-1,0,0)+432d0*Hr3a(0,-1,0)+  &
            216d0*Hr3a(0,0,0)+216d0*Hr3a(0,1,1)-216*Hr3a(1,0,0)+  &
            216d0*Hr3a(1,0,1)+216d0*Hr3a(1,1,0))+za**4*(-3040+  &
            432d0*zeta3+216d0*zeta2*(four+Hr1a(-1))-447*Hr1a(0)-  &
            18*Hr1a(1)-198*Hr2a(0,0)+864d0*Hr2a(1,0)+432d0*Hr3a(-1,-1,0)  &
            -216*Hr3a(-1,0,0)-432*Hr3a(0,-1,0)-432*Hr3a(0,0,0)+  &
            432d0*Hr3a(0,1,0)+216d0*Hr3a(0,1,1)-216*Hr3a(1,0,0)+  &
            216d0*Hr3a(1,0,1)+216d0*Hr3a(1,1,0))-8*(-395+54d0*zeta3+  &
            9d0*zeta2*(11d0+three*Hr1a(-1))+99d0*Hr2a(1,0)+  &
            54d0*Hr3a(-1,-1,0)-27*Hr3a(-1,0,0)-54*Hr3a(0,-1,0)+  &
            54d0*Hr3a(0,1,0)+27d0*Hr3a(0,1,1)-27*Hr3a(1,0,0)+  &
            27d0*Hr3a(1,0,1)+27d0*Hr3a(1,1,0))-four*za**5*(-835+  &
            162d0*zeta3+zeta2*(198d0-54*Hr1a(-1))+402d0*Hr1a(0)-  &
            198*Hr2a(0,0)+198d0*Hr2a(1,0)-108*Hr3a(-1,-1,0)+  &
            54d0*Hr3a(-1,0,0)+108d0*Hr3a(0,-1,0)-54*Hr3a(0,0,0)+  &
            54d0*Hr3a(0,1,1)-54*Hr3a(1,0,0)+54d0*Hr3a(1,0,1)+  &
            54d0*Hr3a(1,1,0))))/(1.08d2*za*(-one+za**2))
    CASE(ss_REALVIRT_REALVIRT)
       res=(four*CA**2*((-one+za)**2*(-one+zb)**2+Lcut**2*(one-one*za+  &
            za**2)**2*(one-one*zb+zb**2)**2))/((-one+za)*za*(-one+  &
            zb)*zb)
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_PLUS,ss_DELTA_REALVIRT)
       res1=(CA*(-(four*(-30*Lcut+9d0*Lcut**2+28d0)*nfTF)+  &
            CA*(108d0*Lcut**3-9*Lcut**2*(-11+12d0*LS+24d0*Lxa+24d0*Lzb)+  &
            404d0+6d0*Lcut*(-67+36d0*zeta2)+54d0*zeta3)))/(5.4d1*(-one+zb))
       IF(ss_piece.EQ.ss_DELTA_REALVIRT)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    CASE(ss_PLUS_DELTA,ss_REALVIRT_DELTA)
       res1=(CA*(-(four*(-30*Lcut+9d0*Lcut**2+28d0)*nfTF)+  &
            CA*(108d0*Lcut**3-9*Lcut**2*(-11+12d0*LS+24d0*Lxb+24d0*Lza)+  &
            404d0+6d0*Lcut*(-67+36d0*zeta2)+54d0*zeta3)))/(5.4d1*(-one+za))
       IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    CASE(ss_PLUS_PLUS,ss_PLUS_REALVIRT,ss_REALVIRT_PLUS,ss_REALVIRT_REALVIRT)
       res1=(four*CA**2*Lcut**2)/((-one+za)*(-one+zb))
       IF(ss_piece.EQ.ss_PLUS_PLUS)THEN
          res=res1
       ELSEIF(ss_piece.EQ.ss_PLUS_REALVIRT.OR.ss_piece.EQ.ss_REALVIRT_PLUS)THEN
          res=res-res1
       ELSE
          res=res+res1
       ENDIF
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_PLUS_REALVIRT,ss_REALVIRT_REALVIRT)
       res1=(four*CA**2*Lcut**2*(one-one*zb+zb**2)**2)/((-one+za)*(-one+zb)*zb)
       IF(ss_piece.EQ.ss_PLUS_REALVIRT)THEN
          res=res+res1
       ELSE
          res=res-res1
       ENDIF
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_PLUS,ss_REALVIRT_REALVIRT)
       res1=(four*CA**2*Lcut**2*(one-one*za+za**2)**2)/((-one+za)*za*(-one+zb))
       IF(ss_piece.EQ.ss_REALVIRT_PLUS)THEN
          res=res+res1
       ELSE
          res=res-res1
       ENDIF
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IggSggIgg2

  ! I1gq*Sgg*I1gg+I2gq*Sgg*I2gg at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IgqSggIgg0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IgqSggIgg0

  ! I1gq*Sgg*I1gg+I2gq*Sgg*I2gg at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IgqSggIgg1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero

    IF(ss_piece.NE.ss_REALVIRT_DELTA)RETURN

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_DELTA)
       res=(CF*(za**2+Lcut*(two-two*za+za**2)))/za
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IgqSggIgg1

  ! I1gq*Sgg*I1gg+I2gq*Sgg*I2gg at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IgqSggIgg2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4a
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1a,HI1a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2a,HI2a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3a,HI3a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4a,HI4a
    REAL(KIND(1d0))::Lxa,Lxb,Lza,Lzb
    res=zero
    Lxb=DLOG(one-xb)

    IF(ss_piece.NE.ss_REALVIRT_DELTA.AND.ss_piece.NE.ss_REALVIRT_PLUS&
         .AND.ss_piece.NE.ss_REALVIRT_REALVIRT)RETURN

    IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
       CALL HPLOG(za,NW,HC1a,HC2a,HC3a,HC4a, HR1a,HR2a,HR3a,HR4a,&
            HI1a,HI2a,HI3a,HI4a,N1,N2)
    ENDIF

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_DELTA)
       res=-(CF*(-8*nfTF*(56d0+9d0*Lcut**2*(two-two*za+za**2)-  &
            30*Hr1a(1)+6d0*Lcut*(two-two*za+za**2)*(-5+three*Hr1a(1))+  &
            two*za*(-28+15d0*Hr1a(1)-9*Hr2a(1,1))+18d0*Hr2a(1,1)+  &
            za**2*(13d0-6*Hr1a(1)+9d0*Hr2a(1,1)))+  &
            two*CA*(54d0*Lcut**3*(two-two*za+za**2)-9*Lcut**2*(-53+  &
            46d0*za-14*za**2+four*za**3+6d0*LS*(two-two*za+za**2)+  &
            12d0*Lxb*(two-two*za+za**2)-12*Hr1a(0)-12*za*Hr1a(0)-  &
            12*za**2*Hr1a(0)-12*Hr1a(1)+12d0*za*Hr1a(1)-6*za**2*Hr1a(1))  &
            +8d0*za**3*(-38+9d0*zeta2+33d0*Hr1a(0)-9*Hr2a(0,0)+  &
            9d0*Hr2a(1,0))+three*Lcut*(8d0*za**3*(-11+6d0*Hr1a(0))-  &
            one*za**2*(18d0*LS+36d0*Lxb+8d0-18*zeta2-90*Hr1a(0)-  &
            102*Hr1a(1)+36d0*Hr1a(0)*Hr1a(1)+27d0*Hr1a(1)**2+  &
            36d0*Hr2a(-1,0)+36d0*Hr2a(0,0)-18*Hr2a(1,1))-two*za*(19d0+  &
            54d0*zeta2+66d0*Hr1a(1)-27*Hr1a(1)**2-36*Hr1a(0)*(three+  &
            Hr1a(1))+36d0*Hr2a(-1,0)+36d0*Hr2a(0,0)+18d0*Hr2a(1,1))+  &
            6d0*(-three+6d0*zeta2+(22d0-12*Hr1a(0))*Hr1a(1)-9*Hr1a(1)**2  &
            -12*Hr2a(-1,0)+6d0*Hr2a(1,1)))+za*(-1582+324d0*zeta3-  &
            108*zeta2*(-four+Hr1a(-1))+747d0*Hr1a(0)+456d0*Hr1a(1)-  &
            324*Hr2a(0,0)+432d0*Hr2a(1,0)-198*Hr2a(1,1)-  &
            216*Hr3a(-1,-1,0)+108d0*Hr3a(-1,0,0)+216d0*Hr3a(0,-1,0)+  &
            108d0*Hr3a(0,0,0)+108d0*Hr3a(0,1,1)-108*Hr3a(1,0,0)+  &
            108d0*Hr3a(1,0,1)+108d0*Hr3a(1,1,0)+108d0*Hr3a(1,1,1))-  &
            two*(-790+108d0*zeta3+18d0*zeta2*(11d0+three*Hr1a(-1))+  &
            228d0*Hr1a(1)+198d0*Hr2a(1,0)-99*Hr2a(1,1)+  &
            108d0*Hr3a(-1,-1,0)-54*Hr3a(-1,0,0)-108*Hr3a(0,-1,0)+  &
            108d0*Hr3a(0,1,0)+54d0*Hr3a(0,1,1)-54*Hr3a(1,0,0)+  &
            54d0*Hr3a(1,0,1)+54d0*Hr3a(1,1,0)+54d0*Hr3a(1,1,1))-  &
            one*za**2*(-536+108d0*zeta3+54d0*zeta2*(one+Hr1a(-1))+  &
            18d0*Hr1a(0)+129d0*Hr1a(1)+54d0*Hr2a(-1,0)+81d0*Hr2a(0,0)+  &
            54d0*Hr2a(0,1)+162d0*Hr2a(1,0)-45*Hr2a(1,1)+  &
            108d0*Hr3a(-1,-1,0)-54*Hr3a(-1,0,0)-108*Hr3a(0,-1,0)-  &
            54*Hr3a(0,0,0)+108d0*Hr3a(0,1,0)+54d0*Hr3a(0,1,1)-  &
            54*Hr3a(1,0,0)+54d0*Hr3a(1,0,1)+54d0*Hr3a(1,1,0)+  &
            54d0*Hr3a(1,1,1)))+27d0*CF*(za**2*(one-5*Hr1a(0)+  &
            10d0*Hr1a(1)+Lcut**2*(one+two*Hr1a(0)+four*Hr1a(1))+  &
            three*Hr2a(0,0)-two*Lcut*(-6+5d0*Hr1a(0)+6d0*Hr1a(1)-  &
            two*Hr1a(1)**2+two*Hr2a(0,0))-two*Hr2a(1,1)+two*Hr3a(0,0,0)+  &
            four*Hr3a(1,1,1))-one*za*(10d0-15*Hr1a(0)+32d0*Hr1a(1)+  &
            four*Lcut**2*(one+Hr1a(0)+two*Hr1a(1))+Lcut*(-6+8d0*Hr1a(0)-  &
            24*Hr1a(1)+8d0*Hr1a(1)**2-8*Hr2a(0,0))-four*Hr2a(0,0)-  &
            12*Hr2a(1,1)+four*Hr3a(0,0,0)+8d0*Hr3a(1,1,1))+  &
            four*(two*(four-three*Lcut+Lcut**2)*Hr1a(1)+  &
            two*Lcut*Hr1a(1)**2-three*Hr2a(1,1)+  &
            two*Hr3a(1,1,1)))))/(1.08d2*za)
    CASE(ss_REALVIRT_PLUS,ss_REALVIRT_REALVIRT)
       res1=-((two*CA*CF*Lcut*(za**2+Lcut*(two-two*za+za**2)))/(za*(-one+zb)))
       IF(ss_piece.EQ.ss_REALVIRT_PLUS)THEN
          res=res1
       ELSE
          res=-res1
       ENDIF
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_REALVIRT)
       res1=-((two*CA*CF*(-(two*(-one+za)*(-one+zb)**2)+Lcut*za**2*(one-  &
            one*zb+zb**2)**2+Lcut**2*(two-two*za+za**2)*(one-one*zb+  &
            zb**2)**2))/(za*(-one+zb)*zb))
       res=res+res1
    END SELECT
    RETURN
  END FUNCTION TMD_BEAMSOFTBEAM_IGQSGGIGG2

  ! I1gg*Sgg*I1gq+I2gg*Sgg*I2gq at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IggSggIgq0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IggSggIgq0

  ! I1gg*Sgg*I1gq+I2gg*Sgg*I2gq at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IggSggIgq1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    INTEGER::ss_piece_save
    LOGICAL::SWAPED
    ss_piece_save=ss_piece
    CALL SWAP_TWOBEAMS_ss_piece(SWAPED)
    res=TMD_BeamSoftBeam_IgqSggIgg1(zb,za,xb,xa,Lcut,LS)
    IF(SWAPED)ss_piece=ss_piece_save
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IggSggIgq1

  ! I1gg*Sgg*I1gq+I2gg*Sgg*I2gq at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IggSggIgq2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    INTEGER::ss_piece_save
    LOGICAL::SWAPED
    ss_piece_save=ss_piece
    CALL SWAP_TWOBEAMS_ss_piece(SWAPED)
    res=TMD_BeamSoftBeam_IgqSggIgg2(zb,za,xb,xa,Lcut,LS)
    IF(SWAPED)ss_piece=ss_piece_save
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IggSggIgq2

  ! I1gq*Sgg*I1gq+I2gq*Sgg*I2gq at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IgqSggIgq0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IgqSggIgq0

  ! I1gq*Sgg*I1gq+I2gq*Sgg*I2gq at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IgqSggIgq1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IgqSggIgq1

  ! I1gq*Sgg*I1gq+I2gq*Sgg*I2gq at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IgqSggIgq2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    IF(ss_piece.NE.ss_REALVIRT_REALVIRT)RETURN
    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_REALVIRT)
       res=(CF**2*(four-four*zb+za**2*zb**2+four*za*(-one+zb)+  &
            Lcut**2*(two-two*za+za**2)*(two-two*zb+zb**2)+  &
            two*Lcut*(zb**2-one*za*zb**2+za**2*(one-one*zb+  &
            zb**2))))/(za*zb)
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IgqSggIgq2

  ! The expressions are exported with ExportExpressin.nb in
  ! /Users/huasheng/Physics/FLibatM/TMDBeamFunctions at the new Mac Pro (2017)
  ! Iqq*Sqq*Iqq at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqq0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero

    IF(ss_piece.EQ.ss_Delta_Delta)THEN
       res=one
       RETURN
    ENDIF
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqq0

  ! Iqq*Sqq*Iqq at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqq1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1
    REAL(KIND(1d0))::Lxa,Lxb
    res=zero

    IF(ss_piece.EQ.ss_Delta_Delta)THEN
       Lxa=DLOG(one-xa)
       Lxb=DLOG(one-xb)
       res=-(one*CF*(Lcut**2-one*Lcut*(LS+two*(Lxa+Lxb))+zeta2))
       RETURN
    ENDIF

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_REALVIRT)
       res=-((one*CF*((-one+zb)**2+Lcut*(one+zb**2)))/(-one+zb))
    CASE(ss_REALVIRT_DELTA)
       res=-((one*CF*((-one+za)**2+Lcut*(one+za**2)))/(-one+za))
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_PLUS,ss_DELTA_REALVIRT)
       res1=-((two*CF*Lcut)/(-one+zb))
       IF(ss_piece.EQ.ss_DELTA_REALVIRT)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    CASE(ss_PLUS_DELTA,ss_REALVIRT_DELTA)
       res1=-((two*CF*Lcut)/(-one+za))
       IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqq1

  ! Iqq*Sqq*Iqq at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqq2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1,res0
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1a,HC1b
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2a,HC2b
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3a,HC3b
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4a,HC4b
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1a,HI1a,HR1b,HI1b
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2a,HI2a,HR2b,HI2b
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3a,HI3a,HR3b,HI3b
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4a,HI4a,HR4b,HI4b
    REAL(KIND(1d0))::Lxa,Lxb,Lza,Lzb
    res=zero
    Lxa=DLOG(one-xa)
    Lxb=DLOG(one-xb)

    IF(ss_piece.EQ.ss_Delta_Delta)THEN
       res=(CF*(two*(81d0*CF*(Lcut**4-two*Lcut**3*(LS+two*(Lxa+Lxb))+  &
            Lcut**2*(LS**2+four*Lxa**2+8d0*Lxa*Lxb+four*Lxb**2+  &
            four*LS*(Lxa+Lxb)-6*zeta2)+zeta2**2-two*Lcut*(LS*zeta2+  &
            two*Lxa*zeta2+two*Lxb*zeta2-8*zeta3)-8*(LS+two*(Lxa+  &
            Lxb))*zeta3)-two*nfTF*(36d0*Lcut**3-84*LS-168*Lxa-168*Lxb-  &
            9*Lcut**2*(three*LS+6d0*Lxa+6d0*Lxb+10d0)+164d0-135*zeta2+  &
            18d0*Lcut*(5d0*LS+10d0*Lxa+10d0*Lxb+6d0*zeta2)+18d0*zeta3))+  &
            CA*(396d0*Lcut**3-2424*Lxa-2424*Lxb+2428d0-  &
            9*Lcut**2*(33d0*LS+66d0*Lxa+66d0*Lxb+134d0-36*zeta2)-  &
            1809*zeta2-18*Lcut*(-67*LS-134*Lxa-134*Lxb-66*zeta2+  &
            18d0*LS*zeta2+36d0*Lxa*zeta2+36d0*Lxb*zeta2)+  &
            2268d0*Lxa*zeta3+2268d0*Lxb*zeta3+198d0*zeta3+6d0*LS*(-202D0+  &
            189d0*zeta3)+810d0*zeta4)))/3.24d2
       RETURN
    ENDIF

    IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
       CALL HPLOG(za,NW,HC1a,HC2a,HC3a,HC4a,HR1a,HR2a,HR3a,HR4a,&
            HI1a,HI2a,HI3a,HI4a,N1,N2)
    ENDIF
    IF(ss_piece.EQ.ss_DELTA_REALVIRT)THEN
       CALL HPLOG(zb,NW,HC1b,HC2b,HC3b,HC4b,HR1b,HR2b,HR3b,HR4b,&
            HI1b,HI2b,HI3b,HI4b,N1,N2)
    ENDIF

    Lza=DLOG(one-za)
    Lzb=DLOG(one-zb)

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_REALVIRT)
       res=(CF*(-(two*Tf*(-one+zb)*(-172+72d0*zeta2+9d0*Lcut**2*(-four+  &
            four*zb**3+zb**2*(three-6*Hr1b(0))-three*zb*(one+  &
            two*Hr1b(0)))-6*Lcut*(26d0+four*zb**3*(-11+6d0*Hr1b(0))+  &
            9d0*zb*(-four+Hr1b(0)-two*Hr2b(0,0))+9d0*zb**2*(6d0+  &
            three*Hr1b(0)-two*Hr2b(0,0)))+72d0*Hr2b(1,0)-8*zb**3*(-17+  &
            9d0*zeta2+24d0*Hr1b(0)-9*Hr2b(0,0)+9d0*Hr2b(1,0))+  &
            9d0*zb**2*(-31+12d0*zeta2+20d0*Hr1b(0)+three*Hr2b(0,0)+  &
            12d0*Hr2b(1,0)-6*Hr3b(0,0,0))-9*zb*(-35+12d0*zeta2+  &
            14d0*Hr1b(0)-three*Hr2b(0,0)+12d0*Hr2b(1,0)+  &
            6d0*Hr3b(0,0,0))))+zb*(CA*(8d0+99d0*Lcut**2*(one+zb**2)+  &
            162d0*zeta2-54*zeta3+174d0*Hr1b(0)+6d0*Lcut*(-154+174d0*zb+  &
            18d0*zeta2-51*Hr1b(0)+zb**2*(-154+18d0*zeta2-15*Hr1b(0)-  &
            18*Hr2b(0,0))-18*Hr2b(0,0))+99d0*Hr2b(0,0)+108d0*Hr2b(1,0)-  &
            18*zb*(-44+18d0*zeta2+12d0*Hr1b(0)-three*Hr1b(1)-6*Hr2b(0,0)  &
            +12d0*Hr2b(1,0))+54d0*Hr3b(0,0,0)+108d0*Hr3b(0,1,0)+  &
            108d0*Hr3b(1,0,1)+108d0*Hr3b(1,1,0)+zb**2*(8d0+162d0*zeta2-  &
            54*zeta3+498d0*Hr1b(0)-54*Hr1b(1)-9*Hr2b(0,0)+  &
            108d0*Hr2b(1,0)+54d0*Hr3b(0,0,0)+108d0*Hr3b(0,1,0)+  &
            108d0*Hr3b(1,0,1)+108d0*Hr3b(1,1,0)))+two*(-(two*nfTF*(19d0+  &
            18d0*zb+9d0*Lcut**2*(one+zb**2)+15d0*Hr1b(0)-6*Lcut*(8d0-  &
            6*zb+three*Hr1b(0)+zb**2*(8d0+three*Hr1b(0)))+9d0*Hr2b(0,0)+  &
            zb**2*(19d0+15d0*Hr1b(0)+9d0*Hr2b(0,0))))+27d0*CF*(11d0+  &
            two*Lcut**3*(one+zb**2)-two*zeta2-four*zeta3-5*Hr1b(0)+  &
            Lcut**2*(four-8*Lzb-8*zb+four*zb**2-two*LS*(one+zb**2)-  &
            four*Lxa*(one+zb**2)+Hr1b(0)+three*zb**2*Hr1b(0)-  &
            four*Hr1b(1)+four*zb**2*Hr1b(1))-three*Hr2b(0,0)-  &
            two*Hr2b(0,1)-6*Hr2b(1,0)+zb*(-22+four*zeta2+13d0*Hr1b(0)-  &
            one*Hr1b(1)-two*Hr2b(0,0)+four*Hr2b(0,1)+12d0*Hr2b(1,0))-  &
            two*Lcut*(-7D0+14d0*zb-7*zb**2+LS*(-one+zb)**2+two*Lxa*(-one+  &
            zb)**2-one*zeta2-one*zb**2*zeta2-four*Hr1b(0)+zb**2*Hr1b(0)-  &
            two*Hr1b(1)+four*zb*Hr1b(1)-two*zb**2*Hr1b(1)-one*Hr2b(0,0)+  &
            zb**2*Hr2b(0,0)+two*Hr2b(0,1)+two*zb**2*Hr2b(0,1)+  &
            two*Hr2b(1,0)+two*zb**2*Hr2b(1,0))-one*Hr3b(0,0,0)-  &
            four*Hr3b(0,1,0)-two*Hr3b(0,1,1)+two*Hr3b(1,0,0)-  &
            four*Hr3b(1,0,1)-four*Hr3b(1,1,0)+zb**2*(11d0-two*zeta2-  &
            four*zeta3-16*Hr1b(0)+Hr1b(1)+two*Hr2b(0,0)-two*Hr2b(0,1)-  &
            6*Hr2b(1,0)+Hr3b(0,0,0)-four*Hr3b(0,1,0)-two*Hr3b(0,1,1)+  &
            two*Hr3b(1,0,0)-four*Hr3b(1,0,1)-  &
            four*Hr3b(1,1,0)))))))/(1.08d2*(-one+zb)*zb)
    CASE(ss_REALVIRT_DELTA)
       res=(CF*(-(two*Tf*(-one+za)*(-172+72d0*zeta2+9d0*Lcut**2*(-four+  &
            four*za**3+za**2*(three-6*Hr1a(0))-three*za*(one+  &
            two*Hr1a(0)))-6*Lcut*(26d0+four*za**3*(-11+6d0*Hr1a(0))+  &
            9d0*za*(-four+Hr1a(0)-two*Hr2a(0,0))+9d0*za**2*(6d0+  &
            three*Hr1a(0)-two*Hr2a(0,0)))+72d0*Hr2a(1,0)-8*za**3*(-17+  &
            9d0*zeta2+24d0*Hr1a(0)-9*Hr2a(0,0)+9d0*Hr2a(1,0))+  &
            9d0*za**2*(-31+12d0*zeta2+20d0*Hr1a(0)+three*Hr2a(0,0)+  &
            12d0*Hr2a(1,0)-6*Hr3a(0,0,0))-9*za*(-35+12d0*zeta2+  &
            14d0*Hr1a(0)-three*Hr2a(0,0)+12d0*Hr2a(1,0)+  &
            6d0*Hr3a(0,0,0))))+za*(CA*(8d0+99d0*Lcut**2*(one+za**2)+  &
            162d0*zeta2-54*zeta3+174d0*Hr1a(0)+6d0*Lcut*(-154+174d0*za+  &
            18d0*zeta2-51*Hr1a(0)+za**2*(-154+18d0*zeta2-15*Hr1a(0)-  &
            18*Hr2a(0,0))-18*Hr2a(0,0))+99d0*Hr2a(0,0)+108d0*Hr2a(1,0)-  &
            18*za*(-44+18d0*zeta2+12d0*Hr1a(0)-three*Hr1a(1)-6*Hr2a(0,0)  &
            +12d0*Hr2a(1,0))+54d0*Hr3a(0,0,0)+108d0*Hr3a(0,1,0)+  &
            108d0*Hr3a(1,0,1)+108d0*Hr3a(1,1,0)+za**2*(8d0+162d0*zeta2-  &
            54*zeta3+498d0*Hr1a(0)-54*Hr1a(1)-9*Hr2a(0,0)+  &
            108d0*Hr2a(1,0)+54d0*Hr3a(0,0,0)+108d0*Hr3a(0,1,0)+  &
            108d0*Hr3a(1,0,1)+108d0*Hr3a(1,1,0)))+two*(-(two*nfTF*(19d0+  &
            18d0*za+9d0*Lcut**2*(one+za**2)+15d0*Hr1a(0)-6*Lcut*(8d0-  &
            6*za+three*Hr1a(0)+za**2*(8d0+three*Hr1a(0)))+9d0*Hr2a(0,0)+  &
            za**2*(19d0+15d0*Hr1a(0)+9d0*Hr2a(0,0))))+27d0*CF*(11d0+  &
            two*Lcut**3*(one+za**2)-two*zeta2-four*zeta3-5*Hr1a(0)+  &
            Lcut**2*(four-8*Lza-8*za+four*za**2-two*LS*(one+za**2)-  &
            four*Lxb*(one+za**2)+Hr1a(0)+three*za**2*Hr1a(0)-  &
            four*Hr1a(1)+four*za**2*Hr1a(1))-three*Hr2a(0,0)-  &
            two*Hr2a(0,1)-6*Hr2a(1,0)+za*(-22+four*zeta2+13d0*Hr1a(0)-  &
            one*Hr1a(1)-two*Hr2a(0,0)+four*Hr2a(0,1)+12d0*Hr2a(1,0))-  &
            two*Lcut*(-7+14d0*za-7*za**2+LS*(-one+za)**2+two*Lxb*(-one+  &
            za)**2-one*zeta2-one*za**2*zeta2-four*Hr1a(0)+za**2*Hr1a(0)-  &
            two*Hr1a(1)+four*za*Hr1a(1)-two*za**2*Hr1a(1)-one*Hr2a(0,0)+  &
            za**2*Hr2a(0,0)+two*Hr2a(0,1)+two*za**2*Hr2a(0,1)+  &
            two*Hr2a(1,0)+two*za**2*Hr2a(1,0))-one*Hr3a(0,0,0)-  &
            four*Hr3a(0,1,0)-two*Hr3a(0,1,1)+two*Hr3a(1,0,0)-  &
            four*Hr3a(1,0,1)-four*Hr3a(1,1,0)+za**2*(11d0-two*zeta2-  &
            four*zeta3-16*Hr1a(0)+Hr1a(1)+two*Hr2a(0,0)-two*Hr2a(0,1)-  &
            6*Hr2a(1,0)+Hr3a(0,0,0)-four*Hr3a(0,1,0)-two*Hr3a(0,1,1)+  &
            two*Hr3a(1,0,0)-four*Hr3a(1,0,1)-  &
            four*Hr3a(1,1,0)))))))/(1.08d2*(-one+za)*za)
    CASE(ss_REALVIRT_REALVIRT)
       res=(CF**2*((-one+za)**2+Lcut*(one+za**2))*((-one+zb)**2+  &
            Lcut*(one+zb**2)))/((-one+za)*(-one+zb))
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_DELTA_PLUS,ss_DELTA_REALVIRT)
       res1=(CF*(CA*(99d0*Lcut**2+404d0+6d0*Lcut*(-67+18d0*zeta2)-  &
            378*zeta3)+four*((-28+30d0*Lcut-9*Lcut**2)*nfTF+  &
            27d0*CF*(Lcut**3-one*Lcut**2*(LS+two*(Lxa+Lzb))+Lcut*zeta2+  &
            four*zeta3))))/(5.4d1*(-one+zb))
       IF(ss_piece.EQ.ss_DELTA_REALVIRT)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    CASE(ss_PLUS_DELTA,ss_REALVIRT_DELTA)
       res1=(CF*(CA*(99d0*Lcut**2+404d0+6d0*Lcut*(-67+18d0*zeta2)-  &
            378*zeta3)+four*((-28+30d0*Lcut-9*Lcut**2)*nfTF+  &
            27d0*CF*(Lcut**3-one*Lcut**2*(LS+two*(Lxb+Lza))+Lcut*zeta2+  &
            four*zeta3))))/(5.4d1*(-one+za))
       IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
          res=res-res1
       ELSE
          res=res1
       ENDIF
    CASE(ss_PLUS_PLUS,ss_PLUS_REALVIRT,ss_REALVIRT_PLUS,ss_REALVIRT_REALVIRT)
       res1=(four*CF**2*Lcut**2)/((-one+za)*(-one+zb))
       IF(ss_piece.EQ.ss_PLUS_PLUS)THEN
          res=res1
       ELSEIF(ss_piece.EQ.ss_PLUS_REALVIRT.OR.ss_piece.EQ.ss_REALVIRT_PLUS)THEN
          res=res-res1
       ELSE
          res=res+res1
       ENDIF
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_PLUS_REALVIRT,ss_REALVIRT_REALVIRT)
       res1=(two*CF**2*Lcut*((-one+zb)**2+Lcut*(one+zb**2)))/((-one+za)*(-one+zb))
       IF(ss_piece.EQ.ss_PLUS_REALVIRT)THEN
          res=res+res1
       ELSE
          res=res-res1
       ENDIF
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_PLUS,ss_REALVIRT_REALVIRT)
       res1=(two*CF**2*Lcut*((-one+za)**2+Lcut*(one+za**2)))/((-one+za)*(-one+zb))
       IF(ss_piece.EQ.ss_REALVIRT_PLUS)THEN
          res=res+res1
       ELSE
          res=res-res1
       ENDIF
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqq2

  ! Iqg*Sqq*Iqq at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqq0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqq0

  ! Iqg*Sqq*Iqq at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqq1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero

    IF(ss_piece.NE.ss_REALVIRT_DELTA)RETURN

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_DELTA)
       res=Tf*(-(two*(-one+za)*za)+Lcut*(one-two*za+two*za**2))
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqq1

  ! Iqg*Sqq*Iqq at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqq2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4a
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1a,HI1a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2a,HI2a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3a,HI3a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4a,HI4a
    REAL(KIND(1d0))::Lxa,Lxb,Lza,Lzb
    res=zero
    Lxb=DLOG(one-xb)

    IF(ss_piece.NE.ss_REALVIRT_DELTA.AND.ss_piece.NE.ss_REALVIRT_PLUS&
         .AND.ss_piece.NE.ss_REALVIRT_REALVIRT)RETURN

    IF(ss_piece.EQ.ss_REALVIRT_DELTA)THEN
       CALL HPLOG(za,NW,HC1a,HC2a,HC3a,HC4a, HR1a,HR2a,HR3a,HR4a,&
            HI1a,HI2a,HI3a,HI4a,N1,N2)
    ENDIF

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_DELTA)
       res=-(Tf*(two*CA*(-172+72d0*zeta2+9d0*Lcut**2*(-four-  &
            12*za**2*(two+two*Hr1a(0)+Hr1a(1))+za**3*(31d0+12d0*Hr1a(1))  &
            +za*(-three-6*Hr1a(0)+6d0*Hr1a(1)))+72d0*Hr2a(1,0)-  &
            6*Lcut*(26d0+four*za**3*(-29+33d0*Hr1a(0)-9*Hr2a(-1,0)-  &
            9*Hr2a(1,1))-9*za**2*(-15+four*zeta2+four*Hr2a(-1,0)+  &
            four*Hr2a(0,0)-four*Hr2a(1,1))+9d0*za*(Hr1a(0)-two*(two+  &
            Hr2a(-1,0)+Hr2a(0,0)+Hr2a(1,1))))+9d0*za**2*(-43-12*zeta3+  &
            12d0*zeta2*(three+Hr1a(-1))+20d0*Hr1a(0)+9d0*Hr1a(1)-  &
            12*Hr2a(-1,0)-12*Hr2a(0,0)+48d0*Hr2a(1,0)+12d0*Hr2a(1,1)+  &
            24d0*Hr3a(-1,-1,0)-12*Hr3a(-1,0,0)-24*Hr3a(0,-1,0)-  &
            12*Hr3a(0,0,0)+24d0*Hr3a(0,1,0)+12d0*Hr3a(1,0,1)+  &
            12d0*Hr3a(1,1,0)-12*Hr3a(1,1,1))+two*za**3*(149d0+  &
            18d0*zeta2*(-11+three*Hr1a(-1))-204*Hr1a(0)-54*Hr1a(1)-  &
            54*Hr2a(-1,0)+198d0*Hr2a(0,0)-198*Hr2a(1,0)-54*Hr2a(1,1)+  &
            108d0*Hr3a(-1,-1,0)-54*Hr3a(-1,0,0)-108*Hr3a(0,-1,0)-  &
            54*Hr3a(1,0,1)-54*Hr3a(1,1,0)+54d0*Hr3a(1,1,1))+9d0*za*(35d0  &
            +6d0*zeta2*(-two+Hr1a(-1))-14*Hr1a(0)+three*Hr2a(0,0)-  &
            12*Hr2a(1,0)+12d0*Hr3a(-1,-1,0)-6*Hr3a(-1,0,0)-  &
            12*Hr3a(0,-1,0)-6*Hr3a(0,0,0)-6*Hr3a(1,0,1)-6*Hr3a(1,1,0)+  &
            6d0*Hr3a(1,1,1)))+27d0*CF*za*(13d0+Lcut**3*(four-8*za+  &
            8d0*za**2)-12*zeta3-8*Hr1a(0)+Lcut**2*(7d0-8*za+four*za**2+  &
            LS*(-four+8d0*za-8*za**2)-8*Lxb*(one-two*za+two*za**2)+  &
            two*Hr1a(0)-four*za*Hr1a(0)+8d0*za**2*Hr1a(0)+four*Hr1a(1)-  &
            8*za*Hr1a(1)+8d0*za**2*Hr1a(1))-one*Hr2a(0,0)+two*Lcut*(-10+  &
            6d0*zeta2-one*Hr1a(0)-two*Hr2a(0,0)-four*Hr2a(0,1)-  &
            four*Hr2a(1,0)-four*Hr2a(1,1)+za*(-(four*LS)-8*Lxb+33d0-  &
            12*zeta2+8d0*Hr1a(0)+8d0*Hr1a(1)+four*Hr2a(0,0)+  &
            8d0*Hr2a(0,1)+8d0*Hr2a(1,0)+8d0*Hr2a(1,1))+four*za**2*(-7+LS  &
            +two*Lxb+three*zeta2-two*Hr1a(0)-two*Hr1a(1)-two*Hr2a(0,0)-  &
            two*Hr2a(0,1)-two*Hr2a(1,0)-two*Hr2a(1,1)))+two*Hr3a(0,0,0)-  &
            four*Hr3a(0,1,1)+four*Hr3a(1,0,0)-four*Hr3a(1,1,1)-  &
            8*za**2*(-9+two*zeta2+three*zeta3-one*Hr1a(0)-one*Hr1a(1)-  &
            one*Hr2a(0,0)-one*Hr2a(0,1)-one*Hr2a(1,0)-one*Hr2a(1,1)-  &
            one*Hr3a(0,0,0)+Hr3a(0,1,1)-one*Hr3a(1,0,0)+Hr3a(1,1,1))+  &
            za*(-75+16d0*zeta2+24d0*zeta3-15*Hr1a(0)-6*Hr1a(1)-  &
            12*Hr2a(0,0)-8*Hr2a(0,1)-8*Hr2a(1,0)-8*Hr2a(1,1)-  &
            four*Hr3a(0,0,0)+8d0*Hr3a(0,1,1)-8*Hr3a(1,0,0)+  &
            8d0*Hr3a(1,1,1)))))/(1.08d2*za)
    CASE(ss_REALVIRT_PLUS,ss_REALVIRT_REALVIRT)
       res1=-((two*CF*Lcut*Tf*(-(two*(-one+za)*za)+Lcut*(one-two*za+two*za**2)))/(-one+zb))
       IF(ss_piece.EQ.ss_REALVIRT_PLUS)THEN
          res=res1
       ELSE
          res=-res1
       ENDIF
    END SELECT

    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_REALVIRT)
       res1=-((one*CF*Tf*(-(two*(-one+za)*za)+Lcut*(one-two*za+  &
            two*za**2))*((-one+zb)**2+Lcut*(one+zb**2)))/(-one+zb))
       res=res+res1
    END SELECT
    RETURN
  END FUNCTION TMD_BEAMSOFTBEAM_IQGSQQIQQ2

  ! Iqq*Sqq*Iqg at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqg0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqg0

  ! Iqq*Sqq*Iqg at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqg1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    INTEGER::ss_piece_save
    LOGICAL::SWAPED
    ss_piece_save=ss_piece
    CALL SWAP_TWOBEAMS_ss_piece(SWAPED)
    res=TMD_BeamSoftBeam_IqgSqqIqq1(zb,za,xb,xa,Lcut,LS)
    IF(SWAPED)ss_piece=ss_piece_save
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqg1

  ! Iqq*Sqq*Iqg at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqg2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    INTEGER::ss_piece_save
    LOGICAL::SWAPED
    ss_piece_save=ss_piece
    CALL SWAP_TWOBEAMS_ss_piece(SWAPED)
    res=TMD_BeamSoftBeam_IqgSqqIqq2(zb,za,xb,xa,Lcut,LS)
    IF(SWAPED)ss_piece=ss_piece_save
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqg2

  ! Iqg*Sqq*Iqg at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqg0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqg0

  ! Iqg*Sqq*Iqg at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqg1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqg1

  ! Iqg*Sqq*Iqg at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqg2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    IF(ss_piece.NE.ss_REALVIRT_REALVIRT)RETURN
    SELECT CASE(ss_piece)
    CASE(ss_REALVIRT_REALVIRT)
       res=Tf**2*(-(two*(-one+za)*za)+Lcut*(one-two*za+  &
            two*za**2))*(-(two*(-one+zb)*zb)+Lcut*(one-two*zb+two*zb**2))
    END SELECT
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqg2

  ! Iqqp*Sqq*Iqq at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqq0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqq0

  ! Iqqp*Sqq*Iqq at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqq1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqq1

  ! Iqqp*Sqq*Iqq at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqq2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4a
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1a,HI1a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2a,HI2a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3a,HI3a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4a,HI4a
    REAL(KIND(1d0))::Lxa,Lxb,Lza,Lzb
    res=zero

    IF(ss_piece.NE.ss_REALVIRT_DELTA)RETURN

    CALL HPLOG(za,NW,HC1a,HC2a,HC3a,HC4a, HR1a,HR2a,HR3a,HR4a,&
         HI1a,HI2a,HI3a,HI4a,N1,N2)
    res=(CF*Tf*(172d0-72*zeta2-9*Lcut**2*(-four+four*za**3+  &
         za**2*(three-6*Hr1a(0))-three*za*(one+two*Hr1a(0)))+  &
         6d0*Lcut*(26d0+four*za**3*(-11+6d0*Hr1a(0))+9d0*za*(-four+  &
         Hr1a(0)-two*Hr2a(0,0))+9d0*za**2*(6d0+three*Hr1a(0)-  &
         two*Hr2a(0,0)))-72*Hr2a(1,0)+8d0*za**3*(-17+9d0*zeta2+  &
         24d0*Hr1a(0)-9*Hr2a(0,0)+9d0*Hr2a(1,0))-9*za**2*(-31+  &
         12d0*zeta2+20d0*Hr1a(0)+three*Hr2a(0,0)+12d0*Hr2a(1,0)-  &
         6*Hr3a(0,0,0))+9d0*za*(-35+12d0*zeta2+14d0*Hr1a(0)-  &
         three*Hr2a(0,0)+12d0*Hr2a(1,0)+6d0*Hr3a(0,0,0))))/(5.4d1*za)
    RETURN
  END FUNCTION TMD_BEAMSOFTBEAM_IQQPSQQIQQ2

  ! Iqq*Sqq*Iqqp at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqqp0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqqp0

  ! Iqq*Sqq*Iqqp at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqqp1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqqp1

  ! Iqq*Sqq*Iqqp at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqqp2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    INTEGER::ss_piece_save
    LOGICAL::SWAPED
    ss_piece_save=ss_piece
    CALL SWAP_TWOBEAMS_ss_piece(SWAPED)
    res=TMD_BeamSoftBeam_IqqpSqqIqq2(zb,za,xb,xa,Lcut,LS)
    IF(SWAPED)ss_piece=ss_piece_save
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqqp2

  ! Iqqp*Sqq*Iqqp at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqp0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqp0

  ! Iqqp*Sqq*Iqqp at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqp1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqp1

  ! Iqqp*Sqq*Iqqp at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqp2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqp2

  ! Iqqb*Sqq*Iqq at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqq0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqq0

  ! Iqqb*Sqq*Iqq at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqq1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqq1

  ! Iqqb*Sqq*Iqq at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqq2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res,res1
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3a
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4a
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1a,HI1a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2a,HI2a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3a,HI3a
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4a,HI4a
    REAL(KIND(1d0))::Lxa,Lxb,Lza,Lzb
    res=zero

    IF(ss_piece.NE.ss_REALVIRT_DELTA)RETURN

    CALL HPLOG(za,NW,HC1a,HC2a,HC3a,HC4a, HR1a,HR2a,HR3a,HR4a,&
         HI1a,HI2a,HI3a,HI4a,N1,N2)
    res=-(CF*(two*Tf*(one+za)*(-172+72d0*zeta2+9d0*Lcut**2*(-four+  &
         four*za**3+za**2*(three-6*Hr1a(0))-three*za*(one+  &
         two*Hr1a(0)))-6*Lcut*(26d0+four*za**3*(-11+6d0*Hr1a(0))+  &
         9d0*za*(-four+Hr1a(0)-two*Hr2a(0,0))+9d0*za**2*(6d0+  &
         three*Hr1a(0)-two*Hr2a(0,0)))+72d0*Hr2a(1,0)-8*za**3*(-17+  &
         9d0*zeta2+24d0*Hr1a(0)-9*Hr2a(0,0)+9d0*Hr2a(1,0))+  &
         9d0*za**2*(-31+12d0*zeta2+20d0*Hr1a(0)+three*Hr2a(0,0)+  &
         12d0*Hr2a(1,0)-6*Hr3a(0,0,0))-9*za*(-35+12d0*zeta2+  &
         14d0*Hr1a(0)-three*Hr2a(0,0)+12d0*Hr2a(1,0)+  &
         6d0*Hr3a(0,0,0)))-27*(CA-two*CF)*za*(-15+6d0*zeta2-two*zeta3  &
         +four*zeta2*Hr1a(-1)-three*Hr1a(0)+four*Hr2a(-1,0)+  &
         two*za*(two*zeta2-7*Hr1a(0)+four*Hr2a(-1,0))+four*Lcut*(-two  &
         +zeta2-one*Hr1a(0)-two*za*Hr1a(0)+two*Hr2a(-1,0)-  &
         one*Hr2a(0,0)+za**2*(two+zeta2-one*Hr1a(0)+two*Hr2a(-1,0)-  &
         one*Hr2a(0,0)))+four*Hr2a(1,0)+8d0*Hr3a(-1,-1,0)-  &
         four*Hr3a(-1,0,0)-8*Hr3a(0,-1,0)+two*Hr3a(0,0,0)+  &
         four*Hr3a(0,1,0)+za**2*(15d0-two*zeta2-two*zeta3+  &
         four*zeta2*Hr1a(-1)-11*Hr1a(0)+four*Hr2a(-1,0)-  &
         four*Hr2a(1,0)+8d0*Hr3a(-1,-1,0)-four*Hr3a(-1,0,0)-  &
         8*Hr3a(0,-1,0)+two*Hr3a(0,0,0)+  &
         four*Hr3a(0,1,0)))))/(1.08d2*za*(one+za))
    RETURN
  END FUNCTION TMD_BEAMSOFTBEAM_IQQBSQQIQQ2

  ! Iqq*Sqq*Iqqb at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqqb0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqqb0

  ! Iqq*Sqq*Iqqb at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqqb1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqqb1

  ! Iqq*Sqq*Iqqb at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqSqqIqqb2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    INTEGER::ss_piece_save
    LOGICAL::SWAPED
    ss_piece_save=ss_piece
    CALL SWAP_TWOBEAMS_ss_piece(SWAPED)
    res=TMD_BeamSoftBeam_IqqbSqqIqq2(zb,za,xb,xa,Lcut,LS)
    IF(SWAPED)ss_piece=ss_piece_save
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqSqqIqqb2

  ! Iqqb*Sqq*Iqqb at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqb0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqb0

  ! Iqqb*Sqq*Iqqb at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqb1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqb1

  ! Iqqb*Sqq*Iqqb at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqb2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqb2

  ! Iqqp*Sqq*Iqg at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqg0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqg0

  ! Iqqp*Sqq*Iqg at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqg1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqg1

  ! Iqqp*Sqq*Iqg at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqg2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqg2

  ! Iqg*Sqq*Iqqp at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqqp0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqqp0

  ! Iqg*Sqq*Iqqp at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqqp1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqqp1

  ! Iqg*Sqq*Iqqp at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqqp2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqqp2

  ! Iqqb*Sqq*Iqg at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqg0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqg0

  ! Iqqb*Sqq*Iqg at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqg1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqg1

  ! Iqqb*Sqq*Iqg at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqg2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqg2

  ! Iqg*Sqq*Iqqb at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqqb0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqqb0

  ! Iqg*Sqq*Iqqb at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqqb1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqqb1

  ! Iqg*Sqq*Iqqb at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqgSqqIqqb2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqgSqqIqqb2

  ! Iqqp*Sqq*Iqqb at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqb0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqb0

  ! Iqqp*Sqq*Iqqb at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqb1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqb1

  ! Iqqp*Sqq*Iqqb at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqb2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqpSqqIqqb2

  ! Iqqb*Sqq*Iqqp at (as/2/Pi)**0
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqp0(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqp0

  ! Iqqb*Sqq*Iqqp at (as/2/Pi)**1
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqp1(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqp1

  ! Iqqb*Sqq*Iqqp at (as/2/Pi)**2
  FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqp2(za,zb,xa,xb,Lcut,LS) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::za,zb,xa,xb
    REAL(KIND(1d0)),INTENT(IN)::Lcut,LS
    REAL(KIND(1d0))::res
    res=zero
    RETURN
  END FUNCTION TMD_BeamSoftBeam_IqqbSqqIqqp2
  
  SUBROUTINE SWAP_TWOBEAMS_ss_piece(SWAPED)
    IMPLICIT NONE
    LOGICAL,INTENT(OUT)::SWAPED
    SWAPED=.TRUE.
    SELECT CASE(ss_piece)
    CASE(ss_DELTA_PLUS)
       ss_piece=ss_PLUS_DELTA
    CASE(ss_PLUS_DELTA)
       ss_piece=ss_DELTA_PLUS
    CASE(ss_DELTA_REALVIRT)
       ss_piece=ss_REALVIRT_DELTA
    CASE(ss_REALVIRT_DELTA)
       ss_piece=ss_DELTA_REALVIRT
    CASE(ss_PLUS_REALVIRT)
       ss_piece=ss_REALVIRT_PLUS
    CASE(ss_REALVIRT_PLUS)
       ss_piece=ss_PLUS_REALVIRT
    CASE DEFAULT
       SWAPED=.FALSE.
    END SELECT
    RETURN
  END SUBROUTINE SWAP_TWOBEAMS_ss_piece
    
END MODULE qcd_tmd_beamsoftbeam_functions_expinas
