PROGRAM test
  USE SM_constants
  USE SM_setup
  USE SM_couplings
  IMPLICIT NONE
  REAL(KIND(1d0)),DIMENSION(7)::ALPHA
  TYPE(aSM_container)::aSM_box
  INTEGER::i,npoints
  REAL(KIND(1d0)),PARAMETER::TWOPI=6.283185307179586476925286766559005768394d0
  REAL(KIND(1d0))::tBefore,tAfter,Q
  REAL(KIND(1d0))::MT,MB,MTAU,MH,MW,MZ,CW2,SW2
  REAL(KIND(1d0))::alphaEM,alphasMZ,GF,vev
  REAL(KIND(1d0))::dstep,log10mu,log10mu_min,log10mu_max
  REAL(KIND(1d0)),DIMENSION(7)::alphaSM
  CALL CPU_time(tBefore)
  ! input parameters
  alphaEM=1d0/128.93d0
  alphasMZ=0.1184d0
  MT=173.1d0
  MB=2.8d0
  MTAU=1.78d0
  MH=125.1d0
  MW=80.379d0
  MZ=91.1876d0
  GF=1.1663787d-5                 ! Fermion constant
  CW2=MW**2/MZ**2
  SW2=1d0-CW2
  vev=1d0/DSQRT(DSQRT(2d0)*GF)
  !alphaEM=DSQRT(2d0)*GF*MW**2*SW2*2d0/TWOPI
  !PRINT *, 1d0/alphaEM
  ! end of input parameters
  alphaSM(1)=5d0/3d0*alphaEM/CW2  ! all order relation
  alphaSM(2)=alphaEM/SW2          ! all order relation
  alphaSM(3)=alphasMZ            
  alphaSM(4)=MT**2/vev**2/TWOPI   ! leading order relation, one-loop correction can be found in hep-ph/9606211
  alphaSM(5)=MB**2/vev**2/TWOPI   ! leading order relation, also see 1612.00660
  alphaSM(6)=MTAU**2/vev**2/TWOPI ! leading order relation
  alphaSM(7)=MH**2/(2d0*vev**2)/(2d0*TWOPI)*1.1d0
  PRINT *, alphaSM(1:7)
  CALL aSM_run_init(aSM_box,alphaSM,MZ,2)
  CALL CPU_time(tAfter)
  PRINT*, "Initial time:",tAfter-tBefore
  CALL CPU_time(tBefore)
  OPEN(UNIT=2133,FILE="alphaSM.dat")
  npoints=200
  log10mu_min=1d0
  log10mu_max=20d0
  dstep=(log10mu_max-log10mu_min)/DBLE(npoints)
  DO i=0, npoints
     log10mu=log10mu_min+i*dstep
     Q=10d0**log10mu
     ALPHA(1:7)=aSM_values(aSM_box, Q)
     WRITE(2133,*)log10mu, ALPHA(1:7)
  ENDDO
  CLOSE(UNIT=2133)
  CALL CPU_time(tAfter)
  PRINT*, "aSM evalute time:",tAfter-tBefore
  PRINT*, "evalue per point time:",(tAfter-tBefore)/DBLE(npoints+1)
  RETURN
END PROGRAM test


