MODULE pQCD_resum
  IMPLICIT NONE
CONTAINS
  ! threshold resummation for 2 ->  2 process
  FUNCTION Thadronic_xs_2to2(NM, S, T)
    IMPLICIT NONE
    COMPLEX(KIND(1d0))::Thadronic_xs_2to2
    COMPLEX(KIND(1d0)),INTENT(IN)::NM
    REAL(KIND(1d0)),INTENT(IN)::S,T
    ! symmetry factor
    REAL(KIND(1d0))::dij=1d0
    ! getting N-space PDF
    COMPLEX(KIND(1d0))::gN
    COMPLEX(KIND(1d0)),DIMENSION(0:1,0:5)::qN
    COMPLEX(KIND(1d0)),PARAMETER::M_EULER=DCMPLX(0.57721566490153286061)
    COMPLEX(KIND(1d0))::lnNb
    REAL(KIND(1d0))::aa
    COMPLEX(KIND(1d0))::RES=DCMPLX(0d0,0d0) ! resummed cross section
    COMPLEX(KIND(1d0))::EXP=DCMPLX(0d0,0d0) ! double counting cross section
    INTEGER::i0,i1
    REAL(KIND(1d0))::brn,vir
    REAL(KIND(1d0)),PARAMETER::M_PI=3.1415926535897932385d0
    CALL pdfN(gN,qN,NM)
    lnNb=LOG(NM)+M_EULER
    aa=aS(murs,set)*4d0/3d0
    DO i0=0,5
       DO i1=0,5
          brn=Born(S,T,i0,i1)
          IF(brn.EQ.0d0)CONTINUE
          vir=M_PI**2/3d0*brn+Virt2(S,T,i0,i1)+DipI(S,T,i0,i1)
          RES=RES+(qN(0,i0)*qN(1-ic,i1)+qN(ic,i0)*qN(1,i1))*brn*&
               DEXP((vir/brn+DLOG(S/mufs)*3D0+(lnNb-0.5*DLOG(S/mufs))/NM*4d0)*aa)
          EXP=EXP+(qN(0,i0)*qN(1-ic,i1)+qN(ic,i0)*qN(1,i1))*&
               (brn+vir*aa+(DLOG(S/mufs)*3D0+(lnNb-0.5d0*DLOG(S/mufs))/NM*4d0&
               +lnNb*(lnNb-DLOG(S/mufs))*4D0)*brn*aa)
       ENDDO
    ENDDO
    Thadronic_xs_2to2=(RES*ThG(lnNb,S,murs,murs,set)-EXP)*dij/24d0/S
    RETURN
  END FUNCTION Thadronic_xs_2to2

  FUNCTION ThG(LL,S,muf2,mur2,set)
    IMPLICIT NONE
    COMPLEX(KIND(1d0))::ThG
    COMPLEX(KIND(1d0)),INTENT(IN)::LL
    REAL(KIND(1d0)),INTENT(IN)::muf2,mur2
    REAL(KIND(1d0)),INTENT(IN)::S
    INTEGER,INTENT(IN)::set
    INTEGER,PARAMETER::nf=5
    REAL(KIND(1d0)),PARAMETER::dnf=DBLE(nf)
    REAL(KIND(1d0)),PARAMETER::M_PI=3.1415926535897932385D0
    REAL(KIND(1d0))::beta0,beta1
    REAL(KIND(1d0))::A1,A2 ! Sudakov coeff. for quarks
    COMPLEX(KIND(1d0))::lmbd, lnlmbd ! Logarithms
    COMPLEX(KIND(1d0))::g1,g2 ! LL+NLL Sudakov
    beta0=5.5D0-dnf/3D0
    beta1=25.5D0-9.5D0*dnf/3D0
    A1=8d0/3d0/beta0
    A2=0.5d0*A1*(67d0/3d0-M_PI**2-10d0/9d0*dnf)/beta0
    lmbd=aS(mur2,set)*beta0*LL
    lnlmbd=DLOG(1d0-2d0*lmbd)
    g1=A1*(2D0-(2D0-1D0/lmbd)*lnlmbd)
    g2=-A2*(2D0*lmbd+lnlmbd)+A1*(2D0*lmbd+lnlmbd)*DLOG(S/mur2)&
         -A1*2D0*lmbd*DLOG(S/muf2)+A1*beta1/beta0**2*(2D0*lmbd+&
         lnlmbd*(1D0+0.5D0*lnlmbd))
    ThG=EXP(LL*g1+g2)
    RETURN
  END FUNCTION ThG

  FUNCTION Thadronic_xs2_2to2(N,S,T)
    IMPLICIT NONE
    COMPLEX(KIND(1d0))::Thadronic_xs2_2to2
    COMPLEX(KIND(1d0)),INTENT(IN)::N
    REAL(KIND(1d0))::S,T
    COMPLEX(KIND(1d0)),PARAMETER::M_EULER=0.57721566490153286061D0
    COMPLEX(KIND(1d0)),PARAMETER::M_PI=3.1415926535897932385D0
    COMPLEX(KIND(1d0))::nb,ll ! logarithm
    COMPLEX(KIND(1d0))::Pgg,Pqg ! splitting functions
    REAL(KIND(1d0))::dij=1d0 ! symmetry factor
    COMPLEX(KIND(1d0))::gE,gN ! Getting N-space PDFs
    COMPLEX(KIND(1d0)),DIMENSION(0:1,0:5)::qE,qN ! Getting N-space PDFs
    INTEGER,PARAMETER::nf=5
    REAL(KIND(1d0)),PARAMETER::dnf=DCMPLX(nf)
    REAL(KIND(1d0))::beta0,aa
    COMPLEX(KIND(1d0))::lmbd
    COMPLEX(KIND(1d0))::RES=DCMPLX(0d0,0d0) ! resummed xs
    COMPLEX(KIND(1d0))::EXP=DCMPLX(0d0,0d0) ! the should-be subtracted xs
    INTEGER::i0,i1
    REAL(KIND(1d0))::brn,vir
    nb=N*DEXP(M_EULER)
    ll=LOG(nb)*2D0
    Pgg=4d0/3d0*(1.5d0+1d0/N/(N+1D0)-2D0*(Psi(N+1D0)+M_EULER))
    Pqg=0.5D0*(2D0+N+N**2)/N/(N+1D0)/(N+2D0)
    CALL pdfN(gN,qN,N)
    CALL pdfN(gE,qE,N)
    ! Evolving factorization scale (NLL)
    beta0=5.5d0-dnf/3d0
    aa=aS(murs,set)
    lmbd=aa*beta0*LOG(mufs/S*nb**2)
    CALL pdfEvolve(gE,qE,N,lmbd)
    ! loop over different initial states
    DO i0=0,5
       DO i1=0,5
          brn=Born(S,T,i0,i1)
          vir=M_PI**2/3D0*brn+Virt(S,T,i0,i1)+DipI(S,T,i0,i1)
          RES=RES+(qE(0,i0)*qE(1-ic,i1)+qE(ic,i0)*qE(1,i1))*(brn+vir*aa*4D0/3D0)
          EXP=EXP+(qN(0,i0)*qN(1-ic,i1)+qN(ic,i0)*qN(1,i1))*(brn+vir*4d0/3d0*aa&
               +(ll*(-ll*4d0/3d0+4d0)-2d0*Pqq*(ll-DLOG(S/mufs)))*aa*brn)
          EXP=EXP+(qN(0,i0)+qN(1-ic,i1)+qN(ic,i0)+qN(1,i1))*gN*&
               (DLOG(S/mufs)-ll)*Pqg*aa*brn
       ENDDO
    ENDDO
    Thadronic_xs2_2to2=(RES*ThG2(ll,S,murs,set)-EXP)*dij/24D0/S
    RETURN
  END FUNCTION Thadronic_xs2_2to2

  FUNCTION ThG2(LL,MIS,MURS,set)
    IMPLICIT NONE
    COMPLEX(KIND(1d0))::ThG2
    COMPLEX(KIND(1d0)),INTENT(IN)::LL
    REAL(KIND(1d0)),INTENT(IN)::MIS,MURS
    INTEGER,INTENT(IN)::set
    INTEGER,PARAMETER::nf=5
    REAL(KIND(1d0)),PARAMETER::dnf=DBLE(nf)
    REAL(KIND(1d0))::beta0,beta1 ! coeff of the beta function
    REAL(KIND(1d0))::A1,B1,A2 ! Sudakov coeff. for quarks
    REAL(KIND(1d0)),PARAMETER::M_PI=3.1415926535897932385D0
    COMPLEX(KIND(1d0))::lmbd,lnlmbd ! Logarithms
    COMPLEX(KIND(1d0))::g1,g2 ! LL+NLL Sudakov
    beta0=5.5d0-dnf/3d0
    beta1=25.5d0-9.5d0*dnf/3d0
    A1=8d0/3d0/beta0
    B1=-4d0/beta0
    A2=0.5d0*A1*(67D0/3D0-M_PI**2-10D0/9D0*dnf)/beta0
    lmbd=aS(MURS,set)*beta0*LL
    lnlmbd=LOG(1D0-lmbd)
    g1=A1*(1D0+lnlmbd/lmbd)
    g2=(lmbd+lnlmbd)*(A1*(DLOG(MIS/MURS)+beta1/beta0**2)-A2+B1)&
         -B1*lmbd+0.5D0*A1*beta1*(lnlmbd/beta0)**2
    ThG2=EXP(LL*g1+g2)
    RETURN
  END FUNCTION ThG2

  
END MODULE pQCD_resum
