MODULE Kinetic_Func
USE Helac_Global
USE Helac_Func_1
IMPLICIT NONE

CONTAINS

FUNCTION flambda(x1,x2,x3)
REAL(KIND=DBL),INTENT(IN)::x1,x2,x3
REAL(KIND=DBL)::flambda,f
f=x1**2+x2**2+x3**2-dnou(2)*x1*x2-dnou(2)*x1*x3-dnou(2)*x2*x3
flambda=DSQRT(f)
END FUNCTION flambda

FUNCTION cosij(a,b)
REAL(KIND=DBL),DIMENSION(3),INTENT(IN)::a,b
REAL(KIND=DBL)::cosij,den
cosij=a(1)*b(1)+a(2)*b(2)+a(3)*b(3)
den=DSQRT(a(1)**2+a(2)**2+a(3)**2)*&
     DSQRT(b(1)**2+b(2)**2+b(3)**2)
IF(den.EQ.0d0)THEN
   cosij=1d0
   RETURN
ENDIF
cosij=cosij/DSQRT(a(1)**2+a(2)**2+a(3)**2)
cosij=cosij/DSQRT(b(1)**2+b(2)**2+b(3)**2)
END FUNCTION cosij

FUNCTION scalar_product(p1,p2)
! momentums are in normal representation with fourth comp is the zero comp
REAL(KIND=DBL),DIMENSION(4),INTENT(IN)::p1,p2
REAL(KIND=DBL)::scalar_product
scalar_product=p1(4)*p2(4)-p1(3)*p2(3)-p1(2)*p2(2)-p1(1)*p2(1)
END FUNCTION scalar_product

SUBROUTINE cmstocollision()
! from the cms to the collision frmae
IMPLICIT NONE
REAL(KIND=DBL),DIMENSION(20,4)::p
REAL(KIND=DBL),DIMENSION(4)::pboo
REAL(KIND=DBL)::exp1,exp2
INTEGER::i
REAL(KIND=DBL)::q,e
DO i=1,n
   p(i,1:4)=phegas_pmom(i,1:4)
ENDDO
! collider lab frame:
! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
! beam2 = (FT_M/2,0,0,-FT_M/2), FT_M=beam(2)
! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
! collision frame:
! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
! ehat=Sqrt(s);e=Sqrt(S)
q=ehat
e=q/SQRT(xp1*xp2)
exp1=e/2d0*xp1
exp2=e/2d0*xp2
pboo(4)=(exp1+exp2)   ! the zero component
pboo(3)=(exp1-exp2)   ! the z component
pboo(1:2)=0
! boost p(i,1:4) via pboo to the collision frame
DO i=1,n
   CALL BOOSTL(q,pboo,p(i,1:4))
ENDDO
phegas_pmom(1:n,1:4)=p(1:n,1:4)
END SUBROUTINE cmstocollision

SUBROUTINE collisiontocms()
! from the collsion to the cms frmae
IMPLICIT NONE
REAL(KIND=DBL),DIMENSION(20,4)::p
REAL(KIND=DBL),DIMENSION(4)::pboo
REAL(KIND=DBL)::exp1,exp2
INTEGER::i
REAL(KIND=DBL)::q,e
DO i=1,n
   p(i,1:4)=phegas_pmom(i,1:4)
ENDDO
! collider lab frame:
! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
! beam2 = (FT_M/2,0,0,-FT_M/2), FT_M=beam(2)
! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
! collision frame:
! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
! ehat=Sqrt(s);e=Sqrt(S)
q=ehat
e=q/SQRT(xp1*xp2)
exp1=e/2d0*xp1
exp2=e/2d0*xp2
pboo(4)=(exp1+exp2)   ! the zero component
pboo(3)=-(exp1-exp2)   ! the z component
pboo(1:2)=0
! boost p(i,1:4) via pboo to the cms frame
DO i=1,n
   CALL BOOSTL(q,pboo,p(i,1:4))
ENDDO
phegas_pmom(1:n,1:4)=p(1:n,1:4)
END SUBROUTINE collisiontocms

SUBROUTINE collisiontolab()
! from the collision to the lab frame
IMPLICIT NONE
REAL(KIND=DBL),DIMENSION(20,4)::p
REAL(KIND=DBL),DIMENSION(4)::pboo
INTEGER::i
REAL(KIND=DBL)::q,e
DO i=1,n
   p(i,1:4)=phegas_pmom(i,1:4)
ENDDO
! collider lab frame:
! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
! beam2 = (FT_M/2,0,0,-FT_M/2), FT_M=beam(2)
! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
! collision frame:
! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
! ehat=Sqrt(s);e=Sqrt(S)
q=ehat
e=q/SQRT(xp1*xp2)
IF(.NOT.fixtarget)THEN
   pboo(4)=(ebeam(1)+ebeam(2))
   pboo(3)=(ebeam(1)-ebeam(2))
ELSE
   IF(.NOT.fixtargetrev)THEN
      pboo(4)=(ebeam(1)+ebeam(2))
      pboo(3)=ebeam(1)
   ELSE
      pboo(4)=(ebeam(1)+ebeam(2))
      pboo(3)=-ebeam(2)
   ENDIF
ENDIF
pboo(1:2)=0
! boost from the collision frame to the lab frame
DO i=1,n
   CALL BOOSTL(e,pboo,p(i,1:4))
ENDDO
phegas_pmom(1:n,1:4)=p(1:n,1:4)
END SUBROUTINE collisiontolab

SUBROUTINE labtocollision()
  ! from the lab to the collision frame
  IMPLICIT NONE
  REAL(KIND=DBL),DIMENSION(20,4)::p
  REAL(KIND=DBL),DIMENSION(4)::pboo
  INTEGER::i
  REAL(KIND=DBL)::q,e
  DO i=1,n
     p(i,1:4)=phegas_pmom(i,1:4)
  ENDDO
  ! collider lab frame:
  ! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
  ! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
  ! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
  ! beam2 = (FT_M/2,0,0,-FT_M/2), FT_M=beam(2)
  ! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
  ! collision frame:
  ! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
  ! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
  ! ehat=Sqrt(s);e=Sqrt(S)
  q=ehat
  e=q/SQRT(xp1*xp2)
  IF(.NOT.fixtarget)THEN
     pboo(4)=(ebeam(1)+ebeam(2))
     pboo(3)=-(ebeam(1)-ebeam(2))
  ELSE
     IF(.NOT.fixtargetrev)THEN
        pboo(4)=(ebeam(1)+ebeam(2))
        pboo(3)=-ebeam(1)
     ELSE
        pboo(4)=(ebeam(1)+ebeam(2))
        pboo(3)=ebeam(2)
     ENDIF
  ENDIF
  pboo(1:2)=0
  ! boost from the lab frame to the collision frame
  DO i=1,n
     CALL BOOSTL(e,pboo,p(i,1:4))
  ENDDO
  phegas_pmom(1:n,1:4)=p(1:n,1:4)
END SUBROUTINE labtocollision

SUBROUTINE cmstolab()
! from the cms to the lab frame
IMPLICIT NONE
REAL(KIND=DBL),DIMENSION(20,4)::p
REAL(KIND=DBL),DIMENSION(4)::pboo
REAL(KIND=DBL)::exp1,exp2
INTEGER::i
REAL(KIND=DBL)::q,e
DO i=1,n
   p(i,1:4)=phegas_pmom(i,1:4)
ENDDO
!IF(istruc.EQ.1..OR.istruc.EQ.2.OR..NOT.labeqcoll)THEN
! collider lab frame:
! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
! beam2 = (FT_M/2,0,0,-FT_M/2), FT_M=beam(2)
! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
! collision frame:
! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
! ehat=Sqrt(s);e=Sqrt(S)
  q=ehat                   
  e=q/SQRT(xp1*xp2)
  exp1=ebeam(1)*xp1
  exp2=ebeam(2)*xp2
  IF(fixtarget)THEN
     IF(.NOT.fixtargetrev)THEN
        exp1=exp1+FT_M*xp1/2d0
        exp2=exp2/2d0
     ELSE
        exp1=exp1/2d0
        exp2=exp2+FT_M*xp2/2d0
     ENDIF
  ENDIF
  pboo(4)=(exp1+exp2)   ! the zero component
  pboo(3)=(exp1-exp2)   ! the z component
  pboo(1:2)=0
  ! boost p(i,1:4) via pboo to the lab frame
  DO i=1,n
     CALL BOOSTL(q,pboo,p(i,1:4))
  ENDDO
!ENDIF
phegas_pmom(1:n,1:4)=p(1:n,1:4)
END SUBROUTINE cmstolab     

SUBROUTINE labtocms()
! from the lab frame to the cms
IMPLICIT NONE
REAL(KIND=DBL),DIMENSION(20,4)::p
REAL(KIND=DBL),DIMENSION(4)::pboo
REAL(KIND=DBL)::exp1,exp2
INTEGER::i
REAL(KIND=DBL)::q,e
DO i=1,n
   p(i,1:4)=phegas_pmom(i,1:4)
ENDDO
!IF(istruc.EQ.1.OR.istruc.EQ.2.OR..NOT.labeqcoll)THEN
! collider lab frame:
! xp1(ebeam(1),0,0,ebeam(1)), xp2(ebeam(2),0,0,-ebeam(2))
! fixed target (fake) lab frame (note it is not the true lab frame of fixed target but close to):
! beam1 = (beam(1)+FT_M/2,0,0,beam(1)+FT_M/2)
! beam2 = (FT_M/2,0,0,-FT_M/2), FT_M=beam(2)
! xp1(ebeam(1)+FT_M/2,0,0,ebeam(1)+FT_M/2), xp2(FT_M/2,0,0,-FT_M/2)
! collision frame:
! xp1(Sqrt(S)/2,0,0,Sqrt(S)/2); xp2(Sqrt(S)/2,0,0,-Sqrt(S)/2)
! Sqrt(s)=SQRT(xp1*xp2)*Sqrt(S)
! ehat=Sqrt(s);e=Sqrt(S)
  q=ehat                   
  e=q/SQRT(xp1*xp2)
  exp1=ebeam(1)*xp1
  exp2=ebeam(2)*xp2
  IF(fixtarget)THEN
     IF(.NOT.fixtargetrev)THEN
        exp1=exp1+FT_M*xp1/2d0
        exp2=exp2/2d0
     ELSE
        exp1=exp1/2d0
        exp2=exp2+FT_M*xp2/2d0
     ENDIF
  ENDIF
  pboo(4)=(exp1+exp2)   ! the zero component
  pboo(3)=-(exp1-exp2)   ! the z component
  pboo(1:2)=0
  ! boost p(i,1:4) via pboo to the cms frame
  DO i=1,n
     CALL BOOSTL(q,pboo,p(i,1:4))
  ENDDO
!ENDIF
phegas_pmom(1:n,1:4)=p(1:n,1:4)
END SUBROUTINE labtocms

FUNCTION prapidity(p)
! psudo-rapidity
REAL(KIND=DBL),DIMENSION(4)::p
REAL(KIND=DBL)::prapidity,c
IF(p(1)**2+p(2)**2+p(3)**2.LE.0d0)THEN
   prapidity = 0d0
   RETURN
ENDIF
c=p(3)/SQRT(p(1)**2+p(2)**2+p(3)**2)
IF(ABS(c).EQ.1d0)THEN
   prapidity = 0d0
ELSE
   prapidity=0.5d0*LOG((1+c)/(1-c))
ENDIF
END FUNCTION prapidity

! vector product
FUNCTION vec_crossprod(vec1,vec2)
  ! vec1 crosstime vec2
  IMPLICIT NONE
  REAL(KIND=DBL),DIMENSION(3),INTENT(IN)::vec1,vec2
  REAL(KIND=DBL),DIMENSION(3)::vec_crossprod
  vec_crossprod(1)=vec1(2)*vec2(3)-vec1(3)*vec2(2)
  vec_crossprod(2)=vec1(3)*vec2(1)-vec1(1)*vec2(3)
  vec_crossprod(3)=vec1(1)*vec2(2)-vec1(2)*vec2(1)
  RETURN
END FUNCTION vec_crossprod

FUNCTION rapidity(p)
! rapidity
REAL(KIND=DBL),DIMENSION(4)::p
REAL(KIND=DBL)::rapidity,c
IF(p(4).EQ.0d0)THEN
   rapidity = 0d0
   RETURN
ENDIF
c=p(3)/ABS(p(4))
IF(ABS(c).GE.1d0)THEN
   rapidity =0d0
ELSE
   rapidity=0.5d0*DLOG((1d0+c)/(1d0-c))
ENDIF
END FUNCTION rapidity

FUNCTION xFeynman(p,Ecms)
! xF is the Feynman parameter (for example see hep-ph/9702263)
! Ecms is the total energy in cms
IMPLICIT NONE
REAL(KIND=DBL),DIMENSION(4)::p
REAL(KIND=DBL)::xFeynman,Ecms
IF(Ecms.LT.0d0)THEN
   xFeynman=0d0
ELSE
   xFeynman=2d0*p(3)/Ecms ! yi = Log[xFi*E/mti/2 + Sqrt[1 + (xFi*E/mti/2)^2]]
ENDIF
END FUNCTION xFeynman
   
FUNCTION transverse(p)
REAL(KIND=DBL),DIMENSION(4)::p
REAL(KIND=DBL)::transverse
transverse=SQRT(p(1)**2+p(2)**2)
END FUNCTION transverse

FUNCTION getrapidity(en,pl)
  IMPLICIT NONE
  REAL(KIND(1d0))::getrapidity
  REAL(KIND(1d0)),INTENT(IN)::en,pl
  REAL(KIND(1d0)),PARAMETER::tiny=1.d-8
  REAL(KIND(1d0))::xplus,xminus,y
  xplus=en+pl
  xminus=en-pl
  IF(xplus.GT.tiny.AND.xminus.GT.tiny)THEN
     IF( (xplus/xminus).GT.tiny.AND.(xminus/xplus).GT.tiny)THEN
        y=0.5d0*LOG( xplus/xminus  )
     ELSE
        y=SIGN(1.d0,pl)*1.d8
     ENDIF
  ELSE
     y=SIGN(1.d0,pl)*1.d8
  ENDIF
  getrapidity=y
  RETURN
END FUNCTION getrapidity

SUBROUTINE BOOST(Q,PBOO,PCM,PLB)
! momentums are in normal representation with fourth comp is the zero comp
! Boost PCM via PBOO(PBOO^2=Q^2) to PLB
IMPLICIT NONE
REAL(KIND=DBL),INTENT(IN)::Q
REAL(KIND=DBL),DIMENSION(4),INTENT(IN)::PBOO,PCM
REAL(KIND=DBL),DIMENSION(4),INTENT(OUT)::PLB
REAL(KIND=DBL)::FACT
INTEGER::J
PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)+PBOO(1)*PCM(1))/Q
FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
DO J=1,3
  PLB(J)=PCM(J)+FACT*PBOO(J)
ENDDO
END SUBROUTINE BOOST


SUBROUTINE BOOSTL(Q,PBOO,P)
! momentums are in normal representation with fourth comp is the zero comp
! Boost P via PBOO(PBOO^2=Q^2) to PLB,and set to P
IMPLICIT NONE
REAL(KIND=DBL),INTENT(IN)::Q
REAL(KIND=DBL),DIMENSION(4),INTENT(IN)::PBOO
REAL(KIND=DBL),DIMENSION(4),INTENT(INOUT)::P
REAL(KIND=DBL),DIMENSION(4)::PCM,PLB
REAL(KIND=DBL)::FACT
INTEGER::J
PCM(1:4)=P(1:4)
PLB(4)=(PBOO(4)*PCM(4)+PBOO(3)*PCM(3)+PBOO(2)*PCM(2)+PBOO(1)*PCM(1))/Q
FACT=(PLB(4)+PCM(4))/(Q+PBOO(4))
DO J=1,3
   PLB(J)=PCM(J)+FACT*PBOO(J)
ENDDO
P(1:4)=PLB(1:4)
END SUBROUTINE BOOSTL

SUBROUTINE BOOST_via_DY(DY,P)
  IMPLICIT NONE
  REAL(KIND(1d0)),INTENT(IN)::DY
  REAL(KIND(1d0)),DIMENSION(4),INTENT(INOUT)::P
  REAL(KIND(1d0)),DIMENSION(4)::PLB
  PLB(1:4)=P(1:4)
  P(4)=DCOSH(DY)*PLB(4)-DSINH(DY)*PLB(3)
  P(3)=-DSINH(DY)*PLB(4)+DCOSH(DY)*PLB(3)
  RETURN
END SUBROUTINE BOOST_via_DY

SUBROUTINE ROTATEL(PROT,P)
  IMPLICIT NONE
  REAL(KIND=DBL),DIMENSION(3),INTENT(IN)::PROT
  REAL(KIND=DBL),DIMENSION(4),INTENT(INOUT)::P
  REAL(KIND=DBL),DIMENSION(3)::PTMP
  REAL(KIND=DBL)::costh,sinth,cosphi,sinphi
  REAL(KIND=DBL)::pp,pt
  pp=PROT(1)**2+PROT(2)**2+PROT(3)**2
  IF(pp.LT.0d0)RETURN
  pp=DSQRT(pp)
  costh=PROT(3)/pp
  sinth=DSQRT(1d0-costh**2)
  pt=PROT(1)**2+PROT(2)**2
  pt=DSQRT(pt)
  IF(pt.EQ.0d0)THEN
     cosphi=1d0
     sinphi=0d0
  ELSE
     cosphi=PROT(1)/pt
     sinphi=PROT(2)/pt
  ENDIF
  PTMP(1:3)=P(1:3)
  ! rotate via y aix by th
  PTMP(1)=costh*P(1)+sinth*P(3)
  PTMP(3)=-sinth*P(1)+costh*P(3)
  P(1:3)=PTMP(1:3)
  ! rotate via z aix by phi
  PTMP(1)=cosphi*P(1)-sinphi*P(2)
  PTMP(2)=sinphi*P(1)+cosphi*P(2)
  P(1:3)=PTMP(1:3)
  RETURN
END SUBROUTINE ROTATEL

FUNCTION ph4(px,py,pz)
IMPLICIT NONE
REAL(KIND=DBL),INTENT(IN)::px,py,pz
REAL(KIND=DBL)::ph4,S2,S
INTEGER::init=0
REAL(KIND=DBL)::pi
SAVE pi,init
IF(init.EQ.0)THEN
   CALL Helac_mypi(pi)
   init=1
ENDIF
IF(px**2+py**2.LE.0d0)THEN
   IF(pz.GE.0d0)ph4=0d0
   IF(pz.LT.0)ph4=pi
   RETURN
ENDIF
S2=px**2/(px**2+py**2)
S=DSQRT(S2)
IF(S.GT.1d0)THEN
   WRITE(*,*)'PH4(X) WARNING S=',S
   ph4=0
   IF(px.LT.0d0)ph4=pi
   RETURN
ENDIF
IF(px.LT.0)S=-DSQRT(S2)
ph4=DACOS(S)
IF(py.LT.0)ph4=2*pi-ph4
RETURN
END FUNCTION ph4


SUBROUTINE trans_p_to_zq(n1,p,zq)
IMPLICIT NONE
INTEGER,INTENT(IN)::n1
COMPLEX(KIND=DBL),DIMENSION(n1,5),INTENT(OUT):: zq
REAL(KIND=DBL),DIMENSION(n1,5),INTENT(IN)::p
INTEGER::i
REAL(KIND=DBL)::pt,pq
DO i=1,n1
   pt=DSQRT(p(i,1)**2+p(i,2)**2)
   pq=DSQRT(pt**2+p(i,3)**2)
   zq(i,1)= DCMPLX(p(i,4)+p(i,3),p(i,3))
   zq(i,2)= DCMPLX(p(i,4)-p(i,3),pt)
   zq(i,3)= DCMPLX(p(i,1), p(i,2))
   zq(i,4)= DCMPLX(p(i,1),-p(i,2))
   zq(i,5)= DCMPLX( p(i,5) , pq )
ENDDO
END SUBROUTINE trans_p_to_zq


SUBROUTINE transform(n1,zq,p)
IMPLICIT NONE
INTEGER,INTENT(IN)::n1
COMPLEX(KIND=DBL),DIMENSION(n1,5),INTENT(IN)::zq
REAL(KIND=DBL),DIMENSION(n1,5),INTENT(OUT)::p
INTEGER::i
DO i=1,n1 
   p(i,4)=DREAL(zq(i,1)+zq(i,2))/2
   p(i,3)=DREAL(zq(i,1)-zq(i,2))/2
   p(i,1)=DREAL(zq(i,3))
   p(i,2)=DIMAG(zq(i,3))
   p(i,5)=DREAL(zq(i,5))
ENDDO
END SUBROUTINE transform

SUBROUTINE InitKtSmearing(NP,POLD,PNEW,ktavg,imeth)
  ! One reference here is: hep-ph/9806253
  !USE Helac_ranmar_mod
  ! performing initial kt smearing
  IMPLICIT NONE
  INTEGER,INTENT(IN)::NP
  REAL(KIND(1d0)),DIMENSION(4,NP),INTENT(IN)::POLD
  REAL(KIND(1d0)),DIMENSION(4,NP),INTENT(OUT)::PNEW
  REAL(KIND(1d0)),INTENT(IN)::ktavg
  INTEGER,INTENT(IN)::imeth
  REAL(KIND(1d0))::wmax,val,rrr,phi,kt2,Pkt2,Etot,Ptot,ktx,kty
  REAL(KIND(1d0))::boostpx,boostpy,boostE,boostpz
  REAL(KIND(1d0)),PARAMETER::boostm = 10d0
  REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
  INTEGER::jj,kk
  REAL(KIND(1d0)),DIMENSION(4)::PBOO
  IF(ktavg.LE.0d0.OR.NP.LE.2)THEN
     PNEW(1:4,1:NP)=POLD(1:4,1:NP)
     RETURN
  ENDIF
  Etot=POLD(4,1)+POLD(4,2)
  IF(Etot.LE.0d0)THEN
     PNEW(1:4,1:NP)=POLD(1:4,1:NP)
     RETURN
  ENDIF
  ! Method -> 1 : Wmax*r <= W; 2 : directly, 
  ! in this case one should know the analytically dG (kt) = P (kt) dkt
  val=64d0*ktavg**4-pi**2
  IF(val.GT.0d0)THEN
     wmax=16d0*ktavg**2*(8d0*ktavg**2+DSQRT(val))/pi**2
     wmax=wmax*DEXP(-1d0-DSQRT(val)/8d0/ktavg**2)
  ELSE
     wmax=1d0
  ENDIF
  ! P[kt2_] := Exp[-kt2*Pi/8/kav^2]*(1 + kt2^2)
  ! InvP[r_] := -8 kav^2/Pi *Log[r]
  boostpz=0d0
  IF(imeth.EQ.1)THEN
     jj=0
     DO WHILE(jj.LT.1)
        ! rand is an internal random generator of Fortran
        phi=2d0*pi*rand()
        kt2=DTAN(pi/2d0*rand())
        Pkt2=DEXP(-kt2*pi/8d0/ktavg**2)*(1d0+kt2**2)
        rrr=rand()
        IF(wmax*rrr.LT.Pkt2)THEN
           ptot=POLD(3,1)+POLD(3,2)
           jj=jj+1
           ktx=DSQRT(kt2)*DCOS(phi)
           kty=DSQRT(kt2)*DSIN(phi)
           boostpx=ktx/Etot*boostm
           boostpy=kty/Etot*boostm
           boostE=DSQRT(boostm**2+boostpx**2+boostpy**2+boostpz**2)
           PBOO(4)=boostE
           PBOO(1)=boostpx
           PBOO(2)=boostpy
           PBOO(3)=boostpz
           DO kk=1, NP
              CALL BOOST(boostm,PBOO,POLD(1:4,kk),PNEW(1:4,kk))
           ENDDO
        ENDIF
     END DO
  ELSE
     phi=2d0*pi*rand()
     rrr=rand()
     DO WHILE(rrr.LE.0d0)
        rrr=rand()
     ENDDO
     kt2=-8d0*ktavg**2/pi*DLOG(rrr)
     ptot=POLD(3,1)+POLD(3,2)
     ktx=DSQRT(kt2)*DCOS(phi)
     kty=DSQRT(kt2)*DSIN(phi)
     boostpx=ktx/Etot*boostm
     boostpy=kty/Etot*boostm
     boostE=DSQRT(boostm**2+boostpx**2+boostpy**2+boostpz**2)
     PBOO(4)=boostE
     PBOO(1)=boostpx
     PBOO(2)=boostpy
     PBOO(3)=boostpz
     DO kk=1, NP
        CALL BOOST(boostm,PBOO,POLD(1:4,kk),PNEW(1:4,kk))
     ENDDO
  ENDIF
  RETURN
END SUBROUTINE InitKtSmearing

SUBROUTINE ggBeamsKtSmearing(NP,POLD,PNEW,ktavg2)
  ! Originally from Mangano, Nason, Ridolfi, Nucl. Phys. B405 (1993) 507
  ! Also see Eq.(6) in arXiv:1210.4610 
  ! the Kt smearing for the two initial gluon beams (more like TMD formalism) 
  IMPLICIT NONE
  INTEGER,INTENT(IN)::NP
  REAL(KIND(1d0)),DIMENSION(4,NP),INTENT(IN)::POLD
  REAL(KIND(1d0)),DIMENSION(4,NP),INTENT(OUT)::PNEW
  REAL(KIND(1d0)),INTENT(IN)::ktavg2 ! <kt^2>
  REAL(KIND(1d0))::wmax,val,kt2,Pkt2,Etot,Ptot,ktx,kty
  REAL(KIND(1d0))::boostpx,boostpy,boostE,boostpz
  REAL(KIND(1d0)),PARAMETER::boostm = 10d0
  REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
  INTEGER::jj,kk
  REAL(KIND(1d0)),DIMENSION(2)::kti1,kti2
  REAL(KIND(1d0)),DIMENSION(4)::PBOO
  IF(ktavg2.LE.0d0.OR.NP.LE.2)THEN
     PNEW(1:4,1:NP)=POLD(1:4,1:NP)
     RETURN
  ENDIF
  Etot=POLD(4,1)+POLD(4,2)
  IF(Etot.LE.0d0)THEN
     PNEW(1:4,1:NP)=POLD(1:4,1:NP)
     RETURN
  ENDIF
  boostpz=0d0
  CALL gBeamKtSmearing(ktavg2,kti1)
  CALL gBeamKtSmearing(ktavg2,kti2)
  ptot=POLD(3,1)+POLD(3,2)
  ktx=kti1(1)+kti2(1)
  kty=kti1(2)+kti2(2)
  boostpx=ktx/Etot*boostm
  boostpy=kty/Etot*boostm
  boostE=DSQRT(boostm**2+boostpx**2+boostpy**2+boostpz**2)
  PBOO(4)=boostE
  PBOO(1)=boostpx
  PBOO(2)=boostpy
  PBOO(3)=boostpz
  DO kk=1,NP
     CALL BOOST(boostm,PBOO,POLD(1:4,kk),PNEW(1:4,kk))
  ENDDO
  RETURN
END SUBROUTINE ggBeamsKtSmearing

SUBROUTINE gBeamKtSmearing(ktavg2,kti)
  ! Originally from Mangano, Nason, Ridolfi, Nucl. Phys. B405 (1993) 507
  ! Also see Eq.(6) in arXiv:1210.4610
  ! the Kt smearing for the initial gluon beam (more like TMD formalism)
  IMPLICIT NONE
  REAL(KIND(1d0)),INTENT(IN)::ktavg2 ! <kt**2>
  REAL(KIND(1d0)),DIMENSION(2),INTENT(OUT)::kti
  REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
  REAL(KIND(1d0))::phi,rrr,kt2,kt
  ! directly,
  ! in this case one should know the analytically dG (kt) = P (kt) dkt
  ! P[kt2_] := Exp[-kt2/ktavg2]/ktavg2
  ! InvP[r_] := -ktavg2*Log[r]
  phi=2d0*pi*rand()
  rrr=rand()
  DO WHILE(rrr.LE.0d0)
     rrr=rand()
  ENDDO
  kt2=DABS(-ktavg2*DLOG(rrr))
  kt=DSQRT(kt2)
  kti(1)=kt*DCOS(phi)
  kti(2)=kt*DSIN(phi)
  RETURN
END SUBROUTINE gBeamKtSmearing

SUBROUTINE OniumRadiationFunction(Lam,bn,PQQ,Ponium,PX)
  ! Eq.(2.22) in hep-ph/0001062
  IMPLICIT NONE
  REAL(KIND(1d0)),DIMENSION(4),INTENT(IN)::PQQ ! momentum of the heavy quark pair
  REAL(KIND(1d0)),INTENT(IN)::Lam,bn
  REAL(KIND(1d0)),DIMENSION(4),INTENT(OUT)::Ponium,PX ! momenta of onium and soft colour multiple field
  REAL(KIND(1d0))::PQQ0,k2,k02,kvec,r1,r2,rr
  REAL(KIND(1d0))::Xmin,Ymin,X,Y,Lam2,wmax,wvalue,bnp1
  INTEGER::jj
  REAL(KIND(1d0)),PARAMETER::pi=3.1415926535897932385d0
  REAL(KIND(1d0))::phi,costheta,sintheta,Q
  REAL(KIND(1d0)),DIMENSION(4)::Ponium0,PX0
  PQQ0=PQQ(4)**2-PQQ(1)**2-PQQ(2)**2-PQQ(3)**2
  IF(PQQ0.LT.0d0)THEN
     WRITE(*,*)"ERROR: the input PQQ is spacelike !"
     STOP
  ENDIF
  ! get the approximated wmax
  bnp1=bn+1d0
  IF(bnp1.GT.-2d0)THEN
     wmax=bnp1**(bnp1/2D0)/(bnp1+2d0)**(bnp1/2D0+1D0)&
          *DSQRT(PQQ0)**(bnp1+1D0)
  ELSE
     WRITE(*,*)"ERROR: bn <= -3 is blowed up"
     STOP
  ENDIF
  !PQQ0=DSQRT(PQQ0)
  Lam2=Lam**2
  Xmin=DEXP(-PQQ0/Lam2)
  jj=0
  DO WHILE(jj.LT.1)
     r1=rand()
     X=(1d0-Xmin)*r1+Xmin
     k2=-DLOG(X)*Lam2
     Ymin=DEXP(-(PQQ0+k2)**2/(4D0*PQQ0*Lam2))
     r2=rand()
     Y=(X-Ymin)*r2+Ymin
     k02=-DLOG(Y)*Lam2
     wvalue=(X-Ymin)*k2/2D0/DSQRT(k02)*DSQRT(k02-k2)**bnp1
     rr=rand()
     IF(rr*wmax.LE.wvalue)jj=1
  ENDDO
  ! in the rest frame of PQQ
  kvec=DSQRT(k02-k2)
  PX0(4)=DSQRT(k02)
  costheta=2D0*rand()-1D0
  sintheta=DSQRT(DABS(1D0-costheta**2))
  phi=2d0*pi*rand()
  PX0(3)=kvec*costheta
  PX0(1)=kvec*sintheta*DCOS(phi)
  PX0(2)=kvec*sintheta*DSIN(phi)
  Q=DSQRT(PQQ0)
  Ponium0(4)=Q-PX0(4)
  Ponium0(1:3)=-PX0(1:3)
  ! boost to the lab frame
  CALL BOOST(Q,PQQ,PX0,PX)
  CALL BOOST(Q,PQQ,Ponium0,Ponium)
  RETURN
END SUBROUTINE OniumRadiationFunction


! the following are momentum reshufflings
! refer to arXiv:1907.04898 (Option A)
SUBROUTINE InitialStateReshuffling(NP,PA,MA,xa,xb,PO,xanew,xbnew)
  IMPLICIT NONE
  INTEGER,INTENT(IN)::NP
  REAL(KIND(1d0)),DIMENSION(NP,4),INTENT(IN)::PA ! original external momenta
  REAL(KIND(1d0)),DIMENSION(NP),INTENT(IN)::MA ! on-shell masses to be ajusted (negative, no need to ajust)
  REAL(KIND(1d0)),DIMENSION(NP,4),INTENT(OUT)::PO ! new external momenta after reshuffling
  REAL(KIND(1d0)),DIMENSION(4)::PSUM
  REAL(KIND(1d0)),INTENT(IN)::xa,xb ! original xa and xb
  REAL(KIND(1d0)),INTENT(OUT)::xanew,xbnew ! new xa and xb
  REAL(KIND(1d0))::snew,sold,stot
  REAL(KIND(1d0)),DIMENSION(4)::PIN
  REAL(KIND(1d0)),DIMENSION(NP,4)::PA_cms
  REAL(KIND(1d0)),DIMENSION(4)::PBOO
  REAL(KIND(1d0))::Q,EBEAM1,EBEAM2
  INTEGER::i
  IF(N.GT.3)THEN
     ! always assume the first two momenta are the initial states
     ! boost to the rest frame of the parton-parton collisions
     DO I=1,NP
        PA_cms(I,1:4)=PA(I,1:4)
     ENDDO
     PIN(1:4)=PA(1,1:4)+PA(2,1:4)
     sold=PIN(4)**2-PIN(1)**2-PIN(2)**2-PIN(3)**2
     EBEAM1=PA(1,4)/xa
     EBEAM2=PA(2,4)/xb
     Q=DSQRT(sold)
     PBOO(4)=PA(1,4)+PA(2,4)
     PBOO(3)=-(PA(1,3)+PA(2,3))
     PBOO(1:2)=0d0
     DO I=1,NP
        !     When PBOO(4)=P(4) and PBOO(1:3)=-P(1:3), it boosts P to its rest frame (0,0,0,Q)                           
        CALL BOOSTL(Q,PBOO(1:4),PA_cms(I,1:4))
     ENDDO
     ! always assume the first two momenta are the initial state  
     PSUM(1:4)=0d0
     DO i=3,NP
        IF(MA(i).GT.0d0)THEN
           PO(i,1:3)=PA(i,1:3)
           PO(i,4)=DSQRT(MA(i)**2+PO(i,1)**2+PO(i,2)**2+PO(i,3)**2)
        ELSE
           PO(i,1:4)=PA(i,1:4)
        ENDIF
        PSUM(1:4)=PSUM(1:4)+PO(i,1:4)
     ENDDO
     snew=PSUM(4)**2-PSUM(1)**2-PSUM(2)**2-PSUM(3)**2
     stot=sold/xa/xb
     xanew=MIN(DSQRT(xa*snew/xb/stot),0.999999999999999D0)
     xbnew=MIN(DSQRT(xb*snew/xa/stot),0.999999999999999D0)
     PO(1,4)=DSQRT(snew)/2d0
     PO(1,3)=PO(1,4)
     PO(1,1:2)=0d0
     PO(2,4)=DSQRT(snew)/2d0
     PO(2,3)=-PO(2,4)
     PO(2,1:2)=0d0
     ! boost back to the lab frame (same as PA)
     Q=DSQRT(snew)
     PBOO(4)=xanew*EBEAM1+xbnew*EBEAM2
     PBOO(3)=xanew*EBEAM1-xbnew*EBEAM2
     PBOO(1:2)=0d0
     DO I=1,NP
        CALL BOOSTL(Q,PBOO(1:4),PO(I,1:4))
     ENDDO
  ELSE
     ! avoid roundingoff errors
     IF(MA(3).LT.0d0)THEN
        DO I=1,NP
           PO(I,1:4)=PA(I,1:4)
        ENDDO
        xanew=xa
        xbnew=xb
     ELSE
        PIN(1:4)=PA(1,1:4)+PA(2,1:4)
        sold=PIN(4)**2-PIN(1)**2-PIN(2)**2-PIN(3)**2
        EBEAM1=PA(1,4)/xa
        EBEAM2=PA(2,4)/xb
        snew=MA(3)**2
        stot=sold/xa/xb
        xanew=MIN(DSQRT(xa*snew/xb/stot),0.9999999999999999D0)
        xbnew=MIN(DSQRT(xb*snew/xa/stot),0.9999999999999999D0)
        PO(1,4)=xanew*EBEAM1
        PO(1,3)=PO(1,4)
        PO(1,1:2)=0d0
        PO(2,4)=xbnew*EBEAM2
        PO(2,3)=-PO(2,4)
        PO(2,1:2)=0d0
        PO(3,1:4)=PO(1,1:4)+PO(2,1:4)
     ENDIF
  ENDIF
  RETURN
END SUBROUTINE InitialStateReshuffling


! the following are momentum reshufflings
! refer to arXiv:1907.04898 (Option B) 
SUBROUTINE FinalStateReshuffling(NP,PA,MA,xa,xb,PO,xanew,xbnew)
  IMPLICIT NONE
  INTEGER,INTENT(IN)::NP
  REAL(KIND(1d0)),DIMENSION(NP,4),INTENT(IN)::PA ! original external momenta
  REAL(KIND(1d0)),DIMENSION(NP),INTENT(IN)::MA ! on-shell masses to be ajusted (negative, no need to ajust)
  REAL(KIND(1d0)),DIMENSION(NP,4),INTENT(OUT)::PO ! new external momenta after reshuffling
  REAL(KIND(1d0)),INTENT(IN)::xa,xb ! original xa and xb
  REAL(KIND(1d0)),INTENT(OUT)::xanew,xbnew ! new xa and xb
  REAL(KIND(1d0))::snew,sold,stot,Monium,Mother,Monium0,qvec,EBEAM1,EBEAM2
  REAL(KIND(1d0)),DIMENSION(4)::PIN,Pother,Ponium,Pother_new,PBOO
  INTEGER::i,i_choose
  INTEGER,DIMENSION(0:NP)::changemass_index
  ! always assume the first two momenta are the initial states
  PIN(1:4)=PA(1,1:4)+PA(2,1:4)
  sold=PIN(4)**2-PIN(1)**2-PIN(2)**2-PIN(3)**2
  stot=sold/xa/xb
  changemass_index(0)=0
  DO i=3,NP
     IF(MA(i).GT.0d0)THEN
        changemass_index(0)=changemass_index(0)+1
        changemass_index(changemass_index(0))=i
     ENDIF
  ENDDO
  snew=sold
  PO(1:NP,1:4)=PA(1:NP,1:4)
  IF(changemass_index(0).EQ.0)THEN
     xanew=xa
     xbnew=xb
     RETURN
  ENDIF
  !IF(xa.NE.xb)THEN
  ! first boost back to the rest frame of the initial partonic frame
  PBOO(4)=PA(1,4)+PA(2,4)
  PBOO(1:3)=-(PA(1,1:3)+PA(2,1:3))
  DO i=3,NP
     CALL BOOSTL(DSQRT(snew),PBOO,PO(i,1:4))
  ENDDO
  !ENDIF
  DO WHILE(changemass_index(0).GT.0)
     Pother(1:4)=0d0
     DO i=3,NP
        IF(i.EQ.changemass_index(1))THEN
           Ponium(1:4)=PO(i,1:4)
           Monium=MA(i)
        ELSE
           Pother(1:4)=Pother(1:4)+PO(i,1:4)
        ENDIF
     ENDDO
     Mother=DSQRT(DABS(Pother(4)**2-Pother(1)**2-Pother(2)**2-Pother(3)**2))
     Monium0=DSQRT(DABS(Ponium(4)**2-Ponium(1)**2-Ponium(2)**2-Ponium(3)**2))
     IF(snew.LT.(Monium+Mother)**2)THEN
        ! increase the snew
        snew=(Monium+Mother)**2/(Monium0+Mother)**2*snew
     ENDIF
     PO(changemass_index(1),4)=(snew+Monium**2-Mother**2)/2d0/DSQRT(snew)
     qvec=DSQRT(PO(changemass_index(1),1)**2+PO(changemass_index(1),2)**2+PO(changemass_index(1),3)**2)
     PO(changemass_index(1),1:3)=PO(changemass_index(1),1:3)&
          *DSQRT(PO(changemass_index(1),4)**2-Monium**2)/qvec
     Pother_new(4)=(snew+Mother**2-Monium**2)/2d0/DSQRT(snew)
     qvec=DSQRT(Pother(1)**2+Pother(2)**2+Pother(3)**2)
     Pother_new(1:3)=Pother(1:3)*DSQRT(Pother_new(4)**2-Mother**2)/qvec
     ! take two boosts
     DO i=3,NP
        PBOO(4)=Pother(4)
        PBOO(1:3)=-Pother(1:3)
        IF(i.NE.changemass_index(1))THEN
           CALL BOOSTL(Mother,PBOO,PO(i,1:4)) ! now PO is in the rest frame of Pother
           CALL BOOSTL(Mother,Pother_new,PO(i,1:4)) ! now PO is boosted back to the lab frame
        ENDIF
     ENDDO
     changemass_index(0)=changemass_index(0)-1
     IF(changemass_index(0).GT.0)THEN
        changemass_index(1:changemass_index(0))=changemass_index(2:changemass_index(0)+1)
     ENDIF
  ENDDO
  !IF(xa.NE.xb)THEN
  ! boost back to the lab frame
  !PBOO(4)=PA(1,4)+PA(2,4)
  !PBOO(1:3)=(PA(1,1:3)+PA(2,1:3))
  !DO i=3,NP
  !   CALL BOOSTL(DSQRT(snew),PBOO,PO(i,1:4))
  !ENDDO
  !ENDIF
  IF(snew.NE.sold)THEN
     ! change the initial states as well
     xanew=MIN(DSQRT(xa*snew/xb/stot),0.99999999999999999999D0)
     xbnew=MIN(DSQRT(xb*snew/xa/stot),0.99999999999999999999D0)
     PO(1,4)=DSQRT(snew)/2D0
     PO(1,3)=PO(1,4)
     PO(1,1:2)=0d0
     PO(2,4)=DSQRT(snew)/2D0
     PO(2,3)=-PO(2,4)
     PO(2,1:2)=0d0
     EBEAM1=PA(1,4)/xa
     EBEAM2=PA(2,4)/xb
     PBOO(4)=xanew*EBEAM1+xbnew*EBEAM2
     PBOO(1:3)=xanew*EBEAM1-xbnew*EBEAM2
     DO i=1,NP
        CALL BOOSTL(DSQRT(snew),PBOO,PO(i,1:4))
     ENDDO
  ELSE
     ! initial state does not change
     PBOO(4)=PA(1,4)+PA(2,4)
     PBOO(1:3)=(PA(1,1:3)+PA(2,1:3))
     DO i=3,NP
        CALL BOOSTL(DSQRT(snew),PBOO,PO(i,1:4))
     ENDDO
     xanew=xa
     xbnew=xb
  ENDIF
  RETURN
END SUBROUTINE FinalStateReshuffling
END MODULE Kinetic_Func
