FUNCTION pdg2lhanpdf(ih,ipdg,x,xmu,a)
!****************************************************************************
!        wrapper for calling the nuclear PDF correction factor in lhapdf
!****************************************************************************
  IMPLICIT NONE
  REAL(KIND(1d0))::pdg2lhanpdf
  INTEGER,INTENT(IN)::ih,ipdg
  REAL(KIND(1d0)),INTENT(IN)::x,xmu,a
  ! EPS09 NLO grid files
  ! He: EPS09NLOR_4
  ! Li:  EPS09NLOR_6
  ! Be: EPS09NLOR_9
  ! C:  EPS09NLOR_12
  ! O:  EPS09NLOR_16
  ! Ne:  EPS09NLOR_20
  ! Al:  EPS09NLOR_27
  ! Ca: EPS09NLOR_40
  ! Fe: EPS09NLOR_56
  ! Cu: EPS09NLOR_64
  ! Kr:  EPS09NLOR_84
  ! Ag: EPS09NLOR_108
  ! Sn: EPS09NLOR_117
  ! In: EPS09NLOR_115
  ! Xe:  EPS09NLOR_131
  ! W:  EPS09NLOR_184
  ! Pt: EPS09NLOR_195
  ! Au: EPS09NLOR_197
  ! Pb: EPS09NLOR_208
  ! U:  EPS09NLOR_238
  INTEGER::i,j,ipart,iporg,ireuse,iset,imem
  INTEGER,DIMENSION(2)::ihlast,imemlast
  REAL(KIND(1d0)),DIMENSION(2)::xlast,xmulast,alast
  REAL(KIND(1d0)),DIMENSION(-7:7,2)::pdflast
  LOGICAL::init=.TRUE.
  SAVE ihlast,xlast,xmulast,alast,pdflast,imemlast,init
  IF(init)THEN
     ihlast(1:2)=-99
     xlast(1:2)=-99d9
     xmulast(1:2)=-99d9
     pdflast(-7:7,1:2)=-99d9
     imemlast(1:2)=-99
     alast(1:2)=-99d9
  ENDIF
  IF(x.EQ.0d0)THEN
     pdg2lhanpdf=0d0
     RETURN
  ELSEIF(x.LT.0d0.OR.x.GT.1d0)THEN
     WRITE(*,*) 'PDF not supported for Bjorken x ', x
     STOP
  ENDIF
  
  ipart=ipdg
  IF(ipart.EQ.21) ipart=0
  IF(IABS(ipart).EQ.22) ipart=7
  iporg=ipart
  
  ! This will be called for any PDG code, but we only support up to 7
  IF(IABS(ipart).GT.7)THEN
     WRITE(*,*) 'PDF not supported for pdg ',ipdg
     STOP
  ENDIF

  ! Determine the iset used in lhapdf
  CALL getnset(iset)
  IF(iset.NE.1)THEN
     WRITE(*,*) 'PDF not supported for Bjorken x ', x
     STOP
  ENDIF
  
  ! Determine the member of the set (function of lhapdf)
  CALL getnmem(iset,imem)

  ireuse = 0
  DO i=1,2
     ! Check if result can be reused since any of last two calls
     IF(x.EQ.xlast(i) .AND. xmu.EQ.xmulast(i) .AND.&
          imem.EQ.imemlast(i) .AND. ih.EQ.ihlast(i).AND.&
          a.EQ.alast(i))THEN
        ireuse = i
     ENDIF
  ENDDO

  !  Reuse previous result, if possible
  IF(ireuse.GT.0)THEN
     IF(pdflast(iporg,ireuse).NE.-99d9)THEN
        pdg2lhanpdf=pdflast(iporg,ireuse)
        RETURN
     ENDIF
  ENDIF

  ! Bjorken x and/or facrorization scale and/or PDF set are not 
  ! identical to the saved values: this means a new event and we
  ! should reset everything to compute new PDF values. Also, determine
  ! if we should fill ireuse=1 or ireuse=2.
  IF(ireuse.EQ.0.AND.xlast(1).NE.-99d9.AND.xlast(2).NE.-99d9)THEN
     DO i=1,2
        xlast(i)=-99d9
        alast(i)=-99d9
        xmulast(i)=-99d9
        DO j=-7,7
           pdflast(j,i)=-99d9
        ENDDO
        imemlast(i)=-99
        ihlast(i)=-99
     ENDDO
     ! everything has been reset. Now set ireuse=1 to fill the first
     ! arrays of saved values below
     ireuse=1
  ELSEIF(ireuse.EQ.0.AND.xlast(1).NE.-99d9)THEN
     ! This is first call after everything has been reset, so the first
     ! arrays are already filled with the saved values (hence
     ! xlast(1).ne.-99d9). Fill the second arrays of saved values (done
     ! below) by setting ireuse=2
     ireuse=2
  ELSEIF(ireuse.EQ.0)THEN
     ! Special: only used for the very first call to this function:
     ! xlast(i) are initialized as data statements to be equal to -99d9
     ireuse=1
  ENDIF

  ! Call lhapdf and give the current values to the arrays that should
  ! be saved
  CALL pftonpdglha(ih,x,xmu,a,pdflast(-7:7,ireuse))
  alast(ireuse)=a
  xlast(ireuse)=x
  xmulast(ireuse)=xmu
  ihlast(ireuse)=ih
  imemlast(ireuse)=imem
  pdg2lhanpdf=pdflast(ipart,ireuse)
  RETURN
  
END FUNCTION pdg2lhanpdf

SUBROUTINE pftonpdglha(ih,x,q,a,pdf)
!***************************************************************************
!     Wrapper for calling the nuclear pdf from lhapdf
!***************************************************************************
  IMPLICIT NONE
  REAL(KIND(1d0)),INTENT(IN)::x,q,a
  REAL(KIND(1d0)),DIMENSION(-7:7),INTENT(OUT)::pdf
  REAL(KIND(1d0)),DIMENSION(-6:6)::f ! nuclear PDF*x
  INTEGER,INTENT(IN)::IH ! 
  INTEGER::I
  REAL(KIND(1d0))::photon

  IF(ABS(ih).EQ.1)THEN
     pdf(-7)=0d0
     pdf(7)=0d0
     CALL evolvePDFa(x,q,a,f)
     DO i=-6,6
        pdf(i)=f(i)/x
     ENDDO
  ELSE
     WRITE(*,*) 'beam type is not supported in lhadpf'
     DO i=-7,7
        pdf(i)=0d0
     ENDDO
  ENDIF

  RETURN
END SUBROUTINE pftonpdglha

SUBROUTINE init_nPDF(nPDF,lparm)
  IMPLICIT NONE
  INTEGER,INTENT(IN)::nPDF
  CHARACTER(len=20),INTENT(OUT)::lparm
  CHARACTER(len=20)::cnum
  SELECT CASE(nPDF)
  CASE(1)
     lparm='EPS08'
  CASE(2)
     lparm='EPS09'
  CASE(3)
     lparm='EPS09LO'
  CASE(301:331)
     lparm='EPS09LO,'
     WRITE(cnum,*)(nPDF-300)
     lparm=ADJUSTL(TRIM(lparm))//ADJUSTL(TRIM(cnum))
  CASE(4)
     lparm='EPS09NLO'
  CASE(401:431)
     lparm='EPS09NLO,'
     WRITE(cnum,*)(nPDF-400)
     lparm=ADJUSTL(TRIM(lparm))//ADJUSTL(TRIM(cnum))
  CASE(5)
     lparm='EKS98'
  CASE DEFAULT
     WRITE(*,*)"ERROR:Unknown nuclear PDF nPDF=",nPDF
     STOP
  END SELECT
  ! internal LHAPDF function
  CALL setlhaparm(ADJUSTL(TRIM(lparm)))
  RETURN
END SUBROUTINE init_nPDF
