MODULE unweight_lhe
IMPLICIT NONE
CONTAINS
SUBROUTINE Generate_lhe(n1,nevent,process,icase,lheversion)
!USE Helac_Global
!USE CONSTANTS
IMPLICIT NONE
INTEGER,INTENT(IN)::n1,nevent,icase
CHARACTER(len=400),INTENT(IN)::process
INTEGER,INTENT(IN),OPTIONAL::lheversion
INTEGER::used_lhever
CHARACTER(len=24),PARAMETER::tmp_dir="./tmp/",output_dir="./output/"
INTEGER::i,jj,nunit4,nunit5,nunit3,nunit30
INTEGER::istop,k
INTEGER,DIMENSION(:),ALLOCATABLE::rwgt_ids
REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::rwgt_vals
REAL(KIND(1d0)),DIMENSION(3)::Rscaleupdown,Fscaleupdown,Mscaleupdown
REAL(KIND(1d0))::p0,px,py,pz,SPINUP,EBMUP1,EBMUP2,XSECUP1,XSECUP2,XERRUP1,XMAXUP1,&
XWGTUP,VTIMUP,SCALUP,PM0,AQEDUP,AQCDUP,XWGTUP2
INTEGER::IDBMUP1,IDBMUP2,IDWTUP,NPRUP,IDPRUP,NUP,IDUP,ISTUP,IMOTHUP1,IMOTHUP2,&
ICOLUP1,ICOLUP2,iPDFGUP1,iPDFGUP2,iPDFSUP1,iPDFSUP2,LPRUP1
INTEGER::irwgt=0,irwgt2
REAL(KIND(1d0))::xxwgt_old
INCLUDE 'pdfreweight_common.inc'
INCLUDE 'pdf2reweight_common.inc'
INCLUDE 'npdfreweight_common.inc'
INCLUDE "ref_reweight_common.inc"
INCLUDE 'scalereweight_common.inc'
INCLUDE 'muNRQCD_reweight_common.inc'
IF(PRESENT(lheversion))THEN
   IF(lheversion.GT.3.OR.lheversion.LT.0)THEN
      WRITE(*,*)"WARNING:the lhe version is not valid, lheversion=",lheversion
      WRITE(*,*)"WARNING:using the lheversion=1 !"
      used_lhever=1
   ELSE
      used_lhever=lheversion
   ENDIF
ELSE
   used_lhever=1
ENDIF
nunit3=30
CLOSE(nunit3)
OPEN(nunit3,FILE=TRIM(tmp_dir)//'even_'//TRIM(process(2:10*n1+1))//'.out',FORM='unformatted')
nunit4=31
CLOSE(nunit4)
IF(scale_reweight_common.OR.pdf_reweight_common.OR.pdf2_reweight_common.OR.npdf_reweight_common&
     .OR.ref_reweight_common.OR.muNRQCD_reweight_common)THEN
   nunit30=300
   CLOSE(nunit30)
   OPEN(nunit30,FILE=TRIM(tmp_dir)//'even_rwgt_'//TRIM(process(2:10*n1+1))//'.out',FORM='unformatted')
   used_lhever=MAX(3,used_lhever) ! see page 158 of arXiv:1405.1067 
ENDIF
OPEN(nunit4,FILE=TRIM(tmp_dir)//'sample'//TRIM(process(2:10*n1+1))//'.init')
nunit5=32
CLOSE(nunit5)
OPEN(nunit5,FILE=TRIM(output_dir)//'sample'//TRIM(process(2:10*n1+1))//'.lhe')
SELECT CASE(used_lhever)
CASE(1)
   WRITE(nunit5,'(A)') '<LesHouchesEvents version="1.0">'
CASE(2)
   WRITE(nunit5,'(A)') '<LesHouchesEvents version="2.0">'
CASE(3)
   WRITE(nunit5,'(A)') '<LesHouchesEvents version="3.0">'
CASE DEFAULT
   WRITE(*,*)"ERROR:Unknown LHE version, ",used_lhever
   STOP
END SELECT
WRITE(nunit5,'(A)') '<!--'
WRITE(nunit5,'(A)') 'File generated with HELAC-ONIA '
WRITE(nunit5,'(A)') '-->'
IF(scale_reweight_common.OR.pdf_reweight_common.OR.pdf2_reweight_common&
     .OR.npdf_reweight_common.OR.ref_reweight_common.OR.muNRQCD_reweight_common)THEN
   irwgt=0
   IF(scale_reweight_common)THEN
      irwgt=irwgt+9
   ENDIF
   IF(muNRQCD_reweight_common)THEN
      irwgt=irwgt+3
   ENDIF
   IF(pdf_reweight_common)THEN
      irwgt=irwgt+pdf_set_max_common-pdf_set_min_common+2
   ENDIF
   IF(pdf2_reweight_common)THEN
      irwgt=irwgt+pdf2_set_max_common-pdf2_set_min_common+2
   ENDIF
   IF(npdf_reweight_common)THEN
      irwgt=irwgt+npdf_set_max_common-npdf_set_min_common+2
   ENDIF
   IF(ref_reweight_common)THEN
      irwgt=irwgt+ref_set_max_common-ref_set_min_common+2
   ENDIF
   IF(ALLOCATED(rwgt_ids))THEN
      DEALLOCATE(rwgt_ids)
   ENDIF
   ALLOCATE(rwgt_ids(irwgt))
   IF(ALLOCATED(rwgt_vals))THEN
      DEALLOCATE(rwgt_vals)
   ENDIF
   ALLOCATE(rwgt_vals(irwgt))

   WRITE(nunit5,'(A)') '  <header>'
   WRITE(nunit5,'(A)') '  <initrwgt>'
   irwgt2=0
   IF(scale_reweight_common)THEN
      WRITE(nunit5,'(A)') "    <weightgroup type='scale_variation' combine='envelope'>"
      Rscaleupdown(1)=1d0
      Rscaleupdown(2)=rw_RScale_up_common
      Rscaleupdown(3)=rw_RScale_down_common
      Fscaleupdown(1)=1d0
      Fscaleupdown(2)=rw_FScale_up_common
      Fscaleupdown(3)=rw_FScale_down_common
      k=1000
      DO i=1,3
         DO jj=1,3
            k=k+1
            irwgt2=irwgt2+1
            rwgt_ids(irwgt2)=k
            WRITE(nunit5,6100)k,Rscaleupdown(i),Fscaleupdown(jj)
         ENDDO
      ENDDO
      WRITE(nunit5,'(A)') "    </weightgroup>"
   ENDIF
   IF(muNRQCD_reweight_common)THEN
      WRITE(nunit5,'(A)') "    <weightgroup type='muNRQCD_variation' combine='envelope'>"
      Mscaleupdown(1)=1d0
      Mscaleupdown(2)=rw_MScale_up_common
      Mscaleupdown(3)=rw_MScale_down_common
      k=1199
      DO i=1,3
         k=k+1
         irwgt2=irwgt2+1
         rwgt_ids(irwgt2)=k
         WRITE(nunit5,6800)k,Mscaleupdown(i)
      ENDDO
      WRITE(nunit5,'(A)') "    </weightgroup>"
   ENDIF
   IF(pdf_reweight_common)THEN
      WRITE(nunit5,'(A)') "    <weightgroup type='PDF_variation' combine='lhapdf'>"
      k=1999
      DO i=pdf_set_min_common-1,pdf_set_max_common
         k=k+1
         irwgt2=irwgt2+1
         rwgt_ids(irwgt2)=k
         WRITE(nunit5,6200)k,i
      ENDDO
      WRITE(nunit5,'(A)') "    </weightgroup>"
   ENDIF
   IF(pdf2_reweight_common)THEN
      WRITE(nunit5,'(A)') "    <weightgroup type='PDF2_variation' combine='lhapdf'>"
      k=4999
      DO i=pdf2_set_min_common-1,pdf2_set_max_common
         k=k+1
         irwgt2=irwgt2+1
         rwgt_ids(irwgt2)=k
         WRITE(nunit5,6700)k,i
      ENDDO
      WRITE(nunit5,'(A)') "    </weightgroup>"
   ENDIF
   IF(npdf_reweight_common)THEN
      WRITE(nunit5,'(A)') "    <weightgroup type='nPDF_variation' combine='lhapdf'>"
      k=5999
      IF(npdf_set_min_common.NE.1001)THEN
         DO i=npdf_set_min_common-1,npdf_set_max_common
            k=k+1
            irwgt2=irwgt2+1
            rwgt_ids(irwgt2)=k
            WRITE(nunit5,6500)k,i
         ENDDO
      ELSE
         READ(nunit4,*)IDBMUP1,IDBMUP2,EBMUP1,EBMUP2,iPDFGUP1,iPDFGUP2,iPDFSUP1,iPDFSUP2,IDWTUP,NPRUP
         REWIND(nunit4)
         DO i=npdf_set_min_common-1,npdf_set_max_common
            k=k+1
            irwgt2=irwgt2+1
            rwgt_ids(irwgt2)=k
            WRITE(nunit5,6500)k,i-1000+IPDFSUP1
         ENDDO
      ENDIF
      WRITE(nunit5,'(A)') "    </weightgroup>"
   ENDIF
   IF(ref_reweight_common)THEN
      WRITE(nunit5,'(A)') "    <weightgroup type='reference' combine='none'>"
      k=8999
      DO i=ref_set_min_common-1,ref_set_max_common
         k=k+1
         irwgt2=irwgt2+1
         rwgt_ids(irwgt2)=k
         WRITE(nunit5,6600)k,i
      ENDDO
      WRITE(nunit5,'(A)') "    </weightgroup>"
   ENDIF
   WRITE(nunit5,'(A)') '  </initrwgt>'
   WRITE(nunit5,'(A)') '  </header>'
   IF(irwgt2.NE.irwgt)THEN
      WRITE(*,*)"ERROR:inconsistece of the number of reweights"
      WRITE(*,*)"irwgt=",irwgt
      WRITE(*,*)"irwgt2=",irwgt2
      STOP
   ENDIF
ENDIF
WRITE(nunit5,'(A)') '<init>' 
READ(nunit4,*)IDBMUP1,IDBMUP2,EBMUP1,EBMUP2,iPDFGUP1,iPDFGUP2,iPDFSUP1,iPDFSUP2,IDWTUP,NPRUP
READ(nunit4,*)XSECUP1,XERRUP1,XMAXUP1,LPRUP1
WRITE(nunit5,5000)IDBMUP1,IDBMUP2,EBMUP1,EBMUP2,iPDFGUP1,iPDFGUP2,iPDFSUP1,iPDFSUP2,IDWTUP,NPRUP
WRITE(nunit5,5100)XSECUP1,XERRUP1,XMAXUP1,LPRUP1       
WRITE(nunit5,'(A)') '</init>'
istop=1
k=0
XWGTUP2=XSECUP1/nevent
DO WHILE(istop.EQ.1)
	k=k+1
	READ(nunit3,END=100) NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP
        IF(icase.EQ.1)XWGTUP=XWGTUP2
	WRITE(nunit5,'(A)') '<event>' 
	WRITE(nunit5,5200) NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP
	DO i=1,NUP
		READ(nunit3)IDUP,ISTUP,iMOTHUP1,iMOTHUP2,ICOLUP1,ICOLUP2,px,py,pz,p0,pm0,VTIMUP,SPINUP
                ! VTIMUP is c*tau in unit of mm
                ! tau is the proper lifetime, which is
                ! -log(r)/width (r is a random number from 0 to 1, width is in GeV)
                ! VTIMUP=c*tau=1.973269718d-13*(-log(r)/width)
		WRITE(nunit5,5300)IDUP,ISTUP,iMOTHUP1,iMOTHUP2,ICOLUP1,ICOLUP2,px,py,pz,p0,pm0,VTIMUP,SPINUP
	ENDDO
        IF(irwgt.GT.0.AND.(scale_reweight_common.OR.pdf_reweight_common.OR.pdf2_reweight_common&
             .OR.npdf_reweight_common.OR.ref_reweight_common.OR.muNRQCD_reweight_common))THEN
           ! filling the rwgt
           WRITE(nunit5,'(A)') '  <rwgt>'
           READ(nunit30)(rwgt_vals(i),i=1,irwgt)
           IF(icase.EQ.1)THEN
              xxwgt_old=rwgt_vals(1)
              IF(xxwgt_old.EQ.0d0)THEN
                 xxwgt_old=1d0
              ENDIF
              DO i=1,irwgt
                 rwgt_vals(i)=rwgt_vals(i)*XWGTUP2/xxwgt_old
              ENDDO
           ENDIF
           DO i=1,irwgt
              WRITE(nunit5,6300)rwgt_ids(i),rwgt_vals(i)
           ENDDO
           WRITE(nunit5,'(A)') '  </rwgt>'
        ENDIF
	WRITE(nunit5,'(A)') '</event>' 
ENDDO
100  CONTINUE
IF(icase.EQ.1.AND.k.NE.nevent+1)THEN
   WRITE(*,*)"WARNING:mismatching of the unweighted lhe events number ",k-1,nevent
ENDIF
WRITE(nunit5,'(A)') '</LesHouchesEvents>' 
CLOSE(nunit3,STATUS='delete')
CLOSE(nunit4,STATUS='delete')
IF(scale_reweight_common.OR.pdf_reweight_common.OR.pdf2_reweight_common.OR.npdf_reweight_common&
     .OR.ref_reweight_common.OR.muNRQCD_reweight_common)THEN
   CLOSE(nunit30,STATUS='delete')
ENDIF
5200 FORMAT(1P,2I6,4E14.6)
5300 FORMAT(1P,I8,5I5,5E18.10,E14.6,E12.4)
5000 FORMAT(1P,2I8,2E14.6,6I8)
5100 FORMAT(1P,3E20.10,I6)
CLOSE(nunit5,STATUS='keep')
6100 FORMAT(6x,"<weight id='",I4,"'>",1x,&
          "muR=",E11.5,1x,"muF=",E11.5,1x,'</weight>')
6200 FORMAT(6x,"<weight id='",I4,"'>",1x,&
          "pdfset=",I6,1x,"</weight>")
6300 FORMAT(4x,"<wgt id='",I4,"'>",1x,&
          E11.5,1x,"</wgt>")
6500 FORMAT(6x,"<weight id='",I4,"'>",1x,&
          "npdfset=",I6,1x,"</weight>")
6600 FORMAT(6x,"<weight id='",I4,"'>",1x,&
          "reference=",I6,1x,"</weight>")
6700 FORMAT(6x,"<weight id='",I4,"'>",1x,&
          "pdf2set=",I6,1x,"</weight>")
6800 FORMAT(6x,"<weight id='",I4,"'>",1x,&
          "muNRQCD=",E11.5,1x,'</weight>')
RETURN
END SUBROUTINE Generate_lhe

SUBROUTINE FILL_MC_MASS(MonteCarlo)
  IMPLICIT NONE
  INTEGER::i
  CHARACTER*10,INTENT(IN)::MonteCarlo
  REAL(KIND(1d0)),DIMENSION(-16:21)::MCMASS
  DO i=-16,21
     MCMASS(i)=-1d10
  ENDDO
  IF(MonteCarlo.EQ.'HERWIG6')THEN
     INCLUDE '../shower/HERWIG6/MCmasses_HERWIG6.inc'
  ELSEIF(MonteCarlo.EQ.'HERWIGPP')THEN
     INCLUDE '../shower/HERWIGPP/MCmasses_HERWIGPP.inc'
  ELSEIF(MonteCarlo.EQ.'PYTHIA6Q')THEN
     INCLUDE '../shower/PYTHIA6/MCmasses_PYTHIA6Q.inc'
  ELSEIF(MonteCarlo.EQ.'PYTHIA6PT')THEN
     INCLUDE '../shower/PYTHIA6/MCmasses_PYTHIA6PT.inc'
  ELSEIF(MonteCarlo.EQ.'PYTHIA8')THEN
     INCLUDE '../shower/PYTHIA8/MCmasses_PYTHIA8.inc'
  ELSE
     WRITE(*,*)'ERROR:Unknown MC = '//TRIM(MonteCarlo)//' in fill_MC_MASS'
     STOP
  ENDIF
  DO i=-5,-1
     MCMASS(i)=MCMASS(-i)
  ENDDO
  DO i=-16,-11
     MCMASS(i)=MCMASS(-i)
  ENDDO
  RETURN
END SUBROUTINE FILL_MC_MASS
END MODULE unweight_lhe
