MODULE qcd_splitting_kernels
  USE convolution_communicator
  USE qcd_constants
  USE global_constants
  IMPLICIT NONE
  PRIVATE
  PUBLIC::AP_SF_P0gg,AP_SF_P0qq,AP_SF_P0gq,AP_SF_P0qg
  PUBLIC::AP_SFS_P1NSplus,AP_SFS_P1NSminus,AP_SFS_P1qqV,AP_SFS_P1qqbarV
  PUBLIC::AP_SFS_P1PSplus,AP_SFS_P1qqS,AP_SFS_P1gg,AP_SFS_P1qg,AP_SFS_P1gq
  PUBLIC::AP_SFS_P2PSplus,AP_SFS_P2qg,AP_SFS_P2gq,AP_SFS_P2gg
  PUBLIC::AP_SFS_P2NSplus,AP_SFS_P2NSminus,AP_SFS_P2NSS
  PUBLIC::AP_SFS_P2qqS,AP_SFS_P2qqbarS,AP_SFS_P2qqV,AP_SFS_P2qqbarV
  PUBLIC::AP_SFS_P2qqS2,AP_SFS_P2qqV2
  PUBLIC::AP_SFT_P1NSplus,AP_SFT_P1NSminus,AP_SFT_P1qqV,AP_SFT_P1qqbarV
  PUBLIC::AP_SFT_P1PSplus,AP_SFT_P1qqS,AP_SFT_P1gg,AP_SFT_p1qg,AP_SFT_P1gq
  PUBLIC::AP_DSFTS_P2PSplus,AP_DSFTS_P2gg,AP_SFT_P2qg,AP_SFT_P2gq
  PUBLIC::AP_DSFTS_P2NSplus,AP_DSFTS_P2NSminus,AP_SFT_P2PSplus
  PUBLIC::AP_SFT_P2NSplus,AP_SFT_P2NSminus,AP_SFT_P2NSS
  PUBLIC::AP_SFT_P2qqS,AP_SFT_P2qqbarS,AP_SFT_P2qqV,AP_SFT_P2qqbarV
  PUBLIC::AP_SFT_P2qqS2,AP_SFT_P2qqV2,AP_SFT_P2gg
CONTAINS
!=====================================================================
! Unpolarized Altarelli-Parisi splitting functions in QCD
! The coefficients of 
! P(x,as)=P0(x)+as/2/Pi*P1(x)+(as/2/Pi)**2*P2(x)+...
! x=exp(-y)
! \int dx f(x)_+ g(x) = \int dy [exp(-y)f(exp(-y))]_+ g(exp(-y))
!=====================================================================

! at LO (one-loop), space like and time like splitting functions are identical

  FUNCTION AP_SF_P0gg(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero
    
    IF(cc_piece.EQ.cc_Delta)THEN
       res=(11d0*CA-4d0*nfTF)/6d0
       RETURN
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=two*CA*(x/(one-x)+(one-x)/x+x*(one-x))
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT)
       res=res-two*CA*one/(one-x)
    CASE(cc_PLUS0)
       res=two*CA
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SF_P0gg

  FUNCTION AP_SF_P0qq(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=3d0/2d0*CF
       RETURN
    ENDIF

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CF*((one+x**2)/(one-x))
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT)
       res=res-CF*two/(one-x)
    CASE(cc_PLUS0)
       res=two*CF
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SF_P0qq

  FUNCTION AP_SF_P0gq(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CF*(one+(one-x)**2)/x
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT)
       res=res-zero
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SF_P0gq

  FUNCTION AP_SF_P0qg(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=TF*(x**2+(one-x)**2)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT)
       res=res-zero
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SF_P0qg

  !--------------------------------------------------!
  ! NLO or two-loop  space like                      !
  !--------------------------------------------------!

  ! Space like
  ! P_{ns}^{+,(1)}=P_{qq}^{v,(1)}+P_{q\bar{q}}^{v,(1)} 
  FUNCTION AP_SFS_P1NSplus(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2,h0o1mx
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=CF*(CA*(17d0/24d0+11d0/3d0*zeta2-3d0*zeta3)&
            +CF*(3d0/8d0-3d0*zeta2+6d0*zeta3)-nfTF*(1d0/6d0+4d0/3d0*zeta2))
       RETURN
    ENDIF
    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    IF(x.GE.one)THEN
       h0o1mx=-one
    ELSE
       h0o1mx=h0/(one-x)
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CF*CA*(17d0/18d0-151d0/18d0*x+two*zeta2*(x**2+x+one)/(one+x)&
            +two*(one+x**2)/(one+x)*S2+(one+x**2)*x/(one+x)*h0o1mx*h0&
            +11d0/6d0*(one+x**2)*h0o1mx)+CF**2*((x-one-two*zeta2*(one+x**2)/(one+x))&
            -four*(one+x**2)/(one+x)*S2+two*(one+x**2)*hp1*h0o1mx&
            +half*(one-x)**2/(one+x)*h0**2-(one+two*x)*h0o1mx)
       res=res+nfTF*CF*(22d0/9d0*x-2d0/9d0-two/three*(one+x**2)*h0o1mx)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REAL)
       res=res+CF*CA*(67d0/9d0-two*zeta2)/(one-x)
       res=res+CF*nfTF*(-20d0/9d0)/(one-x)
    CASE(cc_PLUS0)
       res=CF*CA*(67d0/9d0-two*zeta2)
       res=res+CF*nfTF*(-20d0/9d0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1NSplus

  ! Space like
  ! P_{ns}^{-,(1)}=P_{qq}^{v,(1)}-P_{q\bar{q}}^{v,(1)}
  FUNCTION AP_SFS_P1NSminus(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res,res0
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hm1,hm2
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    IF(PRESENT(y_is_x))THEN
       res0=AP_SFS_P1NSplus(y,y_is_x)
    ELSE
       res0=AP_SFS_P1NSplus(y)
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=res0
       RETURN
    ENDIF

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       hm1=DLOG(one+x)
       hm2=-DDILOG(-x)
       res=res+two*CF*(two*CF-CA)*((one+x**2)/(one+x)*(zeta2-half*h0**2+two*(h0*hm1-hm2))&
            +two*x-two-(one+x)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    res=res+res0
    RETURN
  END FUNCTION AP_SFS_P1NSminus

  ! Space like
  ! P_qq^{v,(1)}
  FUNCTION AP_SFS_P1qqV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,h0o1mx
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=CF*(CA*(17d0/24d0+11d0/3d0*zeta2-3d0*zeta3)&
            +CF*(3d0/8d0-3d0*zeta2+6d0*zeta3)-nfTF*(1d0/6d0+4d0/3d0*zeta2))
       RETURN
    ENDIF
    h0=DLOG(x)
    hp1=-DLOG(one-x)
    IF(x.GE.one)THEN
       h0o1mx=-one
    ELSE
       h0o1mx=h0/(one-x)
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CF*CA*(53d0/18d0-187d0/18d0*x+zeta2*(one+x)&
            +half*(one+x**2)*h0o1mx*h0&
            +one/6d0*(17d0+5d0*x**2)*h0o1mx)+CF**2*((5d0*x-5d0)&
            +two*(one+x**2)*hp1*h0o1mx&
            -half*(one+x)*h0**2+(two*x**2-two*x-three)*h0o1mx)
       res=res+nfTF*CF*(22d0/9d0*x-2d0/9d0-two/three*(one+x**2)*h0o1mx)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REAL)
       res=res+CF*CA*(67d0/9d0-two*zeta2)/(one-x)
       res=res+CF*nfTF*(-20d0/9d0)/(one-x)
    CASE(cc_PLUS0)
       res=CF*CA*(67d0/9d0-two*zeta2)
       res=res+CF*nfTF*(-20d0/9d0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1qqV

  ! Space like
  ! P^{v,(1)}_{q\bar{q}}
  FUNCTION AP_SFS_P1qqbarV(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hm1,hm2
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       hm1=DLOG(one+x)
       hm2=-DDILOG(-x)
       res=-CF*(two*CF-CA)*((one+x**2)/(one+x)*(zeta2-half*h0**2+two*(h0*hm1-hm2))&
            +two*x-two-(one+x)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1qqbarV

  ! Space like
  ! P^{s,(1)}_+=P^{s,(1)}_{qq}+P^{s,(1)}_{q\bar{q}}
  ! P^{s,(1)}_-=0
  FUNCTION AP_SFS_P1PSplus(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       res=two*CF*TF*(two/9d0*(one-x)/x*(28d0*x**2+x+10d0)&
            -(one+x)*h0**2+(8d0/three*x**2+5d0*x+one)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1PSplus

  ! Space like
  ! P^{s,(1)}_{qq}=P^{s,(1)}_{qqbar}=P^{s,(1)}_{qq^+}/2
  FUNCTION AP_SFS_P1qqS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       res=CF*TF*(two/9d0*(one-x)/x*(28d0*x**2+x+10d0)&
            -(one+x)*h0**2+(8d0/three*x**2+5d0*x+one)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1qqS

  ! Space like
  ! P^{(1)}_gg
  FUNCTION AP_SFS_P1gg(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2,h0o1mx
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=CA**2*(three*zeta3+8d0/three)
       res=res-four/three*CA*nfTF-CF*nfTF
       RETURN
    ENDIF
    
    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    IF(x.GE.one)THEN
       h0o1mx=-one
    ELSE
       h0o1mx=h0/(one-x)
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CA**2*(-25d0/18d0-109d0/18d0*x+two*zeta2*(two*x**3+two*x**2&
            +four*x+three)/(one+x)+four*(one+x+x**2)**2/(one+x)/x*S2&
            +four*(one-x+x**2)**2/x*h0o1mx*hp1+two*(x**2-x-one)**2/(one+x)*h0*h0o1mx&
            +(-25d0/three+11d0/three*x-44d0/three*x**2)*h0)
       res=res+CA*nfTF*(-four/three*(one+x)*h0+two/9d0*(23d0*x**3-19d0*x**2+29d0*x-23d0)/x)
       res=res+CF*nfTF*(four/three*(one-x)*(one-11d0*x-5d0*x**2)/x-two*(one+x)*h0**2&
            -two*(5d0*x+three)*h0)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REAL)
       res=res+CA**2*(67d0/9d0-two*zeta2)/(one-x)
       res=res+CA*nfTF*(-20d0/9d0)/(one-x)
    CASE(cc_PLUS0)
       res=CA**2*(67d0/9d0-two*zeta2)
       res=res+CA*nfTF*(-20d0/9d0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1gg

  ! Space like
  ! P^{(1)}_{qg}
  FUNCTION AP_SFS_P1qg(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CA*TF*(-four*zeta2*x-(218d0*x**3-225d0*x**2+18d0*x-20d0)/9d0/x&
            -two*(two*x**2+two*x+one)*S2-(two*x**2-two*x+one)*hp1**2&
            -(one+two*x)*h0**2+four*x*(one-x)*hp1+(44d0/3d0*x**2+8d0*x+one)*h0)&
            +CF*TF*(-two*zeta2*(two*x**2-two*x+one)+(10d0*x**2-29d0/two*x+7d0)&
            +(two*x**2-two*x+one)*hp1*(two*h0+hp1)+(two*x**2-x+half)*h0**2&
            -four*x*(one-x)*hp1+(four*x**2-two*x+three/two)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1qg

  ! Space like
  ! P^{(1)}_{gq}
  FUNCTION AP_SFS_P1gq(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CA*CF*(four*zeta2+(44d0*x**3+37d0*x**2+19d0*x+9d0)/9d0/x&
            +two*(x**2+two*x+two)/x*S2+(x**2-two*x+two)/x*(two*h0+hp1)*hp1&
            +(two+x)*h0**2-(17d0*x**2-22d0*x+22d0)/three/x*hp1&
            -(8d0/three*x**2+5d0*x+12d0)*h0)&
            +CF**2*(-7d0/two*x-5d0/two-(x**2-two*x+two)/x*hp1**2-(two-x)/two*h0**2&
            +(5d0*x**2-6d0*x+6d0)/x*hp1+(7d0/two*x+two)*h0)
       res=res+nfTF*CF*(-8d0/9d0*(four*x**2-5d0*x+5d0)/x+four/three*(x**2-two*x+two)/x*hp1)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P1gq

  !--------------------------------------------------!
  ! NLO or two-loop  time like                       !
  !--------------------------------------------------!

  ! Time like
  ! P_{ns}^{+,(1)}=P_{qq}^{v,(1)}+P_{q\bar{q}}^{v,(1)}
  FUNCTION AP_SFT_P1NSplus(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2,h0o1mx
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=CF*(CA*(17d0/24d0+11d0/3d0*zeta2-3d0*zeta3)&
            +CF*(3d0/8d0-3d0*zeta2+6d0*zeta3)-nfTF*(1d0/6d0+4d0/3d0*zeta2))
       RETURN
    ENDIF
    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    IF(x.GE.one)THEN
       h0o1mx=-one
    ELSE
       h0o1mx=h0/(one-x)
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CF*CA*(17d0/18d0-151d0/18d0*x+two*zeta2*(x**2+x+one)/(one+x)&
            +two*(one+x**2)/(one+x)*S2+(one+x**2)*x/(one+x)*h0o1mx*h0&
            +11d0/6d0*(one+x**2)*h0o1mx)+CF**2*((x-one-two*zeta2*(one+x**2)/(one+x))&
            -four*(one+x**2)/(one+x)*S2-two*(one+x**2)*hp1*h0o1mx&
            -half*(7d0*x**3+three*x**2+5d0*x+one)/(one+x)*h0o1mx*h0+(two+x)*x*h0o1mx)
       res=res+nfTF*CF*(22d0/9d0*x-2d0/9d0-two/three*(one+x**2)*h0o1mx)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REAL)
       res=res+CF*CA*(67d0/9d0-two*zeta2)/(one-x)
       res=res+CF*nfTF*(-20d0/9d0)/(one-x)
    CASE(cc_PLUS0)
       res=CF*CA*(67d0/9d0-two*zeta2)
       res=res+CF*nfTF*(-20d0/9d0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1NSplus

  ! Time like
  ! P_{ns}^{-,(1)}=P_{qq}^{v,(1)}-P_{q\bar{q}}^{v,(1)}
  FUNCTION AP_SFT_P1NSminus(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res,res0
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hm1,hm2
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    IF(PRESENT(y_is_x))THEN
       res0=AP_SFT_P1NSplus(y,y_is_x)
    ELSE
       res0=AP_SFT_P1NSplus(y)
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=res0
       RETURN
    ENDIF

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       hm1=DLOG(one+x)
       hm2=-DDILOG(-x)
       res=res+two*CF*(two*CF-CA)*((one+x**2)/(one+x)*(zeta2-half*h0**2+two*(h0*hm1-hm2))&
            +two*x-two-(one+x)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    res=res+res0
    RETURN
  END FUNCTION AP_SFT_P1NSminus

  ! Time like
  ! P_qq^{v,(1)}
  FUNCTION AP_SFT_P1qqV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,h0o1mx
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=CF*(CA*(17d0/24d0+11d0/3d0*zeta2-3d0*zeta3)&
            +CF*(3d0/8d0-3d0*zeta2+6d0*zeta3)-nfTF*(1d0/6d0+4d0/3d0*zeta2))
       RETURN
    ENDIF
    h0=DLOG(x)
    hp1=-DLOG(one-x)
    IF(x.GE.one)THEN
       h0o1mx=-one
    ELSE
       h0o1mx=h0/(one-x)
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CF*CA*(53d0/18d0-187d0/18d0*x+zeta2*(one+x)&
            +half*(one+x**2)*h0o1mx*h0&
            +one/6d0*(17d0+5d0*x**2)*h0o1mx)+CF**2*((5d0*x-5d0)&
            -two*(one+x**2)*hp1*h0o1mx&
            -half*(three+5d0*x**2)*h0*h0o1mx+(three*x**2+two*x-two)*h0o1mx)
       res=res+nfTF*CF*(22d0/9d0*x-2d0/9d0-two/three*(one+x**2)*h0o1mx)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REAL)
       res=res+CF*CA*(67d0/9d0-two*zeta2)/(one-x)
       res=res+CF*nfTF*(-20d0/9d0)/(one-x)
    CASE(cc_PLUS0)
       res=CF*CA*(67d0/9d0-two*zeta2)
       res=res+CF*nfTF*(-20d0/9d0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1qqV

  ! Time like = Space like
  ! P^{v,(1)}_{q\bar{q}}
  FUNCTION AP_SFT_P1qqbarV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res

    IF(PRESENT(y_is_x))THEN
       res=AP_SFS_P1qqbarV(y,y_is_x)
    ELSE
       res=AP_SFS_P1qqbarV(y,y_is_x)
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1qqbarV

  ! Time like
  ! P^{s,(1)}_+=P^{s,(1)}_{qq}+P^{s,(1)}_{q\bar{q}}
  ! P^{s,(1)}_-=0
  FUNCTION AP_SFT_P1PSplus(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       res=two*CF*TF*(-four/9d0*(one-x)/x*(14d0*x**2+23d0*x+5d0)&
            +(one+x)*h0**2-(8d0/three*x**2+9d0*x+5d0)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1PSplus

  ! Time like
  ! P^{s,(1)}_{qq}=P^{s,(1)}_{qqbar}=P^{s,(1)}_{qq^+}/2
  FUNCTION AP_SFT_P1qqS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       h0=DLOG(x)
       res=CF*TF*(-four/9d0*(one-x)/x*(14d0*x**2+23d0*x+5d0)&
            +(one+x)*h0**2-(8d0/three*x**2+9d0*x+5d0)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1qqS

  ! Time like
  ! P^{(1)}_gg
  FUNCTION AP_SFT_P1gg(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2,h0o1mx
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       res=CA**2*(three*zeta3+8d0/three)
       res=res-four/three*CA*nfTF-CF*nfTF
       RETURN
    ENDIF

    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    IF(x.GE.one)THEN
       h0o1mx=-one
    ELSE
       h0o1mx=h0/(one-x)
    ENDIF
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CA**2*(-25d0/18d0-109d0/18d0*x+two*zeta2*(two*x**3+two*x**2&
            +four*x+three)/(one+x)+four*(one+x+x**2)**2/(one+x)/x*S2&
            -four*(one-x+x**2)**2/x*h0o1mx*hp1-two*(x**5-four*x**4-x**3&
            +four*x**2+x+two)/(one+x)/x*h0*h0o1mx&
            +(22d0*x**4-19d0*x**3+30d0*x**2+11d0*x-22d0)/x/three*h0o1mx)
       res=res+CA*nfTF*(-four/three*(two*x**4-5d0*x**3+6d0*x**2-3d0*x+two)/x*h0o1mx&
            +two/9d0*(23d0*x**3-19d0*x**2+29d0*x-23d0)/x)
       res=res+CF*nfTF*(four/9d0*(one-x)*(23d0+14d0*x+41d0*x**2)/x+two*(one+x)*h0**2&
            +two/three*(8d0*x**3+21d0*x**2+15d0*x+8d0)/x*h0)
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REAL)
       res=res+CA**2*(67d0/9d0-two*zeta2)/(one-x)
       res=res+CA*nfTF*(-20d0/9d0)/(one-x)
    CASE(cc_PLUS0)
       res=CA**2*(67d0/9d0-two*zeta2)
       res=res+CA*nfTF*(-20d0/9d0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x.AND.cc_piece.NE.cc_PLUS0)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1gg

  ! Time like
  ! P^{(1)}_{qg}
  FUNCTION AP_SFT_P1qg(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2,S3,S4
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    ! 4*H_0*H_1-8H_{0,1}+H_1^2
    S3=hp1**2+four*hp1*h0-8d0*DDILOG(x)
    ! 6*H_0*H_1+H_1^2-8H_{0,1}
    S4=6d0*h0*hp1+hp1**2-8d0*DDILOG(x)
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CA*TF*(-four*zeta2*x+(178d0*x**3-95d0*x**2+13d0*x-20d0)/9d0/x&
            -two*(two*x**2+two*x+one)*S2+(two*x**2-two*x+one)*S3&
            +(one+6d0*x)*h0**2-(10d0*x**2-10d0*x+11d0)/three*hp1&
            -(four*x**2+34d0*x+four)/three*h0)&
            +CF*TF*(-two*zeta2*(two*x**2-two*x+one)-(10d0*x**2-23d0/two*x+6d0)&
            -(two*x**2-two*x+one)*S4-(two*x**2-x+half)*h0**2&
            +(two*x**2-two*x+three)*hp1+(two*x**2+two*x-5d0/two)*h0)
       res=res+nfTF*TF*(-four/9d0*(four*x**2-four*x+5d0)&
            +four/three*(two*x**2-two*x+one)*(hp1-h0))
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1qg

  ! Time like
  ! P^{(1)}_{gq}
  FUNCTION AP_SFT_P1gq(y,y_is_x) RESULT(res)
    USE special_functions_2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x
    REAL(KIND(1d0))::h0,hp1,hm1,S2,S3,S4
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN

    h0=DLOG(x)
    hp1=-DLOG(one-x)
    hm1=DLOG(one+x)
    ! H_0*H_{-1}-H_{0,-1}
    S2=h0*hm1+DDILOG(-x)
    ! 6*H_0*H_1+H_1^2-8*H_{0,1}
    S3=6d0*h0*hp1+hp1**2-8d0*DDILOG(x)
    ! 4*H_0*H_1+H_1^2-8*H_{0,1}
    S4=four*h0*hp1+hp1**2-8d0*DDILOG(x)
    SELECT CASE(cc_piece)
    CASE(cc_REAL,cc_REALVIRT)
       res=CA*CF*(four*zeta2-(44d0*x**3+9d0*x**2-45d0*x-17d0)/9d0/x&
            +two*(x**2+two*x+two)/x*S2-(x**2-two*x+two)/x*S3&
            -(three*x**2+two*x+four)/x*h0**2+two*x*hp1&
            +(8d0/three*x**3+9d0*x**2+8d0*x-6d0)/x*h0)&
            +CF**2*(9d0/two*x-half+(x**2-two*x+two)/x*S4+(two-x)/two*h0**2&
            -two*x*hp1+(half*x-8d0)*h0)
    END SELECT
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P1gq

  !--------------------------------------------------!
  ! NNLO or three-loop  space like                   !
  !--------------------------------------------------!

  ! Space like
  ! P^{s,(2)}_+=P^{s,(2)}_{qq}+P^{s,(2)}_{q\bar{q}}
  ! Eq.(4.12) in hep-ph/0404111
  ! Expressions are copyed from subroutine X2PSA in xpij2e.f times 2
  ! by A. Vogt, S. Moch and J. Vermaseren, hep-ph/0404111
  FUNCTION AP_SFS_P2PSplus(y,y_is_x) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
    REAL(KIND(1d0))::x,DX
    REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
    REAL(KIND(1d0))::gqqps2,twoTf,twonfTf
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

    DX=one/x
    twoTf=two*TF
    twonfTf=two*nfTF
    CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
         HI1,HI2,HI3,HI4, N1, N2)

    gqqps2 = &
         TF*CF*CA*(484.D0/3.D0 - 2528.D0/3.D0*x + 82232.D0/81.D0*&
         x**2 - 27044.D0/81.D0*dx - 16.D0/3.D0*z3 + 248.D0/3.D0*z3*x&
         -416.D0/3.D0*z3*x**2 + 96.D0*z3*dx - 1916.D0/9.D0*z2 + 904.D0&
         /9.D0*z2*x + 448.D0/9.D0*z2*x**2 + 512.D0/9.D0*z2*dx + 428.D0&
         /5.D0*z2**2 + 468.D0/5.D0*z2**2*x + 40.D0*Hr1(-1)*z2 + 40.D0*&
         Hr1(-1)*z2*x + 32.D0/3.D0*Hr1(-1)*z2*x**2 + 32.D0/3.D0*Hr1(-1)&
         *z2*dx - 14260.D0/27.D0*Hr1(0) - 13216.D0/27.D0*Hr1(0)*x -&
         18392.D0/27.D0*Hr1(0)*x**2 - 896.D0/27.D0*Hr1(0)*dx + 208.D0*&
         Hr1(0)*z3 + 144.D0*Hr1(0)*z3*x - 236.D0/3.D0*Hr1(0)*z2 + 28.D0&
         /3.D0*Hr1(0)*z2*x - 32.D0/3.D0*Hr1(0)*z2*x**2 + 1748.D0/9.D0*&
         Hr1(1) - 2912.D0/9.D0*Hr1(1)*x + 1208.D0/27.D0*Hr1(1)*x**2 +&
         2284.D0/27.D0*Hr1(1)*dx + 8.D0*Hr1(1)*z2 - 8.D0*Hr1(1)*z2*x&
         -32.D0/3.D0*Hr1(1)*z2*x**2 + 32.D0/3.D0*Hr1(1)*z2*dx + 256.D0&
         /3.D0*Hr2(-1,0) + 544.D0/3.D0*Hr2(-1,0)*x + 1696.D0/9.D0*&
         Hr2(-1,0)*x**2 + 832.D0/9.D0*Hr2(-1,0)*dx + 16.D0*Hr2(0,-1)*&
         z2 )

    gqqps2 = gqqps2 + TF*CF*CA*(-16.D0*Hr2(0,-1)*z2*x + 1076.D0/&
         9.D0*Hr2(0,0) - 1588.D0/9.D0*Hr2(0,0)*x + 664.D0/3.D0*Hr2(0,0)&
         *x**2 + 8.D0*Hr2(0,0)*z2 - 8.D0*Hr2(0,0)*z2*x + 1916.D0/9.D0&
         *Hr2(0,1) + 728.D0/9.D0*Hr2(0,1)*x - 448.D0/9.D0*Hr2(0,1)*&
         x**2 + 320.D0/9.D0*Hr2(0,1)*dx + 16.D0*Hr2(0,1)*z2 + 16.D0*&
         Hr2(0,1)*z2*x + 104.D0/3.D0*Hr2(1,0) + 40.D0/3.D0*Hr2(1,0)*x&
         -72.D0*Hr2(1,0)*x**2 + 24.D0*Hr2(1,0)*dx - 12.D0*Hr2(1,1)&
         +12.D0*Hr2(1,1)*x + 16.D0/9.D0*Hr2(1,1)*x**2 - 16.D0/9.D0*&
         Hr2(1,1)*dx + 16.D0*Hr3(-1,-1,0) + 16.D0*Hr3(-1,-1,0)*x - 64.D0&
         /3.D0*Hr3(-1,-1,0)*x**2 - 64.D0/3.D0*Hr3(-1,-1,0)*dx + 8.D0*&
         Hr3(-1,0,0) + 8.D0*Hr3(-1,0,0)*x - 128.D0/3.D0*Hr3(-1,0,0)*&
         x**2 - 128.D0/3.D0*Hr3(-1,0,0)*dx - 32.D0*Hr3(-1,0,1) - 32.D0&
         *Hr3(-1,0,1)*x - 64.D0/3.D0*Hr3(-1,0,1)*x**2 - 64.D0/3.D0*&
         Hr3(-1,0,1)*dx - 24.D0*Hr3(0,-1,0) + 104.D0*Hr3(0,-1,0)*x -&
         64.D0*Hr3(0,-1,0)*x**2 - 232.D0/3.D0*Hr3(0,0,0) + 248.D0/3.D0&
         *Hr3(0,0,0)*x )

    gqqps2 = gqqps2 + TF*CF*CA*(236.D0/3.D0*Hr3(0,0,1) + 284.D0/3.D0&
         *Hr3(0,0,1)*x + 32.D0/3.D0*Hr3(0,0,1)*x**2 + 16.D0*Hr3(0,1,0)&
         + 40.D0*Hr3(0,1,0)*x + 32.D0/3.D0*Hr3(0,1,0)*x**2 - 88.D0/3.D0&
         *Hr3(0,1,1) - 136.D0/3.D0*Hr3(0,1,1)*x - 32.D0/3.D0*Hr3(0,1,1)&
         *x**2 + 32.D0/3.D0*Hr3(0,1,1)*dx - 36.D0*Hr3(1,0,0) + 36.D0&
         *Hr3(1,0,0)*x + 32.D0*Hr3(1,0,0)*x**2 - 32.D0*Hr3(1,0,0)*dx&
         +16.D0*Hr3(1,1,0) - 16.D0*Hr3(1,1,0)*x - 64.D0/3.D0*Hr3(1,1,0)&
         *x**2 + 64.D0/3.D0*Hr3(1,1,0)*dx + 16.D0*Hr3(1,1,1) - 16.D0&
         *Hr3(1,1,1)*x - 64.D0/3.D0*Hr3(1,1,1)*x**2 + 64.D0/3.D0*Hr3(1,1,1)&
         *dx + 32.D0*Hr4(0,-1,-1,0) - 32.D0*Hr4(0,-1,-1,0)*x + 48.D0&
         *Hr4(0,-1,0,0) - 48.D0*Hr4(0,-1,0,0)*x + 80.D0*Hr4(0,0,-1,0)&
         - 16.D0*Hr4(0,0,-1,0)*x + 64.D0*Hr4(0,0,0,0) - 48.D0*Hr4(0,0,0,0)&
         *x - 8.D0*Hr4(0,0,0,1) - 8.D0*Hr4(0,0,0,1)*x + 16.D0*&
         Hr4(0,0,1,1) + 16.D0*Hr4(0,0,1,1)*x - 56.D0*Hr4(0,1,0,0) - 56.D0&
         *Hr4(0,1,0,0)*x + 32.D0*Hr4(0,1,1,0) + 32.D0*Hr4(0,1,1,0)*x&
         +32.D0*Hr4(0,1,1,1) )

    gqqps2 = gqqps2 + TF*CF*CA* (32.D0*Hr4(0,1,1,1)*x)

    gqqps2 = gqqps2 + TF*CF**2*( 2530.D0/27.D0 + 3338.D0/27.D0*x -&
         872.D0/3.D0*x**2 + 220.D0/3.D0*dx + 160.D0*z3*x + 256.D0/3.D0&
         *z3*x**2 - 64.D0*z3*dx + 272.D0*z2 + 20.D0*z2*x + 448.D0/9.D0&
         *z2*x**2 - 296.D0/5.D0*z2**2 - 296.D0/5.D0*z2**2*x + 1510.D0/&
         9.D0*Hr1(0) + 5486.D0/9.D0*Hr1(0)*x - 104.D0/27.D0*Hr1(0)*&
         x**2 - 112.D0*Hr1(0)*z3 - 112.D0*Hr1(0)*z3*x + 32.D0*Hr1(0)*&
         z2 - 48.D0*Hr1(0)*z2*x - 256.D0/3.D0*Hr1(0)*z2*x**2 - 664.D0/&
         3.D0*Hr1(1) + 1004.D0/3.D0*Hr1(1)*x - 968.D0/27.D0*Hr1(1)*&
         x**2 - 2092.D0/27.D0*Hr1(1)*dx - 512.D0/3.D0*Hr2(0,0) + 212.D0&
         /3.D0*Hr2(0,0)*x - 352.D0/3.D0*Hr2(0,0)*x**2 + 96.D0*Hr2(0,0)&
         *z2 + 96.D0*Hr2(0,0)*z2*x - 272.D0*Hr2(0,1) - 20.D0*Hr2(0,1)*&
         x - 448.D0/9.D0*Hr2(0,1)*x**2 - 128.D0/3.D0*Hr2(1,0) + 272.D0/&
         3.D0*Hr2(1,0)*x - 800.D0/9.D0*Hr2(1,0)*x**2 + 368.D0/9.D0*&
         Hr2(1,0)*dx + 28.D0/3.D0*Hr2(1,1) - 28.D0/3.D0*Hr2(1,1)*x -&
         64.D0/3.D0*Hr2(1,1)*x**2 + 64.D0/3.D0*Hr2(1,1)*dx + 24.D0*&
         Hr3(0,0,0) )

    gqqps2 = gqqps2 + TF*CF**2*( 40.D0*Hr3(0,0,0)*x + 352.D0/3.D0*&
         Hr3(0,0,0)*x**2 - 32.D0*Hr3(0,0,1) + 48.D0*Hr3(0,0,1)*x + 256.D0&
         /3.D0*Hr3(0,0,1)*x**2 + 32.D0*Hr3(0,1,0) + 112.D0*Hr3(0,1,0)&
         *x + 64.D0*Hr3(0,1,0)*x**2 + 48.D0*Hr3(0,1,1) + 64.D0*Hr3(0,1,1)&
         *x + 64.D0/3.D0*Hr3(0,1,1)*x**2 + 8.D0*Hr3(1,0,0) - 8.D0*&
         Hr3(1,0,0)*x - 32.D0/3.D0*Hr3(1,0,0)*x**2 + 32.D0/3.D0*Hr3(1,0,0)&
         *dx - 16.D0*Hr3(1,1,0) + 16.D0*Hr3(1,1,0)*x + 64.D0/3.D0*&
         Hr3(1,1,0)*x**2 - 64.D0/3.D0*Hr3(1,1,0)*dx - 16.D0*Hr3(1,1,1)&
         +16.D0*Hr3(1,1,1)*x + 64.D0/3.D0*Hr3(1,1,1)*x**2 - 64.D0/3.D0&
         *Hr3(1,1,1)*dx - 64.D0*Hr4(0,0,0,0) - 64.D0*Hr4(0,0,0,0)*x&
         -96.D0*Hr4(0,0,0,1) - 96.D0*Hr4(0,0,0,1)*x - 64.D0*Hr4(0,0,1,0)&
         - 64.D0*Hr4(0,0,1,0)*x - 16.D0*Hr4(0,0,1,1) - 16.D0*Hr4(0,0,1,1)&
         *x + 16.D0*Hr4(0,1,0,0) + 16.D0*Hr4(0,1,0,0)*x - 32.D0&
         *Hr4(0,1,1,0) - 32.D0*Hr4(0,1,1,0)*x - 32.D0*Hr4(0,1,1,1) -&
         32.D0*Hr4(0,1,1,1)*x )

    gqqps2 = gqqps2 + TF*twonfTF*CF*(616.D0/27.D0 - 1480.D0/27.D0*x +&
         800.D0/27.D0*x**2 + 64.D0/27.D0*dx + 16.D0/3.D0*z3 + 16.D0/3.D0&
         *z3*x + 80.D0/9.D0*z2 - 64.D0/9.D0*z2*x - 32.D0/3.D0*z2*x**2&
         +592.D0/27.D0*Hr1(0) - 560.D0/27.D0*Hr1(0)*x - 304.D0/9.D0*&
         Hr1(0)*x**2 + 32.D0/3.D0*Hr1(0)*z2 + 32.D0/3.D0*Hr1(0)*z2*x&
         -56.D0/3.D0*Hr1(1) + 104.D0/3.D0*Hr1(1)*x - 592.D0/27.D0*&
         Hr1(1)*x**2 + 160.D0/27.D0*Hr1(1)*dx + 232.D0/9.D0*Hr2(0,0)&
         +232.D0/9.D0*Hr2(0,0)*x - 80.D0/9.D0*Hr2(0,1) + 64.D0/9.D0*&
         Hr2(0,1)*x + 32.D0/3.D0*Hr2(0,1)*x**2 + 8.D0/3.D0*Hr2(1,1) -&
         8.D0/3.D0*Hr2(1,1)*x - 32.D0/9.D0*Hr2(1,1)*x**2 + 32.D0/9.D0*&
         Hr2(1,1)*dx + 16.D0/3.D0*Hr3(0,0,0) + 16.D0/3.D0*Hr3(0,0,0)*x&
         -32.D0/3.D0*Hr3(0,0,1) - 32.D0/3.D0*Hr3(0,0,1)*x + 16.D0/3.D0&
         *Hr3(0,1,1) + 16.D0/3.D0*Hr3(0,1,1)*x )

    res=two*gqqps2/8d0
    
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P2PSplus

   ! Space Like
   ! P^{(2)}_{ns+}=P^{v,(2)}_{qq}+P^{v,(2)}_{qqbar}
   ! Eq.(4.9) in hep-ph/06403192
   ! Expressions are copyed from subroutine X2NSPA and X2NSC in xpns2e.f
   ! by S. Moch, J. Vermaseren and A. Vogt, hep-ph/0403192
   FUNCTION AP_SFS_P2NSplus(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,DM,DP
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4,Z5=zeta5
     REAL(KIND(1d0))::gqq2,twoTf,twonfTf,A3,GQQ2L,P2DELT
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       twoTf=two*TF
       twonfTf=two*nfTF
       P2DELT =&
            29.D0/2.D0*CF**3&
            +151.D0/4.D0*CA*CF**2&
            -1657.D0/36.D0*CA**2*CF&
            -240.D0*z5*CF**3&
            +120.D0*z5*CA*CF**2&
            +40.D0*z5*CA**2*CF&
            +68.D0*z3*CF**3&
            +844.D0/3.D0*z3*CA*CF**2&
            -1552.D0/9.D0*z3*CA**2*CF&
            +18.D0*z2*CF**3&
            -410.D0/3.D0*z2*CA*CF**2&
            +4496.D0/27.D0*z2*CA**2*CF&
            -32.D0*z2*z3*CF**3&
            +16.D0*z2*z3*CA*CF**2&
            +288.D0/5.D0*z2**2*CF**3&
            -988.D0/15.D0*z2**2*CA*CF**2&
            -2.D0*z2**2*CA**2*CF&
            -1336.D0/27.D0*z2*CA*CF*twonfTF&
            +4.D0/5.D0*z2**2*CA*CF*twonfTF&
            +200.D0/9.D0*z3*CA*CF*twonfTF&
            +20.D0*CA*CF*twonfTF&
            +20.D0/3.D0*z2*CF**2*twonfTF&
            +232.D0/15.D0*z2**2*CF**2*twonfTF&
            -136.D0/3.D0*z3*CF**2*twonfTF&
            -23.D0*CF**2*twonfTF&
            +80.D0/27.D0*z2*CF*twonfTF**2&
            -16.D0/9.D0*z3*CF*twonfTF**2&
            -17.D0/9.D0*CF*twonfTF**2

       res=P2DELT/8d0
       RETURN
    ENDIF
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT.AND.cc_piece.NE.cc_PLUS0)RETURN

     DX=one/x
     DM=one/(one-x)
     DP=one/(one+x)
     twoTf=two*TF
     twonfTf=two*nfTF

    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT,cc_REAL)

       CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
            HI1,HI2,HI3,HI4, N1, N2)

       gqq2 =&
            CF*CA**2*(5327.D0/27.D0 - 9737.D0/27.D0*x + 490.D0/3.D0*dm&
            -224.D0*z3*x - 88.D0*z3*dp + 16.D0*z3*dm - 112.D0*z2 + 448.D0&
            /9.D0*z2*x + 1072.D0/9.D0*z2*dp - 1072.D0/9.D0*z2*dm - 62.D0/&
            5.D0*z2**2 - 242.D0/5.D0*z2**2*x - 32.D0*z2**2*dp + 384.D0/5.D0&
            *z2**2*dm - 192.D0*Hr1(-1)*z3 + 192.D0*Hr1(-1)*z3*x + 384.D0&
            *Hr1(-1)*z3*dp + 208.D0/3.D0*Hr1(-1)*z2 + 560.D0/3.D0*Hr1(-1)&
            *z2*x + 352.D0/3.D0*Hr1(-1)*z2*dp + 410.D0/27.D0*Hr1(0) -&
            8686.D0/27.D0*Hr1(0)*x - 24.D0*Hr1(0)*dp + 4172.D0/27.D0*Hr1(0)&
            *dm - 144.D0*Hr1(0)*z3*x - 128.D0*Hr1(0)*z3*dp + 128.D0*&
            Hr1(0)*z3*dm - 4.D0*Hr1(0)*z2 - 148.D0/3.D0*Hr1(0)*z2*x - 16.D0&
            /3.D0*Hr1(0)*z2*dp - 248.D0/3.D0*Hr1(0)*z2*dm + 176.D0*Hr1(1)&
            - 176.D0*Hr1(1)*x - 144.D0*Hr1(1)*z3 - 144.D0*Hr1(1)*z3*x&
            + 288.D0*Hr1(1)*z3*dm + 32.D0*Hr1(1)*z2 - 32.D0*Hr1(1)*z2*x&
            + 256.D0*Hr2(-1,-1)*z2 - 256.D0*Hr2(-1,-1)*z2*x - 512.D0*&
            Hr2(-1,-1)*z2*dp - 688.D0/9.D0*Hr2(-1,0) + 1456.D0/9.D0*Hr2(-1,0)*x )

       gqq2 = gqq2 + CF*CA**2*(2144.D0/9.D0*Hr2(-1,0)*dp - 176.D0*&
            Hr2(-1,0)*z2 + 176.D0*Hr2(-1,0)*z2*x + 352.D0*Hr2(-1,0)*z2*dp&
            -136.D0*Hr2(0,-1)*z2 + 136.D0*Hr2(0,-1)*z2*x + 256.D0*Hr2(0,-1)&
            *z2*dp - 242.D0/9.D0*Hr2(0,0) - 230.D0/3.D0*Hr2(0,0)*x -&
            1072.D0/9.D0*Hr2(0,0)*dp + 1556.D0/9.D0*Hr2(0,0)*dm + 36.D0*&
            Hr2(0,0)*z2 - 68.D0*Hr2(0,0)*z2*x - 96.D0*Hr2(0,0)*z2*dp +&
            112.D0*Hr2(0,1) + 112.D0*Hr2(0,1)*x - 40.D0*Hr2(0,1)*z2 + 24.D0&
            *Hr2(0,1)*z2*x + 64.D0*Hr2(0,1)*z2*dp + 16.D0*Hr2(1,0)*z2 +&
            16.D0*Hr2(1,0)*z2*x - 32.D0*Hr2(1,0)*z2*dm + 64.D0*Hr3(-1,-1,0)&
            + 64.D0*Hr3(-1,-1,0)*x - 328.D0/3.D0*Hr3(-1,0,0) - 152.D0/&
            3.D0*Hr3(-1,0,0)*x + 176.D0/3.D0*Hr3(-1,0,0)*dp - 112.D0/3.D0&
            *Hr3(-1,0,1) - 464.D0/3.D0*Hr3(-1,0,1)*x - 352.D0/3.D0*Hr3(-1,0,1)&
            *dp - 88.D0/3.D0*Hr3(0,-1,0) + 40.D0/3.D0*Hr3(0,-1,0)*x&
            + 176.D0/3.D0*Hr3(0,-1,0)*dp - 48.D0*Hr3(0,-1,0)*dm - 128.D0/&
            3.D0*Hr3(0,0,0)*x - 248.D0/3.D0*Hr3(0,0,0)*dp + 248.D0/3.D0*&
            Hr3(0,0,0)*dm )

       gqq2 = gqq2 + CF*CA**2*(4.D0*Hr3(0,0,1) + 188.D0/3.D0*Hr3(0,0,1)&
            *x + 176.D0/3.D0*Hr3(0,0,1)*dp + 88.D0/3.D0*Hr3(0,0,1)*dm&
            +12.D0*Hr3(1,0,0) + 76.D0*Hr3(1,0,0)*x - 88.D0*Hr3(1,0,0)*&
            dm - 128.D0*Hr4(-1,-1,0,0) + 128.D0*Hr4(-1,-1,0,0)*x + 256.D0&
            *Hr4(-1,-1,0,0)*dp - 256.D0*Hr4(-1,-1,0,1) + 256.D0*Hr4(-1,-1,0,1)&
            *x + 512.D0*Hr4(-1,-1,0,1)*dp + 48.D0*Hr4(-1,0,0,0) - 48.D0&
            *Hr4(-1,0,0,0)*x - 96.D0*Hr4(-1,0,0,0)*dp + 128.D0*Hr4(-1,0,0,1)&
            - 128.D0*Hr4(-1,0,0,1)*x - 256.D0*Hr4(-1,0,0,1)*dp - 80.D0&
            *Hr4(0,-1,-1,0) - 48.D0*Hr4(0,-1,-1,0)*x + 128.D0*Hr4(0,-1,-1,0)&
            *dm + 88.D0*Hr4(0,-1,0,0) - 56.D0*Hr4(0,-1,0,0)*x - 128.D0&
            *Hr4(0,-1,0,0)*dp - 32.D0*Hr4(0,-1,0,0)*dm + 96.D0*Hr4(0,-1,0,1)&
            - 160.D0*Hr4(0,-1,0,1)*x - 256.D0*Hr4(0,-1,0,1)*dp + 64.D0&
            *Hr4(0,-1,0,1)*dm + 8.D0*Hr4(0,0,-1,0) - 40.D0*Hr4(0,0,-1,0)&
            *x - 32.D0*Hr4(0,0,-1,0)*dp + 32.D0*Hr4(0,0,-1,0)*dm + 40.D0*&
            Hr4(0,0,0,0)*x + 32.D0*Hr4(0,0,0,0)*dp - 32.D0*Hr4(0,0,0,0)*&
            dm )

       gqq2 = gqq2 + CF*CA**2*(-36.D0*Hr4(0,0,0,1) + 28.D0*Hr4(0,0,0,1)&
            *x + 64.D0*Hr4(0,0,0,1)*dp + 32.D0*Hr4(0,0,0,1)*dm + 28.D0&
            *Hr4(0,1,0,0) + 28.D0*Hr4(0,1,0,0)*x - 64.D0*Hr4(0,1,0,0)*dm&
            -96.D0*Hr4(1,0,-1,0) - 96.D0*Hr4(1,0,-1,0)*x + 192.D0*Hr4(1,0,-1,0)&
            *dm + 48.D0*Hr4(1,0,0,0) + 48.D0*Hr4(1,0,0,0)*x - 96.D0&
            *Hr4(1,0,0,0)*dm - 64.D0*Hr4(1,0,0,1) - 64.D0*Hr4(1,0,0,1)*x&
            +128.D0*Hr4(1,0,0,1)*dm + 64.D0*Hr4(1,1,0,0) + 64.D0*Hr4(1,1,0,0)&
            *x - 128.D0*Hr4(1,1,0,0)*dm )

       gqq2 = gqq2 + CF**2*CA*(532.D0/9.D0 - 532.D0/9.D0*x - 16.D0/3.D0&
            *z3 + 2336.D0/3.D0*z3*x + 248.D0*z3*dp + 80.D0/3.D0*z3*dm +&
            3448.D0/9.D0*z2 + 2024.D0/9.D0*z2*x - 2144.D0/9.D0*z2*dp - 24.D0&
            /5.D0*z2**2 + 56.D0/5.D0*z2**2*x + 8.D0*z2**2*dp - 552.D0/5.D0&
            *z2**2*dm + 672.D0*Hr1(-1)*z3 - 672.D0*Hr1(-1)*z3*x - 1344.D0&
            *Hr1(-1)*z3*dp - 992.D0/3.D0*Hr1(-1)*z2 - 1984.D0/3.D0*Hr1(-1)&
            *z2*x - 992.D0/3.D0*Hr1(-1)*z2*dp + 628.D0/9.D0*Hr1(0) +&
            572.D0*Hr1(0)*x + 72.D0*Hr1(0)*dp - 302.D0/3.D0*Hr1(0)*dm -&
            144.D0*Hr1(0)*z3 + 400.D0*Hr1(0)*z3*x + 464.D0*Hr1(0)*z3*dp&
            -272.D0*Hr1(0)*z3*dm + 72.D0*Hr1(0)*z2 + 1208.D0/3.D0*Hr1(0)&
            *z2*x + 104.D0/3.D0*Hr1(0)*z2*dp + 328.D0/3.D0*Hr1(0)*z2*dm&
            -1672.D0/3.D0*Hr1(1) + 1672.D0/3.D0*Hr1(1)*x + 384.D0*Hr1(1)&
            *z3 + 384.D0*Hr1(1)*z3*x - 768.D0*Hr1(1)*z3*dm - 112.D0*Hr1(1)&
            *z2 + 112.D0*Hr1(1)*z2*x - 896.D0*Hr2(-1,-1)*z2 + 896.D0*&
            Hr2(-1,-1)*z2*x + 1792.D0*Hr2(-1,-1)*z2*dp + 1520.D0/9.D0*&
            Hr2(-1,0) )

       gqq2 = gqq2 + CF**2*CA*(-2768.D0/9.D0*Hr2(-1,0)*x - 4288.D0/&
            9.D0*Hr2(-1,0)*dp + 672.D0*Hr2(-1,0)*z2 - 672.D0*Hr2(-1,0)*z2&
            *x - 1344.D0*Hr2(-1,0)*z2*dp + 576.D0*Hr2(0,-1)*z2 - 480.D0*&
            Hr2(0,-1)*z2*x - 1024.D0*Hr2(0,-1)*z2*dp - 96.D0*Hr2(0,-1)*z2&
            *dm - 88.D0/3.D0*Hr2(0,0) - 2560.D0/9.D0*Hr2(0,0)*x + 2144.D0/&
            9.D0*Hr2(0,0)*dp - 104.D0*Hr2(0,0)*dm - 96.D0*Hr2(0,0)*z2 +&
            352.D0*Hr2(0,0)*z2*x + 416.D0*Hr2(0,0)*z2*dp - 128.D0*Hr2(0,0)&
            *z2*dm - 3448.D0/9.D0*Hr2(0,1) - 4792.D0/9.D0*Hr2(0,1)*x +&
            2144.D0/9.D0*Hr2(0,1)*dm + 160.D0*Hr2(0,1)*z2 - 64.D0*Hr2(0,1)&
            *z2*x - 224.D0*Hr2(0,1)*z2*dp - 64.D0*Hr2(0,1)*z2*dm - 400.D0&
            /9.D0*Hr2(1,0) - 1744.D0/9.D0*Hr2(1,0)*x + 2144.D0/9.D0*Hr2(1,0)&
            *dm + 32.D0*Hr2(1,0)*z2 + 32.D0*Hr2(1,0)*z2*x - 64.D0*Hr2(1,0)&
            *z2*dm - 224.D0*Hr3(-1,-1,0) - 224.D0*Hr3(-1,-1,0)*x +&
            1352.D0/3.D0*Hr3(-1,0,0) + 856.D0/3.D0*Hr3(-1,0,0)*x - 496.D0/&
            3.D0*Hr3(-1,0,0)*dp + 656.D0/3.D0*Hr3(-1,0,1) + 1648.D0/3.D0*&
            Hr3(-1,0,1)*x )

       gqq2 = gqq2 + CF**2*CA*(992.D0/3.D0*Hr3(-1,0,1)*dp + 368.D0/3.D0&
            *Hr3(0,-1,0) + 208.D0/3.D0*Hr3(0,-1,0)*x - 496.D0/3.D0*Hr3(0,-1,0)&
            *dp + 96.D0*Hr3(0,-1,0)*dm - 132.D0*Hr3(0,0,0) - 476.D0/&
            3.D0*Hr3(0,0,0)*x + 712.D0/3.D0*Hr3(0,0,0)*dp - 184.D0/3.D0*&
            Hr3(0,0,0)*dm - 72.D0*Hr3(0,0,1) - 1000.D0/3.D0*Hr3(0,0,1)*x&
            -496.D0/3.D0*Hr3(0,0,1)*dp + 64.D0/3.D0*Hr3(0,0,1)*dm - 176.D0&
            /3.D0*Hr3(0,1,0) - 176.D0/3.D0*Hr3(0,1,0)*x + 352.D0/3.D0*&
            Hr3(0,1,0)*dm - 304.D0/3.D0*Hr3(1,0,0) - 688.D0/3.D0*Hr3(1,0,0)&
            *x + 992.D0/3.D0*Hr3(1,0,0)*dm + 576.D0*Hr4(-1,-1,0,0) -&
            576.D0*Hr4(-1,-1,0,0)*x - 1152.D0*Hr4(-1,-1,0,0)*dp + 896.D0*&
            Hr4(-1,-1,0,1) - 896.D0*Hr4(-1,-1,0,1)*x - 1792.D0*Hr4(-1,-1,0,1)&
            *dp + 64.D0*Hr4(-1,0,-1,0) - 64.D0*Hr4(-1,0,-1,0)*x - 128.D0&
            *Hr4(-1,0,-1,0)*dp - 272.D0*Hr4(-1,0,0,0) + 272.D0*Hr4(-1,0,0,0)&
            *x + 544.D0*Hr4(-1,0,0,0)*dp - 512.D0*Hr4(-1,0,0,1) +&
            512.D0*Hr4(-1,0,0,1)*x + 1024.D0*Hr4(-1,0,0,1)*dp - 32.D0*&
            Hr4(-1,0,1,0) )

       gqq2 = gqq2 + CF**2*CA*(32.D0*Hr4(-1,0,1,0)*x + 64.D0*Hr4(-1,0,1,0)&
            *dp + 320.D0*Hr4(0,-1,-1,0) + 128.D0*Hr4(0,-1,-1,0)*x -&
            128.D0*Hr4(0,-1,-1,0)*dp - 448.D0*Hr4(0,-1,-1,0)*dm - 464.D0*&
            Hr4(0,-1,0,0) + 304.D0*Hr4(0,-1,0,0)*x + 672.D0*Hr4(0,-1,0,0)&
            *dp + 160.D0*Hr4(0,-1,0,0)*dm - 416.D0*Hr4(0,-1,0,1) + 544.D0&
            *Hr4(0,-1,0,1)*x + 960.D0*Hr4(0,-1,0,1)*dp - 128.D0*Hr4(0,-1,0,1)&
            *dm - 128.D0*Hr4(0,0,-1,0) + 128.D0*Hr4(0,0,-1,0)*x + 160.D0&
            *Hr4(0,0,-1,0)*dp - 32.D0*Hr4(0,0,-1,0)*dm - 224.D0*Hr4(0,0,0,0)&
            *x - 160.D0*Hr4(0,0,0,0)*dp + 160.D0*Hr4(0,0,0,0)*dm +&
            96.D0*Hr4(0,0,0,1) - 224.D0*Hr4(0,0,0,1)*x - 320.D0*Hr4(0,0,0,1)&
            *dp + 32.D0*Hr4(0,0,0,1)*dm - 32.D0*Hr4(0,0,1,0)*x - 32.D0&
            *Hr4(0,0,1,0)*dp + 32.D0*Hr4(0,0,1,0)*dm - 48.D0*Hr4(0,1,0,0)&
            -48.D0*Hr4(0,1,0,0)*x + 160.D0*Hr4(0,1,0,0)*dm + 256.D0*&
            Hr4(1,0,-1,0) + 256.D0*Hr4(1,0,-1,0)*x - 512.D0*Hr4(1,0,-1,0)&
            *dm - 176.D0*Hr4(1,0,0,0) - 176.D0*Hr4(1,0,0,0)*x + 352.D0*&
            Hr4(1,0,0,0)*dm )

       gqq2 = gqq2 + CF**2*CA*(128.D0*Hr4(1,0,0,1) + 128.D0*Hr4(1,0,0,1)&
            *x - 256.D0*Hr4(1,0,0,1)*dm - 128.D0*Hr4(1,1,0,0) - 128.D0&
            *Hr4(1,1,0,0)*x + 256.D0*Hr4(1,1,0,0)*dm )

       gqq2 = gqq2 + CF**3*(-62.D0+62.D0*x - 48.D0*z3 - 720.D0*z3&
            *x - 144.D0*z3*dp - 308.D0*z2 - 372.D0*z2*x - 56.D0/5.D0*&
            z2**2 + 504.D0/5.D0*z2**2*x + 112.D0*z2**2*dp + 144.D0/5.D0*&
            z2**2*dm - 576.D0*Hr1(-1)*z3 + 576.D0*Hr1(-1)*z3*x + 1152.D0*&
            Hr1(-1)*z3*dp + 384.D0*Hr1(-1)*z2 + 576.D0*Hr1(-1)*z2*x + 192.D0&
            *Hr1(-1)*z2*dp + 24.D0*Hr1(0) - 560.D0*Hr1(0)*x - 48.D0*&
            Hr1(0)*dp - 6.D0*Hr1(0)*dm + 288.D0*Hr1(0)*z3 - 224.D0*Hr1(0)&
            *z3*x - 416.D0*Hr1(0)*z3*dp + 32.D0*Hr1(0)*z3*dm - 96.D0*Hr1(0)&
            *z2 - 448.D0*Hr1(0)*z2*x - 48.D0*Hr1(0)*z2*dp - 48.D0*Hr1(0)&
            *z2*dm + 560.D0*Hr1(1) - 560.D0*Hr1(1)*x - 192.D0*Hr1(1)*z3&
            -192.D0*Hr1(1)*z3*x + 384.D0*Hr1(1)*z3*dm + 96.D0*Hr1(1)*z2&
            -96.D0*Hr1(1)*z2*x + 768.D0*Hr2(-1,-1)*z2 - 768.D0*Hr2(-1,-1)&
            *z2*x - 1536.D0*Hr2(-1,-1)*z2*dp - 32.D0*Hr2(-1,0) - 32.D0&
            *Hr2(-1,0)*x - 640.D0*Hr2(-1,0)*z2 + 640.D0*Hr2(-1,0)*z2*x +&
            1280.D0*Hr2(-1,0)*z2*dp - 608.D0*Hr2(0,-1)*z2 + 416.D0*Hr2(0,-1)*z2*x )

       gqq2 = gqq2 + CF**3*(1024.D0*Hr2(0,-1)*z2*dp + 192.D0*Hr2(0,-1)&
            *z2*dm - 44.D0*Hr2(0,0) + 356.D0*Hr2(0,0)*x + 52.D0*Hr2(0,0)&
            *dm + 240.D0*Hr2(0,0)*z2 - 240.D0*Hr2(0,0)*z2*x - 448.D0*Hr2(0,0)&
            *z2*dp + 308.D0*Hr2(0,1) + 340.D0*Hr2(0,1)*x - 96.D0*Hr2(0,1)&
            *z2 + 96.D0*Hr2(0,1)*z2*x + 192.D0*Hr2(0,1)*z2*dp - 16.D0&
            *Hr2(1,0) + 16.D0*Hr2(1,0)*x + 192.D0*Hr3(-1,-1,0) + 192.D0*&
            Hr3(-1,-1,0)*x - 464.D0*Hr3(-1,0,0) - 368.D0*Hr3(-1,0,0)*x +&
            96.D0*Hr3(-1,0,0)*dp - 288.D0*Hr3(-1,0,1) - 480.D0*Hr3(-1,0,1)&
            *x - 192.D0*Hr3(-1,0,1)*dp - 128.D0*Hr3(0,-1,0) - 192.D0*&
            Hr3(0,-1,0)*x + 96.D0*Hr3(0,-1,0)*dp + 120.D0*Hr3(0,0,0) +&
            248.D0*Hr3(0,0,0)*x - 144.D0*Hr3(0,0,0)*dp + 96.D0*Hr3(0,0,1)&
            +256.D0*Hr3(0,0,1)*x + 96.D0*Hr3(0,0,1)*dp + 72.D0*Hr3(0,1,0)&
            + 8.D0*Hr3(0,1,0)*x - 96.D0*Hr3(0,1,0)*dm + 96.D0*Hr3(1,0,0)&
            + 96.D0*Hr3(1,0,0)*x - 192.D0*Hr3(1,0,0)*dm - 640.D0*Hr4(-1,-1,0,0)&
            + 640.D0*Hr4(-1,-1,0,0)*x + 1280.D0*Hr4(-1,-1,0,0)&
            *dp )

       gqq2 = gqq2 + CF**3*(-768.D0*Hr4(-1,-1,0,1) + 768.D0*Hr4(-1,-1,0,1)&
            *x + 1536.D0*Hr4(-1,-1,0,1)*dp - 128.D0*Hr4(-1,0,-1,0)&
            +128.D0*Hr4(-1,0,-1,0)*x + 256.D0*Hr4(-1,0,-1,0)*dp + 352.D0&
            *Hr4(-1,0,0,0) - 352.D0*Hr4(-1,0,0,0)*x - 704.D0*Hr4(-1,0,0,0)&
            *dp + 512.D0*Hr4(-1,0,0,1) - 512.D0*Hr4(-1,0,0,1)*x - 1024.D0&
            *Hr4(-1,0,0,1)*dp + 64.D0*Hr4(-1,0,1,0) - 64.D0*Hr4(-1,0,1,0)&
            *x - 128.D0*Hr4(-1,0,1,0)*dp - 320.D0*Hr4(0,-1,-1,0) - 64.D0*&
            Hr4(0,-1,-1,0)*x + 256.D0*Hr4(0,-1,-1,0)*dp + 384.D0*Hr4(0,-1,-1,0)&
            *dm + 576.D0*Hr4(0,-1,0,0) - 384.D0*Hr4(0,-1,0,0)*x -&
            832.D0*Hr4(0,-1,0,0)*dp - 192.D0*Hr4(0,-1,0,0)*dm + 448.D0*&
            Hr4(0,-1,0,1) - 448.D0*Hr4(0,-1,0,1)*x - 896.D0*Hr4(0,-1,0,1)&
            *dp + 224.D0*Hr4(0,0,-1,0) - 96.D0*Hr4(0,0,-1,0)*x - 192.D0*&
            Hr4(0,0,-1,0)*dp - 64.D0*Hr4(0,0,-1,0)*dm - 112.D0*Hr4(0,0,0,0)&
            + 176.D0*Hr4(0,0,0,0)*x + 192.D0*Hr4(0,0,0,0)*dp - 64.D0*&
            Hr4(0,0,0,0)*dm - 240.D0*Hr4(0,0,0,1) + 144.D0*Hr4(0,0,0,1)*x&
            +384.D0*Hr4(0,0,0,1)*dp )

       gqq2 = gqq2 + CF**3*(64.D0*Hr4(0,0,0,1)*dm - 128.D0*Hr4(0,0,1,0)&
            - 64.D0*Hr4(0,0,1,0)*x + 64.D0*Hr4(0,0,1,0)*dp + 128.D0*&
            Hr4(0,0,1,0)*dm - 64.D0*Hr4(0,0,1,1) - 64.D0*Hr4(0,0,1,1)*x&
            +128.D0*Hr4(0,0,1,1)*dm - 80.D0*Hr4(0,1,0,0) - 80.D0*Hr4(0,1,0,0)&
            *x + 64.D0*Hr4(0,1,0,0)*dm - 64.D0*Hr4(0,1,0,1) - 64.D0&
            *Hr4(0,1,0,1)*x + 128.D0*Hr4(0,1,0,1)*dm - 64.D0*Hr4(0,1,1,0)&
            -64.D0*Hr4(0,1,1,0)*x + 128.D0*Hr4(0,1,1,0)*dm - 128.D0*&
            Hr4(1,0,-1,0) - 128.D0*Hr4(1,0,-1,0)*x + 256.D0*Hr4(1,0,-1,0)&
            *dm + 64.D0*Hr4(1,0,0,0) + 64.D0*Hr4(1,0,0,0)*x - 128.D0*Hr4(1,0,0,0)&
            *dm - 128.D0*Hr4(1,0,0,1) - 128.D0*Hr4(1,0,0,1)*x +&
            256.D0*Hr4(1,0,0,1)*dm - 64.D0*Hr4(1,0,1,0) - 64.D0*Hr4(1,0,1,0)&
            *x + 128.D0*Hr4(1,0,1,0)*dm )


       gqq2 = gqq2 +&
            twonfTF*CF*CA*(-182.D0/3.D0 + 160.D0/9.D0*z2*dm - 160.D0/9.D0&
            *z2*dp - 184.D0/9.D0*z2*x + 8.D0*z2 - 48.D0*z3*dm + 16.D0*z3*&
            dp + 32.D0*z3*x + 16.D0*z3 - 836.D0/27.D0*dm + 2474.D0/27.D0*&
            x - 64.D0/3.D0*Hr1(-1)*z2*dp - 32.D0/3.D0*Hr1(-1)*z2*x + 32.D0&
            /3.D0*Hr1(-1)*z2 + 68.D0/27.D0*Hr1(0) + 32.D0/3.D0*Hr1(0)*z2*&
            dm + 16.D0/3.D0*Hr1(0)*z2*dp - 8.D0/3.D0*Hr1(0)*z2*x - 8.D0*&
            Hr1(0)*z2 - 1336.D0/27.D0*Hr1(0)*dm + 1700.D0/27.D0*Hr1(0)*x&
            -16.D0*Hr1(1) + 16.D0*Hr1(1)*x + 64.D0/9.D0*Hr2(-1,0) - 320.D0&
            /9.D0*Hr2(-1,0)*dp - 256.D0/9.D0*Hr2(-1,0)*x + 88.D0/9.D0*&
            Hr2(0,0) - 112.D0/3.D0*Hr2(0,0)*dm + 160.D0/9.D0*Hr2(0,0)*dp&
            +272.D0/9.D0*Hr2(0,0)*x - 8.D0*Hr2(0,1) - 8.D0*Hr2(0,1)*x&
            +16.D0/3.D0*Hr3(-1,0,0) - 32.D0/3.D0*Hr3(-1,0,0)*dp - 16.D0/&
            3.D0*Hr3(-1,0,0)*x - 32.D0/3.D0*Hr3(-1,0,1) + 64.D0/3.D0*Hr3(-1,0,1)&
            *dp + 32.D0/3.D0*Hr3(-1,0,1)*x + 16.D0/3.D0*Hr3(0,-1,0)&
            - 32.D0/3.D0*Hr3(0,-1,0)*dp - 16.D0/3.D0*Hr3(0,-1,0)*x - 32.D0&
            /3.D0*Hr3(0,0,0)*dm )

       gqq2 = gqq2 + twonfTF*CF*CA*(32.D0/3.D0*Hr3(0,0,0)*dp + 32.D0/3.D0*&
            Hr3(0,0,0)*x + 8.D0*Hr3(0,0,1) - 16.D0/3.D0*Hr3(0,0,1)*dm -&
            32.D0/3.D0*Hr3(0,0,1)*dp - 8.D0/3.D0*Hr3(0,0,1)*x - 8.D0*Hr3(1,0,0)&
            + 16.D0*Hr3(1,0,0)*dm - 8.D0*Hr3(1,0,0)*x )

       gqq2 = gqq2 + twonfTF*CF**2*(5.D0/9.D0 + 320.D0/9.D0*z2*dp + 160.D0/&
            9.D0*z2*x - 160.D0/9.D0*z2 + 160.D0/3.D0*z3*dm - 32.D0*z3*dp&
            -128.D0/3.D0*z3*x - 32.D0/3.D0*z3 - 110.D0/3.D0*dm + 325.D0/&
            9.D0*x + 128.D0/3.D0*Hr1(-1)*z2*dp + 64.D0/3.D0*Hr1(-1)*z2*x&
            -64.D0/3.D0*Hr1(-1)*z2 - 148.D0/9.D0*Hr1(0) + 32.D0/3.D0*&
            Hr1(0)*z2*dm - 32.D0/3.D0*Hr1(0)*z2*dp - 32.D0/3.D0*Hr1(0)*z2&
            *x + 20.D0/3.D0*Hr1(0)*dm - 20.D0*Hr1(0)*x + 64.D0/3.D0*Hr1(1)&
            - 64.D0/3.D0*Hr1(1)*x - 128.D0/9.D0*Hr2(-1,0) + 640.D0/9.D0&
            *Hr2(-1,0)*dp + 512.D0/9.D0*Hr2(-1,0)*x + 16.D0/3.D0*Hr2(0,0)&
            +16.D0*Hr2(0,0)*dm - 320.D0/9.D0*Hr2(0,0)*dp - 80.D0/9.D0*&
            Hr2(0,0)*x + 160.D0/9.D0*Hr2(0,1) - 320.D0/9.D0*Hr2(0,1)*dm&
            +352.D0/9.D0*Hr2(0,1)*x + 64.D0/9.D0*Hr2(1,0) - 320.D0/9.D0&
            *Hr2(1,0)*dm + 256.D0/9.D0*Hr2(1,0)*x - 32.D0/3.D0*Hr3(-1,0,0)&
            + 64.D0/3.D0*Hr3(-1,0,0)*dp + 32.D0/3.D0*Hr3(-1,0,0)*x + 64.D0&
            /3.D0*Hr3(-1,0,1) - 128.D0/3.D0*Hr3(-1,0,1)*dp - 64.D0/3.D0&
            *Hr3(-1,0,1)*x )

       gqq2 = gqq2 + twonfTF*CF**2*(-32.D0/3.D0*Hr3(0,-1,0) + 64.D0/3.D0&
            *Hr3(0,-1,0)*dp + 32.D0/3.D0*Hr3(0,-1,0)*x + 24.D0*Hr3(0,0,0)&
            -32.D0/3.D0*Hr3(0,0,0)*dm - 64.D0/3.D0*Hr3(0,0,0)*dp + 8.D0/&
            3.D0*Hr3(0,0,0)*x - 64.D0/3.D0*Hr3(0,0,1)*dm + 64.D0/3.D0*&
            Hr3(0,0,1)*dp + 64.D0/3.D0*Hr3(0,0,1)*x + 32.D0/3.D0*Hr3(0,1,0)&
            - 64.D0/3.D0*Hr3(0,1,0)*dm + 32.D0/3.D0*Hr3(0,1,0)*x + 64.D0&
            /3.D0*Hr3(1,0,0) - 128.D0/3.D0*Hr3(1,0,0)*dm + 64.D0/3.D0*&
            Hr3(1,0,0)*x )

       gqq2 = gqq2 + twonfTF**2*CF*(112.D0/27.D0 - 16.D0/27.D0*dm - 32.D0/&
            9.D0*x + 8.D0/27.D0*Hr1(0) + 80.D0/27.D0*Hr1(0)*dm - 88.D0/27.D0&
            *Hr1(0)*x - 8.D0/9.D0*Hr2(0,0) + 16.D0/9.D0*Hr2(0,0)*dm - 8.D0&
            /9.D0*Hr2(0,0)*x )

       res=gqq2/8d0

    END SELECT

    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT,cc_PLUS0)
       !
       ! ...The soft (`+'-distribution) part of the splitting function
       !
       A3 =&
            CA**2*CF*(490.D0/3.D0 + 88.D0/3.D0*z3 - 1072.D0/9.D0*z2&
            +176.D0/5.D0*z2**2 )&
            +CA*CF*twonfTF*(-836.D0/27.D0 + 160.D0/9.D0*z2 - 112.D0/3.D0*z3 )&
            +CF**2*twonfTF*(-110.D0/3.D0 + 32.D0*z3 )-CF*twonfTF**2*16.D0/27.D0

       IF(cc_piece.EQ.cc_PLUS0)THEN
          res=A3/8d0
       ELSEIF(cc_piece.EQ.cc_REALVIRT)THEN
          GQQ2L = DM * A3
          res=res-GQQ2L/8d0
       ENDIF
    END SELECT


    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
   END FUNCTION AP_SFS_P2NSplus

   ! Space Like
   ! P^{(2)}_{ns-}=P^{v,(2)}_{qq}-P^{v,(2)}_{qqbar}
   ! Eq.(4.10) in hep-ph/06403192
   ! Expressions are copyed from subroutine X2NSMA and X2NSC in xpns2e.f
   ! by S. Moch, J. Vermaseren and A. Vogt, hep-ph/0403192
   FUNCTION AP_SFS_P2NSminus(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,DM,DP
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4,Z5=zeta5
     REAL(KIND(1d0))::gqq2,twoTf,twonfTf,A3,GQQ2L,P2DELT
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero

   IF(cc_piece.EQ.cc_Delta)THEN
       twoTf=two*TF
       twonfTf=two*nfTF
       P2DELT =&
            29.D0/2.D0*CF**3&
            +151.D0/4.D0*CA*CF**2&
            -1657.D0/36.D0*CA**2*CF&
            -240.D0*z5*CF**3&
            +120.D0*z5*CA*CF**2&
            +40.D0*z5*CA**2*CF&
            +68.D0*z3*CF**3&
            +844.D0/3.D0*z3*CA*CF**2&
            -1552.D0/9.D0*z3*CA**2*CF&
            +18.D0*z2*CF**3&
            -410.D0/3.D0*z2*CA*CF**2&
            +4496.D0/27.D0*z2*CA**2*CF&
            -32.D0*z2*z3*CF**3&
            +16.D0*z2*z3*CA*CF**2&
            +288.D0/5.D0*z2**2*CF**3&
            -988.D0/15.D0*z2**2*CA*CF**2&
            -2.D0*z2**2*CA**2*CF&
            -1336.D0/27.D0*z2*CA*CF*twonfTF&
            +4.D0/5.D0*z2**2*CA*CF*twonfTF&
            +200.D0/9.D0*z3*CA*CF*twonfTF&
            +20.D0*CA*CF*twonfTF&
            +20.D0/3.D0*z2*CF**2*twonfTF&
            +232.D0/15.D0*z2**2*CF**2*twonfTF&
            -136.D0/3.D0*z3*CF**2*twonfTF&
            -23.D0*CF**2*twonfTF&
            +80.D0/27.D0*z2*CF*twonfTF**2&
            -16.D0/9.D0*z3*CF*twonfTF**2&
            -17.D0/9.D0*CF*twonfTF**2

       res=P2DELT/8d0
       RETURN
    ENDIF
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT.AND.cc_piece.NE.cc_PLUS0)RETURN

     DX=one/x
     DM=one/(one-x)
     DP=one/(one+x)
     twoTf=two*TF
     twonfTf=two*nfTF

    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT,cc_REAL)

       CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
            HI1,HI2,HI3,HI4, N1, N2)

       gqq2 =  CF*CA**2*(923.D0/27.D0 - 5333.D0/27.D0*x + 490.D0&
            /3.D0*dm + 112.D0*z3 + 48.D0*z3*x + 88.D0*z3*dp + 16.D0*z3*dm&
            + 1504.D0/9.D0*z2 + 224.D0/3.D0*z2*x - 1072.D0/9.D0*z2*dp -&
            1072.D0/9.D0*z2*dm - 242.D0/5.D0*z2**2 - 62.D0/5.D0*z2**2*x&
            +32.D0*z2**2*dp + 384.D0/5.D0*z2**2*dm + 192.D0*Hr1(-1)*z3&
            -192.D0*Hr1(-1)*z3*x - 384.D0*Hr1(-1)*z3*dp - 208.D0/3.D0*&
            Hr1(-1)*z2 - 560.D0/3.D0*Hr1(-1)*z2*x - 352.D0/3.D0*Hr1(-1)*&
            z2*dp - 1378.D0/27.D0*Hr1(0) - 2266.D0/27.D0*Hr1(0)*x + 24.D0&
            *Hr1(0)*dp + 4172.D0/27.D0*Hr1(0)*dm - 144.D0*Hr1(0)*z3 + 128.D0&
            *Hr1(0)*z3*dp + 128.D0*Hr1(0)*z3*dm + 332.D0/3.D0*Hr1(0)*z2&
            +188.D0*Hr1(0)*z2*x + 16.D0/3.D0*Hr1(0)*z2*dp - 248.D0/3.D0&
            *Hr1(0)*z2*dm - 592.D0/3.D0*Hr1(1) + 592.D0/3.D0*Hr1(1)*x -&
            144.D0*Hr1(1)*z3 - 144.D0*Hr1(1)*z3*x + 288.D0*Hr1(1)*z3*dm&
            -32.D0*Hr1(1)*z2 + 32.D0*Hr1(1)*z2*x - 256.D0*Hr2(-1,-1)*z2&
            +256.D0*Hr2(-1,-1)*z2*x + 512.D0*Hr2(-1,-1)*z2*dp + 832.D0/&
            9.D0*Hr2(-1,0) )

       gqq2 = gqq2 + CF*CA**2*(-1312.D0/9.D0*Hr2(-1,0)*x - 2144.D0/&
            9.D0*Hr2(-1,0)*dp + 176.D0*Hr2(-1,0)*z2 - 176.D0*Hr2(-1,0)*z2&
            *x - 352.D0*Hr2(-1,0)*z2*dp + 136.D0*Hr2(0,-1)*z2 - 136.D0*&
            Hr2(0,-1)*z2*x - 256.D0*Hr2(0,-1)*z2*dp - 130.D0*Hr2(0,0) +&
            238.D0/9.D0*Hr2(0,0)*x + 1072.D0/9.D0*Hr2(0,0)*dp + 1556.D0/9.D0&
            *Hr2(0,0)*dm - 68.D0*Hr2(0,0)*z2 + 36.D0*Hr2(0,0)*z2*x + 96.D0&
            *Hr2(0,0)*z2*dp - 224.D0/3.D0*Hr2(0,1) - 224.D0/3.D0*Hr2(0,1)&
            *x + 24.D0*Hr2(0,1)*z2 - 40.D0*Hr2(0,1)*z2*x - 64.D0*Hr2(0,1)&
            *z2*dp + 16.D0*Hr2(1,0)*z2 + 16.D0*Hr2(1,0)*z2*x - 32.D0*&
            Hr2(1,0)*z2*dm + 64.D0*Hr3(-1,-1,0) + 64.D0*Hr3(-1,-1,0)*x +&
            232.D0/3.D0*Hr3(-1,0,0) + 56.D0/3.D0*Hr3(-1,0,0)*x - 176.D0/3.D0&
            *Hr3(-1,0,0)*dp + 304.D0/3.D0*Hr3(-1,0,1) + 656.D0/3.D0*&
            Hr3(-1,0,1)*x + 352.D0/3.D0*Hr3(-1,0,1)*dp + 328.D0/3.D0*Hr3(0,-1,0)&
            - 376.D0/3.D0*Hr3(0,-1,0)*x - 176.D0/3.D0*Hr3(0,-1,0)&
            *dp - 48.D0*Hr3(0,-1,0)*dm - 416.D0/3.D0*Hr3(0,0,0) + 248.D0/&
            3.D0*Hr3(0,0,0)*dp )

       gqq2 = gqq2 + CF*CA**2*(248.D0/3.D0*Hr3(0,0,0)*dm - 4.D0/3.D0*&
            Hr3(0,0,1) - 188.D0*Hr3(0,0,1)*x - 176.D0/3.D0*Hr3(0,0,1)*dp&
            +88.D0/3.D0*Hr3(0,0,1)*dm + 12.D0*Hr3(1,0,0) + 76.D0*Hr3(1,0,0)&
            *x - 88.D0*Hr3(1,0,0)*dm + 128.D0*Hr4(-1,-1,0,0) - 128.D0&
            *Hr4(-1,-1,0,0)*x - 256.D0*Hr4(-1,-1,0,0)*dp + 256.D0*Hr4(-1,-1,0,1)&
            - 256.D0*Hr4(-1,-1,0,1)*x - 512.D0*Hr4(-1,-1,0,1)*dp&
            -48.D0*Hr4(-1,0,0,0) + 48.D0*Hr4(-1,0,0,0)*x + 96.D0*Hr4(-1,0,0,0)&
            *dp - 128.D0*Hr4(-1,0,0,1) + 128.D0*Hr4(-1,0,0,1)*x +&
            256.D0*Hr4(-1,0,0,1)*dp - 48.D0*Hr4(0,-1,-1,0) - 80.D0*Hr4(0,-1,-1,0)&
            *x + 128.D0*Hr4(0,-1,-1,0)*dm - 56.D0*Hr4(0,-1,0,0)&
            +88.D0*Hr4(0,-1,0,0)*x + 128.D0*Hr4(0,-1,0,0)*dp - 32.D0*&
            Hr4(0,-1,0,0)*dm - 160.D0*Hr4(0,-1,0,1) + 96.D0*Hr4(0,-1,0,1)&
            *x + 256.D0*Hr4(0,-1,0,1)*dp + 64.D0*Hr4(0,-1,0,1)*dm - 40.D0&
            *Hr4(0,0,-1,0) + 8.D0*Hr4(0,0,-1,0)*x + 32.D0*Hr4(0,0,-1,0)*&
            dp + 32.D0*Hr4(0,0,-1,0)*dm + 40.D0*Hr4(0,0,0,0) - 32.D0*Hr4(0,0,0,0)*dp )

       gqq2 = gqq2 + CF*CA**2*(-32.D0*Hr4(0,0,0,0)*dm + 28.D0*Hr4(0,0,0,1)&
            - 36.D0*Hr4(0,0,0,1)*x - 64.D0*Hr4(0,0,0,1)*dp + 32.D0&
            *Hr4(0,0,0,1)*dm + 28.D0*Hr4(0,1,0,0) + 28.D0*Hr4(0,1,0,0)*x&
            -64.D0*Hr4(0,1,0,0)*dm - 96.D0*Hr4(1,0,-1,0) - 96.D0*Hr4(1,0,-1,0)&
            *x + 192.D0*Hr4(1,0,-1,0)*dm + 48.D0*Hr4(1,0,0,0) + 48.D0&
            *Hr4(1,0,0,0)*x - 96.D0*Hr4(1,0,0,0)*dm - 64.D0*Hr4(1,0,0,1)&
            - 64.D0*Hr4(1,0,0,1)*x + 128.D0*Hr4(1,0,0,1)*dm + 64.D0*&
            Hr4(1,1,0,0) + 64.D0*Hr4(1,1,0,0)*x - 128.D0*Hr4(1,1,0,0)*dm)

       gqq2 = gqq2 + CF**2*CA*(1516.D0/3.D0 - 1516.D0/3.D0*x - 832.D0/&
            3.D0*z3 - 880.D0/3.D0*z3*x - 248.D0*z3*dp + 80.D0/3.D0*z3*dm&
            -3880.D0/9.D0*z2 - 152.D0/9.D0*z2*x + 2144.D0/9.D0*z2*dp +&
            56.D0/5.D0*z2**2 - 24.D0/5.D0*z2**2*x - 8.D0*z2**2*dp - 552.D0&
            /5.D0*z2**2*dm - 672.D0*Hr1(-1)*z3 + 672.D0*Hr1(-1)*z3*x +&
            1344.D0*Hr1(-1)*z3*dp + 704.D0/3.D0*Hr1(-1)*z2 + 1696.D0/3.D0&
            *Hr1(-1)*z2*x + 992.D0/3.D0*Hr1(-1)*z2*dp + 3404.D0/9.D0*Hr1(0)&
            - 3740.D0/9.D0*Hr1(0)*x - 72.D0*Hr1(0)*dp - 302.D0/3.D0*&
            Hr1(0)*dm + 400.D0*Hr1(0)*z3 - 144.D0*Hr1(0)*z3*x - 464.D0*&
            Hr1(0)*z3*dp - 272.D0*Hr1(0)*z3*dm - 904.D0/3.D0*Hr1(0)*z2 -&
            568.D0*Hr1(0)*z2*x - 104.D0/3.D0*Hr1(0)*z2*dp + 328.D0/3.D0*&
            Hr1(0)*z2*dm + 2008.D0/3.D0*Hr1(1) - 2008.D0/3.D0*Hr1(1)*x +&
            384.D0*Hr1(1)*z3 + 384.D0*Hr1(1)*z3*x - 768.D0*Hr1(1)*z3*dm&
            +112.D0*Hr1(1)*z2 - 112.D0*Hr1(1)*z2*x + 896.D0*Hr2(-1,-1)*&
            z2 - 896.D0*Hr2(-1,-1)*z2*x - 1792.D0*Hr2(-1,-1)*z2*dp - 1232.D0&
            /9.D0*Hr2(-1,0) )

       gqq2 = gqq2 + CF**2*CA*(3056.D0/9.D0*Hr2(-1,0)*x + 4288.D0/9.D0&
            *Hr2(-1,0)*dp - 672.D0*Hr2(-1,0)*z2 + 672.D0*Hr2(-1,0)*z2*x&
            +1344.D0*Hr2(-1,0)*z2*dp - 480.D0*Hr2(0,-1)*z2 + 576.D0*&
            Hr2(0,-1)*z2*x + 1024.D0*Hr2(0,-1)*z2*dp - 96.D0*Hr2(0,-1)*z2&
            *dm + 2168.D0/9.D0*Hr2(0,0) - 944.D0/3.D0*Hr2(0,0)*x - 2144.D0&
            /9.D0*Hr2(0,0)*dp - 104.D0*Hr2(0,0)*dm + 352.D0*Hr2(0,0)*z2&
            -96.D0*Hr2(0,0)*z2*x - 416.D0*Hr2(0,0)*z2*dp - 128.D0*Hr2(0,0)&
            *z2*dm + 2648.D0/9.D0*Hr2(0,1) + 152.D0/9.D0*Hr2(0,1)*x +&
            2144.D0/9.D0*Hr2(0,1)*dm - 64.D0*Hr2(0,1)*z2 + 160.D0*Hr2(0,1)&
            *z2*x + 224.D0*Hr2(0,1)*z2*dp - 64.D0*Hr2(0,1)*z2*dm + 176.D0&
            /9.D0*Hr2(1,0) - 2320.D0/9.D0*Hr2(1,0)*x + 2144.D0/9.D0*Hr2(1,0)&
            *dm + 32.D0*Hr2(1,0)*z2 + 32.D0*Hr2(1,0)*z2*x - 64.D0*Hr2(1,0)&
            *z2*dm - 224.D0*Hr3(-1,-1,0) - 224.D0*Hr3(-1,-1,0)*x -&
            872.D0/3.D0*Hr3(-1,0,0) - 376.D0/3.D0*Hr3(-1,0,0)*x + 496.D0/&
            3.D0*Hr3(-1,0,0)*dp - 1040.D0/3.D0*Hr3(-1,0,1) - 2032.D0/3.D0&
            *Hr3(-1,0,1)*x )

       gqq2 = gqq2 + CF**2*CA*(-992.D0/3.D0*Hr3(-1,0,1)*dp - 752.D0/&
            3.D0*Hr3(0,-1,0) + 944.D0/3.D0*Hr3(0,-1,0)*x + 496.D0/3.D0*&
            Hr3(0,-1,0)*dp + 96.D0*Hr3(0,-1,0)*dm + 868.D0/3.D0*Hr3(0,0,0)&
            - 36.D0*Hr3(0,0,0)*x - 712.D0/3.D0*Hr3(0,0,0)*dp - 184.D0/3.D0&
            *Hr3(0,0,0)*dm + 152.D0/3.D0*Hr3(0,0,1) + 568.D0*Hr3(0,0,1)&
            *x + 496.D0/3.D0*Hr3(0,0,1)*dp + 64.D0/3.D0*Hr3(0,0,1)*dm -&
            80.D0/3.D0*Hr3(0,1,0) - 80.D0/3.D0*Hr3(0,1,0)*x + 352.D0/3.D0&
            *Hr3(0,1,0)*dm - 304.D0/3.D0*Hr3(1,0,0) - 688.D0/3.D0*Hr3(1,0,0)&
            *x + 992.D0/3.D0*Hr3(1,0,0)*dm - 576.D0*Hr4(-1,-1,0,0) +&
            576.D0*Hr4(-1,-1,0,0)*x + 1152.D0*Hr4(-1,-1,0,0)*dp - 896.D0*&
            Hr4(-1,-1,0,1) + 896.D0*Hr4(-1,-1,0,1)*x + 1792.D0*Hr4(-1,-1,0,1)&
            *dp - 64.D0*Hr4(-1,0,-1,0) + 64.D0*Hr4(-1,0,-1,0)*x + 128.D0&
            *Hr4(-1,0,-1,0)*dp + 272.D0*Hr4(-1,0,0,0) - 272.D0*Hr4(-1,0,0,0)&
            *x - 544.D0*Hr4(-1,0,0,0)*dp + 512.D0*Hr4(-1,0,0,1) -&
            512.D0*Hr4(-1,0,0,1)*x - 1024.D0*Hr4(-1,0,0,1)*dp + 32.D0*&
            Hr4(-1,0,1,0) )

       gqq2 = gqq2 + CF**2*CA*(-32.D0*Hr4(-1,0,1,0)*x - 64.D0*Hr4(-1,0,1,0)&
            *dp + 128.D0*Hr4(0,-1,-1,0) + 320.D0*Hr4(0,-1,-1,0)*&
            x + 128.D0*Hr4(0,-1,-1,0)*dp - 448.D0*Hr4(0,-1,-1,0)*dm + 304.D0&
            *Hr4(0,-1,0,0) - 464.D0*Hr4(0,-1,0,0)*x - 672.D0*Hr4(0,-1,0,0)&
            *dp + 160.D0*Hr4(0,-1,0,0)*dm + 544.D0*Hr4(0,-1,0,1) - 416.D0&
            *Hr4(0,-1,0,1)*x - 960.D0*Hr4(0,-1,0,1)*dp - 128.D0*Hr4(0,-1,0,1)&
            *dm + 128.D0*Hr4(0,0,-1,0) - 128.D0*Hr4(0,0,-1,0)*x -&
            160.D0*Hr4(0,0,-1,0)*dp - 32.D0*Hr4(0,0,-1,0)*dm - 224.D0*&
            Hr4(0,0,0,0) + 160.D0*Hr4(0,0,0,0)*dp + 160.D0*Hr4(0,0,0,0)*&
            dm - 224.D0*Hr4(0,0,0,1) + 96.D0*Hr4(0,0,0,1)*x + 320.D0*Hr4(0,0,0,1)&
            *dp + 32.D0*Hr4(0,0,0,1)*dm - 32.D0*Hr4(0,0,1,0) + 32.D0&
            *Hr4(0,0,1,0)*dp + 32.D0*Hr4(0,0,1,0)*dm - 48.D0*Hr4(0,1,0,0)&
            - 48.D0*Hr4(0,1,0,0)*x + 160.D0*Hr4(0,1,0,0)*dm + 256.D0*&
            Hr4(1,0,-1,0) + 256.D0*Hr4(1,0,-1,0)*x - 512.D0*Hr4(1,0,-1,0)&
            *dm - 176.D0*Hr4(1,0,0,0) - 176.D0*Hr4(1,0,0,0)*x + 352.D0*&
            Hr4(1,0,0,0)*dm )

       gqq2 = gqq2 + CF**2*CA*(128.D0*Hr4(1,0,0,1) + 128.D0*Hr4(1,0,0,1)&
            *x - 256.D0*Hr4(1,0,0,1)*dm - 128.D0*Hr4(1,1,0,0) - 128.D0&
            *Hr4(1,1,0,0)*x + 256.D0*Hr4(1,1,0,0)*dm )

       gqq2 = gqq2 + CF**3*(-302.D0 + 302.D0*x + 48.D0*z3 + 336.D0*&
            z3*x + 144.D0*z3*dp + 204.D0*z2 + 12.D0*z2*x + 504.D0/5.D0*&
            z2**2 - 56.D0/5.D0*z2**2*x - 112.D0*z2**2*dp + 144.D0/5.D0*&
            z2**2*dm + 576.D0*Hr1(-1)*z3 - 576.D0*Hr1(-1)*z3*x - 1152.D0*&
            Hr1(-1)*z3*dp - 192.D0*Hr1(-1)*z2 - 384.D0*Hr1(-1)*z2*x - 192.D0&
            *Hr1(-1)*z2*dp - 328.D0*Hr1(0) + 464.D0*Hr1(0)*x + 48.D0*&
            Hr1(0)*dp - 6.D0*Hr1(0)*dm - 224.D0*Hr1(0)*z3 + 288.D0*Hr1(0)&
            *z3*x + 416.D0*Hr1(0)*z3*dp + 32.D0*Hr1(0)*z3*dm + 192.D0*&
            Hr1(0)*z2 + 544.D0*Hr1(0)*z2*x + 48.D0*Hr1(0)*z2*dp - 48.D0*&
            Hr1(0)*z2*dm - 400.D0*Hr1(1) + 400.D0*Hr1(1)*x - 192.D0*Hr1(1)&
            *z3 - 192.D0*Hr1(1)*z3*x + 384.D0*Hr1(1)*z3*dm - 96.D0*Hr1(1)&
            *z2 + 96.D0*Hr1(1)*z2*x - 768.D0*Hr2(-1,-1)*z2 + 768.D0*Hr2(-1,-1)&
            *z2*x + 1536.D0*Hr2(-1,-1)*z2*dp - 96.D0*Hr2(-1,0) - 96.D0&
            *Hr2(-1,0)*x + 640.D0*Hr2(-1,0)*z2 - 640.D0*Hr2(-1,0)*z2*x&
            -1280.D0*Hr2(-1,0)*z2*dp + 416.D0*Hr2(0,-1)*z2 - 608.D0*&
            Hr2(0,-1)*z2*x )

       gqq2 = gqq2 + CF**3*(-1024.D0*Hr2(0,-1)*z2*dp + 192.D0*Hr2(0,-1)&
            *z2*dm - 172.D0*Hr2(0,0) + 4.D0*Hr2(0,0)*x + 52.D0*Hr2(0,0)&
            *dm - 240.D0*Hr2(0,0)*z2 + 240.D0*Hr2(0,0)*z2*x + 448.D0*&
            Hr2(0,0)*z2*dp - 300.D0*Hr2(0,1) - 12.D0*Hr2(0,1)*x + 96.D0*&
            Hr2(0,1)*z2 - 96.D0*Hr2(0,1)*z2*x - 192.D0*Hr2(0,1)*z2*dp -&
            144.D0*Hr2(1,0) + 144.D0*Hr2(1,0)*x + 192.D0*Hr3(-1,-1,0) +&
            192.D0*Hr3(-1,-1,0)*x + 272.D0*Hr3(-1,0,0) + 176.D0*Hr3(-1,0,0)&
            *x - 96.D0*Hr3(-1,0,0)*dp + 288.D0*Hr3(-1,0,1) + 480.D0*&
            Hr3(-1,0,1)*x + 192.D0*Hr3(-1,0,1)*dp + 64.D0*Hr3(0,-1,0) -&
            128.D0*Hr3(0,-1,0)*x - 96.D0*Hr3(0,-1,0)*dp - 168.D0*Hr3(0,0,0)&
            - 168.D0*Hr3(0,0,0)*x + 144.D0*Hr3(0,0,0)*dp - 128.D0*Hr3(0,0,1)&
            - 544.D0*Hr3(0,0,1)*x - 96.D0*Hr3(0,0,1)*dp + 8.D0*&
            Hr3(0,1,0) - 56.D0*Hr3(0,1,0)*x - 96.D0*Hr3(0,1,0)*dm + 96.D0&
            *Hr3(1,0,0) + 96.D0*Hr3(1,0,0)*x - 192.D0*Hr3(1,0,0)*dm + 640.D0&
            *Hr4(-1,-1,0,0) - 640.D0*Hr4(-1,-1,0,0)*x - 1280.D0*Hr4(-1,-1,0,0)*dp )

       gqq2 = gqq2 + CF**3*(768.D0*Hr4(-1,-1,0,1) - 768.D0*Hr4(-1,-1,0,1)&
            *x - 1536.D0*Hr4(-1,-1,0,1)*dp + 128.D0*Hr4(-1,0,-1,0) -&
            128.D0*Hr4(-1,0,-1,0)*x - 256.D0*Hr4(-1,0,-1,0)*dp - 352.D0*&
            Hr4(-1,0,0,0) + 352.D0*Hr4(-1,0,0,0)*x + 704.D0*Hr4(-1,0,0,0)&
            *dp - 512.D0*Hr4(-1,0,0,1) + 512.D0*Hr4(-1,0,0,1)*x + 1024.D0&
            *Hr4(-1,0,0,1)*dp - 64.D0*Hr4(-1,0,1,0) + 64.D0*Hr4(-1,0,1,0)&
            *x + 128.D0*Hr4(-1,0,1,0)*dp - 64.D0*Hr4(0,-1,-1,0) - 320.D0*&
            Hr4(0,-1,-1,0)*x - 256.D0*Hr4(0,-1,-1,0)*dp + 384.D0*Hr4(0,-1,-1,0)&
            *dm - 384.D0*Hr4(0,-1,0,0) + 576.D0*Hr4(0,-1,0,0)*x +&
            832.D0*Hr4(0,-1,0,0)*dp - 192.D0*Hr4(0,-1,0,0)*dm - 448.D0*&
            Hr4(0,-1,0,1) + 448.D0*Hr4(0,-1,0,1)*x + 896.D0*Hr4(0,-1,0,1)&
            *dp - 96.D0*Hr4(0,0,-1,0) + 224.D0*Hr4(0,0,-1,0)*x + 192.D0*&
            Hr4(0,0,-1,0)*dp - 64.D0*Hr4(0,0,-1,0)*dm + 176.D0*Hr4(0,0,0,0)&
            - 112.D0*Hr4(0,0,0,0)*x - 192.D0*Hr4(0,0,0,0)*dp - 64.D0*&
            Hr4(0,0,0,0)*dm + 144.D0*Hr4(0,0,0,1) - 240.D0*Hr4(0,0,0,1)*x&
            -384.D0*Hr4(0,0,0,1)*dp )

       gqq2 = gqq2 + CF**3*(64.D0*Hr4(0,0,0,1)*dm - 64.D0*Hr4(0,0,1,0)&
            - 128.D0*Hr4(0,0,1,0)*x - 64.D0*Hr4(0,0,1,0)*dp + 128.D0*&
            Hr4(0,0,1,0)*dm - 64.D0*Hr4(0,0,1,1) - 64.D0*Hr4(0,0,1,1)*x&
            +128.D0*Hr4(0,0,1,1)*dm - 80.D0*Hr4(0,1,0,0) - 80.D0*Hr4(0,1,0,0)&
            *x + 64.D0*Hr4(0,1,0,0)*dm - 64.D0*Hr4(0,1,0,1) - 64.D0&
            *Hr4(0,1,0,1)*x + 128.D0*Hr4(0,1,0,1)*dm - 64.D0*Hr4(0,1,1,0)&
            -64.D0*Hr4(0,1,1,0)*x + 128.D0*Hr4(0,1,1,0)*dm - 128.D0*&
            Hr4(1,0,-1,0) - 128.D0*Hr4(1,0,-1,0)*x + 256.D0*Hr4(1,0,-1,0)&
            *dm + 64.D0*Hr4(1,0,0,0) + 64.D0*Hr4(1,0,0,0)*x - 128.D0*Hr4(1,0,0,0)&
            *dm - 128.D0*Hr4(1,0,0,1) - 128.D0*Hr4(1,0,0,1)*x +&
            256.D0*Hr4(1,0,0,1)*dm - 64.D0*Hr4(1,0,1,0) - 64.D0*Hr4(1,0,1,0)&
            *x + 128.D0*Hr4(1,0,1,0)*dm )

       gqq2 = gqq2 + twonfTF*CF*CA*(-1034.D0/9.D0+3938.D0/27.D0*x-&
            836.D0/27.D0*dm + 32.D0*z3 + 16.D0*z3*x - 16.D0*z3*dp - 48.D0&
            *z3*dm - 88.D0/9.D0*z2 - 8.D0/3.D0*z2*x + 160.D0/9.D0*z2*dp&
            +160.D0/9.D0*z2*dm - 32.D0/3.D0*Hr1(-1)*z2 + 32.D0/3.D0*&
            Hr1(-1)*z2*x + 64.D0/3.D0*Hr1(-1)*z2*dp - 916.D0/27.D0*Hr1(0)&
            +716.D0/27.D0*Hr1(0)*x - 1336.D0/27.D0*Hr1(0)*dm - 8.D0/3.D0&
            *Hr1(0)*z2 - 8.D0*Hr1(0)*z2*x - 16.D0/3.D0*Hr1(0)*z2*dp + 32.D0&
            /3.D0*Hr1(0)*z2*dm + 16.D0/3.D0*Hr1(1) - 16.D0/3.D0*Hr1(1)*x&
            -64.D0/9.D0*Hr2(-1,0) + 256.D0/9.D0*Hr2(-1,0)*x + 320.D0/9.D0&
            *Hr2(-1,0)*dp + 104.D0/9.D0*Hr2(0,0) - 32.D0/9.D0*Hr2(0,0)*x&
            -160.D0/9.D0*Hr2(0,0)*dp - 112.D0/3.D0*Hr2(0,0)*dm + 8.D0/3.D0&
            *Hr2(0,1) + 8.D0/3.D0*Hr2(0,1)*x - 16.D0/3.D0*Hr3(-1,0,0)&
            +16.D0/3.D0*Hr3(-1,0,0)*x + 32.D0/3.D0*Hr3(-1,0,0)*dp + 32.D0&
            /3.D0*Hr3(-1,0,1) - 32.D0/3.D0*Hr3(-1,0,1)*x - 64.D0/3.D0*&
            Hr3(-1,0,1)*dp - 16.D0/3.D0*Hr3(0,-1,0) + 16.D0/3.D0*Hr3(0,-1,0)*x )

       gqq2 = gqq2 + twonfTF*CF*CA*(32.D0/3.D0*Hr3(0,-1,0)*dp + 32.D0/3.D0&
            *Hr3(0,0,0) - 32.D0/3.D0*Hr3(0,0,0)*dp - 32.D0/3.D0*Hr3(0,0,0)&
            *dm - 8.D0/3.D0*Hr3(0,0,1) + 8.D0*Hr3(0,0,1)*x + 32.D0/3.D0*&
            Hr3(0,0,1)*dp - 16.D0/3.D0*Hr3(0,0,1)*dm - 8.D0*Hr3(1,0,0) -&
            8.D0*Hr3(1,0,0)*x + 16.D0*Hr3(1,0,0)*dm )

       gqq2 = gqq2 + twonfTF*CF**2*(109.D0 - 217.D0/3.D0*x - 110.D0/3.D0*&
            dm - 128.D0/3.D0*z3 - 32.D0/3.D0*z3*x + 32.D0*z3*dp + 160.D0/&
            3.D0*z3*dm + 160.D0/9.D0*z2 - 160.D0/9.D0*z2*x - 320.D0/9.D0*&
            z2*dp + 64.D0/3.D0*Hr1(-1)*z2 - 64.D0/3.D0*Hr1(-1)*z2*x - 128.D0&
            /3.D0*Hr1(-1)*z2*dp + 508.D0/9.D0*Hr1(0) + 476.D0/9.D0*Hr1(0)&
            *x + 20.D0/3.D0*Hr1(0)*dm - 32.D0/3.D0*Hr1(0)*z2 + 32.D0/3.D0&
            *Hr1(0)*z2*dp + 32.D0/3.D0*Hr1(0)*z2*dm - 64.D0/3.D0*Hr1(1)&
            +64.D0/3.D0*Hr1(1)*x + 128.D0/9.D0*Hr2(-1,0) - 512.D0/9.D0*&
            Hr2(-1,0)*x - 640.D0/9.D0*Hr2(-1,0)*dp + 16.D0/9.D0*Hr2(0,0)&
            +176.D0/3.D0*Hr2(0,0)*x + 320.D0/9.D0*Hr2(0,0)*dp + 16.D0*&
            Hr2(0,0)*dm - 32.D0/9.D0*Hr2(0,1) + 160.D0/9.D0*Hr2(0,1)*x -&
            320.D0/9.D0*Hr2(0,1)*dm + 64.D0/9.D0*Hr2(1,0) + 256.D0/9.D0*&
            Hr2(1,0)*x - 320.D0/9.D0*Hr2(1,0)*dm + 32.D0/3.D0*Hr3(-1,0,0)&
            -32.D0/3.D0*Hr3(-1,0,0)*x - 64.D0/3.D0*Hr3(-1,0,0)*dp - 64.D0&
            /3.D0*Hr3(-1,0,1) + 64.D0/3.D0*Hr3(-1,0,1)*x + 128.D0/3.D0*&
            Hr3(-1,0,1)*dp )

       gqq2 = gqq2 + twonfTF*CF**2*(32.D0/3.D0*Hr3(0,-1,0) - 32.D0/3.D0*&
            Hr3(0,-1,0)*x - 64.D0/3.D0*Hr3(0,-1,0)*dp + 8.D0/3.D0*Hr3(0,0,0)&
            + 24.D0*Hr3(0,0,0)*x + 64.D0/3.D0*Hr3(0,0,0)*dp - 32.D0/3.D0&
            *Hr3(0,0,0)*dm + 64.D0/3.D0*Hr3(0,0,1) - 64.D0/3.D0*Hr3(0,0,1)&
            *dp - 64.D0/3.D0*Hr3(0,0,1)*dm + 32.D0/3.D0*Hr3(0,1,0) +&
            32.D0/3.D0*Hr3(0,1,0)*x - 64.D0/3.D0*Hr3(0,1,0)*dm + 64.D0/3.D0&
            *Hr3(1,0,0) + 64.D0/3.D0*Hr3(1,0,0)*x - 128.D0/3.D0*Hr3(1,0,0)*dm )

       gqq2 = gqq2 + twonfTF**2*CF*(112.D0/27.D0 - 32.D0/9.D0*x - 16.D0/27.D0&
            *dm + 8.D0/27.D0*Hr1(0) - 88.D0/27.D0*Hr1(0)*x + 80.D0/27.D0&
            *Hr1(0)*dm - 8.D0/9.D0*Hr2(0,0) - 8.D0/9.D0*Hr2(0,0)*x + 16.D0&
            /9.D0*Hr2(0,0)*dm )

       res=gqq2/8d0
    END SELECT

    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT,cc_PLUS0)
       !
       ! ...The soft (`+'-distribution) part of the splitting function
       !
       A3 =&
            CA**2*CF*(490.D0/3.D0 + 88.D0/3.D0*z3 - 1072.D0/9.D0*z2&
            +176.D0/5.D0*z2**2 )&
            +CA*CF*twonfTF*(-836.D0/27.D0 + 160.D0/9.D0*z2 - 112.D0/3.D0*z3 )&
            +CF**2*twonfTF*(-110.D0/3.D0 + 32.D0*z3 )-CF*twonfTF**2*16.D0/27.D0
       
       IF(cc_piece.EQ.cc_PLUS0)THEN
          res=A3/8d0
       ELSEIF(cc_piece.EQ.cc_REALVIRT)THEN
          GQQ2L = DM * A3
          res=res-GQQ2L/8d0
       ENDIF
    END SELECT
    
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_SFS_P2NSminus

   ! Space Like
   ! P^{s,(2)}_{ns}=P^{s,(2)}_{qq}-P^{s,(2)}_{qqbar}
   ! Eq.(4.11) in hep-ph/06403192
   FUNCTION AP_SFS_P2NSS(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,DM,DP
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4,Z5=zeta5
     REAL(KIND(1d0))::gqq2,twoTf,twonfTf
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero
     
     IF(cc_piece.EQ.cc_Delta)RETURN
     IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

     DX=one/x
     DM=one/(one-x)
     DP=one/(one+x)
     twoTf=two*TF
     twonfTf=two*nfTF
     CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
          HI1,HI2,HI3,HI4, N1, N2)

  gqq2=&
       (CA*CF*TF*(-2400D0*x+2400D0*x**2+804D0*x*z2+&
       492D0*x**2*z2-108D0*x*z2**2-180D0*x**2*z2**2-&
       288D0*x*z3-480D0*x**3*z3+288D0*z2*Hr1(-1)+&
       216D0*x*z2*Hr1(-1)+216D0*x**2*z2*Hr1(-1)+288D0*x**3*z2*Hr1(-1)-&
       1200D0*x*Hr1(0)-108D0*x**2*Hr1(0)-36D0*x*z2*Hr1(0)&
       -324D0*x**2*z2*Hr1(0)-384D0*x**3*z2*Hr1(0)+288D0*x*z3*Hr1(0)+&
       312D0*x*Hr1(-1)*Hr1(0)+312D0*x**2*Hr1(-1)*Hr1(0)+&
       96D0*Hr1(-1)**2*Hr1(0)-72D0*x*Hr1(-1)**2*Hr1(0)-&
       72D0*x**2*Hr1(-1)**2*Hr1(0)+96D0*x**3*Hr1(-1)**2*Hr1(0)&
       -228D0*x*Hr1(0)**2-174D0*x**2*Hr1(0)**2+180D0*x*z2*Hr1(0)**2&
       +108D0*x**2*z2*Hr1(0)**2-96D0*Hr1(-1)*Hr1(0)**2&
       -36D0*x*Hr1(-1)*Hr1(0)**2-36D0*x**2*Hr1(-1)*Hr1(0)**2&
       -96D0*x**3*Hr1(-1)*Hr1(0)**2+12D0*x*Hr1(0)**3&
       +32D0*x**3*Hr1(0)**3-6D0*x*Hr1(0)**4-1092D0*x*Hr1(1)&
       +1092D0*x**2*Hr1(1)+96D0*z2*Hr1(1)+72D0*x*z2*Hr1(1)&
       -72D0*x**2*z2*Hr1(1)-96D0*x**3*z2*Hr1(1)+54D0*x*Hr1(0)**2*Hr1(1)&
       -54D0*x**2*Hr1(0)**2*Hr1(1)-312D0*x*Hr2(0,-1)-312D0*x**2*Hr2(0,-1)&
       -144D0*x*z2*Hr2(0,-1)+144D0*x**2*z2*Hr2(0,-1)-192D0*Hr1(-1)*Hr2(0,-1)&
       +144D0*x*Hr1(-1)*Hr2(0,-1)+144D0*x**2*Hr1(-1)*Hr2(0,-1)&
       -192D0*x**3*Hr1(-1)*Hr2(0,-1)+192D0*Hr1(0)*Hr2(0,-1)&
       +288D0*x**2*Hr1(0)*Hr2(0,-1)+72D0*x*Hr1(0)**2*Hr2(0,-1)&
       -72D0*x**2*Hr1(0)**2*Hr2(0,-1)+144D0*x*Hr2(0,-1)**2&
       -144D0*x**2*Hr2(0,-1)**2-492D0*x*Hr2(0,1)-492D0*x**2*Hr2(0,1)&
       +144D0*x*z2*Hr2(0,1)+144D0*x**2*z2*Hr2(0,1)-192D0*Hr1(-1)*Hr2(0,1)&
       -288D0*x*Hr1(-1)*Hr2(0,1)-288D0*x**2*Hr1(-1)*Hr2(0,1)&
       -192D0*x**3*Hr1(-1)*Hr2(0,1)-108D0*x*Hr1(0)*Hr2(0,1)&
       +108D0*x**2*Hr1(0)*Hr2(0,1)+36D0*x*Hr1(0)**2*Hr2(0,1)&
       +36D0*x**2*Hr1(0)**2*Hr2(0,1)+192D0*Hr3(0,-1,-1)&
       -144D0*x*Hr3(0,-1,-1)-144D0*x**2*Hr3(0,-1,-1)+192D0*x**3*Hr3(0,-1,-1)&
       -288D0*x*Hr1(0)*Hr3(0,-1,-1)+288D0*x**2*Hr1(0)*Hr3(0,-1,-1)&
       +192D0*Hr3(0,-1,1)+288D0*x*Hr3(0,-1,1)+288D0*x**2*Hr3(0,-1,1)&
       +192D0*x**3*Hr3(0,-1,1)-192D0*Hr3(0,0,-1)+72D0*x*Hr3(0,0,-1)&
       -504D0*x**2*Hr3(0,0,-1)+192D0*x**3*Hr3(0,0,-1)-144D0*x*Hr1(0)*Hr3(0,0,-1)&
       +144D0*x**2*Hr1(0)*Hr3(0,0,-1)+72D0*x*Hr3(0,0,1)+216D0*x**2*Hr3(0,0,1)&
       +192D0*x**3*Hr3(0,0,1)-144D0*x*Hr1(0)*Hr3(0,0,1)&
       -144D0*x**2*Hr1(0)*Hr3(0,0,1)+192D0*Hr3(0,1,-1)+288D0*x*Hr3(0,1,-1)&
       +288D0*x**2*Hr3(0,1,-1)+192D0*x**3*Hr3(0,1,-1)))/(12D0*x)

  gqq2=gqq2+(-2D0*cf**2*Tf*(-2400D0*x+2400D0*x**2&
       +804D0*x*z2+492D0*x**2*z2-108D0*x*z2**2&
       -180D0*x**2*z2**2-288D0*x*z3-480D0*x**3*z3&
       +288D0*z2*Hr1(-1)+216D0*x*z2*Hr1(-1)+216D0*x**2*z2*Hr1(-1)&
       +288D0*x**3*z2*Hr1(-1)-1200D0*x*Hr1(0)-108D0*x**2*Hr1(0)&
       -36D0*x*z2*Hr1(0)-324D0*x**2*z2*Hr1(0)-384D0*x**3*z2*Hr1(0)&
       +288D0*x*z3*Hr1(0)+312D0*x*Hr1(-1)*Hr1(0)+312D0*x**2*Hr1(-1)*Hr1(0)&
       +96D0*Hr1(-1)**2*Hr1(0)-72D0*x*Hr1(-1)**2*Hr1(0)&
       -72D0*x**2*Hr1(-1)**2*Hr1(0)+96D0*x**3*Hr1(-1)**2*Hr1(0)&
       -228D0*x*Hr1(0)**2-174D0*x**2*Hr1(0)**2+180D0*x*z2*Hr1(0)**2&
       +108D0*x**2*z2*Hr1(0)**2-96D0*Hr1(-1)*Hr1(0)**2&
       -36D0*x*Hr1(-1)*Hr1(0)**2-36D0*x**2*Hr1(-1)*Hr1(0)**2&
       -96D0*x**3*Hr1(-1)*Hr1(0)**2+12D0*x*Hr1(0)**3&
       +32D0*x**3*Hr1(0)**3-6D0*x*Hr1(0)**4-1092D0*x*Hr1(1)&
       +1092D0*x**2*Hr1(1)+96D0*z2*Hr1(1)+72D0*x*z2*Hr1(1)&
       -72D0*x**2*z2*Hr1(1)-96D0*x**3*z2*Hr1(1)+54D0*x*Hr1(0)**2*Hr1(1)&
       -54D0*x**2*Hr1(0)**2*Hr1(1)-312D0*x*Hr2(0,-1)-312D0*x**2*Hr2(0,-1)&
       -144D0*x*z2*Hr2(0,-1)+144D0*x**2*z2*Hr2(0,-1)-192D0*Hr1(-1)*Hr2(0,-1)&
       +144D0*x*Hr1(-1)*Hr2(0,-1)+144D0*x**2*Hr1(-1)*Hr2(0,-1)&
       -192D0*x**3*Hr1(-1)*Hr2(0,-1)+192D0*Hr1(0)*Hr2(0,-1)&
       +288D0*x**2*Hr1(0)*Hr2(0,-1)+72D0*x*Hr1(0)**2*Hr2(0,-1)&
       -72D0*x**2*Hr1(0)**2*Hr2(0,-1)+144D0*x*Hr2(0,-1)**2&
       -144D0*x**2*Hr2(0,-1)**2-492D0*x*Hr2(0,1)-492D0*x**2*Hr2(0,1)&
       +144D0*x*z2*Hr2(0,1)+144D0*x**2*z2*Hr2(0,1)-192D0*Hr1(-1)*Hr2(0,1)&
       -288D0*x*Hr1(-1)*Hr2(0,1)-288D0*x**2*Hr1(-1)*Hr2(0,1)&
       -192D0*x**3*Hr1(-1)*Hr2(0,1)-108D0*x*Hr1(0)*Hr2(0,1)&
       +108D0*x**2*Hr1(0)*Hr2(0,1)+36D0*x*Hr1(0)**2*Hr2(0,1)&
       +36D0*x**2*Hr1(0)**2*Hr2(0,1)+192D0*Hr3(0,-1,-1)&
       -144D0*x*Hr3(0,-1,-1)-144D0*x**2*Hr3(0,-1,-1)&
       +192D0*x**3*Hr3(0,-1,-1)-288D0*x*Hr1(0)*Hr3(0,-1,-1)&
       +288D0*x**2*Hr1(0)*Hr3(0,-1,-1)+192D0*Hr3(0,-1,1)&
       +288D0*x*Hr3(0,-1,1)+288D0*x**2*Hr3(0,-1,1)&
       +192D0*x**3*Hr3(0,-1,1)-192D0*Hr3(0,0,-1)+72D0*x*Hr3(0,0,-1)&
       -504D0*x**2*Hr3(0,0,-1)+192D0*x**3*Hr3(0,0,-1)&
       -144D0*x*Hr1(0)*Hr3(0,0,-1)+144D0*x**2*Hr1(0)*Hr3(0,0,-1)&
       +72D0*x*Hr3(0,0,1)+216D0*x**2*Hr3(0,0,1)+192D0*x**3*Hr3(0,0,1)&
       -144D0*x*Hr1(0)*Hr3(0,0,1)-144D0*x**2*Hr1(0)*Hr3(0,0,1)&
       +192D0*Hr3(0,1,-1)+288D0*x*Hr3(0,1,-1)+288D0*x**2*Hr3(0,1,-1)&
       +192D0*x**3*Hr3(0,1,-1)))/(9D0*x)

     res=gqq2
     
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_SFS_P2NSS

  ! Space like
  ! P^{(2)}_{qg}
  ! Eq.(4.13) in hep-ph/0404111
  ! Expressions are copyed from subroutine X2QGA in xpij2e.f
  ! by A. Vogt, S. Moch and J. Vermaseren, hep-ph/0404111
  FUNCTION AP_SFS_P2qg(y,y_is_x) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
    REAL(KIND(1d0))::x,DX
    REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
    REAL(KIND(1d0))::gqg2,twoTf,twonfTf
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

    DX=one/x
    twoTf=two*TF
    twonfTf=two*nfTF
    CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
         HI1,HI2,HI3,HI4, N1, N2)

    gqg2 = &
         TF*CA**2*( 10454.D0/27.D0 - 102608.D0/27.D0*x + 103040.D0/&
         27.D0*x**2 - 9404.D0/27.D0*dx + 48.D0*z3 + 1000.D0*z3*x -&
         2768.D0/3.D0*z3*x**2 + 96.D0*z3*dx - 628.D0/3.D0*z2 + 2096.D0/&
         9.D0*z2*x - 620.D0/9.D0*z2*x**2 + 512.D0/9.D0*z2*dx + 538.D0/&
         5.D0*z2**2 + 396.D0*z2**2*x + 224.D0/5.D0*z2**2*x**2 - 40.D0*&
         Hr1(-1)*z3 - 80.D0*Hr1(-1)*z3*x - 80.D0*Hr1(-1)*z3*x**2 + 120.D0&
         *Hr1(-1)*z2 + 88.D0*Hr1(-1)*z2*x - 64.D0/3.D0*Hr1(-1)*z2*&
         x**2 + 32.D0/3.D0*Hr1(-1)*z2*dx - 8696.D0/27.D0*Hr1(0) -&
         30962.D0/27.D0*Hr1(0)*x - 3040.D0*Hr1(0)*x**2 - 896.D0/27.D0*&
         Hr1(0)*dx + 192.D0*Hr1(0)*z3 + 576.D0*Hr1(0)*z3*x - 44.D0*&
         Hr1(0)*z2 + 108.D0*Hr1(0)*z2*x - 248.D0/3.D0*Hr1(0)*z2*x**2&
         +8288.D0/27.D0*Hr1(1) - 7750.D0/27.D0*Hr1(1)*x - 1520.D0/27.D0&
         *Hr1(1)*x**2 + 644.D0/9.D0*Hr1(1)*dx - 248.D0*Hr1(1)*z3 +&
         496.D0*Hr1(1)*z3*x - 496.D0*Hr1(1)*z3*x**2 + 212.D0/3.D0*Hr1(1)&
         *z2 - 64.D0/3.D0*Hr1(1)*z2*x - 136.D0/3.D0*Hr1(1)*z2*x**2&
         + 32.D0/3.D0*Hr1(1)*z2*dx )

    gqg2 = gqg2 + TF*CA**2*(16.D0*Hr2(-1,-1)*z2 + 32.D0*Hr2(-1,-1)&
         *z2*x + 32.D0*Hr2(-1,-1)*z2*x**2 - 704.D0/9.D0*Hr2(-1,0) +&
         5864.D0/9.D0*Hr2(-1,0)*x + 2216.D0/3.D0*Hr2(-1,0)*x**2 + 832.D0&
         /9.D0*Hr2(-1,0)*dx + 56.D0*Hr2(0,-1)*z2 - 16.D0*Hr2(0,-1)*z2&
         *x + 64.D0*Hr2(0,-1)*z2*x**2 + 2396.D0/9.D0*Hr2(0,0) + 200.D0&
         *Hr2(0,0)*x + 10244.D0/9.D0*Hr2(0,0)*x**2 - 12.D0*Hr2(0,0)*z2&
         +56.D0*Hr2(0,0)*z2*x - 96.D0*Hr2(0,0)*z2*x**2 + 628.D0/3.D0&
         *Hr2(0,1) + 1256.D0/3.D0*Hr2(0,1)*x + 620.D0/9.D0*Hr2(0,1)*&
         x**2 + 320.D0/9.D0*Hr2(0,1)*dx + 24.D0*Hr2(0,1)*z2 + 80.D0*&
         Hr2(0,1)*z2*x + 92.D0/3.D0*Hr2(1,0) + 688.D0/3.D0*Hr2(1,0)*x&
         -276.D0*Hr2(1,0)*x**2 + 24.D0*Hr2(1,0)*dx - 80.D0*Hr2(1,0)*&
         z2 + 160.D0*Hr2(1,0)*z2*x - 160.D0*Hr2(1,0)*z2*x**2 - 164.D0/&
         9.D0*Hr2(1,1) - 1904.D0/9.D0*Hr2(1,1)*x + 188.D0*Hr2(1,1)*&
         x**2 - 160.D0/9.D0*Hr2(1,1)*dx - 16.D0*Hr2(1,1)*z2 + 32.D0*&
         Hr2(1,1)*z2*x - 32.D0*Hr2(1,1)*z2*x**2 + 112.D0*Hr3(-1,-1,0)&
         -80.D0*Hr3(-1,-1,0)*x )

    gqg2 = gqg2 + TF*CA**2*(-640.D0/3.D0*Hr3(-1,-1,0)*x**2 - 64.D0&
         /3.D0*Hr3(-1,-1,0)*dx - 256.D0/3.D0*Hr3(-1,0,0) - 1016.D0/3.D0&
         *Hr3(-1,0,0)*x - 976.D0/3.D0*Hr3(-1,0,0)*x**2 - 128.D0/3.D0*&
         Hr3(-1,0,0)*dx - 64.D0*Hr3(-1,0,1) - 128.D0*Hr3(-1,0,1)*x -&
         256.D0/3.D0*Hr3(-1,0,1)*x**2 - 64.D0/3.D0*Hr3(-1,0,1)*dx - 72.D0&
         *Hr3(0,-1,0) + 216.D0*Hr3(0,-1,0)*x - 400.D0*Hr3(0,-1,0)*&
         x**2 - 56.D0/3.D0*Hr3(0,0,0) + 56.D0/3.D0*Hr3(0,0,0)*x + 96.D0&
         *Hr3(0,0,0)*x**2 + 44.D0*Hr3(0,0,1) + 108.D0*Hr3(0,0,1)*x +&
         248.D0/3.D0*Hr3(0,0,1)*x**2 + 16.D0*Hr3(0,1,0) + 96.D0*Hr3(0,1,0)&
         *x - 40.D0/3.D0*Hr3(0,1,0)*x**2 - 32.D0*Hr3(0,1,1)*x -&
         848.D0/3.D0*Hr3(0,1,1)*x**2 + 32.D0/3.D0*Hr3(0,1,1)*dx - 304.D0&
         /3.D0*Hr3(1,0,0) - 148.D0/3.D0*Hr3(1,0,0)*x + 592.D0/3.D0*&
         Hr3(1,0,0)*x**2 - 32.D0*Hr3(1,0,0)*dx - 44.D0/3.D0*Hr3(1,0,1)&
         +184.D0/3.D0*Hr3(1,0,1)*x - 184.D0/3.D0*Hr3(1,0,1)*x**2 +&
         92.D0/3.D0*Hr3(1,1,0) + 392.D0/3.D0*Hr3(1,1,0)*x - 168.D0*&
         Hr3(1,1,0)*x**2 )

    gqg2 = gqg2 + TF*CA**2*(64.D0/3.D0*Hr3(1,1,0)*dx + 68.D0/3.D0*&
         Hr3(1,1,1) - 184.D0/3.D0*Hr3(1,1,1)*x + 128.D0/3.D0*Hr3(1,1,1)&
         *x**2 + 32.D0/3.D0*Hr3(1,1,1)*dx + 96.D0*Hr4(-1,-1,-1,0) +&
         192.D0*Hr4(-1,-1,-1,0)*x + 192.D0*Hr4(-1,-1,-1,0)*x**2 + 32.D0&
         *Hr4(-1,-1,0,0) + 64.D0*Hr4(-1,-1,0,0)*x + 64.D0*Hr4(-1,-1,0,0)&
         *x**2 + 32.D0*Hr4(-1,-1,0,1) + 64.D0*Hr4(-1,-1,0,1)*x + 64.D0&
         *Hr4(-1,-1,0,1)*x**2 - 16.D0*Hr4(-1,0,-1,0) - 32.D0*Hr4(-1,0,-1,0)&
         *x - 32.D0*Hr4(-1,0,-1,0)*x**2 - 24.D0*Hr4(-1,0,0,0) -&
         48.D0*Hr4(-1,0,0,0)*x - 48.D0*Hr4(-1,0,0,0)*x**2 - 32.D0*Hr4(-1,0,0,1)&
         -64.D0*Hr4(-1,0,0,1)*x - 64.D0*Hr4(-1,0,0,1)*x**2&
         +32.D0*Hr4(-1,0,1,1) + 64.D0*Hr4(-1,0,1,1)*x + 64.D0*Hr4(-1,0,1,1)&
         *x**2 + 48.D0*Hr4(0,-1,-1,0) - 160.D0*Hr4(0,-1,-1,0)*x&
         +24.D0*Hr4(0,-1,0,0) - 208.D0*Hr4(0,-1,0,0)*x - 32.D0*Hr4(0,-1,0,0)&
         *x**2 - 32.D0*Hr4(0,-1,0,1) - 64.D0*Hr4(0,-1,0,1)*x&
         -64.D0*Hr4(0,-1,0,1)*x**2 + 72.D0*Hr4(0,0,-1,0) - 16.D0*&
         Hr4(0,0,-1,0)*x )

    gqg2 = gqg2 + TF*CA**2 * ( 64.D0*Hr4(0,0,0,0) - 240.D0*Hr4(0,0,0,0)&
         *x + 12.D0*Hr4(0,0,0,1) - 72.D0*Hr4(0,0,0,1)*x + 96.D0*Hr4(0,0,0,1)&
         *x**2 + 48.D0*Hr4(0,0,1,1) + 128.D0*Hr4(0,0,1,1)*x -&
         84.D0*Hr4(0,1,0,0) - 200.D0*Hr4(0,1,0,0)*x - 64.D0*Hr4(0,1,0,0)&
         *x**2 + 32.D0*Hr4(0,1,1,0) + 128.D0*Hr4(0,1,1,0)*x + 16.D0*&
         Hr4(0,1,1,1) + 64.D0*Hr4(0,1,1,1)*x - 48.D0*Hr4(1,0,-1,0) +&
         96.D0*Hr4(1,0,-1,0)*x - 96.D0*Hr4(1,0,-1,0)*x**2 + 40.D0*Hr4(1,0,0,0)&
         - 80.D0*Hr4(1,0,0,0)*x + 80.D0*Hr4(1,0,0,0)*x**2 +&
         48.D0*Hr4(1,0,0,1) - 96.D0*Hr4(1,0,0,1)*x + 96.D0*Hr4(1,0,0,1)&
         *x**2 - 32.D0*Hr4(1,0,1,0) + 64.D0*Hr4(1,0,1,0)*x - 64.D0*&
         Hr4(1,0,1,0)*x**2 - 16.D0*Hr4(1,1,0,0) + 32.D0*Hr4(1,1,0,0)*x&
         -32.D0*Hr4(1,1,0,0)*x**2 - 32.D0*Hr4(1,1,0,1) + 64.D0*Hr4(1,1,0,1)&
         *x - 64.D0*Hr4(1,1,0,1)*x**2 - 64.D0*Hr4(1,1,1,0) +&
         128.D0*Hr4(1,1,1,0)*x - 128.D0*Hr4(1,1,1,0)*x**2 + 32.D0*Hr4(1,1,1,1)&
         - 64.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)*x**2 )

    gqg2 = gqg2 + TF*CF*CA*(1429.D0/12.D0 + 2303.D0/2.D0*x - 3997.D0&
         /3.D0*x**2 + 220.D0/3.D0*dx - 100.D0/3.D0*z3 - 1792.D0/3.D0*&
         z3*x + 2240.D0/3.D0*z3*x**2 - 64.D0*z3*dx + 688.D0/9.D0*z2 +&
         268.D0/9.D0*z2*x + 392.D0*z2*x**2 - 292.D0/5.D0*z2**2 - 808.D0&
         /5.D0*z2**2*x - 32.D0*z2**2*x**2 + 136.D0*Hr1(-1)*z3 + 272.D0&
         *Hr1(-1)*z3*x + 272.D0*Hr1(-1)*z3*x**2 + 40.D0*Hr1(-1)*z2 +&
         80.D0*Hr1(-1)*z2*x + 40.D0*Hr1(-1)*z2*x**2 + 2696.D0/27.D0*&
         Hr1(0) + 24569.D0/27.D0*Hr1(0)*x + 11332.D0/27.D0*Hr1(0)*x**2&
         -88.D0*Hr1(0)*z3 - 656.D0*Hr1(0)*z3*x + 32.D0*Hr1(0)*z3*&
         x**2 - 40.D0*Hr1(0)*z2 - 360.D0*Hr1(0)*z2*x - 1384.D0/3.D0*&
         Hr1(0)*z2*x**2 - 3692.D0/27.D0*Hr1(1) + 12892.D0/27.D0*Hr1(1)&
         *x - 9620.D0/27.D0*Hr1(1)*x**2 - 1484.D0/27.D0*Hr1(1)*dx +&
         312.D0*Hr1(1)*z3 - 624.D0*Hr1(1)*z3*x + 624.D0*Hr1(1)*z3*x**2&
         -52.D0*Hr1(1)*z2 - 40.D0*Hr1(1)*z2*x + 80.D0*Hr1(1)*z2*x**2&
         -144.D0*Hr2(-1,-1)*z2 - 288.D0*Hr2(-1,-1)*z2*x - 288.D0*&
         Hr2(-1,-1)*z2*x**2 )

    gqg2 = gqg2 + TF*CF*CA*(296.D0*Hr2(-1,0) + 152.D0*Hr2(-1,0)*x&
         -144.D0*Hr2(-1,0)*x**2 + 144.D0*Hr2(-1,0)*z2 + 288.D0*Hr2(-1,0)&
         *z2*x + 288.D0*Hr2(-1,0)*z2*x**2 + 112.D0*Hr2(0,-1)*z2&
         +128.D0*Hr2(0,-1)*z2*x + 288.D0*Hr2(0,-1)*z2*x**2 - 1267.D0/&
         9.D0*Hr2(0,0) - 3190.D0/9.D0*Hr2(0,0)*x - 2204.D0/9.D0*Hr2(0,0)&
         *x**2 + 96.D0*Hr2(0,0)*z2 + 96.D0*Hr2(0,0)*z2*x + 64.D0*&
         Hr2(0,0)*z2*x**2 - 688.D0/9.D0*Hr2(0,1) + 1100.D0/9.D0*Hr2(0,1)&
         *x - 392.D0*Hr2(0,1)*x**2 + 16.D0*Hr2(0,1)*z2 - 64.D0*Hr2(0,1)&
         *z2*x + 128.D0*Hr2(0,1)*z2*x**2 - 116.D0/9.D0*Hr2(1,0) +&
         1888.D0/9.D0*Hr2(1,0)*x - 212.D0*Hr2(1,0)*x**2 + 368.D0/9.D0*&
         Hr2(1,0)*dx + 144.D0*Hr2(1,0)*z2 - 288.D0*Hr2(1,0)*z2*x + 288.D0&
         *Hr2(1,0)*z2*x**2 - 808.D0/9.D0*Hr2(1,1) + 4568.D0/9.D0*&
         Hr2(1,1)*x - 4096.D0/9.D0*Hr2(1,1)*x**2 + 368.D0/9.D0*Hr2(1,1)&
         *dx + 96.D0*Hr2(1,1)*z2 - 192.D0*Hr2(1,1)*z2*x + 192.D0*Hr2(1,1)&
         *z2*x**2 - 144.D0*Hr3(-1,-1,0) - 224.D0*Hr3(-1,-1,0)*x -&
         80.D0*Hr3(-1,-1,0)*x**2 )

    gqg2 = gqg2 + TF*CF*CA*(40.D0*Hr3(-1,0,0) + 64.D0*Hr3(-1,0,0)*&
         x + 48.D0*Hr3(-1,0,0)*x**2 - 112.D0*Hr3(-1,0,1) - 192.D0*Hr3(-1,0,1)&
         *x - 80.D0*Hr3(-1,0,1)*x**2 + 208.D0*Hr3(0,-1,0) - 64.D0&
         *Hr3(0,-1,0)*x + 80.D0*Hr3(0,-1,0)*x**2 + 44.D0/3.D0*Hr3(0,0,0)&
         + 344.D0/3.D0*Hr3(0,0,0)*x + 1264.D0/3.D0*Hr3(0,0,0)*x**2&
         +40.D0*Hr3(0,0,1) + 296.D0*Hr3(0,0,1)*x + 1384.D0/3.D0*Hr3(0,0,1)&
         *x**2 + 32.D0*Hr3(0,1,0) + 72.D0*Hr3(0,1,0)*x + 392.D0*&
         Hr3(0,1,0)*x**2 - 44.D0/3.D0*Hr3(0,1,1) + 496.D0/3.D0*Hr3(0,1,1)&
         *x + 192.D0*Hr3(0,1,1)*x**2 + 472.D0/3.D0*Hr3(1,0,0) - 488.D0&
         /3.D0*Hr3(1,0,0)*x + 32.D0/3.D0*Hr3(1,0,0)*dx - 20.D0*Hr3(1,0,1)&
         + 152.D0*Hr3(1,0,1)*x - 120.D0*Hr3(1,0,1)*x**2 + 4.D0*&
         Hr3(1,1,0) - 88.D0*Hr3(1,1,0)*x + 280.D0/3.D0*Hr3(1,1,0)*x**2&
         -64.D0/3.D0*Hr3(1,1,0)*dx - 56.D0/3.D0*Hr3(1,1,1) + 424.D0/&
         3.D0*Hr3(1,1,1)*x - 416.D0/3.D0*Hr3(1,1,1)*x**2 - 32.D0/3.D0*&
         Hr3(1,1,1)*dx - 96.D0*Hr4(-1,-1,-1,0) - 192.D0*Hr4(-1,-1,-1,0)*x )

    gqg2 = gqg2 + TF*CF*CA*(-192.D0*Hr4(-1,-1,-1,0)*x**2 + 96.D0&
         *Hr4(-1,-1,0,0) + 192.D0*Hr4(-1,-1,0,0)*x + 192.D0*Hr4(-1,-1,0,0)&
         *x**2 + 96.D0*Hr4(-1,-1,0,1) + 192.D0*Hr4(-1,-1,0,1)*x +&
         192.D0*Hr4(-1,-1,0,1)*x**2 + 48.D0*Hr4(-1,0,-1,0) + 96.D0*&
         Hr4(-1,0,-1,0)*x + 96.D0*Hr4(-1,0,-1,0)*x**2 - 88.D0*Hr4(-1,0,0,0)&
         - 176.D0*Hr4(-1,0,0,0)*x - 176.D0*Hr4(-1,0,0,0)*x**2 -&
         96.D0*Hr4(-1,0,0,1) - 192.D0*Hr4(-1,0,0,1)*x - 192.D0*Hr4(-1,0,0,1)&
         *x**2 - 32.D0*Hr4(-1,0,1,0) - 64.D0*Hr4(-1,0,1,0)*x -&
         64.D0*Hr4(-1,0,1,0)*x**2 - 32.D0*Hr4(-1,0,1,1) - 64.D0*Hr4(-1,0,1,1)&
         *x - 64.D0*Hr4(-1,0,1,1)*x**2 + 32.D0*Hr4(0,-1,-1,0)&
         +128.D0*Hr4(0,-1,-1,0)*x + 192.D0*Hr4(0,-1,-1,0)*x**2 - 64.D0&
         *Hr4(0,-1,0,0) - 96.D0*Hr4(0,-1,0,0)*x - 192.D0*Hr4(0,-1,0,0)&
         *x**2 - 96.D0*Hr4(0,-1,0,1) - 64.D0*Hr4(0,-1,0,1)*x - 192.D0&
         *Hr4(0,-1,0,1)*x**2 + 48.D0*Hr4(0,0,-1,0) - 32.D0*Hr4(0,0,-1,0)&
         *x - 32.D0*Hr4(0,0,0,0) - 64.D0*Hr4(0,0,0,0)*x - 96.D0*Hr4(0,0,0,1) )

    gqg2 = gqg2 + TF*CF*CA*(-128.D0*Hr4(0,0,0,1)*x - 64.D0*Hr4(0,0,0,1)&
         *x**2 - 64.D0*Hr4(0,0,1,0) - 192.D0*Hr4(0,0,1,0)*x -&
         64.D0*Hr4(0,0,1,1) - 96.D0*Hr4(0,0,1,1)*x - 64.D0*Hr4(0,0,1,1)&
         *x**2 + 88.D0*Hr4(0,1,0,0) + 16.D0*Hr4(0,1,0,0)*x + 96.D0*&
         Hr4(0,1,0,0)*x**2 - 32.D0*Hr4(0,1,0,1)*x**2 - 192.D0*Hr4(0,1,1,0)&
         *x + 96.D0*Hr4(0,1,1,0)*x**2 - 56.D0*Hr4(0,1,1,1) + 16.D0&
         *Hr4(0,1,1,1)*x - 64.D0*Hr4(0,1,1,1)*x**2 + 48.D0*Hr4(1,0,-1,0)&
         - 96.D0*Hr4(1,0,-1,0)*x + 96.D0*Hr4(1,0,-1,0)*x**2 - 24.D0&
         *Hr4(1,0,0,0) + 48.D0*Hr4(1,0,0,0)*x - 48.D0*Hr4(1,0,0,0)*&
         x**2 - 96.D0*Hr4(1,0,0,1) + 192.D0*Hr4(1,0,0,1)*x - 192.D0*&
         Hr4(1,0,0,1)*x**2 - 96.D0*Hr4(1,0,1,1) + 192.D0*Hr4(1,0,1,1)*&
         x - 192.D0*Hr4(1,0,1,1)*x**2 + 16.D0*Hr4(1,1,0,0) - 32.D0*&
         Hr4(1,1,0,0)*x + 32.D0*Hr4(1,1,0,0)*x**2 - 48.D0*Hr4(1,1,0,1)&
         +96.D0*Hr4(1,1,0,1)*x - 96.D0*Hr4(1,1,0,1)*x**2 + 48.D0*&
         Hr4(1,1,1,0) - 96.D0*Hr4(1,1,1,0)*x + 96.D0*Hr4(1,1,1,0)*x**2&
         -64.D0*Hr4(1,1,1,1) )

    gqg2 = gqg2 + TF*CF*CA*(128.D0*Hr4(1,1,1,1)*x - 128.D0*Hr4(1,1,1,1)*x**2 )

    gqg2 = gqg2 + TF*CF**2*(963.D0/4.D0 - 641.D0/2.D0*x + 81.D0*&
         x**2 - 24.D0*z3 + 136.D0*z3*x - 304.D0*z3*x**2 - 68.D0*z2 +&
         48.D0*z2*x - 244.D0*z2*x**2 + 32.D0*z2**2 - 192.D0/5.D0*z2**2&
         *x + 352.D0/5.D0*z2**2*x**2 + 64.D0*Hr1(-1)*z2 + 128.D0*Hr1(-1)&
         *z2*x + 64.D0*Hr1(-1)*z2*x**2 + 170.D0*Hr1(0) + 57.D0*Hr1(0)&
         *x + 348.D0*Hr1(0)*x**2 - 56.D0*Hr1(0)*z3 + 208.D0*Hr1(0)*&
         z3*x - 224.D0*Hr1(0)*z3*x**2 - 36.D0*Hr1(0)*z2 + 128.D0*Hr1(0)&
         *z2*x - 176.D0*Hr1(0)*z2*x**2 - 32.D0*Hr1(1) - 282.D0*Hr1(1)&
         *x + 348.D0*Hr1(1)*x**2 - 112.D0*Hr1(1)*z3 + 224.D0*Hr1(1)*z3&
         *x - 224.D0*Hr1(1)*z3*x**2 + 32.D0*Hr1(1)*z2 + 80.D0*Hr1(1)*&
         z2*x - 112.D0*Hr1(1)*z2*x**2 - 320.D0*Hr2(-1,0) - 368.D0*Hr2(-1,0)&
         *x - 80.D0*Hr2(-1,0)*x**2 + 32.D0*Hr2(0,-1)*z2 + 89.D0*&
         Hr2(0,0) + 126.D0*Hr2(0,0)*x + 352.D0*Hr2(0,0)*x**2 - 56.D0*&
         Hr2(0,0)*z2 + 112.D0*Hr2(0,0)*z2*x - 224.D0*Hr2(0,0)*z2*x**2&
         +68.D0*Hr2(0,1) - 416.D0*Hr2(0,1)*x + 244.D0*Hr2(0,1)*x**2&
         -32.D0*Hr2(0,1)*z2 )

    gqg2 = gqg2 + TF*CF**2*(128.D0*Hr2(0,1)*z2*x - 160.D0*Hr2(0,1)&
         *z2*x**2 + 160.D0*Hr2(1,0) - 352.D0*Hr2(1,0)*x + 272.D0*Hr2(1,0)&
         *x**2 - 112.D0*Hr2(1,0)*z2 + 224.D0*Hr2(1,0)*z2*x - 224.D0&
         *Hr2(1,0)*z2*x**2 + 108.D0*Hr2(1,1) - 296.D0*Hr2(1,1)*x + 244.D0&
         *Hr2(1,1)*x**2 - 80.D0*Hr2(1,1)*z2 + 160.D0*Hr2(1,1)*z2*x&
         -160.D0*Hr2(1,1)*z2*x**2 + 128.D0*Hr3(-1,-1,0) + 256.D0*&
         Hr3(-1,-1,0)*x + 128.D0*Hr3(-1,-1,0)*x**2 - 128.D0*Hr3(-1,0,0)&
         - 288.D0*Hr3(-1,0,0)*x - 160.D0*Hr3(-1,0,0)*x**2 - 176.D0*&
         Hr3(0,-1,0) - 32.D0*Hr3(0,-1,0)*x - 128.D0*Hr3(0,-1,0)*x**2&
         +12.D0*Hr3(0,0,0) + 64.D0*Hr3(0,0,0)*x + 240.D0*Hr3(0,0,0)*&
         x**2 + 36.D0*Hr3(0,0,1) - 160.D0*Hr3(0,0,1)*x + 176.D0*Hr3(0,0,1)&
         *x**2 + 48.D0*Hr3(0,1,0) - 80.D0*Hr3(0,1,0)*x + 80.D0*&
         Hr3(0,1,0)*x**2 + 36.D0*Hr3(0,1,1) - 104.D0*Hr3(0,1,1)*x + 96.D0&
         *Hr3(0,1,1)*x**2 - 32.D0*Hr3(1,0,0) - 48.D0*Hr3(1,0,0)*x +&
         80.D0*Hr3(1,0,0)*x**2 + 32.D0*Hr3(1,0,1) - 208.D0*Hr3(1,0,1)*&
         x)

    gqg2 = gqg2 + TF*CF**2*(176.D0*Hr3(1,0,1)*x**2 - 32.D0*Hr3(1,1,0)&
         - 48.D0*Hr3(1,1,0)*x + 80.D0*Hr3(1,1,0)*x**2 - 4.D0*Hr3(1,1,1)&
         - 80.D0*Hr3(1,1,1)*x + 96.D0*Hr3(1,1,1)*x**2 + 64.D0*&
         Hr4(-1,-1,0,0) + 128.D0*Hr4(-1,-1,0,0)*x + 128.D0*Hr4(-1,-1,0,0)&
         *x**2 + 32.D0*Hr4(-1,0,-1,0) + 64.D0*Hr4(-1,0,-1,0)*x + 64.D0&
         *Hr4(-1,0,-1,0)*x**2 - 16.D0*Hr4(-1,0,0,0) - 32.D0*Hr4(-1,0,0,0)&
         *x - 32.D0*Hr4(-1,0,0,0)*x**2 + 64.D0*Hr4(0,-1,-1,0) -&
         64.D0*Hr4(0,-1,0,0) - 64.D0*Hr4(0,-1,0,0)*x - 128.D0*Hr4(0,-1,0,0)&
         *x**2 - 64.D0*Hr4(0,0,-1,0) - 128.D0*Hr4(0,0,-1,0)*x**2&
         +16.D0*Hr4(0,0,0,0) + 128.D0*Hr4(0,0,0,0)*x**2 + 56.D0*Hr4(0,0,0,1)&
         - 112.D0*Hr4(0,0,0,1)*x + 224.D0*Hr4(0,0,0,1)*x**2&
         +64.D0*Hr4(0,0,1,0) - 128.D0*Hr4(0,0,1,0)*x + 192.D0*Hr4(0,0,1,0)&
         *x**2 + 80.D0*Hr4(0,0,1,1) - 160.D0*Hr4(0,0,1,1)*x +&
         192.D0*Hr4(0,0,1,1)*x**2 + 16.D0*Hr4(0,1,0,0) - 96.D0*Hr4(0,1,0,0)&
         *x + 128.D0*Hr4(0,1,0,0)*x**2 + 64.D0*Hr4(0,1,0,1) - 128.D0&
         *Hr4(0,1,0,1)*x )

    gqg2 = gqg2 + TF*CF**2*(160.D0*Hr4(0,1,0,1)*x**2 + 32.D0*Hr4(0,1,1,0)&
         - 64.D0*Hr4(0,1,1,0)*x + 32.D0*Hr4(0,1,1,0)*x**2 + 40.D0&
         *Hr4(0,1,1,1) - 80.D0*Hr4(0,1,1,1)*x + 64.D0*Hr4(0,1,1,1)*&
         x**2 - 32.D0*Hr4(1,0,-1,0) + 64.D0*Hr4(1,0,-1,0)*x - 64.D0*&
         Hr4(1,0,-1,0)*x**2 + 48.D0*Hr4(1,0,0,0) - 96.D0*Hr4(1,0,0,0)*&
         x + 96.D0*Hr4(1,0,0,0)*x**2 + 112.D0*Hr4(1,0,0,1) - 224.D0*&
         Hr4(1,0,0,1)*x + 224.D0*Hr4(1,0,0,1)*x**2 + 96.D0*Hr4(1,0,1,0)&
         - 192.D0*Hr4(1,0,1,0)*x + 192.D0*Hr4(1,0,1,0)*x**2 + 96.D0*&
         Hr4(1,0,1,1) - 192.D0*Hr4(1,0,1,1)*x + 192.D0*Hr4(1,0,1,1)*&
         x**2 + 64.D0*Hr4(1,1,0,0) - 128.D0*Hr4(1,1,0,0)*x + 128.D0*&
         Hr4(1,1,0,0)*x**2 + 80.D0*Hr4(1,1,0,1) - 160.D0*Hr4(1,1,0,1)*&
         x + 160.D0*Hr4(1,1,0,1)*x**2 + 16.D0*Hr4(1,1,1,0) - 32.D0*&
         Hr4(1,1,1,0)*x + 32.D0*Hr4(1,1,1,0)*x**2 + 32.D0*Hr4(1,1,1,1)&
         -64.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)*x**2 )

    gqg2 = gqg2 + TF*twonfTF*CA*(226.D0/27.D0 - 1348.D0/27.D0*x + 2800.D0&
         /81.D0*x**2 - 424.D0/81.D0*dx - 16.D0*z3*x + 88.D0/9.D0*z2*x&
         -160.D0/27.D0*Hr1(0) + 560.D0/27.D0*Hr1(0)*x - 1832.D0/27.D0&
         *Hr1(0)*x**2 - 152.D0/27.D0*Hr1(1) + 112.D0/27.D0*Hr1(1)*x -&
         112.D0/27.D0*Hr1(1)*x**2 - 8.D0/3.D0*Hr1(1)*z2 + 16.D0/3.D0*&
         Hr1(1)*z2*x - 16.D0/3.D0*Hr1(1)*z2*x**2 + 80.D0/9.D0*Hr2(-1,0)&
         + 112.D0/9.D0*Hr2(-1,0)*x + 112.D0/9.D0*Hr2(-1,0)*x**2 + 64.D0&
         /9.D0*Hr2(0,0) + 104.D0/3.D0*Hr2(0,0)*x + 64.D0/9.D0*Hr2(0,0)&
         *x**2 + 8.D0/3.D0*Hr2(0,1)*x + 80.D0/9.D0*Hr2(1,1) - 112.D0/&
         9.D0*Hr2(1,1)*x + 112.D0/9.D0*Hr2(1,1)*x**2 + 16.D0/3.D0*Hr3(-1,0,0)&
         + 32.D0/3.D0*Hr3(-1,0,0)*x + 32.D0/3.D0*Hr3(-1,0,0)*&
         x**2 - 16.D0/3.D0*Hr3(0,0,0) + 64.D0/3.D0*Hr3(0,0,0)*x - 8.D0/&
         3.D0*Hr3(1,0,0) + 16.D0/3.D0*Hr3(1,0,0)*x - 16.D0/3.D0*Hr3(1,0,0)&
         *x**2 + 8.D0/3.D0*Hr3(1,0,1) - 16.D0/3.D0*Hr3(1,0,1)*x +&
         16.D0/3.D0*Hr3(1,0,1)*x**2 - 8.D0/3.D0*Hr3(1,1,0) + 16.D0/3.D0&
         *Hr3(1,1,0)*x )

    gqg2 = gqg2 + TF*twonfTF*CA*(-16.D0/3.D0*Hr3(1,1,0)*x**2 - 8.D0/3.D0&
         *Hr3(1,1,1) + 16.D0/3.D0*Hr3(1,1,1)*x - 16.D0/3.D0*Hr3(1,1,1)*x**2 )

    gqg2 = gqg2 + TF*twonfTF*CF*(-13025.D0/54.D0 - 89.D0/27.D0*x +&
         18424.D0/81.D0*x**2 + 1232.D0/81.D0*dx - 8.D0/3.D0*z3 + 16.D0/&
         3.D0*z3*x - 16.D0/3.D0*z3*x**2 + 80.D0/9.D0*z2 - 112.D0/9.D0*&
         z2*x + 112.D0/9.D0*z2*x**2 - 4856.D0/27.D0*Hr1(0) - 5270.D0/&
         27.D0*Hr1(0)*x - 560.D0/9.D0*Hr1(0)*x**2 + 224.D0/27.D0*Hr1(1)&
         - 148.D0/27.D0*Hr1(1)*x + 112.D0/27.D0*Hr1(1)*x**2 - 1370.D0&
         /9.D0*Hr2(0,0) - 884.D0/9.D0*Hr2(0,0)*x - 352.D0/9.D0*Hr2(0,0)&
         *x**2 - 80.D0/9.D0*Hr2(0,1) + 112.D0/9.D0*Hr2(0,1)*x - 112.D0&
         /9.D0*Hr2(0,1)*x**2 - 40.D0/9.D0*Hr2(1,0) + 128.D0/9.D0*Hr2(1,0)&
         *x - 128.D0/9.D0*Hr2(1,0)*x**2 - 80.D0/9.D0*Hr2(1,1) + 112.D0&
         /9.D0*Hr2(1,1)*x - 112.D0/9.D0*Hr2(1,1)*x**2 - 152.D0/3.D0*&
         Hr3(0,0,0) + 160.D0/3.D0*Hr3(0,0,0)*x + 32.D0*Hr3(0,0,0)*x**2&
         +8.D0/3.D0*Hr3(0,1,1) - 16.D0/3.D0*Hr3(0,1,1)*x + 16.D0/3.D0&
         *Hr3(0,1,1)*x**2 - 16.D0/3.D0*Hr3(1,0,0) + 32.D0/3.D0*Hr3(1,0,0)&
         *x - 32.D0/3.D0*Hr3(1,0,0)*x**2 + 8.D0/3.D0*Hr3(1,1,1) -&
         16.D0/3.D0*Hr3(1,1,1)*x )

    gqg2 = gqg2 + TF*twonfTF*CF*(16.D0/3.D0*Hr3(1,1,1)*x**2 - 32.D0*&
         Hr4(0,0,0,0) + 64.D0*Hr4(0,0,0,0)*x )

    res=gqg2/8d0

    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P2qg

  ! Space like
  ! P^{(2)}_{gq}
  ! Eq.(4.14) in hep-ph/0404111
  ! Expressions are copyed from subroutine X2GQA in xpij2e.f
  ! by A. Vogt, S. Moch and J. Vermaseren, hep-ph/0404111
  FUNCTION AP_SFS_P2gq(y,y_is_x) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
    REAL(KIND(1d0))::x,DX
    REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
    REAL(KIND(1d0))::ggq2,twoTf,twonfTf
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

    DX=one/x
    twoTf=two*TF
    twonfTf=two*nfTF
    CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
         HI1,HI2,HI3,HI4, N1, N2)

    ggq2 =&
         CF*CA**2*(-12710.D0/9.D0 + 1789.D0/3.D0*x - 33680.D0/81.D0&
         *x**2 + 138305.D0/81.D0*dx + 1424.D0/3.D0*z3 + 700.D0/3.D0*&
         z3*x + 256.D0/3.D0*z3*x**2 - 336.D0*z3*dx + 568.D0/9.D0*z2 +&
         568.D0/9.D0*z2*x - 1664.D0/9.D0*z2*x**2 - 872.D0/3.D0*z2*dx&
         -772.D0/5.D0*z2**2 - 366.D0/5.D0*z2**2*x - 544.D0/5.D0*&
         z2**2*dx - 368.D0*Hr1(-1)*z3 - 184.D0*Hr1(-1)*z3*x - 368.D0*&
         Hr1(-1)*z3*dx - 520.D0/3.D0*Hr1(-1)*z2 - 392.D0/3.D0*Hr1(-1)*&
         z2*x - 32.D0/3.D0*Hr1(-1)*z2*x**2 + 104.D0*Hr1(-1)*z2*dx +&
         38224.D0/27.D0*Hr1(0) + 3202.D0/27.D0*Hr1(0)*x + 9344.D0/27.D0&
         *Hr1(0)*x**2 + 6320.D0/27.D0*Hr1(0)*dx - 224.D0*Hr1(0)*z3 -&
         80.D0*Hr1(0)*z3*x - 32.D0*Hr1(0)*z3*dx + 2140.D0/3.D0*Hr1(0)*&
         z2 + 1028.D0/3.D0*Hr1(0)*z2*x + 96.D0*Hr1(0)*z2*x**2 - 176.D0/&
         3.D0*Hr1(0)*z2*dx + 6260.D0/27.D0*Hr1(1) - 766.D0/27.D0*Hr1(1)&
         *x - 1744.D0/27.D0*Hr1(1)*x**2 - 13516.D0/27.D0*Hr1(1)*dx -&
         112.D0*Hr1(1)*z3 + 56.D0*Hr1(1)*z3*x + 112.D0*Hr1(1)*z3*dx -&
         352.D0/3.D0*Hr1(1)*z2 )

    ggq2 = ggq2 + CF*CA**2*(500.D0/3.D0*Hr1(1)*z2*x + 32.D0/3.D0*&
         Hr1(1)*z2*x**2 - 16.D0/3.D0*Hr1(1)*z2*dx + 416.D0*Hr2(-1,-1)*&
         z2 + 208.D0*Hr2(-1,-1)*z2*x + 416.D0*Hr2(-1,-1)*z2*dx - 920.D0&
         /3.D0*Hr2(-1,0) + 212.D0/3.D0*Hr2(-1,0)*x - 160.D0*Hr2(-1,0)*&
         x**2 - 872.D0/3.D0*Hr2(-1,0)*dx - 320.D0*Hr2(-1,0)*z2 - 160.D0&
         *Hr2(-1,0)*z2*x - 320.D0*Hr2(-1,0)*z2*dx - 432.D0*Hr2(0,-1)*&
         z2 - 56.D0*Hr2(0,-1)*z2*x - 160.D0*Hr2(0,-1)*z2*dx - 5080.D0/&
         9.D0*Hr2(0,0) + 392.D0/9.D0*Hr2(0,0)*x - 616.D0/9.D0*Hr2(0,0)&
         *x**2 - 104.D0*Hr2(0,0)*z2 - 92.D0*Hr2(0,0)*z2*x - 568.D0/9.D0&
         *Hr2(0,1) + 68.D0/9.D0*Hr2(0,1)*x + 1664.D0/9.D0*Hr2(0,1)*&
         x**2 - 80.D0*Hr2(0,1)*z2 - 56.D0*Hr2(0,1)*z2*x - 64.D0*Hr2(0,1)&
         *z2*dx + 1096.D0/9.D0*Hr2(1,0) + 400.D0/9.D0*Hr2(1,0)*x +&
         1160.D0/9.D0*Hr2(1,0)*x**2 - 284.D0/3.D0*Hr2(1,0)*dx + 64.D0*&
         Hr2(1,0)*z2 - 32.D0*Hr2(1,0)*z2*x - 64.D0*Hr2(1,0)*z2*dx -&
         100.D0/9.D0*Hr2(1,1) + 914.D0/9.D0*Hr2(1,1)*x + 592.D0/9.D0*&
         Hr2(1,1)*x**2 )

    ggq2 = ggq2 + CF*CA**2*(260.D0/3.D0*Hr2(1,1)*dx + 32.D0*Hr2(1,1)&
         *z2 - 16.D0*Hr2(1,1)*z2*x - 32.D0*Hr2(1,1)*z2*dx + 176.D0/3.D0&
         *Hr3(-1,-1,0) - 176.D0/3.D0*Hr3(-1,-1,0)*x + 64.D0/3.D0*&
         Hr3(-1,-1,0)*x**2 + 304.D0*Hr3(-1,-1,0)*dx + 1160.D0/3.D0*&
         Hr3(-1,0,0) + 232.D0/3.D0*Hr3(-1,0,0)*x + 128.D0/3.D0*Hr3(-1,0,0)&
         *x**2 + 784.D0/3.D0*Hr3(-1,0,0)*dx + 608.D0/3.D0*Hr3(-1,0,1)&
         + 304.D0/3.D0*Hr3(-1,0,1)*x + 64.D0/3.D0*Hr3(-1,0,1)*x**2&
         + 48.D0*Hr3(-1,0,1)*dx + 1160.D0/3.D0*Hr3(0,-1,0) + 16.D0*&
         Hr3(0,-1,0)*x + 64.D0*Hr3(0,-1,0)*x**2 - 176.D0/3.D0*Hr3(0,-1,0)&
         *dx - 272.D0/3.D0*Hr3(0,0,0) - 224.D0*Hr3(0,0,0)*x - 224.D0&
         /3.D0*Hr3(0,0,0)*x**2 - 2140.D0/3.D0*Hr3(0,0,1) - 980.D0/3.D0&
         *Hr3(0,0,1)*x - 96.D0*Hr3(0,0,1)*x**2 - 784.D0/3.D0*Hr3(0,1,0)&
         - 592.D0/3.D0*Hr3(0,1,0)*x - 224.D0/3.D0*Hr3(0,1,0)*x**2 -&
         32.D0/3.D0*Hr3(0,1,0)*dx + 104.D0*Hr3(0,1,1) - 132.D0*Hr3(0,1,1)&
         *x - 128.D0/3.D0*Hr3(0,1,1)*x**2 - 544.D0/3.D0*Hr3(0,1,1)*&
         dx )

    ggq2 = ggq2 + CF*CA**2*(-140.D0/3.D0*Hr3(1,0,0) - 332.D0/3.D0&
         *Hr3(1,0,0)*x - 64.D0/3.D0*Hr3(1,0,0)*x**2 + 176.D0*Hr3(1,0,0)&
         *dx + 440.D0/3.D0*Hr3(1,0,1) - 412.D0/3.D0*Hr3(1,0,1)*x -&
         440.D0/3.D0*Hr3(1,0,1)*dx + 440.D0/3.D0*Hr3(1,1,0) - 412.D0/3.D0&
         *Hr3(1,1,0)*x - 440.D0/3.D0*Hr3(1,1,0)*dx + 632.D0/3.D0*&
         Hr3(1,1,1) - 340.D0/3.D0*Hr3(1,1,1)*x + 32.D0/3.D0*Hr3(1,1,1)&
         *x**2 - 688.D0/3.D0*Hr3(1,1,1)*dx + 192.D0*Hr4(-1,-1,-1,0) +&
         96.D0*Hr4(-1,-1,-1,0)*x + 192.D0*Hr4(-1,-1,-1,0)*dx - 384.D0*&
         Hr4(-1,-1,0,0) - 192.D0*Hr4(-1,-1,0,0)*x - 384.D0*Hr4(-1,-1,0,0)&
         *dx - 320.D0*Hr4(-1,-1,0,1) - 160.D0*Hr4(-1,-1,0,1)*x -&
         320.D0*Hr4(-1,-1,0,1)*dx - 160.D0*Hr4(-1,0,-1,0) - 80.D0*Hr4(-1,0,-1,0)&
         *x - 160.D0*Hr4(-1,0,-1,0)*dx + 176.D0*Hr4(-1,0,0,0)&
         + 88.D0*Hr4(-1,0,0,0)*x + 176.D0*Hr4(-1,0,0,0)*dx + 256.D0*&
         Hr4(-1,0,0,1) + 128.D0*Hr4(-1,0,0,1)*x + 256.D0*Hr4(-1,0,0,1)&
         *dx + 64.D0*Hr4(-1,0,1,0) + 32.D0*Hr4(-1,0,1,0)*x + 64.D0*&
         Hr4(-1,0,1,0)*dx )

    ggq2 = ggq2 + CF*CA**2*(64.D0*Hr4(-1,0,1,1) + 32.D0*Hr4(-1,0,1,1)&
         *x + 64.D0*Hr4(-1,0,1,1)*dx - 352.D0*Hr4(0,-1,-1,0) + 16.D0&
         *Hr4(0,-1,-1,0)*x - 64.D0*Hr4(0,-1,-1,0)*dx + 176.D0*Hr4(0,-1,0,0)&
         + 152.D0*Hr4(0,-1,0,0)*x + 224.D0*Hr4(0,-1,0,0)*dx +&
         256.D0*Hr4(0,-1,0,1) + 64.D0*Hr4(0,-1,0,1)*x + 128.D0*Hr4(0,-1,0,1)&
         *dx - 144.D0*Hr4(0,0,-1,0) + 8.D0*Hr4(0,0,-1,0)*x -&
         128.D0*Hr4(0,0,0,0) + 120.D0*Hr4(0,0,0,0)*x + 104.D0*Hr4(0,0,0,1)&
         + 100.D0*Hr4(0,0,0,1)*x + 64.D0*Hr4(0,0,1,0) + 128.D0*&
         Hr4(0,0,1,0)*x + 128.D0*Hr4(0,0,1,0)*dx - 64.D0*Hr4(0,0,1,1)&
         + 96.D0*Hr4(0,0,1,1)*x + 128.D0*Hr4(0,0,1,1)*dx + 104.D0*&
         Hr4(0,1,0,0) + 84.D0*Hr4(0,1,0,0)*x + 96.D0*Hr4(0,1,0,0)*dx&
         -96.D0*Hr4(0,1,0,1) + 48.D0*Hr4(0,1,0,1)*x + 96.D0*Hr4(0,1,0,1)&
         *dx - 96.D0*Hr4(0,1,1,0) + 48.D0*Hr4(0,1,1,0)*x + 96.D0*&
         Hr4(0,1,1,0)*dx - 128.D0*Hr4(0,1,1,1) + 16.D0*Hr4(0,1,1,1)*x&
         +64.D0*Hr4(0,1,1,1)*dx + 32.D0*Hr4(1,0,-1,0) - 16.D0*Hr4(1,0,-1,0)*x )

    ggq2 = ggq2 + CF*CA**2*(-32.D0*Hr4(1,0,-1,0)*dx - 48.D0*Hr4(1,0,0,0)&
         + 24.D0*Hr4(1,0,0,0)*x + 48.D0*Hr4(1,0,0,0)*dx - 128.D0&
         *Hr4(1,0,0,1) + 64.D0*Hr4(1,0,0,1)*x + 128.D0*Hr4(1,0,0,1)*&
         dx - 192.D0*Hr4(1,0,1,0) + 96.D0*Hr4(1,0,1,0)*x + 192.D0*Hr4(1,0,1,0)&
         *dx - 128.D0*Hr4(1,0,1,1) + 64.D0*Hr4(1,0,1,1)*x +&
         128.D0*Hr4(1,0,1,1)*dx - 64.D0*Hr4(1,1,0,0) + 32.D0*Hr4(1,1,0,0)&
         *x + 64.D0*Hr4(1,1,0,0)*dx - 128.D0*Hr4(1,1,0,1) + 64.D0*&
         Hr4(1,1,0,1)*x + 128.D0*Hr4(1,1,0,1)*dx - 128.D0*Hr4(1,1,1,0)&
         + 64.D0*Hr4(1,1,1,0)*x + 128.D0*Hr4(1,1,1,0)*dx - 64.D0*Hr4(1,1,1,1)&
         + 32.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)*dx )

    ggq2 = ggq2 + CF**2*CA*(-11629.D0/18.D0 + 3589.D0/36.D0*x +&
         56.D0*x**2 + 163.D0*dx - 1472.D0/3.D0*z3 - 884.D0/3.D0*z3*x&
         -32.D0*z3*x**2 + 216.D0*z3*dx + 2456.D0/9.D0*z2 - 5356.D0/9.D0&
         *z2*x + 96.D0*z2*x**2 - 160.D0*z2*dx - 784.D0/5.D0*z2**2 -&
         184.D0/5.D0*z2**2*x - 432.D0/5.D0*z2**2*dx + 176.D0*Hr1(-1)*&
         z3 + 88.D0*Hr1(-1)*z3*x + 176.D0*Hr1(-1)*z3*dx + 144.D0*Hr1(-1)&
         *z2 + 88.D0*Hr1(-1)*z2*x - 48.D0*Hr1(-1)*z2*dx - 9925.D0/&
         27.D0*Hr1(0) + 7061.D0/27.D0*Hr1(0)*x + 2680.D0/27.D0*Hr1(0)*&
         x**2 - 64.D0*Hr1(0)*z3 + 64.D0*Hr1(0)*z3*x - 752.D0/3.D0*Hr1(0)&
         *z2 - 440.D0/3.D0*Hr1(0)*z2*x - 16148.D0/27.D0*Hr1(1) +&
         10936.D0/27.D0*Hr1(1)*x + 1384.D0/27.D0*Hr1(1)*x**2 + 13006.D0&
         /27.D0*Hr1(1)*dx - 48.D0*Hr1(1)*z3 + 24.D0*Hr1(1)*z3*x + 48.D0&
         *Hr1(1)*z3*dx + 1144.D0/3.D0*Hr1(1)*z2 - 788.D0/3.D0*Hr1(1)*&
         z2*x - 472.D0/3.D0*Hr1(1)*z2*dx - 160.D0*Hr2(-1,-1)*z2 - 80.D0&
         *Hr2(-1,-1)*z2*x - 160.D0*Hr2(-1,-1)*z2*dx - 608.D0*Hr2(-1,0)&
         -700.D0*Hr2(-1,0)*x )

    ggq2 = ggq2 + CF**2*CA*(-32.D0*Hr2(-1,0)*dx - 32.D0*Hr2(-1,0)&
         *z2 - 16.D0*Hr2(-1,0)*z2*x - 32.D0*Hr2(-1,0)*z2*dx + 96.D0*&
         Hr2(0,-1)*z2 - 2590.D0/9.D0*Hr2(0,0) + 4631.D0/9.D0*Hr2(0,0)*&
         x - 448.D0/9.D0*Hr2(0,0)*x**2 - 16.D0*Hr2(0,0)*z2*x - 2456.D0/&
         9.D0*Hr2(0,1) - 944.D0/9.D0*Hr2(0,1)*x - 96.D0*Hr2(0,1)*x**2&
         + 128.D0*Hr2(0,1)*dx + 160.D0*Hr2(0,1)*z2 - 32.D0*Hr2(0,1)*&
         z2*x - 32.D0*Hr2(0,1)*z2*dx - 224.D0*Hr2(1,0) + 80.D0/3.D0*&
         Hr2(1,0)*x + 764.D0/3.D0*Hr2(1,0)*dx - 32.D0*Hr2(1,0)*z2 + 16.D0&
         *Hr2(1,0)*z2*x + 32.D0*Hr2(1,0)*z2*dx - 884.D0/9.D0*Hr2(1,1)&
         - 1448.D0/9.D0*Hr2(1,1)*x - 416.D0/9.D0*Hr2(1,1)*x**2 + 64.D0&
         /9.D0*Hr2(1,1)*dx + 64.D0*Hr2(1,1)*z2 - 32.D0*Hr2(1,1)*z2*x&
         -64.D0*Hr2(1,1)*z2*dx + 480.D0*Hr3(-1,-1,0) + 304.D0*Hr3(-1,-1,0)&
         *x + 96.D0*Hr3(-1,-1,0)*dx - 240.D0*Hr3(-1,0,0) - 144.D0&
         *Hr3(-1,0,0)*x - 48.D0*Hr3(-1,0,0)*dx + 96.D0*Hr3(-1,0,1) +&
         64.D0*Hr3(-1,0,1)*x + 96.D0*Hr3(-1,0,1)*dx - 576.D0*Hr3(0,-1,0) )

    ggq2 = ggq2 + CF**2*CA*(-16.D0*Hr3(0,-1,0)*x - 760.D0/3.D0*&
         Hr3(0,0,0) + 608.D0/3.D0*Hr3(0,0,0)*x - 128.D0/3.D0*Hr3(0,0,0)&
         *x**2 + 752.D0/3.D0*Hr3(0,0,1) + 392.D0/3.D0*Hr3(0,0,1)*x -&
         16.D0/3.D0*Hr3(0,1,0) + 224.D0/3.D0*Hr3(0,1,0)*x + 96.D0*Hr3(0,1,0)&
         *dx - 176.D0/3.D0*Hr3(0,1,1) + 448.D0/3.D0*Hr3(0,1,1)*x&
         +32.D0*Hr3(0,1,1)*x**2 + 144.D0*Hr3(0,1,1)*dx - 384.D0*Hr3(1,0,0)&
         + 232.D0*Hr3(1,0,0)*x + 128.D0*Hr3(1,0,0)*dx - 424.D0/&
         3.D0*Hr3(1,0,1) + 332.D0/3.D0*Hr3(1,0,1)*x + 328.D0/3.D0*Hr3(1,0,1)&
         *dx - 8.D0/3.D0*Hr3(1,1,0) + 148.D0/3.D0*Hr3(1,1,0)*x&
         +8.D0/3.D0*Hr3(1,1,0)*dx - 1112.D0/3.D0*Hr3(1,1,1) + 712.D0/&
         3.D0*Hr3(1,1,1)*x - 32.D0/3.D0*Hr3(1,1,1)*x**2 + 1120.D0/3.D0&
         *Hr3(1,1,1)*dx - 192.D0*Hr4(-1,-1,-1,0) - 96.D0*Hr4(-1,-1,-1,0)&
         *x - 192.D0*Hr4(-1,-1,-1,0)*dx + 128.D0*Hr4(-1,-1,0,0) + 64.D0&
         *Hr4(-1,-1,0,0)*x + 128.D0*Hr4(-1,-1,0,0)*dx + 64.D0*Hr4(-1,-1,0,1)&
         + 32.D0*Hr4(-1,-1,0,1)*x + 64.D0*Hr4(-1,-1,0,1)*dx&
         + 96.D0*Hr4(-1,0,-1,0) )

    ggq2 = ggq2 + CF**2*CA*(48.D0*Hr4(-1,0,-1,0)*x + 96.D0*Hr4(-1,0,-1,0)&
         *dx + 48.D0*Hr4(-1,0,0,0) + 24.D0*Hr4(-1,0,0,0)*x + 48.D0&
         *Hr4(-1,0,0,0)*dx - 64.D0*Hr4(-1,0,1,1) - 32.D0*Hr4(-1,0,1,1)&
         *x - 64.D0*Hr4(-1,0,1,1)*dx + 192.D0*Hr4(0,-1,-1,0) - 96.D0&
         *Hr4(0,-1,0,0) - 224.D0*Hr4(0,0,-1,0) + 16.D0*Hr4(0,0,-1,0)*x&
         +64.D0*Hr4(0,0,0,0) + 64.D0*Hr4(0,0,0,0)*x + 32.D0*Hr4(0,0,0,1)&
         *x - 64.D0*Hr4(0,0,1,0) + 32.D0*Hr4(0,0,1,0)*x - 112.D0*&
         Hr4(0,0,1,1) - 8.D0*Hr4(0,0,1,1)*x - 224.D0*Hr4(0,1,0,0) + 64.D0&
         *Hr4(0,1,0,0)*x + 64.D0*Hr4(0,1,0,0)*dx - 64.D0*Hr4(0,1,0,1)&
         + 32.D0*Hr4(0,1,0,1)*x + 32.D0*Hr4(0,1,0,1)*dx - 32.D0*Hr4(0,1,1,0)&
         + 16.D0*Hr4(0,1,1,0)*x + 32.D0*Hr4(0,1,1,0)*dx + 112.D0&
         *Hr4(0,1,1,1) - 8.D0*Hr4(0,1,1,1)*x - 64.D0*Hr4(0,1,1,1)*dx&
         -160.D0*Hr4(1,0,-1,0) + 80.D0*Hr4(1,0,-1,0)*x + 160.D0*Hr4(1,0,-1,0)&
         *dx - 80.D0*Hr4(1,0,0,0) + 40.D0*Hr4(1,0,0,0)*x + 80.D0&
         *Hr4(1,0,0,0)*dx + 64.D0*Hr4(1,0,1,1) - 32.D0*Hr4(1,0,1,1)*&
         x )

    ggq2 = ggq2 + CF**2*CA*(-64.D0*Hr4(1,0,1,1)*dx - 160.D0*Hr4(1,1,0,0)&
         + 80.D0*Hr4(1,1,0,0)*x + 160.D0*Hr4(1,1,0,0)*dx + 32.D0&
         *Hr4(1,1,0,1) - 16.D0*Hr4(1,1,0,1)*x - 32.D0*Hr4(1,1,0,1)*&
         dx + 96.D0*Hr4(1,1,1,0) - 48.D0*Hr4(1,1,1,0)*x - 96.D0*Hr4(1,1,1,0)&
         *dx + 128.D0*Hr4(1,1,1,1) - 64.D0*Hr4(1,1,1,1)*x - 128.D0&
         *Hr4(1,1,1,1)*dx )

    ggq2 = ggq2 + CF**3*(475.D0/2.D0-363.D0/4.D0*x-94.D0*dx +&
         416.D0*z3 + 208.D0*z3*x - 240.D0*z3*dx - 164.D0*z2 + 520.D0*&
         z2*x + 112.D0*z2*dx + 744.D0/5.D0*z2**2 + 12.D0*z2**2*x + 512.D0&
         /5.D0*z2**2*dx - 224.D0*Hr1(-1)*z2 - 160.D0*Hr1(-1)*z2*x -&
         96.D0*Hr1(-1)*z2*dx + 91.D0*Hr1(0) - 245.D0*Hr1(0)*x + 192.D0&
         *Hr1(0)*z3 - 48.D0*Hr1(0)*z3*x + 64.D0*Hr1(0)*z2 + 36.D0*Hr1(0)&
         *z2*x + 316.D0*Hr1(1) - 258.D0*Hr1(1)*x - 94.D0*Hr1(1)*dx&
         + 256.D0*Hr1(1)*z3 - 128.D0*Hr1(1)*z3*x - 256.D0*Hr1(1)*z3*&
         dx - 256.D0*Hr1(1)*z2 + 152.D0*Hr1(1)*z2*x + 96.D0*Hr1(1)*z2*&
         dx + 688.D0*Hr2(-1,0) + 608.D0*Hr2(-1,0)*x + 112.D0*Hr2(-1,0)&
         *dx + 64.D0*Hr2(-1,0)*z2 + 32.D0*Hr2(-1,0)*z2*x + 64.D0*Hr2(-1,0)&
         *z2*dx - 64.D0*Hr2(0,-1)*z2 - 70.D0*Hr2(0,0) - 555.D0*&
         Hr2(0,0)*x + 48.D0*Hr2(0,0)*z2 - 24.D0*Hr2(0,0)*z2*x + 164.D0&
         *Hr2(0,1) + 88.D0*Hr2(0,1)*x - 96.D0*Hr2(0,1)*z2 + 16.D0*Hr2(0,1)&
         *z2*x - 16.D0*Hr2(1,0) - 24.D0*Hr2(1,0)*x - 56.D0*Hr2(1,0)*dx )

    ggq2 = ggq2 + CF**3*(64.D0*Hr2(1,0)*z2 - 32.D0*Hr2(1,0)*z2*x&
         -64.D0*Hr2(1,0)*z2*dx + 96.D0*Hr2(1,1) + 54.D0*Hr2(1,1)*x&
         -92.D0*Hr2(1,1)*dx - 96.D0*Hr2(1,1)*z2 + 48.D0*Hr2(1,1)*z2*&
         x + 96.D0*Hr2(1,1)*z2*dx - 448.D0*Hr3(-1,-1,0) - 320.D0*Hr3(-1,-1,0)&
         *x - 192.D0*Hr3(-1,-1,0)*dx + 384.D0*Hr3(-1,0,0) +&
         224.D0*Hr3(-1,0,0)*x + 192.D0*Hr3(-1,0,0)*dx + 480.D0*Hr3(0,-1,0)&
         + 80.D0*Hr3(0,-1,0)*x + 16.D0*Hr3(0,0,0) - 132.D0*Hr3(0,0,0)&
         *x - 64.D0*Hr3(0,0,1) + 44.D0*Hr3(0,0,1)*x + 48.D0*Hr3(0,1,0)&
         *x - 16.D0*Hr3(0,1,1) - 52.D0*Hr3(0,1,1)*x + 192.D0*Hr3(1,0,0)&
         - 108.D0*Hr3(1,0,0)*x - 48.D0*Hr3(1,0,0)*dx + 32.D0*&
         Hr3(1,0,1) + 8.D0*Hr3(1,0,1)*x - 96.D0*Hr3(1,1,0) + 64.D0*&
         Hr3(1,1,0)*x + 96.D0*Hr3(1,1,0)*dx + 160.D0*Hr3(1,1,1) - 124.D0&
         *Hr3(1,1,1)*x - 144.D0*Hr3(1,1,1)*dx - 128.D0*Hr4(-1,-1,0,0)&
         - 64.D0*Hr4(-1,-1,0,0)*x - 128.D0*Hr4(-1,-1,0,0)*dx - 64.D0*&
         Hr4(-1,0,-1,0) - 32.D0*Hr4(-1,0,-1,0)*x - 64.D0*Hr4(-1,0,-1,0)*dx )

    ggq2 = ggq2 + CF**3*(32.D0*Hr4(-1,0,0,0) + 16.D0*Hr4(-1,0,0,0)&
         *x + 32.D0*Hr4(-1,0,0,0)*dx - 128.D0*Hr4(0,-1,-1,0) + 128.D0*&
         Hr4(0,-1,0,0) + 32.D0*Hr4(0,-1,0,0)*x + 256.D0*Hr4(0,0,-1,0)&
         -32.D0*Hr4(0,0,0,0) - 32.D0*Hr4(0,0,0,0)*x - 48.D0*Hr4(0,0,0,1)&
         + 24.D0*Hr4(0,0,0,1)*x + 48.D0*Hr4(0,0,1,1) - 24.D0*Hr4(0,0,1,1)&
         *x + 80.D0*Hr4(0,1,0,0) - 8.D0*Hr4(0,1,0,0)*x + 32.D0&
         *Hr4(0,1,0,1) - 16.D0*Hr4(0,1,0,1)*x + 16.D0*Hr4(0,1,1,1) - 8.D0&
         *Hr4(0,1,1,1)*x + 192.D0*Hr4(1,0,-1,0) - 96.D0*Hr4(1,0,-1,0)&
         *x - 192.D0*Hr4(1,0,-1,0)*dx + 64.D0*Hr4(1,0,1,0) - 32.D0*&
         Hr4(1,0,1,0)*x - 64.D0*Hr4(1,0,1,0)*dx + 64.D0*Hr4(1,0,1,1)&
         -32.D0*Hr4(1,0,1,1)*x - 64.D0*Hr4(1,0,1,1)*dx + 96.D0*Hr4(1,1,0,0)&
         - 48.D0*Hr4(1,1,0,0)*x - 96.D0*Hr4(1,1,0,0)*dx + 96.D0&
         *Hr4(1,1,0,1) - 48.D0*Hr4(1,1,0,1)*x - 96.D0*Hr4(1,1,0,1)*dx&
         +32.D0*Hr4(1,1,1,0) - 16.D0*Hr4(1,1,1,0)*x - 32.D0*Hr4(1,1,1,0)&
         *dx - 64.D0*Hr4(1,1,1,1) + 32.D0*Hr4(1,1,1,1)*x + 64.D0*&
         Hr4(1,1,1,1)*dx )

    ggq2 = ggq2 + twonfTF*CF*CA*(-2320.D0/9.D0 + 148.D0/3.D0*x - 1048.D0&
         /9.D0*x**2 + 1934.D0/9.D0*dx + 448.D0/3.D0*z3 - 184.D0/3.D0&
         *z3*x - 80.D0*z3*dx - 256.D0/9.D0*z2 + 308.D0/9.D0*z2*x + 32.D0&
         /3.D0*z2*x**2 - 112.D0/3.D0*z2*dx - 128.D0/3.D0*Hr1(-1)*z2&
         -64.D0/3.D0*Hr1(-1)*z2*x - 128.D0/3.D0*Hr1(-1)*z2*dx + 6656.D0&
         /27.D0*Hr1(0) + 1688.D0/27.D0*Hr1(0)*x + 2000.D0/27.D0*Hr1(0)&
         *x**2 + 1208.D0/27.D0*Hr1(0)*dx - 64.D0/3.D0*Hr1(0)*z2 - 80.D0&
         /3.D0*Hr1(0)*z2*x - 32.D0/3.D0*Hr1(0)*z2*dx - 1184.D0/27.D0&
         *Hr1(1) + 1720.D0/27.D0*Hr1(1)*x + 400.D0/27.D0*Hr1(1)*x**2&
         +2008.D0/27.D0*Hr1(1)*dx + 16.D0/3.D0*Hr1(1)*z2 - 8.D0/3.D0&
         *Hr1(1)*z2*x - 16.D0/3.D0*Hr1(1)*z2*dx - 80.D0/3.D0*Hr2(-1,0)&
         +8.D0*Hr2(-1,0)*x - 32.D0/9.D0*Hr2(-1,0)*x**2 - 656.D0/9.D0&
         *Hr2(-1,0)*dx - 1088.D0/9.D0*Hr2(0,0) - 668.D0/9.D0*Hr2(0,0)*&
         x - 32.D0/9.D0*Hr2(0,0)*x**2 + 256.D0/9.D0*Hr2(0,1) - 236.D0/&
         9.D0*Hr2(0,1)*x - 32.D0/3.D0*Hr2(0,1)*x**2 - 320.D0/9.D0*Hr2(0,1)*dx )

    ggq2 = ggq2 + twonfTF*CF*CA*(320.D0/9.D0*Hr2(1,0) - 280.D0/9.D0*&
         Hr2(1,0)*x - 320.D0/9.D0*Hr2(1,0)*dx + 880.D0/9.D0*Hr2(1,1)&
         -488.D0/9.D0*Hr2(1,1)*x + 32.D0/9.D0*Hr2(1,1)*x**2 - 104.D0&
         *Hr2(1,1)*dx - 128.D0/3.D0*Hr3(-1,-1,0) - 64.D0/3.D0*Hr3(-1,-1,0)&
         *x - 128.D0/3.D0*Hr3(-1,-1,0)*dx + 64.D0/3.D0*Hr3(-1,0,0)&
         + 32.D0/3.D0*Hr3(-1,0,0)*x + 64.D0/3.D0*Hr3(-1,0,0)*dx + 64.D0&
         /3.D0*Hr3(-1,0,1) + 32.D0/3.D0*Hr3(-1,0,1)*x + 64.D0/3.D0*&
         Hr3(-1,0,1)*dx + 160.D0/3.D0*Hr3(0,-1,0) - 16.D0*Hr3(0,-1,0)*&
         x - 32.D0/3.D0*Hr3(0,-1,0)*dx + 32.D0/3.D0*Hr3(0,0,0) + 64.D0/&
         3.D0*Hr3(0,0,1) + 32.D0/3.D0*Hr3(0,0,1)*x - 32.D0/3.D0*Hr3(0,1,0)&
         + 16.D0/3.D0*Hr3(0,1,0)*x + 32.D0/3.D0*Hr3(0,1,0)*dx -&
         48.D0*Hr3(0,1,1) + 8.D0*Hr3(0,1,1)*x + 80.D0/3.D0*Hr3(0,1,1)*&
         dx + 32.D0/3.D0*Hr3(1,0,0) - 16.D0/3.D0*Hr3(1,0,0)*x - 32.D0/&
         3.D0*Hr3(1,0,0)*dx - 80.D0/3.D0*Hr3(1,0,1) + 40.D0/3.D0*Hr3(1,0,1)&
         *x + 80.D0/3.D0*Hr3(1,0,1)*dx - 80.D0/3.D0*Hr3(1,1,0) +&
         40.D0/3.D0*Hr3(1,1,0)*x )

    ggq2 = ggq2 + twonfTF*CF*CA*(80.D0/3.D0*Hr3(1,1,0)*dx - 80.D0/3.D0*&
         Hr3(1,1,1) + 40.D0/3.D0*Hr3(1,1,1)*x + 80.D0/3.D0*Hr3(1,1,1)*&
         dx )

    ggq2 = ggq2 + twonfTF*CF**2*(8707.D0/27.D0 + 3301.D0/54.D0*x - 224.D0&
         /9.D0*x**2 - 3250.D0/9.D0*dx - 496.D0/3.D0*z3 + 248.D0/3.D0*&
         z3*x + 96.D0*z3*dx - 128.D0/9.D0*z2 - 560.D0/9.D0*z2*x + 496.D0&
         /9.D0*z2*dx - 5174.D0/27.D0*Hr1(0) - 3134.D0/27.D0*Hr1(0)*x&
         -352.D0/27.D0*Hr1(0)*x**2 - 1520.D0/27.D0*Hr1(0)*dx + 32.D0/&
         3.D0*Hr1(0)*z2 + 80.D0/3.D0*Hr1(0)*z2*x + 64.D0/3.D0*Hr1(0)*&
         z2*dx + 8.D0/27.D0*Hr1(1) - 1360.D0/27.D0*Hr1(1)*x - 56.D0/27.D0&
         *Hr1(1)*dx + 32.D0/3.D0*Hr1(1)*z2 - 16.D0/3.D0*Hr1(1)*z2*x&
         -32.D0/3.D0*Hr1(1)*z2*dx - 48.D0*Hr2(-1,0)*x + 64.D0/9.D0*&
         Hr2(-1,0)*x**2 + 496.D0/9.D0*Hr2(-1,0)*dx + 2236.D0/9.D0*Hr2(0,0)&
         + 1306.D0/9.D0*Hr2(0,0)*x + 64.D0/9.D0*Hr2(0,0)*x**2 +&
         128.D0/9.D0*Hr2(0,1) + 128.D0/9.D0*Hr2(0,1)*x - 8.D0/3.D0*&
         Hr2(1,0)*x - 32.D0/3.D0*Hr2(1,0)*dx - 712.D0/9.D0*Hr2(1,1) +&
         512.D0/9.D0*Hr2(1,1)*x + 664.D0/9.D0*Hr2(1,1)*dx - 64.D0*Hr3(0,-1,0)&
         + 32.D0*Hr3(0,-1,0)*x + 64.D0/3.D0*Hr3(0,-1,0)*dx +&
         16.D0/3.D0*Hr3(0,0,0) )

    ggq2 = ggq2 + twonfTF*CF**2*(-32.D0/3.D0*Hr3(0,0,0)*x - 32.D0/3.D0&
         *Hr3(0,0,1) + 16.D0/3.D0*Hr3(0,0,1)*x - 32.D0/3.D0*Hr3(0,1,0)&
         +16.D0/3.D0*Hr3(0,1,0)*x - 16.D0/3.D0*Hr3(0,1,1) + 8.D0/3.D0&
         *Hr3(0,1,1)*x - 16.D0*Hr3(1,0,0) + 8.D0*Hr3(1,0,0)*x + 16.D0*&
         Hr3(1,0,0)*dx - 32.D0/3.D0*Hr3(1,0,1) + 16.D0/3.D0*Hr3(1,0,1)&
         *x + 32.D0/3.D0*Hr3(1,0,1)*dx - 64.D0/3.D0*Hr3(1,1,0) + 32.D0/&
         3.D0*Hr3(1,1,0)*x + 64.D0/3.D0*Hr3(1,1,0)*dx + 80.D0/3.D0*&
         Hr3(1,1,1) - 40.D0/3.D0*Hr3(1,1,1)*x - 80.D0/3.D0*Hr3(1,1,1)*&
         dx + 64.D0*Hr4(0,0,0,0) - 32.D0*Hr4(0,0,0,0)*x )

    ggq2 = ggq2 + twonfTF**2*CF*(16.D0/9.D0 + 32.D0/9.D0*x - 16.D0/9.D0&
         *dx + 80.D0/9.D0*Hr1(1) - 64.D0/9.D0*Hr1(1)*x - 80.D0/9.D0*&
         Hr1(1)*dx - 16.D0/3.D0*Hr2(1,1) + 8.D0/3.D0*Hr2(1,1)*x + 16.D0&
         /3.D0*Hr2(1,1)*dx )

    res=ggq2/8d0

    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P2gq

  ! Space like
  ! P^{(2)}_{gg}
  ! Eq.(4.15) in hep-ph/0404111
  ! Expressions are copyed from subroutines X2QGA, X2GGC in xpij2e.f
  ! by A. Vogt, S. Moch and J. Vermaseren, hep-ph/0404111
  FUNCTION AP_SFS_P2gg(y,y_is_x) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
    REAL(KIND(1d0))::x,DX,DM,DP
    REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4,Z5=zeta5
    REAL(KIND(1d0))::ggg2,GGG2L,A3G,X2GGA,P2GDELT,twoTf,twonfTf
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)THEN
       twoTf=two*TF
       twonfTf=two*nfTF
       P2GDELT =&
            79.D0/2.D0*CA**3&
            +8.D0/3.D0*z2*CA**3&
            +22.D0/3.D0*z2**2*CA**3&
            +536.D0/3.D0*z3*CA**3&
            -16.D0*z2*z3*CA**3&
            -80.D0*z5*CA**3&
            +CF**2*twonfTF&
            -233.D0/18*CA**2*twonfTF&
            -8.D0/3.D0*z2*CA**2*twonfTF&
            -80.D0/3.D0*z3*CA**2*twonfTF&
            -4.D0/3.D0*z2**2*CA**2*twonfTF&
            -241.D0/18.D0*CA*CF*twonfTF&
            +29.D0/18.D0*CA*twonfTF**2&
            +11.D0/9.D0*CF*twonfTF**2

       res=P2GDELT/8d0
       RETURN
    ENDIF
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT.AND.cc_piece.NE.cc_PLUS0)RETURN

    DX=one/x
    DM=one/(one-x)
    DP=one/(one+x)
    twoTf=two*TF
    twonfTf=two*nfTF

    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT,cc_REAL)

       CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
            HI1,HI2,HI3,HI4, N1, N2)

       ggg2 =&
            CA**3 * (-54088.D0/27.D0 + 49678.D0/27.D0*x - 146182.D0/81.D0&
            *x**2 + 146182.D0/81.D0*dx + 1064.D0/3.D0*&
            z3 + 704.D0/3.D0*z3*x + 1408.D0/3.D0*z3*x**2 - 1144.D0/3.D0*&
            z3*dx + 976.D0/3.D0*z2 - 2632.D0/9.D0*z2*&
            x + 1072.D0/9.D0*z2*x**2 - 3112.D0/9.D0*z2*dx - 1072.D0/9.D0*&
            z2*dp - 272.D0*z2**2 - 1152.D0/5.D0*&
            z2**2*x - 416.D0/5.D0*z2**2*x**2 - 464.D0/5.D0*z2**2*dx + 88.D0&
            *z2**2*dp - 384.D0*Hr1(-1)*z3 - 192.D0&
            *Hr1(-1)*z3*x - 192.D0*Hr1(-1)*z3*x**2 - 192.D0*Hr1(-1)*z3*dx&
            +192.D0*Hr1(-1)*z3*dp - 424.D0*Hr1(-1)*z2 - 424.D0*Hr1(-1)*&
            z2*x - 88.D0*Hr1(-1)*z2*x**2 - 88.D0*Hr1(-1)*z2*dx + 30794.D0/&
            27.D0*Hr1(0) + 9374.D0/27.D0*Hr1(0)*x + 10604.D0/9.D0*Hr1(0)*&
            x**2 + 6320.D0/27.D0*Hr1(0)*dx + 8.D0/3.D0*Hr1(0)*dm - 160.D0&
            *Hr1(0)*z3*x + 192.D0*Hr1(0)*z3*x**2 - 32.D0*Hr1(0)*z3*dx -&
            80.D0*Hr1(0)*z3*dp - 112.D0*Hr1(0)*z3*dm + 1480.D0/3.D0*Hr1(0)*z2 )

       ggg2 = ggg2 + CA**3*(736.D0/3.D0*Hr1(0)*z2*x + 1936.D0/3.D0*&
            Hr1(0)*z2*x**2 - 176.D0/3.D0*Hr1(0)*z2*dx - 88.D0/3.D0*Hr1(0)&
            *z2*dp + 124.D0/9.D0*Hr1(1) - 124.D0/9.D0*Hr1(1)*x + 1652.D0/&
            27.D0*Hr1(1)*x**2 - 1652.D0/27.D0*Hr1(1)*dx + 192.D0*Hr1(1)*&
            z3 - 96.D0*Hr1(1)*z3*x + 96.D0*Hr1(1)*z3*x**2 - 96.D0*Hr1(1)*&
            z3*dx - 24.D0*Hr1(1)*z2 + 24.D0*Hr1(1)*&
            z2*x + 88.D0*Hr1(1)*z2*x**2 - 88.D0*Hr1(1)*z2*dx + 512.D0*&
            Hr2(-1,-1)*z2 + 256.D0*Hr2(-1,-1)*z2*x + 256.D0*Hr2(-1,-1)*z2&
            *x**2 + 256.D0*Hr2(-1,-1)*z2*dx - 256.D0*Hr2(-1,-1)*z2*dp -&
            872.D0/9.D0*Hr2(-1,0) - 3016.D0/9.D0*Hr2(-1,0)*x - 680.D0/3.D0&
            *Hr2(-1,0)*x**2 - 680.D0/3.D0*Hr2(-1,0)*dx - 2144.D0/9.D0*&
            Hr2(-1,0)*dp - 576.D0*Hr2(-1,0)*z2 - 288.D0*Hr2(-1,0)*z2*x -&
            288.D0*Hr2(-1,0)*z2*x**2 - 288.D0*Hr2(-1,0)*z2*dx + 288.D0*&
            Hr2(-1,0)*z2*dp - 768.D0*Hr2(0,-1)*z2 - 96.D0*Hr2(0,-1)*z2*x&
            -352.D0*Hr2(0,-1)*z2*x**2 - 160.D0*Hr2(0,-1)*z2*dx + 256.D0&
            *Hr2(0,-1)*z2*dp )

       ggg2 = ggg2 + CA**3*(- 11620.D0/9.D0*&
            Hr2(0,0) + 1052.D0/3.D0*Hr2(0,0)*x - 12136.D0/9.D0*Hr2(0,0)*&
            x**2 + 1072.D0/9.D0*Hr2(0,0)*dp + 1072.D0/9.D0*Hr2(0,0)*dm +&
            96.D0*Hr2(0,0)*z2 - 352.D0*Hr2(0,0)*z2*x + 256.D0*Hr2(0,0)*z2&
            *x**2 - 128.D0*Hr2(0,0)*z2*dp - 128.D0*Hr2(0,0)*z2*dm - 976.D0&
            /3.D0*Hr2(0,1) - 128.D0/3.D0*Hr2(0,1)*x - 1072.D0/9.D0*Hr2(0,1)&
            *x**2 + 1072.D0/9.D0*Hr2(0,1)*dx&
            -160.D0*Hr2(0,1)*z2*x + 32.D0*Hr2(0,1)*z2*x**2 - 96.D0*Hr2(0,1)&
            *z2*dx + 32.D0*Hr2(0,1)*z2*dp -&
            2344.D0/9.D0*Hr2(1,0) + 200.D0/9.D0*Hr2(1,0)*x - 1072.D0/9.D0&
            *Hr2(1,0)*x**2 + 1072.D0/9.D0*Hr2(1,0)*dx&
            + 192.D0*Hr2(1,0)*z2 - 96.D0*Hr2(1,0)*z2*x + 96.D0*&
            Hr2(1,0)*z2*x**2 - 96.D0*Hr2(1,0)*z2*dx&
            - 48.D0*Hr3(-1,-1,0) - 48.D0*Hr3(-1,-1,0)*x + 176.D0*Hr3(-1,-1,0)&
            *x**2 + 176.D0*Hr3(-1,-1,0)*dx + 608.D0*Hr3(-1,0,0)&
            + 608.D0*Hr3(-1,0,0)*x )

       ggg2 = ggg2 + CA**3*(1408.D0/3.D0*Hr3(-1,0,0)*x**2 + 1408.D0/3.D0&
            *Hr3(-1,0,0)*dx + 400.D0*Hr3(-1,0,1) + 400.D0*Hr3(-1,0,1)*x&
            +176.D0*Hr3(-1,0,1)*x**2 + 176.D0*Hr3(-1,0,1)*dx + 640.D0/3.D0&
            *Hr3(0,-1,0) + 160.D0*Hr3(0,-1,0)*x + 352.D0*Hr3(0,-1,0)*&
            x**2 - 176.D0/3.D0*Hr3(0,-1,0)*dx &
            - 368.D0*Hr3(0,0,0) + 464.D0/3.D0*Hr3(0,0,0)*x - 704.D0*&
            Hr3(0,0,0)*x**2 - 1480.D0/3.D0*Hr3(0,0,1) - 256.D0/3.D0*Hr3(0,0,1)&
            *x - 1936.D0/3.D0*Hr3(0,0,1)*x**2&
            - 688.D0/3.D0*Hr3(0,1,0) + 464.D0/3.D0*Hr3(0,1,0)*x -&
            352.D0*Hr3(0,1,0)*x**2 + 352.D0/3.D0*Hr3(0,1,0)*dx - 776.D0/3.D0&
            *Hr3(1,0,0) + 688.D0/3.D0*Hr3(1,0,0)*x - 880.D0/3.D0*Hr3(1,0,0)&
            *x**2 + 880.D0/3.D0*Hr3(1,0,0)*dx&
            - 768.D0*Hr4(-1,-1,0,0) - 384.D0*Hr4(-1,-1,0,0)*x - 384.D0&
            *Hr4(-1,-1,0,0)*x**2 - 384.D0*Hr4(-1,-1,0,0)*dx + 384.D0*Hr4(-1,-1,0,0)&
            *dp - 512.D0*Hr4(-1,-1,0,1) - 256.D0*Hr4(-1,-1,0,1)&
            *x )

       ggg2 = ggg2 + CA**3*(-256.D0*Hr4(-1,-1,0,1)*x**2 - 256.D0*&
            Hr4(-1,-1,0,1)*dx + 256.D0*Hr4(-1,-1,0,1)*dp - 256.D0*Hr4(-1,0,-1,0)&
            - 128.D0*Hr4(-1,0,-1,0)*x - 128.D0*Hr4(-1,0,-1,0)*&
            x**2 - 128.D0*Hr4(-1,0,-1,0)*dx + 128.D0*Hr4(-1,0,-1,0)*dp +&
            512.D0*Hr4(-1,0,0,0) + 256.D0*Hr4(-1,0,0,0)*x + 256.D0*Hr4(-1,0,0,0)&
            *x**2 + 256.D0*Hr4(-1,0,0,0)*dx - 256.D0*Hr4(-1,0,0,0)&
            *dp + 512.D0*Hr4(-1,0,0,1) + 256.D0*Hr4(-1,0,0,1)*x + 256.D0*&
            Hr4(-1,0,0,1)*x**2 + 256.D0*Hr4(-1,0,0,1)*dx - 256.D0*Hr4(-1,0,0,1)&
            *dp + 128.D0*Hr4(-1,0,1,0) + 64.D0*Hr4(-1,0,1,0)*x + 64.D0&
            *Hr4(-1,0,1,0)*x**2 + 64.D0*Hr4(-1,0,1,0)*dx - 64.D0*Hr4(-1,0,1,0)&
            *dp - 512.D0*Hr4(0,-1,-1,0) + 64.D0*Hr4(0,-1,-1,0)*x&
            -192.D0*Hr4(0,-1,-1,0)*x**2 - 64.D0*Hr4(0,-1,-1,0)*dx + 128.D0&
            *Hr4(0,-1,-1,0)*dp + 512.D0*Hr4(0,-1,0,0)&
            + 416.D0*Hr4(0,-1,0,0)*x + 352.D0*Hr4(0,-1,0,0)*x**2&
            + 224.D0*Hr4(0,-1,0,0)*dx - 288.D0*Hr4(0,-1,0,0)*dp)
       
       ggg2 = ggg2 + CA**3*(512.D0*Hr4(0,-1,0,1) + 128.D0*Hr4(0,-1,0,1)*x&
            + 256.D0*Hr4(0,-1,0,1)*x**2 + 128.D0*Hr4(0,-1,0,1)*dx -&
            192.D0*Hr4(0,-1,0,1)*dp - 64.D0*Hr4(0,0,-1,0)&
            + 64.D0*Hr4(0,0,-1,0)*x + 128.D0*Hr4(0,0,-1,0)*x**2&
            -64.D0*Hr4(0,0,-1,0)*dp - 256.D0*&
            Hr4(0,0,0,0) + 448.D0*Hr4(0,0,0,0)*x - 128.D0*Hr4(0,0,0,0)*&
            x**2 + 64.D0*Hr4(0,0,0,0)*dp + 64.D0*Hr4(0,0,0,0)*dm - 96.D0*&
            Hr4(0,0,0,1) + 416.D0*Hr4(0,0,0,1)*x - 256.D0*Hr4(0,0,0,1)*&
            x**2 + 128.D0*Hr4(0,0,0,1)*dp - 128.D0&
            *Hr4(0,0,1,0) + 384.D0*Hr4(0,0,1,0)*x - 192.D0*Hr4(0,0,1,0)*&
            x**2 + 128.D0*Hr4(0,0,1,0)*dx + 32.D0*Hr4(0,0,1,0)*dp&
            - 256.D0*Hr4(0,0,1,1) + 128.D0*Hr4(0,0,1,1)*&
            x - 128.D0*Hr4(0,0,1,1)*x**2 + 128.D0*Hr4(0,0,1,1)*dx&
            - 160.D0*Hr4(0,1,0,0) + 320.D0*Hr4(0,1,0,0)*&
            x - 160.D0*Hr4(0,1,0,0)*x**2 + 160.D0*Hr4(0,1,0,0)*dx )

       ggg2 = ggg2 + CA**3*(-256.D0*Hr4(0,1,0,1) + 128.D0*Hr4(0,1,0,1)&
            *x - 128.D0*Hr4(0,1,0,1)*x**2 + 128.D0*Hr4(0,1,0,1)*dx&
            - 256.D0*Hr4(0,1,1,0) + 128.D0*Hr4(0,1,1,0)&
            *x - 128.D0*Hr4(0,1,1,0)*x**2 + 128.D0*Hr4(0,1,1,0)*dx&
            + 128.D0*Hr4(1,0,-1,0) - 64.D0*Hr4(1,0,-1,0)&
            *x + 64.D0*Hr4(1,0,-1,0)*x**2 - 64.D0*Hr4(1,0,-1,0)*&
            dx - 256.D0*Hr4(1,0,0,0) + 128.D0*&
            Hr4(1,0,0,0)*x - 128.D0*Hr4(1,0,0,0)*x**2 + 128.D0*Hr4(1,0,0,0)&
            *dx - 256.D0*Hr4(1,0,0,1) + 128.D0&
            *Hr4(1,0,0,1)*x - 128.D0*Hr4(1,0,0,1)*x**2 + 128.D0*Hr4(1,0,0,1)&
            *dx - 256.D0*Hr4(1,0,1,0) + 128.D0&
            *Hr4(1,0,1,0)*x - 128.D0*Hr4(1,0,1,0)*x**2 + 128.D0*Hr4(1,0,1,0)&
            *dx - 256.D0*Hr4(1,1,0,0) + 128.D0&
            *Hr4(1,1,0,0)*x - 128.D0*Hr4(1,1,0,0)*x**2 + 128.D0*Hr4(1,1,0,0)&
            *dx)

       ggg2 = ggg2 + twonfTF*CA**2*(-2174.D0/9.D0 + 7358.D0/27.D0*x -&
            19264.D0/81.D0*x**2 + 19264.D0/81.D0*dx +&
            496.D0/3.D0*z3 + 112.D0/3.D0*z3*x + 128.D0/3.D0*z3*x**2 - 176.D0&
            /3.D0*z3*dx - 1700.D0/9.D0*z2 + 1076.D0/&
            9.D0*z2*x - 176.D0/3.D0*z2*x**2 - 128.D0/3.D0*z2*dx + 160.D0/&
            9.D0*z2*dp + 276.D0/5.D0*z2**2 + 156.D0/5.D0&
            *z2**2*x - 88.D0*Hr1(-1)*z2 - 88.D0*Hr1(-1)*z2*x + 16.D0/3.D0&
            *Hr1(-1)*z2*x**2 + 16.D0/3.D0*Hr1(-1)*z2*dx + 7472.D0/27.D0*&
            Hr1(0) - 3958.D0/27.D0*Hr1(0)*x + 2612.D0/9.D0*Hr1(0)*x**2 +&
            1136.D0/27.D0*Hr1(0)*dx - 8.D0/3.D0*Hr1(0)*dm - 96.D0*Hr1(0)*&
            z3*x - 4.D0/3.D0*Hr1(0)*z2 + 140.D0/3.D0*Hr1(0)*z2*x + 32.D0/&
            3.D0*Hr1(0)*z2*x**2 - 32.D0/3.D0*Hr1(0)*z2*dx + 16.D0/3.D0*&
            Hr1(0)*z2*dp + 1354.D0/9.D0*Hr1(1) - 1354.D0/9.D0*Hr1(1)*x -&
            820.D0/27.D0*Hr1(1)*x**2 + 820.D0/27.D0*Hr1(1)*dx - 56.D0*&
            Hr1(1)*z2 + 56.D0*Hr1(1)*z2*x + 16.D0*Hr1(1)*z2*x**2 - 16.D0*&
            Hr1(1)*z2*dx )

       ggg2 = ggg2 + twonfTF*CA**2*(1208.D0/9.D0*Hr2(-1,0) + 1528.D0/9.D0*&
            Hr2(-1,0)*x - 304.D0/3.D0*Hr2(-1,0)*x**2 - 304.D0/3.D0*Hr2(-1,0)&
            *dx + 320.D0/9.D0*Hr2(-1,0)*dp - 48.D0*Hr2(0,-1)*z2 + 48.D0&
            *Hr2(0,-1)*z2*x - 208.D0/9.D0*Hr2(0,0) - 2420.D0/9.D0*Hr2(0,0)&
            *x + 496.D0/9.D0*Hr2(0,0)*x**2 - 160.D0/9.D0*Hr2(0,0)*dp -&
            160.D0/9.D0*Hr2(0,0)*dm - 24.D0*Hr2(0,0)*z2 - 40.D0*Hr2(0,0)*&
            z2*x + 1700.D0/9.D0*Hr2(0,1) + 452.D0/9.D0*Hr2(0,1)*x + 176.D0&
            /3.D0*Hr2(0,1)*x**2 - 176.D0/3.D0*Hr2(0,1)*dx&
            - 48.D0*Hr2(0,1)*z2 - 48.D0*Hr2(0,1)*z2*x + 784.D0&
            /9.D0*Hr2(1,0) - 464.D0/9.D0*Hr2(1,0)*x + 176.D0/3.D0*Hr2(1,0)&
            *x**2 - 176.D0/3.D0*Hr2(1,0)*dx-&
            112.D0*Hr3(-1,-1,0) - 112.D0*Hr3(-1,-1,0)*x + 32.D0*Hr3(-1,-1,0)&
            *x**2 + 32.D0*Hr3(-1,-1,0)*dx + 72.D0*Hr3(-1,0,0) + 72.D0*&
            Hr3(-1,0,0)*x - 32.D0/3.D0*Hr3(-1,0,0)*x**2 - 32.D0/3.D0*Hr3(-1,0,0)&
            *dx + 32.D0*Hr3(-1,0,1) + 32.D0*Hr3(-1,0,1)*x + 32.D0/&
            3.D0*Hr3(-1,0,1)*x**2 )
       
       ggg2 = ggg2 + twonfTF*CA**2*(32.D0/3.D0*Hr3(-1,0,1)*dx + 440.D0/3.D0&
            *Hr3(0,-1,0) - 24.D0*Hr3(0,-1,0)*x - 64.D0/3.D0*Hr3(0,-1,0)*&
            x**2 - 32.D0/3.D0*Hr3(0,-1,0)*dx&
            + 32.D0*Hr3(0,0,0) - 152.D0/3.D0*Hr3(0,0,0)*x + 4.D0/3.D0*&
            Hr3(0,0,1) - 212.D0/3.D0*Hr3(0,0,1)*x - 32.D0/3.D0*Hr3(0,0,1)&
            *x**2 - 32.D0/3.D0*Hr3(0,1,0) - 32.D0&
            /3.D0*Hr3(0,1,0)*x + 116.D0/3.D0*Hr3(1,0,0) - 100.D0/3.D0*&
            Hr3(1,0,0)*x - 96.D0*Hr4(0,-1,-1,0)&
            + 96.D0*Hr4(0,-1,-1,0)*x + 48.D0*Hr4(0,-1,0,0) - 48.D0*Hr4(0,-1,0,0)&
            *x + 80.D0*Hr4(0,0,-1,0) - 16.D0*Hr4(0,0,-1,0)*x +&
            16.D0*Hr4(0,0,0,0)*x + 24.D0*Hr4(0,0,0,1) + 24.D0*Hr4(0,0,0,1)&
            *x + 24.D0*Hr4(0,1,0,0) + 24.D0*Hr4(0,1,0,0)*x )
       
       ggg2 = ggg2 + twonfTF*CF*CA*(19204.D0/27.D0 + 182.D0/27.D0*x -&
            24526.D0/81.D0*x**2 - 30662.D0/81.D0*dx-&
            464.D0/3.D0*z3 - 1208.D0/3.D0*z3*x + 32.D0/3.D0*z3*x**2 + 224.D0&
            /3.D0*z3*dx + 2696.D0/9.D0*z2 - 3964.D0/9.D0*&
            z2*x - 160.D0/9.D0*z2*x**2 + 608.D0/9.D0*z2*dx - 664.D0/5.D0*&
            z2**2 - 344.D0/5.D0*z2**2*x + 208.D0*Hr1(-1)*z2 + 208.D0*Hr1(-1)&
            *z2*x - 64.D0/3.D0*Hr1(-1)*z2*x**2 - 64.D0/3.D0*Hr1(-1)*z2&
            *dx + 3220.D0/27.D0*Hr1(0) + 13804.D0/27.D0*Hr1(0)*x + 112.D0&
            *Hr1(0)*x**2 - 1376.D0/27.D0*Hr1(0)*dx - 16.D0*Hr1(0)*z3 +&
            144.D0*Hr1(0)*z3*x + 64.D0/3.D0*Hr1(0)*z2 - 488.D0/3.D0*Hr1(0)&
            *z2*x - 32.D0*Hr1(0)*z2*x**2 + 64.D0/3.D0*Hr1(0)*z2*dx -&
            2264.D0/9.D0*Hr1(1) + 3428.D0/9.D0*Hr1(1)*x - 1408.D0/27.D0*&
            Hr1(1)*x**2 - 2084.D0/27.D0*Hr1(1)*dx + 144.D0*Hr1(1)*z2 -&
            144.D0*Hr1(1)*z2*x - 128.D0/3.D0*Hr1(1)*z2*x**2 + 128.D0/3.D0&
            *Hr1(1)*z2*dx - 1232.D0/3.D0*Hr2(-1,0) - 1520.D0/3.D0*Hr2(-1,0)*x )

       ggg2 = ggg2 + twonfTF*CF*CA*(-160.D0/9.D0*Hr2(-1,0)*x**2 + 704.D0/&
            9.D0*Hr2(-1,0)*dx + 128.D0*Hr2(0,-1)*z2 - 128.D0*Hr2(0,-1)*z2&
            *x + 4028.D0/9.D0*Hr2(0,0) + 4832.D0/9.D0*Hr2(0,0)*x + 1016.D0&
            /9.D0*Hr2(0,0)*x**2 + 96.D0*Hr2(0,0)*z2 + 96.D0*Hr2(0,0)*z2*x&
            -2696.D0/9.D0*Hr2(0,1) - 596.D0/9.D0*Hr2(0,1)*x + 160.D0/9.D0&
            *Hr2(0,1)*x**2 + 32.D0/3.D0*Hr2(0,1)*dx + 128.D0*Hr2(0,1)*z2&
            +128.D0*Hr2(0,1)*z2*x - 256.D0/3.D0*Hr2(1,0) + 112.D0/3.D0*&
            Hr2(1,0)*x + 296.D0/9.D0*Hr2(1,0)*x**2 + 136.D0/9.D0*Hr2(1,0)&
            *dx + 12.D0*Hr2(1,1) - 12.D0*Hr2(1,1)*x - 16.D0/9.D0*Hr2(1,1)&
            *x**2 + 16.D0/9.D0*Hr2(1,1)*dx + 288.D0*Hr3(-1,-1,0) + 288.D0&
            *Hr3(-1,-1,0)*x - 256.D0/3.D0*Hr3(-1,-1,0)*x**2 - 256.D0/3.D0&
            *Hr3(-1,-1,0)*dx - 144.D0*Hr3(-1,0,0) - 144.D0*Hr3(-1,0,0)*x&
            -32.D0/3.D0*Hr3(-1,0,0)*x**2 - 32.D0/3.D0*Hr3(-1,0,0)*dx -&
            64.D0*Hr3(-1,0,1) - 64.D0*Hr3(-1,0,1)*x - 64.D0/3.D0*Hr3(-1,0,1)&
            *x**2 - 64.D0/3.D0*Hr3(-1,0,1)*dx - 208.D0*Hr3(0,-1,0) -&
            112.D0*Hr3(0,-1,0)*x )
       
       ggg2 = ggg2 + twonfTF*CF*CA*(64.D0*Hr3(0,-1,0)*x**2 + 64.D0/3.D0*&
            Hr3(0,-1,0)*dx + 232.D0/3.D0*Hr3(0,0,0) - 560.D0/3.D0*Hr3(0,0,0)&
            *x - 64.D0/3.D0*Hr3(0,0,1) + 152.D0/3.D0*Hr3(0,0,1)*x + 32.D0&
            *Hr3(0,0,1)*x**2 - 16.D0/3.D0*Hr3(0,1,0) + 8.D0/3.D0*Hr3(0,1,0)&
            *x + 32.D0/3.D0*Hr3(0,1,0)*x**2 - 64.D0/3.D0*Hr3(0,1,0)*&
            dx + 88.D0/3.D0*Hr3(0,1,1) + 136.D0/3.D0*Hr3(0,1,1)*x + 32.D0/&
            3.D0*Hr3(0,1,1)*x**2 - 32.D0/3.D0*Hr3(0,1,1)*dx - 104.D0*Hr3(1,0,0)&
            + 104.D0*Hr3(1,0,0)*x + 128.D0/3.D0*Hr3(1,0,0)*x**2 -&
            128.D0/3.D0*Hr3(1,0,0)*dx - 16.D0*Hr3(1,1,0) + 16.D0*Hr3(1,1,0)&
            *x + 64.D0/3.D0*Hr3(1,1,0)*x**2 - 64.D0/3.D0*Hr3(1,1,0)*dx&
            -16.D0*Hr3(1,1,1) + 16.D0*Hr3(1,1,1)*x + 64.D0/3.D0*Hr3(1,1,1)&
            *x**2 - 64.D0/3.D0*Hr3(1,1,1)*dx + 256.D0*Hr4(0,-1,-1,0)&
            -256.D0*Hr4(0,-1,-1,0)*x - 64.D0*Hr4(0,-1,0,0) + 64.D0*Hr4(0,-1,0,0)&
            *x - 128.D0*Hr4(0,0,-1,0) + 128.D0*Hr4(0,0,0,0) -&
            192.D0*Hr4(0,0,0,0)*x - 96.D0*Hr4(0,0,0,1) - 96.D0*Hr4(0,0,0,1)*x )

       ggg2 = ggg2 + twonfTF*CF*CA*(-64.D0*Hr4(0,0,1,0) - 64.D0*Hr4(0,0,1,0)&
            *x - 16.D0*Hr4(0,0,1,1) - 16.D0*Hr4(0,0,1,1)*x - 112.D0*&
            Hr4(0,1,0,0) - 112.D0*Hr4(0,1,0,0)*x - 32.D0*Hr4(0,1,1,0) -&
            32.D0*Hr4(0,1,1,0)*x - 32.D0*Hr4(0,1,1,1) - 32.D0*Hr4(0,1,1,1)*x )
       
       ggg2 = ggg2 + twonfTF*CF**2*(-850.D0/3.D0 + 66.D0*x + 232.D0*x**2&
            -44.D0/3.D0*dx - 32.D0*z3*x - 256.D0/3.D0*z3*x**2 + 64.D0/3.D0&
            *z3*dx - 64.D0*z2 + 1012.D0/3.D0*z2*x - 64.D0/3.D0*z2*x**2&
            +104.D0*z2**2 + 72.D0*z2**2*x - 64.D0*Hr1(-1)*z2 - 64.D0*&
            Hr1(-1)*z2*x + 64.D0/3.D0*Hr1(-1)*z2*x**2 + 64.D0/3.D0*Hr1(-1)&
            *z2*dx - 254.D0*Hr1(0) - 1082.D0/3.D0*Hr1(0)*x + 72.D0*Hr1(0)&
            *x**2 + 112.D0*Hr1(0)*z3 - 16.D0*Hr1(0)*z3*x + 64.D0*Hr1(0)*&
            z2 + 80.D0*Hr1(0)*z2*x + 88.D0/3.D0*Hr1(1) - 428.D0/3.D0*Hr1(1)&
            *x + 72.D0*Hr1(1)*x**2 + 124.D0/3.D0*Hr1(1)*dx - 64.D0*Hr1(1)&
            *z2 + 64.D0*Hr1(1)*z2*x + 64.D0/3.D0*Hr1(1)*z2*x**2 - 64.D0/&
            3.D0*Hr1(1)*z2*dx + 1072.D0/3.D0*Hr2(-1,0) + 1072.D0/3.D0*&
            Hr2(-1,0)*x - 64.D0*Hr2(0,-1)*z2 + 64.D0*Hr2(0,-1)*z2*x - 328.D0&
            /3.D0*Hr2(0,0) - 396.D0*Hr2(0,0)*x + 224.D0/3.D0*Hr2(0,0)*&
            x**2 + 32.D0*Hr2(0,0)*z2 + 32.D0*Hr2(0,0)*z2*x + 64.D0*Hr2(0,1)&
            + 20.D0*Hr2(0,1)*x + 64.D0/3.D0*Hr2(0,1)*x**2 - 64.D0*Hr2(0,1)*z2 )

       ggg2 = ggg2 + twonfTF*CF**2*(-64.D0*Hr2(0,1)*z2*x - 184.D0/3.D0*&
            Hr2(1,0) + 40.D0/3.D0*Hr2(1,0)*x + 224.D0/3.D0*Hr2(1,0)*x**2&
            -80.D0/3.D0*Hr2(1,0)*dx - 28.D0/3.D0*Hr2(1,1) + 28.D0/3.D0*&
            Hr2(1,1)*x + 64.D0/3.D0*Hr2(1,1)*x**2 - 64.D0/3.D0*Hr2(1,1)*&
            dx - 128.D0*Hr3(-1,-1,0) - 128.D0*Hr3(-1,-1,0)*x + 128.D0/3.D0&
            *Hr3(-1,-1,0)*x**2 + 128.D0/3.D0*Hr3(-1,-1,0)*dx + 64.D0*Hr3(-1,0,0)&
            + 64.D0*Hr3(-1,0,0)*x - 64.D0/3.D0*Hr3(-1,0,0)*x**2&
            -64.D0/3.D0*Hr3(-1,0,0)*dx + 128.D0*Hr3(0,-1,0) - 128.D0/3.D0&
            *Hr3(0,-1,0)*x**2 - 24.D0*Hr3(0,0,0) - 88.D0*Hr3(0,0,0)*x +&
            32.D0/3.D0*Hr3(0,0,0)*x**2 - 64.D0*Hr3(0,0,1) - 80.D0*Hr3(0,0,1)&
            *x - 80.D0*Hr3(0,1,0) - 128.D0*Hr3(0,1,0)*x - 64.D0/3.D0*&
            Hr3(0,1,0)*x**2 - 48.D0*Hr3(0,1,1) - 64.D0*Hr3(0,1,1)*x - 64.D0&
            /3.D0*Hr3(0,1,1)*x**2 + 56.D0*Hr3(1,0,0) - 56.D0*Hr3(1,0,0)*&
            x - 32.D0/3.D0*Hr3(1,0,0)*x**2 + 32.D0/3.D0*Hr3(1,0,0)*dx +&
            16.D0*Hr3(1,1,0) - 16.D0*Hr3(1,1,0)*x - 64.D0/3.D0*Hr3(1,1,0)&
            *x**2 )
       
       ggg2 = ggg2 + twonfTF*CF**2*(64.D0/3.D0*Hr3(1,1,0)*dx + 16.D0*Hr3(1,1,1)&
            - 16.D0*Hr3(1,1,1)*x - 64.D0/3.D0*Hr3(1,1,1)*x**2 + 64.D0&
            /3.D0*Hr3(1,1,1)*dx - 128.D0*Hr4(0,-1,-1,0) + 128.D0*Hr4(0,-1,-1,0)&
            *x + 64.D0*Hr4(0,-1,0,0) - 64.D0*Hr4(0,-1,0,0)*x +&
            128.D0*Hr4(0,0,-1,0) - 32.D0*Hr4(0,0,0,0) - 32.D0*Hr4(0,0,0,0)&
            *x - 32.D0*Hr4(0,0,0,1) - 32.D0*Hr4(0,0,0,1)*x + 16.D0*Hr4(0,0,1,1)&
            + 16.D0*Hr4(0,0,1,1)*x + 48.D0*Hr4(0,1,0,0) + 48.D0*&
            Hr4(0,1,0,0)*x + 32.D0*Hr4(0,1,1,0) + 32.D0*Hr4(0,1,1,0)*x +&
            32.D0*Hr4(0,1,1,1) + 32.D0*Hr4(0,1,1,1)*x )
       
       ggg2 = ggg2 + twonfTF**2*CA*(-110.D0/27.D0 + 14.D0/3.D0*x - 472.D0&
            /81.D0*x**2 + 472.D0/81.D0*dx + 32.D0/9.D0*&
            z2 + 32.D0/9.D0*z2*x + 152.D0/27.D0*Hr1(0) + 86.D0/27.D0*Hr1(0)&
            *x + 104.D0/27.D0*Hr1(0)*x**2 + 22.D0/9.D0*Hr1(1) - 22.D0/9.D0&
            *Hr1(1)*x + 104.D0/27.D0*Hr1(1)*x**2 - 104.D0/27.D0*Hr1(1)*&
            dx - 16.D0/9.D0*Hr2(0,0) - 16.D0/9.D0*Hr2(0,0)*x - 32.D0/9.D0&
            *Hr2(0,1) - 32.D0/9.D0*Hr2(0,1)*x )
       
       ggg2 = ggg2 + twonfTF**2*CF*(224.D0/9.D0 + 64.D0/9.D0*x - 1360.D0/&
            81.D0*x**2 - 1232.D0/81.D0*dx - 16.D0/3.D0*z3 - 16.D0/3.D0*z3&
            *x + 16.D0/9.D0*z2 + 64.D0/9.D0*z2*x - 32.D0/9.D0*z2*x**2 -&
            352.D0/27.D0*Hr1(0) + 1088.D0/27.D0*Hr1(0)*x + 176.D0/27.D0*&
            Hr1(0)*x**2 + 32.D0/3.D0*Hr1(0)*z2 + 32.D0/3.D0*Hr1(0)*z2*x&
            -104.D0/3.D0*Hr1(1) + 56.D0/3.D0*Hr1(1)*x + 176.D0/27.D0*&
            Hr1(1)*x**2 + 256.D0/27.D0*Hr1(1)*dx + 184.D0/9.D0*Hr2(0,0)&
            +232.D0/9.D0*Hr2(0,0)*x + 64.D0/9.D0*Hr2(0,0)*x**2 - 16.D0/&
            9.D0*Hr2(0,1) - 64.D0/9.D0*Hr2(0,1)*x + 32.D0/9.D0*Hr2(0,1)*&
            x**2 - 16.D0/3.D0*Hr2(1,0) + 16.D0/3.D0*Hr2(1,0)*x + 64.D0/9.D0&
            *Hr2(1,0)*x**2 - 64.D0/9.D0*Hr2(1,0)*dx - 8.D0/3.D0*Hr2(1,1)&
            +8.D0/3.D0*Hr2(1,1)*x + 32.D0/9.D0*Hr2(1,1)*x**2 - 32.D0/9.D0&
            *Hr2(1,1)*dx - 16.D0/3.D0*Hr3(0,0,0) - 16.D0/3.D0*Hr3(0,0,0)&
            *x - 32.D0/3.D0*Hr3(0,0,1) - 32.D0/3.D0*Hr3(0,0,1)*x - 32.D0/&
            3.D0*Hr3(0,1,0) - 32.D0/3.D0*Hr3(0,1,0)*x - 16.D0/3.D0*Hr3(0,1,1) )
       
       ggg2 = ggg2 + twonfTF**2*CF* (-16.D0/3.D0*Hr3(0,1,1)*x )


       ggg2 = ggg2 + CA**3*(- 96.D0*Hr1(1)*z3+96.D0*Hr2(0,-1)*z2+2144.D0/9.D0*Hr2(0,1)&
            - 64.D0*Hr2(0,1)*z2- 96.D0*Hr2(1,0)*z2+2144.D0/9.D0*Hr2(1,0)&
            + 176.D0/3.D0*Hr3(0,-1,0)+ 88.D0/3.D0*Hr3(0,0,1)+ 88.D0/3.D0*Hr3(1,0,0)&
            +64.D0*Hr4(0,-1,-1,0)- 64.D0*Hr4(0,-1,0,0)- 64.D0*Hr4(0,-1,0,1)&
            - 64.D0*Hr4(0,0,-1,0)+128.D0*Hr4(0,0,0,1)+ 160.D0*Hr4(0,0,1,0)&
            + 128.D0*Hr4(0,0,1,1)+ 160.D0*Hr4(0,1,0,0)+128.D0*Hr4(0,1,0,1)&
            +128.D0*Hr4(0,1,1,0)+128.D0*Hr4(1,0,0,0)&
            +490.D0/3.D0+ 176.D0/3.D0*z3&
            - 1072.D0/9.D0*z2- 24.D0/5.D0*z2**2&
            - 64.D0*Hr4(1,0,-1,0)+128.D0*Hr4(1,0,0,1)+128.D0*Hr4(1,0,1,0)&
            +128.D0*Hr4(1,1,0,0))*dm

       ggg2 = ggg2 + twonfTF*CA**2*(- 836.D0/27.D0- 128.D0/3.D0*z3+ 160.D0/9.D0*z2&
            - 320.D0/9.D0*Hr2(0,1)- 320.D0/9.D0*Hr2(1,0)- 32.D0/3.D0*Hr3(0,-1,0)&
            - 16.D0/3.D0*Hr3(0,0,1)- 16.D0/3.D0*Hr3(1,0,0))*dm

       ggg2 = ggg2 + twonfTF*CF*CA*(- 110.D0/3.D0+32.D0*z3)*dm
       
       ggg2 = ggg2 + twonfTF**2*CA*(- 16.D0/27.D0)*dm

       res = ggg2/8d0
    END SELECT
    SELECT CASE(cc_piece)
    CASE(cc_REALVIRT,cc_PLUS0)
       !
       ! ...The soft (`+'-distribution) part of the splitting function 
       A3G =&
            CA**3*(490.D0/3.D0 + 88.D0/3.D0*z3 - 1072.D0/9.D0*z2&
            +176.D0/5.D0*z2**2 )&
            +CA**2*twonfTF*(-836.D0/27.D0 + 160.D0/9.D0*z2 - 112.D0/3.D0*z3)&
            +CA*CF*twonfTF*(-110.D0/3.D0 + 32.D0*z3 )-CA*twonfTF**2*16.D0/27.D0
       
       IF(cc_piece.EQ.cc_PLUS0)THEN
          res=A3G/8d0
       ELSEIF(cc_piece.EQ.cc_REALVIRT)THEN
          GGG2L = DM * A3G
          res=res-GGG2L/8d0
       ENDIF
    END SELECT

    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       ! add the Jacobian
       res=res*x
    ENDIF
    RETURN
  END FUNCTION AP_SFS_P2gg

  ! Space like
  ! P_qq^{v,(2)}=(P_{ns+}^{(2)}+P_{ns-}^{(2)})/2
  FUNCTION AP_SFS_P2qqV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pnsp,pnsm

    IF(PRESENT(y_is_x))THEN
       pnsp=AP_SFS_P2NSplus(y,y_is_x)
       pnsm=AP_SFS_P2NSminus(y,y_is_x)
    ELSE
       pnsp=AP_SFS_P2NSplus(y)
       pnsm=AP_SFS_P2NSminus(y)
    ENDIF
    res=(pnsp+pnsm)/two
    RETURN
  END FUNCTION AP_SFS_P2qqV
    

  ! Space like
  ! P_qqbar^{v,(2)}=(P_{ns+}^{(2)}-P_{ns-}^{(2)})/2
  FUNCTION AP_SFS_P2qqbarV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pnsp,pnsm

    IF(PRESENT(y_is_x))THEN
       pnsp=AP_SFS_P2NSplus(y,y_is_x)
       pnsm=AP_SFS_P2NSminus(y,y_is_x)
    ELSE
       pnsp=AP_SFS_P2NSplus(y)
       pnsm=AP_SFS_P2NSminus(y)
    ENDIF
    res=(pnsp-pnsm)/two
    RETURN
  END FUNCTION AP_SFS_P2qqbarV

  ! Space like
  ! P_qq^{v,(2)}=(P_{ns+}^{(2)}+P_{ns-}^{(2)})/2
  ! P_qqbar^{v,(2)}=(P_{ns+}^{(2)}-P_{ns-}^{(2)})/2
  FUNCTION AP_SFS_P2qqV2(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0)),DIMENSION(2)::res ! res(1) P2qqV, res(2) P2qqbarV
    REAL(KIND(1d0))::x,pnsp,pnsm

    IF(PRESENT(y_is_x))THEN
       pnsp=AP_SFS_P2NSplus(y,y_is_x)
       pnsm=AP_SFS_P2NSminus(y,y_is_x)
    ELSE
       pnsp=AP_SFS_P2NSplus(y)
       pnsm=AP_SFS_P2NSminus(y)
    ENDIF
    res(1)=(pnsp+pnsm)/two
    res(2)=(pnsp-pnsm)/two
    RETURN
  END FUNCTION AP_SFS_P2qqV2


  ! Space like
  ! P_qq^{s,(2)}=(P_{ps}^{(2)}+P_{ns}^{s,(2)})/2
  FUNCTION AP_SFS_P2qqS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pps,pnss

    IF(PRESENT(y_is_x))THEN
       pps=AP_SFS_P2PSplus(y,y_is_x)
       pnss=AP_SFS_P2NSS(y,y_is_x)
    ELSE
       pps=AP_SFS_P2PSplus(y)
       pnss=AP_SFS_P2NSS(y)
    ENDIF
    res=(pps+pnss)/two
    RETURN
  END FUNCTION AP_SFS_P2qqS

  ! Space like
  ! P_qqbar^{s,(2)}=(P_{ps}^{(2)}-P_{ns}^{s,(2)})/2
  FUNCTION AP_SFS_P2qqbarS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pps,pnss

    IF(PRESENT(y_is_x))THEN
       pps=AP_SFS_P2PSplus(y,y_is_x)
       pnss=AP_SFS_P2NSS(y,y_is_x)
    ELSE
       pps=AP_SFS_P2PSplus(y)
       pnss=AP_SFS_P2NSS(y)
    ENDIF
    res=(pps-pnss)/two
    RETURN
  END FUNCTION AP_SFS_P2qqbarS

  ! Space like
  ! P_qq^{s,(2)}=(P_{ps}^{(2)}+P_{ns}^{s,(2)})/2
  ! P_qq^{s,(2)}=(P_{ps}^{(2)}-P_{ns}^{s,(2)})/2
  FUNCTION AP_SFS_P2qqS2(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0)),DIMENSION(2)::res ! res(1) P2qqS, res(2) P2qqbarS 
    REAL(KIND(1d0))::x,pps,pnss

    IF(PRESENT(y_is_x))THEN
       pps=AP_SFS_P2PSplus(y,y_is_x)
       pnss=AP_SFS_P2NSS(y,y_is_x)
    ELSE
       pps=AP_SFS_P2PSplus(y)
       pnss=AP_SFS_P2NSS(y)
    ENDIF
    res(1)=(pps+pnss)/two
    res(2)=(pps-pnss)/two
    RETURN
  END FUNCTION AP_SFS_P2qqS2

  !--------------------------------------------------!
  ! NNLO or three-loop  time like                    !
  !--------------------------------------------------!

  ! Time Like
  ! P^{s,(2)}_+=P^{s,(2)}_{qq}+P^{s,(2)}_{q\bar{q}}
  FUNCTION AP_SFT_P2PSplus(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,dsfts,ppsspace

    IF(PRESENT(y_is_x))THEN
       dsfts=AP_DSFTS_P2PSplus(y,y_is_x)
       ppsspace=AP_SFS_P2PSplus(y,y_is_x)
    ELSE
       dsfts=AP_DSFTS_P2PSplus(y)
       ppsspace=AP_SFS_P2PSplus(y)
    ENDIF

    res=ppsspace+dsfts
    RETURN
  END FUNCTION AP_SFT_P2PSplus

  ! Time Like
  ! P^{(2)}_{ns+}=P^{v,(2)}_{qq}+P^{v,(2)}_{q\bar{q}}
  FUNCTION AP_SFT_P2NSplus(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,dsfts,pnspspace

    IF(PRESENT(y_is_x))THEN
       dsfts=AP_DSFTS_P2NSplus(y,y_is_x)
       pnspspace=AP_SFS_P2NSplus(y,y_is_x)
    ELSE
       dsfts=AP_DSFTS_P2NSplus(y)
       pnspspace=AP_SFS_P2NSplus(y)
    ENDIF

    res=pnspspace+dsfts
    RETURN
  END FUNCTION AP_SFT_P2NSplus

  ! Time Like
  ! P^{(2)}_{ns-}=P^{v,(2)}_{qq}-P^{v,(2)}_{q\bar{q}}
  FUNCTION AP_SFT_P2NSminus(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,dsfts,pnsmspace

    IF(PRESENT(y_is_x))THEN
       dsfts=AP_DSFTS_P2NSminus(y,y_is_x)
       pnsmspace=AP_SFS_P2NSminus(y,y_is_x)
    ELSE
       dsfts=AP_DSFTS_P2NSminus(y)
       pnsmspace=AP_SFS_P2NSminus(y)
    ENDIF

    res=pnsmspace+dsfts
    RETURN
  END FUNCTION AP_SFT_P2NSminus

  ! Time Like
  ! P^{s,(2)}_{ns}=P^{s,(2)}_{qq}-P^{s,(2)}_{q\bar{q}}
  FUNCTION AP_SFT_P2NSS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x

    IF(PRESENT(y_is_x))THEN
       res=AP_SFS_P2NSS(y,y_is_x)
    ELSE
       res=AP_SFS_P2NSS(y)
    ENDIF
    RETURN
  END FUNCTION AP_SFT_P2NSS

  ! Time like
  ! P_qq^{v,(2)}=(P_{ns+}^{(2)}+P_{ns-}^{(2)})/2
  FUNCTION AP_SFT_P2qqV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pnsp,pnsm

    IF(PRESENT(y_is_x))THEN
       pnsp=AP_SFT_P2NSplus(y,y_is_x)
       pnsm=AP_SFT_P2NSminus(y,y_is_x)
    ELSE
       pnsp=AP_SFT_P2NSplus(y)
       pnsm=AP_SFT_P2NSminus(y)
    ENDIF
    res=(pnsp+pnsm)/two
    RETURN
  END FUNCTION AP_SFT_P2qqV


  ! Time like
  ! P_qqbar^{v,(2)}=(P_{ns+}^{(2)}-P_{ns-}^{(2)})/2
  FUNCTION AP_SFT_P2qqbarV(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pnsp,pnsm

    IF(PRESENT(y_is_x))THEN
       pnsp=AP_SFT_P2NSplus(y,y_is_x)
       pnsm=AP_SFT_P2NSminus(y,y_is_x)
    ELSE
       pnsp=AP_SFT_P2NSplus(y)
       pnsm=AP_SFT_P2NSminus(y)
    ENDIF
    res=(pnsp-pnsm)/two
    RETURN
  END FUNCTION AP_SFT_P2qqbarV

  ! Time like
  ! P_qq^{v,(2)}=(P_{ns+}^{(2)}+P_{ns-}^{(2)})/2
  ! P_qqbar^{v,(2)}=(P_{ns+}^{(2)}-P_{ns-}^{(2)})/2
  FUNCTION AP_SFT_P2qqV2(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0)),DIMENSION(2)::res ! res(1) P2qqV, res(2) P2qqbarV
    REAL(KIND(1d0))::x,pnsp,pnsm

    IF(PRESENT(y_is_x))THEN
       pnsp=AP_SFT_P2NSplus(y,y_is_x)
       pnsm=AP_SFT_P2NSminus(y,y_is_x)
    ELSE
       pnsp=AP_SFT_P2NSplus(y)
       pnsm=AP_SFT_P2NSminus(y)
    ENDIF
    res(1)=(pnsp+pnsm)/two
    res(2)=(pnsp-pnsm)/two
    RETURN
  END FUNCTION AP_SFT_P2qqV2

  ! Time like
  ! P_qqbar^{s,(2)}=(P_{ps}^{(2)}+P_{ns}^{s,(2)})/2
  FUNCTION AP_SFT_P2qqS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pps,pnss

    IF(PRESENT(y_is_x))THEN
       pps=AP_SFT_P2PSplus(y,y_is_x)
       pnss=AP_SFT_P2NSS(y,y_is_x)
    ELSE
       pps=AP_SFT_P2PSplus(y)
       pnss=AP_SFT_P2NSS(y)
    ENDIF
    res=(pps+pnss)/two
    RETURN
  END FUNCTION AP_SFT_P2qqS

  ! Time like
  ! P_qqbar^{s,(2)}=(P_{ps}^{(2)}-P_{ns}^{s,(2)})/2
  FUNCTION AP_SFT_P2qqbarS(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,pps,pnss

    IF(PRESENT(y_is_x))THEN
       pps=AP_SFT_P2PSplus(y,y_is_x)
       pnss=AP_SFT_P2NSS(y,y_is_x)
    ELSE
       pps=AP_SFT_P2PSplus(y)
       pnss=AP_SFT_P2NSS(y)
    ENDIF
    res=(pps-pnss)/two
    RETURN
  END FUNCTION AP_SFT_P2qqbarS

  ! Time like
  ! P_qq^{s,(2)}=(P_{ps}^{(2)}+P_{ns}^{s,(2)})/2
  ! P_qq^{s,(2)}=(P_{ps}^{(2)}-P_{ns}^{s,(2)})/2
  FUNCTION AP_SFT_P2qqS2(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0)),DIMENSION(2)::res ! res(1) P2qqS, res(2) P2qqbarS
    REAL(KIND(1d0))::x,pps,pnss

    IF(PRESENT(y_is_x))THEN
       pps=AP_SFT_P2PSplus(y,y_is_x)
       pnss=AP_SFT_P2NSS(y,y_is_x)
    ELSE
       pps=AP_SFT_P2PSplus(y)
       pnss=AP_SFT_P2NSS(y)
    ENDIF
    res(1)=(pps+pnss)/two
    res(2)=(pps-pnss)/two
    RETURN
  END FUNCTION AP_SFT_P2qqS2

  ! Time Like - Space Like
  ! P^{s,(2)}_+=P^{s,(2)}_{qq}+P^{s,(2)}_{q\bar{q}}
  ! Eq.(11) in arXiv:0709.3899
  ! Expressions are copyed from subroutine X2PSTS in xptsdff2.f
  ! by S. Moch and A. Vogt, arXiv:0708.3899 [hep-ph]
  FUNCTION AP_DSFTS_P2PSplus(y,y_is_x) RESULT(res)
    USE hplog_module
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    INTEGER,PARAMETER::N1=-1,N2=1,NW=4
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
    COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
    REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
    REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
    REAL(KIND(1d0))::x,DX
    REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
    REAL(KIND(1d0))::pps2d,twoTf,twonfTf
    IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN
    
    DX=one/x
    twoTf=two*TF
    twonfTf=two*nfTF
    CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
         HI1,HI2,HI3,HI4, N1, N2)
    
    pps2d = &
         twoTf*CF*CA* ( 112.D0 + 452.D0*x - 2768.D0/3.D0*x**2 + 1076.D0/&
         3.D0*dx - 640.D0*z4 - 640.D0*z4*x + 104.D0*z3 - 520.D0*z3*x &
         + 224.D0*z3*x**2 - 96.D0*z3*dx + 1376.D0/3.D0*z2 + 1336.D0/3D0&
         *z2*x + 64.D0/3.D0*z2*x**2 + 440.D0*Hr1(0) + 4084.D0/3.D0*&
         Hr1(0)*x + 9188.D0/9.D0*Hr1(0)*x**2 + 712.D0/9.D0*Hr1(0)*dx&
         -368.D0*Hr1(0)*z3 - 176.D0*Hr1(0)*z3*x + 144.D0*Hr1(0)*z2&
         -144.D0*Hr1(0)*z2*x + 64.D0/3.D0*Hr1(0)*z2*x**2 + 64.D0/3.D0&
         *Hr1(0)*z2*dx - 1448.D0/3.D0*Hr1(1) + 1168.D0/3.D0*Hr1(1)*x&
         +740.D0/3.D0*Hr1(1)*x**2 - 460.D0/3.D0*Hr1(1)*dx - 80.D0*&
         Hr1(1)*z2 + 80.D0*Hr1(1)*z2*x + 320.D0/3.D0*Hr1(1)*z2*x**2-&
         320.D0/3.D0*Hr1(1)*z2*dx + 144.D0*Hr2(-1,0) - 48.D0*Hr2(-1,0)&
         *x - 896.D0/3.D0*Hr2(-1,0)*x**2 - 320.D0/3.D0*Hr2(-1,0)*dx -&
         1172.D0/3.D0*Hr2(0,0) - 2960.D0/3.D0*Hr2(0,0)*x - 4304.D0/9.D0&
         *Hr2(0,0)*x**2 - 64.D0/9.D0*Hr2(0,0)*dx - 16.D0*Hr2(0,0)*z2&
         + 112.D0*Hr2(0,0)*z2*x - 384.D0*Hr2(0,1) - 472.D0*Hr2(0,1)*x&
         - 176.D0*Hr2(0,1)*x**2 )

    pps2d = pps2d + twoTf*CF*CA* (-48.D0*Hr2(0,1)*dx - 160.D0*Hr2(0,1)&
         *z2 - 160.D0*Hr2(0,1)*z2*x - 100.D0/3.D0*Hr2(1,0) + 100.D0/&
         3.D0*Hr2(1,0)*x - 64.D0*Hr3(-1,0,0) - 64.D0*Hr3(-1,0,0)*x +&
         256.D0/3.D0*Hr3(-1,0,0)*x**2 + 256.D0/3.D0*Hr3(-1,0,0)*dx +&
         144.D0*Hr3(0,-1,0) - 336.D0*Hr3(0,-1,0)*x + 448.D0/3.D0*Hr3(0,-1,0)&
         *x**2 + 64.D0/3.D0*Hr3(0,-1,0)*dx + 40.D0*Hr3(0,0,0) +&
         680.D0*Hr3(0,0,0)*x - 128.D0/3.D0*Hr3(0,0,0)*dx + 32.D0*Hr3(0,0,1)&
         *x + 64.D0*Hr3(0,0,1)*x**2 + 64.D0/3.D0*Hr3(0,0,1)*dx -&
         80.D0*Hr3(0,1,0) - 16.D0*Hr3(0,1,0)*x + 64.D0*Hr3(0,1,0)*x**2&
         - 64.D0/3.D0*Hr3(0,1,0)*dx - 16.D0*Hr3(0,1,1) + 16.D0*Hr3(0,1,1)&
         *x + 64.D0/3.D0*Hr3(0,1,1)*x**2 - 64.D0/3.D0*Hr3(0,1,1)*&
         dx + 96.D0*Hr3(1,0,0) - 96.D0*Hr3(1,0,0)*x - 128.D0*Hr3(1,0,0)&
         *x**2 + 128.D0*Hr3(1,0,0)*dx + 48.D0*Hr3(1,0,1) - 48.D0*Hr3(1,0,1)&
         *x - 64.D0*Hr3(1,0,1)*x**2 + 64.D0*Hr3(1,0,1)*dx + 16.D0&
         *Hr3(1,1,0) - 16.D0*Hr3(1,1,0)*x - 64.D0/3.D0*Hr3(1,1,0)*x**2&
         + 64.D0/3.D0*Hr3(1,1,0)*dx )

    pps2d = pps2d + twoTf*CF*CA*(-128.D0*Hr4(0,-1,0,0) + 128.D0*&
         Hr4(0,-1,0,0)*x - 128.D0*Hr4(0,0,-1,0) + 128.D0*Hr4(0,0,-1,0)&
         *x - 256.D0*Hr4(0,0,0,0)*x + 16.D0*Hr4(0,0,0,1) + 16.D0*Hr4(0,0,0,1)&
         *x - 144.D0*Hr4(0,0,1,0) - 144.D0*Hr4(0,0,1,0)*x - 64.D0&
         *Hr4(0,0,1,1) - 64.D0*Hr4(0,0,1,1)*x + 192.D0*Hr4(0,1,0,0)&
         +192.D0*Hr4(0,1,0,0)*x + 96.D0*Hr4(0,1,0,1) + 96.D0*Hr4(0,1,0,1)&
         *x + 32.D0*Hr4(0,1,1,0) + 32.D0*Hr4(0,1,1,0)*x )

    pps2d = pps2d + twoTf*CF**2*(-868.D0/9.D0 + 976.D0/9.D0*x + 404.D0&
         /3.D0*x**2 - 440.D0/3.D0*dx + 640.D0*z4 + 640.D0*z4*x + 288.D0&
         *z3 + 192.D0*z3*x + 128.D0*z3*dx - 1016.D0/3.D0*z2 - 1504.D0&
         /3.D0*z2*x - 1024.D0/3.D0*z2*x**2 - 2108.D0/9.D0*Hr1(0) -&
         9892.D0/9.D0*Hr1(0)*x - 3244.D0/9.D0*Hr1(0)*x**2 + 272.D0*&
         Hr1(0)*z3 + 272.D0*Hr1(0)*z3*x - 32.D0*Hr1(0)*z2 + 128.D0*&
         Hr1(0)*z2*x + 128.D0*Hr1(0)*z2*x**2 + 1520.D0/3.D0*Hr1(1) -&
         1144.D0/3.D0*Hr1(1)*x - 2668.D0/9.D0*Hr1(1)*x**2 + 1540.D0/9.D0&
         *Hr1(1)*dx + 80.D0*Hr1(1)*z2 - 80.D0*Hr1(1)*z2*x - 320.D0/3.D0&
         *Hr1(1)*z2*x**2 + 320.D0/3.D0*Hr1(1)*z2*dx + 496.D0*Hr2(0,0)&
         + 324.D0*Hr2(0,0)*x + 1664.D0/9.D0*Hr2(0,0)*x**2 - 112.D0*&
         Hr2(0,0)*z2 - 112.D0*Hr2(0,0)*z2*x + 568.D0*Hr2(0,1) + 368.D0&
         *Hr2(0,1)*x + 1984.D0/9.D0*Hr2(0,1)*x**2 + 224.D0/9.D0*Hr2(0,1)&
         *dx + 160.D0*Hr2(0,1)*z2 + 160.D0*Hr2(0,1)*z2*x + 556.D0/3.D0&
         *Hr2(1,0) - 556.D0/3.D0*Hr2(1,0)*x - 320.D0/9.D0*Hr2(1,0)*&
         x**2 )

    pps2d = pps2d + twoTf*CF**2*(320.D0/9.D0*Hr2(1,0)*dx + 48.D0*Hr3(0,0,0)&
         + 144.D0*Hr3(0,0,0)*x - 64.D0*Hr3(0,0,0)*x**2 + 64.D0*&
         Hr3(0,0,1) + 32.D0*Hr3(0,0,1)*x - 256.D0/3.D0*Hr3(0,0,1)*x**2&
         -128.D0/3.D0*Hr3(0,0,1)*dx + 176.D0*Hr3(0,1,0) + 112.D0*&
         Hr3(0,1,0)*x - 128.D0/3.D0*Hr3(0,1,0)*x**2 + 128.D0/3.D0*Hr3(0,1,0)&
         *dx + 16.D0*Hr3(0,1,1) - 16.D0*Hr3(0,1,1)*x - 64.D0/3.D0&
         *Hr3(0,1,1)*x**2 + 64.D0/3.D0*Hr3(0,1,1)*dx - 96.D0*Hr3(1,0,0)&
         + 96.D0*Hr3(1,0,0)*x + 128.D0*Hr3(1,0,0)*x**2 - 128.D0*Hr3(1,0,0)&
         *dx - 48.D0*Hr3(1,0,1) + 48.D0*Hr3(1,0,1)*x + 64.D0*&
         Hr3(1,0,1)*x**2 - 64.D0*Hr3(1,0,1)*dx - 16.D0*Hr3(1,1,0) + 16.D0&
         *Hr3(1,1,0)*x + 64.D0/3.D0*Hr3(1,1,0)*x**2 - 64.D0/3.D0*&
         Hr3(1,1,0)*dx - 16.D0*Hr4(0,0,0,1) - 16.D0*Hr4(0,0,0,1)*x +&
         144.D0*Hr4(0,0,1,0) + 144.D0*Hr4(0,0,1,0)*x + 64.D0*Hr4(0,0,1,1)&
         + 64.D0*Hr4(0,0,1,1)*x - 192.D0*Hr4(0,1,0,0) - 192.D0*&
         Hr4(0,1,0,0)*x - 96.D0*Hr4(0,1,0,1) - 96.D0*Hr4(0,1,0,1)*x -&
         32.D0*Hr4(0,1,1,0) )

     pps2d = pps2d + twoTf*CF**2 * (  - 32.D0*Hr4(0,1,1,0)*x )

     pps2d = pps2d + twoTf*twonfTf*CF*(-320.D0/9.D0 + 368.D0/9.D0*x - 16.D0&
          /9.D0*x**2 - 32.D0/9.D0*dx + 16.D0*z3 + 16.D0*z3*x + 24.D0*&
          z2 + 56.D0*z2*x + 64.D0/3.D0*z2*x**2 + 16.D0/3.D0*Hr1(0) + 32.D0&
          *Hr1(0)*x + 160.D0/9.D0*Hr1(0)*x**2 - 16.D0*Hr1(0)*z2 - 16.D0&
          *Hr1(0)*z2*x - 24.D0*Hr1(1) - 8.D0*Hr1(1)*x + 448.D0/9.D0*&
          Hr1(1)*x**2 - 160.D0/9.D0*Hr1(1)*dx - 56.D0/3.D0*Hr2(0,0) - 8.D0&
          /3.D0*Hr2(0,0)*x + 32.D0/3.D0*Hr2(0,0)*x**2 - 24.D0*Hr2(0,1)&
          - 56.D0*Hr2(0,1)*x - 64.D0/3.D0*Hr2(0,1)*x**2 + 8.D0*Hr2(1,0)&
          - 8.D0*Hr2(1,0)*x - 32.D0/3.D0*Hr2(1,0)*x**2 + 32.D0/3.D0*&
          Hr2(1,0)*dx - 16.D0*Hr3(0,0,0) - 16.D0*Hr3(0,0,0)*x + 16.D0*&
          Hr3(0,0,1) + 16.D0*Hr3(0,0,1)*x + 16.D0*Hr3(0,1,0) + 16.D0*&
          Hr3(0,1,0)*x )

     res=pps2d/8d0
     
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_DSFTS_P2PSplus

   ! Time Like - Space Like
   ! P^{(2)}_{gg}
   ! Eq.(12) in arXiv:0709.3899
   ! Expressions are copyed from subroutine X2GGTS in xptsdff2.f
   ! by S. Moch and A. Vogt, arXiv:0708.3899 [hep-ph]
   FUNCTION AP_DSFTS_P2gg(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,DM,DP
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
     REAL(KIND(1d0))::pgg2d,twoTf,twonfTf
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
       x=DEXP(-y)
    ELSE
       x=y
    ENDIF
    res=zero

    IF(cc_piece.EQ.cc_Delta)RETURN
    IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

    DX=one/x
    DM=one/(one-x)
    DP=one/(one+x)
    twoTf=two*TF
    twonfTf=two*nfTF

    CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
         HI1,HI2,HI3,HI4, N1, N2)

    pgg2d = &
         CA**3 * (  - 26972.D0/27.D0*Hr1(0) + 18772.D0/27.D0*Hr1(0)*x&
         - 26288.D0/27.D0*Hr1(0)*x**2 + 26288.D0/27.D0*Hr1(0)*dx +&
         8200.D0/27.D0*Hr1(0)*dm - 640.D0*Hr1(0)*z3*x - 192.D0*Hr1(0)*&
         z3*x**2 - 256.D0*Hr1(0)*z3*dx + 224.D0*Hr1(0)*z3*dp - 32.D0*&
         Hr1(0)*z3*dm - 224.D0*Hr1(0)*z2 - 224.D0/3.D0*Hr1(0)*z2*x -&
         1760.D0/3.D0*Hr1(0)*z2*x**2 - 704.D0/3.D0*Hr1(0)*z2*dx - 176.D0&
         /3.D0*Hr1(0)*z2*dp - 176.D0/3.D0*Hr1(0)*z2*dm + 512.D0*Hr2(-1,0)&
         *z2 + 256.D0*Hr2(-1,0)*z2*x + 256.D0*Hr2(-1,0)*z2*x**2&
         + 256.D0*Hr2(-1,0)*z2*dx - 256.D0*Hr2(-1,0)*z2*dp + 512.D0*&
         Hr2(0,-1)*z2 + 256.D0*Hr2(0,-1)*z2*x + 256.D0*Hr2(0,-1)*z2*&
         x**2 + 256.D0*Hr2(0,-1)*z2*dx - 256.D0*Hr2(0,-1)*z2*dp +&
         11504.D0/9.D0*Hr2(0,0) + 5888.D0/9.D0*Hr2(0,0)*x + 14384.D0/9.D0&
         *Hr2(0,0)*x**2 + 6224.D0/9.D0*Hr2(0,0)*dx - 784.D0/3.D0*&
         Hr2(0,0)*dm - 256.D0*Hr2(0,0)*z2*x - 256.D0*Hr2(0,0)*z2*x**2&
         - 256.D0*Hr2(0,0)*z2*dx + 256.D0*Hr2(0,0)*z2*dp + 4688.D0/9.D0&
         *Hr2(0,1) )

    pgg2d = pgg2d + CA**3 * (  - 400.D0/9.D0*Hr2(0,1)*x + 2144.D0/9.D0&
         *Hr2(0,1)*x**2 - 2144.D0/9.D0*Hr2(0,1)*dx - 4288.D0/9.D0*Hr2(0,1)&
         *dm - 256.D0*Hr2(0,1)*z2 + 128.D0*Hr2(0,1)*z2*x - 128.D0*&
         Hr2(0,1)*z2*x**2 + 128.D0*Hr2(0,1)*z2*dx + 128.D0*Hr2(0,1)*z2&
         *dm + 4688.D0/9.D0*Hr2(1,0) - 400.D0/9.D0*Hr2(1,0)*x + 2144.D0&
         /9.D0*Hr2(1,0)*x**2 - 2144.D0/9.D0*Hr2(1,0)*dx - 4288.D0/9.D0&
         *Hr2(1,0)*dm - 256.D0*Hr2(1,0)*z2 + 128.D0*Hr2(1,0)*z2*x -&
         128.D0*Hr2(1,0)*z2*x**2 + 128.D0*Hr2(1,0)*z2*dx + 128.D0*Hr2(1,0)&
         *z2*dm - 896.D0/3.D0*Hr3(-1,0,0) - 1600.D0/3.D0*Hr3(-1,0,0)&
         *x - 704.D0*Hr3(-1,0,0)*x**2 - 704.D0*Hr3(-1,0,0)*dx - 704.D0&
         /3.D0*Hr3(-1,0,0)*dp - 448.D0/3.D0*Hr3(0,-1,0) - 800.D0/3.D0&
         *Hr3(0,-1,0)*x - 352.D0*Hr3(0,-1,0)*x**2 - 352.D0*Hr3(0,-1,0)&
         *dx - 352.D0/3.D0*Hr3(0,-1,0)*dp + 544.D0*Hr3(0,0,0) - 576.D0&
         *Hr3(0,0,0)*x + 1056.D0*Hr3(0,0,0)*x**2 + 704.D0*Hr3(0,0,0)*&
         dx + 176.D0*Hr3(0,0,0)*dp - 528.D0*Hr3(0,0,0)*dm + 1856.D0/3.D0&
         *Hr3(0,0,1) )

    pgg2d = pgg2d + CA**3 * (  - 256.D0/3.D0*Hr3(0,0,1)*x + 704.D0*&
         Hr3(0,0,1)*x**2 + 704.D0/3.D0*Hr3(0,0,1)*dx - 704.D0/3.D0*&
         Hr3(0,0,1)*dm + 544.D0*Hr3(0,1,0) - 160.D0*Hr3(0,1,0)*x +&
         1408.D0/3.D0*Hr3(0,1,0)*x**2 - 704.D0/3.D0*Hr3(0,1,0)*dm +&
         1408.D0/3.D0*Hr3(1,0,0) - 704.D0/3.D0*Hr3(1,0,0)*x + 704.D0/3.D0&
         *Hr3(1,0,0)*x**2 - 704.D0/3.D0*Hr3(1,0,0)*dx - 704.D0/3.D0*&
         Hr3(1,0,0)*dm + 1024.D0*Hr4(-1,-1,0,0) + 512.D0*Hr4(-1,-1,0,0)&
         *x + 512.D0*Hr4(-1,-1,0,0)*x**2 + 512.D0*Hr4(-1,-1,0,0)*dx&
         - 512.D0*Hr4(-1,-1,0,0)*dp + 512.D0*Hr4(-1,0,-1,0) + 256.D0*&
         Hr4(-1,0,-1,0)*x + 256.D0*Hr4(-1,0,-1,0)*x**2 + 256.D0*Hr4(-1,0,-1,0)&
         *dx - 256.D0*Hr4(-1,0,-1,0)*dp - 1152.D0*Hr4(-1,0,0,0)&
         - 576.D0*Hr4(-1,0,0,0)*x - 576.D0*Hr4(-1,0,0,0)*x**2 - 576.D0&
         *Hr4(-1,0,0,0)*dx + 576.D0*Hr4(-1,0,0,0)*dp - 512.D0*Hr4(-1,0,0,1)&
         - 256.D0*Hr4(-1,0,0,1)*x - 256.D0*Hr4(-1,0,0,1)*x**2&
         - 256.D0*Hr4(-1,0,0,1)*dx + 256.D0*Hr4(-1,0,0,1)*dp - 256.D0&
         *Hr4(-1,0,1,0) )

    pgg2d = pgg2d + CA**3 * (  - 128.D0*Hr4(-1,0,1,0)*x - 128.D0*Hr4(-1,0,1,0)&
         *x**2 - 128.D0*Hr4(-1,0,1,0)*dx + 128.D0*Hr4(-1,0,1,0)&
         *dp + 512.D0*Hr4(0,-1,-1,0) + 256.D0*Hr4(0,-1,-1,0)*x + 256.D0&
         *Hr4(0,-1,-1,0)*x**2 + 256.D0*Hr4(0,-1,-1,0)*dx - 256.D0*&
         Hr4(0,-1,-1,0)*dp - 384.D0*Hr4(0,-1,0,0) - 960.D0*Hr4(0,-1,0,0)&
         *x - 448.D0*Hr4(0,-1,0,0)*x**2 - 448.D0*Hr4(0,-1,0,0)*dx +&
         448.D0*Hr4(0,-1,0,0)*dp - 256.D0*Hr4(0,-1,0,1) - 128.D0*Hr4(0,-1,0,1)&
         *x - 128.D0*Hr4(0,-1,0,1)*x**2 - 128.D0*Hr4(0,-1,0,1)&
         *dx + 128.D0*Hr4(0,-1,0,1)*dp - 768.D0*Hr4(0,0,-1,0)*x - 256.D0&
         *Hr4(0,0,-1,0)*x**2 - 256.D0*Hr4(0,0,-1,0)*dx + 256.D0*Hr4(0,0,-1,0)&
         *dp + 1536.D0*Hr4(0,0,0,0)*x + 512.D0*Hr4(0,0,0,0)*dx&
         - 256.D0*Hr4(0,0,0,0)*dp + 256.D0*Hr4(0,0,0,0)*dm + 384.D0*&
         Hr4(0,0,0,1)*x + 384.D0*Hr4(0,0,0,1)*dx - 192.D0*Hr4(0,0,0,1)&
         *dp + 192.D0*Hr4(0,0,0,1)*dm + 128.D0*Hr4(0,0,1,0)*x + 128.D0&
         *Hr4(0,0,1,0)*dx - 64.D0*Hr4(0,0,1,0)*dp + 64.D0*Hr4(0,0,1,0)&
         *dm )

    pgg2d = pgg2d + CA**3 * (  - 128.D0*Hr4(0,1,0,0) + 64.D0*Hr4(0,1,0,0)&
         *x - 64.D0*Hr4(0,1,0,0)*x**2 + 64.D0*Hr4(0,1,0,0)*dx + 64.D0&
         *Hr4(0,1,0,0)*dm - 384.D0*Hr4(1,0,0,0) + 192.D0*Hr4(1,0,0,0)&
         *x - 192.D0*Hr4(1,0,0,0)*x**2 + 192.D0*Hr4(1,0,0,0)*dx + 192.D0&
         *Hr4(1,0,0,0)*dm )

    pgg2d = pgg2d + twonfTf*CA**2 * (  - 3248.D0/27.D0*Hr1(0) + 5776.D0/27.D0&
         *Hr1(0)*x - 1592.D0/9.D0*Hr1(0)*x**2 + 1592.D0/9.D0*Hr1(0)*&
         dx - 2528.D0/27.D0*Hr1(0)*dm - 64.D0*Hr1(0)*z2 - 64.D0/3.D0*&
         Hr1(0)*z2*x - 64.D0/3.D0*Hr1(0)*z2*x**2 + 32.D0/3.D0*Hr1(0)*&
         z2*dp + 32.D0/3.D0*Hr1(0)*z2*dm + 1472.D0/9.D0*Hr2(0,0) +&
         3008.D0/9.D0*Hr2(0,0)*x + 64.D0/9.D0*Hr2(0,0)*x**2 + 2176.D0/&
         9.D0*Hr2(0,0)*dx - 64.D0/9.D0*Hr2(0,0)*dm - 1568.D0/9.D0*Hr2(0,1)&
         + 928.D0/9.D0*Hr2(0,1)*x - 352.D0/3.D0*Hr2(0,1)*x**2 +&
         352.D0/3.D0*Hr2(0,1)*dx + 640.D0/9.D0*Hr2(0,1)*dm - 1568.D0/9.D0&
         *Hr2(1,0) + 928.D0/9.D0*Hr2(1,0)*x - 352.D0/3.D0*Hr2(1,0)*&
         x**2 + 352.D0/3.D0*Hr2(1,0)*dx + 640.D0/9.D0*Hr2(1,0)*dm -&
         256.D0/3.D0*Hr3(-1,0,0) - 128.D0/3.D0*Hr3(-1,0,0)*x - 128.D0/&
         3.D0*Hr3(-1,0,0)*x**2 - 128.D0/3.D0*Hr3(-1,0,0)*dx + 128.D0/3.D0&
         *Hr3(-1,0,0)*dp - 128.D0/3.D0*Hr3(0,-1,0) - 64.D0/3.D0*Hr3(0,-1,0)&
         *x - 64.D0/3.D0*Hr3(0,-1,0)*x**2 - 64.D0/3.D0*Hr3(0,-1,0)*dx )

    pgg2d = pgg2d + twonfTf*CA**2 * ( 64.D0/3.D0*Hr3(0,-1,0)*dp - 64.D0*&
         Hr3(0,0,0) + 384.D0*Hr3(0,0,0)*x - 64.D0*Hr3(0,0,0)*x**2 +&
         128.D0*Hr3(0,0,0)*dx - 32.D0*Hr3(0,0,0)*dp + 96.D0*Hr3(0,0,0)&
         *dm - 128.D0/3.D0*Hr3(0,0,1) + 256.D0/3.D0*Hr3(0,0,1)*x - 128.D0&
         /3.D0*Hr3(0,0,1)*x**2 + 128.D0/3.D0*Hr3(0,0,1)*dx + 128.D0/&
         3.D0*Hr3(0,0,1)*dm - 64.D0*Hr3(0,1,0) + 64.D0*Hr3(0,1,0)*x -&
         128.D0/3.D0*Hr3(0,1,0)*x**2 + 128.D0/3.D0*Hr3(0,1,0)*dx + 128.D0&
         /3.D0*Hr3(0,1,0)*dm - 256.D0/3.D0*Hr3(1,0,0) + 128.D0/3.D0*&
         Hr3(1,0,0)*x - 128.D0/3.D0*Hr3(1,0,0)*x**2 + 128.D0/3.D0*Hr3(1,0,0)&
         *dx + 128.D0/3.D0*Hr3(1,0,0)*dm )

    pgg2d = pgg2d + twonfTf*CF*CA*(  - 112.D0 - 452.D0*x + 2768.D0/3.D0*&
         x**2 - 1076.D0/3.D0*dx + 640.D0*z4 + 640.D0*z4*x - 104.D0*z3&
         + 520.D0*z3*x - 224.D0*z3*x**2 + 96.D0*z3*dx - 1376.D0/3.D0*&
         z2 - 1336.D0/3.D0*z2*x - 64.D0/3.D0*z2*x**2 - 328.D0*Hr1(0)&
         - 4372.D0/3.D0*Hr1(0)*x - 612.D0*Hr1(0)*x**2 - 488.D0*Hr1(0)&
         *dx - 16.D0*Hr1(0)*dm + 176.D0*Hr1(0)*z3 + 368.D0*Hr1(0)*z3*x&
         - 368.D0/3.D0*Hr1(0)*z2 + 496.D0/3.D0*Hr1(0)*z2*x + 64.D0/3.D0&
         *Hr1(0)*z2*x**2 + 64.D0/3.D0*Hr1(0)*z2*dx + 1448.D0/3.D0*&
         Hr1(1) - 1168.D0/3.D0*Hr1(1)*x - 740.D0/3.D0*Hr1(1)*x**2 +&
         460.D0/3.D0*Hr1(1)*dx + 80.D0*Hr1(1)*z2 - 80.D0*Hr1(1)*z2*x&
         - 320.D0/3.D0*Hr1(1)*z2*x**2 + 320.D0/3.D0*Hr1(1)*z2*dx -&
         144.D0*Hr2(-1,0) + 48.D0*Hr2(-1,0)*x + 896.D0/3.D0*Hr2(-1,0)*&
         x**2 + 320.D0/3.D0*Hr2(-1,0)*dx - 1316.D0/9.D0*Hr2(0,0) +&
         1168.D0/9.D0*Hr2(0,0)*x + 176.D0/3.D0*Hr2(0,0)*x**2 - 3712.D0/&
         9.D0*Hr2(0,0)*dx - 112.D0*Hr2(0,0)*z2 + 16.D0*Hr2(0,0)*z2*x&
         + 1456.D0/3.D0*Hr2(0,1) )

    pgg2d = pgg2d + twonfTf*CF*CA*( 1112.D0/3.D0*Hr2(0,1)*x + 2288.D0/9.D0&
         *Hr2(0,1)*x**2 - 272.D0/9.D0*Hr2(0,1)*dx + 160.D0*Hr2(0,1)*&
         z2 + 160.D0*Hr2(0,1)*z2*x + 404.D0/3.D0*Hr2(1,0) - 404.D0/3.D0&
         *Hr2(1,0)*x + 704.D0/9.D0*Hr2(1,0)*x**2 - 704.D0/9.D0*Hr2(1,0)&
         *dx - 64.D0*Hr3(-1,0,0) - 64.D0*Hr3(-1,0,0)*x + 256.D0/3.D0*&
         Hr3(-1,0,0)*x**2 + 256.D0/3.D0*Hr3(-1,0,0)*dx - 208.D0*Hr3(0,-1,0)&
         + 272.D0*Hr3(0,-1,0)*x - 64.D0*Hr3(0,-1,0)*x**2 + 64.D0&
         *Hr3(0,-1,0)*dx + 136.D0*Hr3(0,0,0) - 120.D0*Hr3(0,0,0)*x -&
         640.D0/3.D0*Hr3(0,0,0)*dx - 320.D0/3.D0*Hr3(0,0,1) - 416.D0/3.D0&
         *Hr3(0,0,1)*x - 64.D0*Hr3(0,0,1)*x**2 - 64.D0/3.D0*Hr3(0,0,1)&
         *dx + 80.D0/3.D0*Hr3(0,1,0) - 112.D0/3.D0*Hr3(0,1,0)*x - 64.D0&
         *Hr3(0,1,0)*x**2 + 64.D0/3.D0*Hr3(0,1,0)*dx + 16.D0*Hr3(0,1,1)&
         - 16.D0*Hr3(0,1,1)*x - 64.D0/3.D0*Hr3(0,1,1)*x**2 + 64.D0/&
         3.D0*Hr3(0,1,1)*dx - 96.D0*Hr3(1,0,0) + 96.D0*Hr3(1,0,0)*x +&
         128.D0*Hr3(1,0,0)*x**2 - 128.D0*Hr3(1,0,0)*dx - 48.D0*Hr3(1,0,1) )

    pgg2d = pgg2d + twonfTf*CF*CA*( 48.D0*Hr3(1,0,1)*x + 64.D0*Hr3(1,0,1)&
         *x**2 - 64.D0*Hr3(1,0,1)*dx - 16.D0*Hr3(1,1,0) + 16.D0*Hr3(1,1,0)&
         *x + 64.D0/3.D0*Hr3(1,1,0)*x**2 - 64.D0/3.D0*Hr3(1,1,0)*&
         dx - 128.D0*Hr4(0,-1,0,0) + 128.D0*Hr4(0,-1,0,0)*x - 128.D0*&
         Hr4(0,0,-1,0) + 128.D0*Hr4(0,0,-1,0)*x - 256.D0*Hr4(0,0,0,0)*&
         x - 16.D0*Hr4(0,0,0,1) - 16.D0*Hr4(0,0,0,1)*x + 144.D0*Hr4(0,0,1,0)&
         + 144.D0*Hr4(0,0,1,0)*x + 64.D0*Hr4(0,0,1,1) + 64.D0*&
         Hr4(0,0,1,1)*x - 192.D0*Hr4(0,1,0,0) - 192.D0*Hr4(0,1,0,0)*x&
         - 96.D0*Hr4(0,1,0,1) - 96.D0*Hr4(0,1,0,1)*x - 32.D0*Hr4(0,1,1,0)&
         - 32.D0*Hr4(0,1,1,0)*x )

    pgg2d = pgg2d + twonfTf*CF**2 * ( 868.D0/9.D0 - 976.D0/9.D0*x - 404.D0/&
         3.D0*x**2 + 440.D0/3.D0*dx - 640.D0*z4 - 640.D0*z4*x - 288.D0&
         *z3 - 192.D0*z3*x - 128.D0*z3*dx + 1016.D0/3.D0*z2 + 1504.D0/&
         3.D0*z2*x + 1024.D0/3.D0*z2*x**2 + 1132.D0/3.D0*Hr1(0) + 956.D0&
         *Hr1(0)*x + 2956.D0/9.D0*Hr1(0)*x**2 + 32.D0*Hr1(0)*dx - 272.D0&
         *Hr1(0)*z3 - 272.D0*Hr1(0)*z3*x - 64.D0*Hr1(0)*z2 - 160.D0*&
         Hr1(0)*z2*x - 128.D0/3.D0*Hr1(0)*z2*x**2 - 1520.D0/3.D0*Hr1(1)&
         + 1144.D0/3.D0*Hr1(1)*x + 2668.D0/9.D0*Hr1(1)*x**2 - 1540.D0&
         /9.D0*Hr1(1)*dx - 80.D0*Hr1(1)*z2 + 80.D0*Hr1(1)*z2*x + 320.D0&
         /3.D0*Hr1(1)*z2*x**2 - 320.D0/3.D0*Hr1(1)*z2*dx - 144.D0*Hr2(0,0)&
         - 388.D0*Hr2(0,0)*x - 128.D0*Hr2(0,0)*x**2 - 16.D0*Hr2(0,0)&
         *z2 - 16.D0*Hr2(0,0)*z2*x - 360.D0*Hr2(0,1) - 576.D0*Hr2(0,1)&
         *x - 192.D0*Hr2(0,1)*x**2 - 160.D0/3.D0*Hr2(0,1)*dx - 160.D0&
         *Hr2(0,1)*z2 - 160.D0*Hr2(0,1)*z2*x + 68.D0/3.D0*Hr2(1,0) -&
         68.D0/3.D0*Hr2(1,0)*x + 64.D0*Hr2(1,0)*x**2 - 64.D0*Hr2(1,0)*&
         dx )

    pgg2d = pgg2d + twonfTf*CF**2 * (  - 48.D0*Hr3(0,0,0) + 48.D0*Hr3(0,0,0)&
         *x - 64.D0*Hr3(0,0,0)*x**2 + 64.D0*Hr3(0,0,1) + 96.D0*Hr3(0,0,1)&
         *x - 128.D0/3.D0*Hr3(0,0,1)*dx - 112.D0*Hr3(0,1,0) - 48.D0&
         *Hr3(0,1,0)*x - 256.D0/3.D0*Hr3(0,1,0)*dx - 16.D0*Hr3(0,1,1)&
         + 16.D0*Hr3(0,1,1)*x + 64.D0/3.D0*Hr3(0,1,1)*x**2 - 64.D0/3.D0&
         *Hr3(0,1,1)*dx + 96.D0*Hr3(1,0,0) - 96.D0*Hr3(1,0,0)*x - 128.D0&
         *Hr3(1,0,0)*x**2 + 128.D0*Hr3(1,0,0)*dx + 48.D0*Hr3(1,0,1)&
         - 48.D0*Hr3(1,0,1)*x - 64.D0*Hr3(1,0,1)*x**2 + 64.D0*Hr3(1,0,1)&
         *dx + 16.D0*Hr3(1,1,0) - 16.D0*Hr3(1,1,0)*x - 64.D0/3.D0*&
         Hr3(1,1,0)*x**2 + 64.D0/3.D0*Hr3(1,1,0)*dx + 16.D0*Hr4(0,0,0,1)&
         + 16.D0*Hr4(0,0,0,1)*x - 144.D0*Hr4(0,0,1,0) - 144.D0*Hr4(0,0,1,0)&
         *x - 64.D0*Hr4(0,0,1,1) - 64.D0*Hr4(0,0,1,1)*x + 192.D0&
         *Hr4(0,1,0,0) + 192.D0*Hr4(0,1,0,0)*x + 96.D0*Hr4(0,1,0,1)&
         + 96.D0*Hr4(0,1,0,1)*x + 32.D0*Hr4(0,1,1,0) + 32.D0*Hr4(0,1,1,0)*x )

    pgg2d = pgg2d + twonfTf**2*CA*(  - 464.D0/27.D0*Hr1(0) + 304.D0/27.D0&
         *Hr1(0)*x - 368.D0/27.D0*Hr1(0)*x**2 + 368.D0/27.D0*Hr1(0)*dx&
         + 160.D0/27.D0*Hr1(0)*dm - 64.D0/9.D0*Hr2(0,0) + 128.D0/9.D0&
         *Hr2(0,0)*x - 64.D0/9.D0*Hr2(0,0)*x**2 + 64.D0/9.D0*Hr2(0,0)*&
         dx + 64.D0/9.D0*Hr2(0,0)*dm )

    pgg2d = pgg2d + twonfTf**2*CF*( 320.D0/9.D0 - 368.D0/9.D0*x + 16.D0/&
         9.D0*x**2 + 32.D0/9.D0*dx - 16.D0*z3 - 16.D0*z3*x - 24.D0*z2&
         - 56.D0*z2*x - 64.D0/3.D0*z2*x**2 + 16.D0/3.D0*Hr1(0) - 128.D0&
         /3.D0*Hr1(0)*x + 256.D0/27.D0*Hr1(0)*x**2 - 736.D0/27.D0*&
         Hr1(0)*dx - 16.D0/3.D0*Hr1(0)*z2 - 16.D0/3.D0*Hr1(0)*z2*x +&
         24.D0*Hr1(1) + 8.D0*Hr1(1)*x - 448.D0/9.D0*Hr1(1)*x**2 + 160.D0&
         /9.D0*Hr1(1)*dx - 664.D0/9.D0*Hr2(0,0) - 808.D0/9.D0*Hr2(0,0)&
         *x - 224.D0/9.D0*Hr2(0,0)*x**2 - 128.D0/9.D0*Hr2(0,0)*dx +&
         104.D0/3.D0*Hr2(0,1) + 136.D0/3.D0*Hr2(0,1)*x + 64.D0/9.D0*&
         Hr2(0,1)*x**2 + 128.D0/9.D0*Hr2(0,1)*dx + 8.D0/3.D0*Hr2(1,0)&
         - 8.D0/3.D0*Hr2(1,0)*x - 32.D0/9.D0*Hr2(1,0)*x**2 + 32.D0/9.D0&
         *Hr2(1,0)*dx - 16.D0*Hr3(0,0,0) - 16.D0*Hr3(0,0,0)*x + 80.D0/&
         3.D0*Hr3(0,0,1) + 80.D0/3.D0*Hr3(0,0,1)*x + 16.D0/3.D0*Hr3(0,1,0)&
         + 16.D0/3.D0*Hr3(0,1,0)*x )

     res=pgg2d/8d0
     
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_DSFTS_P2gg

   ! Time Like
   ! P^{(2)}_{qg}
   ! Eq.(18) in arXiv:1107.2263
   ! uncer is the factor of Eq.(38) in in arXiv:1107.2263 to 
   ! estimte the remaining uncertainty in Pqg for timelike one
   ! Expressions are copyed from subroutine X2QGTA in xpij2te.f
   ! by A. Almasy, S. Moch and A. Vogt, arXiv:1107.2263 [hep-ph]
   FUNCTION AP_SFT_P2qg(y,y_is_x,uncer) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0)),INTENT(IN),OPTIONAL::uncer
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,b0,IMOD
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
     REAL(KIND(1d0))::PTqg2,twoTf,twonfTf
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero
     
     IF(cc_piece.EQ.cc_Delta)RETURN
     IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

     IF(PRESENT(uncer))THEN
        IMOD=uncer
     ELSE
        IMOD=zero
     ENDIF
     
     b0=11d0/three*CA-four/three*nfTF ! beta0*2
     DX=one/x
     twoTf=two*TF
     twonfTf=two*nfTF
     CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
          HI1,HI2,HI3,HI4, N1, N2)

     PTqg2 = &
          TF*CA**2 * ( 2057.D0/9.D0 - 672.D0*x + 53753.D0/81.D0*x**2 +&
          2092.D0/81.D0*dx - 196.D0*z3 - 2416.D0/3.D0*z3*x + 328.D0/3.D0&
          *z3*x**2 - 310.D0/3.D0*z2 + 308.D0*z2*x - 536.D0/3.D0*z2*x**2&
          + 544.D0/9.D0*z2*dx + 34.D0*z2**2 + 252.D0/5.D0*z2**2*x +&
          704.D0/5.D0*z2**2*x**2 + 136.D0*Hr1(-1)*z3 + 272.D0*Hr1(-1)*&
          z3*x + 272.D0*Hr1(-1)*z3*x**2 + 512.D0/3.D0*Hr1(-1)*z2 + 760.D0&
          /3.D0*Hr1(-1)*z2*x - 24.D0*Hr1(-1)*z2*x**2 + 32.D0/3.D0*Hr1(-1)&
          *z2*dx - 2996.D0/27.D0*Hr1(0) - 4832.D0/27.D0*Hr1(0)*x +&
          5044.D0/9.D0*Hr1(0)*x**2 + 40.D0*Hr1(0)*dx + 16.D0*Hr1(0)*z3&
          + 960.D0*Hr1(0)*z3*x + 96.D0*Hr1(0)*z2 - 484.D0/3.D0*Hr1(0)*&
          z2*x + 824.D0/3.D0*Hr1(0)*z2*x**2 + 64.D0/3.D0*Hr1(0)*z2*dx&
          - 3940.D0/27.D0*Hr1(1) - 8416.D0/27.D0*Hr1(1)*x + 7448.D0/27.D0&
          *Hr1(1)*x**2 + 752.D0/27.D0*Hr1(1)*dx + 248.D0*Hr1(1)*z3 -&
          496.D0*Hr1(1)*z3*x + 496.D0*Hr1(1)*z3*x**2 + 440.D0/3.D0*Hr1(1)&
          *z2 - 664.D0/3.D0*Hr1(1)*z2*x + 464.D0/3.D0*Hr1(1)*z2*x**2&
          - 64.D0/3.D0*Hr1(1)*z2*dx )

     PTqg2 = PTqg2 + TF*CA**2*(  - 208.D0*Hr2(-1,-1)*z2 - 416.D0*&
          Hr2(-1,-1)*z2*x - 416.D0*Hr2(-1,-1)*z2*x**2 - 152.D0/3.D0*&
          Hr2(-1,0) - 256.D0/3.D0*Hr2(-1,0)*x - 248.D0*Hr2(-1,0)*x**2&
          - 32.D0/3.D0*Hr2(-1,0)*dx + 112.D0*Hr2(-1,0)*z2 + 224.D0*&
          Hr2(-1,0)*z2*x + 224.D0*Hr2(-1,0)*z2*x**2 + 72.D0*Hr2(0,-1)*&
          z2 - 176.D0*Hr2(0,-1)*z2*x + 96.D0*Hr2(0,-1)*z2*x**2 - 598.D0/&
          9.D0*Hr2(0,0) - 3508.D0/9.D0*Hr2(0,0)*x - 7208.D0/9.D0*Hr2(0,0)&
          *x**2 - 128.D0/9.D0*Hr2(0,0)*dx - 28.D0*Hr2(0,0)*z2 + 632.D0&
          *Hr2(0,0)*z2*x - 96.D0*Hr2(0,0)*z2*x**2 - 1262.D0/9.D0*Hr2(0,1)&
          + 4828.D0/9.D0*Hr2(0,1)*x - 2900.D0/3.D0*Hr2(0,1)*x**2 +&
          112.D0/9.D0*Hr2(0,1)*dx + 168.D0*Hr2(0,1)*z2 - 432.D0*Hr2(0,1)&
          *z2*x + 384.D0*Hr2(0,1)*z2*x**2 - 562.D0/9.D0*Hr2(1,0) + 92.D0&
          /9.D0*Hr2(1,0)*x + 704.D0/3.D0*Hr2(1,0)*x**2 - 8.D0/9.D0*&
          Hr2(1,0)*dx + 48.D0*Hr2(1,0)*z2 - 96.D0*Hr2(1,0)*z2*x + 96.D0&
          *Hr2(1,0)*z2*x**2 + 134.D0/9.D0*Hr2(1,1) - 700.D0/9.D0*Hr2(1,1)*x )

     PTqg2 = PTqg2 + TF*CA**2*( 652.D0/3.D0*Hr2(1,1)*x**2 - 128.D0/9.D0&
          *Hr2(1,1)*dx - 16.D0*Hr2(1,1)*z2 + 32.D0*Hr2(1,1)*z2*x - 32.D0&
          *Hr2(1,1)*z2*x**2 + 320.D0/3.D0*Hr3(-1,-1,0) + 112.D0/3.D0*&
          Hr3(-1,-1,0)*x - 208.D0*Hr3(-1,-1,0)*x**2 - 64.D0/3.D0*Hr3(-1,-1,0)&
          *dx - 872.D0/3.D0*Hr3(-1,0,0) - 520.D0/3.D0*Hr3(-1,0,0)&
          *x + 304.D0/3.D0*Hr3(-1,0,0)*x**2 + 128.D0/3.D0*Hr3(-1,0,0)*&
          dx - 352.D0/3.D0*Hr3(-1,0,1) - 704.D0/3.D0*Hr3(-1,0,1)*x - 80.D0&
          *Hr3(-1,0,1)*x**2 - 64.D0/3.D0*Hr3(-1,0,1)*dx + 184.D0/3.D0&
          *Hr3(0,-1,0) - 344.D0/3.D0*Hr3(0,-1,0)*x + 832.D0/3.D0*Hr3(0,-1,0)&
          *x**2 + 64.D0/3.D0*Hr3(0,-1,0)*dx + 64.D0/3.D0*Hr3(0,0,0)&
          + 2680.D0/3.D0*Hr3(0,0,0)*x - 192.D0*Hr3(0,0,0)*x**2 - 128.D0&
          /3.D0*Hr3(0,0,0)*dx + 64.D0/3.D0*Hr3(0,0,1) + 292.D0*Hr3(0,0,1)&
          *x + 120.D0*Hr3(0,0,1)*x**2 - 64.D0/3.D0*Hr3(0,0,1)*dx -&
          100.D0*Hr3(0,1,0) - 312.D0*Hr3(0,1,0)*x + 72.D0*Hr3(0,1,0)*&
          x**2 + 64.D0/3.D0*Hr3(0,1,0)*dx + 128.D0*Hr3(0,1,1) - 400.D0*&
          Hr3(0,1,1)*x )

     PTqg2 = PTqg2 + TF*CA**2 * ( 640.D0/3.D0*Hr3(0,1,1)*x**2 + 32.D0/&
          3.D0*Hr3(0,1,1)*dx + 728.D0/3.D0*Hr3(1,0,0) + 188.D0/3.D0*&
          Hr3(1,0,0)*x - 304.D0/3.D0*Hr3(1,0,0)*x**2 + 224.D0/3.D0*Hr3(1,0,0)&
          *dx + 160.D0/3.D0*Hr3(1,0,1) + 304.D0/3.D0*Hr3(1,0,1)*x&
          - 472.D0/3.D0*Hr3(1,0,1)*x**2 + 32.D0*Hr3(1,0,1)*dx + 160.D0/&
          3.D0*Hr3(1,1,0) + 304.D0/3.D0*Hr3(1,1,0)*x - 472.D0/3.D0*Hr3(1,1,0)&
          *x**2 + 32.D0*Hr3(1,1,0)*dx - 196.D0/3.D0*Hr3(1,1,1) +&
          344.D0/3.D0*Hr3(1,1,1)*x - 400.D0/3.D0*Hr3(1,1,1)*x**2 + 32.D0&
          /3.D0*Hr3(1,1,1)*dx - 96.D0*Hr4(-1,-1,-1,0) - 192.D0*Hr4(-1,-1,-1,0)&
          *x - 192.D0*Hr4(-1,-1,-1,0)*x**2 + 64.D0*Hr4(-1,-1,0,0)&
          + 128.D0*Hr4(-1,-1,0,0)*x + 128.D0*Hr4(-1,-1,0,0)*x**2 +&
          160.D0*Hr4(-1,-1,0,1) + 320.D0*Hr4(-1,-1,0,1)*x + 320.D0*Hr4(-1,-1,0,1)&
          *x**2 + 112.D0*Hr4(-1,0,-1,0) + 224.D0*Hr4(-1,0,-1,0)&
          *x + 224.D0*Hr4(-1,0,-1,0)*x**2 + 8.D0*Hr4(-1,0,0,0) + 16.D0&
          *Hr4(-1,0,0,0)*x + 16.D0*Hr4(-1,0,0,0)*x**2 - 64.D0*Hr4(-1,0,1,0) )

     PTqg2 = PTqg2 + TF*CA**2*(  - 128.D0*Hr4(-1,0,1,0)*x - 128.D0*&
          Hr4(-1,0,1,0)*x**2 - 32.D0*Hr4(-1,0,1,1) - 64.D0*Hr4(-1,0,1,1)&
          *x - 64.D0*Hr4(-1,0,1,1)*x**2 + 16.D0*Hr4(0,-1,-1,0) - 352.D0&
          *Hr4(0,-1,-1,0)*x - 64.D0*Hr4(0,-1,-1,0)*x**2 - 104.D0*Hr4(0,-1,0,0)&
          + 432.D0*Hr4(0,-1,0,0)*x - 32.D0*Hr4(0,-1,0,0)*x**2&
          - 64.D0*Hr4(0,-1,0,1) - 128.D0*Hr4(0,-1,0,1)*x**2 - 56.D0*&
          Hr4(0,0,-1,0) + 496.D0*Hr4(0,0,-1,0)*x + 64.D0*Hr4(0,0,0,0)&
          - 1264.D0*Hr4(0,0,0,0)*x - 100.D0*Hr4(0,0,0,1) - 776.D0*Hr4(0,0,0,1)&
          *x + 96.D0*Hr4(0,0,0,1)*x**2 + 64.D0*Hr4(0,0,1,0) -&
          96.D0*Hr4(0,0,1,0)*x + 160.D0*Hr4(0,0,1,0)*x**2 + 176.D0*Hr4(0,0,1,1)&
          - 192.D0*Hr4(0,0,1,1)*x + 320.D0*Hr4(0,0,1,1)*x**2&
          + 92.D0*Hr4(0,1,0,0) + 408.D0*Hr4(0,1,0,0)*x - 32.D0*Hr4(0,1,0,0)&
          *x**2 + 288.D0*Hr4(0,1,0,1)*x - 96.D0*Hr4(0,1,0,1)*x**2&
          + 288.D0*Hr4(0,1,1,0)*x - 96.D0*Hr4(0,1,1,0)*x**2 - 80.D0*&
          Hr4(0,1,1,1) + 256.D0*Hr4(0,1,1,1)*x - 192.D0*Hr4(0,1,1,1)*&
          x**2 )

     PTqg2 = PTqg2 + TF*CA**2 * (  - 16.D0*Hr4(1,0,-1,0) + 32.D0*Hr4(1,0,-1,0)&
          *x - 32.D0*Hr4(1,0,-1,0)*x**2 - 120.D0*Hr4(1,0,0,0)&
          + 240.D0*Hr4(1,0,0,0)*x - 240.D0*Hr4(1,0,0,0)*x**2 - 128.D0*&
          Hr4(1,0,0,1) + 256.D0*Hr4(1,0,0,1)*x - 256.D0*Hr4(1,0,0,1)*&
          x**2 - 32.D0*Hr4(1,0,1,1) + 64.D0*Hr4(1,0,1,1)*x - 64.D0*Hr4(1,0,1,1)&
          *x**2 - 160.D0*Hr4(1,1,0,0) + 320.D0*Hr4(1,1,0,0)*x&
          - 320.D0*Hr4(1,1,0,0)*x**2 - 32.D0*Hr4(1,1,0,1) + 64.D0*Hr4(1,1,0,1)&
          *x - 64.D0*Hr4(1,1,0,1)*x**2 - 32.D0*Hr4(1,1,1,0) +&
          64.D0*Hr4(1,1,1,0)*x - 64.D0*Hr4(1,1,1,0)*x**2 + 32.D0*Hr4(1,1,1,1)&
          - 64.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)*x**2 )

     PTqg2 = PTqg2 + TF*CF*CA* ( 17597.D0/36.D0 + 2659.D0/6.D0*x -&
          3092.D0/3.D0*x**2 - 220.D0/3.D0*dx + 1340.D0/3.D0*z3 + 4904.D0&
          /3.D0*z3*x + 488.D0*z3*x**2 + 64.D0*z3*dx + 28.D0/9.D0*z2 +&
          4540.D0/9.D0*z2*x - 2620.D0/3.D0*z2*x**2 - 44.D0/5.D0*z2**2&
          + 2632.D0/5.D0*z2**2*x - 128.D0*z2**2*x**2 - 40.D0*Hr1(-1)*&
          z3 - 80.D0*Hr1(-1)*z3*x - 80.D0*Hr1(-1)*z3*x**2 - 96.D0*Hr1(-1)&
          *z2 - 160.D0*Hr1(-1)*z2*x + 32.D0*Hr1(-1)*z2*x**2 - 2422.D0&
          /27.D0*Hr1(0) + 30203.D0/27.D0*Hr1(0)*x - 17918.D0/27.D0*Hr1(0)&
          *x**2 - 56.D0*Hr1(0)*z3 + 16.D0*Hr1(0)*z3*x - 160.D0*Hr1(0)&
          *z3*x**2 - 260.D0/3.D0*Hr1(0)*z2 - 176.D0/3.D0*Hr1(0)*z2*x -&
          8.D0*Hr1(0)*z2*x**2 + 2788.D0/27.D0*Hr1(1) + 19660.D0/27.D0*&
          Hr1(1)*x - 19970.D0/27.D0*Hr1(1)*x**2 - 80.D0/27.D0*Hr1(1)*dx&
          - 792.D0*Hr1(1)*z3 + 1584.D0*Hr1(1)*z3*x - 1584.D0*Hr1(1)*z3&
          *x**2 - 188.D0*Hr1(1)*z2 + 272.D0*Hr1(1)*z2*x - 120.D0*Hr1(1)&
          *z2*x**2 + 32.D0*Hr1(1)*z2*dx + 80.D0*Hr2(-1,-1)*z2 + 160.D0*&
          Hr2(-1,-1)*z2*x )

     PTqg2 = PTqg2 + TF*CF*CA*( 160.D0*Hr2(-1,-1)*z2*x**2 + 672.D0*&
          Hr2(-1,0) + 544.D0*Hr2(-1,0)*x - 16.D0*Hr2(-1,0)*x**2 - 64.D0&
          *Hr2(0,-1)*z2 - 32.D0*Hr2(0,-1)*z2*x - 128.D0*Hr2(0,-1)*z2*&
          x**2 - 1007.D0/9.D0*Hr2(0,0) - 6254.D0/9.D0*Hr2(0,0)*x + 2444.D0&
          /9.D0*Hr2(0,0)*x**2 + 32.D0*Hr2(0,0)*z2 - 256.D0*Hr2(0,0)*&
          z2*x + 256.D0*Hr2(0,0)*z2*x**2 + 2204.D0/9.D0*Hr2(0,1) - 3772.D0&
          /9.D0*Hr2(0,1)*x + 3176.D0/3.D0*Hr2(0,1)*x**2 - 200.D0*Hr2(0,1)&
          *z2 + 592.D0*Hr2(0,1)*z2*x - 512.D0*Hr2(0,1)*z2*x**2 -&
          212.D0*Hr2(1,0) + 2036.D0/3.D0*Hr2(1,0)*x - 3284.D0/3.D0*Hr2(1,0)&
          *x**2 + 304.D0/3.D0*Hr2(1,0)*dx + 80.D0*Hr2(1,0)*z2 - 160.D0&
          *Hr2(1,0)*z2*x + 160.D0*Hr2(1,0)*z2*x**2 - 488.D0/9.D0*Hr2(1,1)&
          + 868.D0/9.D0*Hr2(1,1)*x - 1904.D0/9.D0*Hr2(1,1)*x**2 +&
          304.D0/9.D0*Hr2(1,1)*dx + 32.D0*Hr2(1,1)*z2 - 64.D0*Hr2(1,1)*&
          z2*x + 64.D0*Hr2(1,1)*z2*x**2 - 288.D0*Hr3(-1,-1,0) - 448.D0*&
          Hr3(-1,-1,0)*x - 64.D0*Hr3(-1,-1,0)*x**2 + 16.D0*Hr3(-1,0,0)&
          - 80.D0*Hr3(-1,0,0)*x )

     PTqg2 = PTqg2 + TF*CF*CA*(-144.D0*Hr3(-1,0,0)*x**2 - 48.D0*&
          Hr3(-1,0,1) - 64.D0*Hr3(-1,0,1)*x - 64.D0*Hr3(-1,0,1)*x**2 +&
          336.D0*Hr3(0,-1,0) + 16.D0*Hr3(0,-1,0)*x + 160.D0*Hr3(0,-1,0)&
          *x**2 - 148.D0/3.D0*Hr3(0,0,0) + 1832.D0/3.D0*Hr3(0,0,0)*x +&
          640.D0/3.D0*Hr3(0,0,0)*x**2 + 388.D0/3.D0*Hr3(0,0,1) + 256.D0/&
          3.D0*Hr3(0,0,1)*x - 40.D0/3.D0*Hr3(0,0,1)*x**2 + 28.D0/3.D0*&
          Hr3(0,1,0) + 2032.D0/3.D0*Hr3(0,1,0)*x + 280.D0/3.D0*Hr3(0,1,0)&
          *x**2 - 592.D0/3.D0*Hr3(0,1,1) + 1352.D0/3.D0*Hr3(0,1,1)*x&
          - 192.D0*Hr3(0,1,1)*x**2 - 16.D0*Hr3(1,0,0) - 504.D0*Hr3(1,0,0)&
          *x + 368.D0*Hr3(1,0,0)*x**2 - 96.D0*Hr3(1,0,0)*dx + 268.D0/&
          3.D0*Hr3(1,0,1) - 608.D0/3.D0*Hr3(1,0,1)*x + 728.D0/3.D0*Hr3(1,0,1)&
          *x**2 - 32.D0*Hr3(1,0,1)*dx + 452.D0/3.D0*Hr3(1,1,0) -&
          1024.D0/3.D0*Hr3(1,1,0)*x + 1048.D0/3.D0*Hr3(1,1,0)*x**2 - 32.D0&
          *Hr3(1,1,0)*dx + 424.D0/3.D0*Hr3(1,1,1) - 536.D0/3.D0*Hr3(1,1,1)&
          *x + 544.D0/3.D0*Hr3(1,1,1)*x**2 - 32.D0/3.D0*Hr3(1,1,1)&
          *dx )

     PTqg2 = PTqg2 + TF*CF*CA*(96.D0*Hr4(-1,-1,-1,0) + 192.D0*Hr4(-1,-1,-1,0)&
          *x + 192.D0*Hr4(-1,-1,-1,0)*x**2 + 64.D0*Hr4(-1,-1,0,0)&
          + 128.D0*Hr4(-1,-1,0,0)*x + 128.D0*Hr4(-1,-1,0,0)*x**2&
          - 32.D0*Hr4(-1,-1,0,1) - 64.D0*Hr4(-1,-1,0,1)*x - 64.D0*Hr4(-1,-1,0,1)&
          *x**2 - 80.D0*Hr4(-1,0,-1,0) - 160.D0*Hr4(-1,0,-1,0)&
          *x - 160.D0*Hr4(-1,0,-1,0)*x**2 - 24.D0*Hr4(-1,0,0,0) - 48.D0&
          *Hr4(-1,0,0,0)*x - 48.D0*Hr4(-1,0,0,0)*x**2 + 96.D0*Hr4(-1,0,1,0)&
          + 192.D0*Hr4(-1,0,1,0)*x + 192.D0*Hr4(-1,0,1,0)*x**2 +&
          32.D0*Hr4(-1,0,1,1) + 64.D0*Hr4(-1,0,1,1)*x + 64.D0*Hr4(-1,0,1,1)&
          *x**2 - 128.D0*Hr4(0,-1,-1,0) - 64.D0*Hr4(0,-1,-1,0)*x -&
          256.D0*Hr4(0,-1,-1,0)*x**2 - 96.D0*Hr4(0,-1,0,0)*x - 128.D0*&
          Hr4(0,-1,0,0)*x**2 + 144.D0*Hr4(0,0,-1,0) + 32.D0*Hr4(0,0,-1,0)&
          *x + 384.D0*Hr4(0,0,-1,0)*x**2 - 32.D0*Hr4(0,0,0,0) - 64.D0&
          *Hr4(0,0,0,0)*x + 64.D0*Hr4(0,0,0,1) + 96.D0*Hr4(0,0,0,1)*x&
          + 128.D0*Hr4(0,0,0,1)*x**2 - 160.D0*Hr4(0,0,1,0) - 160.D0*&
          Hr4(0,0,1,0)*x )

     PTqg2 = PTqg2 + TF*CF*CA*(-320.D0*Hr4(0,0,1,0)*x**2 - 176.D0&
          *Hr4(0,0,1,1) + 192.D0*Hr4(0,0,1,1)*x - 384.D0*Hr4(0,0,1,1)*&
          x**2 + 136.D0*Hr4(0,1,0,0) - 1040.D0*Hr4(0,1,0,0)*x + 672.D0*&
          Hr4(0,1,0,0)*x**2 + 168.D0*Hr4(0,1,0,1) - 624.D0*Hr4(0,1,0,1)&
          *x + 448.D0*Hr4(0,1,0,1)*x**2 + 184.D0*Hr4(0,1,1,0) - 656.D0*&
          Hr4(0,1,1,0)*x + 512.D0*Hr4(0,1,1,0)*x**2 + 136.D0*Hr4(0,1,1,1)&
          - 368.D0*Hr4(0,1,1,1)*x + 320.D0*Hr4(0,1,1,1)*x**2 + 80.D0&
          *Hr4(1,0,-1,0) - 160.D0*Hr4(1,0,-1,0)*x + 160.D0*Hr4(1,0,-1,0)&
          *x**2 - 56.D0*Hr4(1,0,0,0) + 112.D0*Hr4(1,0,0,0)*x - 112.D0*&
          Hr4(1,0,0,0)*x**2 + 112.D0*Hr4(1,0,0,1) - 224.D0*Hr4(1,0,0,1)&
          *x + 224.D0*Hr4(1,0,0,1)*x**2 - 240.D0*Hr4(1,0,1,0) + 480.D0*&
          Hr4(1,0,1,0)*x - 480.D0*Hr4(1,0,1,0)*x**2 - 32.D0*Hr4(1,0,1,1)&
          + 64.D0*Hr4(1,0,1,1)*x - 64.D0*Hr4(1,0,1,1)*x**2 + 208.D0*&
          Hr4(1,1,0,0) - 416.D0*Hr4(1,1,0,0)*x + 416.D0*Hr4(1,1,0,0)*&
          x**2 - 48.D0*Hr4(1,1,0,1) + 96.D0*Hr4(1,1,0,1)*x - 96.D0*Hr4(1,1,0,1)*x**2 )

     PTqg2 = PTqg2 + TF*CF*CA* (-16.D0*Hr4(1,1,1,0) + 32.D0*Hr4(1,1,1,0)&
          *x - 32.D0*Hr4(1,1,1,0)*x**2 - 64.D0*Hr4(1,1,1,1) + 128.D0&
          *Hr4(1,1,1,1)*x - 128.D0*Hr4(1,1,1,1)*x**2 )

     PTqg2 = PTqg2 + TF*CF**2*(-227.D0/4.D0-1967.D0/2.D0*x +&
          1069.D0*x**2 + 156.D0*z3 - 816.D0*z3*x - 640.D0*z3*x**2 + 106.D0&
          *z2 - 564.D0*z2*x + 564.D0*z2*x**2 + 56.D0*z2**2 - 1904.D0/&
          5D0*z2**2*x + 352.D0/5.D0*z2**2*x**2 + 128.D0*Hr1(-1)*z2 +&
          160.D0*Hr1(-1)*z2*x + 32.D0*Hr1(-1)*z2*x**2 + 92.D0*Hr1(0) -&
          1143.D0*Hr1(0)*x + 110.D0*Hr1(0)*x**2 + 40.D0*Hr1(0)*z3 - 304.D0&
          *Hr1(0)*z3*x + 160.D0*Hr1(0)*z3*x**2 + 12.D0*Hr1(0)*z2 +&
          272.D0*Hr1(0)*z2*x - 176.D0*Hr1(0)*z2*x**2 + 180.D0*Hr1(1) -&
          480.D0*Hr1(1)*x + 318.D0*Hr1(1)*x**2 + 496.D0*Hr1(1)*z3 - 992.D0&
          *Hr1(1)*z3*x + 992.D0*Hr1(1)*z3*x**2 + 108.D0*Hr1(1)*z2 -&
          64.D0*Hr1(1)*z2*x - 80.D0*Hr1(1)*z2*x**2 - 496.D0*Hr2(-1,0)&
          -240.D0*Hr2(-1,0)*x + 224.D0*Hr2(-1,0)*x**2 - 96.D0*Hr2(-1,0)&
          *z2 - 192.D0*Hr2(-1,0)*z2*x - 192.D0*Hr2(-1,0)*z2*x**2 + 64.D0&
          *Hr2(0,-1)*z2 + 64.D0*Hr2(0,-1)*z2*x + 128.D0*Hr2(0,-1)*z2*&
          x**2 + 107.D0*Hr2(0,0) - 42.D0*Hr2(0,0)*x - 76.D0*Hr2(0,0)*&
          x**2 )

     PTqg2 = PTqg2 + TF*CF**2*(-40.D0*Hr2(0,0)*z2 + 144.D0*Hr2(0,0)&
          *z2*x - 160.D0*Hr2(0,0)*z2*x**2 - 2.D0*Hr2(0,1) + 116.D0*&
          Hr2(0,1)*x - 324.D0*Hr2(0,1)*x**2 + 104.D0*Hr2(0,1)*z2 - 144.D0&
          *Hr2(0,1)*z2*x + 224.D0*Hr2(0,1)*z2*x**2 + 294.D0*Hr2(1,0)&
          -560.D0*Hr2(1,0)*x + 468.D0*Hr2(1,0)*x**2 - 112.D0*Hr2(1,0)&
          *z2 + 224.D0*Hr2(1,0)*z2*x - 224.D0*Hr2(1,0)*z2*x**2 + 34.D0*&
          Hr2(1,1) - 32.D0*Hr2(1,1)*x - 4.D0*Hr2(1,1)*x**2 - 16.D0*Hr2(1,1)&
          *z2 + 32.D0*Hr2(1,1)*z2*x - 32.D0*Hr2(1,1)*z2*x**2 + 256.D0&
          *Hr3(-1,-1,0) + 320.D0*Hr3(-1,-1,0)*x + 64.D0*Hr3(-1,-1,0)*&
          x**2 + 64.D0*Hr3(-1,0,0) + 320.D0*Hr3(-1,0,0)*x + 256.D0*Hr3(-1,0,0)&
          *x**2 - 240.D0*Hr3(0,-1,0) + 64.D0*Hr3(0,-1,0)*x - 256.D0&
          *Hr3(0,-1,0)*x**2 + 36.D0*Hr3(0,0,0) - 584.D0*Hr3(0,0,0)*x&
          -192.D0*Hr3(0,0,0)*x**2 - 28.D0*Hr3(0,0,1) - 144.D0*Hr3(0,0,1)&
          *x + 16.D0*Hr3(0,0,1)*x**2 + 184.D0*Hr3(0,1,0) - 128.D0*&
          Hr3(0,1,0)*x - 112.D0*Hr3(0,1,0)*x**2 + 104.D0*Hr3(0,1,1) -&
          80.D0*Hr3(0,1,1)*x )

     PTqg2 = PTqg2 + TF*CF**2*(16.D0*Hr3(0,1,1)*x**2 - 288.D0*Hr3(1,0,0)&
          + 352.D0*Hr3(1,0,0)*x - 160.D0*Hr3(1,0,0)*x**2 - 124.D0&
          *Hr3(1,0,1) + 64.D0*Hr3(1,0,1)*x - 48.D0*Hr3(1,0,1)*x**2 -&
          180.D0*Hr3(1,1,0) + 192.D0*Hr3(1,1,0)*x - 144.D0*Hr3(1,1,0)*&
          x**2 - 76.D0*Hr3(1,1,1) + 64.D0*Hr3(1,1,1)*x - 48.D0*Hr3(1,1,1)&
          *x**2 - 192.D0*Hr4(-1,-1,0,0) - 384.D0*Hr4(-1,-1,0,0)*x -&
          384.D0*Hr4(-1,-1,0,0)*x**2 - 96.D0*Hr4(-1,0,-1,0) - 192.D0*&
          Hr4(-1,0,-1,0)*x - 192.D0*Hr4(-1,0,-1,0)*x**2 + 176.D0*Hr4(-1,0,0,0)&
          + 352.D0*Hr4(-1,0,0,0)*x + 352.D0*Hr4(-1,0,0,0)*x**2&
          + 128.D0*Hr4(0,-1,-1,0) + 128.D0*Hr4(0,-1,-1,0)*x + 256.D0*&
          Hr4(0,-1,-1,0)*x**2 + 32.D0*Hr4(0,-1,0,0) + 128.D0*Hr4(0,-1,0,0)&
          *x + 256.D0*Hr4(0,-1,0,0)*x**2 - 96.D0*Hr4(0,0,-1,0) + 64.D0&
          *Hr4(0,0,-1,0)*x - 256.D0*Hr4(0,0,-1,0)*x**2 + 16.D0*Hr4(0,0,0,0)&
          - 256.D0*Hr4(0,0,0,0)*x + 128.D0*Hr4(0,0,0,0)*x**2 + 8.D0&
          *Hr4(0,0,0,1) - 16.D0*Hr4(0,0,0,1)*x + 32.D0*Hr4(0,0,0,1)*&
          x**2 )

     PTqg2 = PTqg2 + TF*CF**2*(96.D0*Hr4(0,0,1,0)-192.D0*Hr4(0,0,1,0)&
          *x + 352.D0*Hr4(0,0,1,0)*x**2 + 64.D0*Hr4(0,0,1,1) - 128.D0&
          *Hr4(0,0,1,1)*x + 192.D0*Hr4(0,0,1,1)*x**2 - 176.D0*Hr4(0,1,0,0)&
          + 288.D0*Hr4(0,1,0,0)*x - 416.D0*Hr4(0,1,0,0)*x**2 - 104.D0&
          *Hr4(0,1,0,1) + 208.D0*Hr4(0,1,0,1)*x - 224.D0*Hr4(0,1,0,1)&
          *x**2 - 120.D0*Hr4(0,1,1,0) + 240.D0*Hr4(0,1,1,0)*x - 288.D0*&
          Hr4(0,1,1,0)*x**2 - 56.D0*Hr4(0,1,1,1) + 112.D0*Hr4(0,1,1,1)*&
          x - 128.D0*Hr4(0,1,1,1)*x**2 - 96.D0*Hr4(1,0,-1,0) + 192.D0*&
          Hr4(1,0,-1,0)*x - 192.D0*Hr4(1,0,-1,0)*x**2 + 336.D0*Hr4(1,0,0,0)&
          - 672.D0*Hr4(1,0,0,0)*x + 672.D0*Hr4(1,0,0,0)*x**2 + 80.D0&
          *Hr4(1,0,0,1) - 160.D0*Hr4(1,0,0,1)*x + 160.D0*Hr4(1,0,0,1)*&
          x**2 + 304.D0*Hr4(1,0,1,0) - 608.D0*Hr4(1,0,1,0)*x + 608.D0*&
          Hr4(1,0,1,0)*x**2 + 64.D0*Hr4(1,0,1,1) - 128.D0*Hr4(1,0,1,1)*&
          x + 128.D0*Hr4(1,0,1,1)*x**2 + 16.D0*Hr4(1,1,0,0) - 32.D0*&
          Hr4(1,1,0,0)*x + 32.D0*Hr4(1,1,0,0)*x**2 + 80.D0*Hr4(1,1,0,1)&
          -160.D0*Hr4(1,1,0,1)*x )

     PTqg2 = PTqg2 + TF*CF**2*( 160.D0*Hr4(1,1,0,1)*x**2 + 48.D0*&
          Hr4(1,1,1,0) - 96.D0*Hr4(1,1,1,0)*x + 96.D0*Hr4(1,1,1,0)*x**2&
          + 32.D0*Hr4(1,1,1,1) - 64.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)&
          *x**2 )

     PTqg2 = PTqg2 + TF*twonfTF*CA*(-14.D0/9.D0-1216.D0/9.D0*x + 916.D0&
          /9.D0*x**2 - 44.D0/9.D0*dx - 40.D0*z3 + 352.D0/3.D0*z3*x -&
          272.D0/3.D0*z3*x**2 + 100.D0/3.D0*z2 + 208.D0/3.D0*z2*x - 16.D0&
          /9.D0*z2*x**2 + 32.D0/9.D0*z2*dx + 64.D0/3.D0*Hr1(-1)*z2 +&
          128.D0/3.D0*Hr1(-1)*z2*x + 128.D0/3.D0*Hr1(-1)*z2*x**2 + 44.D0&
          /27.D0*Hr1(0) + 752.D0/27.D0*Hr1(0)*x - 2212.D0/27.D0*Hr1(0)*&
          x**2 - 256.D0/27.D0*Hr1(0)*dx - 24.D0*Hr1(0)*z2 + 112.D0/3.D0&
          *Hr1(0)*z2*x - 112.D0/3.D0*Hr1(0)*z2*x**2 + 592.D0/27.D0*Hr1(1)&
          - 1232.D0/27.D0*Hr1(1)*x + 2632.D0/27.D0*Hr1(1)*x**2 - 320.D0&
          /27.D0*Hr1(1)*dx - 32.D0/3.D0*Hr1(1)*z2 + 64.D0/3.D0*Hr1(1)&
          *z2*x - 64.D0/3.D0*Hr1(1)*z2*x**2 - 16.D0*Hr2(-1,0) + 32.D0/3.D0&
          *Hr2(-1,0)*x + 512.D0/9.D0*Hr2(-1,0)*x**2 + 32.D0/9.D0*Hr2(-1,0)&
          *dx - 296.D0/9.D0*Hr2(0,0) - 416.D0/9.D0*Hr2(0,0)*x -&
          536.D0/9.D0*Hr2(0,0)*x**2 - 64.D0/9.D0*Hr2(0,0)*dx + 356.D0/9.D0&
          *Hr2(0,1) - 1360.D0/9.D0*Hr2(0,1)*x + 632.D0/9.D0*Hr2(0,1)*&
          x**2 )

     PTqg2 = PTqg2 + TF*twonfTF*CA*(16.D0/9.D0*Hr2(1,0) + 496.D0/9.D0*&
          Hr2(1,0)*x - 664.D0/9.D0*Hr2(1,0)*x**2 + 32.D0/3.D0*Hr2(1,0)*&
          dx - 320.D0/9.D0*Hr2(1,1) + 544.D0/9.D0*Hr2(1,1)*x - 200.D0/3.D0&
          *Hr2(1,1)*x**2 + 32.D0/9.D0*Hr2(1,1)*dx + 64.D0/3.D0*Hr3(-1,-1,0)&
          + 128.D0/3.D0*Hr3(-1,-1,0)*x + 128.D0/3.D0*Hr3(-1,-1,0)&
          *x**2 + 32.D0/3.D0*Hr3(-1,0,0) + 64.D0/3.D0*Hr3(-1,0,0)*x +&
          64.D0/3.D0*Hr3(-1,0,0)*x**2 - 32.D0/3.D0*Hr3(-1,0,1) - 64.D0/&
          3.D0*Hr3(-1,0,1)*x - 64.D0/3.D0*Hr3(-1,0,1)*x**2 - 64.D0/3.D0&
          *Hr3(0,-1,0) + 128.D0/3.D0*Hr3(0,-1,0)*x - 64.D0/3.D0*Hr3(0,-1,0)&
          *x**2 - 64.D0/3.D0*Hr3(0,0,0) - 640.D0/3.D0*Hr3(0,0,0)*x&
          +104.D0/3.D0*Hr3(0,0,1) - 16.D0*Hr3(0,0,1)*x + 176.D0/3.D0*&
          Hr3(0,0,1)*x**2 + 8.D0*Hr3(0,1,0) + 80.D0*Hr3(0,1,0)*x - 16.D0&
          *Hr3(0,1,0)*x**2 - 32.D0*Hr3(0,1,1) + 96.D0*Hr3(0,1,1)*x -&
          224.D0/3.D0*Hr3(0,1,1)*x**2 - 152.D0/3.D0*Hr3(1,0,0) + 304.D0/&
          3.D0*Hr3(1,0,0)*x - 304.D0/3.D0*Hr3(1,0,0)*x**2 - 16.D0/3.D0*&
          Hr3(1,0,1) )

     PTqg2 = PTqg2 + TF*twonfTF*CA*(32.D0/3.D0*Hr3(1,0,1)*x - 32.D0/3.D0&
          *Hr3(1,0,1)*x**2 - 16.D0/3.D0*Hr3(1,1,0) + 32.D0/3.D0*Hr3(1,1,0)&
          *x - 32.D0/3.D0*Hr3(1,1,0)*x**2 + 40.D0/3.D0*Hr3(1,1,1) -&
          80.D0/3.D0*Hr3(1,1,1)*x + 80.D0/3.D0*Hr3(1,1,1)*x**2 )

     PTqg2 = PTqg2 + TF*twonfTF*CF*(-4847.D0/54.D0-2375.D0/27.D0*x&
          + 4066.D0/27.D0*x**2 + 200.D0/27.D0*dx + 304.D0/3.D0*z3 -&
          608.D0/3.D0*z3*x + 160.D0*z3*x**2 + 272.D0/9.D0*z2 + 8.D0/9.D0&
          *z2*x + 32.D0*z2*x**2 - 64.D0/9.D0*z2*dx + 1684.D0/27.D0*Hr1(0)&
          - 4022.D0/27.D0*Hr1(0)*x - 4480.D0/27.D0*Hr1(0)*x**2 + 512.D0&
          /27.D0*Hr1(0)*dx + 80.D0/3.D0*Hr1(0)*z2 - 352.D0/3.D0*Hr1(0)&
          *z2*x + 48.D0*Hr1(0)*z2*x**2 - 808.D0/27.D0*Hr1(1) + 560.D0/&
          27.D0*Hr1(1)*x - 248.D0/27.D0*Hr1(1)*x**2 - 8.D0*Hr1(1)*z2 +&
          16.D0*Hr1(1)*z2*x - 16.D0*Hr1(1)*z2*x**2 + 48.D0*Hr2(-1,0) -&
          496.D0/9.D0*Hr2(-1,0)*x**2 - 64.D0/9.D0*Hr2(-1,0)*dx + 902.D0/&
          9.D0*Hr2(0,0) + 956.D0/9.D0*Hr2(0,0)*x + 1400.D0/9.D0*Hr2(0,0)&
          *x**2 + 128.D0/9.D0*Hr2(0,0)*dx - 416.D0/9.D0*Hr2(0,1) + 376.D0&
          /9.D0*Hr2(0,1)*x - 88.D0/3.D0*Hr2(0,1)*x**2 + 64.D0*Hr2(1,0)&
          - 248.D0/3.D0*Hr2(1,0)*x + 232.D0/3.D0*Hr2(1,0)*x**2 + 344.D0&
          /9.D0*Hr2(1,1) - 376.D0/9.D0*Hr2(1,1)*x + 328.D0/9.D0*Hr2(1,1)*x**2 )

     PTqg2 = PTqg2 + TF*twonfTF*CF*(32.D0*Hr3(0,-1,0) - 64.D0*Hr3(0,-1,0)&
          *x + 64.D0/3.D0*Hr3(0,-1,0)*x**2 - 32.D0/3.D0*Hr3(0,0,0) +&
          112.D0/3.D0*Hr3(0,0,0)*x + 32.D0*Hr3(0,0,0)*x**2 - 64.D0/3.D0&
          *Hr3(0,0,1) + 128.D0/3.D0*Hr3(0,0,1)*x - 176.D0/3.D0*Hr3(0,0,1)&
          *x**2 + 80.D0/3.D0*Hr3(0,1,0) - 160.D0/3.D0*Hr3(0,1,0)*x +&
          176.D0/3.D0*Hr3(0,1,0)*x**2 + 64.D0/3.D0*Hr3(0,1,1) - 128.D0/&
          3.D0*Hr3(0,1,1)*x + 48.D0*Hr3(0,1,1)*x**2 + 32.D0*Hr3(1,0,0)&
          -64.D0*Hr3(1,0,0)*x + 64.D0*Hr3(1,0,0)*x**2 - 40.D0/3.D0*&
          Hr3(1,0,1) + 80.D0/3.D0*Hr3(1,0,1)*x - 80.D0/3.D0*Hr3(1,0,1)*&
          x**2 - 56.D0/3.D0*Hr3(1,1,0) + 112.D0/3.D0*Hr3(1,1,0)*x - 112.D0&
          /3.D0*Hr3(1,1,0)*x**2 - 40.D0/3.D0*Hr3(1,1,1) + 80.D0/3.D0*&
          Hr3(1,1,1)*x - 80.D0/3.D0*Hr3(1,1,1)*x**2 - 32.D0*Hr4(0,0,0,0)&
          + 64.D0*Hr4(0,0,0,0)*x )

     PTqg2 = PTqg2 + TF*twonfTF**2*(-8.D0/9.D0+16.D0/3.D0*x-16.D0/3.D0&
          *x**2 + 8.D0/3.D0*z2 - 16.D0/3.D0*z2*x + 16.D0/3.D0*z2*x**2&
          + 40.D0/9.D0*Hr1(0) - 32.D0/9.D0*Hr1(0)*x + 32.D0/9.D0*Hr1(0)&
          *x**2 - 40.D0/9.D0*Hr1(1) + 32.D0/9.D0*Hr1(1)*x - 32.D0/9.D0&
          *Hr1(1)*x**2 + 8.D0/3.D0*Hr2(0,0) - 16.D0/3.D0*Hr2(0,0)*x +&
          16.D0/3.D0*Hr2(0,0)*x**2 - 8.D0/3.D0*Hr2(0,1) + 16.D0/3.D0*&
          Hr2(0,1)*x - 16.D0/3.D0*Hr2(0,1)*x**2 - 8.D0/3.D0*Hr2(1,0) +&
          16.D0/3.D0*Hr2(1,0)*x - 16.D0/3.D0*Hr2(1,0)*x**2 + 8.D0/3.D0*&
          Hr2(1,1) - 16.D0/3.D0*Hr2(1,1)*x + 16.D0/3.D0*Hr2(1,1)*x**2 )

     ! the uncertainty for PqgT
     ! see Eq.(38) in arXiv:1107.2263
     PTqg2 = PTqg2&
          + two*z2*(CA-CF)*b0*two*TF*(one-two*x+two*x*x)&
          *( 11D0+24D0*Hr1(0))*IMOD
     
     res=PTqg2/8d0

     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF     
     RETURN
   END FUNCTION AP_SFT_P2qg

   ! Time Like
   ! P^{(2)}_{gq}
   ! Eq.(19) in arXiv:1107.2263
   ! Expressions are copyed from subroutine X2GQTA in xpij2te.f
   ! by A. Almasy, S. Moch and A. Vogt, arXiv:1107.2263 [hep-ph]
   FUNCTION AP_SFT_P2gq(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
     REAL(KIND(1d0))::PTgq2,twoTf,twonfTf
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero

     IF(cc_piece.EQ.cc_Delta)RETURN
     IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

     DX=one/x
     twoTf=two*TF
     twonfTf=two*nfTF
     CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
          HI1,HI2,HI3,HI4, N1, N2)

     PTgq2 = &
          CF*CA**2*(-17798.D0/27.D0 - 3436.D0/27.D0*x + 4136.D0/9.D0&
          *x**2 + 3514.D0/9.D0*dx - 400.D0*z3 + 148.D0*z3*x - 640.D0/3.D0&
          *z3*x**2 + 160.D0*z3*dx - 848.D0*z2 - 2800.D0/9.D0*z2*x -&
          352.D0/9.D0*z2*x**2 - 5176.D0/9.D0*z2*dx + 572.D0/5.D0*z2**2&
          + 314.D0*z2**2*x + 1984.D0/5.D0*z2**2*dx - 16.D0*Hr1(-1)*z3&
          - 8.D0*Hr1(-1)*z3*x - 16.D0*Hr1(-1)*z3*dx - 168.D0*Hr1(-1)*&
          z2 - 160.D0*Hr1(-1)*z2*x - 32.D0/3.D0*Hr1(-1)*z2*x**2 - 176.D0&
          /3.D0*Hr1(-1)*z2*dx - 32612.D0/27.D0*Hr1(0) - 23630.D0/27.D0*&
          Hr1(0)*x - 12964.D0/27.D0*Hr1(0)*x**2 + 13384.D0/27.D0*Hr1(0)&
          *dx + 352.D0*Hr1(0)*z3 + 240.D0*Hr1(0)*z3*x + 416.D0*Hr1(0)*&
          z3*dx - 12.D0*Hr1(0)*z2 + 532.D0/3.D0*Hr1(0)*z2*x - 128.D0/3.D0&
          *Hr1(0)*z2*x**2 - 752.D0/3.D0*Hr1(0)*z2*dx - 22270.D0/27.D0*&
          Hr1(1) - 688.D0/27.D0*Hr1(1)*x - 1924.D0/9.D0*Hr1(1)*x**2 +&
          31504.D0/27.D0*Hr1(1)*dx - 656.D0*Hr1(1)*z3 + 328.D0*Hr1(1)*&
          z3*x + 656.D0*Hr1(1)*z3*dx - 280.D0*Hr1(1)*z2 - 32.D0*Hr1(1)*&
          z2*x )

     PTgq2 = PTgq2 + CF*CA**2*(-64.D0*Hr1(1)*z2*x**2 + 344.D0*&
          Hr1(1)*z2*dx - 32.D0*Hr2(-1,-1)*z2 - 16.D0*Hr2(-1,-1)*z2*x -&
          32.D0*Hr2(-1,-1)*z2*dx - 1280.D0/9.D0*Hr2(-1,0) + 1052.D0/9.D0&
          *Hr2(-1,0)*x + 704.D0/9.D0*Hr2(-1,0)*x**2 - 248.D0/3.D0*Hr2(-1,0)&
          *dx + 160.D0*Hr2(-1,0)*z2 + 80.D0*Hr2(-1,0)*z2*x + 160.D0&
          *Hr2(-1,0)*z2*dx - 144.D0*Hr2(0,-1)*z2 - 8.D0*Hr2(0,-1)*z2*x&
          - 64.D0*Hr2(0,-1)*z2*dx + 11848.D0/9.D0*Hr2(0,0) + 7280.D0/9.D0&
          *Hr2(0,0)*x + 3688.D0/9.D0*Hr2(0,0)*x**2 + 3920.D0/9.D0*&
          Hr2(0,0)*dx + 152.D0*Hr2(0,0)*z2 - 156.D0*Hr2(0,0)*z2*x - 128.D0&
          *Hr2(0,0)*z2*dx + 6896.D0/9.D0*Hr2(0,1) + 4220.D0/9.D0*Hr2(0,1)&
          *x + 1552.D0/9.D0*Hr2(0,1)*x**2 + 4664.D0/9.D0*Hr2(0,1)*&
          dx + 48.D0*Hr2(0,1)*z2 + 248.D0*Hr2(0,1)*z2*x + 320.D0*Hr2(0,1)&
          *z2*dx + 1160.D0/9.D0*Hr2(1,0) - 604.D0/9.D0*Hr2(1,0)*x +&
          760.D0/3.D0*Hr2(1,0)*x**2 - 4136.D0/9.D0*Hr2(1,0)*dx + 128.D0&
          *Hr2(1,0)*z2 - 64.D0*Hr2(1,0)*z2*x - 128.D0*Hr2(1,0)*z2*dx +&
          808.D0/9.D0*Hr2(1,1) )

     PTgq2 = PTgq2 + CF*CA**2*(-440.D0/9.D0*Hr2(1,1)*x + 560.D0/9.D0&
          *Hr2(1,1)*x**2 - 532.D0/3.D0*Hr2(1,1)*dx - 96.D0*Hr2(1,1)*&
          z2 + 48.D0*Hr2(1,1)*z2*x + 96.D0*Hr2(1,1)*z2*dx - 16.D0*Hr3(-1,-1,0)&
          - 160.D0*Hr3(-1,-1,0)*x + 64.D0/3.D0*Hr3(-1,-1,0)*&
          x**2 + 352.D0/3.D0*Hr3(-1,-1,0)*dx - 328.D0/3.D0*Hr3(-1,0,0)&
          + 544.D0/3.D0*Hr3(-1,0,0)*x - 128.D0/3.D0*Hr3(-1,0,0)*x**2&
          - 272.D0*Hr3(-1,0,0)*dx + 160.D0*Hr3(-1,0,1) + 80.D0*Hr3(-1,0,1)&
          *x + 64.D0/3.D0*Hr3(-1,0,1)*x**2 + 352.D0/3.D0*Hr3(-1,0,1)&
          *dx - 280.D0/3.D0*Hr3(0,-1,0) + 928.D0/3.D0*Hr3(0,-1,0)*x -&
          256.D0/3.D0*Hr3(0,-1,0)*x**2 - 752.D0/3.D0*Hr3(0,-1,0)*dx -&
          848.D0/3.D0*Hr3(0,0,0) - 3344.D0/3.D0*Hr3(0,0,0)*x - 224.D0/3.D0&
          *Hr3(0,0,0)*x**2 + 1856.D0/3.D0*Hr3(0,0,0)*dx - 1292.D0/3.D0&
          *Hr3(0,0,1) - 284.D0/3.D0*Hr3(0,0,1)*x - 128.D0/3.D0*Hr3(0,0,1)&
          *x**2 - 32.D0/3.D0*Hr3(0,0,1)*dx - 576.D0*Hr3(0,1,0) - 424.D0&
          *Hr3(0,1,0)*x - 512.D0/3.D0*Hr3(0,1,0)*x**2 + 1504.D0/3.D0*&
          Hr3(0,1,0)*dx )

     PTgq2 = PTgq2 + CF*CA**2*(-296.D0/3.D0*Hr3(0,1,1)-260.D0/3.D0&
          *Hr3(0,1,1)*x - 128.D0/3.D0*Hr3(0,1,1)*x**2 + 448.D0/3.D0*&
          Hr3(0,1,1)*dx + 1300.D0/3.D0*Hr3(1,0,0) - 1004.D0/3.D0*Hr3(1,0,0)&
          *x - 1024.D0/3.D0*Hr3(1,0,0)*dx + 192.D0*Hr3(1,0,1) - 72.D0&
          *Hr3(1,0,1)*x + 32.D0*Hr3(1,0,1)*x**2 - 248.D0*Hr3(1,0,1)*dx&
          +368.D0/3.D0*Hr3(1,1,0) - 352.D0/3.D0*Hr3(1,1,0)*x + 32.D0/&
          3.D0*Hr3(1,1,0)*x**2 - 424.D0/3.D0*Hr3(1,1,0)*dx + 104.D0/3.D0&
          *Hr3(1,1,1) - 76.D0/3.D0*Hr3(1,1,1)*x + 32.D0/3.D0*Hr3(1,1,1)&
          *x**2 - 160.D0/3.D0*Hr3(1,1,1)*dx - 192.D0*Hr4(-1,-1,-1,0) -&
          96.D0*Hr4(-1,-1,-1,0)*x - 192.D0*Hr4(-1,-1,-1,0)*dx + 192.D0*&
          Hr4(-1,-1,0,0) + 96.D0*Hr4(-1,-1,0,0)*x + 192.D0*Hr4(-1,-1,0,0)&
          *dx - 64.D0*Hr4(-1,-1,0,1) - 32.D0*Hr4(-1,-1,0,1)*x - 64.D0&
          *Hr4(-1,-1,0,1)*dx + 352.D0*Hr4(-1,0,-1,0) + 176.D0*Hr4(-1,0,-1,0)&
          *x + 352.D0*Hr4(-1,0,-1,0)*dx - 336.D0*Hr4(-1,0,0,0) -&
          168.D0*Hr4(-1,0,0,0)*x - 336.D0*Hr4(-1,0,0,0)*dx + 64.D0*Hr4(-1,0,0,1) )

     PTgq2 = PTgq2 + CF*CA**2*(32.D0*Hr4(-1,0,0,1)*x + 64.D0*Hr4(-1,0,0,1)&
          *dx - 192.D0*Hr4(-1,0,1,0) - 96.D0*Hr4(-1,0,1,0)*x -&
          192.D0*Hr4(-1,0,1,0)*dx - 64.D0*Hr4(-1,0,1,1) - 32.D0*Hr4(-1,0,1,1)&
          *x - 64.D0*Hr4(-1,0,1,1)*dx - 160.D0*Hr4(0,-1,-1,0) +&
          48.D0*Hr4(0,-1,-1,0)*x + 240.D0*Hr4(0,-1,0,0) - 136.D0*Hr4(0,-1,0,0)&
          *x - 96.D0*Hr4(0,-1,0,0)*dx + 64.D0*Hr4(0,-1,0,1) + 32.D0&
          *Hr4(0,-1,0,1)*x + 64.D0*Hr4(0,-1,0,1)*dx + 112.D0*Hr4(0,0,-1,0)&
          - 248.D0*Hr4(0,0,-1,0)*x - 256.D0*Hr4(0,0,-1,0)*dx -&
          128.D0*Hr4(0,0,0,0) + 632.D0*Hr4(0,0,0,0)*x + 512.D0*Hr4(0,0,0,0)&
          *dx - 152.D0*Hr4(0,0,0,1) - 92.D0*Hr4(0,0,0,1)*x - 128.D0&
          *Hr4(0,0,0,1)*dx + 384.D0*Hr4(0,0,1,0) + 480.D0*Hr4(0,0,1,0)*&
          x + 576.D0*Hr4(0,0,1,0)*dx + 128.D0*Hr4(0,0,1,1) + 160.D0*&
          Hr4(0,0,1,1)*x + 192.D0*Hr4(0,0,1,1)*dx + 232.D0*Hr4(0,1,0,0)&
          -76.D0*Hr4(0,1,0,0)*x - 160.D0*Hr4(0,1,0,0)*dx + 64.D0*Hr4(0,1,0,1)&
          - 176.D0*Hr4(0,1,0,1)*x - 256.D0*Hr4(0,1,0,1)*dx +&
          128.D0*Hr4(0,1,1,0) )

     PTgq2 = PTgq2 + CF*CA**2*(-112.D0*Hr4(0,1,1,0)*x - 192.D0*&
          Hr4(0,1,1,0)*dx + 64.D0*Hr4(0,1,1,1) - 80.D0*Hr4(0,1,1,1)*x&
          -128.D0*Hr4(0,1,1,1)*dx + 96.D0*Hr4(1,0,-1,0) - 48.D0*Hr4(1,0,-1,0)&
          *x - 96.D0*Hr4(1,0,-1,0)*dx - 560.D0*Hr4(1,0,0,0) +&
          280.D0*Hr4(1,0,0,0)*x + 560.D0*Hr4(1,0,0,0)*dx - 160.D0*Hr4(1,0,0,1)&
          + 80.D0*Hr4(1,0,0,1)*x + 160.D0*Hr4(1,0,0,1)*dx - 640.D0&
          *Hr4(1,0,1,0) + 320.D0*Hr4(1,0,1,0)*x + 640.D0*Hr4(1,0,1,0)&
          *dx - 64.D0*Hr4(1,0,1,1) + 32.D0*Hr4(1,0,1,1)*x + 64.D0*Hr4(1,0,1,1)&
          *dx - 160.D0*Hr4(1,1,0,0) + 80.D0*Hr4(1,1,0,0)*x + 160.D0&
          *Hr4(1,1,0,0)*dx - 128.D0*Hr4(1,1,0,1) + 64.D0*Hr4(1,1,0,1)&
          *x + 128.D0*Hr4(1,1,0,1)*dx - 192.D0*Hr4(1,1,1,0) + 96.D0*&
          Hr4(1,1,1,0)*x + 192.D0*Hr4(1,1,1,0)*dx - 64.D0*Hr4(1,1,1,1)&
          + 32.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)*dx )

     PTgq2 = PTgq2 + CF**2*CA*(1735.D0/6.D0-4811.D0/12.D0*x-140.D0&
          /3.D0*x**2 + 200.D0*dx + 552.D0*z3 + 424.D0*z3*x + 128.D0/3.D0&
          *z3*x**2 + 264.D0*z3*dx + 3452.D0/3.D0*z2 + 436.D0/3.D0*z2*&
          x + 1760.D0/9.D0*z2*x**2 + 528.D0*z2*dx - 608.D0/5.D0*z2**2&
          -2144.D0/5.D0*z2**2*x - 3664.D0/5.D0*z2**2*dx - 176.D0*Hr1(-1)&
          *z3 - 88.D0*Hr1(-1)*z3*x - 176.D0*Hr1(-1)*z3*dx - 64.D0*&
          Hr1(-1)*z2 - 32.D0*Hr1(-1)*z2*x - 24.D0*Hr1(-1)*z2*dx + 40301.D0&
          /27.D0*Hr1(0) + 18911.D0/27.D0*Hr1(0)*x + 8932.D0/27.D0*&
          Hr1(0)*x**2 + 244.D0*Hr1(0)*dx - 272.D0*Hr1(0)*z3 - 376.D0*&
          Hr1(0)*z3*x - 832.D0*Hr1(0)*z3*dx + 72.D0*Hr1(0)*z2 + 128.D0*&
          Hr1(0)*z2*x - 32.D0/3.D0*Hr1(0)*z2*x**2 + 192.D0*Hr1(0)*z2*dx&
          +26008.D0/27.D0*Hr1(1) + 3988.D0/27.D0*Hr1(1)*x + 5908.D0/&
          27.D0*Hr1(1)*x**2 - 39092.D0/27.D0*Hr1(1)*dx + 816.D0*Hr1(1)*&
          z3 - 408.D0*Hr1(1)*z3*x - 816.D0*Hr1(1)*z3*dx + 672.D0*Hr1(1)&
          *z2 - 180.D0*Hr1(1)*z2*x + 224.D0/3.D0*Hr1(1)*z2*x**2 - 1760.D0&
          /3.D0*Hr1(1)*z2*dx )

     PTgq2 = PTgq2 + CF**2*CA*(288.D0*Hr2(-1,-1)*z2 + 144.D0*Hr2(-1,-1)&
          *z2*x + 288.D0*Hr2(-1,-1)*z2*dx - 88.D0*Hr2(-1,0) - 268.D0&
          *Hr2(-1,0)*x + 192.D0*Hr2(-1,0)*dx - 192.D0*Hr2(-1,0)*z2 - 96.D0&
          *Hr2(-1,0)*z2*x - 192.D0*Hr2(-1,0)*z2*dx - 128.D0*Hr2(0,-1)&
          *z2 - 16.D0*Hr2(0,-1)*z2*x + 32.D0*Hr2(0,-1)*z2*dx - 2338.D0/&
          3.D0*Hr2(0,0) - 441.D0*Hr2(0,0)*x - 1408.D0/9.D0*Hr2(0,0)*&
          x**2 + 80.D0*Hr2(0,0)*dx - 16.D0*Hr2(0,0)*z2 - 40.D0*Hr2(0,0)&
          *z2*x - 128.D0*Hr2(0,0)*z2*dx - 8980.D0/9.D0*Hr2(0,1) - 4576.D0&
          /9.D0*Hr2(0,1)*x - 352.D0/3.D0*Hr2(0,1)*x**2 - 5800.D0/9.D0*&
          Hr2(0,1)*dx + 272.D0*Hr2(0,1)*z2 - 456.D0*Hr2(0,1)*z2*x - 608.D0&
          *Hr2(0,1)*z2*dx + 488.D0/3.D0*Hr2(1,0) - 200.D0*Hr2(1,0)*x&
          -704.D0/9.D0*Hr2(1,0)*x**2 + 620.D0/9.D0*Hr2(1,0)*dx - 32.D0&
          *Hr2(1,0)*z2 + 16.D0*Hr2(1,0)*z2*x + 32.D0*Hr2(1,0)*z2*dx -&
          160.D0/9.D0*Hr2(1,1) - 280.D0/9.D0*Hr2(1,1)*x - 352.D0/9.D0*&
          Hr2(1,1)*x**2 + 848.D0/9.D0*Hr2(1,1)*dx + 192.D0*Hr2(1,1)*z2&
          -96.D0*Hr2(1,1)*z2*x )

     PTgq2 = PTgq2 + CF**2*CA*(-192.D0*Hr2(1,1)*z2*dx + 192.D0*&
          Hr3(-1,-1,0) + 128.D0*Hr3(-1,-1,0)*x + 48.D0*Hr3(-1,-1,0)*dx&
          -192.D0*Hr3(-1,0,0) - 72.D0*Hr3(-1,0,0)*x - 48.D0*Hr3(-1,0,0)&
          *dx + 160.D0*Hr3(-1,0,1) + 96.D0*Hr3(-1,0,1)*x + 48.D0*Hr3(-1,0,1)&
          *dx - 144.D0*Hr3(0,-1,0) + 128.D0*Hr3(0,-1,0)*x + 96.D0&
          *Hr3(0,-1,0)*dx + 368.D0/3.D0*Hr3(0,0,0) - 268.D0/3.D0*Hr3(0,0,0)&
          *x + 64.D0/3.D0*Hr3(0,0,0)*x**2 + 136.D0/3.D0*Hr3(0,0,1)&
          -680.D0/3.D0*Hr3(0,0,1)*x - 32.D0*Hr3(0,0,1)*x**2 + 640.D0/&
          3.D0*Hr3(0,0,1)*dx + 600.D0*Hr3(0,1,0) + 96.D0*Hr3(0,1,0)*x&
          +224.D0/3.D0*Hr3(0,1,0)*x**2 - 1280.D0/3.D0*Hr3(0,1,0)*dx&
          +424.D0/3.D0*Hr3(0,1,1) + 28.D0/3.D0*Hr3(0,1,1)*x + 32.D0*&
          Hr3(0,1,1)*x**2 - 464.D0/3.D0*Hr3(0,1,1)*dx - 1760.D0/3.D0*&
          Hr3(1,0,0) + 832.D0/3.D0*Hr3(1,0,0)*x - 64.D0/3.D0*Hr3(1,0,0)&
          *x**2 + 336.D0*Hr3(1,0,0)*dx - 976.D0/3.D0*Hr3(1,0,1) + 404.D0&
          /3.D0*Hr3(1,0,1)*x - 32.D0*Hr3(1,0,1)*x**2 + 904.D0/3.D0*Hr3(1,0,1)*dx )

     PTgq2 = PTgq2+CF**2*CA*(-256.D0/3.D0*Hr3(1,1,0) + 332.D0/3.D0&
          *Hr3(1,1,0)*x - 32.D0/3.D0*Hr3(1,1,0)*x**2 + 88.D0*Hr3(1,1,0)&
          *dx - 152.D0/3.D0*Hr3(1,1,1) + 232.D0/3.D0*Hr3(1,1,1)*x -&
          32.D0/3.D0*Hr3(1,1,1)*x**2 + 160.D0/3.D0*Hr3(1,1,1)*dx + 192.D0&
          *Hr4(-1,-1,-1,0) + 96.D0*Hr4(-1,-1,-1,0)*x + 192.D0*Hr4(-1,-1,-1,0)&
          *dx + 64.D0*Hr4(-1,-1,0,0) + 32.D0*Hr4(-1,-1,0,0)*x&
          +64.D0*Hr4(-1,-1,0,0)*dx - 192.D0*Hr4(-1,-1,0,1) - 96.D0*&
          Hr4(-1,-1,0,1)*x - 192.D0*Hr4(-1,-1,0,1)*dx - 160.D0*Hr4(-1,0,-1,0)&
          - 80.D0*Hr4(-1,0,-1,0)*x - 160.D0*Hr4(-1,0,-1,0)*dx -&
          16.D0*Hr4(-1,0,0,0) - 8.D0*Hr4(-1,0,0,0)*x - 16.D0*Hr4(-1,0,0,0)&
          *dx - 64.D0*Hr4(-1,0,0,1) - 32.D0*Hr4(-1,0,0,1)*x - 64.D0*&
          Hr4(-1,0,0,1)*dx + 128.D0*Hr4(-1,0,1,0) + 64.D0*Hr4(-1,0,1,0)&
          *x + 128.D0*Hr4(-1,0,1,0)*dx + 64.D0*Hr4(-1,0,1,1) + 32.D0*&
          Hr4(-1,0,1,1)*x + 64.D0*Hr4(-1,0,1,1)*dx - 128.D0*Hr4(0,-1,-1,0)&
          - 96.D0*Hr4(0,-1,-1,0)*x - 64.D0*Hr4(0,-1,-1,0)*dx - 32.D0&
          *Hr4(0,-1,0,0) )

     PTgq2 = PTgq2+CF**2*CA*(-32.D0*Hr4(0,-1,0,0)*x + 64.D0*&
          Hr4(0,-1,0,1) - 32.D0*Hr4(0,-1,0,1)*x - 64.D0*Hr4(0,-1,0,1)*&
          dx - 32.D0*Hr4(0,0,-1,0) + 48.D0*Hr4(0,0,-1,0)*x + 64.D0*Hr4(0,0,0,0)&
          + 64.D0*Hr4(0,0,0,0)*x + 336.D0*Hr4(0,0,0,1) + 376.D0&
          *Hr4(0,0,0,1)*x + 512.D0*Hr4(0,0,0,1)*dx - 208.D0*Hr4(0,0,1,0)&
          - 344.D0*Hr4(0,0,1,0)*x - 448.D0*Hr4(0,0,1,0)*dx - 16.D0*&
          Hr4(0,0,1,1) - 216.D0*Hr4(0,0,1,1)*x - 320.D0*Hr4(0,0,1,1)*dx&
          -320.D0*Hr4(0,1,0,0) + 208.D0*Hr4(0,1,0,0)*x + 256.D0*Hr4(0,1,0,0)&
          *dx - 336.D0*Hr4(0,1,0,1) + 312.D0*Hr4(0,1,0,1)*x +&
          448.D0*Hr4(0,1,0,1)*dx - 336.D0*Hr4(0,1,1,0) + 216.D0*Hr4(0,1,1,0)&
          *x + 384.D0*Hr4(0,1,1,0)*dx - 272.D0*Hr4(0,1,1,1) + 184.D0&
          *Hr4(0,1,1,1)*x + 320.D0*Hr4(0,1,1,1)*dx - 96.D0*Hr4(1,0,-1,0)&
          + 48.D0*Hr4(1,0,-1,0)*x + 96.D0*Hr4(1,0,-1,0)*dx + 48.D0*&
          Hr4(1,0,0,0) - 24.D0*Hr4(1,0,0,0)*x - 48.D0*Hr4(1,0,0,0)*dx&
          -224.D0*Hr4(1,0,0,1) + 112.D0*Hr4(1,0,0,1)*x + 224.D0*Hr4(1,0,0,1)*dx)

     PTgq2 = PTgq2 + CF**2*CA*(544.D0*Hr4(1,0,1,0)-272.D0*Hr4(1,0,1,0)&
          *x - 544.D0*Hr4(1,0,1,0)*dx - 64.D0*Hr4(1,0,1,1) + 32.D0&
          *Hr4(1,0,1,1)*x + 64.D0*Hr4(1,0,1,1)*dx - 160.D0*Hr4(1,1,0,0)&
          +80.D0*Hr4(1,1,0,0)*x + 160.D0*Hr4(1,1,0,0)*dx + 32.D0*Hr4(1,1,0,1)&
          - 16.D0*Hr4(1,1,0,1)*x - 32.D0*Hr4(1,1,0,1)*dx + 224.D0&
          *Hr4(1,1,1,0) - 112.D0*Hr4(1,1,1,0)*x - 224.D0*Hr4(1,1,1,0)&
          *dx + 128.D0*Hr4(1,1,1,1) - 64.D0*Hr4(1,1,1,1)*x - 128.D0*&
          Hr4(1,1,1,1)*dx)

     PTgq2 = PTgq2 + CF**3*(-1915.D0/2.D0+731.D0/4.D0*x+794.D0&
          *dx - 96.D0*z3 - 196.D0*z3*x - 720.D0*z3*dx - 224.D0*z2 + 148.D0&
          *z2*x - 256.D0*z2*dx - 776.D0/5.D0*z2**2 + 84.D0/5.D0*z2**2&
          *x + 1216.D0/5.D0*z2**2*dx - 64.D0*Hr1(-1)*z2 - 32.D0*Hr1(-1)&
          *z2*x + 197.D0*Hr1(0) + 29.D0*Hr1(0)*x + 208.D0*Hr1(0)*dx-&
          80.D0*Hr1(0)*z3 - 200.D0*Hr1(0)*z3*x + 128.D0*Hr1(0)*z3*dx-&
          24.D0*Hr1(0)*z2 - 20.D0*Hr1(0)*z2*x - 192.D0*Hr1(0)*z2*dx-&
          230.D0*Hr1(1) + 16.D0*Hr1(1)*x + 228.D0*Hr1(1)*dx - 64.D0*&
          Hr1(1)*z3 + 32.D0*Hr1(1)*z3*x + 64.D0*Hr1(1)*z3*dx - 352.D0*&
          Hr1(1)*z2 + 252.D0*Hr1(1)*z2*x + 144.D0*Hr1(1)*z2*dx - 80.D0*&
          Hr2(-1,0) + 208.D0*Hr2(-1,0)*x - 256.D0*Hr2(-1,0)*dx + 128.D0&
          *Hr2(0,-1)*z2 + 96.D0*Hr2(0,-1)*z2*x + 128.D0*Hr2(0,-1)*z2*dx&
          -290.D0*Hr2(0,0) - 251.D0*Hr2(0,0)*x - 64.D0*Hr2(0,0)*z2 -&
          64.D0*Hr2(0,0)*z2*x + 208.D0*Hr2(0,1) - 140.D0*Hr2(0,1)*x -&
          464.D0*Hr2(0,1)*z2 + 200.D0*Hr2(0,1)*z2*x + 320.D0*Hr2(0,1)*&
          z2*dx )

     PTgq2 = PTgq2 + CF**3*(-264.D0*Hr2(1,0)+140.D0*Hr2(1,0)*x&
          +180.D0*Hr2(1,0)*dx - 128.D0*Hr2(1,0)*z2 + 64.D0*Hr2(1,0)*&
          z2*x + 128.D0*Hr2(1,0)*z2*dx - 72.D0*Hr2(1,1) + 80.D0*Hr2(1,1)&
          *x + 60.D0*Hr2(1,1)*dx - 96.D0*Hr2(1,1)*z2 + 48.D0*Hr2(1,1)*&
          z2*x + 96.D0*Hr2(1,1)*z2*dx - 128.D0*Hr3(-1,-1,0) - 64.D0*&
          Hr3(-1,-1,0)*x + 224.D0*Hr3(-1,0,0) + 96.D0*Hr3(-1,0,0)*x +&
          96.D0*Hr3(-1,0,0)*dx + 128.D0*Hr3(0,-1,0) - 112.D0*Hr3(0,-1,0)&
          *x - 192.D0*Hr3(0,-1,0)*dx + 160.D0*Hr3(0,0,0) + 40.D0*Hr3(0,0,0)&
          *x + 248.D0*Hr3(0,0,1) - 28.D0*Hr3(0,0,1)*x - 104.D0*&
          Hr3(0,1,0) + 40.D0*Hr3(0,1,0)*x - 72.D0*Hr3(0,1,1) + 56.D0*&
          Hr3(0,1,1)*x + 128.D0*Hr3(1,0,0) - 36.D0*Hr3(1,0,0)*x + 48.D0&
          *Hr3(1,0,0)*dx + 128.D0*Hr3(1,0,1) - 60.D0*Hr3(1,0,1)*x - 48.D0&
          *Hr3(1,0,1)*dx - 32.D0*Hr3(1,1,0) + 4.D0*Hr3(1,1,0)*x + 48.D0&
          *Hr3(1,1,0)*dx + 16.D0*Hr3(1,1,1) - 52.D0*Hr3(1,1,1)*x - 128.D0&
          *Hr4(-1,-1,0,0) - 64.D0*Hr4(-1,-1,0,0)*x - 128.D0*Hr4(-1,-1,0,0)*dx )

     PTgq2 = PTgq2 +CF**3*(-64.D0*Hr4(-1,0,-1,0)-32.D0*Hr4(-1,0,-1,0)&
          *x - 64.D0*Hr4(-1,0,-1,0)*dx + 32.D0*Hr4(-1,0,0,0) +&
          16.D0*Hr4(-1,0,0,0)*x + 32.D0*Hr4(-1,0,0,0)*dx + 256.D0*Hr4(0,-1,-1,0)&
          + 192.D0*Hr4(0,-1,-1,0)*x + 256.D0*Hr4(0,-1,-1,0)*&
          dx - 64.D0*Hr4(0,-1,0,0) - 64.D0*Hr4(0,-1,0,0)*x - 128.D0*&
          Hr4(0,-1,0,0)*dx - 64.D0*Hr4(0,0,-1,0) - 96.D0*Hr4(0,0,-1,0)*&
          x - 32.D0*Hr4(0,0,0,0) + 96.D0*Hr4(0,0,0,0)*x - 128.D0*Hr4(0,0,0,1)&
          + 64.D0*Hr4(0,0,0,1)*x - 176.D0*Hr4(0,0,1,0) + 88.D0*&
          Hr4(0,0,1,0)*x + 128.D0*Hr4(0,0,1,0)*dx - 240.D0*Hr4(0,0,1,1)&
          +120.D0*Hr4(0,0,1,1)*x + 256.D0*Hr4(0,0,1,1)*dx - 16.D0*&
          Hr4(0,1,0,0) + 40.D0*Hr4(0,1,0,0)*x + 128.D0*Hr4(0,1,0,0)*dx&
          +144.D0*Hr4(0,1,0,1) - 72.D0*Hr4(0,1,0,1)*x - 64.D0*Hr4(0,1,0,1)&
          *dx + 80.D0*Hr4(0,1,1,0) - 40.D0*Hr4(0,1,1,0)*x - 64.D0*&
          Hr4(0,1,1,0)*dx + 208.D0*Hr4(0,1,1,1) - 104.D0*Hr4(0,1,1,1)*x&
          -192.D0*Hr4(0,1,1,1)*dx + 64.D0*Hr4(1,0,-1,0) - 32.D0*Hr4(1,0,-1,0)*x )

     PTgq2 = PTgq2+CF**3*(-64.D0*Hr4(1,0,-1,0)*dx+192.D0*Hr4(1,0,0,0)&
          - 96.D0*Hr4(1,0,0,0)*x - 192.D0*Hr4(1,0,0,0)*dx +&
          256.D0*Hr4(1,0,0,1) - 128.D0*Hr4(1,0,0,1)*x - 256.D0*Hr4(1,0,0,1)&
          *dx - 32.D0*Hr4(1,0,1,0) + 16.D0*Hr4(1,0,1,0)*x + 32.D0*&
          Hr4(1,0,1,0)*dx + 128.D0*Hr4(1,0,1,1) - 64.D0*Hr4(1,0,1,1)*x&
          -128.D0*Hr4(1,0,1,1)*dx + 192.D0*Hr4(1,1,0,0) - 96.D0*Hr4(1,1,0,0)*x&
          - 192.D0*Hr4(1,1,0,0)*dx + 96.D0*Hr4(1,1,0,1) - 48.D0&
          *Hr4(1,1,0,1)*x - 96.D0*Hr4(1,1,0,1)*dx - 32.D0*Hr4(1,1,1,0)&
          +16.D0*Hr4(1,1,1,0)*x + 32.D0*Hr4(1,1,1,0)*dx - 64.D0*Hr4(1,1,1,1)&
          + 32.D0*Hr4(1,1,1,1)*x + 64.D0*Hr4(1,1,1,1)*dx )

     PTgq2 = PTgq2 + twonfTf*CF*CA*(332.D0/27.D0-2624.D0/27.D0*x +&
          1892.D0/81.D0*x**2 + 3778.D0/81.D0*dx + 64.D0*z3 - 40.D0*z3*x&
          -80.D0*z3*dx + 8.D0/3.D0*z2 - 176.D0/9.D0*z2*x - 160.D0/9.D0&
          *z2*dx + 1280.D0/27.D0*Hr1(0) + 308.D0/27.D0*Hr1(0)*x + 64.D0/&
          9.D0*Hr1(0)*x**2 + 1156.D0/27.D0*Hr1(0)*dx - 16.D0/3.D0*Hr1(0)&
          *z2*x - 32.D0/3.D0*Hr1(0)*z2*dx + 448.D0/27.D0*Hr1(1) - 392.D0&
          /27.D0*Hr1(1)*x - 448.D0/27.D0*Hr1(1)*dx - 32.D0*Hr1(1)*z2&
          +16.D0*Hr1(1)*z2*x + 32.D0*Hr1(1)*z2*dx - 160.D0/9.D0*Hr2(-1,0)&
          - 104.D0/9.D0*Hr2(-1,0)*x - 160.D0/9.D0*Hr2(-1,0)*dx -&
          256.D0/9.D0*Hr2(0,0) - 464.D0/9.D0*Hr2(0,0)*x - 128.D0/9.D0*&
          Hr2(0,0)*x**2 + 656.D0/9.D0*Hr2(0,0)*dx + 136.D0/9.D0*Hr2(0,1)&
          - 104.D0/9.D0*Hr2(0,1)*x - 160.D0/9.D0*Hr2(0,1)*dx - 320.D0/&
          9.D0*Hr2(1,0) + 136.D0/9.D0*Hr2(1,0)*x + 320.D0/9.D0*Hr2(1,0)&
          *dx - 160.D0/9.D0*Hr2(1,1) + 104.D0/9.D0*Hr2(1,1)*x + 160.D0/&
          9D0*Hr2(1,1)*dx - 32.D0/3.D0*Hr3(-1,0,0) - 16.D0/3.D0*Hr3(-1,0,0)*x )

     PTgq2 = PTgq2 + twonfTf*CF*CA*(-32.D0/3.D0*Hr3(-1,0,0)*dx-32.D0/&
          3.D0*Hr3(0,-1,0) - 16.D0/3.D0*Hr3(0,-1,0)*x - 32.D0/3.D0*Hr3(0,-1,0)&
          *dx + 32.D0/3.D0*Hr3(0,0,0) + 128.D0/3.D0*Hr3(0,0,0)*x&
          +128.D0/3.D0*Hr3(0,0,0)*dx + 32.D0/3.D0*Hr3(0,0,1) - 16.D0/&
          3.D0*Hr3(0,0,1)*x - 32.D0/3.D0*Hr3(0,0,1)*dx - 16.D0/3.D0*&
          Hr3(0,1,1) + 8.D0/3.D0*Hr3(0,1,1)*x + 16.D0/3.D0*Hr3(0,1,1)*&
          dx - 160.D0/3.D0*Hr3(1,0,0) + 80.D0/3.D0*Hr3(1,0,0)*x + 160.D0&
          /3.D0*Hr3(1,0,0)*dx - 32.D0/3.D0*Hr3(1,1,0) + 16.D0/3.D0*Hr3(1,1,0)&
          *x + 32.D0/3.D0*Hr3(1,1,0)*dx + 16.D0/3.D0*Hr3(1,1,1)&
          -8.D0/3.D0*Hr3(1,1,1)*x - 16.D0/3.D0*Hr3(1,1,1)*dx )

     PTgq2 = PTgq2 + twonfTf*CF**2*(-12803.D0/27.D0+30475.D0/54.D0*x&
          -21784.D0/81.D0*x**2 + 14008.D0/81.D0*dx - 2330.D0/27.D0*&
          Hr1(0) + 2494.D0/27.D0*Hr1(0)*x + 3040.D0/27.D0*Hr1(0)*x**2&
          +200.D0/9.D0*Hr1(0)*dx - 484.D0/27.D0*Hr1(1) + 464.D0/27.D0&
          *Hr1(1)*x + 448.D0/27.D0*Hr1(1)*dx + 16.D0*Hr1(1)*z2 - 8.D0*&
          Hr1(1)*z2*x - 16.D0*Hr1(1)*z2*dx - 204.D0*Hr2(0,0) + 142.D0/3.D0&
          *Hr2(0,0)*x - 256.D0/9.D0*Hr2(0,0)*x**2 - 128.D0/3.D0*Hr2(0,0)&
          *dx - 320.D0/9.D0*Hr2(0,1) + 208.D0/9.D0*Hr2(0,1)*x + 320.D0&
          /9.D0*Hr2(0,1)*dx + 80.D0/3.D0*Hr2(1,0) - 40.D0/3.D0*Hr2(1,0)&
          *x - 80.D0/3.D0*Hr2(1,0)*dx + 160.D0/9.D0*Hr2(1,1) - 104.D0/&
          9.D0*Hr2(1,1)*x - 160.D0/9.D0*Hr2(1,1)*dx - 176.D0/3.D0*Hr3(0,0,0)&
          + 160.D0/3.D0*Hr3(0,0,0)*x - 128.D0/3.D0*Hr3(0,0,0)*dx&
          -64.D0/3.D0*Hr3(0,0,1) + 32.D0/3.D0*Hr3(0,0,1)*x + 64.D0/3.D0&
          *Hr3(0,0,1)*dx + 32.D0/3.D0*Hr3(0,1,1) - 16.D0/3.D0*Hr3(0,1,1)&
          *x - 32.D0/3.D0*Hr3(0,1,1)*dx + 80.D0/3.D0*Hr3(1,0,0) - 40.D0&
          /3.D0*Hr3(1,0,0)*x )

     PTgq2 = PTgq2 + twonfTf*CF**2*(-80.D0/3.D0*Hr3(1,0,0)*dx+16.D0/&
          3.D0*Hr3(1,0,1) - 8.D0/3.D0*Hr3(1,0,1)*x - 16.D0/3.D0*Hr3(1,0,1)&
          *dx + 16.D0/3.D0*Hr3(1,1,0) - 8.D0/3.D0*Hr3(1,1,0)*x - 16.D0&
          /3.D0*Hr3(1,1,0)*dx - 16.D0/3.D0*Hr3(1,1,1) + 8.D0/3.D0*Hr3(1,1,1)&
          *x + 16.D0/3.D0*Hr3(1,1,1)*dx + 64.D0*Hr4(0,0,0,0) - 32.D0&
          *Hr4(0,0,0,0)*x )
     
     res=PTgq2/8d0

     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_SFT_P2gq

   ! Time Like-Space Like
   ! P^{(2)}_{ns+}=P^{v,(2)}_{qq}+P^{v,(2)}_{qqbar}
   ! Eq.(15) in hep-ph/0604053
   ! Expressions are copyed from subroutine DTSP2NSP in xptsdiff.f
   ! by A. Mitov, S. Moch and A. Vogt,  hep-ph/0604053
   FUNCTION AP_DSFTS_P2NSplus(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,CAM2CF,PQQ0P,PQQ0M
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
     REAL(KIND(1d0))::ppdiff,twoTf,twonfTf
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero

     IF(cc_piece.EQ.cc_Delta)RETURN
     IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

     DX=one/x
     PQQ0P = two/(one-x)-one-x
     PQQ0M = two/(one+x)-one+x
     CAM2CF=CA-two*CF
     twoTf=two*TF
     twonfTf=two*nfTF
     CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
          HI1,HI2,HI3,HI4, N1, N2)

     ppdiff = &
          CF**2*CAM2CF* (-88.D0/9.D0*Hr1(0) + 88.D0/9.D0*Hr1(0)*x&
          +3.D0*Hr1(0)*z3 - 3.D0*Hr1(0)*z3*x - 4.D0*Hr1(0)*z2*x - 50.D0&
          /9.D0*Hr2(0,0) + 184.D0/9.D0*Hr2(0,0)*x - 4.D0*Hr2(0,0)*z2*x&
          -28.D0/3.D0*Hr2(0,1) + 28.D0/3.D0*Hr2(0,1)*x - 28.D0/3.D0*&
          Hr2(1,0) + 28.D0/3.D0*Hr2(1,0)*x - 8.D0*Hr3(-1,0,0) - 8.D0*&
          Hr3(-1,0,0)*x - 4.D0*Hr3(0,-1,0) - 4.D0*Hr3(0,-1,0)*x + 11.D0/&
          2.D0*Hr3(0,0,0) + 35.D0/2.D0*Hr3(0,0,0)*x + 4.D0*Hr4(0,-1,0,0)&
          - 4.D0*Hr4(0,-1,0,0)*x + 4.D0*Hr4(0,0,-1,0) - 4.D0*Hr4(0,0,-1,0)&
          *x + 8.D0*Hr4(0,0,0,0)*x )

     ppdiff = ppdiff + CF**2*CAM2CF*PQQ0M*(-7.D0*Hr1(0)*z3 + 3.D0&
          /2.D0*Hr1(0)*z2 + 8.D0*Hr2(-1,0)*z2 + 8.D0*Hr2(0,-1)*z2 - 8.D0&
          *Hr2(0,0)*z2 + 6.D0*Hr3(-1,0,0) + 3.D0*Hr3(0,-1,0) - 9.D0/2.D0&
          *Hr3(0,0,0) + 16.D0*Hr4(-1,-1,0,0) + 8.D0*Hr4(-1,0,-1,0) -&
          18.D0*Hr4(-1,0,0,0) - 8.D0*Hr4(-1,0,0,1) - 4.D0*Hr4(-1,0,1,0)&
          +8.D0*Hr4(0,-1,-1,0) - 14.D0*Hr4(0,-1,0,0) - 4.D0*Hr4(0,-1,0,1)&
          - 8.D0*Hr4(0,0,-1,0) + 8.D0*Hr4(0,0,0,0) + 6.D0*Hr4(0,0,0,1)&
          + 2.D0*Hr4(0,0,1,0) )

     ppdiff = ppdiff + CF**2*CAM2CF*PQQ0P*(151.D0/24.D0*Hr1(0) +&
          Hr1(0)*z3 + 13.D0/6.D0*Hr1(0)*z2 - 169.D0/18.D0*Hr2(0,0) + 8.D0&
          *Hr2(0,0)*z2 - 134.D0/9.D0*Hr2(0,1) + 4.D0*Hr2(0,1)*z2 - 134.D0&
          /9.D0*Hr2(1,0) + 4.D0*Hr2(1,0)*z2 - 13.D0/2.D0*Hr3(0,0,0)&
          -22.D0/3.D0*Hr3(0,0,1) - 22.D0/3.D0*Hr3(0,1,0) - 22.D0/3.D0&
          *Hr3(1,0,0) - 8.D0*Hr4(0,0,0,0) - 6.D0*Hr4(0,0,0,1) - 2.D0*&
          Hr4(0,0,1,0) - 2.D0*Hr4(0,1,0,0) - 6.D0*Hr4(1,0,0,0) )

     ppdiff = ppdiff+CF**3*(-325.D0/18.D0*Hr1(0) + 325.D0/18.D0&
          *Hr1(0)*x + 3.D0*Hr1(0)*z2 - 5.D0*Hr1(0)*z2*x - 173.D0/18.D0*&
          Hr2(0,0) + 691.D0/18.D0*Hr2(0,0)*x - 4.D0*Hr2(0,0)*z2 - 4.D0*&
          Hr2(0,0)*z2*x - 50.D0/3.D0*Hr2(0,1) + 50.D0/3.D0*Hr2(0,1)*x&
          -50.D0/3.D0*Hr2(1,0) + 50.D0/3.D0*Hr2(1,0)*x + 25.D0/2.D0*&
          Hr3(0,0,0) + 25.D0/2.D0*Hr3(0,0,0)*x + 2.D0*Hr3(0,0,1) + 2.D0&
          *Hr3(0,0,1)*x + Hr3(0,1,0) + Hr3(0,1,0)*x )

     ppdiff = ppdiff + CF**3*PQQ0P*(311.D0/24.D0*Hr1(0) + 4.D0/3.D0&
          *Hr1(0)*z2 - 169.D0/9.D0*Hr2(0,0) + 8.D0*Hr2(0,0)*z2 - 268.D0/&
          9.D0*Hr2(0,1) + 8.D0*Hr2(0,1)*z2 - 268.D0/9.D0*Hr2(1,0) + 8.D0&
          *Hr2(1,0)*z2 - 22.D0*Hr3(0,0,0) - 44.D0/3.D0*Hr3(0,0,1) - 44.D0&
          /3.D0*Hr3(0,1,0) - 44.D0/3.D0*Hr3(1,0,0) )

     ppdiff = ppdiff + twonfTF*CF**2*(13.D0/9.D0*Hr1(0) - 13.D0/9.D0*&
          Hr1(0)*x + 8.D0/9.D0*Hr2(0,0) - 28.D0/9.D0*Hr2(0,0)*x + 4.D0/&
          3.D0*Hr2(0,1) - 4.D0/3.D0*Hr2(0,1)*x + 4.D0/3.D0*Hr2(1,0) - 4.D0&
          /3.D0*Hr2(1,0)*x - Hr3(0,0,0) - Hr3(0,0,0)*x )

     ppdiff = ppdiff + twonfTF*CF**2*PQQ0P*(-11.D0/12.D0*Hr1(0)-2.D0/&
          3.D0*Hr1(0)*z2 + 11.D0/9.D0*Hr2(0,0) + 20.D0/9.D0*Hr2(0,1) +&
          20.D0/9.D0*Hr2(1,0) + 2.D0*Hr3(0,0,0) + 4.D0/3.D0*Hr3(0,0,1)&
          +4.D0/3.D0*Hr3(0,1,0) + 4.D0/3.D0*Hr3(1,0,0) )

     res=ppdiff*two
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_DSFTS_P2NSplus

   ! Time Like-Space Like
   ! P^{(2)}_{ns-}=P^{v,(2)}_{qq}-P^{v,(2)}_{qqbar}
   ! Eq.(16)+Eq.(15) in hep-ph/0604053
   ! Expressions are copyed from subroutine DTSP2NSM in xptsdiff.f
   ! by A. Mitov, S. Moch and A. Vogt,  hep-ph/0604053
   FUNCTION AP_DSFTS_P2NSminus(y,y_is_x) RESULT(res)
     USE hplog_module
     IMPLICIT NONE
     REAL(KIND(1d0)),INTENT(IN)::y
     LOGICAL,INTENT(IN),OPTIONAL::y_is_x
     REAL(KIND(1d0))::res
     INTEGER,PARAMETER::N1=-1,N2=1,NW=4
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2)::HC1
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HC2
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HC3
     COMPLEX(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HC4
     REAL(KIND(1d0)),DIMENSION(N1:N2)::HR1,HI1
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2)::HR2,HI2
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2)::HR3,HI3
     REAL(KIND(1d0)),DIMENSION(N1:N2,N1:N2,N1:N2,N1:N2)::HR4,HI4
     REAL(KIND(1d0))::x,DX,CAM2CF,PQQ0P,PQQ0M
     REAL(KIND(1d0)),PARAMETER::Z2=zeta2,Z3=zeta3,Z4=zeta4
     REAL(KIND(1d0))::pmdiff,twoTf,twonfTf
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        x=DEXP(-y)
     ELSE
        x=y
     ENDIF
     res=zero

     IF(cc_piece.EQ.cc_Delta)RETURN
     IF(cc_piece.NE.cc_REAL.AND.cc_piece.NE.cc_REALVIRT)RETURN

     DX=one/x
     PQQ0P = two/(one-x)-one-x
     PQQ0M = two/(one+x)-one+x
     CAM2CF=CA-two*CF
     twoTf=two*TF
     twonfTf=two*nfTF
     CALL HPLOG(X, NW, HC1,HC2,HC3,HC4, HR1,HR2,HR3,HR4,&
          HI1,HI2,HI3,HI4, N1, N2)

     pmdiff =&
          CF**2*CAM2CF*(-178.D0/9.D0*Hr1(0) + 178.D0/9.D0*Hr1(0)&
          *x - 3.D0*Hr1(0)*z3 + 3.D0*Hr1(0)*z3*x + 8.D0*Hr1(0)*z2 + 4.D0&
          *Hr1(0)*z2*x - 140.D0/9.D0*Hr2(0,0) + 238.D0/9.D0*Hr2(0,0)*x&
          -4.D0*Hr2(0,0)*z2 - 52.D0/3.D0*Hr2(0,1) + 52.D0/3.D0*Hr2(0,1)&
          *x - 52.D0/3.D0*Hr2(1,0) + 52.D0/3.D0*Hr2(1,0)*x + 8.D0*&
          Hr3(-1,0,0) + 8.D0*Hr3(-1,0,0)*x + 4.D0*Hr3(0,-1,0) + 4.D0*&
          Hr3(0,-1,0)*x - 13.D0/2.D0*Hr3(0,0,0) - 13.D0/2.D0*Hr3(0,0,0)&
          *x - 8.D0*Hr3(0,0,1) - 8.D0*Hr3(0,0,1)*x - 4.D0*Hr3(0,1,0) -&
          4.D0*Hr3(0,1,0)*x - 4.D0*Hr4(0,-1,0,0) + 4.D0*Hr4(0,-1,0,0)*x&
          -4.D0*Hr4(0,0,-1,0) + 4.D0*Hr4(0,0,-1,0)*x + 8.D0*Hr4(0,0,0,0) )

     pmdiff = pmdiff + CF**2*CAM2CF*PQQ0M*(7.D0*Hr1(0)*z3 - 3.D0/&
          2.D0*Hr1(0)*z2 - 8.D0*Hr2(-1,0)*z2 - 8.D0*Hr2(0,-1)*z2 + 8.D0&
          *Hr2(0,0)*z2 - 6.D0*Hr3(-1,0,0) - 3.D0*Hr3(0,-1,0) + 9.D0/2.D0&
          *Hr3(0,0,0) - 16.D0*Hr4(-1,-1,0,0) - 8.D0*Hr4(-1,0,-1,0) + 18.D0&
          *Hr4(-1,0,0,0) + 8.D0*Hr4(-1,0,0,1) + 4.D0*Hr4(-1,0,1,0) -&
          8.D0*Hr4(0,-1,-1,0) + 14.D0*Hr4(0,-1,0,0) + 4.D0*Hr4(0,-1,0,1)&
          + 8.D0*Hr4(0,0,-1,0) - 8.D0*Hr4(0,0,0,0) - 6.D0*Hr4(0,0,0,1)&
          - 2.D0*Hr4(0,0,1,0) )

     pmdiff = pmdiff + CF**2*CAM2CF*PQQ0P*(151.D0/24.D0*Hr1(0) +&
          Hr1(0)*z3 + 13.D0/6.D0*Hr1(0)*z2 - 169.D0/18.D0*Hr2(0,0) + 8.D0&
          *Hr2(0,0)*z2 - 134.D0/9.D0*Hr2(0,1) + 4.D0*Hr2(0,1)*z2 - 134.D0&
          /9.D0*Hr2(1,0) + 4.D0*Hr2(1,0)*z2 - 13.D0/2.D0*Hr3(0,0,0)&
          -22.D0/3.D0*Hr3(0,0,1) - 22.D0/3.D0*Hr3(0,1,0) - 22.D0/3.D0&
          *Hr3(1,0,0) - 8.D0*Hr4(0,0,0,0) - 6.D0*Hr4(0,0,0,1) - 2.D0*&
          Hr4(0,0,1,0) - 2.D0*Hr4(0,1,0,0) - 6.D0*Hr4(1,0,0,0) )

     pmdiff = pmdiff + CF**3*(-325.D0/18.D0*Hr1(0) + 325.D0/18.D0&
          *Hr1(0)*x + 3.D0*Hr1(0)*z2 - 5.D0*Hr1(0)*z2*x - 173.D0/18.D0*&
          Hr2(0,0) + 691.D0/18.D0*Hr2(0,0)*x - 4.D0*Hr2(0,0)*z2 - 4.D0*&
          Hr2(0,0)*z2*x - 50.D0/3.D0*Hr2(0,1) + 50.D0/3.D0*Hr2(0,1)*x&
          -50.D0/3.D0*Hr2(1,0) + 50.D0/3.D0*Hr2(1,0)*x + 25.D0/2.D0*&
          Hr3(0,0,0) + 25.D0/2.D0*Hr3(0,0,0)*x + 2.D0*Hr3(0,0,1) + 2.D0&
          *Hr3(0,0,1)*x + Hr3(0,1,0) + Hr3(0,1,0)*x )

     pmdiff = pmdiff + CF**3*PQQ0P*(311.D0/24.D0*Hr1(0) + 4.D0/3.D0&
          *Hr1(0)*z2 - 169.D0/9.D0*Hr2(0,0) + 8.D0*Hr2(0,0)*z2 - 268.D0/&
          9.D0*Hr2(0,1) + 8.D0*Hr2(0,1)*z2 - 268.D0/9.D0*Hr2(1,0) + 8.D0&
          *Hr2(1,0)*z2 - 22.D0*Hr3(0,0,0) - 44.D0/3.D0*Hr3(0,0,1) - 44.D0&
          /3.D0*Hr3(0,1,0) - 44.D0/3.D0*Hr3(1,0,0) )

     pmdiff = pmdiff + twonfTF*CF**2*(13.D0/9.D0*Hr1(0) - 13.D0/9.D0*&
          Hr1(0)*x + 8.D0/9.D0*Hr2(0,0) - 28.D0/9.D0*Hr2(0,0)*x + 4.D0/&
          3.D0*Hr2(0,1) - 4.D0/3.D0*Hr2(0,1)*x + 4.D0/3.D0*Hr2(1,0) - 4.D0&
          /3.D0*Hr2(1,0)*x - Hr3(0,0,0) - Hr3(0,0,0)*x )

     pmdiff = pmdiff + twonfTF*CF**2*PQQ0P*(-11.D0/12.D0*Hr1(0) - 2.D0/&
          3.D0*Hr1(0)*z2 + 11.D0/9.D0*Hr2(0,0) + 20.D0/9.D0*Hr2(0,1) +&
          20.D0/9.D0*Hr2(1,0) + 2.D0*Hr3(0,0,0) + 4.D0/3.D0*Hr3(0,0,1)&
          +4.D0/3.D0*Hr3(0,1,0) + 4.D0/3.D0*Hr3(1,0,0) )

     res=pmdiff*two
     IF(PRESENT(y_is_x).AND..NOT.y_is_x)THEN
        ! add the Jacobian
        res=res*x
     ENDIF
     RETURN
   END FUNCTION AP_DSFTS_P2NSminus

  ! Time Like
  ! P^{(2)}_{gg}
  FUNCTION AP_SFT_P2gg(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    REAL(KIND(1d0))::x,dsfts,pggspace

    IF(PRESENT(y_is_x))THEN
       dsfts=AP_DSFTS_P2gg(y,y_is_x)
       pggspace=AP_SFS_P2gg(y,y_is_x)
    ELSE
       dsfts=AP_DSFTS_P2gg(y)
       pggspace=AP_SFS_P2gg(y)
    ENDIF

    res=pggspace+dsfts
    RETURN
  END FUNCTION AP_SFT_P2gg
!=====================================================================
! Polarized Altarelli-Parisi splitting functions in QCD
! The coefficients of
! P(x,as)=P0(x)+as/2/Pi*P1(x)+(as/2/Pi)**2*P2(x)+...
! x=exp(-y)
! \int dx f(x)_+ g(x) = \int dy [exp(-y)f(exp(-y))]_+ g(exp(-y))
!=====================================================================

! WARNING: check the spin-dependent splitting functions in hep-ph/9603366 to 2-loop
! also check those in HOPPET code

  FUNCTION AP_SFS_DP1gg(y,y_is_x) RESULT(res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::y
    LOGICAL,INTENT(IN),OPTIONAL::y_is_x
    REAL(KIND(1d0))::res
    res=zero
  END FUNCTION AP_SFS_DP1gg
END MODULE qcd_splitting_kernels
